(* Mathematica Package *)

(* Created by the Wolfram Workbench Jan 22, 2010 *)



DifferenceQuotient::usage = "DifferenceQuotient[{fun,var}, \[Delta]] returns the standard difference quotient for f with offset \[Delta]."

TimeIntervalPlot::usage = "TimeIntervalPlot[fun,{x,xmin,xmax},ints,opts] creates a plot of the function fun on the interval xmin to smax and includes bars along the left side indicating the change in fun for each of the intervals ints."

BarWidths::usage = "BarWidths an option for TimeIntervalPlot that determines the widths of the bars. It should be given as a fraction of the width of the plot."

ReferenceLine::usage = "ReferenceLine and option for TimeIntervalPlot that determines the vertical location of the reference line."

ArrowStyles::usage = "ArrowStyles is an option for TimeIntervalPlot that sets the styles of arrows in the plot. It should be given as a pair of values where each element of the pair is a set of directives that would be used with Arrow. Setting the value to None eliminates the arrows."

DirectionFieldPlot::usage = "DirectionFieldPlot[expr,{var1,var1min,var1max},{var2,var2min,var2max}] plots the direction field for the ODE y'= expr."

LinearEstimate::usage = "LinearEstimate[f,x,{x1,x2}] gives an estimate of f(x2) based on the value of f(x1) and the tangent at x1.\nLinearEstimate[f,x,pt,stp] returns an approximation to f by moving a distance stp along the tangent to f at pt."

EulerEstimate::usage = "EulerEstimate[der,{var1,var2},pt,stp] estimates the value of the solution function for y'= der stp units distant from pt."

EulerSolve::usage = "EulerSolve[der,{var,varmin,varmax},pt,stp] gives and Euler method approximation to the solution of the differential equation y'==der starting at pt and continuing to var=varmax."

(* Exported symbols added here with SymbolName::usage *) 

Unprotect[DifferenceQuotient,TimeIntervalPlot,BarWidths,ReferenceLine,ArrowStyles,BarStyles,DirectionFieldPlot,PointStyle,LinearEstimate,
EulerEstimate,EulerSolve]

Begin["`Private`"]
(* Implementation of the package *)

SetAttributes[DifferenceQuotient, {HoldFirst}]

DifferenceQuotient[fun_Symbol, var : _Symbol | _?NumericQ, 
       diff : _Symbol | _?NumericQ] := (fun[Plus[var , diff]] - 
    fun[var])/diff

DifferenceQuotient[fun_Symbol, 
       var : {Repeated[_Symbol | _?NumericQ|_Subscript, {2}]}] := 
     (fun[var[[1]]] - fun[var[[2]]])/(var[[1]] - var[[2]])

DifferenceQuotient[{expr_, var_Symbol}, 
       var2 : {Repeated[_Symbol | _?NumericQ|_Subscript, {2}]}] := 
Block[{fun,var22},
  fun[var22_] = expr/.var:> var22;
  DifferenceQuotient[fun, var2]]

DifferenceQuotient[{expr_, var_Symbol}, 
       diff : _Symbol | _?NumericQ] := 
 Block[{fun,var2}, fun[var2_] = expr/.var:> var2;
  DifferenceQuotient[fun, var, diff]]
  
(*TimeIntervalPlot needs a better name*)

makeFunVals[expr_, var_Symbol, locs_List] := 
 expr /. {var -> #} & /@ locs

makeRecs[funvals_List, pos_?NumericQ, offst_?NumericQ] := 
 Block[{spltlis = 
    Split[N@Partition[funvals, 2, 1], 
     Greater @@ #1 === Greater @@ #2 &], len, strtpts, scledpts, 
   rvrspts, scledrvrspts},
  len = Length[spltlis];
  strtpts = Map[Thread[{N[pos], #1}] &, spltlis, {2}];
  scledpts = 
   MapIndexed[Scaled[(len - (#2[[1]] - 1)) {offst, 0}, #1] &, 
    strtpts, {3}];
  rvrspts = Map[Reverse, strtpts, {2}];
  scledrvrspts = 
   MapIndexed[Scaled[((len - #2[[1]])) {offst, 0}, #1] &, 
    rvrspts, {3}];
  Partition[
   Flatten[MapThread[Riffle[#1, #2] &, {scledpts, scledrvrspts}], 2], 
   4]
  ]
  
makeRecSets[recs_ /; Depth[recs] == 5, barstyls_List] := 
 Sequence@@Riffle[barstyls, {Polygon[recs[[1 ;; -1 ;; 2]]], 
   Polygon[recs[[2 ;; -1 ;; 2]]]}]

makeRecSets[recs_ /; Depth[recs] == 6, barstyls_List] := 
 Sequence@@Riffle[barstyls, {Polygon[Flatten[recs[[All, 1 ;; -1 ;; 2]], 1]], 
   Polygon[Flatten[recs[[All, 2 ;; -1 ;; 2]], 1]]}]

makeLns[recs_List, locs_List, funpts_List, pos_?NumericQ,refval_?NumericQ] := 
 Block[{recPts = recs[[All,2]], 
   axespts = Thread[{locs, refval}]},
  Thread[{recPts, Rest[funpts], Rest[axespts]}]]
  
  makeArrows[funvals_List, pos_, offst_?NumericQ] := 
 Block[{spltlis = 
    Split[N@Partition[funvals, 2, 1], 
     Greater @@ #1 === Greater @@ #2 &], len, arrowsplt, strtpts, 
   scledpts,arrw},
  len = Length[spltlis];
  arrowsplt = 
   Map[{Total[{3/4, 1/4} #], Total[{1/4, 3/4} #]} &, spltlis, {2}];
  strtpts = Map[Thread[{N[pos], #1}] &, arrowsplt, {2}];
  scledpts = 
   MapIndexed[Scaled[{(len - (#2[[1]]-1))offst - offst/2, 0}, #1] &, 
    strtpts, {3}];
   Flatten[Map[Arrow, scledpts, {2}]]
  ]
  
makeArrows[funvals_?MatrixQ, pos_List, offst_List] := 
 MapThread[makeArrows[#1,#2,#3]&,{funvals,pos,offst}]
  
makeArrowSets[arrows_/;Depth[arrows]==6, funvals_List, frac_?NumericQ,arrwstyl_List] := 
 Block[{rng = Differences[{Min[#], Max[#]} &[funvals]][[1]]},
  deletes = 
   Extract[arrows, 
    Position[N@Differences[funvals], df_ /; Abs[df] < rng frac]];
Sequence@@Flatten[Riffle[arrwstyl, {Complement[Flatten[arrows][[1 ;; -1 ;; 2]], deletes], 
   Complement[Flatten[arrows][[2 ;; -1 ;; 2]], deletes]}]]
  ]
  
makeArrowSets[arrows_/;Depth[arrows]==7, funvals_?MatrixQ, frac_?NumericQ,arrwstyl_List] := 
 Sequence@@MapThread[makeArrowSets[#1,#2,frac,arrwstyl]&,{arrows,funvals}]
  
Options[TimeIntervalPlot] = {AlignmentPoint -> Center, ArrowStyles -> Automatic, AspectRatio -> 1,
  Axes -> True, AxesLabel -> None, AxesOrigin -> Automatic, 
 AxesStyle -> {}, Background -> None, BarStyles -> Automatic, 
 BarWidths -> Automatic, BaselinePosition -> Automatic, 
 BaseStyle -> {}, ClippingStyle -> None, ColorFunction -> Automatic, 
 ColorFunctionScaling -> True, ColorOutput -> Automatic, 
 ContentSelectable -> Automatic, CoordinatesToolOptions -> Automatic, 
 Epilog -> {}, Evaluated -> Automatic, EvaluationMonitor -> None, 
 Exclusions -> Automatic, ExclusionsStyle -> None, Filling -> None, 
 FillingStyle -> Automatic, Frame -> False, FrameLabel -> None, 
 FrameStyle -> {}, FrameTicks -> {{None, None}, {Automatic, None}}, 
 FrameTicksStyle -> {}, GridLines -> None, GridLinesStyle -> {}, 
 ImageMargins -> 0., ImagePadding -> All, ImageSize -> Automatic, 
 ImageSizeRaw -> Automatic, LabelStyle -> {}, 
 MaxRecursion -> Automatic, Mesh -> None, MeshFunctions -> {#1 &}, 
 MeshShading -> None, MeshStyle -> Automatic, Method -> Automatic, 
 PlotLabel -> None, PlotPoints -> Automatic, PlotRange -> All, 
 PlotRangeClipping -> False, PlotRangePadding -> Automatic, 
 PlotRegion -> Automatic, PlotStyle -> Automatic, 
 PreserveImageOptions -> Automatic, Prolog -> {}, 
 ReferenceLine -> Automatic, RegionFunction -> (True &), 
 RotateLabel -> True, Ticks -> {Automatic, None}, TicksStyle -> {}, 
 WorkingPrecision -> MachinePrecision, 
 DisplayFunction :> $DisplayFunction, FormatType :> TraditionalForm, 
 PerformanceGoal :> $PerformanceGoal}

TimeIntervalPlot[
  fun_, {var_Symbol, varmin_?NumericQ, varmax_?NumericQ}, 
  tmints_Integer, opts : OptionsPattern[]] := 
 Block[{locs = Range[varmin, varmax, (varmax - varmin)/tmints],
   arrwstyl = (OptionValue[
       ArrowStyles] /. {Automatic -> {{Thickness[Medium],Arrowheads[Small],RGBColor[0.9`, 0, 1.`, 0.95`]}, 
       	{Thickness[Large],RGBColor[1, 0.95`, 0, 0.95`]}}, 
       None -> {Opacity[0],Opacity[0]},x_!List:> {x,x}}),
   brstyl = (OptionValue[BarStyles] /. 
     {Automatic -> {{EdgeForm[{AbsoluteThickness[.4], Gray}], 
         RGBColor[1, .95, 
          0, .4]}, {EdgeForm[{AbsoluteThickness[.4], Gray}], 
         RGBColor[.9, 0, 1., .4]}}, 
       None -> {Opacity[0],Opacity[0]},(x_!List):> {x,x}}),
   brwdth = (OptionValue[BarWidths] /. Automatic -> .04) (-1),
   funvals, funpts, refval, recs, recSets, arrws, arrwSets, lns, refln},
  
  funvals = makeFunVals[fun, var, locs];
  refval = OptionValue[ReferenceLine] /. Automatic -> Min[funvals];
  refln= Line[{{varmin,refval},{varmax,refval}}];
  funpts = Transpose[{locs, funvals}];
  
  recs = makeRecs[funvals, varmin, brwdth];
  recSets = makeRecSets[recs, brstyl];
  arrws = makeArrows[funvals, varmin, brwdth];
  arrwSets = makeArrowSets[arrws,funvals,1/16, arrwstyl];
  lns = Sequence[GrayLevel[.4],Thickness[Small],Line[makeLns[recs, locs, funpts, varmin, refval]]];
  
  Show[{Graphics[Flatten[{recSets,Opacity[1], arrwSets,Opacity[1],lns,refln,Red,PointSize[Medium],Point[funpts]}]], 
    Plot[fun, {var, varmin, varmax}, 
     Evaluate@FilterRules[{opts}, Options[Plot]], 
     Evaluate@FilterRules[Options[TimeIntervalPlot], Options[Plot]]]},
    FilterRules[{opts}, Options[Graphics]], 
   FilterRules[Options[TimeIntervalPlot], Options[Graphics]]]]
  
TimeIntervalPlot[
  fun_List, {var_Symbol, varmin_?NumericQ, varmax_?NumericQ}, 
  tmints_Integer, opts : OptionsPattern[]] := 
 Block[{locs = Range[varmin, varmax, (varmax - varmin)/tmints], 
   arrwstyl = (OptionValue[
       ArrowStyles] /. {Automatic -> {{Thickness[Medium], 
          Arrowheads[Small], 
          RGBColor[0.9`, 0, 1.`, 0.95`]}, {Thickness[Large], 
          RGBColor[1, 0.95`, 0, 0.95`]}}, None ->{Opacity[0], Opacity[0]},x_!List:> {x,x}}), 
   brstyl = (OptionValue[BarStyles] /. 
      {Automatic -> {{EdgeForm[{AbsoluteThickness[.4], Gray}], 
         RGBColor[1, .95, 
          0, .4]}, {EdgeForm[{AbsoluteThickness[.4], Gray}], 
         RGBColor[.9, 0, 1., .4]}}, 
       None -> {Opacity[0],Opacity[0]},x_!List:> {x,x}}), 
   brwdth = (OptionValue[BarWidths] /. Automatic -> .04) (-1), 
   funvals, funpts, refval, recs, recSets, arrws, arrwSets, lns, refln}, 
  funvals = makeFunVals[fun[[1]], var, locs];
  refval = OptionValue[ReferenceLine] /. Automatic -> Min[funvals]; 
  refln= Line[{{varmin,refval},{varmax,refval}}];
  funpts = Transpose[{locs, funvals}];
  recs = makeRecs[funvals, varmin, brwdth];
  recSets = makeRecSets[recs, brstyl];
  arrws = makeArrows[funvals, varmin, brwdth];
  arrwSets = makeArrowSets[arrws, funvals, 1/16, arrwstyl];
  lns = Sequence[GrayLevel[.4], Thickness[Small], 
    Line[makeLns[recs, locs, funpts, varmin, refval]]];
  Show[{Graphics[
     Flatten[{recSets,Opacity[1], arrwSets,Opacity[1], lns,refln, Red, PointSize[Medium], 
       Point[funpts]}]], 
    Plot[fun, {var, varmin, varmax}, 
     Evaluate@FilterRules[{opts}, Options[Plot]], 
     Evaluate@FilterRules[Options[TimeIntervalPlot], Options[Plot]]]},
    FilterRules[{opts}, Options[Graphics]], 
   FilterRules[Options[TimeIntervalPlot], Options[Graphics]]]]

TimeIntervalPlot[{fns : {Repeated[(_), {2}]}, funs___}, {var_Symbol, 
   varmin_?NumericQ, varmax_?NumericQ}, tmints_Integer, 
  opts : OptionsPattern[]] :=
 
 Block[
  {locs = Range[varmin, varmax, (varmax - varmin)/tmints], 
   arrwstyl = (OptionValue[
       ArrowStyles] /. {Automatic -> {{Thickness[Medium], 
          Arrowheads[Small], 
          RGBColor[0.9`, 0, 1.`, 0.95`]}, {Thickness[Large], 
          RGBColor[1, 0.95`, 0, 0.95`]}}, None -> {Opacity[0],Opacity[0]},x_!List:> {x,x}}), 
   brstyl = (OptionValue[BarStyles] /. 
      {Automatic -> {{EdgeForm[{AbsoluteThickness[.4], Gray}], 
         RGBColor[1, .95, 
          0, .4]}, {EdgeForm[{AbsoluteThickness[.4], Gray}], 
         RGBColor[.9, 0, 1., .4]}}, 
       None -> {Opacity[0],Opacity[0]},x_!List:> {x,x}}), 
   brwdth = (OptionValue[BarWidths] /. Automatic -> .04) {-1, 1},
   funvals, funpts, refval, recs, recSets, arrws, arrwSets, lns, refln},
  
  funvals = makeFunVals[#, var, locs] & /@ fns;
  refval = OptionValue[ReferenceLine] /. Automatic -> Min[funvals];
  refln= Line[{{varmin,refval},{varmax,refval}}];
  funpts = Transpose[{locs, #}] & /@ funvals;
  
  recs = Thread[makeRecs[funvals, {varmin, varmax}, brwdth]];
  recSets = makeRecSets[recs, brstyl];
  
  
  arrws = makeArrows[funvals, {varmin,varmax}, brwdth]; 
  arrwSets = makeArrowSets[arrws, funvals, 1/16, arrwstyl];
  lns = 
  Sequence[GrayLevel[.4], Thickness[Small], 
    Line[Flatten[MapThread[makeLns[#1,#2,#3,#4,refval]&,{recs,{locs,locs},funpts,{varmin,varmax}}],1]]];  
  
  Show[{Graphics[
     Flatten[{recSets,Opacity[1], arrwSets,Opacity[1], lns,refln, Red, PointSize[Medium], 
       Point[Flatten[funpts,1]]}]], 
    Plot[Evaluate[Join[fns, {funs}]], {var, varmin, varmax}, 
     Evaluate@FilterRules[{opts}, Options[Plot]], 
     Evaluate@FilterRules[Options[TimeIntervalPlot], Options[Plot]]]},
    FilterRules[{opts}, Options[Graphics]], 
   FilterRules[Options[TimeIntervalPlot], Options[Graphics]]]]
   
   
(*DirectionFieldPlot function definitions*)

Options[DirectionFieldPlot] =
 {AlignmentPoint -> Center, AspectRatio -> Automatic, Axes -> False, 
  AxesLabel -> None, AxesOrigin -> Automatic, AxesStyle -> {}, 
  Background -> None, BaselinePosition -> Automatic, BaseStyle -> {}, 
  BoundaryStyle -> None, BoxRatios -> Automatic, 
  ColorFunction -> None, ColorFunctionScaling -> True, 
  ColorOutput -> Automatic, ContentSelectable -> Automatic, 
  CoordinatesToolOptions -> Automatic, Epilog -> {}, 
  Evaluated -> Automatic, EvaluationMonitor -> None, Frame -> True, 
  FrameLabel -> None, FrameStyle -> {}, FrameTicks -> Automatic, 
  FrameTicksStyle -> {}, GridLines -> None, GridLinesStyle -> {}, 
  ImageMargins -> 0.`, ImagePadding -> All, ImageSize -> Automatic, 
  ImageSizeRaw -> Automatic, LabelStyle -> {}, LightingAngle -> None, 
  MaxRecursion -> Automatic, Mesh -> None, MeshFunctions -> Automatic,
  MeshShading -> None, MeshStyle -> Automatic, Method -> Automatic, 
  NormalsFunction -> Automatic, PlotLabel -> None, 
  PlotRange -> {Full, Full}, PlotRangeClipping -> True, 
  PlotRangePadding -> Automatic, PlotRegion -> Automatic, 
  PointStyle -> Automatic, PreserveImageOptions -> Automatic, 
  Prolog -> {}, RegionFunction -> (True &), RotateLabel -> True, 
  StreamColorFunction -> None, StreamColorFunctionScaling -> True, 
  StreamPoints -> 5, StreamScale -> Full, StreamStyle -> Automatic, 
  Ticks -> Automatic, TicksStyle -> {}, VectorColorFunction -> None, 
  VectorColorFunctionScaling -> True, VectorPoints -> Automatic, 
  VectorScale -> Automatic, VectorStyle -> Automatic, 
  WorkingPrecision -> MachinePrecision, 
  DisplayFunction :> $DisplayFunction, FormatType :> TraditionalForm, 
  PerformanceGoal :> $PerformanceGoal}
  
  
  adjusts={ Axes -> True,
     VectorPoints -> pts,
     VectorScale -> {Scaled[.02], Scaled[.001], None},
     StreamPoints ->5,
     StreamScale -> Full,
     StreamStyle -> {Arrowheads[Medium],Thickness[Medium], Red},
     ImageSize -> Medium,
     AspectRatio -> Automatic}

DirectionFieldPlot[
  fun_, {var1_, var1min_, var1max_}, {var2_, var2min_, var2max_}, 
  opts : OptionsPattern[]] := 
 Block[{pts = 
    Flatten[Table[{x, y}, {x, Floor[var1min], Ceiling[var1max], 
       1/4}, {y, Floor[var2min], Ceiling[var2max], 1/4}], 1],
       ptstyl=OptionValue[PointStyle]/.{Automatic->{RGBColor[0.525475, 0.596429, 0.891402], PointSize[.01]}} }, 
      
  Show[{VectorPlot[{1, fun}, {var1, var1min, var1max}, {var2, var2min,
       var2max},Evaluate[FilterRules[{opts},Options[VectorPlot]]],Evaluate[adjusts],
       Evaluate[FilterRules[Options[DirectionFieldPlot],Options[VectorPlot]]]
    ],
    Graphics[{Sequence@@ptstyl, 
      Point[Cases[pts, pt_ /; FreeQ[pt, Rational]]]}]}]]
      
(*The LinearEstimate function*)

LinearEstimate[fun_, var_Symbol, {pt_?NumericQ, pt2_?NumericQ}] := 
 Block[{slpFun = D[fun, var], funPt = {var, fun} /. var :> pt, 
   stp = pt2 - pt, slpPt},
  slpPt = (funPt + {stp, slpFun stp } /. {var :> pt});
  slpPt]

LinearEstimate[fun_, var_Symbol, pt_?NumericQ, stp_?NumericQ] := 
 Block[{slpFun = D[fun, var], funPt = {var, fun} /. var :> pt, 
   slpPt},
  slpPt = (funPt + stp Normalize[{1, slpFun } /. {var :> funPt[[1]]}]);
  slpPt]

LinearEstimate[fun_, var_Symbol, pt_List, stp_?NumericQ] := 
 Block[{slpFun = D[fun, var], funPt = {var, fun} /. var :> pt[[1]], 
   slpPt},
  slpPt = (funPt + stp Normalize[{1, slpFun } /. {var :> funPt[[1]]}]);
  slpPt]
  
(*The Euler functions for approximating solutions to ODEs*)
      
EulerEstimate[fun_, {var1_Symbol, var2_Symbol}, pt_List, 
  stp_?NumericQ] := 
 Block[{slp = fun /. {var1 :> pt[[1]], var2 :> pt[[2]]}, dervec},
  dervec = Normalize[{1, slp}];
  pt + stp dervec]
  
 EulerSolve[derfun_, {var_Symbol, varmin_?NumericQ, varmax_?NumericQ}, 
  strtPt_List, stp_?NumericQ, maxStps_Integer: 200] := 
 Block[{var2 = 
    ((Cases[N[{derfun}], sym_Symbol /; sym =!= var, Infinity]) /. {} :> { $var2})[[1]]},
  NestWhileList[EulerEstimate[derfun, {var, var2}, #, stp] &, 
   N[strtPt], (#1[[1]] < varmax && Abs[#1[[2]]] < Infinity &), 1, 
   maxStps]]
   


End[]

SetAttributes[{DifferenceQuotient,TimeIntervalPlot,BarWidths,ReferenceLine,ArrowStyles,BarStyles,DirectionFieldPlot,PointStyle,LinearEstimate,
EulerEstimate,EulerSolve},{Protected,ReadProtected}]




