(* Mathematica Package *)

(* Created by the Wolfram Workbench Oct 5, 2009 *)

(*Modified June 2010 to format Int as a defered Integrate as per suggestion from Bhuvanesh Bhatt.*)

(* Exported symbols added here with SymbolName::usage *) 



(*Int::usage= "Int is a placeholder head for expressions used in displaying results from PartsIntegrate and SubstituteIntegrate."*)

SubstituteIntegrate::usage = "SubstituteIntegrate[fun,var,{sub-> u,subDx-> aDu}] gives the expression Int[a f[u],u] and when Int is replaced by Integrate will give a solution to Integrate[fun,var]."

AutoSimplify::usage = "AutoSimplify is an option for SIntegrate that indicates whether the result should be automatically simplified. The default is False."

(*RemoveInt::usage = "RemoveInt[expr] converts any expression with head Int to an equivalent expression with the head Integrate and evaluates the result."*)

PartsIntegrate::usage = "PartsIntegrate[integrand,variable,{u,dv}] returns the result of integrating the integrand with respect to the variable using the method integral[u dv]= u v - integral[v du]"

Subintervals::usage = "Subintervals is an option for IntegratePlot that allows random widths for the polygons."

Uniform::usage= "Uniform is an option value for BinWidths indicating all bars should have the same width."

Nonuniform::usage= "Nonuniform is an option value for BinWidths indicating the bars should have random widths."

MonoColor::usage= "MonoColor is an option for BarStyles indicating all polygons should have the same color."

RiemannSumPlot::usage = "RiemannSumPlot[fun,{var,varmin,varmax},n,\"type\",opts] gives a plot of fun on the interval varmin to varmax with n polygons satisfying type. Type can be any of \"Left\", \"Center\", \"Right\" or \"Trapezoid\"."

RiemannSum::usage = "RiemannSum[fun, {var,varmin,varmax},n,\"type\",opts] returns the numerical approximation to the integral Integrate[fun, {var,varmin,varmax}] approximated using n polygons of \"type\"."

Estimate::usage = "Estimate is an option for IntegralEstimate to determine if the estimate should give the value of the integral or the area. Automatic returns the value for the integral. Area will give the total area of the polygons. "

Area::usage="Area is an option value for the option Estimate which gives the total area of the polygons."

BarStyles::usage= "BarStyles is an option for RiemannSumPlot"

Unprotect[SubstituteIntegrate,AutoSimplify,PartsIntegrate,RiemannSumPlot,RiemannSum]


Begin["`Private`"]

(*Format[Int[stuff__]]:= With[{Int = Integrate}, Defer[Int[stuff]]]*)

(*Definition for RemoveInt*)

(*RemoveInt := # /. Int :> Integrate &*)

(*RemoveInt := #/.Int[a_,b_]:> Defer[\[Integral]a \[DifferentialD]b]&*)

Format[Int[a_, b_]] :=Defer[Integrate[a, b]]


(* Deifnitions for validTest a function to check arguments for SubstituteIntegrate *)

validTest[r1 : (a1_ -> b1_), r2 : (a2_ -> b2_), var_] := 
   Block[{der = D[a1, var]}, 
    TrueQ[Simplify[(DifferentialD[var] /. 
             ReleaseHold@Solve[der*DifferentialD[var] == DifferentialD[b1], 
                  DifferentialD[var]][[1, 1]]) == b2]]]
                  
(* Deifnitions for SubstituteIntegrate *)

Attributes[SubstituteIntegrate] = {HoldAll}

Options[SubstituteIntegrate] = {AutoSimplify -> False}

SubstituteIntegrate::incrct = "At least one of your substitutions `1` or `2` is incorrect";

SubstituteIntegrate[(epow : E^(var_))*(coef___)*(expr2 : 
     Cos[E^(var_)] | Sin[E^(var_)] | 
            Tan[E^(var_)] | Cot[E^(var_)] | Cosh[E^(var_)] | 
      Sinh[E^(var_)] | 
            Tanh[E^(var_)] | Coth[E^(var_)]), var_, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, 
     opts : OptionsPattern[]] /; validTest[r1, r2, var]:= 
 Which[OptionValue[AutoSimplify]===False,
 Int[epow*(coef*expr2*Dx /. {r1, r2}) /. 
       {DifferentialD[u] -> 1}, u],
       OptionValue[AutoSimplify]===True,
       Simplify[ Int[epow*(coef*expr2*Dx /. {r1, r2}) /. 
       {DifferentialD[u] -> 1}, u]]
 ]

SubstituteIntegrate[(epow : E^((num_)?NumericQ*(var_Symbol)))*(coef___)*
       (expr2 : Cos[E^((num_)?NumericQ*(var_Symbol))] | 
            Sin[E^((num_)?NumericQ*(var_Symbol))] | 
            Tan[E^((num_)?NumericQ*(var_Symbol))] | Cot[E^(var_)] | 
            Cosh[E^((num_)?NumericQ*(var_Symbol))] | 
            Sinh[E^((num_)?NumericQ*(var_Symbol))] | 
            Tanh[E^((num_)?NumericQ*(var_Symbol))] | 
            Coth[E^((num_)?NumericQ*(var_Symbol))]), var_, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, 
     opts : OptionsPattern[]]/; validTest[r1, r2, var] := 
Which[OptionValue[AutoSimplify]===False,
 Int[epow*(coef*expr2*Dx /. {r1, r2}) /. 
       {DifferentialD[u] -> 1}, u],
       OptionValue[AutoSimplify]===True,
       Simplify[ Int[epow*(coef*expr2*Dx /. {r1, r2}) /. 
       {DifferentialD[u] -> 1}, u]]
 ]

SubstituteIntegrate[(epow : E^(var_)^(num_))*(coef___)*(expr2 : 
     Cos[E^(var_)^(num_)] | 
            Sin[E^(var_)^(num_)] | Tan[E^(var_)^(num_)] | 
            Cot[E^(var_)^(num_)] | Cosh[E^(var_)^(num_)] | 
            Sinh[E^(var_)^(num_)] | Tanh[E^(var_)^(num_)] | 
            Coth[Power[(var_)^(num_)]]), 
  var_, {r1 : (exp_ -> u_Symbol), 
       r2 : (Dx_ -> expr__)}, opts : OptionsPattern[]]/; validTest[r1, r2, var] := 
   Which[OptionValue[AutoSimplify]===False,
 Int[epow*(coef*expr2*Dx /. {r1, r2}) /. 
       {DifferentialD[u] -> 1}, u],
       OptionValue[AutoSimplify]===True,
       Simplify[ Int[epow*(coef*expr2*Dx /. {r1, r2}) /. 
       {DifferentialD[u] -> 1}, u]]
 ]

SubstituteIntegrate[(epow : E^((num2_)?NumericQ*(var_)^(num_)))*(coef___)*
       (expr2 : Cos[E^((num2_)?NumericQ*(var_)^(num_))] | 
            Sin[E^((num2_)?NumericQ*(var_)^(num_))] | 
            Tan[E^((num2_)?NumericQ*(var_)^(num_))] | 
            Cot[E^((num2_)?NumericQ*(var_)^(num_))] | 
            Cosh[E^((num2_)?NumericQ*(var_)^(num_))] | 
            Sinh[E^((num2_)?NumericQ*(var_)^(num_))] | 
            Tanh[E^((num2_)?NumericQ*(var_)^(num_))] | 
            Coth[E^((num2_)?NumericQ*(var_)^(num_))]), var_, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, 
     opts : OptionsPattern[]]/; validTest[r1, r2, var] := 
 Which[OptionValue[AutoSimplify]===False,
 Int[epow*(coef*expr2*Dx /. {r1, r2}) /. 
       {DifferentialD[u] -> 1}, u],
       OptionValue[AutoSimplify]===True,
       Simplify[ Int[epow*(coef*expr2*Dx /. {r1, r2}) /. 
       {DifferentialD[u] -> 1}, u]]
 ]

SubstituteIntegrate[integrnd:Sin[v_Symbol]/Cos[v_Symbol], 
     var_Symbol, {r1:(exp_ -> u_Symbol), r2:(Dx_ -> expr__)}, 
  opts:OptionsPattern[]]/; validTest[r1, r2, var] := Module[{newint},
 (*Print["def 1"];*)
  newint = DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
        DifferentialD[u], 4];
  Which[OptionValue[AutoSimplify] === False,
   Int[newint, u],
   OptionValue[AutoSimplify] === True,
   Simplify[Int[newint, u]]]
  ]

SubstituteIntegrate[
  integrnd:Cos[v_Symbol]/Sin[v_Symbol], 
     var_Symbol, {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]]/; validTest[r1, r2, var] := 
    ( (*Print["def 2"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], DifferentialD[u], 4], u], 
       OptionValue[AutoSimplify] === True, 
       Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], DifferentialD[u], 4], u]]])

SubstituteIntegrate[integrnd:Sin[v_Symbol]/
           Cos[v_Symbol]^(n_Integer), var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]]/; validTest[r1, r2, var] := 
    ( (*Print["def 3"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}],  DifferentialD[u], 4], u], 
  	OptionValue[AutoSimplify] === True, 
  	Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}],  DifferentialD[u], 4], u]]])

SubstituteIntegrate[integrnd:Cos[v_Symbol]/
           Sin[v_Symbol]^(n_Integer), var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]] /; validTest[r1, r2, var]:= 
    ( (*Print["def 4"];*)
   Which[OptionValue[AutoSimplify] === False,
   	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], DifferentialD[u], 4], u],
   	OptionValue[AutoSimplify] === True, 
   	Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], DifferentialD[u], 4], u]]])

SubstituteIntegrate[integrnd:Sin[v_Symbol]/
           Cos[v_Symbol]^(n_Integer), var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]]/; validTest[r1, r2, var] := 
    ( (*Print["def 5"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], DifferentialD[u], 4], u], 
  	OptionValue[AutoSimplify] === True, 
  	Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], DifferentialD[u], 4], u]]])

SubstituteIntegrate[
  integrnd:Sin[(num_)?NumericQ*(v_Symbol)]/
           Cos[(num_)?NumericQ*(v_Symbol)]^(n_Integer), var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]] /; validTest[r1, r2, var]:= 
     ((*Print["def 6"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], DifferentialD[u], 4], u], 
  	OptionValue[AutoSimplify] === True, 
  	Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], DifferentialD[u], 4], u]]])

SubstituteIntegrate[integrnd:Cos[v_Symbol]/
           Sin[v_Symbol]^(n_Integer), var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]] /; validTest[r1, r2, var]:= 
     ((*Print["def 7"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u], 
       OptionValue[AutoSimplify] === True, 
       Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u]]])

SubstituteIntegrate[
  integrnd:Cos[(num_)?NumericQ*(v_Symbol)]/
           Sin[(num_)?NumericQ*(v_Symbol)]^(n_Integer), var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]]/; validTest[r1, r2, var] := 
    ( (*Print["def 8"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u], 
       OptionValue[AutoSimplify] === True, 
       Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u]]])

SubstituteIntegrate[integrnd:Sin[v_Symbol]/
           Cos[v_Symbol]^(n_Integer), var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]] /; validTest[r1, r2, var]:= 
     ((*Print["def 9"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u], 
       OptionValue[AutoSimplify] === True, 
       Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u]]])

SubstituteIntegrate[integrnd:Sinh[v_Symbol]/
           Cosh[v_Symbol]^(n_Integer), var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]]/; validTest[r1, r2, var] := 
    ( (*Print["def 10"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u],
        OptionValue[AutoSimplify] === True, 
        Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u]]])

SubstituteIntegrate[
  integrnd:Sinh[(num_)?NumericQ*(v_Symbol)]/
           Cosh[(num_)?NumericQ*(v_Symbol)]^(n_Integer), 
  var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]]/; validTest[r1, r2, var] := 
    ( (*Print["def 11"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u],
        OptionValue[AutoSimplify] === True, 
        Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u]]])

SubstituteIntegrate[integrnd:Cosh[v_Symbol]/
           Sinh[v_Symbol]^(n_Integer), var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]]/; validTest[r1, r2, var] := 
     ((*Print["def 12"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u], 
       OptionValue[AutoSimplify] === True, 
       Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u]]])

SubstituteIntegrate[
  integrnd:Cosh[(num_)?NumericQ*(v_Symbol)]/
           Sinh[(num_)?NumericQ*(v_Symbol)]^(n_Integer), 
  var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]]/; validTest[r1, r2, var] := 
    ( (*Print["def 13"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u], 
       OptionValue[AutoSimplify] === True, 
       Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u]]])

SubstituteIntegrate[integrnd_, var_Symbol, {r1 : (exp_ -> u_Symbol), 
       r2 : (Dx_DifferentialD -> expr_)}, opts:OptionsPattern[]]/; validTest[r1, r2, var] := 
      ( (*Print["def 14"];*)
   Which[OptionValue[AutoSimplify] === False,
   	Int[DeleteCases[integrnd*Dx /. {r1, r2}, DifferentialD[u], 4], u], 
   	OptionValue[AutoSimplify] === True, 
   	Simplify[Int[DeleteCases[integrnd*Dx /. {r1, r2}, DifferentialD[u], 4], u]]])

SubstituteIntegrate[integrnd_, var_Symbol, {r1 : (exp_ -> u_Symbol), 
       r2 : (Dx_DifferentialD -> Du_DifferentialD)}, opts:OptionsPattern[]] /; validTest[r1, r2, var]:= 
      ( (*Print["def 15"];*)
   Which[OptionValue[AutoSimplify] === False,
   	Int[DeleteCases[integrnd*Dx /. {r1, r2}, Du, 4], u], 
   	OptionValue[AutoSimplify] === True, 
   	Simplify[Int[DeleteCases[integrnd*Dx /. {r1, r2}, Du, 4], u]]])
   	
SubstituteIntegrate[integrnd_, var_Symbol, {r1 : (exp_ -> pow:Power[u_Symbol,n_Integer]), 
       r2 : (Dx_DifferentialD -> expr_)}, opts:OptionsPattern[]]/;D[pow,u]==Most[expr] := 
      ( (*Print["def 16"];*)
   Which[OptionValue[AutoSimplify] === False,
   	Int[DeleteCases[integrnd*Dx /. {r1, r2}, DifferentialD[u], 4], u], 
   	OptionValue[AutoSimplify] === True, 
   	Simplify[Int[DeleteCases[integrnd*Dx /. {r1, r2}, DifferentialD[u], 4], u]]])
   	
SubstituteIntegrate[integrnd_,var_Symbol,lis_List,OptionsPattern[]]/;Message[SubstituteIntegrate::incrct,lis[[1]],lis[[2]]]:=Null

(*Definitions for the function PartsIntegrate*)

PartsIntegrate::incrct="At least one of your choices `1` or `2` is incorrect."

prtint[intgrnd_, var_, {u_, dv_}] := 
 Block[{du = D[u, var], v = Integrate[dv, var], res},
  If[Simplify[u dv === intgrnd], 
   u v - Int[v du, var], (Message[PartsIntegrate::incrct, u, 
     dv]; $Failed)]
  ]
  
PartsIntegrate[intgrnd_, var_Symbol, {u_, dv_}] := 
 Block[{res = prtint[intgrnd, var, {u, dv}]},
  res /; res =!= $Failed]

PartsIntegrate[fun_Int, var_Symbol, {u_, dv_}] := 
 PartsIntegrate[fun[[1]], var, {u, dv}]

(*PartsIntegrate[fun_, var_Symbol, {u_, dv_}] := 
 Block[{du = D[u, var], v = Integrate[dv, var]}, 
  If[Not[Simplify[u dv === fun]], 
   Message[PartsIntegrate::incrct,u,dv];Null
   (*StringForm["PartsIntegrate[`1`,`2`,{`3`,`4`}]",fun,var,u,dv]*), 
   u v - Int[v du, var]]]*)

(*Definitions for IntegralPlot*)

(*Utility functions*)

Options[binVals] = {Subintervals -> Uniform}

Options[intervals] = {Subintervals -> Uniform}

binVals[min_?NumericQ, max_?NumericQ, n_Integer, opts : OptionsPattern[]] /; 
  min < max := 
 Which[OptionValue[Subintervals] === Uniform, Range[min, max, (max - min)/n], 
  OptionValue[Subintervals] === Nonuniform, 
  Join[{min}, Sort[RandomReal[{min, max}, n - 1]], {max}]]

intervals[min_?NumericQ, max_?NumericQ, n_Integer, opts : OptionsPattern[]] /;
   min < max := Partition[N@ binVals[min, max, n, opts], 2, 1]

lftRools[fun_, 
  var_Symbol] := {a_?NumericQ, b_?NumericQ} :> {{a, 0}, {b, fun /. var :> a}}
  
lftRools2[fun1_, fun2_, 
  var_Symbol] := {a_?NumericQ, 
   b_?NumericQ} :> {{a, fun1 /. var :> a}, {b, fun2 /. var :> a}}

rtRools[fun_, 
  var_Symbol] := {a_?NumericQ, b_?NumericQ} :> {{a, 0}, {b, fun /. var :> b}}
  
rtRools2[fun1_, fun2_, 
  var_Symbol] := {a_?NumericQ, 
   b_?NumericQ} :> {{a, fun1 /. var :> b}, {b, fun2 /. var :> b}}

cntRools[fun_, 
  var_Symbol] := {a_?NumericQ, 
   b_?NumericQ} :> {{a, 0}, {b, fun /. var :> Mean[{a, b}]}}
   
cntRools2[fun1_, fun2_, 
  var_Symbol] := {a_?NumericQ, 
   b_?NumericQ} :> {{a, fun1 /. var :> Mean[{a, b}]}, {b, fun2 /. var :> Mean[{a, b}]}}
   
rtlftRools[fun1_, fun2_, 
  var_Symbol] := {a_?NumericQ, 
   b_?NumericQ} :> {{a, fun1 /. var :> b}, {a, fun2 /. var :> a}, {b, 
    fun2 /. var :> a}, {b, fun1 /. var :> b}}

lftrtRools[fun1_, fun2_, 
  var_Symbol] := {a_?NumericQ, 
   b_?NumericQ} :> {{a, fun1 /. var :> a}, {a, fun2 /. var :> b}, {b, 
    fun2 /. var :> b}, {b, fun1 /. var :> a}}

trpRools[fun_, 
  var_Symbol] := {a_?NumericQ, 
   b_?NumericQ} :> {{a, 0}, {a, fun /. var :> a}, {b, fun /. var :> b}, {b, 0}}

    

trpRools2[fun1_, fun2_, 
  var_Symbol] := {a_?NumericQ, 
   b_?NumericQ} :> {{a, fun1 /. var :> a}, {a, fun2 /. var :> a}, {b, 
    fun2 /. var :> b}, {b, fun1 /. var :> b}}
    
R11[col_List] := {{a_, z1_}, {a_, z2_}, {b_, y2_}, {b_, y1_}} /; 
   Abs[z1 - z2] < 1/10^12 && 
    y2 < y1 :> {col[[1]], {{a, z1}, {b, y2}, {b, y1}}} 

R12[col_List] := {{a_, z1_}, {a_, z2_}, {b_, y2_}, {b_, y1_}} /; 
   Abs[z1 - z2] < 1/10^12 && 
    y2 > y1 :> {col[[2]], {{a, z1}, {b, y2}, {b, y1}}} 


R21[col_List] := {{a_, z1_}, {a_, z2_}, {b_, y2_}, {b_, y1_}} /; 
   y2 > y1 && z1 > z2 :> 
  Sequence[{col[[1]], {{a, z1}, {a, 
      z2}, {(a y1 - a y2 - b z1 + b z2)/(y1 - y2 - z1 + z2), (-y2 z1 +
          y1 z2)/(y1 - y2 - z1 + z2)}}}, {col[[2]], {{(a y1 - a y2 - 
         b z1 + b z2)/(y1 - y2 - z1 + z2), (-y2 z1 + y1 z2)/(y1 - y2 -
          z1 + z2)}, {b, y2}, {b, y1}}}]

R22[col_List] := {{a_, z1_}, {a_, z2_}, {b_, y2_}, {b_, y1_}} /; 
   y2 < y1 && z1 < z2 :> Sequence[{col[[2]],
    {{a, z1}, {a, 
      z2}, {(a y1 - a y2 - b z1 + b z2)/(y1 - y2 - z1 + z2), (-y2 z1 +
          y1 z2)/(y1 - y2 - z1 + z2)}}}
   , {col[[1]],
    {{(a y1 - a y2 - b z1 + b z2)/(y1 - y2 - z1 + z2), (-y2 z1 + 
         y1 z2)/(y1 - y2 - z1 + z2)}, {b, y2}, {b, y1}}}] 

R31[col_List] := {{a_, z1_}, {a_, z2_}, {b_, y2_}, {b_, 
     y1_}} /; (y1 > 
      y2 || (Abs[y1 - y2] < 10^-12 && z1 > z2)) :> {col[[1]], {{a, 
     z1}, {a, z2}, {b, y2}, {b, y1}}} 

R32[col_List] := {{a_, z1_}, {a_, z2_}, {b_, y2_}, {b_, 
     y1_}} /; (y1 < 
      y2 || (Abs[y1 - y2] < 10^-12 && z1 < z2)) :> {col[[2]], {{a, 
     z1}, {a, z2}, {b, y2}, {b, y1}}} 
    
    (*Building Polygons*)
    
makeBars[fun_, var_Symbol, ints_?TensorQ, brstyls_List, "Left"] := 
 Module[{ptSets, posSets, negSets},
  Which[Length[brstyls] == 1, Flatten[{brstyls,Rectangle@@@(ints /. lftRools[fun, var])}],
   Length[brstyls] == 2, ptSets = ints /. lftRools[fun, var];
   posSets = Cases[ptSets, xx_ /; xx[[2, 2]] >= 0];
   negSets = Complement[ptSets, posSets];
   Flatten[{brstyls[[1]], Rectangle @@@ posSets, brstyls[[2]], 
     Rectangle @@@ negSets}]
   ]
  ]
  
makeBars[fun_, var_Symbol, ints_?TensorQ, brstyls_List, "Right"] := 
 Module[{ptSets, posSets, negSets},
  Which[Length[brstyls] == 1, Flatten[{brstyls,Rectangle@@@(ints /. rtRools[fun, var])}],
   Length[brstyls] == 2, ptSets = ints /. rtRools[fun, var];
   posSets = Cases[ptSets, xx_ /; xx[[2, 2]] >= 0];
   negSets = Complement[ptSets, posSets];
   Flatten[{brstyls[[1]], Rectangle @@@ posSets, brstyls[[2]], 
     Rectangle @@@ negSets}]
   ]]
   
makeBars[fun_, var_Symbol, ints_?TensorQ, brstyls_List, "Center"] := 
 Module[{ptSets, posSets, negSets},
  Which[Length[brstyls] == 1, Flatten[{brstyls,Rectangle@@@(ints /. cntRools[fun, var])}],
   Length[brstyls] == 2, ptSets = ints /. cntRools[fun, var];
   posSets = Cases[ptSets, xx_ /; xx[[2, 2]] >= 0];
   negSets = Complement[ptSets, posSets];
   Flatten[{brstyls[[1]], Rectangle @@@ posSets, brstyls[[2]], 
     Rectangle @@@ negSets}]
   ]
  ]
  
makeBars2[fun1_, fun2_, var_Symbol, ints_?TensorQ, brstyls_List, 
  "Left"] := 
 Module[{ptSets, posSets, negSets}, 
  Which[Length[brstyls] == 1, 
   Flatten[{brstyls,  Rectangle @@@ (ints /. lftRools2[fun1, fun2, var])}], 
   Length[brstyls] == 2, 
   ptSets = ints /. lftRools2[fun1, fun2, var];
   posSets = Cases[ptSets, xx_ /; xx[[1, 2]] >= xx[[2, 2]]];
   negSets = Complement[ptSets, posSets];
   Flatten[{brstyls[[1]], Rectangle @@@ posSets, brstyls[[2]], 
     Rectangle @@@ negSets}]]]
     
makeBars2[fun1_, fun2_, var_Symbol, ints_?TensorQ, brstyls_List, 
  "Center"] := 
 Module[{ptSets, posSets, negSets}, 
  Which[Length[brstyls] == 1, 
   Flatten[{brstyls,  Rectangle @@@ (ints /. cntRools2[fun1, fun2, var])}], 
   Length[brstyls] == 2, 
   ptSets = ints /. cntRools2[fun1, fun2, var];
   posSets = Cases[ptSets, xx_ /; xx[[1, 2]] >= xx[[2, 2]]];
   negSets = Complement[ptSets, posSets];
   Flatten[{brstyls[[1]], Rectangle @@@ posSets, brstyls[[2]], 
     Rectangle @@@ negSets}]]]
     
makeBars2[fun1_, fun2_, var_Symbol, ints_?TensorQ, brstyls_List, 
  "Right"] := 
 Module[{ptSets, posSets, negSets}, 
  Which[Length[brstyls] == 1, 
   Flatten[{brstyls,  Rectangle @@@ (ints /. rtRools2[fun1, fun2, var])}], 
   Length[brstyls] == 2, 
   ptSets = ints /. rtRools2[fun1, fun2, var];
   posSets = Cases[ptSets, xx_ /; xx[[1, 2]] >= xx[[2, 2]]];
   negSets = Complement[ptSets, posSets];
   Flatten[{brstyls[[1]], Rectangle @@@ posSets, brstyls[[2]], 
     Rectangle @@@ negSets}]]]
     
makePolys[pts_?TensorQ, brstyl_List /; Length[brstyl] == 1] := 
Flatten[{brstyl,Polygon[pts]}]
  
makePolys[pts_?TensorQ, brstyl_List /; Length[brstyl] == 2, 
  collis_List] := 
 Flatten[Map[Join[{#[[1, 1]]}, {Polygon[#[[All, 2]]]}] &, 
    GatherBy[
     pts /. {R11[collis], R12[collis], R21[collis], R22[collis], 
       R31[collis], R32[collis]}, First]] /. Thread[collis -> Reverse[brstyl]]]
  
makePolys2[pts_?TensorQ, brstyl_List/;Length[brstyl]==1] := 
 Flatten[{brstyl,Polygon[pts]}]
  
 makePolys2[pts_?TensorQ, brstyl_List /; Length[brstyl] == 2, 
  collis_List] := 
 Flatten[Map[Join[{#[[1, 1]]}, {Polygon[#[[All, 2]]]}] &, 
    GatherBy[
     pts /. {R11[collis], R12[collis], R21[collis], R22[collis], 
       R31[collis], R32[collis]}, First]] /. Thread[collis -> brstyl]]
 
    
    (*IPlot*)
 
 Options[RiemannSumPlot] = {AlignmentPoint -> Center, AspectRatio -> Automatic, Axes -> True, 
 AxesLabel -> None, AxesOrigin -> Automatic, AxesStyle -> {}, 
 Background -> None, BarStyles-> Automatic,
 BaselinePosition -> Automatic, BaseStyle -> {}, Subintervals-> Uniform,
 ClippingStyle -> None, ColorFunction -> Automatic, 
 ColorFunctionScaling -> True, ColorOutput -> Automatic, 
 ContentSelectable -> Automatic, CoordinatesToolOptions -> Automatic, 
 DisplayFunction :> $DisplayFunction, Epilog -> {}, 
 Evaluated -> Automatic, EvaluationMonitor -> None, 
 Exclusions -> Automatic, ExclusionsStyle -> None, Filling -> None, 
 FillingStyle -> Automatic, FormatType :> TraditionalForm, 
 Frame -> False, FrameLabel -> None, FrameStyle -> {}, 
 FrameTicks -> Automatic, FrameTicksStyle -> {}, GridLines -> None, 
 GridLinesStyle -> {}, ImageMargins -> 0., ImagePadding -> All, 
 ImageSize -> Automatic, ImageSizeRaw -> Automatic, LabelStyle -> {}, 
 MaxRecursion -> Automatic, Mesh -> None, MeshFunctions -> {#1 &}, 
 MeshShading -> None, MeshStyle -> Automatic, Method -> Automatic, 
 PerformanceGoal :> $PerformanceGoal, PlotLabel -> None, 
 PlotPoints -> Automatic, PlotRange -> All, 
 PlotRangeClipping -> False, PlotRangePadding -> Automatic, 
 PlotRegion -> Automatic, PlotStyle -> Automatic, 
 PreserveImageOptions -> Automatic, Prolog -> {}, 
 RegionFunction -> (True &), RotateLabel -> True, Ticks -> Automatic, 
 TicksStyle -> {}, WorkingPrecision -> MachinePrecision}
 
 (*BarStyle rules*)
 
 barRools = {Automatic -> {{EdgeForm[{AbsoluteThickness[.4], Gray}], 
  RGBColor[1, .95, 0, .4]}, {EdgeForm[{AbsoluteThickness[.4], Gray}], 
  RGBColor[.9, 0, 1., .4]}}, 
  None -> {{EdgeForm[{AbsoluteThickness[.4], Gray}], Opacity[0.0]}}, 
  MonoColor -> {{EdgeForm[{AbsoluteThickness[.4], Gray}], 
     RGBColor[1, .95, 0, .4]}}}



(*These are the functions to use in the final package*)
    
RiemannSumPlot[fun_, {var_Symbol, varmin_?NumericQ, varmax_?NumericQ}, 
  n_Integer?Positive, "Left", opts : OptionsPattern[]] := 
 Block[{ints = 
    intervals[varmin, varmax, n, Subintervals -> OptionValue[Subintervals]], recs,
    pltopts, plt, brstyls = OptionValue[BarStyles] /. barRools}, 
  recs = makeBars[fun, var, ints, brstyls, "Left"];
  pltopts = FilterRules[{opts}, Options[Plot]];
  Show[{Graphics[{recs}], 
    plt[fun, {var, varmin, varmax}, pltopts] /. plt -> Plot}, 
   FilterRules[{opts}, Options[Graphics]], 
   FilterRules[Options[RiemannSumPlot], Options[Graphics]]]]
   
RiemannSumPlot[{fun1_, fun2_},{var_Symbol, varmin_?NumericQ, varmax_?NumericQ}, 
  n_Integer?Positive, "Left", opts : OptionsPattern[]] := 
 Block[{ints = 
    intervals[varmin, varmax, n, Subintervals -> OptionValue[Subintervals]], recs,
    pltopts, plt, brstyls = OptionValue[BarStyles] /. barRools}, 
  recs = makeBars2[fun1,fun2, var, ints, brstyls, "Left"];
  pltopts = FilterRules[{opts}, Options[Plot]];
  Show[{Graphics[{recs}], 
    plt[{fun1,fun2}, {var, varmin, varmax}, pltopts] /. plt -> Plot}, 
   FilterRules[{opts}, Options[Graphics]], 
   FilterRules[Options[RiemannSumPlot], Options[Graphics]]]]
   
RiemannSumPlot[fun_, {var_Symbol, varmin_?NumericQ, varmax_?NumericQ}, 
  n_Integer?Positive, "Center", opts : OptionsPattern[]] := 
 Block[{ints = 
    intervals[varmin, varmax, n, Subintervals -> OptionValue[Subintervals]], recs,
    pltopts, plt, brstyls = OptionValue[BarStyles] /. barRools}, 
  recs = makeBars[fun, var, ints, brstyls, "Center"];
  pltopts = FilterRules[{opts}, Options[Plot]];
  Show[{Graphics[{recs}], 
    plt[fun, {var, varmin, varmax}, pltopts] /. plt -> Plot}, 
   FilterRules[{opts}, Options[Graphics]], 
   FilterRules[Options[RiemannSumPlot], Options[Graphics]]]]
   
RiemannSumPlot[{fun1_, fun2_},{var_Symbol, varmin_?NumericQ, varmax_?NumericQ}, 
  n_Integer?Positive, "Center", opts : OptionsPattern[]] := 
 Block[{ints = 
    intervals[varmin, varmax, n, Subintervals -> OptionValue[Subintervals]], recs,
    pltopts, plt, brstyls = OptionValue[BarStyles] /. barRools}, 
  recs = makeBars2[fun1,fun2, var, ints, brstyls, "Center"];
  pltopts = FilterRules[{opts}, Options[Plot]];
  Show[{Graphics[{recs}], 
    plt[{fun1,fun2}, {var, varmin, varmax}, pltopts] /. plt -> Plot}, 
   FilterRules[{opts}, Options[Graphics]], 
   FilterRules[Options[RiemannSumPlot], Options[Graphics]]]]
   
RiemannSumPlot[fun_, {var_Symbol, varmin_?NumericQ, varmax_?NumericQ}, 
  n_Integer?Positive, "Right", opts : OptionsPattern[]] := 
 Block[{ints = 
    intervals[varmin, varmax, n, Subintervals -> OptionValue[Subintervals]], recs,
    pltopts, plt, brstyls = OptionValue[BarStyles] /. barRools}, 
  recs = makeBars[fun, var, ints, brstyls, "Right"];
  pltopts = FilterRules[{opts}, Options[Plot]];
  Show[{Graphics[{recs}], 
    plt[fun, {var, varmin, varmax}, pltopts] /. plt -> Plot}, 
   FilterRules[{opts}, Options[Graphics]], 
   FilterRules[Options[RiemannSumPlot], Options[Graphics]]]]  
   
RiemannSumPlot[{fun1_, fun2_},{var_Symbol, varmin_?NumericQ, varmax_?NumericQ}, 
  n_Integer?Positive, "Right", opts : OptionsPattern[]] := 
 Block[{ints = 
    intervals[varmin, varmax, n, Subintervals -> OptionValue[Subintervals]], recs,
    pltopts, plt, brstyls = OptionValue[BarStyles] /. barRools}, 
  recs = makeBars2[fun1,fun2, var, ints, brstyls, "Right"];
  pltopts = FilterRules[{opts}, Options[Plot]];
  Show[{Graphics[{recs}], 
    plt[{fun1,fun2}, {var, varmin, varmax}, pltopts] /. plt -> Plot}, 
   FilterRules[{opts}, Options[Graphics]], 
   FilterRules[Options[RiemannSumPlot], Options[Graphics]]]] 
   
RiemannSumPlot[{fun1_, fun2_}, {var_Symbol, varmin_?NumericQ, 
   varmax_?NumericQ}, n_Integer?Positive, "LeftRight", 
  opts : OptionsPattern[]] := 
 Block[{ints = 
    intervals[varmin, varmax, n, 
      Subintervals -> OptionValue[Subintervals]] /. 
     lftrtRools[fun1, fun2, var], recs, pltopts, plt, 
   brstyls = OptionValue[BarStyles] /. barRools},
  recs = If[Length[brstyls]==2,makePolys2[ints, brstyls,{colOne,colTwo}],makePolys2[ints, brstyls]];
  pltopts = FilterRules[{opts}, Options[Plot]];
  Show[{Graphics[recs], 
    plt[{fun1, fun2}, {var, varmin, varmax}, pltopts] /. plt -> Plot},
    FilterRules[{opts}, Options[Graphics]], 
   FilterRules[Options[RiemannSumPlot], Options[Graphics]]]]
   
RiemannSumPlot[{fun1_, fun2_}, {var_Symbol, varmin_?NumericQ, 
   varmax_?NumericQ}, n_Integer?Positive, "RightLeft", 
  opts : OptionsPattern[]] := 
 Block[{ints = 
    intervals[varmin, varmax, n, 
      Subintervals -> OptionValue[Subintervals]] /. 
   rtlftRools[fun1, fun2, var], recs, pltopts, plt, 
   brstyls = OptionValue[BarStyles] /. barRools},
  recs = If[Length[brstyls]==2,makePolys2[ints, brstyls,{colOne,colTwo}],makePolys2[ints, brstyls]];
  pltopts = FilterRules[{opts}, Options[Plot]];
  Show[{Graphics[recs], 
    plt[{fun1, fun2}, {var, varmin, varmax}, pltopts] /. plt -> Plot},
    FilterRules[{opts}, Options[Graphics]], 
   FilterRules[Options[RiemannSumPlot], Options[Graphics]]]]
   
RiemannSumPlot[fun_, {var_Symbol, varmin_?NumericQ, varmax_?NumericQ}, 
  n_Integer?Positive, "Trapezoid", opts : OptionsPattern[]] := 
 Block[{ints = 
    intervals[varmin, varmax, n, Subintervals -> OptionValue[Subintervals]]/.trpRools[fun,var], recs,
    pltopts, plt, brstyls = OptionValue[BarStyles] /. barRools}, 
  recs = If[Length[brstyls]==2,makePolys[ints, brstyls,{colOne,colTwo}],makePolys[ints, brstyls]];
  pltopts = FilterRules[{opts}, Options[Plot]];
  Show[{Graphics[recs], 
    plt[fun, {var, varmin, varmax}, pltopts] /. plt -> Plot}, 
   FilterRules[{opts}, Options[Graphics]], 
   FilterRules[Options[RiemannSumPlot], Options[Graphics]]]] 
   
   
RiemannSumPlot[{fun1_, fun2_}, {var_Symbol, varmin_?NumericQ, 
   varmax_?NumericQ}, n_Integer?Positive, "Trapezoid", 
  opts : OptionsPattern[]] := 
 Block[{ints = 
    intervals[varmin, varmax, n, 
      Subintervals -> OptionValue[Subintervals]] /.  
     trpRools2[fun1, fun2, var], recs, pltopts, plt, 
   brstyls = OptionValue[BarStyles] /. barRools},
  recs = If[Length[brstyls]==2,makePolys2[ints, brstyls,{colOne,colTwo}],makePolys2[ints, brstyls]];
  pltopts = FilterRules[{opts}, Options[Plot]];
  Show[{Graphics[recs], 
    plt[{fun1, fun2}, {var, varmin, varmax}, pltopts] /. plt -> Plot},
    FilterRules[{opts}, Options[Graphics]], 
   FilterRules[Options[RiemannSumPlot], Options[Graphics]]]] 
   
RiemannSumPlot[aa_, bb_List, nn_Integer, tt_String, rr___, pp : Except[_?OptionQ], 
  qq___] /;Message[RiemannSumPlot::nonopt,pp,4,RiemannSumPlot]:=Null
   
RiemannSumPlot[arg___/;Length[{arg}]<4]/;Message[General::argrx,RiemannSumPlot,Length[{arg}],4]:=Null

(*Integral Estimation with RiemannSum*)

Options[RiemannSum] = {Subintervals-> Uniform,Estimate-> Automatic}

RiemannSum[fun_, {var_Symbol, varmin_?NumericQ, varmax_?NumericQ}, 
  n_Integer?Positive, "Left", opts : OptionsPattern[]] := 
 Block[{ff = Function[var, fun, {Listable}], 
   lis = binVals[varmin, varmax, n, 
      Subintervals -> OptionValue[Subintervals]], 
   difs},
  difs = Differences[lis];
  Which[OptionValue[Estimate]===Automatic,Simplify[Total[difs ff[Most[lis]]]],
  	OptionValue[Estimate]===Area,Simplify[Total[Abs[difs ff[Most[lis]]]]]]]

RiemannSum[fun_, {var_Symbol, varmin_?NumericQ, varmax_?NumericQ}, 
  n_Integer?Positive, "Right", opts : OptionsPattern[]] := 
 Block[{ff = Function[var, fun, {Listable}], 
   lis = binVals[varmin, varmax, n, 
      Subintervals -> OptionValue[Subintervals]], 
   difs},
  difs = Differences[lis];
  Which[OptionValue[Estimate]===Automatic,Simplify[Total[difs ff[Rest[lis]]]],
  	OptionValue[Estimate]===Area,Simplify[Total[Abs[difs ff[Rest[lis]]]]]]]

RiemannSum[fun_, {var_Symbol, varmin_?NumericQ, varmax_?NumericQ}, 
  n_Integer?Positive, "Center", opts : OptionsPattern[]] := 
 Block[{ff = Function[var, fun, {Listable}], 
   lis = binVals[varmin, varmax, n, 
      Subintervals -> OptionValue[Subintervals]], 
   difs},
  difs = Differences[lis];
  Which[OptionValue[Estimate]===Automatic,Simplify[Total[difs ff[(Most[lis] + Rest[lis])/2]]],
  	OptionValue[Estimate]===Area,Simplify[Total[Abs[difs ff[(Most[lis] + Rest[lis])/2]]]]]]

RiemannSum[fun_, {var_Symbol, varmin_?NumericQ, varmax_?NumericQ}, 
  n_Integer?Positive, "Trapezoid", opts : OptionsPattern[]] := 
 Block[{ff = Function[var, fun, {Listable}], 
   lis = binVals[varmin, varmax, n, 
      Subintervals -> OptionValue[Subintervals]], 
   difs},
  difs = Differences[lis];
  Which[OptionValue[Estimate]===Automatic,Simplify[Total[difs (Mean /@ Partition[ff[lis], 2, 1])]],
  	OptionValue[Estimate]===Area,Simplify[Total[Abs[difs (Mean /@ Partition[ff[lis], 2, 1])]]]]]
  
RiemannSum[{fun1_, fun2_}, {var_Symbol, varmin_?NumericQ, 
   varmax_?NumericQ}, n_Integer?Positive, "Left", opts : OptionsPattern[]] := 
 Block[{ff1 = Function[var, fun1 - fun2, {Listable}], 
   lis = binVals[varmin, varmax, n, 
      Subintervals -> OptionValue[Subintervals]], 
   difs},
  difs = Differences[lis];
  Which[OptionValue[Estimate]===Automatic,Simplify[Total[difs (ff1[Most[lis]])]],
  	OptionValue[Estimate]===Area,Simplify[Total[Abs[difs (ff1[Most[lis]])]]]]]

RiemannSum[{fun1_, fun2_}, {var_Symbol, varmin_?NumericQ, 
   varmax_?NumericQ}, n_Integer?Positive, "Right", opts : OptionsPattern[]] := 
 Block[{ff1 = Function[var, fun1 - fun2, {Listable}], 
   lis = binVals[varmin, varmax, n, 
      Subintervals -> OptionValue[Subintervals]], 
   difs},
  difs = Differences[lis];
  Which[OptionValue[Estimate]===Automatic,Simplify[Total[difs (ff1[Rest[lis]])]],
  	OptionValue[Estimate]===Area,Simplify[Total[Abs[difs (ff1[Rest[lis]])]]]]]

RiemannSum[{fun1_, fun2_}, {var_Symbol, varmin_?NumericQ, 
   varmax_?NumericQ}, n_Integer?Positive, "Center", opts : OptionsPattern[]] := 
 Block[{ff1 = Function[var, fun1 - fun2, {Listable}], 
   lis = binVals[varmin, varmax, n, 
      Subintervals -> OptionValue[Subintervals]], 
   difs},
  difs = Differences[lis];
  Which[OptionValue[Estimate]===Automatic,Simplify[Total[difs (ff1[(Most[lis] + Rest[lis])/2])]],
  	OptionValue[Estimate]===Area,Simplify[Total[Abs[difs (ff1[(Most[lis] + Rest[lis])/2])]]]]]

RiemannSum[{fun1_, fun2_}, {var_Symbol, varmin_?NumericQ, 
   varmax_?NumericQ}, n_Integer?Positive, "Trapezoid", opts : OptionsPattern[]] := 
 Block[{ff = Function[var, fun1 - fun2, {Listable}], 
   lis = binVals[varmin, varmax, n, 
      Subintervals -> OptionValue[Subintervals]], 
   difs},
  difs = Differences[lis];
  Which[OptionValue[Estimate]===Automatic,Simplify[Total[difs (Mean /@ Partition[ff[lis], 2, 1])]],
  	OptionValue[Estimate]===Area,Simplify[Total[Abs[difs (Mean /@ Partition[ff[lis], 2, 1])]]]]]
  	
(*Messages for the symbolic end points*)

RiemannSum::subint = "The value for Subintervals must be Uniform when \
symbolic values for the endpoints of the interval are used."
  	
(*Left with left end required to be a symbol*)

RiemannSum[fun_, {var_Symbol, varmin_Symbol, varmax : _Symbol | _?NumericQ}, 
  n_Integer?Positive, "Left", opts : OptionsPattern[]] := 
 Block[{sbint = OptionValue[Subintervals], est = OptionValue[Estimate], 
   ff = Function[var, fun], dif = (varmax - varmin)/n},
  Which[sbint =!= Uniform, Message[RiemannSum::subint]; 
   StringForm["RiemannSum[`1`,{`2`,`3`,`4`},`5`,`6`,`7`]", fun, var, varmin, 
    varmax, n, "\"Left\"", opts],
   est === Automatic,
   dif Total[(ff /@ Table[varmin + k dif, {k, 0, n - 1}])],
   est === Area,
   dif Total[(Abs[ff /@ Table[varmin + k dif, {k, 0, n - 1}]])]]
  ]

(*Left with right end required to be a symbol*)

RiemannSum[fun_, {var_Symbol, varmin : _Symbol | _?NumericQ, varmax_Symbol}, 
  n_Integer?Positive, "Left", opts : OptionsPattern[]] := 
 Block[{sbint = OptionValue[Subintervals], est = OptionValue[Estimate], 
   ff = Function[var, fun], dif = (varmax - varmin)/n},
  Which[sbint =!= Uniform, Message[RiemannSum::subint]; 
   StringForm["RiemannSum[`1`,{`2`,`3`,`4`},`5`,`6`,`7`]", fun, var, varmin, 
    varmax, n, "\"Left\"", opts],
   est === Automatic,
   dif Total[(ff /@ Table[varmin + k dif, {k, 0, n - 1}])],
   est === Area,
   dif Total[(Abs[ff /@ Table[varmin + k dif, {k, 0, n - 1}]])]]
  ]

(*Center with left end required to be a symbol*)

RiemannSum[fun_, {var_Symbol, varmin_Symbol, varmax : _Symbol | _?NumericQ}, 
  n_Integer?Positive, "Center", opts : OptionsPattern[]] := 
 Block[{sbint = OptionValue[Subintervals], est = OptionValue[Estimate], 
   ff = Function[var, fun], dif = (varmax - varmin)/n},
  Which[sbint =!= Uniform, Message[RiemannSum::subint]; 
   StringForm["RiemannSum[`1`,{`2`,`3`,`4`},`5`,`6`,`7`]", fun, var, varmin, 
    varmax, n, "\"Left\"", opts],
   est === Automatic,
   dif Total[(ff /@ Table[varmin + dif/2 + k dif, {k, 0, n - 1}])],
   est === Area,
   dif Total[(Abs[ff /@ Table[varmin + dif/2 + k dif, {k, 0, n - 1}]])]]
  ]

(*Center with right end required to be a symbol*)

RiemannSum[fun_, {var_Symbol, varmin : _Symbol | _?NumericQ, varmax_Symbol}, 
  n_Integer?Positive, "Center", opts : OptionsPattern[]] := 
 Block[{sbint = OptionValue[Subintervals], est = OptionValue[Estimate], 
   ff = Function[var, fun], dif = (varmax - varmin)/n},
  Which[sbint =!= Uniform, Message[RiemannSum::subint]; 
   StringForm["RiemannSum[`1`,{`2`,`3`,`4`},`5`,`6`,`7`]", fun, var, varmin, 
    varmax, n, "\"Left\"", opts],
   est === Automatic,
   dif Total[(ff /@ Table[varmin + dif/2 + k dif, {k, 0, n - 1}])],
   est === Area,
   dif Total[(Abs[ff /@ Table[varmin + dif/2 + k dif, {k, 0, n - 1}]])]]
  ]

(*Right with left end required to be a symbol*)

RiemannSum[fun_, {var_Symbol, varmin_Symbol, varmax : _Symbol | _?NumericQ}, 
  n_Integer?Positive, "Right", opts : OptionsPattern[]] := 
 Block[{sbint = OptionValue[Subintervals], est = OptionValue[Estimate], 
   ff = Function[var, fun], dif = (varmax - varmin)/n},
  Which[sbint =!= Uniform, Message[RiemannSum::subint]; 
   StringForm["RiemannSum[`1`,{`2`,`3`,`4`},`5`,`6`,`7`]", fun, var, varmin, 
    varmax, n, "\"Left\"", opts],
   est === Automatic,
   dif Total[(ff /@ Table[varmin + k dif, {k, 1, n}])],
   est === Area,
   dif Total[(Abs[ff /@ Table[varmin + k dif, {k, 1, n}]])]]
  ]

(*Right with right end required to be a symbol*)

RiemannSum[fun_, {var_Symbol, varmin : _Symbol | _?NumericQ, varmax_Symbol}, 
  n_Integer?Positive, "Right", opts : OptionsPattern[]] := 
 Block[{sbint = OptionValue[Subintervals], est = OptionValue[Estimate], 
   ff = Function[var, fun], dif = (varmax - varmin)/n},
  Which[sbint =!= Uniform, Message[RiemannSum::subint]; 
   StringForm["RiemannSum[`1`,{`2`,`3`,`4`},`5`,`6`,`7`]", fun, var, varmin, 
    varmax, n, "\"Left\"", opts],
   est === Automatic,
   dif Total[(ff /@ Table[varmin + k dif, {k, 1, n}])],
   est === Area,
   dif Total[(Abs[ff /@ Table[varmin + k dif, {k, 1, n}]])]]
  ]

(*Trapezoid with left end required to be a symbol*)

RiemannSum[fun_, {var_Symbol, varmin_Symbol, varmax : _Symbol | _?NumericQ}, 
  n_Integer?Positive, "Trapezoid", opts : OptionsPattern[]] := 
 Block[{sbint = OptionValue[Subintervals], est = OptionValue[Estimate], 
   ff = Function[var, fun], dif = (varmax - varmin)/n},
  Which[sbint =!= Uniform, Message[RiemannSum::subint]; 
   StringForm["RiemannSum[`1`,{`2`,`3`,`4`},`5`,`6`,`7`]", fun, var, varmin, 
    varmax, n, "\"Left\"", opts],
   est === Automatic,
   dif With[{diff = dif, ff2 = ff, \[Alpha] = varmin,  
      nn = n},
      Sum[(ff2[\[Alpha] + (i + 1) diff] + ff2[\[Alpha] + i diff])/2, {i, 0, 
       nn - 1}]],
   est === Area,
   dif With[{diff = dif, ff2 = ff, \[Alpha] = varmin,
      nn = n},
      Sum[Abs[(ff2[\[Alpha] + (i + 1) diff] + ff2[\[Alpha] + i diff])/2], {i, 
       0, nn - 1}]]]
  ]

(*Trapezoid with right end required to be a symbol*)

RiemannSum[fun_, {var_Symbol, varmin : _Symbol | _?NumericQ, varmax_Symbol}, 
  n_Integer?Positive, "Trapezoid", opts : OptionsPattern[]] := 
 Block[{sbint = OptionValue[Subintervals], est = OptionValue[Estimate], 
   ff = Function[var, fun], dif = (varmax - varmin)/n},
  Which[sbint =!= Uniform, Message[RiemannSum::subint]; 
   StringForm["RiemannSum[`1`,{`2`,`3`,`4`},`5`,`6`,`7`]", fun, var, varmin, 
    varmax, n, "\"Left\"", opts],
   est === Automatic,
   dif With[{diff = dif, ff2 = ff, \[Alpha] = varmin, 
      nn = n},
      Sum[(ff2[\[Alpha] + (i + 1) diff] + ff2[\[Alpha] + i diff])/2, {i, 0, 
       nn - 1}]],
   est === Area,
   dif With[{diff = dif, ff2 = ff, \[Alpha] = varmin, 
      nn = n},
      Sum[Abs[(ff2[\[Alpha] + (i + 1) diff] + ff2[\[Alpha] + i diff])/2], {i, 
       0, nn - 1}]]]
  ]
  	
RiemannSum[aa_, bb_List, nn_Integer, tt_String, rr___, pp : Except[_?OptionQ], 
  qq___] /;Message[RiemannSum::nonopt,pp,4,RiemannSum]:=Null
   
RiemannSum[arg___/;Length[{arg}]<4]/;Message[General::argrx,RiemannSum,Length[{arg}],4]:=Null
      
End[]

SetAttributes[{SubstituteIntegrate,AutoSimplify,PartsIntegrate,RiemannSumPlot,RiemannSum},{Protected,ReadProtected}]



