(* Mathematica Package *)

(* Created by the Wolfram Workbench Jan 5, 2010 *)





MultiplyARow::usage = 
  "MultiplyARow[mat ,n ,mult] multiplies the nth row of the \
matrix mat by mult.";

AddToRow::usage = 
  "AddToRow[mat,row, addrow, mult] adds to row of mat mult*addrow].";

SwitchRows::usage = 
  "SwitchRows[mat,rowA,rowB]switches the positions of rowA and rowB \
in the matrix mat.";

CreateAugmentedMatrix::usage = 
  "CreateAugmentedMatrix[eqns,vars] creates an augmented matrix from \
a set of equations eqns involving the variables vars.";

GetColumns::usage = 
  "GetColumns[mat, n] gives the nth column vector of the matrix mat. \
GetColumns[mat, {n,m}] gives columns n and m.";

AugmentIdentityMatrix::usage = 
  "AugmentIdentityMatrix[mat] augments the square matrix mat with the \
identity matrix of the same dimension.";

CreateEquations::usage = 
  "CreateEquations[mat,vars] turns a matrix mat into a set of \
equations in the variables vars.";

Begin["`Private`"]
(*Implementation of the package*)

Unprotect[MultiplyARow, AddToRow, SwitchRows, CreateAugmentedMatrix, \
GetColumns, AugmentIdentityMatrix, CreateEquations]

MultiplyARow[mat_?MatrixQ, row_Integer, val_] := 
 Module[{len = Length[mat], newmat}, newmat = IdentityMatrix[len];
  newmat[[row]] = val newmat[[row]];
  newmat.mat]

AddToRow[mat_?MatrixQ, row_Integer, addrow_Integer, val_] := 
 Module[{len = Length[mat], newmat}, newmat = IdentityMatrix[len];
  newmat[[row, addrow]] = val;
  newmat.mat]

SwitchRows[mat_?MatrixQ, row1_Integer, row2_Integer] := 
 Module[{len = Length[mat], newmat, lis}, newmat = IdentityMatrix[len];
  lis = Range[len] /. {row1 -> row2, row2 -> row1};
  newmat = newmat[[lis]];
  newmat.mat]

(*CreateAugmentedMatrix::eqnum = 
  "The number of equations must be the same as the number of \
variables.";

CreateAugmentedMatrix[eqs : {_Equal ..}, vars_List] /; 
  Length[eqs] == Length[vars] := 
 Module[{coeFun, coefficient, fun}, 
  coeFun = fun[xx, 
     Table[coefficient[xx, vars[[i]]], {i, Length[vars]}]] /. {fun -> 
      Function, coefficient -> Coefficient};
  MapThread[
   Join, {Map[coeFun, eqs[[All, 1]]], List /@ eqs[[All, -1]]}]]

CreateAugmentedMatrix[___] /; 
  Message[CreateAugmentedMatrix::eqnum] := Null*)
  
CreateAugmentedMatrix[lis_List, vars_] := 
 MapThread[Join[#2, {#1}] &, Normal[CoefficientArrays[lis, vars]]]

GetColumns::baddim = 
  "The second argument of GetColumns, `1`, should be a positive \
integer between 1 and the number of columns of the matrix.";

GetColumns[mat_, n_Integer] := 
 If[n < 1 || n > Length[Transpose[mat]], 
  Message[GetColumns::baddim, n], Part[mat, All, n]]

GetColumns[mat_, cols_List] := Part[mat, All, cols]

GetColumns[mat_, cols_Span] := Part[mat, All, cols]

CreateEquations::eqnum = 
  "The number of columns in the matrix `1` must be one greater than the number of \
variables `2`.";

CreateEquations[mat_?MatrixQ, vars_List] /; 
 Length[vars]==Dimensions[mat][[2]]-1 := 
 Map[Equal[Total[Most[#]], #[[-1]]] &, 
  Map[MapThread[Times, {#, Join[vars, {1}]}] &, mat]]

CreateEquations[m_,v_] /; Message[CreateEquations::eqnum,m ,v] := Null

AugmentIdentityMatrix::baddim = 
  "The argument to AugmentIdentityMatrix should be a square matrix.";

AugmentIdentityMatrix[mat_] := 
 If[Apply[Equal, Dimensions[mat]], 
  MapThread[Join, {mat, IdentityMatrix[Length[mat]]}], 
  Message[AugmentIdentityMatrix::baddim]]

SetAttributes[{MultiplyARow, AddToRow, SwitchRows, CreateAugmentedMatrix, \
GetColumns, AugmentIdentityMatrix, CreateEquations},{Protected,ReadProtected}]

End[]





