(* Mathematica Package *)

(* Created by the Wolfram Workbench Oct 4, 2007 *)

(*:Mathematica Version: 6.0 *)

(*:Package Version: 1.1 *)

(*:Name: PiecewisePlots` *)

(*:Context:PiecewisePlots` *)

(*:Title: PiecewisePlots *)

(*:Authors: harry Calkins and Adam Berry *)

(*:History:Created July 2005*)

(*Major rewrite of code Oct. 2005 to make options more versitlle and eliminate some plot errors.*)

(*As suggested by DarrenG. Added Assumptions and Refine to deal with issues arising from
PiecewiseExpand.*)

(*Oct 2007:Added some changes to account for limits not evaluating correctly.*)

(*July 2009:Added some code to insure break points at the end of the plot interval are rendered*)

(*July 2009:Added PiecewisePolarPlot*)

(*:Copyright: Copyright 2005-2009, Wolfram Research, Inc. *)

(*:Reference: Usage messages only. *)

(*:Summary: This package provides functionality for textbook style plotting of
piecewise continuous functions.*)

(*:Keywords: PiecewisePlot *)

(*:Requirements: No special system requirements. *)


(*:Limitations: PiecewiseExpand can introduce extraneous break points.*)

(*The extraneous points issue should be corrected with the introduction of Refine.*)



PiecewisePlot::usage= "PiecewisePlot[fn, {var,varmin,varmax},options] generates a textbook style plot of a piecewise continuous function where the function fn is defined using Piecewise.\nPiecewisePlot[{fn1,fn2,...},{var,varmin,varmax},opts] generates a plot of a number of functions which can be a mix of ordinary functions and Piecewise functions. PiecewisePlot will honor any options you can use with Plot."

PiecewisePolarPlot::usage = "PiecewisePolarPlot[fun,{var,varmin,varmax},opts] creates a polar plot of a piecewise defined function using standard textbook notation of open and filled points denoting the ends of continuous segments."

OddRootPlot::usage="OddRootPlot[fun,{var,varmin,varmax}] will plot functions containing expressions raised to fractional exponents with the denominator of the exponent an odd number over both positive and negative values for the base. The function will take a list of functions and will accept any options that can be used with Plot." 
 
Unprotect[PiecewisePlot,PiecewisePolarPlot,PointStyle];

Begin["`Private`"]

(*These functions locate the break points and intervals of definition.*)

ProcessPcws[g_Piecewise, {var_, varmin_, varmax_}] := 
 Module[{ineqs, brks, varsol}, 
  ineqs = Cases[
    g, (_Less | _LessEqual | _Equal | _Inequality | _Greater | \
_GreaterEqual), Infinity];
  brks = Cases[(Union@
       Cases[Flatten[
         ineqs /. 
          Thread[{Less, LessEqual, Equal, Inequality, Greater, 
             GreaterEqual} -> List]], _?
         NumericQ]) /. {a__} :> {varmin, a, varmax}, 
    x_ /; varmin <= x <= varmax];
  varsol = Cases[ineqs, aa_ /; Not[FreeQ[aa, var]], 2][[1]];
  sols=Join[{varmin},var /. Flatten[Map[Solve[varsol == #, var] &, brks[[2 ;; -2]]], 1],{varmax}]
  ]

ProcessPcws[fn_, {var_, varmin_, varmax_}] := 
 Union[Flatten@(Simplify`PWPlotCases[
      Evaluate[fn], {var, varmin, varmax}] /. {_, {{_, a_, b_}}} :> {a, b})];


(*These functions determine the line segments, points and non-points in the plots*)


GetPcwsParts[fn_, {var_, lwr_?NumericQ, upr_?NumericQ}, 
   opts___?OptionQ] := 
  Module[{aa, fnpts, limpts, repts, nonpts}, 
   aa = ProcessPcws[fn, {var, lwr, upr}][[2 ;; -2]];
   fnpts = 
    DeleteCases[Map[{#, fn /. var :> #} &, aa], 
      xx_ /; Or[Not[FreeQ[xx, Indeterminate]], 
        Not[FreeQ[xx, ComplexInfinity]],Not[FreeQ[xx,Infinity]]]];
   limpts = 
    DeleteCases[
     Flatten[
       Map[{{#, Limit[fn, var -> #, Direction -> 1]}, {#, 
           Limit[fn, var -> #, Direction -> -1]}} &, aa], 1], 
     xx_ /; Or[Not[FreeQ[xx, Indeterminate]], 
       Not[FreeQ[xx, ComplexInfinity]],Not[FreeQ[xx,Infinity]]]];
   repts = 
    Union[Join[fnpts, limpts], 
     SameTest -> (N@Max[Abs[#1 - #2]] < 10^(-8) &)];
   nonpts = 
    Complement[limpts, fnpts, 
     SameTest -> (N@Max[Abs[#1 - #2]] < 10^(-8) &)];
   {Point[N@repts], Point[N@nonpts]}];
   
(*This allows edge points to be rendered*)

intervalAdjust[mn_?NumericQ, mx_?NumericQ, scl_?NumericQ] := 
 Module[{newscl = (mx - mn) scl}, 
  Rationalize[{mn - newscl, mx + newscl}]]

(*These functions are used to process the options*)

ptTest[lis_List] := FreeQ[lis, _AbsolutePointSize|_PointSize];

(*Definitions for point styles*)

getPointOpts[pltopt_, Except[Automatic, ptopt_List]] := ptopt

getPointOpts[pltopt_, Except[Automatic, ptopt_]] := {ptopt}

getPointOpts[Except[_List, pltopt_], Automatic] := 
 Cases[{pltopt}, (_AbsolutePointSize | _PointSize | _RGBColor | \
_CMYKColor | _Hue | _GrayLevel | None | Automatic)]

getPointOpts[pltopt_List, Automatic] := 
 Cases[pltopt, (_AbsolutePointSize | _PointSize | _RGBColor | \
_CMYKColor | _Hue | _GrayLevel | None | Automatic)]

ptSizeRules = {AbsolutePointSize[Large] :> AbsolutePointSize[Medium], 
  AbsolutePointSize[Medium] :> AbsolutePointSize[Small], 
  AbsolutePointSize[Small] :> AbsolutePointSize[Tiny], 
  AbsolutePointSize[sz_] :> AbsolutePointSize[2 sz/3],
  PointSize[Large] :> PointSize[Medium],
  PointSize[Medium] :> PointSize[Small],
  PointSize[Small] :> PointSize[Tiny],
  PointSize[sz_] :> PointSize[2 sz/3]}
  
ptStyle[None, pts_, i_Integer?Positive] := {Opacity[0.0], pts}

ptStyle[{None}, pts_, i_Integer?Positive] := {Opacity[0.0], pts}

ptStyle[{}, pts_, i_Integer?Positive] := ptStyle[Automatic,pts,i]

ptStyle[Directive[None], pts_, i_Integer?Positive] := {Opacity[0.0], pts}

ptStyle[{Automatic}, pts_List, i_Integer?Positive] := 
 Flatten[Riffle[{{ColorData[1][i], AbsolutePointSize[6]}, {RGBColor[1, 1, 1], 
     AbsolutePointSize[4]}}, pts], 1]
     
ptStyle[Automatic, pts_List, i_Integer?Positive] := 
 Which[FreeQ[OptionValue[PlotStyle], Automatic] && 
   Length[optAdjust[PiecewisePlot, PlotStyle]] == 1, 
  Flatten[Riffle[{{optAdjust[PiecewisePlot, PlotStyle], 
      AbsolutePointSize[6]}, {RGBColor[1, 1, 1], 
      AbsolutePointSize[4]}}, pts], 1],
  FreeQ[OptionValue[PlotStyle], Automatic],
  Flatten[
   Riffle[{{OptionValue[PlotStyle][i], 
      AbsolutePointSize[6]}, {RGBColor[1, 1, 1], 
      AbsolutePointSize[4]}}, pts], 1],
  True,
  Flatten[
   Riffle[{{ColorData[1][i], 
      AbsolutePointSize[6]}, {RGBColor[1, 1, 1], 
      AbsolutePointSize[4]}}, pts], 1]]

ptStyle[Automatic, pts_List, i_Integer?Positive] := 
 Flatten[Riffle[{{ColorData[1][i], AbsolutePointSize[6]}, {RGBColor[1, 1, 1], 
     AbsolutePointSize[4]}}, pts], 1]
     
ptStyle[{Automatic}, pts_List, i_Integer?Positive] := 
 Flatten[Riffle[{{ColorData[1][i], AbsolutePointSize[6]}, {RGBColor[1, 1, 1], 
     AbsolutePointSize[4]}}, pts], 1]
     
ptStyle[Directive[Automatic], pts_List, i_Integer?Positive] := 
 Flatten[Riffle[{{ColorData[1][i], AbsolutePointSize[6]}, {RGBColor[1, 1, 1], 
     AbsolutePointSize[4]}}, pts], 1]

ptStyle[col : _RGBColor | _CMYKColor | _Hue | _GrayLevel, pts_List, 
  i_Integer?Positive] := 
 Flatten[Riffle[{{col, AbsolutePointSize[6]}, {RGBColor[1, 1, 1], 
     AbsolutePointSize[4]}}, pts], 1]
     
ptStyle[{col : _RGBColor | _CMYKColor | _Hue | _GrayLevel}, pts_List, 
  i_Integer?Positive] := 
 Flatten[Riffle[{{col, AbsolutePointSize[6]}, {RGBColor[1, 1, 1], 
     AbsolutePointSize[4]}}, pts], 1]
     
ptStyle[Directive[col : _RGBColor | _CMYKColor | _Hue | _GrayLevel], pts_List, 
  i_Integer?Positive] := 
 Flatten[Riffle[{{col, AbsolutePointSize[6]}, {RGBColor[1, 1, 1], 
     AbsolutePointSize[4]}}, pts], 1]

ptStyle[{col1 : _RGBColor | _CMYKColor | _Hue | _GrayLevel, 
   col2 : _RGBColor | _CMYKColor | _Hue | _GrayLevel}, pts_List, 
  i_Integer?Positive] := 
 Flatten[Riffle[{{col1, AbsolutePointSize[6]}, {col2, AbsolutePointSize[4]}}, 
   pts], 1]

ptStyle[{{col1 : _RGBColor | _CMYKColor | _Hue | _GrayLevel}, {col2 : \
_RGBColor | _CMYKColor | _Hue | _GrayLevel}}, pts_List, i_Integer?Positive] :=
  Flatten[Riffle[{{col1, AbsolutePointSize[6]}, {col2, AbsolutePointSize[4]}},
    pts], 1]

ptStyle[ptsz : _AbsolutePointSize | _PointSize, pts_List, 
  i_Integer?Positive] := 
 Flatten[Riffle[{{ColorData[1][i], ptsz}, {RGBColor[1, 1, 1], 
     ptsz /. ptSizeRules}}, pts], 1]
     
ptStyle[{ptsz : _AbsolutePointSize | _PointSize}, pts_List, 
  i_Integer?Positive] := 
 Flatten[Riffle[{{ColorData[1][i], ptsz}, {RGBColor[1, 1, 1], 
     ptsz /. ptSizeRules}}, pts], 1]

ptStyle[{ptsz : _AbsolutePointSize | _PointSize, 
   col : _RGBColor | _CMYKColor | _Hue | _GrayLevel}, pts_List, 
  i_Integer?Positive] := 
 Flatten[Riffle[{{col, ptsz}, {RGBColor[1, 1, 1], 
     ptsz /. ptSizeRules}}, pts], 1]
     
ptStyle[{{ptsz : _AbsolutePointSize | _PointSize, 
   col : _RGBColor | _CMYKColor | _Hue | _GrayLevel}}, pts_List, 
  i_Integer?Positive] := 
 Flatten[Riffle[{{col, ptsz}, {RGBColor[1, 1, 1], 
     ptsz /. ptSizeRules}}, pts], 1]

ptStyle[{col : _RGBColor | _CMYKColor | _Hue | _GrayLevel, 
   ptsz : _AbsolutePointSize | _PointSize}, pts_List, i_Integer?Positive] := 
 Flatten[Riffle[{{col, ptsz}, {RGBColor[1, 1, 1], 
     ptsz /. ptSizeRules}}, pts], 1]
     
ptStyle[{{col : _RGBColor | _CMYKColor | _Hue | _GrayLevel, 
   ptsz : _AbsolutePointSize | _PointSize}}, pts_List, i_Integer?Positive] := 
 Flatten[Riffle[{{col, ptsz}, {RGBColor[1, 1, 1], 
     ptsz /. ptSizeRules}}, pts], 1]

ptStyle[{{ptsz : _AbsolutePointSize | _PointSize, 
    col : _RGBColor | _CMYKColor | _Hue | _GrayLevel}, Automatic}, pts_List, 
  i_Integer?Positive] := 
 Flatten[Riffle[{{col, ptsz}, {RGBColor[1, 1, 1], 
     ptsz /. ptSizeRules}}, pts], 1]

ptStyle[{{col : _RGBColor | _CMYKColor | _Hue | _GrayLevel, 
    ptsz : _AbsolutePointSize | _PointSize}, Automatic}, pts_List, 
  i_Integer?Positive] := 
 Flatten[Riffle[{{col, ptsz}, {RGBColor[1, 1, 1], 
     ptsz /. ptSizeRules}}, pts], 1]

ptStyle[{{col : _RGBColor | _CMYKColor | _Hue | _GrayLevel, 
    ptsz1 : _AbsolutePointSize | _PointSize}, {ptsz2 : _AbsolutePointSize | \
_PointSize}}, pts_List, i_Integer?Positive] := 
 Flatten[Riffle[{{col, ptsz1}, {RGBColor[1, 1, 1], ptsz2}}, pts], 1]

ptStyle[{{ptsz1 : _AbsolutePointSize | _PointSize, 
    col : _RGBColor | _CMYKColor | _Hue | _GrayLevel}, {ptsz2 : \
_AbsolutePointSize | _PointSize}}, pts_List, i_Integer?Positive] := 
 ptStyle[{{col, ptsz1}, {ptsz2}}, pts, i]

ptStyle[{{col1 : _RGBColor | _CMYKColor | _Hue | _GrayLevel, 
    ptsz1 : _AbsolutePointSize | _PointSize}, {col2 : _RGBColor | _CMYKColor \
| _Hue | _GrayLevel}}, pts_List, i_Integer?Positive] := 
 Flatten[Riffle[{{col1, ptsz1}, {col2, 
     ptsz1 /. ptSizeRules}}, pts], 1]

ptStyle[{{ptsz1 : _AbsolutePointSize | _PointSize, 
    col1 : _RGBColor | _CMYKColor | _Hue | _GrayLevel}, {col2 : _RGBColor | \
_CMYKColor | _Hue | _GrayLevel}}, pts_List, i_Integer?Positive] := 
 ptStyle[{{col1, ptsz1}, {col2}}, pts, 1]

ptStyle[{{ptsz1 : _AbsolutePointSize | _PointSize}, {col : _RGBColor | \
_CMYKColor | _Hue | _GrayLevel, ptsz2 : _AbsolutePointSize | _PointSize}}, 
  pts_List, i_Integer?Positive] := 
 Flatten[Riffle[{{ptsz1}, {col, ptsz2}}, pts], 1]

ptStyle[{{ptsz1 : _AbsolutePointSize | _PointSize}, {ptsz2 : \
_AbsolutePointSize | _PointSize, 
  
 ptStyle[{{col1, ptsz1}, {col2, ptsz2}}, pts, i]  col : _RGBColor | _CMYKColor | _Hue | _GrayLevel}}, pts_List, 
  i_Integer?Positive] := ptStyle[{{ptsz1}, {col, ptsz2}}, pts, i]

ptStyle[{{col1 : _RGBColor | _CMYKColor | _Hue | _GrayLevel, 
    ptsz1 : _AbsolutePointSize | _PointSize}, {col2 : _RGBColor | _CMYKColor \
| _Hue | _GrayLevel, ptsz2 : _AbsolutePointSize | _PointSize}}, pts_List, 
  i_Integer?Positive] := 
 Flatten[Riffle[{{col1, ptsz1}, {col2, ptsz2}}, pts], 1]

ptStyle[{{col1 : _RGBColor | _CMYKColor | _Hue | _GrayLevel, 
    ptsz1 : _AbsolutePointSize | _PointSize}, {ptsz2 : _AbsolutePointSize | \
_PointSize, col2 : _RGBColor | _CMYKColor | _Hue | _GrayLevel}}, pts_List, 
  i_Integer?Positive] := ptStyle[{{col1, ptsz1}, {col2, ptsz2}}, pts, i]

ptStyle[{{ptsz1 : _AbsolutePointSize | _PointSize, 
    col1 : _RGBColor | _CMYKColor | _Hue | _GrayLevel}, {ptsz2 : \
_AbsolutePointSize | _PointSize, 
    col2 : _RGBColor | _CMYKColor | _Hue | _GrayLevel}}, pts_List, 
  i_Integer?Positive] := ptStyle[{{col1, ptsz1}, {col2, ptsz2}}, pts, i]

ptStyle[{{ptsz1 : _AbsolutePointSize | _PointSize, 
    col1 : _RGBColor | _CMYKColor | _Hue | _GrayLevel}, {col2 : _RGBColor | \
_CMYKColor | _Hue | _GrayLevel, ptsz2 : _AbsolutePointSize | _PointSize}}, 
  pts_List, i_Integer?Positive] := 
 
 (*These are the definitions for the PiecewisePlot functions.*)
 Attributes[PiecewisePlot] = {HoldAll}


Options[PiecewisePlot]= 
  Sort[Join[Options[
    Plot],{PointStyle \[Rule] Automatic}]];
       
       
PiecewisePlot[fn_/;Head[fn]=!=List,{var_Symbol,varmin_?NumericQ,varmax_?NumericQ},opts:OptionsPattern[]]:= 
 Module[{newvar,newfun,preptstyl, repts, nonpts, vrMn,vrMx},
 	With[{var=newvar},
   preptstyl= getPointOpts[OptionValue[PlotStyle],OptionValue[PointStyle]];
  {vrMn,vrMx} = intervalAdjust[varmin, varmax, 5/1000];
  newfun= If[Head[fn]===Piecewise,fn,PiecewiseExpand[fn,Assumptions-> vrMn<var<vrMx]];
  {repts,nonpts}=Quiet@GetPcwsParts[newfun,{var,vrMn,vrMx}];
  Show[{Plot[Evaluate[newfun],{var,varmin,varmax},
     Evaluate[FilterRules[{opts},Options[Plot]]]],Graphics[ptStyle[preptstyl,{repts,nonpts},1]]}]]
  ];
  
  

PiecewisePlot[
  fn_List, {var_Symbol, varmin_?NumericQ, varmax_?NumericQ}, 
  opts : OptionsPattern[]] := 
 Module[{newvar, newfun,len = Length[fn],  pltPoints, vrMn, vrMx, pltstyl, ptstyl, ptOpts},
  With[{var = newvar},
{pltstyl,ptstyl}={OptionValue[PlotStyle],OptionValue[PointStyle]}/.Directive-> List;
{pltstyl,ptstyl}={Flatten[{pltstyl},1],Flatten[{ptstyl},1]};
ptOpts=If[ptstyl==={Automatic},PadRight[pltstyl,len,pltstyl],PadRight[ptstyl,len,ptstyl]];
ptOpts=Map[If[Head[#]===List,#,{#}]&,ptOpts];
ptOpts=If[ptstyl==={Automatic},Map[Cases[#,(_AbsolutePointSize | _PointSize | _RGBColor | \
_CMYKColor | _Hue | _GrayLevel|None|Automatic)]&,ptOpts],ptOpts];
{vrMn, vrMx} = intervalAdjust[varmin, varmax, 5/1000];
newfun= Map[If[Head[#]===Piecewise,#,PiecewiseExpand[#,Assumptions->vrMn<var<vrMx]]&,fn];
pltPoints = 
  Table[ptStyle[ptOpts[[i]], Quiet@GetPcwsParts[newfun[[i]],{var,vrMn,vrMx}], 
    i], {i, len}];
Show[{Plot[Evaluate[newfun], {var, varmin, varmax}, 
   Evaluate[FilterRules[{opts}, Options[Plot]]]], 
  Graphics[pltPoints]}]
]

]

Options[PiecewisePolarPlot]=Join[{PointStyle-> Automatic},Options[PolarPlot]]

Attributes[PiecewisePolarPlot]= {HoldAll}

PiecewisePolarPlot[fn_/;Head[fn]=!=List,{var_Symbol,varmin_?NumericQ,varmax_?NumericQ},opts:OptionsPattern[]]:= 
 Module[{newvar,newfun,preptstyl, repts, nonpts, vrMn,vrMx},
 	With[{var=newvar},
 preptstyl= getPointOpts[OptionValue[PlotStyle],OptionValue[PointStyle]];
  {vrMn,vrMx} = intervalAdjust[varmin, varmax, 5/1000];
  newfun= If[Head[fn]===Piecewise,fn,PiecewiseExpand[fn,Assumptions-> vrMn<var<vrMx]];
  {repts,nonpts}=Quiet@GetPcwsParts[newfun,{var,vrMn,vrMx}];
  Show[{PolarPlot[Evaluate[newfun],{var,varmin,varmax},
     Evaluate[FilterRules[{opts},Options[PolarPlot]]]],Graphics[ptStyle[preptstyl,
     	{repts,nonpts}/. {a_Real, b_Real} :> b {Cos[a], Sin[a]},1]]}]]
  ];
  
PiecewisePolarPlot[
  fn_List, {var_Symbol, varmin_?NumericQ, varmax_?NumericQ}, 
  opts : OptionsPattern[]] := 
 Module[{newvar,newfun, len = Length[fn],  pltPoints, vrMn, vrMx, pltstyl, ptstyl, ptOpts},
  With[{var = newvar},
{pltstyl,ptstyl}={OptionValue[PlotStyle],OptionValue[PointStyle]}/.Directive-> List;
{pltstyl,ptstyl}={Flatten[{pltstyl},1],Flatten[{ptstyl},1]};
ptOpts=If[ptstyl==={Automatic},PadRight[pltstyl,len,pltstyl],PadRight[ptstyl,len,ptstyl]];
ptOpts=Map[If[Head[#]===List,#,{#}]&,ptOpts];
ptOpts=If[ptstyl==={Automatic},Map[Cases[#,(_AbsolutePointSize | _PointSize | _RGBColor | \
_CMYKColor | _Hue | _GrayLevel|None|Automatic)]&,ptOpts],ptOpts];
{vrMn, vrMx} = intervalAdjust[varmin, varmax, 5/1000];
newfun= Map[If[Head[#]===Piecewise,#,PiecewiseExpand[#,Assumptions->vrMn<var<vrMx]]&,fn];
pltPoints = 
  Table[ptStyle[ptOpts[[i]], Quiet@GetPcwsParts[newfun[[i]],{var,vrMn,vrMx}], 
    i], {i, len}];
Show[{PolarPlot[Evaluate[newfun], {var, varmin, varmax}, 
   Evaluate[FilterRules[{opts}, Options[PolarPlot]]]], 
  Graphics[pltPoints/. {a_Real, b_Real} :> b {Cos[a], Sin[a]}]}]
]

]

(*A function to convert expressions containing odd roots.*)

oddRoot[(a_)^(Rational[b_Integer, (n_)?OddQ])] := 
  If[a >= 0, a^(b/n), (-(-a)^(1/n))^b]
  
OddRootPlot[fun_, {var_, varmin_, varmax_}, 
  opts : OptionsPattern[Plot]] := Block[{pltfun},
  pltfun[dumvar_Symbol] := 
   fun /. Power[a_, Rational[b_Integer, n_?OddQ]] :> 
     oddRoot[a^(b/n)];
  Plot[Evaluate@pltfun[var], {var, varmin, varmax}, opts]]

End[]

SetAttributes[{PiecewisePlot, PointStyle,PiecewisePolarPlot},{Protected,ReadProtected}];



