


(* Exported symbols added here with SymbolName::usage *)  

KSeparateSamples::usage = "KSeparateSamples[lis,{sz,n}] extracts n disjoint samples of size sz from lis."

CreatePopulation::usage="CreatePopulation[img,n] takes a list of images or graphics and generates a list that will place the images in a SRSChart."

SRSChart::usage = "SRSChart[popsz,{smplsz,n}] creates a population of size popsz and highlights n simple random samples of size smplsz." 

Happy::usage="Happy[col] creates a happy face with color col. The default color is yellow."

Sleepy::usage="Sleepy[col] [col] creates a sleepy happy face with color col. The default color is yellow."

Oh::usage="Oh[col] [col] creates a startled happy face with color col. The default color is yellow."

Grumpy::usage="Grumpy[col] [col] creates a grumpy happy face with color col. The default color is yellow."

MakeColorList::usage= "MakeColorList[\"nm\",colrs] creates a function value for ColorList[\"nm\"] to render a sequence of colors specified by colrs."

ColorList::usage= "ColorList[\"nm\"] returns the ColorListfunction for the value \"nm\"."

ColorListFunction::usage= "ColorFunction[rng,colrs,fn] is a value returned by ColorList."

SampleColorFunction::usage="SampleColorFunction is an option for SRSChart that controls how wample are represented."

MatStyle::usage="MatStyle is an option to set the color of the background disk in SRSChart."

HexagonScale::usage="HexagonScale is an option for SRSChart used to set the size of the hexagons in the chart."

TrapezoidalDistribution::usage = "TrapezoidalDistribution[{min, max},{modemin, modemax}] represents a trapezoidal distribution giving values between min and max and having a maximum on the interval modemin to modemax."

FiveNumberSummary::usage = "FiveNumberSummary[lis] returns the minimum, twenty-fifth percentile, median, seventy-fifth percentile and the maximum of a numerical vector."

FiveNumberSummaryTable::usage = "FiveNumberSummaryTable[data] creates a formatted table of five number summaries for the data vectors."

ColumnHeadings::usage = "ColumnHeadings is an option for FiveNumberSummaryTable."

MakePlotBands::usage = 
  "MakePlotBands[fm[\"bands\"] ,cl ,var ] creates a pure function from the FittedModel fm with the confidence level cl using the variable var used in creating the fitted model fm."

MeanConfidenceChart::usage = "MeanConfidenceChart[data,cnflvl,samplsize,trials] plots trials confidence intervale for the estimate of the meand of data with a sample size os samplesz. Data is assumed normally distributed. "

Marker::usage = "Marker[i] creates the ith marker in the set of twenty built-in markers. \
Marker[i,col] creates the ith marker from the set of twenty and renders it with the color col."

DataStyle::usage = "DataStyle is an option for ModelPlot which specifies the style of the data. \
The values for DataStyle will apply to DataMarkers that are not graphic objects, Text for example."

DataMarkers::usage = "DataMarkers is an option for ModelPlot which specifies the PlotMarkers to 
use for the data. DataMarkers can take any value that can be used with the option PlotMarkers in
ListPlot. DataMarkers consisting of Graphics objects should include a built-in style."

FittedModelPlot::usage = "FittedModelPlot[data, funs,var, opts] plots one or more sets of data \
together with a set of functions fitted to the data. Options can include any options for \
ListPlot and Plot."

RollTheDice::usage = "RollTheDice[dice, \"properties\"] returns the requested property for the list of dice specified."

TenSidedDie

Begin["`Private`"] (* Begin Private Context *) 

(*A function to create a hexagonal grid or hexagons*)

hexGrid[n_Integer?NonNegative, scl_ /; 0 < scl <= 1] := 
 Block[{pts = 
    Table[scl {Cos[t], Sin[t]}, {t, \[Pi]/6., 
      2 \[Pi] + \[Pi]/6, \[Pi]/3}], diagPts, grdPts},
  diagPts = 
   Table[TranslationTransform[k {Sqrt[3]/2, 3/2}][pts], {k, -n, n}];
  grdPts = 
   Reverse@ 
    Flatten[Table[
      Which[k <= n, 
       Table[TranslationTransform[{j Sqrt[3], 0}][diagPts[[k]]], {j, 
         n, 1 - k, -1}],
       k == n + 1, 
       Reverse@Table[
         TranslationTransform[{j Sqrt[3], 0}][diagPts[[k]]], {j, -n, 
          n}],
       k > n + 1, 
       Reverse@Table[
         TranslationTransform[{j Sqrt[3], 0}][diagPts[[k]]], {j, -n, 
          2 n - k + 1}]], {k, 1, 2 n + 1}], 1]]

(*Two functions to create a grid of centers of the hexagons in the hexgrid*)

hexPoints[hexlis_?ArrayQ] := Map[Mean[Most[#]] &, hexlis]

hexPoints[n_Integer?Positive] := Block[{diagPts, grdPts},
  diagPts = 
   Table[TranslationTransform[k {Sqrt[3]/2, 3/2}][{0, 0}], {k, -n, n}];
  grdPts = 
   Reverse@ 
    Flatten[Table[
      Which[k <= n, 
       Table[TranslationTransform[{j Sqrt[3], 0}][diagPts[[k]]], {j, 
         n, 1 - k, -1}],
       k == n + 1, 
       Reverse@Table[
         TranslationTransform[{j Sqrt[3], 0}][diagPts[[k]]], {j, -n, 
          n}],
       k > n + 1, 
       Reverse@Table[
         TranslationTransform[{j Sqrt[3], 0}][diagPts[[k]]], {j, -n, 
          2 n - k + 1}]], {k, 1, 2 n + 1}], 1]]
          
(*Functions for generating samples from a list of values*)

KSeparateSamples[data_List, {sz_Integer?Positive, 
   smps_Integer?Positive}] /; smps <= IntegerPart[Length[data]/sz] := 
 Partition[RandomSample[data,sz smps],sz]
 
KSeparateSamples[data_List, sz_Integer?Positive]:= KSeparateSamples[data,{sz,1}]

KSeparateSamples[data_List, {sz_Integer?Positive}]:= KSeparateSamples[data,{sz,1}]

KSeparateSamples[data_List, {sz_Integer?Positive, 
   smps_Integer?Positive}] /; 
  Message[KSeparateSamples::toomny, sz, 
   IntegerPart[Length[data]/sz]] := Null

KSeparateSamples::toomny = 
  "You have requested more disjoint samples of size `1` than can be \
obtained from the data. At most `2` samples of the requested size can \
be obtained.";

(*==========================================================================*)
(*==========================================================================*)

(*ColorList functions*)

(*ColorList is intended to work like ColorDataFunction. i don't want to try to overload the ColorDatafunction
as it make use of a paclet and i suspect that could lead to problems on down the line.*)

makePanel[colLis : {(_RGBColor | _GrayLevel) ..}] := 
 Panel[Style[
   Graphics[
    MapIndexed[{#, EdgeForm[Opacity[0.1]], 
       Rectangle[{#2[[1]], 0}, Offset[{-1, -1}, {#2[[1]] + 1, 1}]]} &,
      colLis], AspectRatio -> 1/10, 
        ImageSize -> 65, PlotRangePadding -> None], 
   CacheGraphics -> False] , FrameMargins -> 5]
   
MakeColorList[nm_String, colLis : {(_RGBColor | _GrayLevel) ..}] := 
 With[{len = Length[colLis]},
  ColorList[nm] = 
   ColorListFunction[{1, len, 1}, colLis, 
    Function[colLis[[Mod[#, len, 1]]]]]]
   
Format[ColorListFunction[dom_, cols_, __]] := 
 ColorListFunction[dom, makePanel[cols]]
   
   
ColorListFunction[rnglis_List, cols_List, fun_Function][
  i_Integer?Positive] := fun[i]


(*==========================================================================*)
(*==========================================================================*)

(*Functions to create a population. These are intended for use in the main function SRSChart*)
    
(*For images*)    
    
CreatePopulation[lis_/; VectorQ[lis, ImageQ], popsz_Integer?Positive] /; 
  Length[lis] >= popsz := 
 Block[{newlis= (Graphics[Raster[Reverse@ImageData[#]]]& /@ lis),n = Ceiling[ 1/6 (-3 + Sqrt[3] Sqrt[-1 + 4 popsz])]}, 
 MapThread[
   Inset[#1, #2, Center,1.2] &, {Take[newlis, popsz], 
    Take[hexPoints[n], popsz]}]]
    
CreatePopulation[lis_/; VectorQ[lis, ImageQ], popsz_Integer?Positive,scl_?Positive] /; 
  Length[lis] >= popsz := 
 Block[{newlis= (Graphics[Raster[Reverse@ImageData[#]]]& /@ lis),n = Ceiling[ 1/6 (-3 + Sqrt[3] Sqrt[-1 + 4 popsz])]}, 
 MapThread[
   Inset[#1, #2, Center, scl] &, {Take[newlis, popsz], 
    Take[hexPoints[n], popsz]}]]

CreatePopulation[lis_/; VectorQ[lis, ImageQ], popsz_Integer?Positive] /; 
  Length[lis] < popsz := 
 Block[{newlis=(Graphics[Raster[Reverse@ImageData[#]]]& /@ lis),n = Ceiling[ 1/6 (-3 + Sqrt[3] Sqrt[-1 + 4 popsz])]}, 
 MapThread[
   Inset[#1, #2, Center, 1.2] &, {PadRight[ newlis, popsz, newlis], 
    Take[hexPoints[n], popsz]}]]
    
CreatePopulation[lis_/; VectorQ[lis, ImageQ], popsz_Integer?Positive,scl_?Positive] /; 
  Length[lis] < popsz := 
 Block[{newlis=(Graphics[Raster[Reverse@ImageData[#]]]& /@ lis),n = Ceiling[ 1/6 (-3 + Sqrt[3] Sqrt[-1 + 4 popsz])]}, 
 MapThread[
   Inset[#1, #2, Center, scl] &, {PadRight[ newlis, popsz, newlis], 
    Take[hexPoints[n], popsz]}]]
    
(*the general case*)   

CreatePopulation[lis_List]  := 
 Block[{newlis= (Graphics /@ lis),popsz= Length[lis],n}, 
 n = Ceiling[ 1/6 (-3 + Sqrt[3] Sqrt[-1 + 4 popsz])];
 MapThread[
   Inset[#1, #2, Center,1.2] &, {Take[newlis, popsz], 
    Take[hexPoints[n], popsz]}]] 
    
CreatePopulation[lis_List]  := 
 Block[{newlis= (Graphics /@ lis),popsz= Length[lis],n}, 
 n = Ceiling[ 1/6 (-3 + Sqrt[3] Sqrt[-1 + 4 popsz])];
 MapThread[
   Inset[#1, #2, Center,1.2] &, {Take[newlis, popsz], 
    Take[hexPoints[n], popsz]}]] 

CreatePopulation[lis_List, popsz_Integer?Positive] /; 
  Length[lis] >= popsz := 
 Block[{newlis= (Graphics /@ lis),n = Ceiling[ 1/6 (-3 + Sqrt[3] Sqrt[-1 + 4 popsz])]}, 
 MapThread[
   Inset[#1, #2, Center,1.2] &, {Take[newlis, popsz], 
    Take[hexPoints[n], popsz]}]]
    
CreatePopulation[lis_List, popsz_Integer?Positive,scl_?Positive] /; 
  Length[lis] >= popsz := 
 Block[{newlis= (Graphics /@ lis),n = Ceiling[ 1/6 (-3 + Sqrt[3] Sqrt[-1 + 4 popsz])]}, 
 MapThread[
   Inset[#1, #2, Center, scl] &, {Take[newlis, popsz], 
    Take[hexPoints[n], popsz]}]]

CreatePopulation[lis_List, popsz_Integer?Positive] /; 
  Length[lis] < popsz := 
 Block[{newlis=(Graphics /@ lis),n = Ceiling[ 1/6 (-3 + Sqrt[3] Sqrt[-1 + 4 popsz])]}, 
 MapThread[
   Inset[#1, #2, Center, 1.2] &, {PadRight[ newlis, popsz, newlis], 
    Take[hexPoints[n], popsz]}]]
    
CreatePopulation[lis_List, popsz_Integer?Positive,scl_?Positive] /; 
  Length[lis] < popsz := 
 Block[{newlis=(Graphics /@ lis),n = Ceiling[ 1/6 (-3 + Sqrt[3] Sqrt[-1 + 4 popsz])]}, 
 MapThread[
   Inset[#1, #2, Center, scl] &, {PadRight[ newlis, popsz, newlis], 
    Take[hexPoints[n], popsz]}]]
    
(*The SRSChart functions*)

Options[SRSChart] = {AlignmentPoint -> Center, AspectRatio -> Automatic, Axes -> False, 
 AxesLabel -> None, AxesOrigin -> Automatic, AxesStyle -> {}, 
 Background -> None, BaselinePosition -> Automatic, BaseStyle -> {}, 
 ChartStyle -> Automatic, 
 ColorOutput -> Automatic, ContentSelectable -> Automatic, 
 CoordinatesToolOptions -> Automatic, 
 DisplayFunction :> $DisplayFunction, Epilog -> {}, 
 FormatType :> TraditionalForm, Frame -> False, FrameLabel -> None, 
 FrameStyle -> {}, FrameTicks -> Automatic, FrameTicksStyle -> {}, 
 GridLines -> None, GridLinesStyle -> {}, HexagonScale -> 0.91, 
 ImageMargins -> 0., ImagePadding -> All, ImageSize -> Automatic, 
 ImageSizeRaw -> Automatic, LabelStyle -> {}, 
 MatStyle -> GrayLevel[0.75], Method -> Automatic, PlotLabel -> None, 
 PlotRange -> All, PlotRangeClipping -> False, 
 PlotRangePadding -> Automatic, PlotRegion -> Automatic, 
 PreserveImageOptions -> Automatic, Prolog -> {}, RotateLabel -> True,
  SampleColorFunction -> Automatic, Ticks -> Automatic, TicksStyle -> {}}

SRSChart[popsz_Integer?Positive,opts:OptionsPattern[]]:= 
Block[{n = Ceiling[ 1/6 (-3 + Sqrt[3] Sqrt[-1 + 4 popsz])], hxgrd,pltstyl=(OptionValue[ChartStyle] ),
	smplstyl=OptionValue[SampleColorFunction],matstyl=(OptionValue[MatStyle]/.None-> Opacity[0]),hxscl=OptionValue[HexagonScale]},
  {pltstyl,smplstyl,matstyl}={pltstyl/.Automatic->RGBColor[0.15, 0.15, 0.7],
  	smplstyl/.(Automatic-> (Directive[Thickness[1/(10 (2 n + 1))], ColorData[3][#1+1]] &)),
  	matstyl/.(Automatic-> GrayLevel[0.75]) };
  hxgrd = Take[hexGrid[n, hxscl], popsz];
  Graphics[{{matstyl, Disk[{0, 0}, 1.025 ((2 n + 1) Sqrt[3/4])]},
     pltstyl, Polygon[hxgrd]},FilterRules[{opts}, Options[Graphics]]]]

SRSChart[popsz_Integer?Positive, {srs_Integer?NonNegative, 
    num_Integer?Positive}, opts : OptionsPattern[]] /; srs < popsz := 
 Block[{n = Ceiling[ 1/6 (-3 + Sqrt[3] Sqrt[-1 + 4 popsz])], hxgrd, 
   smpl, smpllis,pltstyl=OptionValue[ChartStyle],
	smplstyl=OptionValue[SampleColorFunction],matstyl=(OptionValue[MatStyle]/.None-> Opacity[0]),hxscl=OptionValue[HexagonScale]},
	 {pltstyl,smplstyl,matstyl}={pltstyl/.Automatic->RGBColor[0.15, 0.15, 0.7],
  	smplstyl/.(Automatic-> (Directive[Thickness[1/(10 (2 n + 1))], ColorData[3][#1+1]] &)),
  	matstyl/.(Automatic-> GrayLevel[0.75]) };
  hxgrd = Take[hexGrid[n, hxscl], popsz];
  smpl = KSeparateSamples[hxgrd, {srs, num}];
  smpllis = 
   Table[{EdgeForm[
     smplstyl[i]], 
     Polygon[smpl[[i]]]}, {i, num}];
  Graphics[{{matstyl, Disk[{0, 0}, 1.025 ((2 n + 1) Sqrt[3/4])]},
     pltstyl, 
    Polygon[Complement[hxgrd, Sequence @@ smpl]], smpllis},FilterRules[{opts}, Options[Graphics]]]]
    
SRSChart[popsz_Integer?Positive, srs_Integer?NonNegative, 
   opts : OptionsPattern[]] /; srs < popsz := SRSChart[popsz,{srs,1},opts]
   
SRSChart[popsz_Integer?Positive, {srs_Integer?NonNegative}, 
   opts : OptionsPattern[]] /; srs < popsz := SRSChart[popsz,{srs,1},opts]
   
SRSChart[poplis_List, 
    opts : OptionsPattern[]] := 
 Block[{popsz = Length[poplis], n, hxgrd,pltstyl=OptionValue[ChartStyle],
	smplstyl=OptionValue[SampleColorFunction],matstyl=(OptionValue[MatStyle]/.None-> Opacity[0]),hxscl=OptionValue[HexagonScale]},
	 {pltstyl,smplstyl,matstyl}={pltstyl/.Automatic->RGBColor[0.15, 0.15, 0.7],
  	smplstyl/.(Automatic-> (Directive[Thickness[1/(10 (2 n + 1))], ColorData[3][#1+1]] &)),
  	matstyl/.(Automatic-> GrayLevel[0.75]) };
  n = Ceiling[ 1/6 (-3 + Sqrt[3] Sqrt[-1 + 4 popsz])];
  hxgrd = Take[hexGrid[n, hxscl], popsz];
  Graphics[{{matstyl, Disk[{0, 0}, 1.025 ((2 n + 1) Sqrt[3/4])]},pltstyl,  
    Polygon[hxgrd], poplis},FilterRules[{opts}, Options[Graphics]]]]
   

SRSChart[poplis_List, {srs_Integer?NonNegative, num_Integer?Positive},
    opts : OptionsPattern[]] /; srs < Length[poplis] := 
 Block[{popsz = Length[poplis], n, hxgrd, smpl, smpllis,pltstyl=OptionValue[ChartStyle],
	smplstyl=OptionValue[SampleColorFunction],matstyl=(OptionValue[MatStyle]/.None-> Opacity[0]),hxscl=OptionValue[HexagonScale]},
	 {pltstyl,smplstyl,matstyl}={pltstyl/.Automatic->RGBColor[0.15, 0.15, 0.7],
  	smplstyl/.(Automatic-> (Directive[Thickness[1/(10 (2 n + 1))], ColorData[3][#1+1]] &)),
  	matstyl/.(Automatic-> GrayLevel[0.75]) };
  n = Ceiling[ 1/6 (-3 + Sqrt[3] Sqrt[-1 + 4 popsz])];
  hxgrd = Take[hexGrid[n, hxscl], popsz];
  smpl = KSeparateSamples[hxgrd, {srs, num}];
  smpllis = 
   Table[{EdgeForm[
      smplstyl[i]], 
     Polygon[smpl[[i]]]}, {i, num}];
  Graphics[{{matstyl, Disk[{0, 0}, 1.025 ((2 n + 1) Sqrt[3/4])]},
     pltstyl, 
    Polygon[Complement[hxgrd, Sequence @@ smpl]], smpllis, poplis},FilterRules[{opts}, Options[Graphics]]]]
    
  SRSChart[poplis_List,n_Integer?Positive,opts:OptionsPattern[]]:= SRSChart[poplis,{n,1},opts]
  
  SRSChart[poplis_List,{n_Integer?Positive},opts:OptionsPattern[]]:= SRSChart[poplis,{n,1},opts]
    
(*Some faces for use in the Population functions*)


Happy[col_: Yellow] = {EdgeForm[Black], col, Disk[], Black, 
   Thickness[.03], Disk[9 {Cos[\[Pi]/6], Sin[\[Pi]/6]}/16, 1/6], 
   Disk[9 {Cos[5 \[Pi]/6], Sin[5 \[Pi]/6]}/16, 1/6], Red, 
   Thickness[.03], Circle[{0, 0}, 5/8, {-\[Pi]/6, -5 \[Pi]/6}]};

Sleepy[col_: Yellow] = {EdgeForm[Black], col, Disk[], Black, 
   Thickness[.02], 
   Circle[9 {Cos[\[Pi]/6], Sin[\[Pi]/6]}/16, 
    1/6, {-\[Pi]/6, -5 \[Pi]/6}], 
   Circle[9 {Cos[5 \[Pi]/6], Sin[5 \[Pi]/6]}/16, 
    1/6, {-\[Pi]/6, -5 \[Pi]/6}], Red, Thickness[.03], 
   Circle[{0, 0}, 5/8, {-\[Pi]/6, -5 \[Pi]/6}]};

Oh[col_: Yellow] = {EdgeForm[Black], col, Disk[], Black, 
   Thickness[.02], 
   Disk[9 {Cos[\[Pi]/6], Sin[\[Pi]/6]}/16, {1/9, 1/6}], 
   Disk[9 { Cos[5 \[Pi]/6], Sin[\[Pi]/6]}/16, {1/9, 1/6}], 
   Circle[9 {Cos[5 \[Pi]/6], Sin[5 \[Pi]/6]}/16, {1/8, 1/4}, {\[Pi]/3,
      2 \[Pi]/3}], 
   Circle[9 {Cos[\[Pi]/6], Sin[ \[Pi]/6]}/16, {1/8, 1/4}, {\[Pi]/3, 
     2 \[Pi]/3}], Red, Thickness[.02], Circle[{0, -1/2}, {1/5, 1/3}], 
   Black, Disk[{0, -1/2}, {1/5.2, 1/3.2}]};

Grumpy[col_: Yellow] = {EdgeForm[Black], col, Disk[], Black, 
   Thickness[.03], Disk[9 {Cos[\[Pi]/6], Sin[\[Pi]/6]}/16, 1/6], 
   Disk[9 {Cos[5 \[Pi]/6], Sin[5 \[Pi]/6]}/16, 1/6], 
   Circle[9 {Cos[\[Pi]/6], Sin[\[Pi]/6]}/16, 1/4, {\[Pi]/2, \[Pi]}], 
   Circle[9 {Cos[5 \[Pi]/6], Sin[5 \[Pi]/6]}/16, 1/4, {\[Pi]/2, 0}], 
   Red, Thickness[.03], Circle[{0, -7/8}, 5/8, {\[Pi]/6, 5 \[Pi]/6}]};
   
(*=================================================================================*)
(*Distributions listed here can be used without calculus. Almost all calculations can be done using geometry.*)   
(*=================================================================================*)

(*TrapezoidalDistribution*)


TrapezoidalDistribution[{a_,b_},{c_,d_}] := ProbabilityDistribution[Piecewise[
     {{(2*(\[FormalX] - a))/((a - c)*(a - b + c - d)), 
         Inequality[a, LessEqual, \[FormalX], Less, c]}, 
       {-(2/(a - b + c - d)), Inequality[c, LessEqual, 
           \[FormalX], Less, d]}, 
       {-((2*(\[FormalX] - b))/((b - d)*(-a + b - c + d))), 
         Inequality[d, LessEqual, \[FormalX], Less, b]}}, 0], 
   {\[FormalX], a, b}, Assumptions -> Inequality[a, 
       LessEqual, c, Less, d, LessEqual, b]]
       
(*==================================================================================*)
(*FiveNumberSummary is a function to use on vectors *)
(*==================================================================================*)

FiveNumberSummary[vec : {Repeated[(_?NumericQ), {5, Infinity}]}] := 
 Quantile[vec, {0, 1/4, 1/2, 3/4, 1}]
 
(*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*)
(*FiveNumberSummaryTable formats a table of five number summaries*)
(*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*)

processData2[data_List /; Depth[data] == 2, optVal_] := processData2[{data},optVal]

processData2[data_List /; Depth[data] == 3, optVal_] :=
 
 Block[{lbls = {"min", "25%", "median", "75%" , "max"}, 
   vals = FiveNumberSummary /@ data, len = Length[data], hdrs, bdy},
  Which[optVal === None, Reverse@Transpose[Join[{lbls}, vals]],
   Length[optVal] == len,
   hdrs = PadLeft[optVal, len + 1, ""];
   bdy = (Reverse@Transpose[Join[{lbls}, vals]]);
   Join[List[hdrs], bdy],
   True, Message[FiveNumberSummaryTable::colnm];
   Reverse@Transpose[Join[{lbls}, vals]]]]
   
Options[FiveNumberSummaryTable] = {Alignment -> {Center, Baseline}, 
   AllowedDimensions -> Automatic, AllowScriptLevelChange -> True, 
   AutoDelete -> False, Background -> RGBColor[1, 1, 0.85`], 
   BaselinePosition -> Automatic, BaseStyle -> {}, 
   ColumnHeadings -> None, DefaultBaseStyle -> "Grid", 
   DefaultElement -> "\[Placeholder]", DeleteWithContents -> True, 
   Dividers -> Automatic, Editable -> Automatic, Frame -> None, 
   FrameStyle -> Automatic, ItemSize -> Automatic, ItemStyle -> None, 
   Selectable -> Automatic, Spacings -> Automatic};

FiveNumberSummaryTable::colnm = 
  "The length of the column headings list must equal the number of \
data lists. Column headings will be omitted.";

FiveNumberSummaryTable[data_ /; VectorQ[data, NumericQ], 
  opts : OptionsPattern[]] := 
 Block[{grdVals = processData2[data, OptionValue[ColumnHeadings]]},
  Grid[grdVals,
   FilterRules[{opts}, Options[Grid]],
   FilterRules[Options[FiveNumberSummaryTable], Options[Grid]] /. 
    Rule[Dividers, Automatic] :> 
     Rule[Dividers, {{Thick, {True}, Thick}, {Thick, {True}, Thick}}]]]

FiveNumberSummaryTable[data : {Repeated[_List, {2, Infinity}]}, 
  opts : OptionsPattern[]] := 
 Block[{grdVals = processData2[data, OptionValue[ColumnHeadings]]},
  Grid[grdVals, FilterRules[{opts}, Options[Grid]],
   FilterRules[Options[FiveNumberSummaryTable], Options[Grid]] /. 
    Rule[Dividers, Automatic] :> 
     Rule[Dividers, {{Thick, {True}, Thick}, {Thick, {True}, 
        Thick}}]]]
 
(*===================================================================================*) 
(*A function to create a pure function for use in plotting confidence bands.*)
(*===================================================================================*)

SetAttributes[MakePlotBands, HoldFirst]

MakePlotBands[
  ff_[bnd : ("MeanPredictionBands" | "SinglePredictionBands")], 
  conf_?NumericQ, var_Symbol] := 
 Block[{fun}, (fun[(ff[bnd, ConfidenceLevel -> conf] /. 
       var :> #)]) /. fun :> Function]

(*====================================================================================*)
(*Visualizing confidence*)
(*====================================================================================*)

Options[MeanConfidenceChart] = {AlignmentPoint -> Center, 
   AspectRatio -> 1/GoldenRatio, Axes -> None, AxesLabel -> None, 
   AxesOrigin -> Automatic, AxesStyle -> {}, Background -> None, 
   BaselinePosition -> Automatic, BaseStyle -> {}, 
   ClippingStyle -> None, ColorFunction -> Automatic, 
   ColorFunctionScaling -> True, ColorOutput -> Automatic, 
   ContentSelectable -> Automatic, 
   CoordinatesToolOptions -> Automatic, DataRange -> Automatic, 
   Epilog -> {}, Filling-> Automatic, FillingStyle -> RGBColor[0, 0, 1], Frame -> False, 
   FrameLabel -> None, FrameStyle -> {}, FrameTicks -> Automatic, 
   FrameTicksStyle -> {}, GridLines -> Automatic, GridLinesStyle -> {}, ImageMargins -> 0.`, 
   ImagePadding -> All, ImageSize -> Automatic, 
   ImageSizeRaw -> Automatic, InterpolationOrder -> None, 
   Joined -> False, LabelStyle -> {}, MaxPlotPoints -> \[Infinity], 
   Mesh -> None, MeshFunctions -> {#1 &}, MeshShading -> None, 
   MeshStyle -> Automatic, Method -> Automatic, PlotLabel -> None, 
   PlotMarkers -> None, PlotRange -> Automatic, 
   PlotRangeClipping -> True, PlotRangePadding -> {Automatic,{Scaled[.12],Scaled[.12]}}, 
   PlotRegion -> Automatic, 
   PlotStyle -> {Directive[PointSize[0.007`], RGBColor[0, 0, 1]], 
     Directive[PointSize[0.008`], RGBColor[1, 0, 0]], 
     Directive[PointSize[0.008`], RGBColor[1, 0, 0]]}, 
   PreserveImageOptions -> Automatic, Prolog -> {}, 
   RotateLabel -> True, Ticks -> Automatic, TicksStyle -> {}, 
   DisplayFunction :> $DisplayFunction, FormatType :> TraditionalForm,
    PerformanceGoal :> $PerformanceGoal}
    
critValue[num_ /; 0 < num < 1] := -Sqrt[2] InverseErfc[1 + num]

getCurve[mn_?NumericQ, stddev_?Positive, crtval_?Positive, 
  trls_?Positive] := Block[{std = stddev/Sqrt[trls], scl},
  scl = Floor[trls/6] Sqrt[2 Pi] std;
  (Cases[Plot[
      scl PDF[NormalDistribution[0, std], t] {1, 
        Boole[Abs[t] < crtval std]}, {t, -4.0 std, 4.0 std}, 
      Filling -> {2 -> Axis}, 
      Evaluated -> True], _GraphicsComplex] /. {GraphicsComplex[
       pts_List, rst__] :> 
      GraphicsComplex[
       Composition[TranslationTransform[{0, mn}], 
         RotationTransform[Pi/2]][pts], rst]})
  ]
    
MeanConfidenceChart[data_ /; VectorQ[data, NumericQ], 
   conlvl_ /; 0 < conlvl < 1, smplsz_Integer, trials_Integer, 
   opts : OptionsPattern[]] /; smplsz < Length[data] := 
 Block[{datamn = Mean[data], datasd = StandardDeviation[data], 
   crtval = critValue[conlvl], 
   prs = Table[{Mean[#], StandardDeviation[#]} &[
      RandomSample[data, smplsz]], {trials}], datasets,crv},
  crv = getCurve[datamn,datasd,crtval,trials];
  datasets = 
   Transpose[
    prs /. {a_, b_} :> {a,a - ( b crtval) b/Sqrt[smplsz], 
      a + ( b crtval)b/Sqrt[smplsz]}];
  ListPlot[datasets,
   Filling ->OptionValue[Filling]/.Automatic->  {2 -> {3}},
   GridLines -> OptionValue[GridLines]/.Automatic -> {None, {datamn}},
   PlotRange-> OptionValue[PlotRange]/.Automatic-> {{-Ceiling[trials/6],trials+1},Automatic},
   Prolog-> {crv},
   FilterRules[{opts}, Options[MeanConfidenceChart]],
   Options[MeanConfidenceChart]]]
   
(*===============================================================================*)
(*ModelPlot also exists in the applied statistics package so the two definition should be kept in sinc.*)
(*================================================================================*)

(* ModelPlot definitions *)


Options[ModelPlot] = 
  Join[{DataStyle -> Automatic, DataMarkers -> None}, 
   Union[Options[ListPlot], Options[Plot]]]; 

(* code for markers defined by this package *)

triPts = (N[
      Table[{Cos[t], Sin[t]}, {t, #1, 2*Pi + #1, 
        2*(Pi/3)}]] & ) /@ {0, Pi/6, Pi/3, Pi/2}; 

fourPts = (N[
      Table[{Cos[t], Sin[t]}, {t, #1, 2*Pi + #1, Pi/2}]] & ) /@ {0, 
    Pi/4}; 

fivePts = (N[
      Table[{Cos[t], Sin[t]}, {t, #1, 4*Pi + #1, 
        4*(Pi/5)}]] & ) /@ {0, Pi/2, Pi, 3*(Pi/2)}; 

outLns = Line /@ Join[triPts, fourPts, fivePts]; 

polyFrms = Polygon /@ Join[triPts, fourPts, fivePts]; 

shapes = Join[outLns, Reverse[polyFrms]]; 

Marker::invarg = "`1` is not an integer in the range 1 to 20";

Table[{Marker[i] = Graphics[shapes[[i]],PlotRange->{{-1,1},{-1,1}}]; , 
   Marker[i, col_] = Graphics[{col, shapes[[i]]},
   		PlotRange->{{-1,1},{-1,1}}]; }, {i, 20}]; 
   
Marker[i_]/; Message[Marker::invarg,i]:=Null

 
sel[x_] := Head[x] === Symbol && ! NumericQ[x]&& Context[x] =!= "System`";

processData[vec_ /; VectorQ[vec, NumericQ], var_Symbol] := {var, 1, 
  Length[vec]}

processData[mat : {{_?NumericQ, _?NumericQ} ..}, var_Symbol] := 
 Block[{vals = mat[[All, 1]]}, {var, Min[vals], Max[vals]}]

processData[lis : {(_?VectorQ | _?MatrixQ) ..}, var_Symbol] := 
 Block[{prs = 
    Map[If[VectorQ[#], {1, 
        Length[#]}, {Min[#], Max[#]} &[#[[All, 1]]]] &, lis]}, 
  Join[{var}, (Union @@ prs)[[{1, -1}]]]]
  
augmentInt[{s_Symbol, mn_?NumericQ, mx_?NumericQ}, frac_?Positive] := 
 Block[{adj = frac (mx - mn)}, {s, mn - adj, mx + adj}]
  
Options[FittedModelPlot] = {AlignmentPoint -> Center, 
   AspectRatio -> 1/GoldenRatio, Axes -> True, AxesLabel -> None, 
   AxesOrigin -> Automatic, AxesStyle -> {}, Background -> None, 
   BaselinePosition -> Automatic, BaseStyle -> {}, 
   ClippingStyle -> None, ColorFunction -> Automatic, 
   ColorFunctionScaling -> True, ColorOutput -> Automatic, 
   ContentSelectable -> Automatic, 
   CoordinatesToolOptions -> Automatic, DataRange -> Automatic, 
   DisplayFunction :> $DisplayFunction, Epilog -> {}, 
   Evaluated -> True, EvaluationMonitor -> None, 
   Exclusions -> Automatic, ExclusionsStyle -> None, Filling -> None, 
   FillingStyle -> Automatic, FormatType :> TraditionalForm, 
   Frame -> False, FrameLabel -> None, FrameStyle -> {}, 
   FrameTicks -> Automatic, FrameTicksStyle -> {}, GridLines -> None, 
   GridLinesStyle -> {}, ImageMargins -> 0.`, ImagePadding -> All, 
   ImageSize -> Automatic, ImageSizeRaw -> Automatic, 
   InterpolationOrder -> None, Joined -> False, LabelStyle -> {}, 
   MaxPlotPoints -> \[Infinity], MaxRecursion -> Automatic, 
   Mesh -> None, MeshFunctions -> {#1 &}, MeshShading -> None, 
   MeshStyle -> Automatic, Method -> Automatic, 
   PerformanceGoal :> $PerformanceGoal, PlotLabel -> None, 
   PlotMarkers -> None, PlotPoints -> Automatic, 
   PlotRange -> Automatic, PlotRangeClipping -> True, 
   PlotRangePadding -> Automatic, PlotRegion -> Automatic, 
   PlotStyle -> Automatic, PreserveImageOptions -> Automatic, 
   Prolog -> {}, RegionFunction -> (True &), RotateLabel -> True, 
   Ticks -> Automatic, TicksStyle -> {}, 
   WorkingPrecision -> MachinePrecision};
   
FittedModelPlot[data_List, model_, var_Symbol, 
  opts : OptionsPattern[]] := 
 Block[{pltrng = processData[data, var], 
   dataopts = 
    DeleteCases[FilterRules[{opts}, Options[ListPlot]], 
     Rule[Filling, __]]},
  Show[{Plot[model, Evaluate[augmentInt[pltrng, .04]], 
     Evaluate[FilterRules[{opts}, Options[Plot]]], 
     Evaluate[FilterRules[Options[FittedModelPlot], Options[Plot]]]], 
    ListPlot[data, dataopts]}, 
   FilterRules[Options[FittedModelPlot], Options[Show]]]]
   
(*==============================================================================*)
(*Dice rolling functions*)

(*These functions all simulation of dice that would typically be used in game playing*)

(*Auxiliary functions*)
(*==============================================================================*)

Clear[diceOutcomes]

outComes[dice_List, len_Integer] := 
 Flatten[Outer[List, Sequence @@ (Range[#] & /@ dice)], 
  If[len === 1, len, len - 1]]

diceOutcomes[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ], "Outcomes"] := 
 Block[{len = Length[dice]},
  outComes[dice, len]]

diceOutcomes[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ], "OutcomesTable"] := 
 Block[{len = Length[dice], outcomes},
  outcomes = outComes[dice, len];
  Grid[GatherBy[outcomes, First], Frame -> True]]

diceOutcomes[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ], "Distribution"] := 
 Block[{len = Length[dice], outcomes},
  outcomes = outComes[dice, len];
  EmpiricalDistribution[outcomes]]

diceOutcomes[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ], n_Integer] := Block[{samples},
  samples = Map[RandomChoice[Range[#], n] &, dice];
  Transpose[samples]]

diceOutcomesDistribution[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ], n_Integer] := 
 EmpiricalDistribution[diceOutcomes[dice, n]]

Clear[diceTotals, diceTotalsDistribution]

diceTotals[dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]}, 
  "Totals"] := Block[{len = Length[dice], outcomes},
  outcomes = outComes[dice, len];
  Total[Transpose[outcomes]]]

diceTotals[dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]}, 
  "TotalsFrequencies"] := Block[{len = Length[dice], outcomes},
  outcomes = outComes[dice, len];
  Tally[Total[Transpose[outcomes]]]]

diceTotals[dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]}, 
  "Distribution"] := Block[{len = Length[dice], outcomes},
  outcomes = outComes[dice, len];
  EmpiricalDistribution[Total[Transpose[outcomes]]]]

diceTotals[dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]}, 
  n_Integer] := Block[{samples},
  samples = Map[RandomChoice[Range[#], n] &, dice];
  Total[samples]]

diceTotalsDistribution[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]}, n_Integer] := 
 EmpiricalDistribution[diceTotals[dice, n]] 
  
(****************************************************************)
(*This is the function that gets exported for public use*)
(****************************************************************)

RollTheDice[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ], "Outcomes"] := 
 diceOutcomes[dice, "Outcomes"]

RollTheDice[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ] && Length[dice] == 1, "OutcomesTable"] :=
  Grid[{RollTheDice[dice, "Outcomes"]}, Frame -> True, 
  FrameStyle -> Thick]

RollTheDice[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ] && Length[dice] == 2, "OutcomesTable"] :=
 Grid[GatherBy[RollTheDice[dice, "Outcomes"], {First}], Frame -> True,
   FrameStyle -> Thick]

RollTheDice[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ] && Length[dice] == 3, "OutcomesTable"] :=
 Grid[Partition[
   Grid[#, Frame -> True, FrameStyle -> Darker[Blue]] & /@ 
    GatherBy[RollTheDice[dice, "Outcomes"], {First, (#[[2]] &)}], 1], 
  Frame -> True, FrameStyle -> Thick]

RollTheDice[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ] && Length[dice] == 4, "OutcomesTable"] :=
 Grid[Table[{Grid[
     Partition[
      Grid[#, Frame -> True, FrameStyle -> Red] & /@ 
       GatherBy[
         RollTheDice[dice, 
          "Outcomes"], {First, (#[[2]] &), (#[[3]] &)}][[i]], 
      2]]}, {i, 4}], Frame -> True, FrameStyle -> Thick]
      
   
RollTheDice[dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ],n_Integer,"Outcomes"]:= diceOutcomes[dice,n]
   
RollTheDice[dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ]&&Length[dice]==1,n_Integer,"OutcomesTable"]:= 
   Grid[{Split[Sort[diceOutcomes[dice,n]]]},Frame-> True,FrameStyle-> Thick]
   
RollTheDice[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ] && Length[dice] == 2, n_Integer,"OutcomesTable"] :=
 Grid[GatherBy[Sort[diceOutcomes[dice,n]], {First}], Frame -> True,
   FrameStyle -> Thick]
   
RollTheDice[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ] && Length[dice] == 3, n_Integer,"OutcomesTable"] :=
 Grid[Partition[
   Grid[#, Frame -> True, FrameStyle -> Darker[Blue]] & /@ 
    GatherBy[Sort[diceOutcomes[dice,n]], {First, (#[[2]] &)}], 1], 
  Frame -> True, FrameStyle -> Thick]

RollTheDice[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ] && Length[dice] == 4, n_Integer,"OutcomesTable"] :=
 Grid[Table[{Grid[
     Partition[
      Grid[#, Frame -> True, FrameStyle -> Red] & /@ 
       GatherBy[
         Sort[diceOutcomes[dice,n]], {First, (#[[2]] &), (#[[3]] &)}][[i]], 
      2]]}, {i, 4}], Frame -> True, FrameStyle -> Thick]

RollTheDice[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ], "Totals"] :=  diceTotals[dice, "Totals"]

RollTheDice[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ], "TotalsFrequencies"] :=  
 diceTotals[dice, "TotalsFrequencies"]

RollTheDice[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ], "Distribution"] := 
 diceOutcomes[dice, "Distribution"]
 
RollTheDice[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ], "TotalsDistribution"] := 
 diceTotals[dice, "Distribution"]

RollTheDice[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ], n_, "SimulatedDistribution"] := 
 diceOutcomesDistribution[dice, n]
 
 RollTheDice[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ], n_, "SimulatedTotalsDistribution"] := 
   diceTotalsDistribution[dice, n]
 
RollTheDice[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ], "Dice"] := 
 dice /. {4 :> PolyhedronData["Tetrahedron"], 
   6 :> PolyhedronData["Cube"], 8 :> PolyhedronData["Octahedron"],
   10 :> TenSidedDie[], 12 :> PolyhedronData["Dodecahedron"], 
   20 :> PolyhedronData["Icosahedron"]}
   
RollTheDice[
  dice : {Repeated[(4 | 6 | 8 | 10 | 12 | 20), 6]} /; 
   VectorQ[dice, IntegerQ], {"Dice", prop_String}] := 
 dice /. {4 :> PolyhedronData["Tetrahedron", prop], 
   6 :> PolyhedronData["Cube", prop], 
   8 :> PolyhedronData["Octahedron", prop],
   10 :> TenSidedDie[prop], 
   12 :> PolyhedronData["Dodecahedron", prop], 
   20 :> PolyhedronData["Icosahedron", prop]}
 
RollTheDice[dice : {Repeated[
      4 | {4, _Integer} | 6 | {6, _Integer} | 8 | {8, _Integer} | 
       10 | {10, _Integer} |     12 | {12, _Integer} | 
       20 | {20, _Integer}, 6]} /; 
   Total[(If[Head[#] === Integer, {#, 1}, #] & /@ dice)][[2]] <= 6, 
  arg2 : _Integer | _String|_List] := 
 With[{dice2 = 
    Flatten[(If[IntegerQ[#], #, ConstantArray[#[[1]], #[[2]]]] & /@ 
       dice)]},
   RollTheDice[dice2, arg2]]
   
RollTheDice[dice : {Repeated[
      4 | {4, _Integer} | 6 | {6, _Integer} | 8 | {8, _Integer} | 
       10 | {10, _Integer} |     12 | {12, _Integer} | 
       20 | {20, _Integer}, 6]} /; 
   Total[(If[Head[#] === Integer, {#, 1}, #] & /@ dice)][[2]] <= 6, 
  arg2 : _Integer, arg3_String] := 
 With[{dice2 = 
    Flatten[(If[IntegerQ[#], #, ConstantArray[#[[1]], #[[2]]]] & /@ 
       dice)]},
   RollTheDice[dice2, arg2,arg3]]



End[] (* End Private Context *)

