(* Mathematica Package *)

(* Created by the Wolfram Workbench Dec 3, 2009 *)



(* Exported symbols added here with SymbolName::usage *) 

FunctionTable::usage = "FunctionTable[fun,{x,xmin,xmax,incr}] creates a table of discrete values for the function fun with the independent variable x running from xmin to xmax in increments of incr.\nFunctionTable[fun,{x,{val1,val2,\[Ellipsis],valn}}] creates a table of discrete values for the function fun with the independent variable x taking on the values val1, val2,\[Ellipsis],valn."

FunctionValues::usage = "FunctionValues is an option for Function table that determines how the function values are rendered in in the table. Permissible values are Automatice giving machine numbers and Exact giving exact values for the function values inthe table."

Exact::usage = "Exact is a possible value for the option FunctionValues giving function values as exact numbers."

FunctionArrowDiagram::usage = "FunctionArrowDiagram[dompts,rngpts,type,opts] creates an arrow diagram representing various types of functions such as one to one or constant."

ArrowStyle::usage = "ArrowStyle is an option for FunctionArrowDiagram that determines how the arrows are rendered."

SetsStyle::usage = "SetsStyle is an option for FunctionArrowDiagram that determines how the domain and range sets are rendered."

PointLabels::usage = "PointLabels is an option for FunctionDiagramPlot allowing labeling of the points in the domain and range sets."

PointLabelStyle::usage = "PointLabelStyle is an option for FunctionArrowDiagram allowing the point labels to be styled."

PointLabelOffset::usage = "PointLabelOffset is an option for FunctionArrowDiagram for controlling the amount of offset from the point used by the point labels."

PlotGrid::usage = "PlotGrid[quad,opts] creates an empty grid to use in plotting exercises including the quadrants specified in quad."

XGrid::usage = "XGrid is an option value for GridLineStyle in PlotGrid to have the grid rendered as a set of crosses."

PointGrid::usage = "PointGrid is an option value for GridLineStyle in PlotGrid to have the grid rendered as points."

QuadrantI::usage= ""

QuadrantII::usage=""

QuadrantIII::usage= ""

QuadrantIV::usage=""

PolarPlotGrid::usage = "PolarPlotGrid[] creates a plot grid using polar coordinates."

Unprotect[FunctionValues,Exact,FunctionArrowDiagram,ArrowStyle,SetsStyle,PointLabels,PointLabelStyle,PointLabelOffset, FunctionTable,
PlotGrid,PolarPlotGrid,XGrid,PointGrid,QuadrantI,QuadrantII,QuadrantIII,QuadrantIV,PolarPlotGrid]


Begin["`Private`"]
(* FunctionTable functions *)

(*Auxiliary functions for FunctionTable*)



    
makeGrid[headings_, vals_, opts : OptionsPattern[Grid]] := 
Grid[Join[{headings}, vals], opts, BaseStyle -> {"Menu"}, 
    Alignment -> {{Left, "."}, Automatic}, Background -> LightYellow, 
    Dividers -> {{AbsoluteThickness[1.5], None, 
       AbsoluteThickness[
        1.5]}, {All, {2 -> Directive[AbsoluteThickness[1.5]], 
        1 -> Directive[AbsoluteThickness[1.5]], -1 -> 
         Directive[AbsoluteThickness[1.5]]}}}, Spacings -> {1, 0.62}]
    
makeHeadings[fun_, 
  var_Symbol,fnNm_] := {Item[TraditionalForm[var], Alignment -> Center, 
   BaseStyle -> {FontWeight -> "Bold"}], 
  Item[StringForm["`3`(`2`) = `1`", TraditionalForm[fun], 
    TraditionalForm[var],TraditionalForm[fnNm]], Alignment -> Center, 
   BaseStyle -> {FontWeight -> "Bold"}]}
   
headngRools = { {None, rr_} :> {"", rr}, 
	{rr_, None} :> {rr, "            "}, 
	{None, None} :> {"", "            "}, 
	None :> {"", "            "}}
    
valueTable[fun_, {x_, xmin_, xmax_, incr_: 1}] := 
 Table[{x, NumberForm[N@Chop[fun, 10^-12], 5]}, {x, xmin, xmax, incr}]


valueTable[fun_, {x_, lis_List}] := 
 Table[{x, NumberForm[N@Chop[fun, 10^-12], 5]}, {x, lis}]


valueTable[fun_, {x_, xmin_, xmax_, incr_: 1}, 
  exclud_ /; VectorQ[exclud, IntegerQ]] := 
 Block[{tab =  
    Table[{x, NumberForm[N@Chop[fun, 10^-12], 5]}, {x, xmin, xmax, 
      incr}]},
  tab[[exclud]] = tab[[exclud]] /. {a_?NumericQ, _} :> {a, "   "}; tab]
  

valueTable[fun_, {x_, lis_List}, 
  exclud_ /; VectorQ[exclud, IntegerQ]] := 
 Block[{tab =  
    Table[{x, NumberForm[N@Chop[fun, 10^-12], 5]}, {x, lis}]},
  tab[[exclud]] = (tab[[exclud]] /. {a_?NumericQ, _} :> {a, "   "}); 
  tab]
  
valueTable[
  fun_, {x_, xmin_, xmax_, 
   incr_: 1}, {exclud1_ /; VectorQ[exclud1, IntegerQ], 
   exclud2_ /; VectorQ[exclud2, IntegerQ]}] := 
 Block[{tab = 
    Table[{x, NumberForm[N@Chop[fun, 10^-12], 5]}, {x, xmin, xmax, 
      incr}]},
  tab[[exclud1]] = tab[[exclud1]] /. {a_?NumericQ, b_} :> {"   ", b};
  tab[[exclud2]] = 
   tab[[exclud2]] /. {a : _?NumericQ | _String, _} :> {a, "   "};
  tab]

valueTable[
  fun_, {x_, lis_List}, {exclud1_ /; VectorQ[exclud1, IntegerQ], 
   exclud2_ /; VectorQ[exclud2, IntegerQ]}] := 
 Block[{tab = 
    Table[{x, NumberForm[N@Chop[fun, 10^-12], 5]}, {x, lis}]}, 
  tab[[exclud1]] = tab[[exclud1]] /. {a_?NumericQ, b_} :> {"   ", b};
  tab[[exclud2]] = 
   tab[[exclud2]] /. {a_?NumericQ | _String, _} :> {a, "   "};
  tab]
  
valueTableExct[fun_, {x_, xmin_, xmax_, incr_: 1}] := 
 Table[{x, fun}, {x, xmin, xmax, incr}]


valueTableExct[fun_, {x_, lis_List}] := 
 Table[{x, fun}, {x, lis}]


valueTableExct[fun_, {x_, xmin_, xmax_, incr_: 1}, 
  exclud_ /; VectorQ[exclud, IntegerQ]] := 
 Block[{tab =  
    Table[{x, fun}, {x, xmin, xmax, 
      incr}]},
  tab[[exclud]] = tab[[exclud]] /. {a_?NumericQ, _} :> {a, "   "}; tab]
  

valueTableExct[fun_, {x_, lis_List}, 
  exclud_ /; VectorQ[exclud, IntegerQ]] := 
 Block[{tab =  
    Table[{x, fun}, {x, lis}]},
  tab[[exclud]] = (tab[[exclud]] /. {a_?NumericQ, _} :> {a, "   "}); 
  tab]
  
valueTableExct[
  fun_, {x_, xmin_, xmax_, 
   incr_: 1}, {exclud1_ /; VectorQ[exclud1, IntegerQ], 
   exclud2_ /; VectorQ[exclud2, IntegerQ]}] := 
 Block[{tab = 
    Table[{x, fun}, {x, xmin, xmax, 
      incr}]},
  tab[[exclud1]] = tab[[exclud1]] /. {a_?NumericQ, b_} :> {"   ", b};
  tab[[exclud2]] = 
   tab[[exclud2]] /. {a : _?NumericQ | _String, _} :> {a, "   "};
  tab]

valueTableExct[
  fun_, {x_, lis_List}, {exclud1_ /; VectorQ[exclud1, IntegerQ], 
   exclud2_ /; VectorQ[exclud2, IntegerQ]}] := 
 Block[{tab = 
    Table[{x, fun}, {x, lis}]}, 
  tab[[exclud1]] = tab[[exclud1]] /. {a_?NumericQ, b_} :> {"   ", b};
  tab[[exclud2]] = 
   tab[[exclud2]] /. {a_?NumericQ | _String, _} :> {a, "   "};
  tab]

(*FunctionTable definitions*)

Attributes[FunctionTable]={HoldFirst}

Options[FunctionTable]= {Alignment -> Automatic,
 AutoDelete -> False,
 Background -> RGBColor[1, 1, 0.85`],
 BaselinePosition -> Automatic,
 BaseStyle -> {},
 DefaultBaseStyle -> {"Menu"},
 DefaultElement -> "\[Placeholder]",
 Dividers -> Automatic,
 Frame -> None,
 FrameStyle -> Automatic,
 FunctionValues -> Automatic,
 ItemSize -> Automatic,
 ItemStyle -> None,
 Spacings -> {1, 0.62`},
 TableHeadings -> Automatic}
 
 adjusts = {Alignment -> {{Left, "."}, Automatic},
  Dividers -> {{AbsoluteThickness[1.5`],
     None, 
     AbsoluteThickness[
      1.5`]}, {All, {2 -> Directive[AbsoluteThickness[1.5`]], 
      1 -> Directive[AbsoluteThickness[1.5`]], -1 -> 
       Directive[AbsoluteThickness[1.5`]]}}}}

FunctionTable[fun_, {x_, xmin_, xmax_, incr_: 1}, 
  opts : OptionsPattern[]] :=
  Block[{fnNm = Head[Unevaluated[fun]], fnNm2},
 fnNm2 = If[Context[Evaluate[fnNm]] === "System`", "\[ScriptF]", 
    fnNm];
 Module[{
   headings = If[OptionValue[TableHeadings]===Automatic,makeHeadings[fun,x,fnNm2],
   	OptionValue[TableHeadings]/.headngRools],
   vals = If[OptionValue[FunctionValues]===Automatic||Precision[{x, xmin, xmax, incr}]<Infinity, 
   valueTable[fun, {x, xmin, xmax, incr}],
   valueTableExct[fun, {x, xmin, xmax, incr}]]},
  makeGrid[headings, vals, FilterRules[{opts},Options[Grid]],adjusts]
  ]]
  

FunctionTable[fun_, {x_, xmin_, xmax_, incr_: 1}, 
  exclud_List, opts : OptionsPattern[]] :=
 Block[{fnNm = Head[Unevaluated[fun]], fnNm2},
 fnNm2 = If[Context[Evaluate[fnNm]] === "System`", "\[ScriptF]", 
    fnNm];
  Module[{
   headings = If[OptionValue[TableHeadings]===Automatic,makeHeadings[fun,x,fnNm2],
   	OptionValue[TableHeadings]/.headngRools],
   vals = If[OptionValue[FunctionValues]===Automatic||Precision[{x, xmin, xmax, incr}]<Infinity, 
   valueTable[fun, {x, xmin, xmax, incr}, exclud],
   valueTableExct[fun, {x, xmin, xmax, incr}, exclud]]}, 
  makeGrid[headings, vals, FilterRules[{opts},Options[Grid]],adjusts]
  ]]
  

FunctionTable[fun_, {x_, lis_List}, opts : OptionsPattern[]] :=
  Block[{fnNm = Head[Unevaluated[fun]], fnNm2},
 fnNm2 = If[Context[Evaluate[fnNm]] === "System`", "\[ScriptF]", 
    fnNm];
 Module[{
   headings = If[OptionValue[TableHeadings]===Automatic,makeHeadings[fun,x,fnNm2],
   	OptionValue[TableHeadings]/.headngRools],
   vals = If[OptionValue[FunctionValues]===Automatic||Precision[{x, lis}]<Infinity, 
   valueTable[fun, {x, lis}],valueTableExct[fun, {x, lis}]]}, 
   makeGrid[headings, vals, FilterRules[{opts},Options[Grid]],adjusts]
  ]]
  

FunctionTable[fun_, {x_, lis_List}, 
  exclud_List, opts : OptionsPattern[]] :=
 Block[{fnNm = Head[Unevaluated[fun]], fnNm2},
 fnNm2 = If[Context[Evaluate[fnNm]] === "System`", "\[ScriptF]", 
    fnNm];
  Module[{
   headings = If[OptionValue[TableHeadings]===Automatic,makeHeadings[fun,x,fnNm2],
   	OptionValue[TableHeadings]/.headngRools], 
  vals=If[OptionValue[FunctionValues]===Automatic||Precision[{x, lis}]<Infinity,  
  	valueTable[fun, {x, lis}, exclud],
    valueTableExct[fun, {x, lis}, exclud]]}, 
  makeGrid[headings, vals, FilterRules[{opts},Options[Grid]],adjusts]
  ]];
  
  
(*Functions for creating FunctionArrowDiagrams*)

(* *************************************************************************************** *)


Options[FunctionArrowDiagram] = {AlignmentPoint -> Center, ArrowStyle -> Automatic, 
 AspectRatio -> Automatic, Axes -> False, AxesLabel -> None, 
 AxesOrigin -> Automatic, AxesStyle -> {}, Background -> None, 
 BaselinePosition -> Automatic, BaseStyle -> {}, 
 ColorOutput -> Automatic, ContentSelectable -> Automatic, 
 CoordinatesToolOptions -> Automatic, Epilog -> {}, Frame -> True, 
 FrameLabel -> Automatic, FrameStyle -> {}, FrameTicks -> None, 
 FrameTicksStyle -> {}, GridLines -> None, GridLinesStyle -> {}, 
 ImageMargins -> 0., ImagePadding -> All, ImageSize -> Automatic, 
 ImageSizeRaw -> Automatic, LabelStyle -> {}, Method -> Automatic, 
 PointLabels-> None,
 PlotLabel -> None, PlotRange -> All, PlotRangeClipping -> False, 
 PlotRangePadding -> Automatic, PlotRegion -> Automatic, 
 PlotStyle -> Automatic, PointLabelOffset-> Automatic,
 PointLabelStyle-> Automatic,
 PreserveImageOptions -> Automatic, 
 Prolog -> {}, RotateLabel -> True, SetsStyle -> Automatic, 
 Ticks -> Automatic, TicksStyle -> {}, 
 DisplayFunction :> $DisplayFunction, FormatType :> TraditionalForm}
 
FunctionArrowDiagram::onto = "The first two arguments `1` aand `2` \
should be positive integers with `1` greater than or equal to `2`."

FunctionArrowDiagram::injct = "The first two arguments `1` aand `2` \
should be positive integers with `1` less than or equal to `2`."

FunctionArrowDiagram::bijct = "The first two arguments `1` aand `2` \
should be positive integers with `1` equal to `2`."
 
 setsOptRules = {{Automatic, 
    Automatic} -> {{RGBColor[0.986419, 0.970581, 0.342397], 
     Opacity[.2], 
     EdgeForm[Black]}, {RGBColor[0.0692454, 0.645548, 0.968322]}},
     {Automatic}-> {{RGBColor[0.986419, 0.970581, 0.342397], Opacity[.2], 
     EdgeForm[Black]}, {RGBColor[0.0692454, 0.645548, 
      0.968322]}},
  Automatic -> {{RGBColor[0.986419, 0.970581, 0.342397], Opacity[.2], 
     EdgeForm[Black]}, {RGBColor[0.0692454, 0.645548, 
      0.968322]}}, {a_, b_} -> {a, b},a_-> {a,a},{a_}:>{a,a}}
      
arrwRules= {Automatic-> {RGBColor[0.5, 0, 0.5],Arrowheads[Medium]}}

frmLblRules={Automatic-> Style["\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])",18,Bold],None-> {}}
      
frmLblRulesInv= {Automatic-> Style[#,18,Bold]&[\!\(\*
StyleBox["\"\<\\!\\(\\*SuperscriptBox[\\(\[ScriptF]\\), \\(-1\\)]\\)(\
\[ScriptX]) \[LeftTeeArrow] \[ScriptX]\>\""
]\)],None-> {}}


ptLblRules[n_,aa_,fun_,pts_,ptlblstyl_,offSet_]:= 
	{None-> {},Automatic->makePointLabels[n,Automatic,fun,pts,ptlblstyl,offSet],
	{vars_List,func_String}:>makePointLabels[n,vars,func,pts,offSet]}
	
ptLblRules[n_,"Relation",fun_,pts_,ptlblstyl_,offSet_]:= 
	{None-> {},Automatic->makePointLabels[n,"Relation",fun,pts,ptlblstyl,offSet],
	{vars_List,func_String}:>makePointLabels[n,vars,func,pts,offSet]}
	
ptLblStylRules={Automatic-> {14, FontColor -> Black, Italic}}

lblOffSetRules = {Automatic->{{-6,6},{10,6}}};
      
pltStyleRules = {Automatic -> {Opacity[1], PointSize[Medium], 
    GrayLevel[0]}, None -> {}}

ptSet = Cases[Flatten[
  Table[{If[EvenQ[j], .2 i, .2 i + .1], .2 Sqrt[3] j/2}, {i, -10, 
    10}, {j, -15, 15}], 1], pt_ /; pt[[1]]^2 + 4 pt[[2]]^2/9 < 1]
    
argSet = ToString[#, FormatType -> TraditionalForm] & /@ 
 Flatten@Map[
   Thread[Subscript[ToExpression[CharacterRange["a", "z"]], #]] &, 
   Range[5]]
   
szTst[val_, lim_] := val < lim
   
separatePts[arry_?MatrixQ, dis_?NumericQ, len_Integer?Positive, 
  max_Integer?Positive] := 
 Block[{ff = 
    Function[lis, 
     pt = RandomChoice[lis[[2]]]; {pt, 
      Select[lis[[2]], EuclideanDistance[pt, #] > dis &]}]}, 
  Rest[NestWhileList[ff, {{0, 0}, arry}, Length[#[[2]]] > len &, 1, 
     max]][[All, 1]]]

arrowPts[pts_Integer?Positive] := 
 N[TranslationTransform[-{Cos[2 \[Pi]/3], Sin[2 \[Pi]/3]}] [
   Table[ {Cos[t], Sin[t]}, {t, 2 \[Pi]/3, \[Pi]/3, - \[Pi]/(3 (pts - 1))}]]]

curvedArrow[{pt1_ /; ArrayQ[pt1, 1, NumericQ], 
   pt2_ /; ArrayQ[pt2, 1, NumericQ]}] := 
 Block[{len = EuclideanDistance[pt1, pt2], ang = ArcTan @@ (pt2 - pt1)}, 
  Arrow[Composition[TranslationTransform[pt1], RotationTransform[ang]][
    len arrowPts[24]]]]

makePointLabels[n_Integer, None,fun_String, {domPts_, rngPts_}, 
  ptlblstyl_List, {lftOffSt_, rtOffSt_}] := {}

makePointLabels[n_Integer, chars_List, 
   fun_String, {domPts_, rngPts_}, 
   ptlblstyl_List, {lftOffSt_, rtOffSt_}] /; Length[domPts] === n := 
 Block[{charlis = PadRight[ToString /@ chars, n, ToString /@ chars], 
   charfuns, lblLocs}, 
  charfuns = StringJoin[fun, "(", #, ")"] & /@ charlis;
  lblLocs = Join[{charlis}, {charfuns}, {domPts, rngPts}];
  MapThread[{Inset[Style[#1, ptlblstyl], Offset[lftOffSt, #3]], 
     Inset[Style[#2, ptlblstyl], Offset[rtOffSt, #4]]} &, lblLocs]]

makePointLabels[n_Integer, chars_List, 
   fun_String, {domPts_, rngPts_}, 
   ptlblstyl_List, {lftOffSt_, rtOffSt_}] /; Length[domPts] > n := 
 Block[{charlis = PadRight[ToString /@ chars, n, ToString /@ chars], 
   charrools, charLis, charfuns, lblLocs}, 
  charrools = Thread[Rule[DeleteDuplicates[domPts], charlis]];
  charLis = domPts /. charrools;
  charfuns = StringJoin[fun, "(", #, ")"] & /@ charLis;
  lblLocs = Join[{charLis}, {charfuns}, {domPts, rngPts}];
  MapThread[{Inset[Style[#1, ptlblstyl], Offset[lftOffSt, #3]], 
     Inset[Style[#2, ptlblstyl], Offset[rtOffSt, #4]]} &, lblLocs]]

makePointLabels[n_Integer, Automatic, fun_String, {domPts_, rngPts_},
    ptlblstyl_List, {lftOffSt_, rtOffSt_}] /; (Length[domPts] == n) :=
  Block[{charlis = Take[argSet, n], charfuns, lblLocs}, 
  charfuns = StringJoin[fun, "(", #, ")"] & /@ charlis;
  lblLocs = Join[{charlis}, {charfuns}, {domPts, rngPts}];
  MapThread[{Inset[Style[#1, ptlblstyl], Offset[lftOffSt, #3]], 
     Inset[Style[#2, ptlblstyl], Offset[rtOffSt, #4]]} &, lblLocs]]

makePointLabels[n_Integer, Automatic, fun_String, {domPts_, rngPts_},
    ptlblstyl_List, {lftOffSt_, rtOffSt_}] /; (n < Length[domPts]) := 
 Block[{charlis = Take[argSet, n], charrools, charLis, charfuns, 
   lblLocs},
  charrools = Thread[Rule[DeleteDuplicates[domPts], charlis]];
  charLis = domPts /. charrools;
  charfuns = StringJoin[fun, "(", #, ")"] & /@ charLis;
  lblLocs = Join[{charLis}, {charfuns}, {domPts, rngPts}];
  MapThread[{Inset[Style[#1, ptlblstyl], Offset[lftOffSt, #3]], 
     Inset[Style[#2, ptlblstyl], Offset[rtOffSt, #4]]} &, lblLocs]]
     
refnLabels[{}] :={}
     
refnLabels[mat_?MatrixQ] := 
 Join[mat[[All, 1]], 
  DeleteDuplicates[mat[[All, 2]], #1[[-1, -1]] == #2[[-1, -1]] &]]

makePtSets[n1_Integer /; ! szTst[n1, 12], 
  n2_Integer /; ! szTst[n2, 12], 
  pts_?ArrayQ] := {RandomSample[pts, n1], 
  TranslationTransform[{3, 0}][RandomSample[pts, n2]]}

makePtSets[n1_Integer /; szTst[n1, 12], n2_Integer /; ! szTst[n2, 12],
   pts_?ArrayQ] := {separatePts[pts, .5, 2, n1], 
  TranslationTransform[{3, 0}][RandomSample[pts, n2]]}

makePtSets[n1_Integer /; ! szTst[n1, 12], n2_Integer /; szTst[n2, 12],
   pts_?ArrayQ] := {RandomSample[pts, n1], 
  TranslationTransform[{3, 0}][separatePts[pts, .5, 2, n2]]}

makePtSets[n1_Integer /; szTst[n1, 12], n2_Integer /; szTst[n2, 12], 
  pts_?ArrayQ] := {separatePts[pts, .5, 2, n1], 
  TranslationTransform[{3, 0}][separatePts[pts, .5, 2, n2]]}
  
maxPoint[mat_?MatrixQ] := Block[{distSet = Map[Norm, mat], bgpt},
  bgpt = Extract[mat, Position[distSet, Max[distSet]]][[1]];
  Transpose[{-{bgpt, Normalize[bgpt]}, {bgpt, Normalize[bgpt]}}, {3, 
    1, 2}]]

scalePtsD[mat_?MatrixQ] := 
 Block[{pts = 
    maxPoint[mat] /. {{0, 0} -> {0, 1}, {0., 0.} -> {0., 1.}}, sclpts},
  sclpts = (RescalingTransform @@ pts)[mat];
  Map[{1, 3/2} # &, sclpts]]

scalePtsR[mat_?MatrixQ] := 
 Block[{pts = 
    maxPoint[mat] /. {{0, 0} -> {0, 1}, {0., 0.} -> {0., 1.}}, sclpts},
  sclpts = (Composition[TranslationTransform[{3, 0}], 
      RescalingTransform @@ pts])[mat];
  Map[{1, 3/2} # &, sclpts]]

makePtSets[domPts_?MatrixQ, rngPts_?MatrixQ] := {scalePtsD[domPts], 
  scalePtsR[rngPts]}
  
makeArrowSets[pts_List, "OneToOne"] := 
  Transpose[{SortBy[pts[[1]], Last], 
    SortBy[RandomSample[pts[[2]], Length[pts[[1]]]], Last]}]
    
makeArrowSetsUS[pts_List, "OneToOne"] := 
  Transpose[{pts[[1]], 
   Take[pts[[2]], Length[pts[[1]]]]}]

makeArrowSets[pts_List, "Onto"] :=  
  Transpose[{SortBy[pts[[1]], Last], 
    SortBy[Join[pts[[2]], 
      RandomChoice[pts[[2]], {Length[pts[[1]]] - Length[pts[[2]]]}]], 
     Last]}]
     
makeArrowSetsUS[pts_List, "Onto"] :=  
  Transpose[{pts[[1]], 
    Join[pts[[2]], 
      Take[pts[[2]], Length[pts[[1]]] - Length[pts[[2]]]]]}]

makeArrowSets[pts_List, "OneToOneOnto"] := 
  Transpose[{SortBy[pts[[1]], Last], SortBy[pts[[2]], Last]}]
  
makeArrowSetsUS[pts_List, "OneToOneOnto"] := 
  Transpose[{pts[[1]], pts[[2]]}]

makeArrowSets[pts_List, "Constant"] := 
 Block[{selpt = RandomSample[pts[[2]], 1][[1]]}, 
   Transpose[{SortBy[pts[[1]], Last], 
     Table[selpt, {Length[pts[[1]]]}]}]]
     
makeArrowSetsUS[pts_List, "Constant"] := 
 Block[{selpt = pts[[2,1]]}, 
   Transpose[{pts[[1]], 
     Table[selpt, {Length[pts[[1]]]}]}]]

makeArrowSets[pts_List, "Relation"] := 
 Block[{extrpts = 
    RandomChoice[pts[[1]], Ceiling[Length[pts[[1]]]/2]]}, 
   Transpose[{SortBy[Join[pts[[1]], extrpts], Last], 
     PadRight[pts[[2]], Length[pts[[1]]] + Length[extrpts], 
      pts[[2]]]}]]
      
makeArrowSetsUS[pts_List, "Relation"] := 
 Block[{uniquepts = 
    DeleteDuplicates[pts[[1]]]}, 
   Transpose[{pts[[1]], 
     PadRight[pts[[2]], Length[pts[[1]]], 
      pts[[2]]]}]]
      
(*The plot functions*)

(*TODO: add the makePointLabel functions and modify the existing FunctionArrowDiagram functions.*)

FunctionArrowDiagram[dompts_Integer,rngpts_Integer, "OneToOne",opts:OptionsPattern[]]/;dompts<=rngpts := 
 Block[{pts = makePtSets[dompts, rngpts,ptSet], domOpts, rngOpts,ptOpts,frmLbl,
 	arrwOpts,ptLbls,ptlblstyl, lftOffSt, rtOffSt,endpts,seq},
 	endpts= makeArrowSets[pts,"OneToOne"];
 	{domOpts, rngOpts} = OptionValue[SetsStyle] /. setsOptRules;
 	ptOpts= OptionValue[PlotStyle]/.pltStyleRules;
 	arrwOpts= OptionValue[ArrowStyle]/.arrwRules;
 	frmLbl= OptionValue[FrameLabel]/.frmLblRules;
 	ptlblstyl=OptionValue[PointLabelStyle]/.ptLblStylRules;
 	{lftOffSt,rtOffSt}= OptionValue[PointLabelOffset]/.lblOffSetRules;
 	seq= Which[OptionValue[PointLabels] === Automatic, Sequence@@{Automatic,"f"}, 
 		OptionValue[PointLabels]===None,Sequence@@{None,"f"},
 		True,Sequence @@ OptionValue[PointLabels]];
 	ptLbls=refnLabels[makePointLabels[dompts,seq,Transpose[endpts],ptlblstyl,
 		{lftOffSt,rtOffSt}]];
 Graphics[Flatten[{domOpts, 
    Disk[{0,0},1.1{1,3/2}], rngOpts, 
    Disk[{3,0},1.1{1,3/2}], Opacity[1],ptOpts,  
   Point[Flatten[pts, 1]], arrwOpts, 
    curvedArrow/@endpts,GrayLevel[0],ptLbls}], 
    FilterRules[{opts},Options[Graphics]],
    FrameLabel-> frmLbl,
    FilterRules[Options[FunctionArrowDiagram],Options[Graphics]]]]
    
FunctionArrowDiagram[dompts_?MatrixQ,rngpts_?MatrixQ, "OneToOne",opts:OptionsPattern[]]/;
	Length[dompts]<=Length[rngpts]&& Length[dompts] == Length[DeleteDuplicates[dompts]]&&Length[rngpts]==Length[DeleteDuplicates[rngpts]] := 
 Block[{pts = makePtSets[dompts, rngpts], domOpts, rngOpts,ptOpts,frmLbl,
 	arrwOpts,ptLbls,ptlblstyl, lftOffSt, rtOffSt,endpts,seq},
 	endpts= makeArrowSetsUS[pts,"OneToOne"];
 	{domOpts, rngOpts} = OptionValue[SetsStyle] /. setsOptRules;
 	ptOpts= OptionValue[PlotStyle]/.pltStyleRules;
 	arrwOpts= OptionValue[ArrowStyle]/.arrwRules;
 	frmLbl= OptionValue[FrameLabel]/.frmLblRules;
 	ptlblstyl=OptionValue[PointLabelStyle]/.ptLblStylRules;
 	{lftOffSt,rtOffSt}= OptionValue[PointLabelOffset]/.lblOffSetRules;
 	seq= Which[OptionValue[PointLabels] === Automatic, Sequence@@{Automatic,"f"}, 
 		OptionValue[PointLabels]===None,Sequence@@{None,"f"},
 		True,Sequence @@ OptionValue[PointLabels]];
 	ptLbls=refnLabels[makePointLabels[Length[dompts],seq,Transpose[endpts],ptlblstyl,
 		{lftOffSt,rtOffSt}]];
 Graphics[Flatten[{domOpts, 
    Disk[{0,0},1.1{1,3/2}], rngOpts, 
    Disk[{3,0},1.1{1,3/2}], Opacity[1],ptOpts,  
   Point[Flatten[pts, 1]], arrwOpts, 
    curvedArrow/@endpts,GrayLevel[0],ptLbls}], 
    FilterRules[{opts},Options[Graphics]],
    FrameLabel-> frmLbl,
    FilterRules[Options[FunctionArrowDiagram],Options[Graphics]]]]
    
FunctionArrowDiagram[dompts_Integer?Positive,rngpts_Integer?Positive, "Onto", 
  opts : OptionsPattern[]]/;dompts>=rngpts := 
 Block[{pts = makePtSets[dompts, rngpts,ptSet], domOpts, rngOpts,ptOpts,frmLbl,
 	arrwOpts,ptLbls,ptlblstyl, lftOffSt, rtOffSt,seq},
 	endpts= makeArrowSets[pts,"Onto"];
 	{domOpts, rngOpts} = OptionValue[SetsStyle] /. setsOptRules;
 	ptOpts= OptionValue[PlotStyle]/.pltStyleRules;
 	arrwOpts= OptionValue[ArrowStyle]/.arrwRules;
 	frmLbl= OptionValue[FrameLabel]/.frmLblRules;
 	ptlblstyl=OptionValue[PointLabelStyle]/.ptLblStylRules;
 	{lftOffSt,rtOffSt}= OptionValue[PointLabelOffset]/.lblOffSetRules;
 	seq= Which[OptionValue[PointLabels] === Automatic, Sequence@@{Automatic,"f"}, 
 		OptionValue[PointLabels]===None,Sequence@@{None,"f"},
 		True,Sequence @@ OptionValue[PointLabels]];
 	ptLbls=refnLabels[makePointLabels[dompts,seq,Transpose[endpts],ptlblstyl,
 		{lftOffSt,rtOffSt}]];
 Graphics[Flatten[{domOpts, 
    Disk[{0,0},1.1{1,3/2}], rngOpts, 
    Disk[{3,0},1.1{1,3/2}], Opacity[1],ptOpts,  
   Point[Flatten[pts, 1]], arrwOpts, 
    curvedArrow/@endpts,GrayLevel[0],ptLbls}], 
    FilterRules[{opts},Options[Graphics]],
    FrameLabel-> frmLbl,
    FilterRules[Options[FunctionArrowDiagram],Options[Graphics]]]]
    
FunctionArrowDiagram[dompts_?MatrixQ,rngpts_?MatrixQ, "Onto", 
  opts : OptionsPattern[]]/;Length[dompts]>=Length[rngpts]&& Length[dompts] == Length[DeleteDuplicates[dompts]] := 
 Block[{pts = makePtSets[dompts, rngpts], domOpts, rngOpts,ptOpts,frmLbl,
 	arrwOpts,ptLbls,ptlblstyl, lftOffSt, rtOffSt,seq},
 	endpts= makeArrowSetsUS[pts,"Onto"];
 	{domOpts, rngOpts} = OptionValue[SetsStyle] /. setsOptRules;
 	ptOpts= OptionValue[PlotStyle]/.pltStyleRules;
 	arrwOpts= OptionValue[ArrowStyle]/.arrwRules;
 	frmLbl= OptionValue[FrameLabel]/.frmLblRules;
 	ptlblstyl=OptionValue[PointLabelStyle]/.ptLblStylRules;
 	{lftOffSt,rtOffSt}= OptionValue[PointLabelOffset]/.lblOffSetRules;
 	seq= Which[OptionValue[PointLabels] === Automatic, Sequence@@{Automatic,"f"}, 
 		OptionValue[PointLabels]===None,Sequence@@{None,"f"},
 		True,Sequence @@ OptionValue[PointLabels]];
 	ptLbls=refnLabels[makePointLabels[Length[dompts],seq,Transpose[endpts],ptlblstyl,
 		{lftOffSt,rtOffSt}]];
 Graphics[Flatten[{domOpts, 
    Disk[{0,0},1.1{1,3/2}], rngOpts, 
    Disk[{3,0},1.1{1,3/2}], Opacity[1],ptOpts,  
   Point[Flatten[pts, 1]], arrwOpts, 
    curvedArrow/@endpts,GrayLevel[0],ptLbls}], 
    FilterRules[{opts},Options[Graphics]],
    FrameLabel-> frmLbl,
    FilterRules[Options[FunctionArrowDiagram],Options[Graphics]]]]
    
FunctionArrowDiagram[dompts_Integer?Positive,rngpts_Integer?Positive, "OneToOneOnto", 
  opts : OptionsPattern[]]/;dompts==rngpts := 
 Block[{pts = makePtSets[dompts, rngpts,ptSet], domOpts, rngOpts,ptOpts,frmLbl,
 	arrwOpts,ptLbls,ptlblstyl, lftOffSt, rtOffSt,seq, endpts},
 	endpts= makeArrowSets[pts,"OneToOneOnto"];
 	{domOpts, rngOpts} = OptionValue[SetsStyle] /. setsOptRules;
 	ptOpts= OptionValue[PlotStyle]/.pltStyleRules;
 	arrwOpts= OptionValue[ArrowStyle]/.arrwRules;
 	frmLbl= OptionValue[FrameLabel]/.frmLblRules;
 	ptlblstyl=OptionValue[PointLabelStyle]/.ptLblStylRules;
 	{lftOffSt,rtOffSt}= OptionValue[PointLabelOffset]/.lblOffSetRules;
 	seq= Which[OptionValue[PointLabels] === Automatic, Sequence@@{Automatic,"f"}, 
 		OptionValue[PointLabels]===None,Sequence@@{None,"f"},
 		True,Sequence @@ OptionValue[PointLabels]];
 	ptLbls=refnLabels[makePointLabels[dompts,seq,Transpose[endpts],ptlblstyl,
 		{lftOffSt,rtOffSt}]];
 Graphics[Flatten[{domOpts, 
    Disk[{0,0},1.1{1,3/2}], rngOpts, 
    Disk[{3,0},1.1{1,3/2}], Opacity[1],ptOpts,  
   Point[Flatten[pts, 1]], arrwOpts, 
    curvedArrow/@endpts,GrayLevel[0],ptLbls}], 
    FilterRules[{opts},Options[Graphics]],
    FrameLabel-> frmLbl,
    FilterRules[Options[FunctionArrowDiagram],Options[Graphics]]]]
    
FunctionArrowDiagram[dompts_?MatrixQ,rngpts_?MatrixQ, "OneToOneOnto", 
  opts : OptionsPattern[]]/;Length[dompts]==Length[rngpts]&&
  Length[dompts] == Length[DeleteDuplicates[dompts]]&&Length[rngpts]==Length[DeleteDuplicates[rngpts]] := 
 Block[{pts = makePtSets[dompts, rngpts], domOpts, rngOpts,ptOpts,frmLbl,
 	arrwOpts,ptLbls,ptlblstyl, lftOffSt, rtOffSt,seq, endpts},
 	endpts= makeArrowSetsUS[pts,"OneToOneOnto"];
 	{domOpts, rngOpts} = OptionValue[SetsStyle] /. setsOptRules;
 	ptOpts= OptionValue[PlotStyle]/.pltStyleRules;
 	arrwOpts= OptionValue[ArrowStyle]/.arrwRules;
 	frmLbl= OptionValue[FrameLabel]/.frmLblRules;
 	ptlblstyl=OptionValue[PointLabelStyle]/.ptLblStylRules;
 	{lftOffSt,rtOffSt}= OptionValue[PointLabelOffset]/.lblOffSetRules;
 	seq= Which[OptionValue[PointLabels] === Automatic, Sequence@@{Automatic,"f"}, 
 		OptionValue[PointLabels]===None,Sequence@@{None,"f"},
 		True,Sequence @@ OptionValue[PointLabels]];
 	ptLbls=refnLabels[makePointLabels[Length[dompts],seq,Transpose[endpts],ptlblstyl,
 		{lftOffSt,rtOffSt}]];
 Graphics[Flatten[{domOpts, 
    Disk[{0,0},1.1{1,3/2}], rngOpts, 
    Disk[{3,0},1.1{1,3/2}], Opacity[1],ptOpts,  
   Point[Flatten[pts, 1]], arrwOpts, 
    curvedArrow/@endpts,GrayLevel[0],ptLbls}], 
    FilterRules[{opts},Options[Graphics]],
    FrameLabel-> frmLbl,
    FilterRules[Options[FunctionArrowDiagram],Options[Graphics]]]]
    
FunctionArrowDiagram[dompts_Integer?Positive,rngpts_Integer?Positive, "Constant", 
  opts : OptionsPattern[]] := 
 Block[{pts = makePtSets[dompts, rngpts,ptSet], domOpts, rngOpts,ptOpts,frmLbl,
 	arrwOpts,ptLbls,ptlblstyl, lftOffSt, rtOffSt,seq},
 	endpts= makeArrowSets[pts,"Constant"];
 	{domOpts, rngOpts} = OptionValue[SetsStyle] /. setsOptRules;
 	ptOpts= OptionValue[PlotStyle]/.pltStyleRules;
 	arrwOpts= OptionValue[ArrowStyle]/.arrwRules;
 	frmLbl= OptionValue[FrameLabel]/.frmLblRules;
 	ptlblstyl=OptionValue[PointLabelStyle]/.ptLblStylRules;
 	{lftOffSt,rtOffSt}= OptionValue[PointLabelOffset]/.lblOffSetRules;
 	seq= Which[OptionValue[PointLabels] === Automatic, Sequence@@{Automatic,"f"}, 
 		OptionValue[PointLabels]===None,Sequence@@{None,"f"},
 		True,Sequence @@ OptionValue[PointLabels]];
 	ptLbls=refnLabels[makePointLabels[dompts,seq,Transpose[endpts],ptlblstyl,
 		{lftOffSt,rtOffSt}]];
 Graphics[Flatten[{domOpts, 
    Disk[{0,0},1.1{1,3/2}], rngOpts, 
    Disk[{3,0},1.1{1,3/2}], Opacity[1],ptOpts,  
   Point[Flatten[pts, 1]], arrwOpts, 
    curvedArrow/@endpts,GrayLevel[0],ptLbls}], 
    FilterRules[{opts},Options[Graphics]],
    FrameLabel-> frmLbl,
    FilterRules[Options[FunctionArrowDiagram],Options[Graphics]]]]
    
FunctionArrowDiagram[dompts_?MatrixQ,rngpts_?MatrixQ, "Constant", 
  opts : OptionsPattern[]]/; Length[dompts] == Length[DeleteDuplicates[dompts]] := 
 Block[{pts = makePtSets[dompts, rngpts], domOpts, rngOpts,ptOpts,frmLbl,
 	arrwOpts,ptLbls,ptlblstyl, lftOffSt, rtOffSt,seq},
 	endpts= makeArrowSetsUS[pts,"Constant"];
 	{domOpts, rngOpts} = OptionValue[SetsStyle] /. setsOptRules;
 	ptOpts= OptionValue[PlotStyle]/.pltStyleRules;
 	arrwOpts= OptionValue[ArrowStyle]/.arrwRules;
 	frmLbl= OptionValue[FrameLabel]/.frmLblRules;
 	ptlblstyl=OptionValue[PointLabelStyle]/.ptLblStylRules;
 	{lftOffSt,rtOffSt}= OptionValue[PointLabelOffset]/.lblOffSetRules;
 	seq= Which[OptionValue[PointLabels] === Automatic, Sequence@@{Automatic,"f"}, 
 		OptionValue[PointLabels]===None,Sequence@@{None,"f"},
 		True,Sequence @@ OptionValue[PointLabels]];
 	ptLbls=refnLabels[makePointLabels[Length[dompts],seq,Transpose[endpts],ptlblstyl,
 		{lftOffSt,rtOffSt}]];
 Graphics[Flatten[{domOpts, 
    Disk[{0,0},1.1{1,3/2}], rngOpts, 
    Disk[{3,0},1.1{1,3/2}], Opacity[1],ptOpts,  
   Point[Flatten[pts, 1]], arrwOpts, 
    curvedArrow/@endpts,GrayLevel[0],ptLbls}], 
    FilterRules[{opts},Options[Graphics]],
    FrameLabel-> frmLbl,
    FilterRules[Options[FunctionArrowDiagram],Options[Graphics]]]]
    
    
FunctionArrowDiagram[dompts_Integer?Positive,rngpts_Integer?Positive, "Relation", 
  opts : OptionsPattern[]] := 
 Block[{pts = makePtSets[dompts, rngpts,ptSet], domOpts, rngOpts,ptOpts,frmLbl,
 	arrwOpts,ptLbls,ptlblstyl, lftOffSt, rtOffSt,seq},
 	endpts= makeArrowSets[pts,"Relation"];
 	{domOpts, rngOpts} = OptionValue[SetsStyle] /. setsOptRules;
 	ptOpts= OptionValue[PlotStyle]/.pltStyleRules;
 	arrwOpts= OptionValue[ArrowStyle]/.arrwRules;
 	frmLbl= OptionValue[FrameLabel]/.frmLblRules;
 	ptlblstyl=OptionValue[PointLabelStyle]/.ptLblStylRules;
 	{lftOffSt,rtOffSt}= OptionValue[PointLabelOffset]/.lblOffSetRules;
 	seq= Which[OptionValue[PointLabels] === Automatic, Sequence@@{Automatic,"f"}, 
 		OptionValue[PointLabels]===None,Sequence@@{None,"f"},
 		True,Sequence @@ OptionValue[PointLabels]];
 	ptLbls=refnLabels[makePointLabels[dompts,seq,Transpose[endpts],ptlblstyl,
 		{lftOffSt,rtOffSt}]];
 Graphics[Flatten[{domOpts, 
    Disk[{0,0},1.1{1,3/2}], rngOpts, 
    Disk[{3,0},1.1{1,3/2}], Opacity[1],ptOpts,  
   Point[Flatten[pts, 1]], arrwOpts, 
    curvedArrow/@endpts,GrayLevel[0],ptLbls}], 
    FilterRules[{opts},Options[Graphics]],
    FrameLabel-> frmLbl,
    FilterRules[Options[FunctionArrowDiagram],Options[Graphics]]]]
    
FunctionArrowDiagram[dompts_?MatrixQ,rngpts_?MatrixQ, "Relation", 
  opts : OptionsPattern[]] := 
 Block[{pts = makePtSets[dompts, rngpts], domOpts, rngOpts,ptOpts,frmLbl,
 	arrwOpts,ptLbls,ptlblstyl, lftOffSt, rtOffSt,seq},
 	endpts= makeArrowSetsUS[pts,"Relation"];
 	{domOpts, rngOpts} = OptionValue[SetsStyle] /. setsOptRules;
 	ptOpts= OptionValue[PlotStyle]/.pltStyleRules;
 	arrwOpts= OptionValue[ArrowStyle]/.arrwRules;
 	frmLbl= OptionValue[FrameLabel]/.frmLblRules;
 	ptlblstyl=OptionValue[PointLabelStyle]/.ptLblStylRules;
 	{lftOffSt,rtOffSt}= OptionValue[PointLabelOffset]/.lblOffSetRules;
 	seq= Which[OptionValue[PointLabels] === Automatic, Sequence@@{Automatic,"f"}, 
 		OptionValue[PointLabels]===None,Sequence@@{None,"f"},
 		True,Sequence @@ OptionValue[PointLabels]];
 	ptLbls=refnLabels[makePointLabels[Length[dompts],seq,Transpose[endpts],ptlblstyl,
 		{lftOffSt,rtOffSt}]];
 Graphics[Flatten[{domOpts, 
    Disk[{0,0},1.1{1,3/2}], rngOpts, 
    Disk[{3,0},1.1{1,3/2}], Opacity[1],ptOpts,  
   Point[Flatten[pts, 1]], arrwOpts, 
    curvedArrow/@endpts,GrayLevel[0],ptLbls}], 
    FilterRules[{opts},Options[Graphics]],
    FrameLabel-> frmLbl,
    FilterRules[Options[FunctionArrowDiagram],Options[Graphics]]]]
    
FunctionArrowDiagram[dompts_Integer?Positive,rngpts_Integer?Positive, "Inverse", 
  opts : OptionsPattern[]]/;dompts==rngpts := 
 Block[{pts = makePtSets[dompts, rngpts,ptSet], domOpts, rngOpts,ptOpts,frmLbl,
 	arrwOpts,ptLbls,ptlblstyl, lftOffSt, rtOffSt,seq, endpts},
 	endpts= Reverse/@makeArrowSets[pts,"OneToOneOnto"];
 	{domOpts, rngOpts} = OptionValue[SetsStyle] /. setsOptRules;
 	ptOpts= OptionValue[PlotStyle]/.pltStyleRules;
 	arrwOpts= OptionValue[ArrowStyle]/.arrwRules;
 	frmLbl= OptionValue[FrameLabel]/.frmLblRulesInv;
 	ptlblstyl=OptionValue[PointLabelStyle]/.ptLblStylRules;
 	{lftOffSt,rtOffSt}= OptionValue[PointLabelOffset]/.lblOffSetRules;
 	seq= Which[OptionValue[PointLabels] === Automatic, Sequence@@{Automatic,"f"}, 
 		OptionValue[PointLabels]===None,Sequence@@{None,"f"},
 		True,Sequence @@ OptionValue[PointLabels]];
 	ptLbls=refnLabels[makePointLabels[dompts,seq,Transpose[endpts],ptlblstyl,
 		{lftOffSt,rtOffSt}]];
 Graphics[Flatten[{domOpts, 
    Disk[{0,0},1.1{1,3/2}], rngOpts, 
    Disk[{3,0},1.1{1,3/2}], Opacity[1],ptOpts,  
   Point[Flatten[pts, 1]], arrwOpts, 
    curvedArrow/@endpts,GrayLevel[0],ptLbls}], 
    FilterRules[{opts},Options[Graphics]],
    FrameLabel-> frmLbl,
    FilterRules[Options[FunctionArrowDiagram],Options[Graphics]]]]
    
FunctionArrowDiagram[dompts_?MatrixQ,rngpts_?MatrixQ, "Inverse", 
  opts : OptionsPattern[]]/;Length[dompts]==Length[rngpts]&&
  Length[dompts] == Length[DeleteDuplicates[dompts]]&&Length[rngpts]==Length[DeleteDuplicates[rngpts]] := 
 Block[{pts = makePtSets[dompts, rngpts], domOpts, rngOpts,ptOpts,frmLbl,
 	arrwOpts,ptLbls,ptlblstyl, lftOffSt, rtOffSt,seq, endpts},
 	endpts= Reverse/@makeArrowSetsUS[pts,"OneToOneOnto"];
 	{domOpts, rngOpts} = OptionValue[SetsStyle] /. setsOptRules;
 	ptOpts= OptionValue[PlotStyle]/.pltStyleRules;
 	arrwOpts= OptionValue[ArrowStyle]/.arrwRules;
 	frmLbl= OptionValue[FrameLabel]/.frmLblRulesInv;
 	ptlblstyl=OptionValue[PointLabelStyle]/.ptLblStylRules;
 	{lftOffSt,rtOffSt}= OptionValue[PointLabelOffset]/.lblOffSetRules;
 	seq= Which[OptionValue[PointLabels] === Automatic, Sequence@@{Automatic,"f"}, 
 		OptionValue[PointLabels]===None,Sequence@@{None,"f"},
 		True,Sequence @@ OptionValue[PointLabels]];
 	ptLbls=refnLabels[makePointLabels[Length[dompts],seq,Transpose[endpts],ptlblstyl,
 		{lftOffSt,rtOffSt}]];
 Graphics[Flatten[{domOpts, 
    Disk[{0,0},1.1{1,3/2}], rngOpts, 
    Disk[{3,0},1.1{1,3/2}], Opacity[1],ptOpts,  
   Point[Flatten[pts, 1]], arrwOpts, 
    curvedArrow/@endpts,GrayLevel[0],ptLbls}], 
    FilterRules[{opts},Options[Graphics]],
    FrameLabel-> frmLbl,
    FilterRules[Options[FunctionArrowDiagram],Options[Graphics]]]]
    
FunctionArrowDiagram[dompts_Integer,rngpts_Integer,"OneToOne",opts:OptionsPattern[]]/;
!Or[Message[FunctionArrowDiagram::injct,dompts,rngpts],rngpts>dompts]:=Null

FunctionArrowDiagram[dompts_?MatrixQ,rngpts_?MatrixQ,"OneToOne",opts:OptionsPattern[]]/;
!Or[Message[FunctionArrowDiagram::injct,dompts,rngpts],Length[rngpts]>Length[dompts]]:=Null
    
FunctionArrowDiagram[dompts_Integer,rngpts_Integer,"Onto",opts:OptionsPattern[]]/;
!Or[Message[FunctionArrowDiagram::onto,dompts,rngpts],rngpts<dompts]:=Null

FunctionArrowDiagram[dompts_?MatrixQ,rngpts_?MatrixQ,"Onto",opts:OptionsPattern[]]/;
!Or[Message[FunctionArrowDiagram::onto,dompts,rngpts],Length[rngpts]<Length[dompts]]:=Null

FunctionArrowDiagram[dompts_Integer,rngpts_Integer,"OneToOneOnto",opts:OptionsPattern[]]/;
!Or[Message[FunctionArrowDiagram::bijct,dompts,rngpts],rngpts!=dompts]:=Null

FunctionArrowDiagram[dompts_?MatrixQ,rngpts_?MatrixQ,"OneToOneOnto",opts:OptionsPattern[]]/;
!Or[Message[FunctionArrowDiagram::bijct,dompts,rngpts],Length[rngpts]!=Length[dompts]]:=Null

(********************************************************************************************)
(********************************************************************************************)
(*PlotGrid functions auxiliary functions*)

Options[PlotGrid] = {AlignmentPoint -> Center, 
   AspectRatio -> Automatic, Axes -> True, AxesLabel -> None, 
   AxesOrigin -> Automatic, AxesStyle -> {}, Background -> None, 
   BaselinePosition -> Automatic, BaseStyle -> {14, Italic}, 
   ColorOutput -> Automatic, ContentSelectable -> Automatic, 
   CoordinatesToolOptions -> Automatic, 
   DisplayFunction :> $DisplayFunction, Epilog -> {}, 
   FormatType :> TraditionalForm, Frame -> False, FrameLabel -> None, 
   FrameStyle -> {}, FrameTicks -> Automatic, FrameTicksStyle -> {}, 
   GridLines -> Automatic, 
   GridLinesStyle -> {}, ImageMargins -> 0.`, ImagePadding -> All, 
   ImageSize -> 432, ImageSizeRaw -> Automatic, LabelStyle -> {}, 
   Method -> Automatic, PlotLabel -> None, PlotRange -> Automatic, 
   PlotRangeClipping -> False, PlotRangePadding -> Automatic, 
   PlotRegion -> Automatic, PreserveImageOptions -> Automatic, 
   Prolog -> {}, RotateLabel -> True, Ticks -> Automatic, 
   TicksStyle -> {}};
   
gridRules[{xmin_, xmax_}, {ymin_, 
   ymax_}] := {Rule[GridLines, Automatic] -> 
   Rule[GridLines, {Range[xmin, xmax], Range[ymin, ymax]}],Rule[PlotRange, Automatic] -> 
   Rule[PlotRange, {{xmin,xmax}, {ymin, ymax}}]}

makeGridPts[{{xmin_, xmax_}, {ymin_, ymax_}}] := 
 Flatten[Table[{i, j}, {i, Floor[xmin], Ceiling[xmax]}, {j, 
    Floor[ymin], Ceiling[ymax]}], 1]

makeXGrid[{{xmin_, xmax_}, {ymin_, ymax_}}] := 
 Flatten[Map[{{Scaled[{-.005, 0.}, #], 
      Scaled[{.005, 0.}, #]}, {Scaled[{0., -.005}, #], 
      Scaled[{0., .005}, #]}} &, 
   makeGridPts[{{xmin, xmax}, {ymin, ymax}}]], 1]

PlotGrid[] := Graphics[{}, Options[PlotGrid]/.gridRules[{-10,10},{-10,10}]]

PlotGrid[All, 
  opts : OptionsPattern[] /; ! FreeQ[{opts}, XGrid]] := (Graphics[
   Flatten[{DeleteCases[Flatten[{OptionValue[GridLinesStyle]}], 
      XGrid], Line[makeXGrid[{{-10, 10}, {-10, 10}}]]}], 
   GridLines -> None, GridLinesStyle -> Automatic, 
   FilterRules[{opts}, Options[Graphics]], Options[PlotGrid]])

PlotGrid[All, 
  opts : OptionsPattern[] /; ! FreeQ[{opts}, PointGrid]] := (Graphics[
   Flatten[{DeleteCases[Flatten[{OptionValue[GridLinesStyle]}], 
      PointGrid], Point[makeGridPts[{{-10, 10}, {-10, 10}}]]}], 
   GridLines -> None, GridLinesStyle -> Automatic, 
   FilterRules[{opts}, Options[Graphics]], Options[PlotGrid]])

PlotGrid[All, opts : OptionsPattern[]] := (
 Graphics[{}, opts, Options[PlotGrid]/.gridRules[{-10,10},{-10,10}]])

PlotGrid[{QuadrantI, QuadrantIII}, opts : OptionsPattern[]] := 
 PlotGrid[All, opts]

PlotGrid[{QuadrantIII, QuadrantI}, opts : OptionsPattern[]] := 
 PlotGrid[All, opts]

PlotGrid[{QuadrantII, QuadrantIV}, opts : OptionsPattern[]] := 
 PlotGrid[All, opts]

PlotGrid[{QuadrantIV, QuadrantII}, opts : OptionsPattern[]] := 
 PlotGrid[All, opts]

PlotGrid[QuadrantI, 
  opts : OptionsPattern[] /; ! FreeQ[{opts}, XGrid]] := (Graphics[
   Flatten[{DeleteCases[Flatten[{OptionValue[GridLinesStyle]}], 
      XGrid], Line[makeXGrid[{{-10, 10}, {-10, 10}}]]}], 
   PlotRange -> {{0, 10}, {0, 10}},
   GridLines -> None, GridLinesStyle -> Automatic, 
   FilterRules[{opts}, Options[Graphics]], Options[PlotGrid]])

PlotGrid[QuadrantI, 
  opts : OptionsPattern[] /; ! FreeQ[{opts}, PointGrid]] := (Graphics[
   Flatten[{DeleteCases[Flatten[{OptionValue[GridLinesStyle]}], 
      PointGrid], Point[makeGridPts[{{-10, 10}, {-10, 10}}]]}], 
   PlotRange -> {{0, 10}, {0, 10}}, 
   GridLines -> None, GridLinesStyle -> Automatic, 
   FilterRules[{opts}, Options[Graphics]], Options[PlotGrid]])

PlotGrid[QuadrantI, opts : OptionsPattern[]] := (
 Graphics[{}, PlotRange -> {{0, 10}, {0, 10}}, opts, 
  ImageSize -> 3 72, Options[PlotGrid]/.gridRules[{-10,10},{-10,10}]])

PlotGrid[QuadrantII, 
  opts : OptionsPattern[] /; ! FreeQ[{opts}, XGrid]] := (Graphics[
   Flatten[{DeleteCases[Flatten[{OptionValue[GridLinesStyle]}], 
      XGrid], Line[makeXGrid[{{-10, 10}, {-10, 10}}]]}], 
   PlotRange -> {{-10, 0}, {0, 10}}, GridLines -> None, 
   GridLinesStyle -> Automatic, 
   FilterRules[{opts}, Options[Graphics]], Options[PlotGrid]])

PlotGrid[QuadrantII, 
  opts : OptionsPattern[] /; ! FreeQ[{opts}, PointGrid]] := (Graphics[
   Flatten[{DeleteCases[Flatten[{OptionValue[GridLinesStyle]}], 
      PointGrid], Point[makeGridPts[{{-10, 10}, {-10, 10}}]]}], 
   PlotRange -> {{-10, 0}, {0, 10}}, GridLines -> None, 
   GridLinesStyle -> Automatic, 
   FilterRules[{opts}, Options[Graphics]], Options[PlotGrid]])

PlotGrid[QuadrantII, opts : OptionsPattern[]] := 
 Graphics[{}, PlotRange -> {{-10, 0}, {0, 10}}, opts, 
 Options[PlotGrid]/.gridRules[{-10,10},{-10,10}]]

PlotGrid[QuadrantIII, 
  opts : OptionsPattern[] /; ! FreeQ[{opts}, XGrid]] := (Graphics[
   Flatten[{DeleteCases[Flatten[{OptionValue[GridLinesStyle]}], 
      XGrid], Line[makeXGrid[{{-10, 10}, {-10, 10}}]]}], 
   PlotRange -> {{-10, 0}, {-10, 0}}, 
   GridLines -> None, GridLinesStyle -> Automatic, 
   FilterRules[{opts}, Options[Graphics]], Options[PlotGrid]])

PlotGrid[QuadrantIII, 
  opts : OptionsPattern[] /; ! FreeQ[{opts}, PointGrid]] := (Graphics[
   Flatten[{DeleteCases[Flatten[{OptionValue[GridLinesStyle]}], 
      PointGrid], Point[makeGridPts[{{-10, 10}, {-10, 10}}]]}], 
   PlotRange -> {{-10, 0}, {-10, 0}},  
   GridLines -> None, GridLinesStyle -> Automatic, 
   FilterRules[{opts}, Options[Graphics]], Options[PlotGrid]])

PlotGrid[QuadrantIII, opts : OptionsPattern[]] := 
 Graphics[{}, PlotRange -> {{-10, 0}, {-10, 0}}, opts, 
  Options[PlotGrid]/.gridRules[{-10,10},{-10,10}]]

PlotGrid[QuadrantIV, 
  opts : OptionsPattern[] /; ! FreeQ[{opts}, XGrid]] := (Graphics[
   Flatten[{DeleteCases[Flatten[{OptionValue[GridLinesStyle]}], 
      XGrid], Line[makeXGrid[{{-10, 10}, {-10, 10}}]]}], 
   PlotRange -> {{0, 10}, {-10, 0}},  
   GridLines -> None, GridLinesStyle -> Automatic, 
   FilterRules[{opts}, Options[Graphics]], Options[PlotGrid]])

PlotGrid[QuadrantIV, 
  opts : OptionsPattern[] /; ! FreeQ[{opts}, PointGrid]] := (Graphics[
   Flatten[{DeleteCases[Flatten[{OptionValue[GridLinesStyle]}], 
      PointGrid], Point[makeGridPts[{{-10, 10}, {-10, 10}}]]}], 
   PlotRange -> {{0, 10}, {-10, 0}}, 
   GridLines -> None, GridLinesStyle -> Automatic, 
   FilterRules[{opts}, Options[Graphics]], Options[PlotGrid]])

PlotGrid[QuadrantIV, opts : OptionsPattern[]] := 
 Graphics[{}, PlotRange -> {{0, 10}, {-10, 0}}, opts, 
  Options[PlotGrid]/.gridRules[{-10,10},{-10,10}]]

PlotGrid[{QuadrantI, QuadrantII}, 
  opts : OptionsPattern[] /; ! FreeQ[{opts}, XGrid]] := (Graphics[
   Flatten[{DeleteCases[Flatten[{OptionValue[GridLinesStyle]}], 
      XGrid], Line[makeXGrid[{{-10, 10}, {-10, 10}}]]}], 
   PlotRange -> {{-10, 10}, {0, 10}}, GridLines -> None, 
   GridLinesStyle -> Automatic, 
   FilterRules[{opts}, Options[Graphics]], Options[PlotGrid]])

PlotGrid[{QuadrantI, QuadrantII}, 
  opts : OptionsPattern[] /; ! FreeQ[{opts}, PointGrid]] := (Graphics[
   Flatten[{DeleteCases[Flatten[{OptionValue[GridLinesStyle]}], 
      PointGrid], Point[makeGridPts[{{-10, 10}, {-10, 10}}]]}], 
   PlotRange -> {{-10, 10}, {0, 10}}, GridLines -> None, 
   GridLinesStyle -> Automatic, 
   FilterRules[{opts}, Options[Graphics]], Options[PlotGrid]])

PlotGrid[{QuadrantI, QuadrantII}, opts : OptionsPattern[]] := (
 Graphics[{}, PlotRange -> {{-10, 10}, {0, 10}}, opts, 
  Options[PlotGrid]/.gridRules[{-10,10},{-10,10}]])

PlotGrid[{QuadrantII, QuadrantI}, opts : OptionsPattern[]] := 
 PlotGrid[{QuadrantI, QuadrantII}, opts]

PlotGrid[{QuadrantIII, QuadrantIV}, 
  opts : OptionsPattern[] /; ! FreeQ[{opts}, XGrid]] := (Graphics[
   Flatten[{DeleteCases[Flatten[{OptionValue[GridLinesStyle]}], 
      XGrid], Line[makeXGrid[{{-10, 10}, {-10, 10}}]]}], 
   PlotRange -> {{-10, 10}, {-10, 0}}, GridLines -> None, 
   GridLinesStyle -> Automatic, 
   FilterRules[{opts}, Options[Graphics]], Options[PlotGrid]])

PlotGrid[{QuadrantIII, QuadrantIV}, 
  opts : OptionsPattern[] /; ! FreeQ[{opts}, PointGrid]] := (Graphics[
   Flatten[{DeleteCases[Flatten[{OptionValue[GridLinesStyle]}], 
      PointGrid], Point[makeGridPts[{{-10, 10}, {-10, 10}}]]}], 
   PlotRange -> {{-10, 10}, {-10, 0}}, GridLines -> None, 
   GridLinesStyle -> Automatic, 
   FilterRules[{opts}, Options[Graphics]], Options[PlotGrid]])

PlotGrid[{QuadrantIII, QuadrantIV}, opts : OptionsPattern[]] := 
 Graphics[{}, PlotRange -> {{-10, 10}, {-10, 0}}, opts, 
  Options[PlotGrid]/.gridRules[{-10,10},{-10,10}]]

PlotGrid[{QuadrantIV, QuadrantIII}, opts : OptionsPattern[]] := 
 PlotGrid[{QuadrantIII, QuadrantIV}, opts]

PlotGrid[{QuadrantI, QuadrantIV}, 
  opts : OptionsPattern[] /; ! FreeQ[{opts}, XGrid]] := (Graphics[
   Flatten[{DeleteCases[Flatten[{OptionValue[GridLinesStyle]}], 
      XGrid], Line[makeXGrid[{{-10, 10}, {-10, 10}}]]}], 
   PlotRange -> {{0, 10}, {-10, 10}}, 
   GridLines -> None, GridLinesStyle -> Automatic, 
   FilterRules[{opts}, Options[Graphics]], Options[PlotGrid]])

PlotGrid[{QuadrantI, QuadrantIV}, 
  opts : OptionsPattern[] /; ! FreeQ[{opts}, PointGrid]] := (Graphics[
   Flatten[{DeleteCases[Flatten[{OptionValue[GridLinesStyle]}], 
      PointGrid], Point[makeGridPts[{{-10, 10}, {-10, 10}}]]}], 
   PlotRange -> {{0, 10}, {-10, 10}}, 
   GridLines -> None, GridLinesStyle -> Automatic, 
   FilterRules[{opts}, Options[Graphics]], Options[PlotGrid]])

PlotGrid[{QuadrantI, QuadrantIV}, opts : OptionsPattern[]] := 
 Graphics[{}, PlotRange -> {{0, 10}, {-10, 10}}, opts, 
  ImageSize -> 3 72, Options[PlotGrid]/.gridRules[{-10,10},{-10,10}]]

PlotGrid[{QuadrantIV, QuadrantI}, opts : OptionsPattern[]] := 
 PlotGrid[{QuadrantI, QuadrantII}, opts]

PlotGrid[{QuadrantII, QuadrantIII}, 
  opts : OptionsPattern[] /; ! FreeQ[{opts}, XGrid]] := (Graphics[
   Flatten[{DeleteCases[Flatten[{OptionValue[GridLinesStyle]}], 
      XGrid], Line[makeXGrid[{{-10, 10}, {-10, 10}}]]}], 
   PlotRange -> {{-10, 0}, {-10, 10}},
   GridLines -> None, GridLinesStyle -> Automatic, 
   FilterRules[{opts}, Options[Graphics]], Options[PlotGrid]])

PlotGrid[{QuadrantII, QuadrantIII}, 
  opts : OptionsPattern[] /; ! FreeQ[{opts}, PointGrid]] := (Graphics[
   Flatten[{DeleteCases[Flatten[{OptionValue[GridLinesStyle]}], 
      PointGrid], Point[makeGridPts[{{-10, 10}, {-10, 10}}]]}], 
   PlotRange -> {{-10, 0}, {-10, 10}}, 
   GridLines -> None, GridLinesStyle -> Automatic, 
   FilterRules[{opts}, Options[Graphics]], Options[PlotGrid]])

PlotGrid[{QuadrantII, QuadrantIII}, opts : OptionsPattern[]] := 
 Graphics[{}, PlotRange -> {{-10, 0}, {-10, 10}}, opts, 
  Options[PlotGrid]/.gridRules[{-10,10},{-10,10}]]

PlotGrid[{QuadrantIII, QuadrantII}, opts : OptionsPattern[]] := 
 PlotGrid[{QuadrantII, QuadrantIII}, opts]
 
PlotGrid[{xrng_List, yrng_List}, 
  opts : OptionsPattern[] /; ! FreeQ[{opts}, XGrid]] := (Graphics[
   Flatten[{DeleteCases[Flatten[{OptionValue[GridLinesStyle]}], 
      XGrid], Line[makeXGrid[{xrng, yrng}]]}], GridLines -> None, 
   GridLinesStyle -> Automatic, 
   FilterRules[{opts}, Options[Graphics]], Options[PlotGrid]])


PlotGrid[{xrng_List, yrng_List}, 
  opts : OptionsPattern[] /; ! FreeQ[{opts}, PointGrid]] := (Graphics[
   Flatten[{DeleteCases[Flatten[{OptionValue[GridLinesStyle]}], 
      PointGrid], Point[makeGridPts[{xrng, yrng}]]}], 
   GridLines -> None, GridLinesStyle -> Automatic, 
   FilterRules[{opts}, Options[Graphics]], Options[PlotGrid]])
 
PlotGrid[{xrng_List, yrng_List},opts:OptionsPattern[]] := 
 Graphics[{},PlotRange -> {xrng, yrng}, FilterRules[{opts},Options[Graphics]], 
  Options[PlotGrid]/.gridRules[xrng,yrng]]
  
(*PolarPlotGrid functions*)

makeCircles[r_?Positive] := Circle[{0, 0}, #] & /@ Range[r]

makeCircles[{rmin_?Positive, rmax_?Positive}] := 
 Circle[{0, 0}, #] & /@ Range[rmin, rmax]

makeCircles[{rmin_?Positive, rmax_?Positive, dr_?Positive}] := 
 Circle[{0, 0}, #] & /@ Range[rmin, rmax, dr]

makeRadLines[r_?Positive] := 
 Line[{-r {Cos[#], Sin[#]}, r {Cos[#], Sin[#]}} & /@ 
   Range[0, \[Pi] - \[Pi]/12, \[Pi]/6]]

makeShortLines[{rmin_?Positive, rmax_?Positive}] := 
 Line[{rmin {Cos[#], Sin[#]}, rmax {Cos[#], Sin[#]}} & /@ 
   Range[\[Pi]/12, 2 \[Pi], \[Pi]/6]]

makeDottedLines[{rmin_?Positive, rmax_?Positive}] := 
 Line[{rmin {Cos[#], Sin[#]}, rmax {Cos[#], Sin[#]}} & /@ 
   Complement[Range[\[Pi]/36, 2 \[Pi], \[Pi]/36], 
    Range[0, 2 \[Pi], \[Pi]/12]]]

makeTickLabels[r_?Positive,styls_, "Degrees"] := 
 Join[{Text[Column[{Style["0",styls], Style["360",styls]}, Center], {1, 0} r, {-2.0, 1/5}]}, 
  Table[Text[
   Style[t 360/(2 \[Pi]),styls], {Cos[t], 
      Sin[t]} r, -{Cos[t], Sin[t]} 2.0  ], {t, \[Pi]/12, 
    2 \[Pi] - \[Pi]/12, \[Pi]/12}]]

makeTickLabels[r_?Positive,styls_, "Radians"] := 
  Join[{Text[
     Column[{Style["0", 10], 
       Style[ToString[TraditionalForm[2 \[Pi]]], styls]}, 
      Center], {1, 0} r, {-3.50, 1/5}]}, 
   Table[Inset[Style[TraditionalForm[t], styls], 
     Scaled[.05 {Cos[t], Sin[t]}, {Cos[t], Sin[t]} r] ], {t, \[Pi]/
      12, 2 \[Pi] - \[Pi]/12, \[Pi]/12}]];

makeAxesTickLabels[r_?Positive,styls_] := 
 Inset[Style[#, styls], Scaled[{-.01, -.015}, {#, 0}]] & /@ Range[r]

makeAxesTickLabels[{rmin_?Positive, rmax_?Positive},styls_] := 
 Inset[Style[#, styls], Scaled[{-.01, -.015}, {#, 0}]] & /@ 
  Range[rmin, rmax]

makeAxesTickLabels[{rmin_?Positive, rmax_?Positive, dr_?Positive},styls_] := 
 Inset[Style[#, styls], Scaled[{-.01, -.015}, {#, 0}]] & /@ 
  Range[rmin, rmax, dr]
  
(*The grid functions*)

processOpts[{}] := {Directive[GrayLevel[.2], 10,Italic], 
  Directive[GrayLevel[.2], 10,Italic]}
  
processOpts[Automatic] := {Directive[RGBColor[0.2472, 0.24, 0.6], 
     Thickness[Tiny]], Directive[Dotted, Thickness[Small]]}

processOpts[Directive[a__]] := {Directive[a], Directive[a]}

processOpts[{Directive[a__], Directive[b__]}] := {Directive[a], 
  Directive[b]}

processOpts[{{a__}, {b__}}] := {Directive[a], Directive[b]}

processOpts[{a_, b_}] := {Directive[a], Directive[b]}

processOpts[a_] :=  {Directive[a], Directive[a]}

processOpts[{a : Except[_Directive], Directive[b__]}] := {Directive[
   a], Directive[b]}

processOpts[{Directive[a__], b : Except[_Directive]}] := {Directive[
   a], Directive[b]}

Options[PolarPlotGrid]= {AlignmentPoint -> Center, AspectRatio -> Automatic, Axes -> False, 
 AxesLabel -> None, AxesOrigin -> Automatic, AxesStyle -> {}, 
 Background -> None, BaselinePosition -> Automatic, BaseStyle -> {}, 
 ColorOutput -> Automatic, ContentSelectable -> Automatic, 
 CoordinatesToolOptions -> Automatic, Epilog -> {}, Frame -> False, 
 FrameLabel -> None, FrameStyle -> {}, FrameTicks -> Automatic, 
 FrameTicksStyle -> {}, GridLinesStyle -> Automatic, 
 ImageMargins -> 0., ImagePadding -> All, ImageSize -> Automatic, 
 ImageSizeRaw -> Automatic, LabelStyle -> {}, Method -> Automatic, 
 PlotLabel -> None, PlotRange -> All, PlotRangeClipping -> False, 
 PlotRangePadding -> Automatic, PlotRegion -> Automatic, 
 PreserveImageOptions -> Automatic, Prolog -> {}, RotateLabel -> True,
  Ticks -> Automatic, TicksStyle -> {}, 
 DisplayFunction :> $DisplayFunction, FormatType :> TraditionalForm}
 
 grdStylRools = {Automatic -> {Directive[RGBColor[0.2472, 0.24, 0.6], 
     Thickness[Tiny]], 
    Directive[Dotted, Thickness[Small]]},    
{a:Except[Directive],b:Except[Directive]}:> {Directive[a],Directive[b]},     	
{{a__List}, {b__List}} :> {Directive[a], Directive[b]}, 
 a_ :> {Directive[a], Directive[a]}, {a:Except[Directive]} :>  {Directive[a], 
    Directive[a]}}

PolarPlotGrid[] := PolarPlotGrid[5]

    
PolarPlotGrid[r_?Positive, opts : OptionsPattern[]] := Block[
	{grdStyl= processOpts[OptionValue[GridLinesStyle]],
		opts2= DeleteCases[Options[PolarPlotGrid],Rule[GridLinesStyle,_]],tcksStyl=processOpts[OptionValue[TicksStyle]]},
 Graphics[{grdStyl[[1]], 
   makeCircles[r], makeRadLines[r], makeShortLines[{1, r}], grdStyl[[2]], makeDottedLines[{1, r}], Dashing[{}], 
   Thickness[Small], makeTickLabels[r, tcksStyl[[1]],"Degrees"], 
   makeAxesTickLabels[r,tcksStyl[[2]]]}, FilterRules[{opts},opts2],opts2]]

PolarPlotGrid[r_?Positive, "Degrees", opts : OptionsPattern[]] := 
 PolarPlotGrid[r, opts]

PolarPlotGrid[{rmin_?Positive, rmax_?Positive}, 
  opts : OptionsPattern[]] := Block[
	{grdStyl= processOpts[OptionValue[GridLinesStyle]],
		opts2= DeleteCases[Options[PolarPlotGrid],Rule[GridLinesStyle,_]],
		tcksStyl=processOpts[OptionValue[TicksStyle]]},
 Graphics[{grdStyl[[1]], 
   makeCircles[{rmin, rmax}], makeRadLines[rmax], 
   makeShortLines[{rmin, rmax}], grdStyl[[2]], 
   makeDottedLines[{rmin, rmax}], Dashing[{}], Thickness[Small],
   makeTickLabels[rmax, tcksStyl[[1]],"Degrees"],
   makeAxesTickLabels[{rmin, rmax},tcksStyl[[2]]]},
   FilterRules[{opts},opts2],opts2]]

PolarPlotGrid[{rmin_?Positive, rmax_?Positive}, "Degrees", 
  opts :OptionsPattern[]] := PolarPlotGrid[{rmin, rmax},opts]

PolarPlotGrid[{rmin_?Positive, rmax_?Positive, dr_?Positive}, 
  opts : OptionsPattern[]] := Block[
	{grdStyl= processOpts[OptionValue[GridLinesStyle]],
		opts2= DeleteCases[Options[PolarPlotGrid],Rule[GridLinesStyle,_]],tcksStyl=processOpts[OptionValue[TicksStyle]]},
 Graphics[{grdStyl[[1]], 
   makeCircles[{rmin, rmax, dr}], makeRadLines[rmax], 
   makeShortLines[{rmin, rmax}], grdStyl[[2]], 
   makeDottedLines[{rmin, rmax}], Dashing[{}], Thickness[Small],
   makeTickLabels[rmax, tcksStyl[[1]],"Degrees"], 
   makeAxesTickLabels[{rmin, rmax, dr},tcksStyl[[2]]]}, FilterRules[{opts},opts2],opts2]]

PolarPlotGrid[{rmin_?Positive, rmax_?Positive, dr_?Positive}, 
  "Degrees", opts : OptionsPattern[]] := 
 PolarPlotGrid[{rmin, rmax, dr}, opts]

PolarPlotGrid[r_?Positive, "Radians", opts : OptionsPattern[]] := Block[
	{grdStyl= processOpts[OptionValue[GridLinesStyle]],
		opts2= DeleteCases[Options[PolarPlotGrid],Rule[GridLinesStyle,_]],tcksStyl=processOpts[OptionValue[TicksStyle]]},
 Graphics[{grdStyl[[1]], 
   makeCircles[r], makeRadLines[r], makeShortLines[{1, r}], grdStyl[[2]], 
   makeDottedLines[{1, r}], Dashing[{}], 
   Thickness[Small], makeTickLabels[r, tcksStyl,"Radians"], GrayLevel[.45],
   makeAxesTickLabels[r,tcksStyl[[2]]]}, FilterRules[{opts},opts2],opts2]]

PolarPlotGrid[{rmin_?Positive, rmax_?Positive}, "Radians", 
  opts : OptionsPattern[]] := Block[
	{grdStyl= processOpts[OptionValue[GridLinesStyle]],
		opts2= DeleteCases[Options[PolarPlotGrid],Rule[GridLinesStyle,_]],tcksStyl=processOpts[OptionValue[TicksStyle]]},
 Graphics[{grdStyl[[1]], 
   makeCircles[{rmin, rmax}], makeRadLines[rmax], 
   makeShortLines[{rmin, rmax}], grdStyl[[2]], 
   makeDottedLines[{rmin, rmax}], Dashing[{}], Thickness[Small], GrayLevel[.45],
   makeTickLabels[rmax,tcksStyl, "Radians"], makeAxesTickLabels[{rmin, rmax},tcksStyl[[2]]]},
   FilterRules[{opts},opts2],opts2]]

PolarPlotGrid[{rmin_?Positive, rmax_?Positive, dr_?Positive}, 
  "Radians", opts : OptionsPattern[]] := Block[
	{grdStyl= processOpts[OptionValue[GridLinesStyle]],
		opts2= DeleteCases[Options[PolarPlotGrid],Rule[GridLinesStyle,_]],tcksStyl=processOpts[OptionValue[TicksStyle]]},
 Graphics[{grdStyl[[1]], 
   makeCircles[{rmin, rmax, dr}], makeRadLines[rmax], 
   makeShortLines[{rmin, rmax}], grdStyl[[2]], 
   makeDottedLines[{rmin, rmax}], Dashing[{}], Thickness[Small],GrayLevel[.45], 
   makeTickLabels[rmax, tcksStyl,"Radians"], 
   makeAxesTickLabels[{rmin, rmax, dr},tcksStyl[[2]]]}, FilterRules[{opts},opts2],opts2]]


End[]

SetAttributes[{FunctionValues,Exact,FunctionArrowDiagram,ArrowStyle,SetsStyle,PointLabels,PointLabelStyle,PointLabelOffset, FunctionTable,
PlotGrid,PolarPlotGrid,XGrid,PointGrid,QuadrantI,QuadrantII,QuadrantIII,QuadrantIV},{Protected,ReadProtected}]





