(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     11108,        299]
NotebookOptionsPosition[     10205,        263]
NotebookOutlinePosition[     10683,        282]
CellTagsIndexPosition[     10640,        279]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Integration by Parts", "Section",
 CellChangeTimes->{{3.510070823535241*^9, 3.510070829306452*^9}}],

Cell[TextData[{
 "This notebook contains an alternate interface for using the ",
 StyleBox[ButtonBox["PartsIntegrate",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/PartsIntegrate"], "MR"],
 " function. This was constructed using the ",
 StyleBox[ButtonBox["Interpretation",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Interpretation"], "MR"],
 " function. To use this interface you need to enter values in the input \
fields, tabbing from one to another. When all the fields are filled in with \
the correct values you can obtain the results by using \[ShiftKey]\[KeyBar]\
\[EnterKey]."
}], "Text",
 CellChangeTimes->{{3.510070852291627*^9, 3.5100709043715353`*^9}, {
  3.510071137585517*^9, 3.510071137586866*^9}, {3.5100711873928823`*^9, 
  3.510071232144582*^9}, {3.5100712845145082`*^9, 3.510071392483325*^9}, {
  3.51007150890561*^9, 3.510071563661757*^9}}],

Cell[CellGroupData[{

Cell["Using the Interface", "Subsection",
 CellChangeTimes->{{3.5100717248006983`*^9, 3.510071731472039*^9}}],

Cell[CellGroupData[{

Cell["\<\
Enter the integrand in the field labeled Integrand.\
\>", "Item",
 CellChangeTimes->{{3.5100717666009893`*^9, 3.510071786440764*^9}}],

Cell["\<\
Tab to the Variable field and enter the variable\
\>", "Item",
 CellChangeTimes->{{3.510071794841124*^9, 3.5100718112496767`*^9}}],

Cell[TextData[{
 "Tab to the next field and enter the expression for ",
 StyleBox["u",
  FontSlant->"Italic"]
}], "Item",
 CellChangeTimes->{{3.5100718172016563`*^9, 3.510071853202026*^9}}],

Cell[TextData[{
 "Tab to the next field and enter the expression for \[DifferentialD]",
 StyleBox["v",
  FontSlant->"Italic"],
 " without the differential"
}], "Item",
 CellChangeTimes->{{3.510071867826253*^9, 3.5100718955711117`*^9}, {
  3.510071945419833*^9, 3.510071968339872*^9}}],

Cell["\<\
Use \[ShiftKey]\[KeyBar]\[EnterKey] to evaluate the input. The result will \
appear with the integral unevaluated.\
\>", "Item",
 CellChangeTimes->{{3.510071978436822*^9, 3.510072051029006*^9}}],

Cell["\<\
You can evaluate the output by putting your cursor in the output cell and \
using \[ShiftKey]\[KeyBar]\[EnterKey].\
\>", "Item",
 CellChangeTimes->{{3.5100720571335287`*^9, 3.510072115270059*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Interpretation", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"intgrnd", "=", 
      RowBox[{"(", 
       RowBox[{"x", " ", 
        RowBox[{"Cos", "[", "x", "]"}]}], ")"}]}], ",", 
     RowBox[{"var", "=", "x"}], ",", 
     RowBox[{"u", "=", 
      RowBox[{"(", "x", ")"}]}], ",", 
     RowBox[{"dvar", "=", 
      RowBox[{"Cos", "[", "x", "]"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
   RowBox[{"Framed", "[", "\[IndentingNewLine]", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Panel", "[", "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Grid", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Item", "[", 
              RowBox[{
               RowBox[{"Style", "[", 
                RowBox[{
                "\"\<\[Integral]u \[DifferentialD]v = u v -\[Integral]v\
\[DifferentialD]u\>\"", ",", "\"\<Subsection\>\""}], "]"}], ",", 
               RowBox[{"Alignment", "\[Rule]", " ", "Center"}]}], "]"}], ",", 
             "SpanFromLeft"}], "}"}], ",", "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Style", "[", 
              RowBox[{"\"\<Integrand\>\"", ",", "\"\<Text\>\""}], "]"}], ",", 
             
             RowBox[{"InputField", "[", 
              RowBox[{"Dynamic", "[", "intgrnd", "]"}], "]"}]}], "}"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Style", "[", 
              RowBox[{"\"\<Variable\>\"", ",", "\"\<Text\>\""}], "]"}], ",", 
             RowBox[{"InputField", "[", 
              RowBox[{"Dynamic", "[", "var", "]"}], "]"}]}], "}"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Style", "[", 
              RowBox[{"\"\<u\>\"", ",", "\"\<Text\>\""}], "]"}], ",", 
             RowBox[{"InputField", "[", 
              RowBox[{
               RowBox[{"Dynamic", "[", "u", "]"}], ",", 
               RowBox[{"FieldSize", "\[Rule]", "6"}]}], "]"}]}], "}"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Style", "[", 
              RowBox[{"\"\<dv\>\"", ",", "\"\<Text\>\""}], "]"}], ",", 
             RowBox[{"InputField", "[", 
              RowBox[{
               RowBox[{"Dynamic", "[", "dvar", "]"}], ",", 
               RowBox[{"FieldSize", "\[Rule]", " ", "6"}]}], "]"}]}], "}"}]}],
           "}"}], ",", "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"Alignment", "\[Rule]", " ", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"Right", ",", 
              RowBox[{"{", "Left", "}"}]}], "}"}], ",", "Automatic"}], 
           "}"}]}]}], "]"}], ",", 
       RowBox[{"Background", "\[Rule]", " ", "LightBrown"}]}], "]"}], ",", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"RoundingRadius", "\[Rule]", " ", "8"}], ",", 
     RowBox[{"FrameStyle", "\[Rule]", " ", 
      RowBox[{"Directive", "[", 
       RowBox[{
        RowBox[{"AbsoluteThickness", "[", "18", "]"}], ",", "Brown"}], 
       "]"}]}], ",", 
     RowBox[{"Background", "\[Rule]", " ", "Brown"}]}], "]"}], ",", 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"ClassroomUtilities`PartsIntegrate", "[", 
    RowBox[{"intgrnd", ",", "var", ",", 
     RowBox[{"{", 
      RowBox[{"u", ",", "dvar"}], "}"}]}], "]"}]}], "\[IndentingNewLine]", 
  "]"}]], "Input",
 CellOpen->False,
 CellChangeTimes->{{3.510042454592737*^9, 3.510042478760118*^9}, {
   3.5100427297890778`*^9, 3.5100428924621162`*^9}, {3.5100429395927258`*^9, 
   3.510042979570385*^9}, {3.510043111810903*^9, 3.510043257915653*^9}, {
   3.5100432922229424`*^9, 3.510043378464986*^9}, {3.5100434177887993`*^9, 
   3.5100436904322987`*^9}, {3.510043765016597*^9, 3.510044068272746*^9}, {
   3.51004410676397*^9, 3.510044111440846*^9}, {3.510044181779675*^9, 
   3.510044196146817*^9}, {3.510044379340261*^9, 3.5100444149313393`*^9}, {
   3.510044523059084*^9, 3.510044592150811*^9}, {3.510044712210698*^9, 
   3.5100447549063177`*^9}, {3.510044789157382*^9, 3.510044808869438*^9}, {
   3.510044963912077*^9, 3.510045003399816*^9}, {3.510045033963263*^9, 
   3.5100450525289087`*^9}, {3.510045096691729*^9, 3.510045140490109*^9}, {
   3.510045320022073*^9, 3.5100453271562*^9}, {3.510045392111587*^9, 
   3.5100454012428017`*^9}, {3.510050515777484*^9, 3.5100505337209682`*^9}, {
   3.510050564114478*^9, 3.510050567648623*^9}, {3.510050671303708*^9, 
   3.510050685712549*^9}, {3.5100507990870237`*^9, 3.5100508086880198`*^9}, {
   3.5100508633987923`*^9, 3.5100508862709208`*^9}, {3.510050967912056*^9, 
   3.510051226810869*^9}, {3.510051535318035*^9, 3.5100515756975946`*^9}, {
   3.510052091532201*^9, 3.510052092735968*^9}, {3.5100521613135643`*^9, 
   3.51005233724502*^9}, {3.5100525283738613`*^9, 3.510052536590681*^9}, {
   3.510052567751289*^9, 3.510052627823735*^9}, {3.510053004536727*^9, 
   3.5100530296575737`*^9}, {3.5100530625530663`*^9, 3.510053261416339*^9}, {
   3.510053305208788*^9, 3.510053400281732*^9}, {3.510053437873057*^9, 
   3.510053452890319*^9}, 3.510070727524283*^9, {3.510301030869093*^9, 
   3.510301038757773*^9}}],

Cell[BoxData[
 TagBox[
  DynamicModuleBox[{$CellContext`intgrnd$$ = $CellContext`x 
   Cos[$CellContext`x], $CellContext`var$$ = $CellContext`x, $CellContext`u$$ \
= $CellContext`x, $CellContext`dvar$$ = Cos[$CellContext`x]}, 
   InterpretationBox[
    FrameBox[
     PanelBox[
      TagBox[GridBox[{
         {
          ItemBox[
           
           StyleBox["\<\"\[Integral]u \[DifferentialD]v = u v -\[Integral]v\
\[DifferentialD]u\"\>", "Subsection",
            StripOnInput->False],
           Alignment->Center,
           StripOnInput->False], "\[SpanFromLeft]"},
         {
          StyleBox["\<\"Integrand\"\>", "Text",
           StripOnInput->False], 
          InputFieldBox[Dynamic[$CellContext`intgrnd$$]]},
         {
          StyleBox["\<\"Variable\"\>", "Text",
           StripOnInput->False], 
          InputFieldBox[Dynamic[$CellContext`var$$]]},
         {
          StyleBox["\<\"u\"\>", "Text",
           StripOnInput->False], 
          InputFieldBox[Dynamic[$CellContext`u$$],
           FieldSize->6]},
         {
          StyleBox["\<\"dv\"\>", "Text",
           StripOnInput->False], 
          InputFieldBox[Dynamic[$CellContext`dvar$$],
           FieldSize->6]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, {Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{
         "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
       "Grid"],
      Background->RGBColor[0.94, 0.91, 0.88]],
     Background->RGBColor[0.6, 0.4, 0.2],
     FrameStyle->Directive[
       AbsoluteThickness[18], 
       RGBColor[0.6, 0.4, 0.2]],
     RoundingRadius->8,
     StripOnInput->False],
    ClassroomUtilities`PartsIntegrate[$CellContext`intgrnd$$, \
$CellContext`var$$, {$CellContext`u$$, $CellContext`dvar$$}]],
   DynamicModuleValues:>{}],
  Setting[#, {0}]& ]], "Output",
 CellChangeTimes->{3.510301070986274*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowToolbars->{},
WindowSize->{1065, 763},
WindowMargins->{{101, Automatic}, {34, Automatic}},
Magnification:>FEPrivate`If[
  FEPrivate`Equal[FEPrivate`$VersionNumber, 6.], 1.5, 1.5 Inherited],
FrontEndVersion->"8.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (February 23, \
2011)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 105, 1, 100, "Section"],
Cell[687, 25, 886, 17, 84, "Text"],
Cell[CellGroupData[{
Cell[1598, 46, 109, 1, 51, "Subsection"],
Cell[CellGroupData[{
Cell[1732, 51, 143, 3, 30, "Item"],
Cell[1878, 56, 140, 3, 30, "Item"],
Cell[2021, 61, 189, 5, 30, "Item"],
Cell[2213, 68, 284, 7, 30, "Item"],
Cell[2500, 77, 204, 4, 30, "Item"],
Cell[2707, 83, 206, 4, 30, "Item"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2950, 92, 5309, 111, 28, "Input",
 CellOpen->False],
Cell[8262, 205, 1903, 53, 234, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
