(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     12302,        335]
NotebookOptionsPosition[     11409,        301]
NotebookOutlinePosition[     11867,        319]
CellTagsIndexPosition[     11824,        316]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Integration using Substitution", "Section",
 CellChangeTimes->{{3.510070823535241*^9, 3.510070829306452*^9}, {
  3.510072642429431*^9, 3.510072651751055*^9}}],

Cell[TextData[{
 "This notebook contains an alternate interface for using the ",
 StyleBox[ButtonBox["SubstituteIntegrate",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/SubstituteIntegrate"], "MR"],
 " function. This was constructed using the ",
 StyleBox[ButtonBox["Interpretation",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Interpretation"], "MR"],
 " function. To use this interface you need to enter values in the input \
fields, tabbing from one to another. When all the fields are filled in with \
the correct values you can obtain the results by using \[ShiftKey]\[KeyBar]\
\[EnterKey]."
}], "Text",
 CellChangeTimes->{{3.510070852291627*^9, 3.5100709043715353`*^9}, {
  3.510071137585517*^9, 3.510071137586866*^9}, {3.5100711873928823`*^9, 
  3.510071232144582*^9}, {3.5100712845145082`*^9, 3.510071392483325*^9}, {
  3.51007150890561*^9, 3.510071563661757*^9}, {3.510072286117893*^9, 
  3.510072299355362*^9}, {3.510072331146343*^9, 3.510072331147496*^9}}],

Cell[BoxData[
 RowBox[{"Interpretation", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"intgrnd", "=", 
      RowBox[{"(", 
       RowBox[{"x", " ", 
        RowBox[{"Sqrt", "[", 
         RowBox[{"1", " ", "-", " ", 
          RowBox[{"x", "^", "2"}]}], "]"}]}], ")"}]}], ",", 
     RowBox[{"var", "=", "x"}], ",", 
     RowBox[{"expr", "=", 
      RowBox[{"(", 
       RowBox[{"1", "-", 
        RowBox[{"x", "^", "2"}]}], ")"}]}], ",", 
     RowBox[{"newvar", "=", "u"}], ",", 
     RowBox[{"coef", "=", 
      RowBox[{"-", 
       RowBox[{"(", 
        RowBox[{"1", "/", 
         RowBox[{"(", 
          RowBox[{"2", " ", "x"}], ")"}]}], ")"}]}]}]}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"Framed", "[", "\[IndentingNewLine]", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Panel", "[", "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Grid", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Item", "[", 
              RowBox[{
               RowBox[{"Style", "[", 
                RowBox[{
                "\"\<Integration by substitution\>\"", ",", 
                 "\"\<Subsection\>\""}], "]"}], ",", 
               RowBox[{"Alignment", "\[Rule]", " ", "Center"}]}], "]"}], ",", 
             "SpanFromLeft", ",", "SpanFromLeft", ",", "SpanFromLeft"}], 
            "}"}], ",", "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Style", "[", 
              RowBox[{"\"\<Integrand\>\"", ",", "\"\<Text\>\""}], "]"}], ",", 
             
             RowBox[{"InputField", "[", 
              RowBox[{"Dynamic", "[", "intgrnd", "]"}], "]"}], ",", 
             "SpanFromLeft", ",", "SpanFromLeft"}], "}"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Style", "[", 
              RowBox[{"\"\<Variable\>\"", ",", "\"\<Text\>\""}], "]"}], ",", 
             RowBox[{"InputField", "[", 
              RowBox[{"Dynamic", "[", "var", "]"}], "]"}], ",", 
             "SpanFromLeft", ",", "SpanFromLeft"}], "}"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Style", "[", 
              RowBox[{"\"\<Expression\>\"", ",", "\"\<Text\>\""}], "]"}], ",", 
             RowBox[{"InputField", "[", 
              RowBox[{
               RowBox[{"Dynamic", "[", "expr", "]"}], ",", 
               RowBox[{"FieldSize", "\[Rule]", "6"}]}], "]"}], ",", 
             RowBox[{"Style", "[", 
              RowBox[{"\"\<\[Rule]\>\"", ",", "\"\<Subsubsection\>\""}], 
              "]"}], ",", 
             RowBox[{"InputField", "[", 
              RowBox[{"Dynamic", "[", "newvar", "]"}], "]"}]}], "}"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"StringForm", "[", 
              RowBox[{"\"\<\[DifferentialD]`1`\[Rule]\>\"", ",", "var"}], 
              "]"}], ",", 
             RowBox[{"InputField", "[", 
              RowBox[{
               RowBox[{"Dynamic", "[", "coef", "]"}], ",", 
               RowBox[{"FieldSize", "\[Rule]", " ", "6"}]}], "]"}], ",", 
             RowBox[{"Style", "[", 
              RowBox[{
               RowBox[{"StringForm", "[", 
                RowBox[{"\"\<\[DifferentialD]`1`\>\"", ",", "newvar"}], "]"}],
                ",", "\"\<Text\>\""}], "]"}], ",", "SpanFromLeft"}], "}"}]}], 
          "}"}], ",", "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"Alignment", "\[Rule]", " ", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"Right", ",", 
              RowBox[{"{", "Left", "}"}]}], "}"}], ",", "Automatic"}], 
           "}"}]}]}], "]"}], ",", 
       RowBox[{"Background", "\[Rule]", " ", "LightBrown"}]}], "]"}], ",", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"RoundingRadius", "\[Rule]", " ", "8"}], ",", 
     RowBox[{"FrameStyle", "\[Rule]", " ", 
      RowBox[{"Directive", "[", 
       RowBox[{
        RowBox[{"AbsoluteThickness", "[", "18", "]"}], ",", "Brown"}], 
       "]"}]}], ",", 
     RowBox[{"Background", "\[Rule]", " ", "Brown"}]}], "]"}], ",", 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"SubstituteIntegrate", "[", 
    RowBox[{"intgrnd", ",", "var", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"expr", "\[Rule]", " ", "newvar"}], ",", 
       RowBox[{
        RowBox[{"\[DifferentialD]", "var"}], "\[Rule]", " ", 
        RowBox[{"coef", " ", 
         RowBox[{"\[DifferentialD]", "newvar"}]}]}]}], "}"}]}], "]"}]}], 
  "\[IndentingNewLine]", "]"}]], "Input",
 CellOpen->False,
 CellChangeTimes->CompressedData["
1:eJwdyDtIQgEARuFLaZlhkIMQEaWEkigO2VBkWENLWGQQ2kPoNfSA8k4RNJXo
kkZLDUENCUbRIJImISQFVtiglGGG+UDKtMBCKxC7/x0OHxz+5KJ6powgiAYq
6CgE0lFhujuhfszCJmU754XSdm3mw90CKYI7SlIC33O/nZClUgxAjcA4BMWx
fh18VLRPwel7lQHmjftmOOw2OaCe1JzB2foP2rovlxe67euXcD4x9wRN/uco
ZCQnspAMMvKQ7algxijNxDgb9lZZa+BSoL4WnstWm2Hf7UIL5AS7pDCQ2ZRB
q4DXBn9PLB30dxCDcLuUpG3cKh+BrCvRKBTGlsMJSgnXGYF3h2sv0HDxTfvn
fMtCuf/0E6Z88SLsYfBK8NWbYSYpJ498LLgpK3LhQU7bCjXStBba1XkdvKnU
70HOxjatiyW2QXlI5IHH+eoHGElxQ9DC/glD9thKHP4Dhz75Lg==
  "]],

Cell[CellGroupData[{

Cell["Using the Interface", "Subsection",
 CellChangeTimes->{{3.5100717248006983`*^9, 3.510071731472039*^9}}],

Cell[CellGroupData[{

Cell["\<\
Enter the integrand in the field labeled Integrand.\
\>", "Item",
 CellChangeTimes->{{3.5100717666009893`*^9, 3.510071786440764*^9}}],

Cell["\<\
Tab to the Variable field and enter the variable\
\>", "Item",
 CellChangeTimes->{{3.510071794841124*^9, 3.5100718112496767`*^9}}],

Cell[TextData[{
 "Tab to the next field and enter the expression to be replaced by ",
 StyleBox["u",
  FontSlant->"Italic"]
}], "Item",
 CellChangeTimes->{{3.5100718172016563`*^9, 3.510071853202026*^9}, {
  3.510072415126645*^9, 3.510072420092128*^9}}],

Cell[TextData[{
 "Tab to the next field and enter ",
 StyleBox["u",
  FontSlant->"Italic"],
 " or whatever variable you want to use for the substitution"
}], "Item",
 CellChangeTimes->{{3.510071867826253*^9, 3.5100718955711117`*^9}, {
  3.510071945419833*^9, 3.510071968339872*^9}, {3.5100724375886993`*^9, 
  3.51007247330902*^9}}],

Cell[TextData[{
 "Solve for \[DifferentialD]",
 StyleBox["x",
  FontSlant->"Italic"],
 " and enter the expression in the next field"
}], "Item",
 CellChangeTimes->{{3.5100724931807737`*^9, 3.51007254336523*^9}}],

Cell["\<\
Use \[ShiftKey]\[KeyBar]\[EnterKey] to evaluate the input. The result will \
appear with the integral unevaluated.\
\>", "Item",
 CellChangeTimes->{{3.510071978436822*^9, 3.510072051029006*^9}}],

Cell["\<\
You can evaluate the output by putting your cursor in the output cell and \
using \[ShiftKey]\[KeyBar]\[EnterKey].\
\>", "Item",
 CellChangeTimes->{{3.5100720571335287`*^9, 3.510072115270059*^9}}]
}, Open  ]],

Cell[BoxData[
 TagBox[
  DynamicModuleBox[{$CellContext`intgrnd$$ = $CellContext`x (
     1 - $CellContext`x^2)^
    Rational[1, 2], $CellContext`var$$ = $CellContext`x, $CellContext`expr$$ =
    1 - $CellContext`x^2, $CellContext`newvar$$ = $CellContext`u, \
$CellContext`coef$$ = Rational[-1, 2]/$CellContext`x}, 
   InterpretationBox[
    FrameBox[
     PanelBox[
      TagBox[GridBox[{
         {
          ItemBox[
           StyleBox["\<\"Integration by substitution\"\>", "Subsection",
            StripOnInput->False],
           Alignment->Center,
           StripOnInput->False], "\[SpanFromLeft]", "\[SpanFromLeft]", 
          "\[SpanFromLeft]"},
         {
          StyleBox["\<\"Integrand\"\>", "Text",
           StripOnInput->False], 
          InputFieldBox[Dynamic[$CellContext`intgrnd$$]], "\[SpanFromLeft]", 
          "\[SpanFromLeft]"},
         {
          StyleBox["\<\"Variable\"\>", "Text",
           StripOnInput->False], 
          InputFieldBox[Dynamic[$CellContext`var$$]], "\[SpanFromLeft]", 
          "\[SpanFromLeft]"},
         {
          StyleBox["\<\"Expression\"\>", "Text",
           StripOnInput->False], 
          InputFieldBox[Dynamic[$CellContext`expr$$],
           FieldSize->6], 
          StyleBox["\<\"\[Rule]\"\>", "Subsubsection",
           StripOnInput->False], 
          InputFieldBox[Dynamic[$CellContext`newvar$$]]},
         {
          InterpretationBox["\<\"\[DifferentialD]\\!\\(x\\)\[Rule]\"\>",
           StringForm["\[DifferentialD]`1`\[Rule]", $CellContext`x],
           Editable->False], 
          InputFieldBox[Dynamic[$CellContext`coef$$],
           FieldSize->6], 
          StyleBox[
           InterpretationBox["\<\"\[DifferentialD]\\!\\(u\\)\"\>",
            StringForm["\[DifferentialD]`1`", $CellContext`u],
            Editable->False], "Text",
           StripOnInput->False], "\[SpanFromLeft]"}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, {Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{
         "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
       "Grid"],
      Background->RGBColor[0.94, 0.91, 0.88]],
     Background->RGBColor[0.6, 0.4, 0.2],
     FrameStyle->Directive[
       AbsoluteThickness[18], 
       RGBColor[0.6, 0.4, 0.2]],
     RoundingRadius->8,
     StripOnInput->False],
    ClassroomUtilities`SubstituteIntegrate[$CellContext`intgrnd$$, \
$CellContext`var$$, {$CellContext`expr$$ -> $CellContext`newvar$$, 
      DifferentialD[$CellContext`var$$] -> $CellContext`coef$$ 
       DifferentialD[$CellContext`newvar$$]}]],
   DynamicModuleValues:>{}],
  Setting[#, {0}]& ]], "Input",
 CellChangeTimes->{3.510052770985424*^9}]
}, Open  ]]
}, Open  ]]
},
WindowSize->{1065, 763},
WindowMargins->{{101, Automatic}, {34, Automatic}},
Magnification:>FEPrivate`If[
  FEPrivate`Equal[FEPrivate`$VersionNumber, 6.], 1.5, 1.5 Inherited],
FrontEndVersion->"8.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (February 23, \
2011)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 164, 2, 100, "Section"],
Cell[746, 26, 991, 18, 84, "Text"],
Cell[1740, 46, 5258, 126, 28, "Input",
 CellOpen->False],
Cell[CellGroupData[{
Cell[7023, 176, 109, 1, 51, "Subsection"],
Cell[CellGroupData[{
Cell[7157, 181, 143, 3, 30, "Item"],
Cell[7303, 186, 140, 3, 30, "Item"],
Cell[7446, 191, 252, 6, 30, "Item"],
Cell[7701, 199, 332, 8, 30, "Item"],
Cell[8036, 209, 211, 6, 30, "Item"],
Cell[8250, 217, 204, 4, 30, "Item"],
Cell[8457, 223, 206, 4, 30, "Item"]
}, Open  ]],
Cell[8678, 230, 2703, 67, 285, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
