(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     17754,        523]
NotebookOptionsPosition[     10390,        336]
NotebookOutlinePosition[     16242,        472]
CellTagsIndexPosition[     16199,        469]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["All About Packages", "Title",
 CellMargins->{{52, 0}, {0, 0}},
 FontWeight->"Bold",
 FontColor->GrayLevel[0.6]],

Cell[CellGroupData[{

Cell["Why use packages?", "Section",
 CellChangeTimes->{{3.479559983687936*^9, 3.479559994246139*^9}}],

Cell["\<\
Sometimes you find that you are performing the same task over and over, but \
there is no particular set of functions to perform that task. Of course you \
can write a function or two to do exactly what you wish, but now you need to \
remember where the notebook with the function definitions lives. A much \
simpler solution is to write the function definitions once and to store them \
in a package.\
\>", "Text"],

Cell[CellGroupData[{

Cell["So, what is a package?", "Subsection",
 CellChangeTimes->{{3.4795601444501038`*^9, 3.479560151056699*^9}}],

Cell[TextData[{
 "A package is a plain text file that contains definitions for any number of  \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions. It is stored in a standard location in your ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " layout and is easily accessed using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s",
 StyleBox[" ",
  FontSlant->"Italic"],
 " ",
 Cell[BoxData[
  ButtonBox["Get",
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Get"]], "InlineFormula"],
 " or ",
 Cell[BoxData[
  ButtonBox["Needs",
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Needs"]], "InlineFormula"],
 " functions. Once you have installed the package in the correct location you \
can use it anytime you want by loading it with a command like ",
 StyleBox["<<ClassroomUtilities`", "MR"],
 ". Once loaded in a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " session, you can use all of the functions defined in the package just as \
you would any function built in to ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["So, how do I know what is in the package?", "Subsection",
 CellChangeTimes->{{3.479560457694462*^9, 3.479560473927733*^9}}],

Cell["\<\
There are basically two types of packages: those created by a user for their \
own convenience; these packages may or may not have documentation. The second \
type of package is one that you might obtain from Wolfram Research or another \
developer and is intended for use by a wide audience. These contain full \
documentation accessed from the Documentation Center. These often contain \
tutorials describing how to best use the functions in addition to the \
standard function reference pages.\
\>", "Text"],

Cell[TextData[{
 "You will find a link ",
 ButtonBox["Add-Ons and Packages",
  BaseStyle->"Link",
  ButtonData->"paclet:guide/InstalledAddOns"],
 " at the lower left of the front page of the Documentation Center that will \
take you to the listing of all of the packages you have installed on your \
computer."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Does this mean that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " uses packages?"
}], "Subsection",
 CellChangeTimes->{{3.479560745901348*^9, 3.4795607602527323`*^9}}],

Cell[TextData[{
 "Yes, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " makes use of a number of packages, and a list of these can be found at the \
bottom of the Documentation Center front page under ",
 ButtonBox["Standard Extra Packages",
  BaseStyle->"Link",
  ButtonData->"paclet:guide/StandardExtraPackages"],
 ". "
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Where do these packages live on my computer?", "Subsection",
 CellChangeTimes->{{3.4795608915539503`*^9, 3.4795609104716682`*^9}}],

Cell[TextData[{
 "The packages that come with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " are located in the installation directory for ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". Additional packages would normally be located in one of two places that \
can be found by evaluating ",
 Cell[BoxData[
  ButtonBox["$BaseDirectory",
   BaseStyle->"Link",
   ButtonData->"paclet:ref/$BaseDirectory"]], "InlineFormula"],
 " or ",
 Cell[BoxData[
  ButtonBox["$UserBaseDirectory",
   BaseStyle->"Link",
   ButtonData->"paclet:ref/$UserBaseDirectory"]], "InlineFormula"],
 ". Evaluating the cells below will take you to these locations on your \
machine."
}], "Text"],

Cell["\<\
For those with full permissions to install things anywhere on their computer:\
\
\>", "MathCaption"],

Cell[BoxData[
 RowBox[{"SystemOpen", "[", 
  RowBox[{"FileNameJoin", "[", 
   RowBox[{"{", 
    RowBox[{"$BaseDirectory", ",", "\"\<Applications\>\""}], "}"}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.4795610636062527`*^9, 3.4795610935481987`*^9}, {
  3.496215256411015*^9, 3.496215269178452*^9}}],

Cell["For those with limited permissions on their computer:", "MathCaption"],

Cell[BoxData[
 RowBox[{"SystemOpen", "[", 
  RowBox[{"FileNameJoin", "[", 
   RowBox[{"{", 
    RowBox[{"$UserBaseDirectory", ",", "\"\<Applications\>\""}], "}"}], "]"}],
   "]"}]], "Input",
 CellChangeTimes->{{3.479561101124815*^9, 3.4795611310043497`*^9}, 
   3.4795769701758013`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Installing the Seminar Packages", "Section",
 CellChangeTimes->{{3.479561445929161*^9, 3.479561456617325*^9}}],

Cell[CellGroupData[{

Cell["Do I need to install the seminar packages to use them?", "Subsection",
 CellChangeTimes->{{3.479561506362135*^9, 3.479561516242392*^9}, {
  3.479561630772524*^9, 3.479561634756544*^9}}],

Cell["\<\
The answer is no, you do not need to install them, but it is much easier to \
use the built-in installer and then to uninstall them when  you are done.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Can I break my ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " by installing the packages?"
}], "Subsection",
 CellChangeTimes->{{3.479561976667302*^9, 3.479561991915023*^9}}],

Cell[TextData[{
 "No, installing the packages will not break ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". If you have packages of your own installed you could have some name \
conflicts, but that is the worst that would happen. A name conflict occurs \
when a function name has two or more conflicting definitions. Such situations \
can be avoided by not loading both packages at the same time."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["So, how do I install the seminar packages?", "Subsection",
 CellChangeTimes->{{3.479576466388275*^9, 3.47957650708537*^9}}],

Cell[TextData[{
 "You can install the packages included in this seminar by clicking one of \
the buttons below, or clicking the appropriate button in the seminar \
notebook. You need to install the packages only once. Once they have been \
installed they can be loaded using ",
 StyleBox["<<", "MR"],
 StyleBox["packagename", "IT"],
 StyleBox["`", "MR"],
 ". Note: the backtick is a context symbol and is necessary for the package \
to load correctly. "
}], "Text"],

Cell["Install the packages in $BaseDirectory.", "MathCaption",
 CellChangeTimes->{{3.486797682988394*^9, 3.486797696576787*^9}}],

Cell[BoxData[
 ButtonBox[
  RowBox[{"Install", " ", "seminar", " ", "packages"}],
  Appearance->"Palette",
  ButtonFunction:>
   With[{dir = 
     FileNameJoin[{$BaseDirectory, "Applications", "ClassroomUtilities"}]}, 
    If[
     DirectoryQ[dir], 
     CellPrint[
      TextCell[
       StringForm[
       "The directory `1` already exists. Try deleting it and then \
re-installing the package", dir], "Message", "MSG"]], 
     CopyDirectory[
      FileNameJoin[{
        NotebookDirectory[], "Packages", "ClassroomUtilities"}], dir]]],
  Evaluator->Automatic,
  Method->"Preemptive"]], "Text",
 CellChangeTimes->{{3.510389119024005*^9, 3.510389136107407*^9}}],

Cell["Install the packages in $UserBaseDirectory", "MathCaption",
 CellChangeTimes->{{3.4867977056182117`*^9, 3.4867977219052353`*^9}}],

Cell[BoxData[
 ButtonBox[
  RowBox[{
  "Install", " ", "seminar", " ", "packages", " ", "in", " ", 
   "$UserBaseDirectory"}],
  Appearance->"Palette",
  ButtonFunction:>
   With[{dir = 
     FileNameJoin[{$UserBaseDirectory, "Applications", 
        "ClassroomUtilities"}]}, 
    If[
     DirectoryQ[dir], 
     CellPrint[
      TextCell[
       StringForm[
       "The directory `1` already exists. Try deleting it and then \
re-installing the package", dir], "Message", "MSG"]], 
     CopyDirectory[
      FileNameJoin[{
        NotebookDirectory[], "Packages", "ClassroomUtilities"}], dir]]],
  Evaluator->Automatic,
  Method->"Preemptive"]], "Text",
 CellChangeTimes->{{3.510389176516184*^9, 3.510389182116593*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
What is the difference between Installing a package and loading a package?\
\>", "Subsection",
 CellChangeTimes->{{3.4795767131283693`*^9, 3.479576736111801*^9}}],

Cell[TextData[{
 "When you install a package you move the file into the correct location on \
your computer for ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find it when you want to use it."
}], "Text",
 CellChangeTimes->{{3.479576750167856*^9, 3.479576801849999*^9}}],

Cell[TextData[{
 "When you load a package  you make the functions defined in the package \
available to ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " for use in your work."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["How do I find out more about packages?", "Subsection",
 CellChangeTimes->{{3.4795781689043694`*^9, 3.479578183741006*^9}}],

Cell[TextData[{
 "Go to the tutorial ",
 StyleBox[ButtonBox["Mathematica",
  BaseStyle->"Link",
  ButtonData->"paclet:tutorial/MathematicaPackages"],
  FontSlant->"Italic"],
 ButtonBox[" Packages",
  BaseStyle->"Link",
  ButtonData->"paclet:tutorial/MathematicaPackages"],
 " in the Documentation Center."
}], "Text"]
}, Open  ]]
}, Open  ]]
},
WindowToolbars->{},
WindowSize->{724, 763},
WindowMargins->{{221, Automatic}, {Automatic, 93}},
Magnification:>FEPrivate`If[
  FEPrivate`Equal[FEPrivate`$VersionNumber, 6.], 1.25, 1.25 Inherited],
FrontEndVersion->"8.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (February 23, \
2011)",
StyleDefinitions->Notebook[{
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Title"], CellMargins -> {{0, 0}, {0, 0}}, 
       CellGroupingRules -> {"TitleGrouping", 30}, PageBreakBelow -> False, 
       CellFrameColor -> GrayLevel[0.4], LineSpacing -> {1.125, 0}, 
       CounterIncrements -> "Title", 
       CounterAssignments -> {{"Subtitle", 0}, {"Section", 0}, {
         "Subsection", 0}, {"Subsubsection", 0}, {"List", 0}, {
         "NumberedFigure", 0}, {"NumberedTable", 0}, {"NumberedProgram", 0}, {
         "Equation", 0}, {"Text", 0}, {"SmallText", 0}, {"Reference", 0}}, 
       MenuSortingValue -> 1100, MenuCommandKey -> "1", FontFamily -> "Times",
        FontSize -> 36, FontSlant -> "Italic", FontColor -> GrayLevel[1]], 
      Cell[
       StyleData["Title", "Presentation"], ShowCellBracket -> False, 
       CellFrameMargins -> {{50, Inherited}, {20, 30}}, LineSpacing -> {1, 2},
        FontSize -> 40, FontSlant -> "Italic"], 
      Cell[
       StyleData["Title", "SlideShow"], ShowCellBracket -> False, 
       CellMargins -> {{0, 0}, {Inherited, 0}}, 
       CellFrameMargins -> {{48, 0}, {20, 20}}, LineSpacing -> {1, 2}, 
       FontSize -> 35, FontSlant -> "Italic"], 
      Cell[
       StyleData["Title", "Printout"], 
       CellMargins -> {{54, 0}, {Inherited, Inherited}}, CellFrameMargins -> 
       False, LineSpacing -> {0.95, 0}, FontFamily -> "Times", FontColor -> 
       GrayLevel[0], Background -> None]}, Closed]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Section"], CellFrame -> None, 
       CellMargins -> {{54, 25}, {10, 30}}, 
       CellGroupingRules -> {"SectionGrouping", 30}, PageBreakBelow -> False, 
       CounterIncrements -> "Section", 
       CounterAssignments -> {{"Subsection", 0}, {"Subsubsection", 0}}, 
       MenuSortingValue -> 1250, MenuCommandKey -> "4", FontFamily -> 
       "Helvetica", FontSize -> 18, FontWeight -> "Bold"], 
      Cell[
       StyleData["Section", "Presentation"], FontSize -> 28], 
      Cell[
       StyleData["Section", "SlideShow"], CellFrame -> None, 
       CellMargins -> {{0, 0}, {30, 0}}, FontSize -> 25], 
      Cell[
       StyleData["Section", "Printout"], CellMargins -> {{54, 24}, {3, 15}}, 
       CellFrameMargins -> False, CellFrameColor -> GrayLevel[0], FontSize -> 
       14]}, Closed]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Subsection"], CellDingbat -> None, 
       CellMargins -> {{54, 25}, {10, 26}}, 
       CellGroupingRules -> {"SectionGrouping", 40}, PageBreakBelow -> False, 
       CounterIncrements -> "Subsection", 
       CounterAssignments -> {{"Subsubsection", 0}}, MenuSortingValue -> 1300,
        MenuCommandKey -> "5", FontFamily -> "Helvetica", FontSize -> 14, 
       FontWeight -> "Bold"], 
      Cell[
       StyleData["Subsection", "Presentation"], CellDingbat -> None, 
       CellMargins -> {{74, 25}, {10, 26}}, FontSize -> 18], 
      Cell[
       StyleData["Subsection", "SlideShow"], CellFrame -> None, 
       CellMargins -> {{0, 0}, {16, 0}}, FontSize -> 25], 
      Cell[
       StyleData["Subsection", "Printout"], 
       CellMargins -> {{54, 24}, {Inherited, Inherited}}, FontSize -> 12]}, 
     Closed]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Text"], CellMargins -> {{54, 24}, {Inherited, Inherited}}, 
       LineSpacing -> {1, 4}, CounterIncrements -> "Text", MenuSortingValue -> 
       1400, MenuCommandKey -> "7", FontFamily -> "Georgia", FontSize -> 12], 
      
      Cell[
       StyleData["Text", "Presentation"], 
       CellMargins -> {{Inherited, Inherited}, {6, 24}}, FontSize -> 24], 
      Cell[
       StyleData["Text", "SlideShow"], 
       CellMargins -> {{54, Inherited}, {6, 36}}, FontSize -> 19], 
      Cell[
       StyleData["Text", "Printout"], 
       CellMargins -> {{54, 24}, {Inherited, 0}}, LineSpacing -> {1, 3}, 
       FontFamily -> "Times", FontSize -> 10]}, Closed]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Input"], CellFrame -> {{0.25, 0.25}, {0, 0.25}}, 
       CellMargins -> {{72, 24}, {0, 10}}, MenuSortingValue -> 1700, 
       MenuCommandKey -> "9", FontSize -> 12, FontWeight -> "Bold", 
       Background -> RGBColor[0.799313, 0.800504, 0.879561]], 
      Cell[
       StyleData["Input", "Presentation"], FontSize -> 22], 
      Cell[
       StyleData["Input", "SlideShow"], 
       CellMargins -> {{72, Inherited}, {0, 18}}, FontSize -> 19], 
      Cell[
       StyleData["Input", "Printout"], CellMargins -> {{54, 24}, {-1, 10}}, 
       FontSize -> 9, Background -> None]}, Closed]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Output"], CellFrame -> {{0.25, 0.25}, {0.25, 0}}, 
       CellMargins -> {{72, 24}, {10, 0}}, 
       CellLabelMargins -> {{23, Inherited}, {Inherited, Inherited}}, 
       FontSize -> 12, Background -> RGBColor[0.930663, 0.899779, 0.852537]], 
      
      Cell[
       StyleData["Output", "Presentation"], FontSize -> 22], 
      Cell[
       StyleData["Output", "SlideShow"], 
       CellMargins -> {{72, Inherited}, {Inherited, 0}}, FontSize -> 19], 
      Cell[
       StyleData["Output", "Printout"], CellMargins -> {{54, 24}, {10, -2}}, 
       FontSize -> 9, Background -> None]}, Closed]]}, Visible -> False, 
  FrontEndVersion -> 
  "8.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (February 23, 2011)", 
  StyleDefinitions -> "PrivateStylesheetFormatting.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[545, 20, 117, 3, 45, "Title"],
Cell[CellGroupData[{
Cell[687, 27, 102, 1, 72, "Section"],
Cell[792, 30, 425, 7, 115, "Text"],
Cell[CellGroupData[{
Cell[1242, 41, 112, 1, 62, "Subsection"],
Cell[1357, 44, 1090, 35, 162, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2484, 84, 129, 1, 62, "Subsection"],
Cell[2616, 87, 520, 8, 137, "Text"],
Cell[3139, 97, 322, 8, 71, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[3498, 110, 194, 6, 62, "Subsection"],
Cell[3695, 118, 339, 10, 49, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4071, 133, 136, 1, 62, "Subsection"],
Cell[4210, 136, 681, 20, 94, "Text"],
Cell[4894, 158, 110, 3, 50, "MathCaption"],
Cell[5007, 163, 305, 7, 47, "Input"],
Cell[5315, 172, 76, 0, 50, "MathCaption"],
Cell[5394, 174, 286, 7, 67, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[5729, 187, 116, 1, 72, "Section"],
Cell[CellGroupData[{
Cell[5870, 192, 191, 2, 62, "Subsection"],
Cell[6064, 196, 175, 3, 49, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6276, 204, 200, 6, 62, "Subsection"],
Cell[6479, 212, 418, 8, 115, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6934, 225, 129, 1, 62, "Subsection"],
Cell[7066, 228, 465, 10, 117, "Text"],
Cell[7534, 240, 128, 1, 50, "MathCaption"],
Cell[7665, 243, 662, 19, 40, "Text"],
Cell[8330, 264, 135, 1, 50, "MathCaption"],
Cell[8468, 267, 720, 22, 40, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[9225, 294, 172, 3, 81, "Subsection"],
Cell[9400, 299, 281, 7, 61, "Text"],
Cell[9684, 308, 193, 6, 61, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[9914, 319, 128, 1, 77, "Subsection"],
Cell[10045, 322, 317, 10, 35, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
