(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     12008,        359]
NotebookOptionsPosition[      9218,        279]
NotebookOutlinePosition[     10894,        321]
CellTagsIndexPosition[     10851,        318]
WindowTitle->Analytic Geometry - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES GUIDE", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Functions \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Distance\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/Distance"], "\<\"IntermediatePoint\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/IntermediatePoint"], \
"\<\"MakeCircle\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/MakeCircle"], \
"\<\"PerpendicularLine\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/PerpendicularLine"], \
"\<\"PointLineDistance\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/PointLineDistance"], \
"\<\"PointSlopeLine\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/PointSlopeLine"], \
"\<\"PolygonInteriiorQ\"\>":>Documentation`HelpLookup[
       "paclet:Classroom Utilities/ref/PolygonInteriiorQ"], "\<\"SecantLine\"\
\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/SecantLine"], "\<\"SecantLinePlot\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/SecantLinePlot"], "\<\"Slope\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/Slope"], "\<\"TwoPointLine\"\>":>
       Documentation`HelpLookup["paclet:ClassroomUtilities/ref/TwoPointLine"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"GuideFunction"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Classroom Utilities\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/guide/ClassroomUtilities"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"GuideMoreAboutSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Problems in Analytic Geometry\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/tutorial/ProblemsInAnalyticGeometry"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"GuideTutorialsSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"ClassroomUtilities/guide/AnalyticGeometry\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["ClassroomUtilities/guide/AnalyticGeometry"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/ClassroomUtilities/\
guide/AnalyticGeometry.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "ClassroomUtilities/guide/AnalyticGeometry", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["Analytic Geometry", "GuideTitle",
 CellID->942062912],

Cell[TextData[{
 "The functions in this package are intended to make it easy to solve \
problems in analytic geometry using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "GuideAbstract",
 CellID->2001916300],

Cell[TextData[{
 Cell[TextData[ButtonBox["Distance",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/Distance"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " get the distance between two points"
}], "GuideText",
 CellID->203374175],

Cell[TextData[{
 Cell[TextData[ButtonBox["MakeCircle",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/MakeCircle"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " construct the expression for a circle"
}], "GuideText",
 CellID->1463276848],

Cell[TextData[{
 Cell[TextData[ButtonBox["PerpendicularLine",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/PerpendicularLine"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " construct a line perpendicular to a given line"
}], "GuideText",
 CellID->253883508],

Cell[TextData[{
 Cell[TextData[ButtonBox["PointLineDistance",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/PointLineDistance"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " find the distance from a point to a line"
}], "GuideText",
 CellID->2117596014],

Cell[TextData[{
 Cell[TextData[ButtonBox["PointSlopeLine",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/PointSlopeLine"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " ",
 "construct a line given a point and a slope"
}], "GuideText",
 CellID->189725567],

Cell[TextData[{
 Cell[TextData[ButtonBox["SecantLinePlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/SecantLinePlot"]], 
  "InlineFunctionSans",
  FontWeight->"Bold"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " plot a secant line given two points on a curve"
}], "GuideText",
 CellID->1805348849],

Cell[TextData[{
 Cell[TextData[ButtonBox["TwoPointLine",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/TwoPointLine"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " construct a line given two points"
}], "GuideText",
 CellID->1851872421],

Cell[TextData[{
 StyleBox[ButtonBox["PolygonInteriiorQ",
  BaseStyle->"Link",
  ButtonData->"paclet:Classroom Utilities/ref/PolygonInteriiorQ"],
  FontWeight->"Bold"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " determine of points are in the interior of a polygon"
}], "GuideText",
 CellID->1405359366],

Cell["\t", "GuideDelimiter",
 CellID->311258892],

Cell[TextData[{
 Cell[TextData[ButtonBox["Slope",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/Slope"]], "InlineFunctionSans"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 Cell[TextData[ButtonBox["IntermediatePoint",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/IntermediatePoint"]], 
  "InlineFunctionSans"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 Cell[TextData[ButtonBox["SecantLine",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/SecantLine"]], 
  "InlineFunctionSans"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " "
}], "InlineGuideFunctionListing",
 CellID->58033752]
}, Open  ]],

Cell[CellGroupData[{

Cell["TUTORIALS", "GuideTutorialsSection",
 WholeCellGroupOpener->True,
 CellID->415694126],

Cell[TextData[ButtonBox["Problems in Analytic Geometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/tutorial/ProblemsInAnalyticGeometry"]], \
"GuideTutorial",
 CellID->806871991]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "GuideMoreAboutSection",
 WholeCellGroupOpener->True,
 CellID->23220180],

Cell[TextData[ButtonBox["Classroom Utilities",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/guide/ClassroomUtilities"]], "GuideMoreAbout",
 CellID->252299663]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Analytic Geometry - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "GuideColor", "LinkTrails" -> 
  GridBox[{{
      RowBox[{
        ButtonBox[
        "Classroom Utilities", ButtonData -> 
         "paclet:ClassroomUtilities/guide/ClassroomUtilities", 
         BaseStyle -> {"Link", "DockedLinkTrail"}], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 24, 25.8760624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Guide", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "The functions in this package are intended to make it easy to solve \
problems in analytic geometry using Mathematica.", "synonyms" -> {}, "title" -> 
    "Analytic Geometry", "titlemodifier" -> "", "windowtitle" -> 
    "Analytic Geometry", "type" -> "Guide", "uri" -> 
    "ClassroomUtilities/guide/AnalyticGeometry"}, "SearchTextTranslated" -> 
  ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[610, 21, 4360, 95, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4995, 120, 59, 1, 70, "GuideTitle",
 CellID->942062912],
Cell[5057, 123, 222, 7, 70, "GuideAbstract",
 CellID->2001916300],
Cell[5282, 132, 281, 9, 70, "GuideText",
 CellID->203374175],
Cell[5566, 143, 288, 9, 70, "GuideText",
 CellID->1463276848],
Cell[5857, 154, 310, 9, 70, "GuideText",
 CellID->253883508],
Cell[6170, 165, 305, 9, 70, "GuideText",
 CellID->2117596014],
Cell[6478, 176, 305, 10, 70, "GuideText",
 CellID->189725567],
Cell[6786, 188, 327, 10, 70, "GuideText",
 CellID->1805348849],
Cell[7116, 200, 288, 9, 70, "GuideText",
 CellID->1851872421],
Cell[7407, 211, 309, 9, 70, "GuideText",
 CellID->1405359366],
Cell[7719, 222, 48, 1, 70, "GuideDelimiter",
 CellID->311258892],
Cell[7770, 225, 770, 22, 70, "InlineGuideFunctionListing",
 CellID->58033752]
}, Open  ]],
Cell[CellGroupData[{
Cell[8577, 252, 91, 2, 70, "GuideTutorialsSection",
 CellID->415694126],
Cell[8671, 256, 198, 5, 70, "GuideTutorial",
 CellID->806871991]
}, Open  ]],
Cell[CellGroupData[{
Cell[8906, 266, 91, 2, 70, "GuideMoreAboutSection",
 CellID->23220180],
Cell[9000, 270, 176, 4, 70, "GuideMoreAbout",
 CellID->252299663]
}, Open  ]],
Cell[9191, 277, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

