(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     12597,        365]
NotebookOptionsPosition[      9363,        277]
NotebookOutlinePosition[     11348,        323]
CellTagsIndexPosition[     11305,        320]
WindowTitle->Calculus Applications - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES GUIDE", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Functions \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"ChangeIndices\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/ChangeIndices"], "\<\"CubicBezier\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/CubicBezier"], \
"\<\"CubicBezierComponents\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/CubicBezierComponents"], \
"\<\"GetSummands\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/GetSummands"], "\<\"KPH\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/KPH"], "\<\"MakeSeries\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/MakeSeries"], "\<\"MPH\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/MPH"], "\<\"PartitionSum\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/PartitionSum"], "\<\"RealSolve\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/RealSolve"], "\<\"Trajectory\"\>":>
       Documentation`HelpLookup["paclet:ClassroomUtilities/ref/Trajectory"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"GuideFunction"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Classroom Utiliities\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/guide/ClassroomUtilities"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"GuideMoreAboutSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Falling Body Problems\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/tutorial/FallingBodyProblems"], "\<\"Series \
Solutions Of ODEs\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/tutorial/SeriesSolutionsOfODEs"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"GuideTutorialsSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"ClassroomUtilities/guide/Calculus Applications\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["ClassroomUtilities/guide/Calculus Applications"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/ClassroomUtilities/\
guide/Calculus Applications.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "ClassroomUtilities/guide/Calculus Applications", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["Calculus Applications", "GuideTitle",
 CellID->942062912],

Cell["\<\
The calculus applications functions are intended to make solutions to real \
world problems easier so students can focus on the ideas involved in the \
solutions and spend less time with the arithmetic. Included are functions for \
working with falling body problems with air resistance, Bezier curves, and \
solving ordinary differential equations assuming the solution can be \
expressed as a Taylor series.\
\>", "GuideAbstract",
 CellID->2001916300],

Cell[TextData[{
 Cell[TextData[ButtonBox["CubicBezier",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/CubicBezier"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " create a piecewise continuous parametric representation of a cubic Bezier \
curve."
}], "GuideText",
 CellID->203374175],

Cell[TextData[{
 Cell[TextData[ButtonBox["CubicBezierComponents",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/CubicBezierComponents"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " get the components of a cubic Bezier curve"
}], "GuideText",
 CellID->1463276848],

Cell["\t", "GuideDelimiter",
 CellID->1784261898],

Cell[TextData[{
 Cell[TextData[ButtonBox["Trajectory",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/Trajectory"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " create a parametric representation of a falling body with air resistance"
}], "GuideText",
 CellID->628072978],

Cell["\t", "GuideDelimiter",
 CellID->1140416241],

Cell[TextData[{
 Cell[TextData[ButtonBox["MakeSeries",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/MakeSeries"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " create a symbolic series together with its derivatives"
}], "GuideText",
 CellID->1940646646],

Cell[TextData[{
 Cell[TextData[ButtonBox["ChangeIndices",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/ChangeIndices"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " change the indices in a series expression"
}], "GuideText",
 CellID->466548686],

Cell[TextData[{
 Cell[TextData[ButtonBox["PartitionSum",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/PartitionSum"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " extract initial terms from a symbolic series expression"
}], "GuideText",
 CellID->820970405],

Cell[TextData[{
 Cell[TextData[ButtonBox["GetSummands",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/GetSummands"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " get the general term from a symbolic expression"
}], "GuideText",
 CellID->467164860],

Cell["\t", "GuideDelimiter",
 CellID->311258892],

Cell[TextData[{
 Cell[TextData[ButtonBox["KPH",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/KPH"]], "InlineFunctionSans"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 Cell[TextData[ButtonBox["MPH",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/MPH"]], "InlineFunctionSans"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 ButtonBox["RealSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/RealSolve"]
}], "InlineGuideFunctionListing",
 CellID->58033752]
}, Open  ]],

Cell[CellGroupData[{

Cell["TUTORIALS", "GuideTutorialsSection",
 WholeCellGroupOpener->True,
 CellID->415694126],

Cell[TextData[ButtonBox["Falling Body Problems",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/tutorial/FallingBodyProblems"]], "GuideTutorial",
 CellID->806871991],

Cell[TextData[ButtonBox["Series Solutions Of ODEs",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/tutorial/SeriesSolutionsOfODEs"]], \
"GuideTutorial",
 CellID->1885805579]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "GuideMoreAboutSection",
 WholeCellGroupOpener->True,
 CellID->23220180],

Cell[TextData[ButtonBox["Classroom Utiliities",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/guide/ClassroomUtilities"]], "GuideMoreAbout",
 CellID->252299663]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Calculus Applications - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "GuideColor", "LinkTrails" -> 
  GridBox[{{
      RowBox[{
        StyleBox[
         ButtonBox[
         "Mathematica", ButtonData -> "paclet:guide/Mathematica", 
          BaseStyle -> {"Link", "DockedLinkTrail"}], FontSlant -> "Italic"], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 24, 28.4500624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Guide", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "The calculus applications functions are intended to make solutions to \
real world problems easier so students can focus on the ideas involved in the \
solutions and spend less time with the arithmetic. Included are functions for \
working with falling body problems with air resistance, Bezier curves, and \
solving ordinary differential equations assuming the solution can be \
expressed as a Taylor series.", "synonyms" -> {}, "title" -> 
    "Calculus Applications", "titlemodifier" -> "", "windowtitle" -> 
    "Calculus Applications", "type" -> "Guide", "uri" -> 
    "ClassroomUtilities/guide/Calculus Applications"}, "SearchTextTranslated" -> 
  ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[614, 21, 4377, 97, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[5016, 122, 63, 1, 70, "GuideTitle",
 CellID->942062912],
Cell[5082, 125, 463, 8, 70, "GuideAbstract",
 CellID->2001916300],
Cell[5548, 135, 334, 10, 70, "GuideText",
 CellID->203374175],
Cell[5885, 147, 315, 9, 70, "GuideText",
 CellID->1463276848],
Cell[6203, 158, 49, 1, 70, "GuideDelimiter",
 CellID->1784261898],
Cell[6255, 161, 322, 9, 70, "GuideText",
 CellID->628072978],
Cell[6580, 172, 49, 1, 70, "GuideDelimiter",
 CellID->1140416241],
Cell[6632, 175, 305, 9, 70, "GuideText",
 CellID->1940646646],
Cell[6940, 186, 297, 9, 70, "GuideText",
 CellID->466548686],
Cell[7240, 197, 309, 9, 70, "GuideText",
 CellID->820970405],
Cell[7552, 208, 299, 9, 70, "GuideText",
 CellID->467164860],
Cell[7854, 219, 48, 1, 70, "GuideDelimiter",
 CellID->311258892],
Cell[7905, 222, 604, 17, 70, "InlineGuideFunctionListing",
 CellID->58033752]
}, Open  ]],
Cell[CellGroupData[{
Cell[8546, 244, 91, 2, 70, "GuideTutorialsSection",
 CellID->415694126],
Cell[8640, 248, 181, 4, 70, "GuideTutorial",
 CellID->806871991],
Cell[8824, 254, 189, 5, 70, "GuideTutorial",
 CellID->1885805579]
}, Open  ]],
Cell[CellGroupData[{
Cell[9050, 264, 91, 2, 70, "GuideMoreAboutSection",
 CellID->23220180],
Cell[9144, 268, 177, 4, 70, "GuideMoreAbout",
 CellID->252299663]
}, Open  ]],
Cell[9336, 275, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

