(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     26443,        915]
NotebookOptionsPosition[     20368,        731]
NotebookOutlinePosition[     21836,        767]
CellTagsIndexPosition[     21793,        764]
WindowTitle->ClassroomFunctions - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "GuideColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Mathematica",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:ClassroomUtilities/guide/ClassroomUtilities"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[CellGroupData[{

Cell["ClassroomFunctions", "GuideTitle",
 CellID->942062912],

Cell[TextData[{
 "The functions described here are defined in a set of packages developed for \
use in teaching mathematics at the high school and community college level. \
They extend the standard functions of calculus and linear algebra built in to \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " in a way that forces students to think about the steps needed to solve a \
problem and help develop a clearer understanding of the processes of \
mathematics."
}], "GuideAbstract",
 CellID->2001916300],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox[ButtonBox["Analytic Geometry",
  BaseStyle->{"Link", "GuideFunctionsSubsection"},
  ButtonData->"paclet:ClassroomUtilities/guide/AnalyticGeometry"], 
  "GuideFunctionsSubsection"],
 " \[RightGuillemet]"
}], "GuideFunctionsSubsection",
 CellID->2118385067],

Cell[TextData[{
 Cell[TextData[ButtonBox["Distance",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/Distance"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " ",
 "find the Euclidean distance between two points"
}], "GuideText",
 CellID->974120541],

Cell[TextData[{
 Cell[TextData[ButtonBox["Slope",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/Slope"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " ",
 "find the slope of a line passing through two points"
}], "GuideText",
 CellID->1776091506],

Cell[TextData[{
 Cell[TextData[ButtonBox["MakeCircle",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/MakeCircle"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " create an expression for a circle given the radius and center"
}], "GuideText",
 CellID->1363329760],

Cell[TextData[{
 Cell[TextData[ButtonBox["PointSlopeLine",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/PointSlopeLine"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " construct a slope-intercept expression given a point and slope"
}], "GuideText",
 CellID->1980246477],

Cell[TextData[{
 Cell[TextData[ButtonBox["TwoPointLine",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/TwoPointLine"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " construct the slope-intercept form of the line containing two points"
}], "GuideText",
 CellID->824530996],

Cell[TextData[{
 Cell[TextData[ButtonBox["PerpendicularLine",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/PerpendicularLine"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " construct and expression for a line perpendicular to a given line"
}], "GuideText",
 CellID->1931928580],

Cell[TextData[{
 Cell[TextData[ButtonBox["PointLineSegment",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/PointLineSegment"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " find the shortest line from a given point to a line"
}], "GuideText",
 CellID->1208769846],

Cell[TextData[{
 Cell[TextData[ButtonBox["PointLineDistance",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/PointLineDistance"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " find the shortest distance from a given point to a line"
}], "GuideText",
 CellID->116067096],

Cell[TextData[{
 Cell[TextData[ButtonBox["SecantLine",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/SecantLine"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " construct the expression for a line passing through two points on a curve"
}], "GuideText",
 CellID->180475681],

Cell[TextData[{
 Cell[TextData[ButtonBox["SecantLinePlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/SecantLinePlot"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " visualize a secant line with a curve"
}], "GuideText",
 CellID->1007861003],

Cell[TextData[{
 Cell[TextData[ButtonBox["TangentLine",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/TangentLine"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " construct an expression for a tangent to a curve"
}], "GuideText",
 CellID->1916353532],

Cell[TextData[{
 Cell[TextData[ButtonBox["TangentLinePlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/TangentLinePlot"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " visualize a tangent to a curve"
}], "GuideText",
 CellID->194074618],

Cell["\t", "GuideDelimiterSubsection",
 CellID->566437456]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox[ButtonBox["Matrix Operations",
  BaseStyle->{"Link", "GuideFunctionsSubsection"},
  ButtonData->"paclet:ClassroomUtilities/guide/MatrixOperations"], 
  "GuideFunctionsSubsection"],
 " \[RightGuillemet]"
}], "GuideFunctionsSubsection",
 CellID->1446971400],

Cell[TextData[{
 Cell[TextData[ButtonBox["AddToRow",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/AddToRow"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " add a multiple of one row to another"
}], "GuideText",
 CellID->2139084322],

Cell[TextData[{
 Cell[TextData[ButtonBox["AugmentIdentityMatrix",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/AugmentIdentityMatrix"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " augment a square matrix with an identity matrix"
}], "GuideText",
 CellID->1984484197],

Cell[TextData[{
 Cell[TextData[ButtonBox["CreateAugmentedMatrix",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/CreateAugmentedMatrix"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " create an augmented matrix from a system of equations"
}], "GuideText",
 CellID->566465427],

Cell[TextData[{
 Cell[TextData[ButtonBox["CreateEquations",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/CreateEquations"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " create a list of equations from a matrix and a list of variables"
}], "GuideText",
 CellID->702558300],

Cell[TextData[{
 Cell[TextData[ButtonBox["GetColumns",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/GetColumns"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " extract columns from a matrix"
}], "GuideText",
 CellID->1032256894],

Cell[TextData[{
 Cell[TextData[ButtonBox["MultiplyARow",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/MultiplyARow"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " multiply a row in a matrix by a given value"
}], "GuideText",
 CellID->1768716992],

Cell[TextData[{
 Cell[TextData[ButtonBox["SwitchRows",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/SwitchRows"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " switch two rows in a matrix"
}], "GuideText",
 CellID->1357842770],

Cell["\t", "GuideDelimiterSubsection",
 CellID->1661903926]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox[ButtonBox["Functions \[RightGuillemet]",
 BaseStyle->{"Link", "GuideFunctionsSubsection"},
 ButtonData->
  "paclet:ClassroomUtilities/guide/UnderstandingFunctions"], \
"GuideFunctionsSubsection"]], "GuideFunctionsSubsection",
 CellID->319315841],

Cell[TextData[{
 Cell[TextData[ButtonBox["FunctionArrowDiagram",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/FunctionArrowDiagram"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " create a function arrow diagram for functions and relations"
}], "GuideText",
 CellID->179705834],

Cell[TextData[{
 Cell[TextData[ButtonBox["FunctionTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/FunctionTable"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " create a table of function values"
}], "GuideText",
 CellID->953944356],

Cell[TextData[{
 Cell[TextData[ButtonBox["PlotGrid",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/PlotGrid"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " create a grid for use in plotting"
}], "GuideText",
 CellID->1977366437],

Cell[TextData[{
 Cell[TextData[ButtonBox["PolarPlotGrid",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/PolarPlotGrid"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " create a grid for plotting using polar coordinates"
}], "GuideText",
 CellID->1288269334],

Cell[TextData[{
 Cell[TextData[ButtonBox["PiecewisePlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/PiecewisePlot"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " plot a piecewise continuous function using traditional textbook \
representations"
}], "GuideText",
 CellID->479410447],

Cell[TextData[{
 Cell[TextData[ButtonBox["PiecewisePolarPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/PiecewisePolarPlot"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " plot a piecewise continuous function in polar coordinates"
}], "GuideText",
 CellID->92739185],

Cell["\t", "GuideDelimiterSubsection",
 CellID->768614888]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox[ButtonBox["Derivatives \[RightGuillemet]",
 BaseStyle->{"Link", "GuideFunctionsSubsection"},
 ButtonData->
  "paclet:ClassroomUtilities/guide/Differentiation"], \
"GuideFunctionsSubsection"]], "GuideFunctionsSubsection",
 CellID->1561284760],

Cell[TextData[{
 Cell[TextData[ButtonBox["TimeIntervalPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/TimeIntervalPlot"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " visualize rate of change compared to slope"
}], "GuideText",
 CellID->410278627],

Cell[TextData[{
 Cell[TextData[ButtonBox["DifferenceQuotient",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/DifferenceQuotient"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " create a difference quotient for a function"
}], "GuideText",
 CellID->2130377119],

Cell[TextData[{
 Cell[TextData[ButtonBox["LinearEstimate",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/LinearEstimate"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " estimate the value of a function at a point near a point of known function \
value"
}], "GuideText",
 CellID->309389813],

Cell[TextData[{
 Cell[TextData[ButtonBox["DirectionFieldPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/DirectionFieldPlot"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " plot a direction field for a differential equation"
}], "GuideText",
 CellID->856677704],

Cell[TextData[{
 Cell[TextData[ButtonBox["EulerEstimate",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/EulerEstimate"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " estimate the value of a function from its slope and given point"
}], "GuideText",
 CellID->1451801640],

Cell[TextData[{
 Cell[TextData[ButtonBox["EulerSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/EulerSolve"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " approximate a solution for a differential equation using Euler's method"
}], "GuideText",
 CellID->720966266],

Cell["\t", "GuideDelimiterSubsection",
 CellID->932087504]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox[ButtonBox["Integration \[RightGuillemet]",
 BaseStyle->{"Link", "GuideFunctionsSubsection"},
 ButtonData->
  "paclet:ClassroomUtilities/guide/IntegrationTechniques"], \
"GuideFunctionsSubsection"]], "GuideFunctionsSubsection",
 CellID->1530468709],

Cell[TextData[{
 Cell[TextData[ButtonBox["SubstituteIntegrate",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/SubstituteIntegrate"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " use substitution to convert an integrand to a recognizable form"
}], "GuideText",
 CellID->28468397],

Cell[TextData[{
 Cell[TextData[ButtonBox["PartsIntegrate",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/PartsIntegrate"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " select the parts to use integration by parts"
}], "GuideText",
 CellID->1726978745],

Cell[TextData[{
 StyleBox[ButtonBox["RiemannSumPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/RiemannSumPlot"],
  FontWeight->"Bold"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " estimate the area under a curve using rectangles"
}], "GuideText",
 CellID->2090278010],

Cell[TextData[{
 StyleBox[ButtonBox["RiemannSum",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/RiemannSum"],
  FontWeight->"Bold"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " estimate the value of a definite integral by adding areas of rectangles"
}], "GuideText",
 CellID->617532797],

Cell["\t", "GuideDelimiterSubsection",
 CellID->1379494588]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox[ButtonBox["Applications of Calculus \[RightGuillemet]",
 
 BaseStyle->{"Link", "GuideFunctionsSubsection"},
 ButtonData->
  "paclet:ClassroomUtilities/guide/CalculusApplications"], \
"GuideFunctionsSubsection"]], "GuideFunctionsSubsection",
 CellID->1741838220],

Cell[TextData[{
 Cell[TextData[ButtonBox["RealSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/RealSolve"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " returns only real valued solution when using ",
 ButtonBox["Solve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Solve"]
}], "GuideText",
 CellID->997189376],

Cell[TextData[{
 Cell[TextData[ButtonBox["Trajectory",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/Trajectory"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " model trajectories subject to air resistance"
}], "GuideText",
 CellID->799461244],

Cell[TextData[{
 Cell[TextData[ButtonBox["CubicBezier",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/CubicBezier"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " create a parametric representation for a cubic Bezier curve"
}], "GuideText",
 CellID->1887432276],

Cell[TextData[{
 Cell[TextData[ButtonBox["CubicBezierComponents",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/CubicBezierComponents"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " get the components of a cubic Bezier representation"
}], "GuideText",
 CellID->1593312895],

Cell[TextData[{
 Cell[TextData[ButtonBox["MakeSeries",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/MakeSeries"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " create a symbolic Taylor series with its derivatives"
}], "GuideText",
 CellID->1836231822],

Cell[TextData[{
 Cell[TextData[ButtonBox["ChangeIndices",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/ChangeIndices"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " change the index set for a Taylor series"
}], "GuideText",
 CellID->60741317],

Cell[TextData[{
 Cell[TextData[ButtonBox["GetSummands",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/GetSummands"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " get the general term from a sum of Taylor series"
}], "GuideText",
 CellID->1046002567],

Cell["\t", "GuideDelimiter",
 CellID->1913215114],

Cell[TextData[{
 ButtonBox["D",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/D"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 ButtonBox["Integrate",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Integrate"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 ButtonBox["Solve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Solve"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 ButtonBox["DSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DSolve"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 ButtonBox["FindRoot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FindRoot"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 ButtonBox["NIntegrate",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/NIntegrate"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 ButtonBox["NDSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/NDSolve"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 ButtonBox["RowReduce",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RowReduce"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 ButtonBox["LinearSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/LinearSolve"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 ButtonBox["Inverse",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Inverse"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 ButtonBox["NullSpace",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/NullSpace"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 ButtonBox["Det",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Det"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " "
}], "InlineGuideFunctionListing",
 CellID->1848958510]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TUTORIALS", "GuideTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["Problems in Analytic Geometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/tutorial/ProblemsInAnalyticGeometry"]], \
"GuideTutorial",
 CellID->806871991],

Cell[TextData[ButtonBox["Building an Understanding of Functions",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/tutorial/BuildingAnUnderstandingOfFunctions"]], \
"GuideTutorial",
 CellID->864009227],

Cell[TextData[ButtonBox["Using Integration Techniques Functions",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/tutorial/UsingIntegrationTechniquesFunctions"]], \
"GuideTutorial",
 CellID->1406315809],

Cell[TextData[ButtonBox["Falling Body Problems",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/tutorial/FallingBodyProblems"]], "GuideTutorial",\

 CellID->744782981],

Cell[TextData[ButtonBox["Series Solutions of ODEs",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/tutorial/SeriesSolutionsOfODEs"]], \
"GuideTutorial",
 CellID->2032866951]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "GuideMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["Calculus",
 BaseStyle->"Link",
 ButtonData->"paclet:ref/Calculus"]], "GuideMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"ClassroomFunctions - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2011, 2, 22, 6, 57, 30.303521}", "context" -> 
    "ClassroomUtilities`", "keywords" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Guide", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "The functions described here are defined in a set of packages developed \
for use in teaching mathematics at the high school and community college \
level. They extend the standard functions of calculus and linear algebra \
built in to Mathematica in a way that forces students to think about the \
steps needed to solve a problem and help develop a clearer understanding of \
the processes of mathematics.", "synonyms" -> {}, "title" -> 
    "ClassroomFunctions", "type" -> "Guide", "uri" -> 
    "ClassroomUtilities/guide/ClassroomFunctions"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (November 7, \
2010)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "UTF-8"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[611, 21, 75, 1, 70, "GuideColorBar"],
Cell[689, 24, 202, 5, 70, "LinkTrail"],
Cell[CellGroupData[{
Cell[916, 33, 60, 1, 70, "GuideTitle",
 CellID->942062912],
Cell[979, 36, 510, 11, 70, "GuideAbstract",
 CellID->2001916300],
Cell[CellGroupData[{
Cell[1514, 51, 281, 7, 70, "GuideFunctionsSubsection",
 CellID->2118385067],
Cell[1798, 60, 297, 10, 70, "GuideText",
 CellID->974120541],
Cell[2098, 72, 294, 9, 70, "GuideText",
 CellID->1776091506],
Cell[2395, 83, 312, 9, 70, "GuideText",
 CellID->1363329760],
Cell[2710, 94, 321, 9, 70, "GuideText",
 CellID->1980246477],
Cell[3034, 105, 322, 9, 70, "GuideText",
 CellID->824530996],
Cell[3359, 116, 330, 9, 70, "GuideText",
 CellID->1931928580],
Cell[3692, 127, 314, 9, 70, "GuideText",
 CellID->1208769846],
Cell[4009, 138, 319, 9, 70, "GuideText",
 CellID->116067096],
Cell[4331, 149, 323, 9, 70, "GuideText",
 CellID->180475681],
Cell[4657, 160, 295, 9, 70, "GuideText",
 CellID->1007861003],
Cell[4955, 171, 301, 9, 70, "GuideText",
 CellID->1916353532],
Cell[5259, 182, 290, 9, 70, "GuideText",
 CellID->194074618],
Cell[5552, 193, 58, 1, 70, "GuideDelimiterSubsection",
 CellID->566437456]
}, Open  ]],
Cell[CellGroupData[{
Cell[5647, 199, 281, 7, 70, "GuideFunctionsSubsection",
 CellID->1446971400],
Cell[5931, 208, 283, 9, 70, "GuideText",
 CellID->2139084322],
Cell[6217, 219, 320, 9, 70, "GuideText",
 CellID->1984484197],
Cell[6540, 230, 325, 9, 70, "GuideText",
 CellID->566465427],
Cell[6868, 241, 324, 9, 70, "GuideText",
 CellID->702558300],
Cell[7195, 252, 280, 9, 70, "GuideText",
 CellID->1032256894],
Cell[7478, 263, 298, 9, 70, "GuideText",
 CellID->1768716992],
Cell[7779, 274, 278, 9, 70, "GuideText",
 CellID->1357842770],
Cell[8060, 285, 59, 1, 70, "GuideDelimiterSubsection",
 CellID->1661903926]
}, Open  ]],
Cell[CellGroupData[{
Cell[8156, 291, 268, 5, 70, "GuideFunctionsSubsection",
 CellID->319315841],
Cell[8427, 298, 329, 9, 70, "GuideText",
 CellID->179705834],
Cell[8759, 309, 289, 9, 70, "GuideText",
 CellID->953944356],
Cell[9051, 320, 280, 9, 70, "GuideText",
 CellID->1977366437],
Cell[9334, 331, 307, 9, 70, "GuideText",
 CellID->1288269334],
Cell[9644, 342, 337, 10, 70, "GuideText",
 CellID->479410447],
Cell[9984, 354, 322, 9, 70, "GuideText",
 CellID->92739185],
Cell[10309, 365, 58, 1, 70, "GuideDelimiterSubsection",
 CellID->768614888]
}, Open  ]],
Cell[CellGroupData[{
Cell[10404, 371, 264, 5, 70, "GuideFunctionsSubsection",
 CellID->1561284760],
Cell[10671, 378, 304, 9, 70, "GuideText",
 CellID->410278627],
Cell[10978, 389, 310, 9, 70, "GuideText",
 CellID->2130377119],
Cell[11291, 400, 340, 10, 70, "GuideText",
 CellID->309389813],
Cell[11634, 412, 316, 9, 70, "GuideText",
 CellID->856677704],
Cell[11953, 423, 320, 9, 70, "GuideText",
 CellID->1451801640],
Cell[12276, 434, 321, 9, 70, "GuideText",
 CellID->720966266],
Cell[12600, 445, 58, 1, 70, "GuideDelimiterSubsection",
 CellID->932087504]
}, Open  ]],
Cell[CellGroupData[{
Cell[12695, 451, 270, 5, 70, "GuideFunctionsSubsection",
 CellID->1530468709],
Cell[12968, 458, 330, 9, 70, "GuideText",
 CellID->28468397],
Cell[13301, 469, 303, 9, 70, "GuideText",
 CellID->1726978745],
Cell[13607, 480, 298, 9, 70, "GuideText",
 CellID->2090278010],
Cell[13908, 491, 312, 9, 70, "GuideText",
 CellID->617532797],
Cell[14223, 502, 59, 1, 70, "GuideDelimiterSubsection",
 CellID->1379494588]
}, Open  ]],
Cell[CellGroupData[{
Cell[14319, 508, 284, 6, 70, "GuideFunctionsSubsection",
 CellID->1741838220],
Cell[14606, 516, 369, 12, 70, "GuideText",
 CellID->997189376],
Cell[14978, 530, 294, 9, 70, "GuideText",
 CellID->799461244],
Cell[15275, 541, 312, 9, 70, "GuideText",
 CellID->1887432276],
Cell[15590, 552, 324, 9, 70, "GuideText",
 CellID->1593312895],
Cell[15917, 563, 303, 9, 70, "GuideText",
 CellID->1836231822],
Cell[16223, 574, 295, 9, 70, "GuideText",
 CellID->60741317],
Cell[16521, 585, 301, 9, 70, "GuideText",
 CellID->1046002567],
Cell[16825, 596, 49, 1, 70, "GuideDelimiter",
 CellID->1913215114],
Cell[16877, 599, 2087, 74, 70, "InlineGuideFunctionListing",
 CellID->1848958510]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[19013, 679, 62, 1, 70, "GuideTutorialsSection",
 CellID->415694126],
Cell[19078, 682, 198, 5, 70, "GuideTutorial",
 CellID->806871991],
Cell[19279, 689, 215, 5, 70, "GuideTutorial",
 CellID->864009227],
Cell[19497, 696, 217, 5, 70, "GuideTutorial",
 CellID->1406315809],
Cell[19717, 703, 183, 5, 70, "GuideTutorial",
 CellID->744782981],
Cell[19903, 710, 189, 5, 70, "GuideTutorial",
 CellID->2032866951]
}, Open  ]],
Cell[CellGroupData[{
Cell[20129, 720, 62, 1, 70, "GuideMoreAboutSection",
 CellID->23220180],
Cell[20194, 723, 132, 3, 70, "GuideMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[20341, 729, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

