(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     12177,        358]
NotebookOptionsPosition[      9226,        276]
NotebookOutlinePosition[     11055,        320]
CellTagsIndexPosition[     11012,        317]
WindowTitle->Differentiation - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES GUIDE", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Functions \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"D\"\>":>Documentation`HelpLookup[
       "paclet:ref/D"], "\<\"Derivative\"\>":>Documentation`HelpLookup[
       "paclet:ref/Derivative"], "\<\"DifferenceQuotient\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/DifferenceQuotient"], \
"\<\"DirectionFieldPlot\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/DirectionFieldPlot"], "\<\"DSolve\"\>":>
       Documentation`HelpLookup["paclet:ref/DSolve"], "\<\"EulerEstimate\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/EulerEstimate"], "\<\"EulerSolve\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/EulerSolve"], "\<\"LinearEstimate\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/LinearEstimate"], "\<\"NDSolve\"\>":>
       Documentation`HelpLookup[
       "paclet:ref/NDSolve"], "\<\"TimeIntervalPlot\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/TimeIntervalPlot"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"GuideFunction"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Classroom Utilities\"\>":>
       Documentation`HelpLookup[
       "paclet:Classroom Utilities/guide/ClassroomUtilities"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"GuideMoreAboutSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Series Solutions of ODEs\"\>":>
       Documentation`HelpLookup[
       "paclet:Classroom Utilities/tutorial/SeriesSolutionsOfODEs"], \
"\<\"Falling Body Problems\"\>":>Documentation`HelpLookup[
       "paclet:Classroom Utilities/tutorial/FallingBodyProblems"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"GuideTutorialsSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Classroom Utilities/guide/Differentiation.nb\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Classroom Utilities/guide/Differentiation.nb"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Classroom \
Utilities/guide/Differentiation.nb.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Classroom Utilities/guide/Differentiation.nb", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["Differentiation", "GuideTitle",
 CellID->942062912],

Cell["\<\
These functions are intended to help students understand the motivation for \
working with derivatives. Included are functions for investigating the \
definitions for the derivative and plots to make visual comparisons between \
rates of change and the slope of a function.\
\>", "GuideAbstract",
 CellID->2030526857],

Cell[TextData[{
 Cell[TextData[ButtonBox["DifferenceQuotient",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/DifferenceQuotient"]], 
  "InlineFunctionSans",
  FontWeight->"Bold"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " generate a difference quotient for a function"
}], "GuideText",
 CellID->203374175],

Cell[TextData[{
 Cell[TextData[ButtonBox["TimeIntervalPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/TimeIntervalPlot"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " ",
 "visualize the relationships between slope and rate of change"
}], "GuideText",
 CellID->1463276848],

Cell[TextData[{
 Cell[TextData[ButtonBox["LinearEstimate",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/LinearEstimate"]], 
  "InlineFunctionSans",
  FontWeight->"Bold"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " estimate the value of a function near a point with known value"
}], "GuideText",
 CellID->1069257971],

Cell[TextData[{
 Cell[TextData[ButtonBox["DirectionFieldPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/DirectionFieldPlot"]], 
  "InlineFunctionSans",
  FontWeight->"Bold"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " plot a direction field for an ordinary differential equation"
}], "GuideText",
 CellID->1236968210],

Cell[TextData[{
 Cell[TextData[ButtonBox["LinearEstimate",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/LinearEstimate"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " estimate the value of a function near a point with known function value"
}], "GuideText",
 CellID->2083002510],

Cell[TextData[{
 Cell[TextData[ButtonBox["EulerEstimate",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/EulerEstimate"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " estimate the value of a function given a point and the derivative"
}], "GuideText",
 CellID->744772176],

Cell[TextData[{
 Cell[TextData[ButtonBox["EulerSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/EulerSolve"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " use Euler's method to approximate a solution to a differential equation"
}], "GuideText",
 CellID->1506702276],

Cell["\t", "GuideDelimiter",
 CellID->311258892],

Cell[TextData[{
 ButtonBox["D",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/D"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 ButtonBox["DSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DSolve"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 ButtonBox["NDSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/NDSolve"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 ButtonBox["Derivative",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Derivative"]
}], "InlineGuideFunctionListing",
 CellID->58033752]
}, Open  ]],

Cell[CellGroupData[{

Cell["TUTORIALS", "GuideTutorialsSection",
 WholeCellGroupOpener->True,
 CellID->415694126],

Cell[TextData[ButtonBox["Series Solutions of ODEs",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Classroom Utilities/tutorial/SeriesSolutionsOfODEs"]], \
"GuideTutorial",
 CellID->806871991],

Cell[TextData[ButtonBox["Falling Body Problems",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Classroom Utilities/tutorial/FallingBodyProblems"]], "GuideTutorial",
 CellID->1885805579]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "GuideMoreAboutSection",
 WholeCellGroupOpener->True,
 CellID->23220180],

Cell[TextData[ButtonBox["Classroom Utilities",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Classroom Utilities/guide/ClassroomUtilities"]], "GuideMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Differentiation - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "GuideColor", "LinkTrails" -> 
  GridBox[{{
      RowBox[{
        StyleBox[
         ButtonBox[
         "Mathematica", ButtonData -> "paclet:guide/Mathematica", 
          BaseStyle -> {"Link", "DockedLinkTrail"}], FontSlant -> "Italic"], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 24, 33.2080624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Guide", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "These functions are intended to help students understand the motivation \
for working with derivatives. Included are functions for investigating the \
definitions for the derivative and plots to make visual comparisons between \
rates of change and the slope of a function.", "synonyms" -> {}, "title" -> 
    "Differentiation", "titlemodifier" -> "", "windowtitle" -> 
    "Differentiation", "type" -> "Guide", "uri" -> 
    "Classroom Utilities/guide/Differentiation.nb"}, "SearchTextTranslated" -> 
  ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[608, 21, 4305, 95, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4938, 120, 57, 1, 70, "GuideTitle",
 CellID->942062912],
Cell[4998, 123, 327, 6, 70, "GuideAbstract",
 CellID->2030526857],
Cell[5328, 131, 333, 10, 70, "GuideText",
 CellID->203374175],
Cell[5664, 143, 328, 10, 70, "GuideText",
 CellID->1463276848],
Cell[5995, 155, 343, 10, 70, "GuideText",
 CellID->1069257971],
Cell[6341, 167, 349, 10, 70, "GuideText",
 CellID->1236968210],
Cell[6693, 179, 330, 9, 70, "GuideText",
 CellID->2083002510],
Cell[7026, 190, 321, 9, 70, "GuideText",
 CellID->744772176],
Cell[7350, 201, 322, 9, 70, "GuideText",
 CellID->1506702276],
Cell[7675, 212, 48, 1, 70, "GuideDelimiter",
 CellID->311258892],
Cell[7726, 215, 643, 23, 70, "InlineGuideFunctionListing",
 CellID->58033752]
}, Open  ]],
Cell[CellGroupData[{
Cell[8406, 243, 91, 2, 70, "GuideTutorialsSection",
 CellID->415694126],
Cell[8500, 247, 189, 5, 70, "GuideTutorial",
 CellID->806871991],
Cell[8692, 254, 183, 4, 70, "GuideTutorial",
 CellID->1885805579]
}, Open  ]],
Cell[CellGroupData[{
Cell[8912, 263, 91, 2, 70, "GuideMoreAboutSection",
 CellID->23220180],
Cell[9006, 267, 178, 4, 70, "GuideMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[9199, 274, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

