(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     13765,        410]
NotebookOptionsPosition[     10239,        311]
NotebookOutlinePosition[     12155,        356]
CellTagsIndexPosition[     12112,        353]
WindowTitle->Integration Techniques - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES GUIDE", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Functions \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"AutoSimplify\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/AutoSimplify"], "\<\"MakeRadiusFunction\
\"\>":>Documentation`HelpLookup[
       "paclet:Classroom Utilities/ref/MakeRadiusFunction"], \
"\<\"MakeVerticalSegment\"\>":>Documentation`HelpLookup[
       "paclet:Classroom Utilities/ref/MakeVerticalSegment"], \
"\<\"PartsIntegrate\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/PartsIntegrate"], "\<\"RiemannSum\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/RiemannSum"], "\<\"RiemannSum3D\"\>":>
       Documentation`HelpLookup[
       "paclet:Classroom Utilities/ref/RiemannSum3D"], \
"\<\"RiemannSumPlot\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/RiemannSumPlot"], "\<\"RiemannSumPlot3D\
\"\>":>Documentation`HelpLookup[
       "paclet:Classroom Utilities/ref/RiemannSumPlot3D"], \
"\<\"Subintervals\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/Subintervals"], \
"\<\"SubstituteIntegrate\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/SubstituteIntegrate"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"GuideFunction"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Classroom Utilities\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/guide/ClassroomUtilities"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"GuideMoreAboutSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Using Integration Techniques Functions\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/tutorial/\
UsingIntegrationTechniquesFunctions"], "\<\"Integrals as Limits\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/tutorial/IntegralsAsLimits"], "\<\"Working \
with Riemann Sums\"\>":>Documentation`HelpLookup[
       "paclet:Classroom Utilities/tutorial/WorkingWithRiemannSums"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"GuideTutorialsSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"ClassroomUtilities/guide/IntegrationTechniques\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["ClassroomUtilities/guide/IntegrationTechniques"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/ClassroomUtilities/\
guide/IntegrationTechniques.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "ClassroomUtilities/guide/IntegrationTechniques", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["Integration Techniques", "GuideTitle",
 CellID->942062912],

Cell["\<\
The functions PartsIntegrate and SubstituteIntegrate defined here are \
intended to be used in an interactive classroom session to assist students in \
learning to recognize appropriate substitutions for simplifying integrals. \
The RiemannSumPlot function is used for investigating integrals as limiting \
cases of known geometric forms.\
\>", "GuideAbstract",
 CellID->2001916300],

Cell[CellGroupData[{

Cell["Simplifying Integrals", "GuideFunctionsSubsection",
 CellID->259153847],

Cell[TextData[{
 Cell[TextData[ButtonBox["PartsIntegrate",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/PartsIntegrate"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " using the method of integration by parts"
}], "GuideText",
 CellID->203374175],

Cell[TextData[{
 Cell[TextData[ButtonBox["SubstituteIntegrate",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/SubstituteIntegrate"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " using the method of substitution in integrating"
}], "GuideText",
 CellID->1463276848],

Cell["\t", "GuideDelimiter",
 CellID->311258892],

Cell[TextData[{
 Cell[TextData[ButtonBox["AutoSimplify",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/AutoSimplify"]], 
  "InlineFunctionSans"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " "
}], "InlineGuideFunctionListing",
 CellID->58033752]
}, Open  ]],

Cell[CellGroupData[{

Cell["Integrals as Limits", "GuideFunctionsSubsection",
 CellID->1882021494],

Cell[TextData[{
 Cell[TextData[ButtonBox["RiemannSumPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/RiemannSumPlot"]], 
  "InlineFunctionSans",
  FontWeight->"Bold"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " using polygons to approximate areas"
}], "GuideText",
 CellID->848612995],

Cell[TextData[{
 Cell[TextData[ButtonBox["RiemannSum",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/RiemannSum"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " get the sum of the areas of the polygons"
}], "GuideText",
 CellID->2006225339],

Cell["\t", "GuideDelimiter",
 CellID->1199707238],

Cell[TextData[{
 Cell[TextData[ButtonBox["Subintervals",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/Subintervals"]], 
  "InlineFunctionSans"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " "
}], "InlineGuideFunctionListing",
 CellID->755587383],

Cell[TextData[{
 Cell[TextData[ButtonBox["RiemannSumPlot3D",
  BaseStyle->"Link",
  ButtonData->"paclet:Classroom Utilities/ref/RiemannSumPlot3D"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " using shells and washers to approximate volumes"
}], "GuideText",
 CellID->59193836],

Cell[TextData[{
 Cell[TextData[ButtonBox["MakeRadiusFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:Classroom Utilities/ref/MakeRadiusFunction"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " a function to determine the radii of washers"
}], "GuideText",
 CellID->585873091],

Cell[TextData[{
 Cell[TextData[ButtonBox["MakeVerticalSegment",
  BaseStyle->"Link",
  ButtonData->"paclet:Classroom Utilities/ref/MakeVerticalSegment"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " create vertical segments to use in RiemannSumPlot and MakeRadiusFunction"
}], "GuideText",
 CellID->556518942],

Cell[TextData[{
 Cell[TextData[ButtonBox["RiemannSum3D",
  BaseStyle->"Link",
  ButtonData->"paclet:Classroom Utilities/ref/RiemannSum3D"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " give the numerical value of the estimate"
}], "GuideText",
 CellID->157495550]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TUTORIALS", "GuideTutorialsSection",
 WholeCellGroupOpener->True,
 CellID->415694126],

Cell[TextData[ButtonBox["Using Integration Techniques Functions",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/tutorial/UsingIntegrationTechniquesFunctions"]], \
"GuideTutorial",
 CellID->806871991],

Cell[TextData[ButtonBox["Integrals as Limits",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/tutorial/IntegralsAsLimits"]], "GuideTutorial",
 CellID->1885805579],

Cell[TextData[ButtonBox["Working with Riemann Sums",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Classroom Utilities/tutorial/WorkingWithRiemannSums"]], \
"GuideTutorial",
 CellID->1327855746]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "GuideMoreAboutSection",
 WholeCellGroupOpener->True,
 CellID->23220180],

Cell[TextData[ButtonBox["Classroom Utilities",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/guide/ClassroomUtilities"]], "GuideMoreAbout",
 CellID->252299663]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Integration Techniques - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "GuideColor", "LinkTrails" -> 
  GridBox[{{
      RowBox[{
        ButtonBox[
        "Classroom Utilities", ButtonData -> 
         "paclet:ClassroomUtilities/guide/ClassroomUtilities", 
         BaseStyle -> {"Link", "DockedLinkTrail"}], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 24, 35.6260624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Guide", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "The functions PartsIntegrate and SubstituteIntegrate defined here are \
intended to be used in an interactive classroom session to assist students in \
learning to recognize appropriate substitutions for simplifying integrals. \
The RiemannSumPlot function is used for investigating integrals as limiting \
cases of known geometric forms.", "synonyms" -> {}, "title" -> 
    "Integration Techniques", "titlemodifier" -> "", "windowtitle" -> 
    "Integration Techniques", "type" -> "Guide", "uri" -> 
    "ClassroomUtilities/guide/IntegrationTechniques"}, "SearchTextTranslated" -> 
  ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[615, 21, 4621, 102, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[5261, 127, 64, 1, 70, "GuideTitle",
 CellID->942062912],
Cell[5328, 130, 392, 7, 70, "GuideAbstract",
 CellID->2001916300],
Cell[CellGroupData[{
Cell[5745, 141, 77, 1, 70, "GuideFunctionsSubsection",
 CellID->259153847],
Cell[5825, 144, 298, 9, 70, "GuideText",
 CellID->203374175],
Cell[6126, 155, 316, 9, 70, "GuideText",
 CellID->1463276848],
Cell[6445, 166, 48, 1, 70, "GuideDelimiter",
 CellID->311258892],
Cell[6496, 169, 305, 9, 70, "InlineGuideFunctionListing",
 CellID->58033752]
}, Open  ]],
Cell[CellGroupData[{
Cell[6838, 183, 76, 1, 70, "GuideFunctionsSubsection",
 CellID->1882021494],
Cell[6917, 186, 315, 10, 70, "GuideText",
 CellID->848612995],
Cell[7235, 198, 291, 9, 70, "GuideText",
 CellID->2006225339],
Cell[7529, 209, 49, 1, 70, "GuideDelimiter",
 CellID->1199707238],
Cell[7581, 212, 306, 9, 70, "InlineGuideFunctionListing",
 CellID->755587383],
Cell[7890, 223, 309, 9, 70, "GuideText",
 CellID->59193836],
Cell[8202, 234, 311, 9, 70, "GuideText",
 CellID->585873091],
Cell[8516, 245, 341, 9, 70, "GuideText",
 CellID->556518942],
Cell[8860, 256, 295, 9, 70, "GuideText",
 CellID->157495550]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[9204, 271, 91, 2, 70, "GuideTutorialsSection",
 CellID->415694126],
Cell[9298, 275, 216, 5, 70, "GuideTutorial",
 CellID->806871991],
Cell[9517, 282, 178, 4, 70, "GuideTutorial",
 CellID->1885805579],
Cell[9698, 288, 192, 5, 70, "GuideTutorial",
 CellID->1327855746]
}, Open  ]],
Cell[CellGroupData[{
Cell[9927, 298, 91, 2, 70, "GuideMoreAboutSection",
 CellID->23220180],
Cell[10021, 302, 176, 4, 70, "GuideMoreAbout",
 CellID->252299663]
}, Open  ]],
Cell[10212, 309, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

