(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     10804,        315]
NotebookOptionsPosition[      8022,        237]
NotebookOutlinePosition[      9831,        281]
CellTagsIndexPosition[      9788,        278]
WindowTitle->Matrix Operations - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES GUIDE", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Functions \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"AddToRow\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/AddToRow"], \
"\<\"AugmentIdentityMatrix\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/AugmentIdentityMatrix"], \
"\<\"CreateAugmentedMatrix\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/CreateAugmentedMatrix"], \
"\<\"CreateEquations\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/CreateEquations"], "\<\"GetColumns\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/GetColumns"], "\<\"MultiplyARow\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/MultiplyARow"], "\<\"SwitchRows\"\>":>
       Documentation`HelpLookup["paclet:ClassroomUtilities/ref/SwitchRows"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"GuideFunction"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Classroom Utilities\"\>":>
       Documentation`HelpLookup[
       "paclet:Classroom Utilities/guide/ClassroomUtilities"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"GuideMoreAboutSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Elementary Matrix Operations\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/tutorial/ElementaryMatrixOperations"], \
"\<\"Solving Linear Systems\"\>":>Documentation`HelpLookup[
       "paclet:tutorial/SolvingLinearSystems"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"GuideTutorialsSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"ClassroomUtilities/guide/MatrixOperations\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["ClassroomUtilities/guide/MatrixOperations"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/ClassroomUtilities/\
guide/MatrixOperations.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "ClassroomUtilities/guide/MatrixOperations", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["Matrix Operations", "GuideTitle",
 CellID->942062912],

Cell["\<\
The matrix operations functions are intended to help students develop an \
understanding of basic linear algebra operations such as row reduction, \
finding inverses, and solving systems of equations while avoiding the \
pitfalls of arithmetic errors.\
\>", "GuideAbstract",
 CellID->2001916300],

Cell[TextData[{
 Cell[TextData[ButtonBox["AddToRow",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/AddToRow"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " add a multiple of one row to another"
}], "GuideText",
 CellID->2139084322],

Cell[TextData[{
 Cell[TextData[ButtonBox["AugmentIdentityMatrix",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/AugmentIdentityMatrix"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " augment a square matrix with an identity matrix"
}], "GuideText",
 CellID->1984484197],

Cell[TextData[{
 Cell[TextData[ButtonBox["CreateAugmentedMatrix",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/CreateAugmentedMatrix"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " create an augmented matrix from a system of equations"
}], "GuideText",
 CellID->566465427],

Cell[TextData[{
 Cell[TextData[ButtonBox["CreateEquations",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/CreateEquations"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " create a list of equations from a matrix and a list of variables"
}], "GuideText",
 CellID->702558300],

Cell[TextData[{
 Cell[TextData[ButtonBox["GetColumns",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/GetColumns"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " extract columns from a matrix"
}], "GuideText",
 CellID->1032256894],

Cell[TextData[{
 Cell[TextData[ButtonBox["MultiplyARow",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/MultiplyARow"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " multiply a row in a matrix by a given value"
}], "GuideText",
 CellID->1768716992],

Cell[TextData[{
 Cell[TextData[ButtonBox["SwitchRows",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/SwitchRows"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " switch two rows in a matrix"
}], "GuideText",
 CellID->1357842770]
}, Open  ]],

Cell[CellGroupData[{

Cell["TUTORIALS", "GuideTutorialsSection",
 WholeCellGroupOpener->True,
 CellID->415694126],

Cell[TextData[ButtonBox["Elementary Matrix Operations",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/tutorial/ElementaryMatrixOperations"]], \
"GuideTutorial",
 CellID->806871991],

Cell[TextData[ButtonBox["Solving Linear Systems",
 BaseStyle->"Link",
 ButtonData->"paclet:tutorial/SolvingLinearSystems"]], "GuideTutorial",
 CellID->1885805579]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "GuideMoreAboutSection",
 WholeCellGroupOpener->True,
 CellID->23220180],

Cell[TextData[ButtonBox["Classroom Utilities",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Classroom Utilities/guide/ClassroomUtilities"]], "GuideMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Matrix Operations - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "GuideColor", "LinkTrails" -> 
  GridBox[{{
      RowBox[{
        ButtonBox[
        "Classroom Utilities", ButtonData -> 
         "paclet:ClassroomUtilities/guide/ClassroomUtilities", 
         BaseStyle -> {"Link", "DockedLinkTrail"}], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 24, 37.7632624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Guide", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "The matrix operations functions are intended to help students develop an \
understanding of basic linear algebra operations such as row reduction, \
finding inverses, and solving systems of equations while avoiding the \
pitfalls of arithmetic errors.", "synonyms" -> {}, "title" -> 
    "Matrix Operations", "titlemodifier" -> "", "windowtitle" -> 
    "Matrix Operations", "type" -> "Guide", "uri" -> 
    "ClassroomUtilities/guide/MatrixOperations"}, "SearchTextTranslated" -> 
  ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[610, 21, 4047, 89, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4682, 114, 59, 1, 70, "GuideTitle",
 CellID->942062912],
Cell[4744, 117, 305, 6, 70, "GuideAbstract",
 CellID->2001916300],
Cell[5052, 125, 283, 9, 70, "GuideText",
 CellID->2139084322],
Cell[5338, 136, 320, 9, 70, "GuideText",
 CellID->1984484197],
Cell[5661, 147, 325, 9, 70, "GuideText",
 CellID->566465427],
Cell[5989, 158, 324, 9, 70, "GuideText",
 CellID->702558300],
Cell[6316, 169, 280, 9, 70, "GuideText",
 CellID->1032256894],
Cell[6599, 180, 298, 9, 70, "GuideText",
 CellID->1768716992],
Cell[6900, 191, 278, 9, 70, "GuideText",
 CellID->1357842770]
}, Open  ]],
Cell[CellGroupData[{
Cell[7215, 205, 91, 2, 70, "GuideTutorialsSection",
 CellID->415694126],
Cell[7309, 209, 197, 5, 70, "GuideTutorial",
 CellID->806871991],
Cell[7509, 216, 162, 3, 70, "GuideTutorial",
 CellID->1885805579]
}, Open  ]],
Cell[CellGroupData[{
Cell[7708, 224, 91, 2, 70, "GuideMoreAboutSection",
 CellID->23220180],
Cell[7802, 228, 178, 4, 70, "GuideMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[7995, 235, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

