(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     11044,        320]
NotebookOptionsPosition[      8400,        247]
NotebookOutlinePosition[     10180,        290]
CellTagsIndexPosition[     10137,        287]
WindowTitle->Understanding Functions - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES GUIDE", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Functions \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"FunctionArrowDiagram\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/FunctionArrowDiagram"], \
"\<\"FunctionTable\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/FunctionTable"], "\<\"LogPlot\"\>":>
       Documentation`HelpLookup[
       "paclet:ref/LogPlot"], "\<\"ParametricPlot\"\>":>
       Documentation`HelpLookup[
       "paclet:ref/ParametricPlot"], "\<\"PiecewisePlot\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/PiecewisePlot"], \
"\<\"PiecewisePolarPlot\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/PiecewisePolarPlot"], "\<\"Plot\"\>":>
       Documentation`HelpLookup["paclet:ref/Plot"], "\<\"PlotGrid\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/PlotGrid"], "\<\"PolarPlot\"\>":>
       Documentation`HelpLookup[
       "paclet:ref/PolarPlot"], "\<\"PolarPlotGrid\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/ref/PolarPlotGrid"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"GuideFunction"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Classroom Utilities\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/guide/ClassroomUtilities"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"GuideMoreAboutSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Building an Understanding of Functions\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/tutorial/\
BuildingAnUnderstandingOfFunctions"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"GuideTutorialsSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"ClassroomUtilities/guide/UnderstandingFunctions\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["ClassroomUtilities/guide/UnderstandingFunctions"]}, 
            Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/ClassroomUtilities/\
guide/UnderstandingFunctions.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "ClassroomUtilities/guide/UnderstandingFunctions", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["Understanding Functions", "GuideTitle",
 CellID->942062912],

Cell["\<\
These functions are intended to introduce and review the properties of \
functions. The functions can be used in constructing slide show presentations \
for class discussion or homework assignments.\
\>", "GuideAbstract",
 CellID->276238632],

Cell[TextData[{
 Cell[TextData[ButtonBox["FunctionTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/FunctionTable"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 "  construct a table of values for a function\t"
}], "GuideText",
 CellID->203374175],

Cell[TextData[{
 Cell[TextData[ButtonBox["FunctionArrowDiagram",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/FunctionArrowDiagram"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " represent a function as two set of points with arrows indicating the values"
}], "GuideText",
 CellID->1463276848],

Cell[TextData[{
 Cell[TextData[ButtonBox["PlotGrid",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/PlotGrid"]], 
  "InlineFunctionSans"],
 StyleBox["\[LongDash]", "GuideEmDash"],
 " create a grid to use for plotting a function"
}], "GuideText",
 CellID->1295055018],

Cell[TextData[{
 Cell[TextData[ButtonBox["PolarPlotGrid",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/PolarPlotGrid"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " create a grid to use for plotting functions in polar coordinates"
}], "GuideText",
 CellID->1242925181],

Cell["\t", "GuideDelimiter",
 CellID->311258892],

Cell[TextData[{
 ButtonBox["Plot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Plot"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 ButtonBox["ParametricPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ParametricPlot"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 ButtonBox["PolarPlot",
  BaseStyle->"NewInVersionLink",
  ButtonData->"paclet:ref/PolarPlot"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 ButtonBox["LogPlot",
  BaseStyle->"NewInVersionLink",
  ButtonData->"paclet:ref/LogPlot"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 Cell[TextData[ButtonBox["PiecewisePlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/PiecewisePlot"]], 
  "InlineFunctionSans"],
 "\[NonBreakingSpace]",
 StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"],
 " ",
 Cell[TextData[ButtonBox["PiecewisePolarPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/PiecewisePolarPlot"]], 
  "InlineFunctionSans"]
}], "InlineGuideFunctionListing",
 CellID->1792299592]
}, Open  ]],

Cell[CellGroupData[{

Cell["TUTORIALS", "GuideTutorialsSection",
 WholeCellGroupOpener->True,
 CellID->415694126],

Cell[TextData[ButtonBox["Building an Understanding of Functions",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/tutorial/BuildingAnUnderstandingOfFunctions"]], \
"GuideTutorial",
 CellID->806871991]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "GuideMoreAboutSection",
 WholeCellGroupOpener->True,
 CellID->23220180],

Cell[TextData[ButtonBox["Classroom Utilities",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/guide/ClassroomUtilities"]], "GuideMoreAbout",
 CellID->252299663]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Understanding Functions - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "GuideColor", "LinkTrails" -> 
  GridBox[{{
      RowBox[{
        ButtonBox[
        "Classroom Utilities", ButtonData -> 
         "paclet:ClassroomUtilities/guide/ClassroomUtilities", 
         BaseStyle -> {"Link", "DockedLinkTrail"}], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 24, 42.3808624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Guide", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "These functions are intended to introduce and review the properties of \
functions. The functions can be used in constructing slide show presentations \
for class discussion or homework assignments.", "synonyms" -> {}, "title" -> 
    "Understanding Functions", "titlemodifier" -> "", "windowtitle" -> 
    "Understanding Functions", "type" -> "Guide", "uri" -> 
    "ClassroomUtilities/guide/UnderstandingFunctions"}, 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[616, 21, 4247, 97, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4888, 122, 65, 1, 70, "GuideTitle",
 CellID->942062912],
Cell[4956, 125, 251, 5, 70, "GuideAbstract",
 CellID->276238632],
Cell[5210, 132, 301, 9, 70, "GuideText",
 CellID->203374175],
Cell[5514, 143, 346, 9, 70, "GuideText",
 CellID->1463276848],
Cell[5863, 154, 285, 8, 70, "GuideText",
 CellID->1295055018],
Cell[6151, 164, 321, 9, 70, "GuideText",
 CellID->1242925181],
Cell[6475, 175, 48, 1, 70, "GuideDelimiter",
 CellID->311258892],
Cell[6526, 178, 1179, 37, 70, "InlineGuideFunctionListing",
 CellID->1792299592]
}, Open  ]],
Cell[CellGroupData[{
Cell[7742, 220, 91, 2, 70, "GuideTutorialsSection",
 CellID->415694126],
Cell[7836, 224, 215, 5, 70, "GuideTutorial",
 CellID->806871991]
}, Open  ]],
Cell[CellGroupData[{
Cell[8088, 234, 91, 2, 70, "GuideMoreAboutSection",
 CellID->23220180],
Cell[8182, 238, 176, 4, 70, "GuideMoreAbout",
 CellID->252299663]
}, Open  ]],
Cell[8373, 245, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

