(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     13990,        444]
NotebookOptionsPosition[     10239,        327]
NotebookOutlinePosition[     12368,        384]
CellTagsIndexPosition[     12283,        379]
WindowTitle->ColorListFunction - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Visualizing Statistical Concepts\"\>":>
       Documentation`HelpLookup[
       "paclet:Classroom Utilities/tutorial/VisualizingStatisticalConcepts"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"TutorialsSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Classroom Utilities/ref/ColorListFunction\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Classroom Utilities/ref/ColorListFunction"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Classroom \
Utilities/ref/ColorListFunction.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Classroom Utilities/ref/ColorListFunction", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell["ColorListFunction", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ColorListFunction", "[", 
       RowBox[{
        StyleBox["range", "TI"], ",", "\[Ellipsis]"}], "]"}]], 
      "InlineFormula"],
     " \[LineSeparator]is a function representing a color scheme"
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}},
 CellID->982511436]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 WholeCellGroupOpener->True,
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->144152998],

Cell["The ColorListFunction is created by MakeColorList.", "Notes",
 CellID->1067943069],

Cell["ColorList takes integer values for arguments.", "Notes",
 CellID->1039445295],

Cell["\<\
It can be used anyplace a color function would be used with integer arguments.\
\>", "Notes",
 CellID->177604720]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 WholeCellGroupOpener->True,
 CellTags->"PrimaryExamplesSection",
 CellID->1007346944],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->291610544],

Cell["This loads the package.", "ExampleText",
 CellID->69978676],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1458052623],

Cell["Create a ColorListFunction.", "ExampleText",
 CellID->554538844],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MakeColorList", "[", 
  RowBox[{"\"\<GoodColors\>\"", ",", 
   RowBox[{"{", 
    RowBox[{
    "Red", ",", "Green", ",", " ", "Blue", ",", "Yellow", ",", "Orange", ",", 
     "Purple"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->55323128],

Cell[BoxData[
 RowBox[{"ColorListFunction", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "6", ",", "1"}], "}"}], ",", 
   PanelBox[
    StyleBox[
     GraphicsBox[{
       {RGBColor[1, 0, 0], EdgeForm[Opacity[0.1]], 
        RectangleBox[{1, 0}, Offset[{-1, -1}, {2, 1}]]}, 
       {RGBColor[0, 1, 0], EdgeForm[Opacity[0.1]], 
        RectangleBox[{2, 0}, Offset[{-1, -1}, {3, 1}]]}, 
       {RGBColor[0, 0, 1], EdgeForm[Opacity[0.1]], 
        RectangleBox[{3, 0}, Offset[{-1, -1}, {4, 1}]]}, 
       {RGBColor[1, 1, 0], EdgeForm[Opacity[0.1]], 
        RectangleBox[{4, 0}, Offset[{-1, -1}, {5, 1}]]}, 
       {RGBColor[1, 0.5, 0], EdgeForm[Opacity[0.1]], 
        RectangleBox[{5, 0}, Offset[{-1, -1}, {6, 1}]]}, 
       {RGBColor[0.5, 0, 0.5], EdgeForm[Opacity[0.1]], 
        RectangleBox[{6, 0}, Offset[{-1, -1}, {7, 1}]]}},
      AspectRatio->NCache[
        Rational[1, 10], 0.1],
      ImageSize->65,
      PlotRangePadding->None],
     StripOnInput->False,
     CacheGraphics->False],
    FrameMargins->5]}], "]"}]], "Output",
 ImageSize->{288, 28},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->2102876792]
}, Open  ]],

Cell["Call the function using ColorList.", "ExampleText",
 CellID->1911864465],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ColorList", "[", "\"\<GoodColors\>\"", "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->2055736841],

Cell[BoxData[
 RowBox[{"ColorListFunction", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "6", ",", "1"}], "}"}], ",", 
   PanelBox[
    StyleBox[
     GraphicsBox[{
       {RGBColor[1, 0, 0], EdgeForm[Opacity[0.1]], 
        RectangleBox[{1, 0}, Offset[{-1, -1}, {2, 1}]]}, 
       {RGBColor[0, 1, 0], EdgeForm[Opacity[0.1]], 
        RectangleBox[{2, 0}, Offset[{-1, -1}, {3, 1}]]}, 
       {RGBColor[0, 0, 1], EdgeForm[Opacity[0.1]], 
        RectangleBox[{3, 0}, Offset[{-1, -1}, {4, 1}]]}, 
       {RGBColor[1, 1, 0], EdgeForm[Opacity[0.1]], 
        RectangleBox[{4, 0}, Offset[{-1, -1}, {5, 1}]]}, 
       {RGBColor[1, 0.5, 0], EdgeForm[Opacity[0.1]], 
        RectangleBox[{5, 0}, Offset[{-1, -1}, {6, 1}]]}, 
       {RGBColor[0.5, 0, 0.5], EdgeForm[Opacity[0.1]], 
        RectangleBox[{6, 0}, Offset[{-1, -1}, {7, 1}]]}},
      AspectRatio->NCache[
        Rational[1, 10], 0.1],
      ImageSize->65,
      PlotRangePadding->None],
     StripOnInput->False,
     CacheGraphics->False],
    FrameMargins->5]}], "]"}]], "Output",
 ImageSize->{288, 28},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->2052173751]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 WholeCellGroupOpener->True,
 CellID->250839057],

Cell[TextData[StyleBox[ButtonBox["Visualizing Statistical Concepts",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Classroom Utilities/tutorial/VisualizingStatisticalConcepts"],
 FontWeight->"Bold"]], "Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["RELATED LINKS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "RelatedLinksSection",
 WholeCellGroupOpener->True,
 CellID->1584193535],

Cell[TextData[Cell[BoxData[
 ButtonBox["MakeColorList",
  BaseStyle->"Link",
  ButtonData->
   "paclet:Classroom Utilities/ref/MakeColorList"]], "InlineFormula",
 FontWeight->"Bold"]], "SeeAlso",
 CellID->929782353],

Cell[TextData[Cell[BoxData[
 ButtonBox["ColorList",
  BaseStyle->"Link",
  ButtonData->"paclet:Classroom Utilities/ref/ColorList"]], "InlineFormula",
 FontWeight->"Bold"]], "SeeAlso",
 CellID->259710]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"ColorListFunction - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "SymbolColor", "LinkTrails" -> 
  GridBox[{{
      RowBox[{
        StyleBox[
         ButtonBox[
         "Mathematica", ButtonData -> "paclet:guide/Mathematica", 
          BaseStyle -> {"Link", "DockedLinkTrail"}], FontSlant -> "Italic"], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 25, 25.0468624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Symbol", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "ColorListFunction[range, ...] is a function representing a color scheme",
     "synonyms" -> {}, "title" -> "ColorListFunction", "titlemodifier" -> "", 
    "windowtitle" -> "ColorListFunction", "type" -> "Symbol", "uri" -> 
    "Classroom Utilities/ref/ColorListFunction"}, "SearchTextTranslated" -> 
  ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[4650, 138, 491, 14, 45, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->1007346944]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 12139, 372}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[610, 21, 2508, 57, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3143, 82, 60, 1, 51, "ObjectName",
 CellID->1224892054],
Cell[3206, 85, 458, 14, 83, "Usage",
 CellID->982511436]
}, Open  ]],
Cell[CellGroupData[{
Cell[3701, 104, 609, 18, 39, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->144152998],
Cell[4313, 124, 88, 1, 70, "Notes",
 CellID->1067943069],
Cell[4404, 127, 83, 1, 70, "Notes",
 CellID->1039445295],
Cell[4490, 130, 123, 3, 70, "Notes",
 CellID->177604720]
}, Closed]],
Cell[CellGroupData[{
Cell[4650, 138, 491, 14, 45, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->1007346944],
Cell[CellGroupData[{
Cell[5166, 156, 195, 6, 31, "ExampleSection",
 CellID->291610544],
Cell[5364, 164, 65, 1, 32, "ExampleText",
 CellID->69978676],
Cell[5432, 167, 128, 3, 28, "Input",
 CellID->1458052623],
Cell[5563, 172, 70, 1, 32, "ExampleText",
 CellID->554538844],
Cell[CellGroupData[{
Cell[5658, 177, 277, 8, 28, "Input",
 CellID->55323128],
Cell[5938, 187, 1179, 31, 49, "Output",
 CellID->2102876792]
}, Open  ]],
Cell[7132, 221, 78, 1, 32, "ExampleText",
 CellID->1911864465],
Cell[CellGroupData[{
Cell[7235, 226, 123, 3, 28, "Input",
 CellID->2055736841],
Cell[7361, 231, 1179, 31, 49, "Output",
 CellID->2052173751]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[8601, 269, 448, 13, 70, "TutorialsSection",
 CellID->250839057],
Cell[9052, 284, 231, 5, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[CellGroupData[{
Cell[9320, 294, 456, 13, 70, "RelatedLinksSection",
 CellID->1584193535],
Cell[9779, 309, 215, 6, 70, "SeeAlso",
 CellID->929782353],
Cell[9997, 317, 200, 5, 70, "SeeAlso",
 CellID->259710]
}, Open  ]],
Cell[10212, 325, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

