(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     31703,        936]
NotebookOptionsPosition[     27274,        797]
NotebookOutlinePosition[     29625,        860]
CellTagsIndexPosition[     29505,        854]
WindowTitle->CubicBezier - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Calculus Applications\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/guide/CalculusApplications"], \
"\<\"Classroom Utilities\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/guide/ClassroomUtilities"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"MoreAboutSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"ClassroomUtilities/ref/CubicBezier\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["ClassroomUtilities/ref/CubicBezier"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/ClassroomUtilities/\
ref/CubicBezier.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "ClassroomUtilities/ref/CubicBezier", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell["CubicBezier", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"CubicBezier", "[", 
       RowBox[{
        StyleBox["var", "TI"], ",", 
        RowBox[{"{", 
         RowBox[{
          SubscriptBox[
           StyleBox["pt", "TI"], 
           StyleBox["1", "TR"]], ",", 
          SubscriptBox[
           StyleBox["pt", "TI"], 
           StyleBox["2", "TR"]], ",", 
          SubscriptBox[
           StyleBox["pt", "TI"], 
           StyleBox["3", "TR"]], ",", 
          SubscriptBox[
           StyleBox["pt", "TI"], 
           StyleBox["4", "TR"]]}], "}"}]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]creates a parametric representation of a cubic Bezier \
curve with anchor points ",
     Cell[BoxData[
      StyleBox["pt1", "TI"]], "InlineFormula"],
     " and ",
     Cell[BoxData[
      StyleBox["pt4", "TI"]], "InlineFormula"],
     " and control points ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["pt", "TI"], 
       StyleBox["2", "TR"]]], "InlineFormula"],
     " and ",
     Cell[BoxData[
      StyleBox["pt3", "TI"]], "InlineFormula"],
     "."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 WholeCellGroupOpener->True,
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->160999823],

Cell[TextData[{
 "The points ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["p", "TI"], 
   StyleBox["i", "TI"]]], "InlineFormula"],
 " can be numeric or symbolic but should always be ordered pairs."
}], "Notes",
 CellID->1067943069],

Cell["The variable should be a symbol.", "Notes",
 CellID->1074556442],

Cell["\<\
The result is given as a piecewise continuous vector valued function.\
\>", "Notes",
 CellID->1880133071]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 WholeCellGroupOpener->True,
 CellTags->"PrimaryExamplesSection",
 CellID->48055711],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->475476220],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->247295146],

Cell["\<\
Create a cubic Bezier function with anchors {-1,0} and {1,1} and control \
points {-2,1} and {2,3}.\
\>", "ExampleText",
 CellID->1232996064],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CubicBezier", "[", 
  RowBox[{"t", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "1"}], "}"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->1895137745],

Cell[BoxData[
 RowBox[{"\[Piecewise]", GridBox[{
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "1"}], "-", 
        RowBox[{"3", " ", "t"}], "+", 
        RowBox[{"15", " ", 
         SuperscriptBox["t", "2"]}], "-", 
        RowBox[{"10", " ", 
         SuperscriptBox["t", "3"]}]}], ",", 
       RowBox[{"t", " ", 
        RowBox[{"(", 
         RowBox[{"3", "+", 
          RowBox[{"3", " ", "t"}], "-", 
          RowBox[{"5", " ", 
           SuperscriptBox["t", "2"]}]}], ")"}]}]}], "}"}], 
     RowBox[{"0", "\[LessEqual]", "t", "\[LessEqual]", "1"}]},
    {
     RowBox[{"{", 
      RowBox[{"Indeterminate", ",", "Indeterminate"}], "}"}], 
     TagBox["True",
      "PiecewiseDefault",
      AutoDelete->False,
      DeletionWarning->True]}
   },
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}},
   GridBoxItemSize->{
    "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
     "RowsIndexed" -> {}},
   GridBoxSpacings->{"Columns" -> {
       Offset[0.27999999999999997`], {
        Offset[0.84]}, 
       Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
       Offset[0.2], {
        Offset[0.4]}, 
       Offset[0.2]}, "RowsIndexed" -> {}}]}]], "Output",
 ImageSize->{329, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->1355563993]
}, Open  ]],

Cell["Plot the function.", "ExampleText",
 CellID->437931636],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ParametricPlot", "[", 
  RowBox[{
   RowBox[{"CubicBezier", "[", 
    RowBox[{"t", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "2"}], ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}]}], "}"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->890547983],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnauWq0gUhrPWvMC8wjzCkcdGRkZGHmTECGRbZGQkFokbJBaJRGKRyMie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    "], {{0, 156}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 156},
  PlotRange->{{0, 180}, {0, 156}}]], "Output",
 ImageSize->{184, 156},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[3]=",
 CellID->84291219]
}, Open  ]],

Cell["Create a symbolic function.", "ExampleText",
 CellID->1341859394],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CubicBezier", "[", 
  RowBox[{"t", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"a", ",", "b"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"c", ",", "d"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"e", ",", "f"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"g", ",", "h"}], "}"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->1905057811],

Cell[BoxData[
 RowBox[{"\[Piecewise]", GridBox[{
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"-", "a"}], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", "t"}], ")"}], "3"]}], "+", 
        RowBox[{"t", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"3", " ", "c", " ", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", "1"}], "+", "t"}], ")"}], "2"]}], "+", 
           RowBox[{"t", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"3", " ", "e"}], "-", 
              RowBox[{"3", " ", "e", " ", "t"}], "+", 
              RowBox[{"g", " ", "t"}]}], ")"}]}]}], ")"}]}]}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "b"}], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", "t"}], ")"}], "3"]}], "+", 
        RowBox[{"t", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"3", " ", "d", " ", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", "1"}], "+", "t"}], ")"}], "2"]}], "+", 
           RowBox[{"t", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"3", " ", "f"}], "-", 
              RowBox[{"3", " ", "f", " ", "t"}], "+", 
              RowBox[{"h", " ", "t"}]}], ")"}]}]}], ")"}]}]}]}], "}"}], 
     RowBox[{"0", "\[LessEqual]", "t", "\[LessEqual]", "1"}]},
    {
     RowBox[{"{", 
      RowBox[{"Indeterminate", ",", "Indeterminate"}], "}"}], 
     TagBox["True",
      "PiecewiseDefault",
      AutoDelete->False,
      DeletionWarning->True]}
   },
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}},
   GridBoxItemSize->{
    "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
     "RowsIndexed" -> {}},
   GridBoxSpacings->{"Columns" -> {
       Offset[0.27999999999999997`], {
        Offset[0.84]}, 
       Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
       Offset[0.2], {
        Offset[0.4]}, 
       Offset[0.2]}, "RowsIndexed" -> {}}]}]], "Output",
 ImageSize->{400, 53},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->699518074]
}, Open  ]],

Cell["Use fixed anchor points and symbolic control points.", "ExampleText",
 CellID->1597302709],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CubicBezier", "[", 
  RowBox[{"t", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"c", ",", "d"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"e", ",", "f"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "1"}], "}"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->173559016],

Cell[BoxData[
 RowBox[{"\[Piecewise]", GridBox[{
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "1"}], "+", "t"}], ")"}], "3"], "+", 
        RowBox[{"3", " ", "c", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", "t"}], ")"}], "2"], " ", "t"}], "-", 
        RowBox[{"3", " ", "e", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "1"}], "+", "t"}], ")"}], " ", 
         SuperscriptBox["t", "2"]}], "+", 
        SuperscriptBox["t", "3"]}], ",", 
       RowBox[{"t", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", "d", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], "+", "t"}], ")"}], "2"]}], "+", 
          RowBox[{"t", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"-", "3"}], " ", "f", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"-", "1"}], "+", "t"}], ")"}]}], "+", "t"}], 
            ")"}]}]}], ")"}]}]}], "}"}], 
     RowBox[{"0", "\[LessEqual]", "t", "\[LessEqual]", "1"}]},
    {
     RowBox[{"{", 
      RowBox[{"Indeterminate", ",", "Indeterminate"}], "}"}], 
     TagBox["True",
      "PiecewiseDefault",
      AutoDelete->False,
      DeletionWarning->True]}
   },
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}},
   GridBoxItemSize->{
    "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
     "RowsIndexed" -> {}},
   GridBoxSpacings->{"Columns" -> {
       Offset[0.27999999999999997`], {
        Offset[0.84]}, 
       Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
       Offset[0.2], {
        Offset[0.4]}, 
       Offset[0.2]}, "RowsIndexed" -> {}}]}]], "Output",
 ImageSize->{597, 53},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->1474786040]
}, Open  ]],

Cell["Plot the result using Manipulate.", "ExampleText",
 CellID->165236841],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Manipulate", "[", 
  RowBox[{
   RowBox[{"ParametricPlot", "[", 
    RowBox[{
     RowBox[{"\[Piecewise]", GridBox[{
        {
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{"-", "1"}], "+", "t"}], ")"}], "3"], "+", 
             RowBox[{"3", " ", "c", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"-", "1"}], "+", "t"}], ")"}], "2"], " ", "t"}], "-", 
             RowBox[{"3", " ", "e", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"-", "1"}], "+", "t"}], ")"}], " ", 
              SuperscriptBox["t", "2"]}], "+", 
             SuperscriptBox["t", "3"]}], ",", 
            RowBox[{"t", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"3", " ", "d", " ", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"-", "1"}], "+", "t"}], ")"}], "2"]}], "+", 
               RowBox[{"t", " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"-", "3"}], " ", "f", " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"-", "1"}], "+", "t"}], ")"}]}], "+", "t"}], 
                 ")"}]}]}], ")"}]}]}], "}"}], "/.", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"c", "\[Rule]", " ", 
             RowBox[{"pt2", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], ",", 
            RowBox[{"d", "\[Rule]", " ", 
             RowBox[{"pt2", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], ",", 
            RowBox[{"e", "\[Rule]", " ", 
             RowBox[{"pt3", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], ",", 
            RowBox[{"f", "\[Rule]", " ", 
             RowBox[{"pt3", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}]}], 
         RowBox[{"0", "\[LessEqual]", "t", "\[LessEqual]", "1"}]},
        {
         RowBox[{"{", 
          RowBox[{"Indeterminate", ",", "Indeterminate"}], "}"}], 
         TagBox["True",
          "PiecewiseDefault",
          AutoDelete->False,
          DeletionWarning->True]}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxItemSize->{
        "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
         "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.84]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.4]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}]}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"Epilog", "\[Rule]", " ", 
      RowBox[{"{", 
       RowBox[{"Red", ",", 
        RowBox[{"PointSize", "[", "Large", "]"}], ",", 
        RowBox[{"Point", "[", 
         RowBox[{"{", 
          RowBox[{"pt2", ",", "pt3"}], "}"}], "]"}], ",", "Blue", ",", 
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", "pt2"}], "}"}], 
           ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"1", ",", "1"}], "}"}], ",", "pt3"}], "}"}]}], "}"}], 
         "]"}]}], "}"}]}], ",", 
     RowBox[{"PlotRange", "\[Rule]", " ", "3.5"}]}], "]"}], ",", 
   RowBox[{"Row", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Control", "[", 
       RowBox[{"{", 
        RowBox[{"pt2", ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "2"}], ",", 
           RowBox[{"-", "2"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"3", ",", "3"}], "}"}], ",", 
         RowBox[{"ImageSize", "\[Rule]", " ", "Small"}]}], "}"}], "]"}], ",", 
      RowBox[{"Control", "[", 
       RowBox[{"{", 
        RowBox[{"pt3", ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "2"}], ",", 
           RowBox[{"-", "2"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"3", ",", "3"}], "}"}], ",", 
         RowBox[{"ImageSize", "\[Rule]", " ", "Small"}]}], "}"}], "]"}]}], 
     "}"}], "]"}]}], "]"}]], "Input",
 CellLabel->"In[6]:=",
 CellID->2108806873],

Cell[BoxData[
 TagBox[
  StyleBox[
   DynamicModuleBox[{$CellContext`pt2$$ = {-2, -2}, $CellContext`pt3$$ = {
    3., -1.85}, Typeset`show$$ = True, Typeset`bookmarkList$$ = {}, 
    Typeset`bookmarkMode$$ = "Menu", Typeset`animator$$, Typeset`animvar$$ = 
    1, Typeset`name$$ = "\"untitled\"", Typeset`specs$$ = {{
      Hold[$CellContext`pt2$$], {-2, -2}, {3, 3}}, {
      Hold[$CellContext`pt3$$], {-2, -2}, {3, 3}}, {
      Hold[
       Row[{
         Manipulate`Place[1], 
         Manipulate`Place[2]}]], Manipulate`Dump`ThisIsNotAControl}}, 
    Typeset`size$$ = {180., {88., 92.}}, Typeset`update$$ = 0, 
    Typeset`initDone$$, Typeset`skipInitDone$$ = 
    True, $CellContext`pt2$839$$ = {0, 0}, $CellContext`pt3$840$$ = {0, 0}}, 
    DynamicBox[Manipulate`ManipulateBoxes[
     2, StandardForm, 
      "Variables" :> {$CellContext`pt2$$ = {-2, -2}, $CellContext`pt3$$ = \
{-2, -2}}, "ControllerVariables" :> {
        Hold[$CellContext`pt2$$, $CellContext`pt2$839$$, {0, 0}], 
        Hold[$CellContext`pt3$$, $CellContext`pt3$840$$, {0, 0}]}, 
      "OtherVariables" :> {
       Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
        Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
        Typeset`specs$$, Typeset`size$$, Typeset`update$$, Typeset`initDone$$,
         Typeset`skipInitDone$$}, "Body" :> ParametricPlot[
        Piecewise[{{
           
           ReplaceAll[{(-1 + $CellContext`t)^3 + ((
                3 $CellContext`c) (-1 + $CellContext`t)^2) $CellContext`t - ((
               3 $CellContext`e) (-1 + $CellContext`t)) $CellContext`t^2 + \
$CellContext`t^3, $CellContext`t ((
                3 $CellContext`d) (-1 + $CellContext`t)^2 + $CellContext`t \
(((-3) $CellContext`f) (-1 + $CellContext`t) + $CellContext`t))}, \
{$CellContext`c -> Part[$CellContext`pt2$$, 1], $CellContext`d -> 
             Part[$CellContext`pt2$$, 2], $CellContext`e -> 
             Part[$CellContext`pt3$$, 1], $CellContext`f -> 
             Part[$CellContext`pt3$$, 2]}], 0 <= $CellContext`t <= 1}}, {
         Indeterminate, Indeterminate}], {$CellContext`t, 0, 1}, 
        Epilog -> {Red, 
          PointSize[Large], 
          Point[{$CellContext`pt2$$, $CellContext`pt3$$}], Blue, 
          
          Line[{{{-1, 0}, $CellContext`pt2$$}, {{1, 
             1}, $CellContext`pt3$$}}]}, PlotRange -> 3.5], 
      "Specifications" :> {{$CellContext`pt2$$, {-2, -2}, {3, 3}, ImageSize -> 
         Small, ControlPlacement -> 1}, {$CellContext`pt3$$, {-2, -2}, {3, 3},
          ImageSize -> Small, ControlPlacement -> 2}, 
        Row[{
          Manipulate`Place[1], 
          Manipulate`Place[2]}]}, "Options" :> {}, "DefaultOptions" :> {}],
     ImageSizeCache->{229., {153., 158.}},
     SingleEvaluation->True],
    Deinitialization:>None,
    DynamicModuleValues:>{},
    SynchronousInitialization->True,
    UnsavedVariables:>{Typeset`initDone$$},
    UntrackedVariables:>{Typeset`size$$}], "Manipulate",
   Deployed->True,
   StripOnInput->False],
  Manipulate`InterpretManipulate[1]]], "Output",
 ImageSize->{233, 313},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->472720606]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 WholeCellGroupOpener->True,
 CellID->38303248],

Cell[TextData[ButtonBox["Calculus Applications",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/guide/CalculusApplications"]], "MoreAbout",
 CellID->158379480],

Cell[TextData[ButtonBox["Classroom Utilities",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/guide/ClassroomUtilities"]], "MoreAbout",
 CellID->24999684]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"CubicBezier - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "SymbolColor", "LinkTrails" -> 
  GridBox[{{
      RowBox[{
        ButtonBox[
        "Classroom Utilities", ButtonData -> 
         "paclet:ClassroomUtilities/guide/ClassroomUtilities", 
         BaseStyle -> {"Link", "DockedLinkTrail"}], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 25, 52.9708624}", 
    "history" -> {"XX", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Symbol", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "CubicBezier[var, {pt_1, pt_2, pt_3, pt_4}] creates a parametric \
representation of a cubic Bezier curve with anchor points pt1 and pt4 and \
control points pt_2 and pt3.", "synonyms" -> {}, "title" -> "CubicBezier", 
    "titlemodifier" -> "", "windowtitle" -> "CubicBezier", "type" -> "Symbol",
     "uri" -> "ClassroomUtilities/ref/CubicBezier"}, "SearchTextTranslated" -> 
  ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[5612, 172, 489, 14, 45, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->48055711]},
 "RasterizedOutput"->{
  Cell[9244, 303, 3584, 67, 177, "Output",
   CellTags->"RasterizedOutput",
   CellID->84291219]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 29241, 843},
 {"RasterizedOutput", 29383, 847}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[604, 21, 2553, 57, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3182, 82, 54, 1, 51, "ObjectName",
 CellID->1224892054],
Cell[3239, 85, 1264, 41, 105, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4540, 131, 609, 18, 39, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->160999823],
Cell[5152, 151, 232, 8, 70, "Notes",
 CellID->1067943069],
Cell[5387, 161, 70, 1, 70, "Notes",
 CellID->1074556442],
Cell[5460, 164, 115, 3, 70, "Notes",
 CellID->1880133071]
}, Closed]],
Cell[CellGroupData[{
Cell[5612, 172, 489, 14, 45, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->48055711],
Cell[CellGroupData[{
Cell[6126, 190, 195, 6, 31, "ExampleSection",
 CellID->475476220],
Cell[6324, 198, 127, 3, 28, "Input",
 CellID->247295146],
Cell[6454, 203, 151, 4, 32, "ExampleText",
 CellID->1232996064],
Cell[CellGroupData[{
Cell[6630, 211, 458, 16, 28, "Input",
 CellID->1895137745],
Cell[7091, 229, 1442, 44, 54, "Output",
 CellID->1355563993]
}, Open  ]],
Cell[8548, 276, 61, 1, 32, "ExampleText",
 CellID->437931636],
Cell[CellGroupData[{
Cell[8634, 281, 607, 20, 28, "Input",
 CellID->890547983],
Cell[9244, 303, 3584, 67, 177, "Output",
 CellTags->"RasterizedOutput",
 CellID->84291219]
}, Open  ]],
Cell[12843, 373, 71, 1, 70, "ExampleText",
 CellID->1341859394],
Cell[CellGroupData[{
Cell[12939, 378, 412, 14, 70, "Input",
 CellID->1905057811],
Cell[13354, 394, 2414, 73, 74, "Output",
 CellID->699518074]
}, Open  ]],
Cell[15783, 470, 96, 1, 70, "ExampleText",
 CellID->1597302709],
Cell[CellGroupData[{
Cell[15904, 475, 434, 15, 70, "Input",
 CellID->173559016],
Cell[16341, 492, 2106, 64, 74, "Output",
 CellID->1474786040]
}, Open  ]],
Cell[18462, 559, 76, 1, 70, "ExampleText",
 CellID->165236841],
Cell[CellGroupData[{
Cell[18563, 564, 4621, 128, 70, "Input",
 CellID->2108806873],
Cell[23187, 694, 3185, 66, 70, "Output",
 CellID->472720606]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[26433, 767, 448, 13, 70, "MoreAboutSection",
 CellID->38303248],
Cell[26884, 782, 175, 4, 70, "MoreAbout",
 CellID->158379480],
Cell[27062, 788, 170, 4, 70, "MoreAbout",
 CellID->24999684]
}, Open  ]],
Cell[27247, 795, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

