(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     19520,        545]
NotebookOptionsPosition[     15556,        423]
NotebookOutlinePosition[     17930,        486]
CellTagsIndexPosition[     17810,        480]
WindowTitle->FittedModelPlot - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Visualizing Statistical Concepts\"\>":>
       Documentation`HelpLookup[
       "paclet:Classroom Utilities/tutorial/VisualizingStatisticalConcepts"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"TutorialsSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Classroom Utilities/ref/FittedModelPlot\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Classroom Utilities/ref/FittedModelPlot"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Classroom \
Utilities/ref/FittedModelPlot.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Classroom Utilities/ref/FittedModelPlot", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell["FittedModelPlot", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"FittedModelPlot", "[", 
       RowBox[{
        StyleBox["data", "TI"], ",", 
        StyleBox["fns", "TI"], ",", 
        StyleBox["var", "TI"]}], "]"}]], "InlineFormula"],
     " \[LineSeparator]plots a model or list of models specified by ",
     Cell[BoxData[
      StyleBox["fns", "TI"]], "InlineFormula"],
     " together with the ",
     Cell[BoxData[
      StyleBox["data", "TI"]], "InlineFormula"],
     ". It will take any option that can be used with ListPlot or Plot."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}},
 CellID->982511436]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 WholeCellGroupOpener->True,
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->363202889],

Cell["\<\
The variable vars should agree with the variable used in specifying your \
functions.\
\>", "Notes",
 CellID->1067943069]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 WholeCellGroupOpener->True,
 CellTags->"PrimaryExamplesSection",
 CellID->205996845],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->76826482],

Cell["This loads the package.", "ExampleText",
 CellID->1557831234],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->348925614],

Cell["Create some data and fit a quadratic model.", "ExampleText",
 CellID->1762438188],

Cell[BoxData[
 RowBox[{
  RowBox[{"data", " ", "=", " ", 
   RowBox[{
    RowBox[{"Range", "[", "8", "]"}], "+", 
    RowBox[{"RandomReal", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", ".3"}], ",", ".5"}], "}"}], ",", "8"}], "]"}]}]}], 
  ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->1056682645],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"mod", "=", " ", 
  RowBox[{"LinearModelFit", "[", 
   RowBox[{"data", ",", 
    RowBox[{"{", "x", "}"}], ",", "x"}], "]"}]}]], "Input",
 CellLabel->"In[3]:=",
 CellID->1916780646],

Cell[BoxData[
 RowBox[{"FittedModel", "[", 
  PanelBox[
   TagBox[
    RowBox[{
     RowBox[{"-", "0.002320513542215133`"}], "+", 
     RowBox[{"1.0065985345182056`", " ", "x"}]}],
    Short],
   FrameMargins->5], "]"}]], "Output",
 ImageSize->{231, 35},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->930128675]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FittedModelPlot", "[", 
  RowBox[{"data", ",", 
   RowBox[{"mod", "[", "t", "]"}], ",", "t"}], "]"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->1139174212],

Cell[BoxData[
 GraphicsBox[{{{}, {}, 
    {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwVj3k4FHgAhuXK0QwzmHHN+KF02kISsb8vyaJnU2Ek0uMoNZQjSTlyTs7U
5kHSliO1qVw7abdLSataytGjVbJbNESRI2dZ/fE973/v+3z6vsHbdktLSUn9
PLfvlBNH1tkoSWhBVPc9I76E2gpWOVnzJHRraoLXRhMJjR2XPLdaJaHyuWTS
z05CJyzd31gIJDSkysu0UCihA3fNJ02KJNS+r61Ep0ZC2xqGfzCy6qWfBfWp
qlt6adHroHymsI9uNC11mYrvo1TWJ7TrcT8d7A2QNPb005ytla//PTxEg3Jz
TUzShqiw1s12cdcwlc8UfxSNDVPt6MCPnlZjdEm1XMYN1zFa3JXZU9EwTrPr
D7d5ScZp6lsXPc/lU/TisoVnmZunqGOmUemi+zNUOnpY2aJ3hnYnCcPeOM3S
nq/D3pYRs/QTYyki2qRg0cfJMxyQgrtx8X4GTxqFjvUvetZI44Du4LpnzjLw
bj3Zci1QBiOVIytSomXhfehQrW++LHafjbVrfiCHJEagTOtbOYhcX83Yjcjj
a1OMnKPGfJz2ufvKwUYBZ0WFXkd9FMDU2NG5IV4RMYI3JXmXFLE2769j+8qV
UOW89urtdiVsDr7loDeuDGVarFCgvwBqnPiq2SUM3HRfvjDejYGHqo22IeFM
pO55mK1fzMS7yuax9j9UcD4z4sv1DyrYzmpbKA5XhcxgkbL+CVWID0r72mux
4MQ+bd9gxkLfnTODA1UsWD1h+YQ1seDqmcnMcmRjE5M3yt7DhtjgISYH2Ahu
rDA5paSGT81+dZHJarDRb93MLlEDr8zeo3aZOvhDMZxQJ3UktifHyTSqg2wS
Cx59VEdtv/GEp78G/raKiuCLNDCyVCsxWoGD1IuP4bqcg2/O+72rCjn4XFL6
IaOeAzzmPX1nwUXUKlXGbS8uIoNYmzo7uOBaSHc0SGni+Ph+nYEwTexsSRC9
PKOJzKuM2QdcLYh70zvrqRaEjhyz8kot1Jw2VL/RoYVYy6iJHAdtvHryU11B
mDZSXAyb7Qe0wYoZ1fXh6mCf2+op5WQdDNWtFCyp1MH6+JQ1Z5bp4m3yiGWj
uy62SFnKGjTqYrx1Q87eaV2sCdhmUOvPw+83DFrks3lgT1x3MVLkI0k97WaC
JR+PQsJfFBbx8U0+LnZeGx+V1jm9mpZ6yE+WvhMRqIdfTKeZ21/roeSefGv3
fAKjlG5bgRXBc5FLTZYrwUvu3h1X1hGQIVfGezeCtEv9od+sCUI8BH7W7gSf
Hg1fKKUEqis8VPo8CGrk5s1+sSPY2uwdYLuLwCGRdyvXmaBZR8gd20cgjBWY
/eNP0FIRd3jHMQJdZoeT8R4CA+2Exoo4gqZzO33jAwjCEhMN5ycQmN7xP7VU
SMAWiJ5VJxFMT4cNHgkm2DadvpiRRpBxJKtM+yhB68bc9nvZBD8qqtUdiCIw
LM8z5uQQDOXldDyIJjiomZ8YlEvgdvOcYuBcV72/YKVWPgF/vCzg1pzX9WTR
8bDz33+vPKYiIiiZLO5suECQwKnO8TtOMOp70UyviOC9+Z/1ynPdbPPLXU9L
CPLqbTp3pRN0//qbuUEpgZPb/dHqDILVCmXpkZcIZrrtFiicIEgKvfpf02WC
8vAGQ68sghcd1ywWXSH4HwmGF4A=
      "]]}}, {{}, 
    {Hue[0.67, 0.6, 0.6], 
     PointBox[{{1., 0.7949705879858887}, {2., 2.399637223435103}, {3., 
      2.867160208391553}, {4., 3.7948367381238897`}, {5., 
      5.168830700379835}, {6., 6.01725372597639}, {7., 7.47053751628976}, {8.,
       7.705756433735248}}]}, {}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesOrigin->{0, 0},
  PlotRange->{{0., 8.279999845714284}, {0., 8.332315196964752}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 ImageSize->{184, 122},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->1284268079]
}, Open  ]],

Cell["Add some confidence bands.", "ExampleText",
 CellID->335467756],

Cell[BoxData[
 RowBox[{
  RowBox[{"pltbnds", "=", 
   RowBox[{"MakePlotBands", "[", 
    RowBox[{
     RowBox[{"mod", "[", "\"\<MeanPredictionBands\>\"", "]"}], " ", ",", ".9",
      " ", ",", "x"}], " ", "]"}]}], ";"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->1062001717],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FittedModelPlot", "[", 
  RowBox[{"data", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"mod", "[", "t", "]"}], ",", 
     RowBox[{"pltbnds", "[", "t", "]"}]}], "}"}], ",", "t", ",", 
   RowBox[{"Filling", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"2", "\[Rule]", " ", 
      RowBox[{"{", "3", "}"}]}], "}"}]}]}], "]"}]], "Input",
 CellLabel->"In[6]:=",
 CellID->1326267402],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztXSl37DwSzTnzS+ZnhDYMbGj4GTYYYDZNDQ0NRQXFRlBUUFBQ1NDQsKdK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    "], {{0, 122}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 122},
  PlotRange->{{0, 180}, {0, 122}}]], "Output",
 ImageSize->{184, 122},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[6]=",
 CellID->82605737]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 WholeCellGroupOpener->True,
 CellID->250839057],

Cell[TextData[ButtonBox["Visualizing Statistical Concepts",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Classroom Utilities/tutorial/VisualizingStatisticalConcepts"]], \
"Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"FittedModelPlot - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "SymbolColor", "LinkTrails" -> 
  GridBox[{{
      RowBox[{
        StyleBox[
         ButtonBox[
         "Mathematica", ButtonData -> "paclet:guide/Mathematica", 
          BaseStyle -> {"Link", "DockedLinkTrail"}], FontSlant -> "Italic"], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 26, 35.2156624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Symbol", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "FittedModelPlot[data, fns, var] plots a model or list of models \
specified by fns together with the data. It will take any option that can be \
used with ListPlot or Plot.", "synonyms" -> {}, "title" -> "FittedModelPlot", 
    "titlemodifier" -> "", "windowtitle" -> "FittedModelPlot", "type" -> 
    "Symbol", "uri" -> "Classroom Utilities/ref/FittedModelPlot"}, 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[4754, 140, 490, 14, 45, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->205996845]},
 "RasterizedOutput"->{
  Cell[10147, 307, 4653, 84, 143, "Output",
   CellTags->"RasterizedOutput",
   CellID->82605737]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 17544, 469},
 {"RasterizedOutput", 17687, 473}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[608, 21, 2500, 57, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3133, 82, 58, 1, 51, "ObjectName",
 CellID->1224892054],
Cell[3194, 85, 743, 21, 99, "Usage",
 CellID->982511436]
}, Open  ]],
Cell[CellGroupData[{
Cell[3974, 111, 609, 18, 39, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->363202889],
Cell[4586, 131, 131, 4, 70, "Notes",
 CellID->1067943069]
}, Closed]],
Cell[CellGroupData[{
Cell[4754, 140, 490, 14, 45, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->205996845],
Cell[CellGroupData[{
Cell[5269, 158, 194, 6, 31, "ExampleSection",
 CellID->76826482],
Cell[5466, 166, 67, 1, 32, "ExampleText",
 CellID->1557831234],
Cell[5536, 169, 127, 3, 28, "Input",
 CellID->348925614],
Cell[5666, 174, 87, 1, 32, "ExampleText",
 CellID->1762438188],
Cell[5756, 177, 331, 12, 28, "Input",
 CellID->1056682645],
Cell[CellGroupData[{
Cell[6112, 193, 203, 6, 28, "Input",
 CellID->1916780646],
Cell[6318, 201, 362, 13, 56, "Output",
 CellID->930128675]
}, Open  ]],
Cell[CellGroupData[{
Cell[6717, 219, 179, 5, 28, "Input",
 CellID->1139174212],
Cell[6899, 226, 2452, 49, 143, "Output",
 CellID->1284268079]
}, Open  ]],
Cell[9366, 278, 69, 1, 70, "ExampleText",
 CellID->335467756],
Cell[9438, 281, 274, 8, 70, "Input",
 CellID->1062001717],
Cell[CellGroupData[{
Cell[9737, 293, 407, 12, 28, "Input",
 CellID->1326267402],
Cell[10147, 307, 4653, 84, 143, "Output",
 CellTags->"RasterizedOutput",
 CellID->82605737]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[14861, 398, 448, 13, 70, "TutorialsSection",
 CellID->250839057],
Cell[15312, 413, 202, 5, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[15529, 421, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

