(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     23459,        775]
NotebookOptionsPosition[     18353,        611]
NotebookOutlinePosition[     20546,        669]
CellTagsIndexPosition[     20461,        664]
WindowTitle->Five Number Summary Table - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Visualizing Statistical Concepts\"\>":>
       Documentation`HelpLookup[
       "paclet:Classroom Utilities/tutorial/VisualizingStatisticalConcepts"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"TutorialsSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Classroom Utilities/ref/FiveNumberSummaryTable\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Classroom Utilities/ref/FiveNumberSummaryTable"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Classroom \
Utilities/ref/FiveNumberSummaryTable.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Classroom Utilities/ref/FiveNumberSummaryTable", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell["Five Number Summary Table", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"FiveNumberSummaryTable", "[", 
       StyleBox["data", "TI"], "]"}]], "InlineFormula"],
     " \[LineSeparator]create a formatted table of five number summaries \
together with row headings."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}},
 CellID->982511436]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 WholeCellGroupOpener->True,
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->955011129],

Cell[TextData[{
 "The ",
 Cell[BoxData[
  StyleBox["data", "TI"]], "InlineFormula"],
 " can be either a numerical vector or a list of numerical vectors."
}], "Notes",
 CellID->1067943069],

Cell[TextData[{
 "Column headings can be added using the option ",
 Cell[BoxData[
  ButtonBox["ColumnHeadings",
   BaseStyle->"Link",
   ButtonData->"paclet:Classroom Utilities/ref/ColumnHeadings"]], 
  "InlineFormula"],
 "."
}], "Notes",
 CellID->766416007],

Cell[TextData[{
 "Any options for ",
 ButtonBox["Grid",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Grid"],
 " can be used to modify the appearance of the table."
}], "Notes",
 CellID->546914943]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 WholeCellGroupOpener->True,
 CellTags->"PrimaryExamplesSection",
 CellID->57027548],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->619711741],

Cell["This loads the package.", "ExampleText",
 CellID->769437841],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1942755100],

Cell["Get some data.", "ExampleText",
 CellID->1627482824],

Cell[BoxData[
 RowBox[{
  RowBox[{"ofData", "=", " ", 
   RowBox[{"ExampleData", "[", 
    RowBox[{"{", 
     RowBox[{"\"\<Statistics\>\"", ",", "\"\<OldFaithful\>\""}], "}"}], 
    "]"}]}], ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->604611129],

Cell["Create a five number summary table.", "ExampleText",
 CellID->1954111146],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FiveNumberSummaryTable", "[", 
  RowBox[{"Transpose", "[", "ofData", "]"}], "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->1278617251],

Cell[BoxData[
 TagBox[GridBox[{
    {"\<\"max\"\>", "5.1`", "96"},
    {"\<\"75%\"\>", "4.45`", "82"},
    {"\<\"median\"\>", "4.`", "76"},
    {"\<\"25%\"\>", "2.15`", "58"},
    {"\<\"min\"\>", "1.6`", "43"}
   },
   AllowScriptLevelChange->True,
   AllowedDimensions->Automatic,
   AutoDelete->False,
   BaseStyle->{},
   BaselinePosition->Automatic,
   DefaultBaseStyle->"Grid",
   DeleteWithContents->True,
   Editable->Automatic,
   FrameStyle->Automatic,
   GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
   GridBoxBackground->{"Columns" -> {{
        RGBColor[1, 1, 0.85]}}},
   GridBoxDividers->{"Columns" -> {
       Thickness[Large], {True}, 
       Thickness[Large]}, "Rows" -> {
       Thickness[Large], {True}, 
       Thickness[Large]}},
   GridBoxFrame->{"Columns" -> None, "Rows" -> None},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
   GridBoxItemStyle->{"Columns" -> {{None}}, "Rows" -> {{None}}},
   GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
   GridDefaultElement->"\[Placeholder]",
   Selectable->Automatic],
  "Grid"]], "Output",
 ImageSize->{120, 86},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->1826967568]
}, Open  ]],

Cell["Add column headings.", "ExampleText",
 CellID->1122730570],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FiveNumberSummaryTable", "[", 
  RowBox[{
   RowBox[{"Transpose", "[", "ofData", "]"}], ",", 
   RowBox[{"ColumnHeadings", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"\"\<duration\>\"", ",", "\"\<waiting\>\""}], "}"}]}]}], 
  "]"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->1941367448],

Cell[BoxData[
 TagBox[GridBox[{
    {"\<\"\"\>", "\<\"duration\"\>", "\<\"waiting\"\>"},
    {"\<\"max\"\>", "5.1`", "96"},
    {"\<\"75%\"\>", "4.45`", "82"},
    {"\<\"median\"\>", "4.`", "76"},
    {"\<\"25%\"\>", "2.15`", "58"},
    {"\<\"min\"\>", "1.6`", "43"}
   },
   AllowScriptLevelChange->True,
   AllowedDimensions->Automatic,
   AutoDelete->False,
   BaseStyle->{},
   BaselinePosition->Automatic,
   DefaultBaseStyle->"Grid",
   DeleteWithContents->True,
   Editable->Automatic,
   FrameStyle->Automatic,
   GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
   GridBoxBackground->{"Columns" -> {{
        RGBColor[1, 1, 0.85]}}},
   GridBoxDividers->{"Columns" -> {
       Thickness[Large], {True}, 
       Thickness[Large]}, "Rows" -> {
       Thickness[Large], {True}, 
       Thickness[Large]}},
   GridBoxFrame->{"Columns" -> None, "Rows" -> None},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
   GridBoxItemStyle->{"Columns" -> {{None}}, "Rows" -> {{None}}},
   GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
   GridDefaultElement->"\[Placeholder]",
   Selectable->Automatic],
  "Grid"]], "Output",
 ImageSize->{183, 102},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->1843649665]
}, Open  ]],

Cell["A single data vector can be used.", "ExampleText",
 CellID->178377092],

Cell[BoxData[
 RowBox[{
  RowBox[{"ofData1", "=", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"Transpose", "[", "ofData", "]"}], ")"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->96614032],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FiveNumberSummaryTable", "[", "ofData1", "]"}]], "Input",
 CellLabel->"In[6]:=",
 CellID->1670573164],

Cell[BoxData[
 TagBox[GridBox[{
    {"\<\"max\"\>", "5.1`"},
    {"\<\"75%\"\>", "4.45`"},
    {"\<\"median\"\>", "4.`"},
    {"\<\"25%\"\>", "2.15`"},
    {"\<\"min\"\>", "1.6`"}
   },
   AllowScriptLevelChange->True,
   AllowedDimensions->Automatic,
   AutoDelete->False,
   BaseStyle->{},
   BaselinePosition->Automatic,
   DefaultBaseStyle->"Grid",
   DeleteWithContents->True,
   Editable->Automatic,
   FrameStyle->Automatic,
   GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
   GridBoxBackground->{"Columns" -> {{
        RGBColor[1, 1, 0.85]}}},
   GridBoxDividers->{"Columns" -> {
       Thickness[Large], {True}, 
       Thickness[Large]}, "Rows" -> {
       Thickness[Large], {True}, 
       Thickness[Large]}},
   GridBoxFrame->{"Columns" -> None, "Rows" -> None},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
   GridBoxItemStyle->{"Columns" -> {{None}}, "Rows" -> {{None}}},
   GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
   GridDefaultElement->"\[Placeholder]",
   Selectable->Automatic],
  "Grid"]], "Output",
 ImageSize->{97, 86},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->1029045617]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FiveNumberSummaryTable", "[", 
  RowBox[{"ofData1", ",", 
   RowBox[{"ColumnHeadings", "\[Rule]", " ", 
    RowBox[{"{", "\"\<duration\>\"", "}"}]}]}], "]"}]], "Input",
 CellLabel->"In[7]:=",
 CellID->1483875628],

Cell[BoxData[
 TagBox[GridBox[{
    {"\<\"\"\>", "\<\"duration\"\>"},
    {"\<\"max\"\>", "5.1`"},
    {"\<\"75%\"\>", "4.45`"},
    {"\<\"median\"\>", "4.`"},
    {"\<\"25%\"\>", "2.15`"},
    {"\<\"min\"\>", "1.6`"}
   },
   AllowScriptLevelChange->True,
   AllowedDimensions->Automatic,
   AutoDelete->False,
   BaseStyle->{},
   BaselinePosition->Automatic,
   DefaultBaseStyle->"Grid",
   DeleteWithContents->True,
   Editable->Automatic,
   FrameStyle->Automatic,
   GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
   GridBoxBackground->{"Columns" -> {{
        RGBColor[1, 1, 0.85]}}},
   GridBoxDividers->{"Columns" -> {
       Thickness[Large], {True}, 
       Thickness[Large]}, "Rows" -> {
       Thickness[Large], {True}, 
       Thickness[Large]}},
   GridBoxFrame->{"Columns" -> None, "Rows" -> None},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
   GridBoxItemStyle->{"Columns" -> {{None}}, "Rows" -> {{None}}},
   GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
   GridDefaultElement->"\[Placeholder]",
   Selectable->Automatic],
  "Grid"]], "Output",
 ImageSize->{125, 102},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->2130942157]
}, Open  ]],

Cell[BoxData[""], "Input",
 CellLabel->"In[8]:=",
 CellID->1543596712]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Options",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->2061341341],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "ColumnHeadings",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection", "ExampleSubsection",
 WholeCellGroupOpener->True,
 CellID->1757724783],

Cell["Load the package.", "ExampleText",
 CellID->1929021335],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->500958586],

Cell["Get some data.", "ExampleText",
 CellID->1160051680],

Cell[BoxData[
 RowBox[{
  RowBox[{"apgData", "=", 
   RowBox[{"ExampleData", "[", 
    RowBox[{"{", 
     RowBox[{"\"\<Statistics\>\"", ",", "\"\<AirplaneGlass\>\""}], "}"}], 
    "]"}]}], ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->2132038294],

Cell["\<\
Column headings must be given in a list of the same length as the number of \
data vectors.\
\>", "ExampleText",
 CellID->255473416],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FiveNumberSummaryTable", "[", 
  RowBox[{"apgData", ",", 
   RowBox[{"ColumnHeadings", "\[Rule]", " ", 
    RowBox[{"{", "\"\<breakpoint\>\"", "}"}]}]}], "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->921892359],

Cell[BoxData[
 TagBox[GridBox[{
    {"\<\"\"\>", "\<\"breakpoint\"\>"},
    {"\<\"max\"\>", "45.381`"},
    {"\<\"75%\"\>", "35.91`"},
    {"\<\"median\"\>", "29.9`"},
    {"\<\"25%\"\>", "25.5`"},
    {"\<\"min\"\>", "18.83`"}
   },
   AllowScriptLevelChange->True,
   AllowedDimensions->Automatic,
   AutoDelete->False,
   BaseStyle->{},
   BaselinePosition->Automatic,
   DefaultBaseStyle->"Grid",
   DeleteWithContents->True,
   Editable->Automatic,
   FrameStyle->Automatic,
   GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
   GridBoxBackground->{"Columns" -> {{
        RGBColor[1, 1, 0.85]}}},
   GridBoxDividers->{"Columns" -> {
       Thickness[Large], {True}, 
       Thickness[Large]}, "Rows" -> {
       Thickness[Large], {True}, 
       Thickness[Large]}},
   GridBoxFrame->{"Columns" -> None, "Rows" -> None},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
   GridBoxItemStyle->{"Columns" -> {{None}}, "Rows" -> {{None}}},
   GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
   GridDefaultElement->"\[Placeholder]",
   Selectable->Automatic],
  "Grid"]], "Output",
 ImageSize->{139, 102},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->150364608]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FiveNumberSummaryTable", "[", 
  RowBox[{"apgData", ",", 
   RowBox[{"ColumnHeadings", "\[Rule]", " ", "\"\<breakpoint\>\""}]}], 
  "]"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->1737197784],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FiveNumberSummaryTable", "::", "colnm"}], "MessageName"], ":", 
  " ", "\<\"The length of the column headings list must equal the number of \
data lists. Column headings will be omitted.\"\>"}]], "Message", "MSG",
 CellLabel->"During evaluation of In[4]:=",
 CellID->2042974723],

Cell[BoxData[
 TagBox[GridBox[{
    {"\<\"max\"\>", "45.381`"},
    {"\<\"75%\"\>", "35.91`"},
    {"\<\"median\"\>", "29.9`"},
    {"\<\"25%\"\>", "25.5`"},
    {"\<\"min\"\>", "18.83`"}
   },
   AllowScriptLevelChange->True,
   AllowedDimensions->Automatic,
   AutoDelete->False,
   BaseStyle->{},
   BaselinePosition->Automatic,
   DefaultBaseStyle->"Grid",
   DeleteWithContents->True,
   Editable->Automatic,
   FrameStyle->Automatic,
   GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
   GridBoxBackground->{"Columns" -> {{
        RGBColor[1, 1, 0.85]}}},
   GridBoxDividers->{"Columns" -> {
       Thickness[Large], {True}, 
       Thickness[Large]}, "Rows" -> {
       Thickness[Large], {True}, 
       Thickness[Large]}},
   GridBoxFrame->{"Columns" -> None, "Rows" -> None},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
   GridBoxItemStyle->{"Columns" -> {{None}}, "Rows" -> {{None}}},
   GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
   GridDefaultElement->"\[Placeholder]",
   Selectable->Automatic],
  "Grid"]], "Output",
 ImageSize->{111, 86},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->160137578]
}, Open  ]]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 WholeCellGroupOpener->True,
 CellID->250839057],

Cell[TextData[StyleBox[ButtonBox["Visualizing Statistical Concepts",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Classroom Utilities/tutorial/VisualizingStatisticalConcepts"],
 FontWeight->"Bold"]], "Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Five Number Summary Table - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "SymbolColor", "LinkTrails" -> 
  GridBox[{{
      RowBox[{
        StyleBox[
         ButtonBox[
         "Mathematica", ButtonData -> "paclet:guide/Mathematica", 
          BaseStyle -> {"Link", "DockedLinkTrail"}], FontSlant -> "Italic"], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 26, 43.2496624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Symbol", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "FiveNumberSummaryTable[data] create a formatted table of five number \
summaries together with row headings.", "synonyms" -> {}, "title" -> 
    "Five Number Summary Table", "titlemodifier" -> "", "windowtitle" -> 
    "Five Number Summary Table", "type" -> "Symbol", "uri" -> 
    "Classroom Utilities/ref/FiveNumberSummaryTable"}, "SearchTextTranslated" -> 
  ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[5043, 155, 489, 14, 45, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->57027548]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 20319, 657}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[618, 21, 2540, 58, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3183, 83, 68, 1, 51, "ObjectName",
 CellID->1224892054],
Cell[3254, 86, 454, 13, 83, "Usage",
 CellID->982511436]
}, Open  ]],
Cell[CellGroupData[{
Cell[3745, 104, 609, 18, 39, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->955011129],
Cell[4357, 124, 187, 6, 70, "Notes",
 CellID->1067943069],
Cell[4547, 132, 258, 9, 70, "Notes",
 CellID->766416007],
Cell[4808, 143, 198, 7, 70, "Notes",
 CellID->546914943]
}, Closed]],
Cell[CellGroupData[{
Cell[5043, 155, 489, 14, 45, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->57027548],
Cell[CellGroupData[{
Cell[5557, 173, 195, 6, 31, "ExampleSection",
 CellID->619711741],
Cell[5755, 181, 66, 1, 32, "ExampleText",
 CellID->769437841],
Cell[5824, 184, 128, 3, 28, "Input",
 CellID->1942755100],
Cell[5955, 189, 58, 1, 32, "ExampleText",
 CellID->1627482824],
Cell[6016, 192, 250, 8, 28, "Input",
 CellID->604611129],
Cell[6269, 202, 79, 1, 32, "ExampleText",
 CellID->1954111146],
Cell[CellGroupData[{
Cell[6373, 207, 160, 4, 28, "Input",
 CellID->1278617251],
Cell[6536, 213, 1266, 36, 107, "Output",
 CellID->1826967568]
}, Open  ]],
Cell[7817, 252, 64, 1, 70, "ExampleText",
 CellID->1122730570],
Cell[CellGroupData[{
Cell[7906, 257, 315, 9, 70, "Input",
 CellID->1941367448],
Cell[8224, 268, 1324, 37, 123, "Output",
 CellID->1843649665]
}, Open  ]],
Cell[9563, 308, 76, 1, 70, "ExampleText",
 CellID->178377092],
Cell[9642, 311, 243, 8, 70, "Input",
 CellID->96614032],
Cell[CellGroupData[{
Cell[9910, 323, 125, 3, 70, "Input",
 CellID->1670573164],
Cell[10038, 328, 1235, 36, 107, "Output",
 CellID->1029045617]
}, Open  ]],
Cell[CellGroupData[{
Cell[11310, 369, 236, 6, 70, "Input",
 CellID->1483875628],
Cell[11549, 377, 1275, 37, 123, "Output",
 CellID->2130942157]
}, Open  ]],
Cell[12839, 417, 70, 2, 70, "Input",
 CellID->1543596712]
}, Open  ]],
Cell[CellGroupData[{
Cell[12946, 424, 270, 8, 70, "ExampleSection",
 CellID->2061341341],
Cell[CellGroupData[{
Cell[13241, 436, 286, 8, 70, "ExampleSubsection",
 CellID->1757724783],
Cell[13530, 446, 61, 1, 70, "ExampleText",
 CellID->1929021335],
Cell[13594, 449, 127, 3, 70, "Input",
 CellID->500958586],
Cell[13724, 454, 58, 1, 70, "ExampleText",
 CellID->1160051680],
Cell[13785, 457, 249, 8, 70, "Input",
 CellID->2132038294],
Cell[14037, 467, 142, 4, 70, "ExampleText",
 CellID->255473416],
Cell[CellGroupData[{
Cell[14204, 475, 237, 6, 70, "Input",
 CellID->921892359],
Cell[14444, 483, 1284, 37, 123, "Output",
 CellID->150364608]
}, Open  ]],
Cell[CellGroupData[{
Cell[15765, 525, 216, 6, 70, "Input",
 CellID->1737197784],
Cell[15984, 533, 326, 7, 70, "Message",
 CellID->2042974723],
Cell[16313, 542, 1243, 36, 107, "Output",
 CellID->160137578]
}, Open  ]]
}, Closed]]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[17629, 586, 448, 13, 70, "TutorialsSection",
 CellID->250839057],
Cell[18080, 601, 231, 5, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[18326, 609, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

