(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    198738,       4812]
NotebookOptionsPosition[    189630,       4512]
NotebookOutlinePosition[    191842,       4570]
CellTagsIndexPosition[    191755,       4565]
WindowTitle->FunctionArrowDiagram - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Building an Understanding of Functions\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/tutorial/\
BuildingAnUnderstandingOfFunctions"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"TutorialsSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Classroom Utilities\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/guide/ClassroomUtilities"], \
"\<\"Understanding Functions\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/guide/UnderstandingFunctions"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"MoreAboutSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"ClassroomUtilities/ref/FunctionArrowDiagram\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["ClassroomUtilities/ref/FunctionArrowDiagram"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/ClassroomUtilities/\
ref/FunctionArrowDiagram.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "ClassroomUtilities/ref/FunctionArrowDiagram", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["FunctionArrowDiagram", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"FunctionArrowDiagram", "[", 
       RowBox[{
        StyleBox["dompts", "TI"], ",", 
        StyleBox["rngpts", "TI"], ",", 
        StyleBox["type", "TI"], ",", 
        StyleBox["opts", "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]creates an arrow diagram representing various types of \
functions such as one to one or constant."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 WholeCellGroupOpener->True,
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->295067813],

Cell["The type should be given as a string.", "ExampleText",
 CellID->1805848246],

Cell["\<\
Possible types include \"OneToOne\", \"Onto\", \"OneToOneOnto\", \
\"Constant\", \"Inverse\" and \"Relation\".\
\>", "ExampleText",
 CellID->281283540],

Cell["\<\
Type \"Inverse\" require both sets have the same number of points.\
\>", "ExampleText",
 CellID->1868009087],

Cell["\<\
Domain and range values can be positive integers or sets of specific points.\
\>", "ExampleText",
 CellID->1619558343],

Cell["\<\
If point sets are given they should be constructed from points around the \
origin.\
\>", "ExampleText",
 CellID->2034052995],

Cell["\<\
The points will be rescaled to fit in an ellipse centered on the origin two \
units wide and three units high.\
\>", "ExampleText",
 CellID->536510372],

Cell["\<\
Except for the type \"Relation\" the domain points should not include \
duplicates.\
\>", "ExampleText",
 CellID->1993472098],

Cell["\<\
Points in the domain and range set will paired in the order in which they are \
given.\
\>", "ExampleText",
 CellID->1393174239],

Cell["\<\
FunctionArrowDiagram takes all options for Graphics with the following \
additions and modifications: \
\>", "Notes"],

Cell[BoxData[GridBox[{
   {Cell["      ", "TableRowIcon"], 
    ButtonBox["ArrowStyle",
     BaseStyle->"Link",
     ButtonData->"paclet:UnderstandingFunctions/ref/ArrowStyle"], 
    ButtonBox["Automatic",
     BaseStyle->"Link",
     ButtonData->"paclet:ref/Automatic"], Cell[TextData[{
     Cell[BoxData[
      ButtonBox["ArrowStyle",
       BaseStyle->"Link",
       ButtonData->"paclet:UnderstandingFunctions/ref/ArrowStyle"]], 
      "InlineFormula"],
     " is an option for ",
     Cell[BoxData["FunctionArrowDiagram"], "InlineFormula"],
     " that determines how the arrows are rendered."
    }], "TableText"]},
   {Cell["      ", "TableRowIcon"], 
    ButtonBox["PointLabels",
     BaseStyle->"Link",
     ButtonData->"paclet:UnderstandingFunctions/ref/PointLabels"], 
    ButtonBox["None",
     BaseStyle->"Link",
     ButtonData->"paclet:ref/None"], Cell[TextData[{
     Cell[BoxData[
      ButtonBox["PointLabels",
       BaseStyle->"Link",
       ButtonData->"paclet:UnderstandingFunctions/ref/PointLabels"]], 
      "InlineFormula"],
     " is an option for FunctionDiagramPlot allowing labeling of the points \
in the\ndomain and range sets."
    }], "TableText"]},
   {Cell["      ", "TableRowIcon"], 
    ButtonBox["PlotStyle",
     BaseStyle->"Link",
     ButtonData->"paclet:ref/PlotStyle"], 
    ButtonBox["Automatic",
     BaseStyle->"Link",
     ButtonData->"paclet:ref/Automatic"], Cell["\<\
PlotStyle is an option for plotting and related functions that specifies \
styles in which objects are to be drawn. \
\>", "TableText"]},
   {Cell["      ", "TableRowIcon"], 
    ButtonBox["PointLabelOffset",
     BaseStyle->"Link",
     ButtonData->"paclet:UnderstandingFunctions/ref/PointLabelOffset"], 
    ButtonBox["Automatic",
     BaseStyle->"Link",
     ButtonData->"paclet:ref/Automatic"], Cell[TextData[{
     Cell[BoxData[
      ButtonBox["PointLabelOffset",
       BaseStyle->"Link",
       ButtonData->"paclet:UnderstandingFunctions/ref/PointLabelOffset"]], 
      "InlineFormula"],
     " is an option for ",
     Cell[BoxData["FunctionArrowDiagram"], "InlineFormula"],
     " for controlling the amount of\noffset from the point used by the point \
labels."
    }], "TableText"]},
   {Cell["      ", "TableRowIcon"], 
    ButtonBox["PointLabelStyle",
     BaseStyle->"Link",
     ButtonData->"paclet:UnderstandingFunctions/ref/PointLabelStyle"], 
    ButtonBox["Automatic",
     BaseStyle->"Link",
     ButtonData->"paclet:ref/Automatic"], Cell[TextData[{
     Cell[BoxData[
      ButtonBox["PointLabelStyle",
       BaseStyle->"Link",
       ButtonData->"paclet:UnderstandingFunctions/ref/PointLabelStyle"]], 
      "InlineFormula"],
     " is an option for ",
     Cell[BoxData["FunctionArrowDiagram"], "InlineFormula"],
     " allowing the point labels to be\nstyled."
    }], "TableText"]},
   {Cell["      ", "TableRowIcon"], 
    ButtonBox["SetsStyle",
     BaseStyle->"Link",
     ButtonData->"paclet:UnderstandingFunctions/ref/SetsStyle"], 
    ButtonBox["Automatic",
     BaseStyle->"Link",
     ButtonData->"paclet:ref/Automatic"], Cell[TextData[{
     Cell[BoxData[
      ButtonBox["SetsStyle",
       BaseStyle->"Link",
       ButtonData->"paclet:UnderstandingFunctions/ref/SetsStyle"]], 
      "InlineFormula"],
     " is an option for ",
     Cell[BoxData["FunctionArrowDiagram"], "InlineFormula"],
     " that determines how the domain and range\nsets are rendered."
    }], "TableText"]}
  }]], "3ColumnTableMod",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, 
   "Rows" -> {None, None, None, None, None, {None}}, "RowsIndexed" -> {}},
 GridBoxDividers->{"Rows" -> {{True, True, True, True, True, True, True}}}}]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 WholeCellGroupOpener->True,
 CellTags->"PrimaryExamplesSection",
 CellID->486231370],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->308284301],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->950986302],

Cell["A basic one-to=one function.", "ExampleText",
 CellID->1423066363],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionArrowDiagram", "[", 
  RowBox[{"7", ",", "11", ",", "\"\<OneToOne\>\""}], "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->1653476740],

Cell[BoxData[
 GraphicsBox[
  {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
   0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
   {RGBColor[0.0692454, 0.645548, 0.968322], 
    DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
   {GrayLevel[0], PointSize[Medium], Opacity[1], 
    PointBox[{{0.5, -0.17320508075688773`}, {0.6000000000000001, 
      0.6928203230275509}, {-0.1, 0.5196152422706632}, {0.2, 
      1.3856406460551018`}, {0, 
      0}, {-0.6000000000000001, -1.0392304845413265`}, {-0.5000000000000001, 
      0.17320508075688773`}, {2.9, 1.2124355652982142`}, {3.4, 
      0.6928203230275509}, {3.1, -0.17320508075688773`}, {
      3.6, -0.6928203230275509}, {2.6, -1.0392304845413265`}, {2.7, 
      0.17320508075688773`}, {3.7, -0.17320508075688773`}, {
      2.1, -0.5196152422706632}, {2.2, 0.34641016151377546`}, {2.3, 
      0.8660254037844386}, {3.9, 0.5196152422706632}}], 
    {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQbWIMAo/3zzrOqLphwYf9TY/uZaywuLefRa+sL5Tl
/f46bnXWNzeu7i/hfru94/ub/RuSLFNcp5/af8/9uLWAzOv9D45Oc48I3bTf
yvNM0ouil/sTijbdzBTdaM8n7X/405/n+2+c606tqD9ln6BZc/Lrrmf7p6ma
zeMOvWZfcFPAb87ap/td/rO8Kmu4b89wd4t16dUn+x1YdN0mxD2xPxKp6vzZ
6Ml+GYv+VwLML+1TFKPm5Rx7vJ9r753Dbqvf2v9yPvpIasLj/dULBefFaH2E
8z8l7+gqr/sMV/+g9uWd7Re+ws0zOFDn+kj+B9y+WWVFEnuzfsHdo9Lm91lg
wx+4e2/dSEla+e4f3D/622wOrZrE4ADzr/Y057CSlYwOsPAoeLikQXY3kwMs
vALuSPOKHmd2gIXn2TMgwOIAC28A0tbOcQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAA4D8iCq2GlSvGv39V6hl4
SeI/nvyiB892w79INA/jNprkP5Mj6kFXLcG/pf3Y2AHx5j/ieCm3yaC+vxJV
PEObTOk/9jrfy+nBu78L9UzdwqvrP1hgdq6Uv7m/le4sfzYN7j9/N3JG25q4
v1QUfmTZN/A/DAaL5VhUuL9otjbn+WjxP4CARPUy7Li/JftS4VqZ8j8e+BPj
GGK6v9EoBtJayPM/xscgS0S1vL+9sdXzWPX0P8ZPiGF55L+/OFPnkbUf9j++
0/bMA/fBv8KMvlzSRvc/ZA1+RuVnxL8FeTy+Emr4P7hWsY2VQ8e/mHa1LNyI
+T98H9dmkIjKv++K8HyWovo/dc4juBk1zr9w/eQyrLb7P766rjqfI9G/X1ML
0YrE/D/ik1PUal7Tv+eWFyajy/0/k7nzy8DJ1b8pq/OYacv+P2AnF4NYZNi/
A13RclbD/z/7+jdI0Czbv7BxlhNzWQBAltNtEq4h3r/NzMzMzMwAQJrHASWw
oOC/ru3T4A==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQjQ5mV7gHbAw5YL/xx9usDvX19gv2frnxrP6CvVzM
kTNXl+21P/k9t8nuzw37T8WVdfckjtpPyP5Tn3/2gX3TfFnOl9NO2c/yZ9o3
S/up/Ytd/6sWeJ23d05etM5/2kv7n3PnCx6ecdF+SZhanBn3O3uB42r/D6tc
tq9czrfjh/NH+8WXJj9v8Lhi3yDosYG/9bN9mIHG/zjmq/a/Pm15vPb4V/uc
7Vzyv0Ou2ptO3uK3gOuH/aYjj39q2l61F97z7cBmn1/2eySvG/3YfcV+rklV
w9aeP/bcvufDVl28bJ9U6vOm+OQ/+9WRcjv21F2yfy0b1X+/gcHhyqw2nceH
L9j3VVgcN5nF6JCVMS9a0OWs/dFVMeEqG5gcRPh1d801OWH/0/XSMalDzA5y
+z0vScUcsr+/O2RH23kWh4hM5wTX6u32bzcWmN+8zuqgs3rTkodFC+1reeJY
2m6xOQjcZGU/Jzpvf2pR1j3m6+wOJusOPj9hsXP/2TMgwOGgxLW2bar2sf0A
0qm9bg==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAQAAAAAA4L8iCq2GlSvGP9g5LW7l
x9S/wUmhc18T0D8sO7yVPbHCv3COHDPojNQ/MPNuoRU+oj/WnwQwBYDYP1B6
QpYnJsw/Gg1v653q2z+EXAVuEgPaP8uu9FXiyt4/Me5mjbQI4z/cQPbipY/g
P4fB2xrZG+k//lda405z4T8bDhm3vTfvP8zlTWDzD+I/rBGiSpKs8j94kYg8
QGXiP2gkHX5mvvU/UhVsNQhz4j9WKzSAus/4PzYYCPtDOeI/OnMin+3e+z9A
bfszErjhP8IqQUpg6v4/8aowbbfv4D8MfkN3OvgAQJt1/eo7wd8/xZ/8aMh3
AkD5W4FKqxbdPwsszXYO8wNARYoFJCfh2T+dy7BiQ2kFQNtU2lxjItY/Lgjl
nqDZBkAAsz6tXNzRP+CFQ7diQwhAiETmJK8iyj/9Jfe4yaUJQMBa32p8D78/
EwxVmBkAC0DwU5q9OGyfPxh7o5SaUQxAFC7sa5xLsb+amZmZmZkNQCIKrYaV
K8a/4Gy0hA==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZub+axwElsKDgP8DEImlP
33G/2U7N/OUD4j/8Afpoh9i3Py6yFPoUQ+M/PUjqXU6eyD/BEhHBk13kP35D
coQ0wNI//F17bcxS5T8OeGne3kXZP0uZFOI8IuY/rEyTULDc3z8iva8Nd8vm
P/KRX+qUQOM/sX2YJSFO5z8Kr9yS4pfmP14EONX1qec/8eEET3vy6T9pUt5i
xN7nP+0XxoyXTu0/28ecyXDs5z+T+zh2N1XwPyYXJMjz0uc/tVNymRwC8j+q
wZ3kWpLnP6EbzJ0XrfM/qxJ/Zcgq5z/qFRL1RVX1PxRnWT9znOY/yYtUjcb5
9j93aLH3puflP3ZTVEi6mfg/Q7TsfMMM5T8AtOVxRDT6P+oxa/M8DOQ/vVUL
NYvI+z9YGud3m+biP6Yciw+4Vf0/R2M813qc4T/EmL9D+Nr+P3fgvTuKLuA/
IdQ0pL4rAEDD74egFzvdP1F7I5s+5QBADnNMsqjU2T+amZmZmZkBQCIKrYaV
K9Y/b4bI9Q==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz4z8iCq2GlSvmP3pNMXfN
iec/wL8Uud5B6D/bjLdykvfrP+L9ZrkNJeo/YD4GSRQ98D+UKy8nItTrPzXS
6ZqVh/I/YuZnRzdO7T9yDQ78Fdr0Px5A2H2Eku4/cnxP+1kz9z97T4W3XaDv
Pxw3v5Aikvk/g8z/4pk78D+p7unGLfX7P0vvrVVKi/A/1qysZTdb/j/Sglfr
Fb/wP/ZPHs98YQBAWn52KOHW8D8oVgPclpUBQJhoCW2f0vA/r4QFX0bJAkDa
PUb7UrLwPwh3LBbo+wNAwqNn9gx28D85IKdO2SwFQCgNlVntHfA/hq0fO3hb
BkCvQtXNRVTvP9yllUkkhwdAmM9THdY17j+PuJB4Pq8IQETEQJ0j4ew/WQZ/
qynTCUAswaoT41brP9sgEv5K8gpArOsoseWX6T/phG4WCgwMQEr526EYpec/
LfQBdtEfDUCWQ5aPhH/lP8TJ1cgOLQ5Ab6ptFU0o4z8zMzMzMzMPQJrHASWw
oOA/FmC2EA==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZyT8iCq2GlSv2P52YP7Vt
8dM/1aNLHgEB9z8Qn5lZBDvbP6jnV2plwfc/+sZYbNlS4T8rdJhVXGz4P6jG
GuHEFuU/s0ldKYsB+T/QhZsWRefoPz26ob2igPk/zAR261PC7D9DewyjX+n5
PyV6IdLyUvA/ipC3Roo7+j9/f74A9UfyP6gTrQ/3dvo/O0HlqyY/9D/gMgl2
hpv6P3f/C9V8N/Y/ICG1EyWp+j9kx3bi6y/4P5sWsq7Ln/o/5Ic0LWgn+j/e
6+48f3/6Px8oI4/mHPw/iEam4VBI+j+mRa/wXA/+Pza9RuRd+vk/zmME1sL9
/z+0xemgz5X5PyVRsfWI8wBA06thctsa+T9jwidII+UBQKc16pXCifg/bPir
MDDTAkDE+ooI0uL3PxklDWAxvQNAWMw9XmIm9z8nP6isqqIEQAbb7pLXVPY/
T3BJVCKDBUCCim/VoG71P9z2xjwhXgZALBl4TDh09D80MzMzMzMHQN5o19Ui
ZvM/fYDAkQ==
      "]], 
     {GrayLevel[0]}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
     StripOnInput -> False], TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 261},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->2070927553]
}, Open  ]],

Cell["\<\
For one-to-one and onto domain and range should be of the same size.\
\>", "ExampleText",
 CellID->270095829],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionArrowDiagram", "[", 
  RowBox[{"6", ",", "6", ",", "\"\<OneToOneOnto\>\""}], "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->1867651892],

Cell[BoxData[
 GraphicsBox[
  {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
   0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
   {RGBColor[0.0692454, 0.645548, 0.968322], 
    DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
   {GrayLevel[0], PointSize[Medium], Opacity[1], 
    PointBox[{{
      0.5, -1.2124355652982142`}, {-0.4, -1.0392304845413265`}, {-0.1, 
      0.17320508075688773`}, {-0.8, -0.34641016151377546`}, {
      0, 1.3856406460551018`}, {-0.8, 0.34641016151377546`}, {3., 
      1.0392304845413265`}, {3.7, -0.5196152422706632}, {3., 0.}, {2.4, 
      0.34641016151377546`}, {3., -1.0392304845413265`}, {3.6, 
      0.6928203230275509}}], 
    {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAA4D/eaNfVImbzv0O1xOnZ
EuM/miMTtfVp8r+9EJ2o1zvmP1j7z5Lzf/G/ioKJ/Et56T/w2DGZmKjwv8Iy
9Ml+yew/KH9VGa7I77+EKZsBVxXwPzP1sh4uZ+6/azCyS4fN8T+HrpHTbC3t
v+1R8bJmjPM/L3dZshAc7L8BVSAbCFH1PzQDb8aqM+u/BR/sWXsa9z9jDD5f
tnTqv+0qVbbN5/g/Rq3Lzpjf6b8BX3lpCrj6P3Gw8zOhdOm/Uet0IDuK/D/2
Wm1QCDTpvxZHFX9oXf4/FvitavAd6b/ZeYxRTRgAQA4huTtlMum/pjDb02wB
AUBMZ+joW3Hpv54VkhSX6QFAwq2tCbPa6b8Cce7jUNACQFAjTbkybuq/LF6s
1R+1A0BBdoW0jCvrv1np+4GKlwRAc4QWg1wS7L9G++vFGHcFQJh/EK0nIu2/
beQoA1RTBkBkPM/7XVruv4217F7HKwdA4jF/xlm6778AAAAAAAAIQJrHASWw
oPC/i6zBCw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ2b+axwElsKDwv1isuvgv
ss+/3Caw7Cy57b8QF/q0diO3vwvqDsO5auq/AJVEiPlGsj9uO0Ltx1fnv3Ct
rZB7Ys4/z1Gi+PiB5L9iJQyP/Q/aPwG1LwLO6uG/YcwAmByU4j+EA3jUTSff
v2f0Z9IGOug/UMFqM4P72r8/IzxDvvbtP96B77JyVNe/Ntnizpvj8T/cr/p3
DDTUv2LRyYwu1PQ/+Ex8M/mb0b/+PsUKiMv3P/BsEYIyG8+/zGzSXxXI+j8o
a8XXBxTMv3xRQuBAyP0/rBuUtQ0kyr/ilXx6OWUAQLhXukVLTMm/Dm/ZeYnm
AUDgtI8DM43Jvw+l7PtDZwNAeBHIfqLmyr/5zrzdnOYEQHCAuW3iV82/hLry
t8hjBkBQcE6H02/Qv7Btxkr93QdAXMPKawi+0r95VU/pcVQJQCBJkbpWldW/
+LD+41/GCkBIdjmLPPTYv8q5C/ICMwxAzM11A/DY3L+dmZmZmZkNQJnHASWw
oOC/ObXTBw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ6b8iCq2GlSvWv8+XmWV5
/OS/xNFpGunvz78sWZs6cTzgv4IBBTj6YsS/88yopAy41r/cALew/myzvwxY
JVY8d8m/ANQuO1W7Pz/QQUjJG0ekv5RD7CDa1bE/0Mu2w+hqvz+q7MPy+9XA
P7SnnKK3adI/hK9kw9bMxz+4Lqeneh7dP7RULofLy80/rs2SgKr54z8uY+/l
1WfRP9opkrRDcek/WnUgLOdq0z9Q6KVPIvPuP6DC0EoI7tQ/rdM8Hy0+8j+O
GOrYa/DVP0sCv8P9BPU/HEqXvIhx1j/L8fPsicz3P4QQA3QacdY/H9T2DliT
+j/4mbE5Ie/VP5sDtwLvV/0/en1hBOLr1D+q9QpnawwAQDIjdGLmZ9M/eqCy
tUxqAUCwDPEw/GPRPy7b7cfhxAJAFG2VXGrCzT8Ia0u4chsEQOARPtLMwcc/
HsHgwkltBUDg0IAWT8nAP0rFu6WzuQZAkLGdBUi5sT8AAAAAAAAIQAAAAAAA
AAAA4BTFAA==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZub8iCq2GlSvGPxD4Prek
LHC/RjTPjP4Mzj9M7E6rI0W4Pyo7dM+HrtI/WT1ZJWMYyT/HxOy18wvWPx7/
gS2XLNM/OioJ6Pkc2T+oPwag9e7ZPyM+Tt3539s/oy0xZNtn4D97y5BzfFPe
P6dwrzKa5eM/Fp0AWxo74D/Pdg0D3W7nPxER60aAI+E/1wqlgMMB6z/iBxyu
dOLhP6cidzlonO4/AGeOPpJ34j9nxd/PcB7xP9RjZtmJ4uI/t1HbhqHw8j9P
uey8IiPjP3yte+XOw/Q/MBysojo54z8YWn8JAZf2PzfzoNHFJOM/ssccDkBp
+D/4rHEkz+XiP6ORio+UOfo/hGasA3h84j9rSEMuCAf8P/XwDFT46OE/vSK/
EabQ/T8EntRYnivhPxg5Xmp7lf8/0o9Dis5E4D95Lh/5S6oAQFspk8AGat4/
oBdcNoeGAUDCrxUjmvnbP8DoH5L6XgJAxsS1jaI52T8zMzMzMzMDQCIKrYaV
K9Y/Wwa7+w==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ6b8iCq2GlSvWP3Hj1JUv
OOS/1C96hho23T9M0fqCW17dv3YCEGSv4OE/APva8gcD0r9od0Gz9OTkP2Ao
6myskbm/HM2ubEOm5z9gIfrAQt21P1xoDFQlI+o/kAGmreCJ0T9laHd7SFrs
P/QBegPdzN0/GbjD9n9K7j/mqF0T4hzlP7lgJ3vE8u8/FthUJ/9k6z/fMnd1
GqnwP/9OEz243fA/tIR2Zgs08T+vcvyWbA70P8eMuFfrmfE/w+6sSetC9z9C
4j47hNrxP9dhjvqAefo/5DmUyrP18T8zhiwzebD9PzQe/Jhr6/E/fLJlpA9z
AEBETBobsbvxP1SjDKJfDAJAeqUNpJ1m8T+muEBkU6MDQGo/AFhe7PA/NntS
/RI3BUAgszMUNE3wP76jvzLIxgZAh8IsmeYS7z+Yfd7unlEIQPvb570HQ+0/
yhFmscXWCUAOd7G3wSvrPwBll/5tVQtAjkKLijDO6D/OzMzMzMwMQCIKrYaV
K+Y/qxfDEg==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAAAAAiCq2GlSv2P2jpf6Xm
TcA/FTmVr4j/9j9VIKpQU3LQP8ZTOneEu/c/gjqSULnd2D9zQy0fJV/4P6d8
uNVWsuA/Bs9Y1RPq+D+0HZsNVQHlP8m+EuIGXPk/nJ+uEI5Z6T81JjnPwbT5
P0ln/rmzuO0/yBJHiBX0+T+HdQIcOg7xP1qaT3PgGfo/Rgfgoz1B8z+VB9KC
Dib6P0qwiLg5dPU/8K5dQJkY+j9LLuGhA6b3P9/G/8+H8fk/Ej1CSnHV+T9j
cXns7rD5P69rotxZAfw/v/k/3PBW+T9K0xVilij+P0AcTV+94/g/8A4oLwEl
AEBy/siVkVf4PwLpajU+MgFAW0mb37ey9z+qRUVoczsCQP6a9LSH9fY/ulKl
DxRAA0DWP+V3ZSD2P8L0n+GVPwRAUdQZP8Iz9T/hANFLcTkFQCoR2pkbMPQ/
vGpKuyEtBkB6mWlN+xXzP6w77OIlGgdAxCPuC/fl8T8AAAAAAAAIQJrHASWw
oPA/gLm0fw==
      "]], 
     {GrayLevel[0]}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
     StripOnInput -> False], TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 259},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->1811916342]
}, Open  ]],

Cell["\<\
For a relation some points in the domain have more than one image.\
\>", "ExampleText",
 CellID->1922168802],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionArrowDiagram", "[", 
  RowBox[{"4", ",", "7", ",", "\"\<Relation\>\""}], "]"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->1294553393],

Cell[BoxData[
 GraphicsBox[
  {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
   0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
   {RGBColor[0.0692454, 0.645548, 0.968322], 
    DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
   {GrayLevel[0], PointSize[Medium], Opacity[1], 
    PointBox[{{0.4, 
      0.6928203230275509}, {-0.30000000000000004`, -0.17320508075688773`}, {
      0, -1.3856406460551018`}, {0.4, -0.6928203230275509}, {2.8, 
      1.3856406460551018`}, {2.2, -0.34641016151377546`}, {
      3.2, -0.34641016151377546`}, {2.8, -1.3856406460551018`}, {3.5, 
      1.2124355652982142`}, {3.1, 0.17320508075688773`}, {2.5, 
      0.5196152422706632}}], 
    {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAAAAAiCq2GlSv2v2JYeqrk
O6o/wnqO3JFr87/zWoakGTm8P40PWVfWtfC/tfCuArmkxj/h0yQephfsv+Yg
Lmh7EdA/cninx+Pd5r/xXbVfHknVP/URjnsrweG/FRLpt4D22j9EwVtDZ4jZ
v/BPXKNPi+A/kLGnI4emz79MYZoAHdPjP3AagP1JoLm/CFyF2upQ5z+AQbGd
ZKGlPyA+sfDeAus/UFoaVUj5xj9WlxJXA+fuP/CrRzyr7NM/QyYowKN98T9c
WxKtx//bP/dNG6vAnvM/mOkhV9jY4T9MStsvt9X1P2zJKYIof+U/+ADreVoh
+D8ou7mY5PDoP4R3drxygPo/eKrSwTgs7D9Fr8PXvfH8P9TSmgJuL+8/99Id
BPBz/z+c7qSTdfzwP+4C8EDaAgFAchkkzppD8j/WNSEn11ICQFwohxt5bPM/
Wr1EbjypA0DWwiz3cnb0P+2qmmlUBQVAzC+qQPtg9T9lZmZmZmYGQCIKrYaV
K/Y/Pu7DdQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAAAAAiCq2GlSv2vzC+afz7
rLA/5ywuOZy89L9nur7EZDDBP32ArRgpWvO/apSWCgKJyj8+OUQ3+ATyv16N
byWwLdI/WJ4Pnr698L9Q2D+zJFHXP6i5RNpTCu+/wFIDtCSs3D9P2Mj+v7fs
vyK/FlrsHeE//sskJf2E6r/CMK2kpv7jP9wFFuc1c+i/cpaQUbr25j/sSS5e
g4Pmv91V9BSUBOo/u/Csjuy25L8Ys3AVlSbtP2R58NtlDuO/lpcQ5Ikt8D+A
eMqG0Irhv1y0N2ku0PE/mrH6Nfos4L9NWIn4WXrzP9qLIRI5692/o+DkcSor
9T/gvNxrucrbv6XCqC664fY/1CY8Ypb52b8jzYx7IJ34PwQqW8LGeNi/QKAa
FHJc+j/sf2e6FknXv1jYgp/BHvw/3LhKbSdr1r+Er4wuIOP9P4CJKp1u39W/
Q0Ndup2o/z9cRu1sNqbVv+Cc6dEktwBAyLPjOJ2/1b+amZmZmZkBQCAKrYaV
K9a/GuzPvQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ2T8iCq2GlSvmv/yhL1HH
IuA/4SeNZNjE478GJ5kM2pPjP57O/q6cheG/pHvmfzEe5z/udsA7J97evyD0
qMPsv+o/KMrgobEE27/T6VCHHnfuP9TBonLjgNe/3il3C+cg8T9ws58dmlTU
vzIifjP8DvM/+GNyooSB0b/ymtSSyAT1P0weDlhFEs6/YpGF6EEB9z+wEBGK
h9nJv17n1WhaA/k/zPKsSw1bxr/UqWtMAQr7PwhtxjixmMO/bIQKYSMU/T8s
wZMc6pPBv0aWmJurIP8/SEm/KspNwL/VFo/VQZcAQBBs09v8jb+/ozU5Rkqe
AUAgfrLXnP+/v+A4I4/jpAJAgHyG/xv4wL9PSqBagqoDQCQNG2ljr8K/M3js
15uuBEAQjNmTuyTFv9p71ASmsAVApL7JkdZWyL+15e/2F7AGQI5b50wCRMy/
RtRGJGqsB0AKtLq2FHXQvwyqPKsWpQhAe/p9Omoj07+bmZmZmZkJQCIKrYaV
K9a/gU/I/w==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ2T8iCq2GlSvmv0+H8EAF
WuA/rFsL9nRg5b/+ScNGi+/jP8s30UjsvuS/dZH1B3iL5z8f9a80UUfkv1Rg
tUjhK+s/TSUcMOP547+DVsxr2s7uPzE6oVDL1uO/HpAAvDo58T9m7BQ1HN7j
v0/GGA9iCvM/BfS1+9EP5L/cHM1f7Nn0P75RO0TSa+S/wFXgueOm9j90ENM9
7PHkv5xquoZTcPg/dhIJwdih5b/Yyy8PSTX6P9IriHU6e+a/ED5K/NP0+z+/
gaEDnn3nv4YAz9YGrv0/Nud/UXqo6L+Qkj6F91//P8m95ssw++m/kZ4F5N+E
AEBcwVW6DXXrvxs84tk+VQFAvBllnkgV7b+qLHsUqiACQEQZJ54E2+6/RLi+
pLXmAkD+mqt8qGLwv1ppiHX4pgNAEQiMxJRp8b+dxWKDDGEEQCJf/qS7gfK/
piSoEo8UBUB+EAp4iKrzv0vs5eMgwQVAedhTwl3j9L9mZmZmZmYGQCIKrYaV
K/a/Ze7L2Q==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz078iCq2GlSvGv4LVy9RM
2sa/QKQqrprRob8wHZetoQ+qv+TWzNnbEbk/GEO/UjcotT+mTHKCZMPMP+jJ
iICtZMw/t/8uXTAZ1j8GvkWiLHLXP3crhBjZZt0/GkjRjVOC4D+mY5A0ZiPi
P91Cz/lYcuU/utmCPbJa5T8s3dvkh4bqP5jL5M+bV+g/AD6xbi687z+4+RUP
jRjrP/pqiHpEiPI/QCVk8A+c7T8g053FYUD1PwBrggHP4O8/pbIHvf0E+D9u
i9oOy/LwP1hY/mOg1Po/YeeoiKnU8T/1KQPky639PzGqnhWLlfI/hHu9q35H
AEBq1QxeCTXzP469e8tWuwFAk5Jwwc+y8z9SkWEEqTEDQLJeWoObDvQ/Qnkg
ua6pBECpBtfuO0j0P0y9WGWgIgZAw6pHcJJf9D++fnYHtpsHQLsSm6WSVPQ/
9DrRiicUCUCftt9kQif0P+Zt1TEtiwpAJAArubnX8z8AAAAAAAAMQN1o19Ui
ZvM/7ja9tA==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ2T8iCq2GlSvmPzunjtsl
neA/UBDm9xhl5z98k/LdwHrkP/I8KBbZceg/wr2FvpBj6D/1zStIR1HpP+VU
nXWCVew/cEGhAO0C6j9CEI0RPyfwP8KmGP1rhuo/KiNfFTQm8j8TNQV4ftvq
Py4wPymRJvQ/153CTfcB6z9m/BpxRif2P4x2iBTC+eo/uGET4kMn+D9sBD8n
48LqP6jA1NJ5Jfo/xqovo3dd6j9GsLuL2SD8Pwc2kli1yek/xnB51lUY/j+V
Nf+t6gfpP6j0dcZxBQBANYvWdn4Y6D8Ss24TvfsAQGxCsLzv++Y/W3AwI4ru
AUDWovR71bLlPyksQCFY3QJAjkG/U9495D/E0vlXqMcDQGyTNynQneI/GZ/L
c/6sBEDpKZC+h9PgP6nILcXgjAVAnqLIe/C/3T/afDOB2GYGQC5uZHBViNk/
TOGSAHI6B0AYHGwxewLVP5atAv08BwhAtaDhGcgw0D/OzMzMzMwIQBwKrYaV
K8Y/Vwe9XA==
      "]], 
     {GrayLevel[0]}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
     StripOnInput -> False], TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 259},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->1559845534]
}, Open  ]],

Cell["To make a diagram reproducible use SeedRandom.", "ExampleText",
 CellID->2036783074],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SeedRandom", "[", "4321", "]"}], ";", 
  RowBox[{"FunctionArrowDiagram", "[", 
   RowBox[{"7", ",", "5", ",", "\"\<Onto\>\""}], "]"}]}]], "Input",
 CellLabel->"In[5]:=",
 CellID->1323791538],

Cell[BoxData[
 GraphicsBox[
  {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
   0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
   {RGBColor[0.0692454, 0.645548, 0.968322], 
    DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
   {GrayLevel[0], PointSize[Medium], Opacity[1], 
    PointBox[{{0.30000000000000004`, 0.5196152422706632}, {0.1, 
      1.2124355652982142`}, {0, -1.0392304845413265`}, {
      0.7000000000000001, -0.8660254037844386}, {-0.4, 
      0.34641016151377546`}, {0.8, 0.6928203230275509}, {
      0.6000000000000001, 0}, {3.4, -0.6928203230275509}, {3.5, 
      1.2124355652982142`}, {3.5, -0.17320508075688773`}, {2.8, 
      1.0392304845413265`}, {2.5, 0.17320508075688773`}}], 
    {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAAAACaxwElsKDwvzImnlAR
acA/uiq5JDFv7r9+ldRxoajQP/wa8QaPzeu/ni8BGM5X2T8pc4te313pv0g4
yH65HuE/CI3sGG0h579N5TfPa6rlPzYQZvNnGeW/N659iJRM6j+z+wza40bj
v7Qknkq+Au8/+jRSVdiq4b9NPTOMNOXxPz1LqwYgRuC/LTCA1YVQ9D90PiJp
8DLevwa7OWUKwvY/XvojzABL3L/ywS5Vdjj5P3TkWBx01dq/FzkUJXuy+z+G
jj+OENPZv2ePAWzILv4/nCWtO19E2b/ULXhFBlYAQCCRDturKdm/2krwr3qU
AUDcJjyXBIPZvzAnoXwY0gJAPET1BzpQ2r8U2qIkNw4EQIjKB0vfkNu/uh9Z
7C5IBUCgJBY+SkTdvzQRczxZfwZAOi/d2JNp378PxU/6ELMHQJUO5VPM/+C/
vfGY37LiCEBjpk+zfILiv40W5dCdDQpAFgBr3Q085L8yMzMzMzMLQCIKrYaV
K+a/Ppi0IQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAZ2ZmZmZm5j+qTFjoerbrvxDHQFBY
fuk/Mj00CGre6L8UY3Nbm7bsP+mIz9glK+a/bsLcFr0G8D/AbTMjHZ7jv/Iy
k5eXwPE/yV8vZ6o44b+QJ2m98ofzP7m//UYm+N2/GgnT69xb9T/eyShSDtPZ
vylsYNxdO/c/1PbJ8z8E1r8MHngidyX5P/RGFGfAjdK/53BXsiQZ+z8EviQY
zOLOvy4mDWtdFf0/jMcj3K5hyb/0wCajExn/P/TC38YUm8S/xs3i25qRAEBA
a3qOhpHAv4rq505XmQFAEOmQNFGOur+YBng6M6MCQGARrL9ze7W/L/cKjqGu
A0Dgnn3rJe2xv9tdlGMUuwRAIFtdc5XKr7/uOc5K/ccFQBCBIwDsya6/iNHM
lM3UBkCIIEfWa2ywv0zYs5/24AdAkKebcSD7sr8CqGQi6usIQMgkNW+4D7e/
SWP9dxr1CUDA/WSMCai8v1HzAOv6+wpArJ9f5o3gwb8BAAAAAAAMQBwKrYaV
K8a/QlnGPw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz4z8AAAAAAAAAADvVWJfx
Buc/gZdgr33qrD8sl6PMuu7qPz4auQPqgbs/cOHHVnzo7j8aYup6io/DP47M
RRgNefE/As5PD0ujyD9adSf1t4TzP96dUhmF+cw/BBqG5iiW9T8+sY7B9UfQ
P7xSmgBHrPc/VnCof0uy0T97k2bc9sX5P1v8yYUDu9I/XqkWLhvi+z+naUxe
kWHTPzh5YFyV//0/sImVqZyl0z+m1koMow4AQEr7+0wBh9M//tCK+wYdAUBT
UO+FzwXTPy5X5wNnKgJAUB9a4Usi0j/C7j43NDYDQPySQhfv3NA/6CcufeA/
BEAYiXmVy2zOP9jkdN7eRgVA9rqhXCBfyj+TNsLPo0oGQJ41CxoDk8U/Mh+/
e6VKB0AUopxw/wrAP3vwMAxcRghAWsanbAmUsz+JeAzyQT0JQKDH7Cwjm5Y/
Zr5TLNQuCkBKcJmVvlSjv5+0mI2SGgtAzF6IRU5Xur8AAAAAAAAMQCIKrYaV
K8a/yoGvNQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ2b8iCq2GlSvWP4xVTtEc
8tG/Eh2ZPOXI2T9Uo3HNFEXEv7JQmwcQDN0/GOmBkzp4ob8vOyLEWvPfP9jI
XITRkLc/kngqh50+4T/QqjupvyXMP4gsqwksVOI/DGgYmqNY1j+w3R2kxTnj
P+xKaQIcsd4/PL0qg/Du4z/1Js247YvjPz6DO4ZMc+Q/u1ItXDbE5z/kuXxy
k8bkP2/ywLgq/+s/6UkhGJno5D9MrZUYRh3wP7aC1GlL2eQ/+qEV9w068j86
LU6GspjkP1yuzgfOVPQ/uZQDtPAm5D+C3X1uaGz2P4/O905ChOM/0E9c+sB/
+D9X57So/bDiP7DJ6by9jfo/zvN+2pKt4T8mbYSfR5X8P3hS2YmLeuA/ZD5+
90qV/j8sW/s+FTHeP3vwMAxcRgBAuPvW4ZcQ2z+JeAzyQT0BQJzWe7lHldc/
Zr5TLNQuAkAZ3Pmz/cDTP5+0mI2SGgNA3uSV6oMrzz8AAAAAAAAEQCIKrYaV
K8Y/fdfEwg==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz0z+axwElsKDgP+5GkQa3
Ttk/NM3ux1O54T/OfI3rJ4LfP/K5liEYruI/OIBZyx3l4j9mKHNSe37jP6P/
wWTOEeY/4n+LyA4q5D/tVpqLdkXpP98TIHp3sOQ/e68iV2N+7D9KJvgVbhHl
P2NvBBPfuu8/dis5Kb9M5T/6i8gTmXzxP3w7szpLYuU/JM2AAdIb8z8kNZTb
BlLlP64vnQE+uvQ/wLV4rfob5T8Q/csvAVf2P0Su111DwOQ/GTskiUDx9z9O
A8uWET/kPxm5WmAiiPk/YEs95amY4z852kHRzhr7P3leipRkzeI/O9RWM3Co
/D+uA6Z/rd3hP8aaH4szMP4/mpjh1wPK4D8xUB36SLH/P7Yf3sLzJd8/0OqL
FnKVAEDyK4pLaXLcP2n6Q2QeTgFA7xQbONd62T+KTnNpRwICQKaHmY/QQNY/
lGVUjo2xAkCyiuqbC8bSP8x32NKTWwNAQoqCAsIYzj8AAAAAAAAEQCQKrYaV
K8Y/kqO2kg==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ6T8iCq2GlSvmP0x+S17c
6es/F5Tp+Q8D6D92CD6N907uP7ak1XkYv+k/5wgV69Jj8D/3xuprw17rPyfV
8sJLqfE/sd2bQjTh7D8fq0GcOffyP1CWW/KdRe4/ZstKSOtM9D8XYWheQ4vv
P9e/mnmrqfU/axqS3rtY8D/8SjQkwQz3PxJw4XnP2/A/2fC/3291+D9eIY/0
lk7xP1kAhEv44vk/Dj7CZ9Ww8T8xEvBzmFT7PxpzrLJWAvI/CxyGOYzJ/D+V
yDKW70LyP0d96rgNQf4/PvPfy31y8j8wzuOzVbr/P7MMFBjokPI/nF2J/U2a
AED5C2hXHp7yPwzqFOyLVwFA2eM9hxma8j+KeTY8gBQCQMO7eMnbhPI/GN2f
q8bQAkAbSltjcF7yP01ORVT7iwNAjumMt+sm8j8oAF/hukUEQMWVSDtr3vE/
6ZsdxKL9BECojLtmFYXxP/ay9WdRswVAXd+boBkb8T9mZmZmZmYGQJrHASWw
oPA/I77Fag==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQPWsmCOy0v5dx/apS2md7saM97Qx65+wbWqo6J0z7
Yv9v287cHX+u29+JK46OXfvVPiX6l1Z25wP7I3npsV9XfrMPmiz+wFnrqT3X
sTUdl7q/25/2aV5ReO+l/SS3dNbaoB/23XsmTSld/M7+W5h8VvffH/Yn9S4/
St790T4vcsPnRT0/7WcrfEt6o/XFvtO8cGv5/5/2/NtFijjnf7N3Kq09eCb0
l/2cRxknc8V+2ptkc3qW9P+ytxK8lXKq/7f9lYYNW/zX/LJP6Nhvcp7zH5xf
Hy3xYqojgwNMvWvhIWGPWkYHmHkNewJnh21ncoDZl2GafFvqPbMDzD2rZFek
v1dmdYC5d9bP1UkGoWwOMP/8eH7sinUTuwPMv19M+ZiKVnE4wMLj7VTR/aZn
OR1g4VXNEHLkzUsuB1h4MoABjwMsvAELgcMN
      "]], 
     {GrayLevel[0]}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
     StripOnInput -> False], TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 261},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->1991011009]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SeedRandom", "[", "4321", "]"}], ";", 
  RowBox[{"FunctionArrowDiagram", "[", 
   RowBox[{"7", ",", "5", ",", "\"\<Onto\>\""}], "]"}]}]], "Input",
 CellLabel->"In[6]:=",
 CellID->1870802292],

Cell[BoxData[
 GraphicsBox[
  {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
   0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
   {RGBColor[0.0692454, 0.645548, 0.968322], 
    DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
   {GrayLevel[0], PointSize[Medium], Opacity[1], 
    PointBox[{{0.30000000000000004`, 0.5196152422706632}, {0.1, 
      1.2124355652982142`}, {0, -1.0392304845413265`}, {
      0.7000000000000001, -0.8660254037844386}, {-0.4, 
      0.34641016151377546`}, {0.8, 0.6928203230275509}, {
      0.6000000000000001, 0}, {3.4, -0.6928203230275509}, {3.5, 
      1.2124355652982142`}, {3.5, -0.17320508075688773`}, {2.8, 
      1.0392304845413265`}, {2.5, 0.17320508075688773`}}], 
    {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAAAACaxwElsKDwvzImnlAR
acA/uiq5JDFv7r9+ldRxoajQP/wa8QaPzeu/ni8BGM5X2T8pc4te313pv0g4
yH65HuE/CI3sGG0h579N5TfPa6rlPzYQZvNnGeW/N659iJRM6j+z+wza40bj
v7Qknkq+Au8/+jRSVdiq4b9NPTOMNOXxPz1LqwYgRuC/LTCA1YVQ9D90PiJp
8DLevwa7OWUKwvY/XvojzABL3L/ywS5Vdjj5P3TkWBx01dq/FzkUJXuy+z+G
jj+OENPZv2ePAWzILv4/nCWtO19E2b/ULXhFBlYAQCCRDturKdm/2krwr3qU
AUDcJjyXBIPZvzAnoXwY0gJAPET1BzpQ2r8U2qIkNw4EQIjKB0vfkNu/uh9Z
7C5IBUCgJBY+SkTdvzQRczxZfwZAOi/d2JNp378PxU/6ELMHQJUO5VPM/+C/
vfGY37LiCEBjpk+zfILiv40W5dCdDQpAFgBr3Q085L8yMzMzMzMLQCIKrYaV
K+a/Ppi0IQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAZ2ZmZmZm5j+qTFjoerbrvxDHQFBY
fuk/Mj00CGre6L8UY3Nbm7bsP+mIz9glK+a/bsLcFr0G8D/AbTMjHZ7jv/Iy
k5eXwPE/yV8vZ6o44b+QJ2m98ofzP7m//UYm+N2/GgnT69xb9T/eyShSDtPZ
vylsYNxdO/c/1PbJ8z8E1r8MHngidyX5P/RGFGfAjdK/53BXsiQZ+z8EviQY
zOLOvy4mDWtdFf0/jMcj3K5hyb/0wCajExn/P/TC38YUm8S/xs3i25qRAEBA
a3qOhpHAv4rq505XmQFAEOmQNFGOur+YBng6M6MCQGARrL9ze7W/L/cKjqGu
A0Dgnn3rJe2xv9tdlGMUuwRAIFtdc5XKr7/uOc5K/ccFQBCBIwDsya6/iNHM
lM3UBkCIIEfWa2ywv0zYs5/24AdAkKebcSD7sr8CqGQi6usIQMgkNW+4D7e/
SWP9dxr1CUDA/WSMCai8v1HzAOv6+wpArJ9f5o3gwb8BAAAAAAAMQBwKrYaV
K8a/QlnGPw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz4z8AAAAAAAAAADvVWJfx
Buc/gZdgr33qrD8sl6PMuu7qPz4auQPqgbs/cOHHVnzo7j8aYup6io/DP47M
RRgNefE/As5PD0ujyD9adSf1t4TzP96dUhmF+cw/BBqG5iiW9T8+sY7B9UfQ
P7xSmgBHrPc/VnCof0uy0T97k2bc9sX5P1v8yYUDu9I/XqkWLhvi+z+naUxe
kWHTPzh5YFyV//0/sImVqZyl0z+m1koMow4AQEr7+0wBh9M//tCK+wYdAUBT
UO+FzwXTPy5X5wNnKgJAUB9a4Usi0j/C7j43NDYDQPySQhfv3NA/6CcufeA/
BEAYiXmVy2zOP9jkdN7eRgVA9rqhXCBfyj+TNsLPo0oGQJ41CxoDk8U/Mh+/
e6VKB0AUopxw/wrAP3vwMAxcRghAWsanbAmUsz+JeAzyQT0JQKDH7Cwjm5Y/
Zr5TLNQuCkBKcJmVvlSjv5+0mI2SGgtAzF6IRU5Xur8AAAAAAAAMQCIKrYaV
K8a/yoGvNQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ2b8iCq2GlSvWP4xVTtEc
8tG/Eh2ZPOXI2T9Uo3HNFEXEv7JQmwcQDN0/GOmBkzp4ob8vOyLEWvPfP9jI
XITRkLc/kngqh50+4T/QqjupvyXMP4gsqwksVOI/DGgYmqNY1j+w3R2kxTnj
P+xKaQIcsd4/PL0qg/Du4z/1Js247YvjPz6DO4ZMc+Q/u1ItXDbE5z/kuXxy
k8bkP2/ywLgq/+s/6UkhGJno5D9MrZUYRh3wP7aC1GlL2eQ/+qEV9w068j86
LU6GspjkP1yuzgfOVPQ/uZQDtPAm5D+C3X1uaGz2P4/O905ChOM/0E9c+sB/
+D9X57So/bDiP7DJ6by9jfo/zvN+2pKt4T8mbYSfR5X8P3hS2YmLeuA/ZD5+
90qV/j8sW/s+FTHeP3vwMAxcRgBAuPvW4ZcQ2z+JeAzyQT0BQJzWe7lHldc/
Zr5TLNQuAkAZ3Pmz/cDTP5+0mI2SGgNA3uSV6oMrzz8AAAAAAAAEQCIKrYaV
K8Y/fdfEwg==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz0z+axwElsKDgP+5GkQa3
Ttk/NM3ux1O54T/OfI3rJ4LfP/K5liEYruI/OIBZyx3l4j9mKHNSe37jP6P/
wWTOEeY/4n+LyA4q5D/tVpqLdkXpP98TIHp3sOQ/e68iV2N+7D9KJvgVbhHl
P2NvBBPfuu8/dis5Kb9M5T/6i8gTmXzxP3w7szpLYuU/JM2AAdIb8z8kNZTb
BlLlP64vnQE+uvQ/wLV4rfob5T8Q/csvAVf2P0Su111DwOQ/GTskiUDx9z9O
A8uWET/kPxm5WmAiiPk/YEs95amY4z852kHRzhr7P3leipRkzeI/O9RWM3Co
/D+uA6Z/rd3hP8aaH4szMP4/mpjh1wPK4D8xUB36SLH/P7Yf3sLzJd8/0OqL
FnKVAEDyK4pLaXLcP2n6Q2QeTgFA7xQbONd62T+KTnNpRwICQKaHmY/QQNY/
lGVUjo2xAkCyiuqbC8bSP8x32NKTWwNAQoqCAsIYzj8AAAAAAAAEQCQKrYaV
K8Y/kqO2kg==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ6T8iCq2GlSvmP0x+S17c
6es/F5Tp+Q8D6D92CD6N907uP7ak1XkYv+k/5wgV69Jj8D/3xuprw17rPyfV
8sJLqfE/sd2bQjTh7D8fq0GcOffyP1CWW/KdRe4/ZstKSOtM9D8XYWheQ4vv
P9e/mnmrqfU/axqS3rtY8D/8SjQkwQz3PxJw4XnP2/A/2fC/3291+D9eIY/0
lk7xP1kAhEv44vk/Dj7CZ9Ww8T8xEvBzmFT7PxpzrLJWAvI/CxyGOYzJ/D+V
yDKW70LyP0d96rgNQf4/PvPfy31y8j8wzuOzVbr/P7MMFBjokPI/nF2J/U2a
AED5C2hXHp7yPwzqFOyLVwFA2eM9hxma8j+KeTY8gBQCQMO7eMnbhPI/GN2f
q8bQAkAbSltjcF7yP01ORVT7iwNAjumMt+sm8j8oAF/hukUEQMWVSDtr3vE/
6ZsdxKL9BECojLtmFYXxP/ay9WdRswVAXd+boBkb8T9mZmZmZmYGQJrHASWw
oPA/I77Fag==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQPWsmCOy0v5dx/apS2md7saM97Qx65+wbWqo6J0z7
Yv9v287cHX+u29+JK46OXfvVPiX6l1Z25wP7I3npsV9XfrMPmiz+wFnrqT3X
sTUdl7q/25/2aV5ReO+l/SS3dNbaoB/23XsmTSld/M7+W5h8VvffH/Yn9S4/
St790T4vcsPnRT0/7WcrfEt6o/XFvtO8cGv5/5/2/NtFijjnf7N3Kq09eCb0
l/2cRxknc8V+2ptkc3qW9P+ytxK8lXKq/7f9lYYNW/zX/LJP6Nhvcp7zH5xf
Hy3xYqojgwNMvWvhIWGPWkYHmHkNewJnh21ncoDZl2GafFvqPbMDzD2rZFek
v1dmdYC5d9bP1UkGoWwOMP/8eH7sinUTuwPMv19M+ZiKVnE4wMLj7VTR/aZn
OR1g4VXNEHLkzUsuB1h4MoABjwMsvAELgcMN
      "]], 
     {GrayLevel[0]}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
     StripOnInput -> False], TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 261},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->831840709]
}, Open  ]],

Cell["Using a set of points for the domain and range sets.", "ExampleText",
 CellID->1065231632],

Cell[BoxData[
 RowBox[{
  RowBox[{"ptSet", "=", 
   RowBox[{"N", "@", 
    RowBox[{"Transpose", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"x", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"x", ",", 
           RowBox[{
            SuperscriptBox["x", "2"], "-", "3"}]}], "}"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", 
         RowBox[{"-", "3"}], ",", "3"}], "}"}]}], "]"}], "]"}]}]}], 
  ";"}]], "Input",
 CellLabel->"In[7]:=",
 CellID->1924357632],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionArrowDiagram", "[", 
  RowBox[{
   RowBox[{"ptSet", "[", 
    RowBox[{"[", "1", "]"}], "]"}], ",", 
   RowBox[{"ptSet", "[", 
    RowBox[{"[", "2", "]"}], "]"}], ",", "\"\<OneToOne\>\""}], "]"}]], "Input",
 CellLabel->"In[8]:=",
 CellID->1574997204],

Cell[BoxData[
 GraphicsBox[
  {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
   0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
   {RGBColor[0.0692454, 0.645548, 0.968322], 
    DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
   {GrayLevel[0], PointSize[Medium], Opacity[1], 
    PointBox[{{-1., 0.}, {-0.6666666666666666, 0.}, {-0.3333333333333333, 
     0.}, {0., 0.}, {0.3333333333333333, 0.}, {0.6666666666666666, 0.}, {1., 
     0.}, {2.552786404500042, 1.341640786499874}, {2.701857603000028, 
     0.223606797749979}, {2.850928801500014, -0.447213595499958}, {
     3., -0.670820393249937}, {3.149071198499986, -0.447213595499958}, {
     3.298142396999972, 0.223606797749979}, {3.447213595499958, 
     1.341640786499874}}], 
    {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAA8L8AAAAAAAAAAAbOUpH4
Zey/DTiPjobMwD++L7iZ9pvov+ks45xddtA/Pj6ZyPyj5L94aMfKByzYP725
YTEmgOC/rjhslyqD3z9iGO9aSmXYv+RlunjwO+M/uP5jxwj3zr9eVF0APYPm
PwDZ/8fUHrm/TonRjb2V6T+g+BLlCYqpP49c2s/Qcew/ENa7crDUyT+mejlW
8hXvP6yhvw5n3tY/8uXkr13A8D+Eab7eEoTgPz8kbcxx2PE/1MDo5Bex5T9k
3LHkoNLyPxIN/ouD8+o/L3leOWau8z8eeZS1RSTwP0/yPC5Na/Q/HBTYnK3W
8j+lWhWI8Qj1P2I2oieLj/U/XqLcof+G9T8E7F+UbE34P7RUFpk05fU/YidE
eN0O+z/jxFFxXiP2P4Zv4oRn0v0/utWvLlxB9j9bFWWnSUsAQCJJY+cdP/Y/
0ZFcivSsAUBCTiLMpBz2P9nLfcN4DQNAldOEJwPa9T9FAkhHG2wEQGTyT1Rc
d/U/BrPDVg==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV5b8AAAAAAAAAAHode15f
L+G/caaBmPsitT95eolydNfZvy3XqBPXXsQ/IXwNP1IZ0b9eiO1g2GLNPyBF
d1/3UcC/WjH6iVzM0j/AffBXIZqOP/3vzfUOftY/4IwHQXt2xD+WHb6qjcTZ
P55oiTNepdM/qf4ywRue3D/OJx30My7dP8DkQR82Cd8/Vb2041do4z8YOQkj
SoLgPzWCUnvaQ+g//58VgJRH4T9vzrzyDSftP2C1JHgR1OE/lN6MM60H8T9s
nTuAdifiP1A+kaASffM/vnuTWJdB4j8+TGpLafL1P2z1EyRmIuI/on/rRmNm
+D9/Vq5v88nhPz5c616z1/o/5nOVKW444T+uog/JDUX9P0zyVogjbuA/1obZ
1Cit/z9IUcbD/dbeP6xBSs1eBwFAZLYy5RJi3D9JfGVURDQCQDEO7SjUftk/
wUbyV6VcA0DwwhrNyS7WP+ShyZXkfwRAVAftybVz0j8trNqEZ50FQNntv8Ul
n8w/pDG86g==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV1b8AAAAAAAAAAN14NcRv
I8m/RINkILR+qj8QhSQGt06tv2EnfVm44rg/bM4s7E92tT+omR8B93HBP+LE
b4N8/8w/2TdfCNaexT/BCGlSz7fXP7JTHR7C9cg/WJOodnGA4D+6hGOj9XTL
PzJ1D+YFK+U/BfFVbx0bzT8QMJwfK9npPy5+KINZ580/nH8uZ2WI7j+62viA
PdnNP4tr0jYcm/E/Uk9O5dDwzD/JMT5RFPDzP73WIAOPLss/3PSFQt5B9j9b
lmnCZpPIP9lHU/o+j/g/WnFfIbogxT+bHHq//db6Pxr4onhd2MA/DQfn1eQX
/T9ounoFLXm3P9WK9yLCUP8/0DHZnWxApz/7n/LnM8AAQEDqcov+PX2/NdD2
dNbSAUD0Yogz8tuwvxnC8P+23wJATdyCZAO4wL8iWWTeRuYDQGCOueCow8m/
Tryjv/rlBEDdZC1wDsbRv5/7uvZK3gVABquMQxcG178YVm3Cs84GQNrtv8Ul
n9y/o6nD2A==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQjQ449t+87WZ20N6vOYEj8M5ie6a4e42vAy7ab5Dj
55rqscl+8yRx2Z0yt+xbOb3eXOPaaX+w9/SODzUP7WsfGkpEGO6z59jNZnfj
41P7NZ9yq9bdOWC/SiD64NasV/aSetqvmy4ctK9xf3Ba8+k7e2cGsX/LPx+0
/8dYy561/qO94YRwXm63g/Yacn3Bq159tteZfThPYMJ++03p2uvVlL/Z/7GS
jbnyZZd9yZR/ZRvDf9gbeKtNu7xni73V4nh5+fZf9goijCu2XltpL6Hsetlt
wx/7BRytTQ0qPfY6rw4LMl/6Z3/B6McKu1Nz9+uEPlWZWcrg0OM06ZnRl837
+RMf74yuZXSYUrYz9/afA/vfJhaE7K1ncvhRXVxxde+J/XxWB26b1DA79Gpt
0imdc2E/x86UKJsiFod9YnO5z5pc3f98daJrYCKrw7LoAL0m1Vv7iz8VHPzi
weaQZuff/jX7/n72m4HPVmuwO8Q/0XFd2/RoPzSUHVI++YfElD/dDwCohK/i

      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV1T8AAAAAAAAAAOSYIDPT
J90/BE336M1epj+8qK9aaYziP0iUBzlE77Q/lNCLkw6S5j9pk/t/azq9P1QE
8e+2ouo/bI0hHzsGwj/uZdZDOrzuP+e8jQDjsMQ/ma0s2DVu8T9MN9Msw5vG
P5mVm2WNgPM/ESumLtfFxz/YBxRkCpT1P5f/EN2ALsg/u/m+0ZKn9z/9vF+v
iNXHP5g1tKYMuvk/i2qX2h27xj85j59qXsr7PyLCaTjW38Q/tukZym/X/T/A
gbL3rUTCPxXQZisq4P8/KBlRLQ7WvT982iOhvPEAQLy0E05RqbU/wzRL0SXw
AUB4epdy9w6oPw+BrKjJ6gJAgKChsTm1bj8rFwEqI+EDQPiSr6uJDKe/9wDJ
nq/SBECIkKdusGW5v3gmptzuvgVAu4bATZ5QxL+WdF2JY6UGQFJe2yFqmMy/
MOlZXZOFB0CiyUp468LSvxs6jmQHXwhAzlF6ChSK17/oqZI9TDEJQNntv8Ul
n9y/G/2/Aw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV5T8AAAAAAAAAANk3wLYr
iug/NXAUYRUDsT/gJN9c79brPxhEM/Nia8A/fq+FDOA57z9/6KneVbnHP3oq
JuOYWPE/4Q15QYNnzj+MwlHdhh3zPzRZPtgvOdI/ernRocnq9D8s0C00W+vU
PywQCnJsv/Y/ujiDgNVI1z+YKZOldpr4Py5mKnldUNk/itIpLux6+j95VHl0
3wDbPzzVbB3OX/w/PFt09XVZ3D8eIiEsG0j+PytWkiVqWd0/J2RaIWgZAEBE
JsA1NADeP2gnXIF0DwFAuRJwpntN3j+AvoIosAUCQCLNjnYXQd4/TiIlcJj7
AkBsLkU5DtvdP2wl192q8ANA3iF7EpYb3T+4v6RoZeQEQC6XHZoUA9w/8SQR
vkbWBUB+ujamHpLaP2wKtYbOxQZA+gv1+3fJ2D80s1eqfbIHQEI+y+cSqtY/
iqBek9abCEBm+d+8DzXUP98ecHFdgQlAxLkRPLxr0T/SUyV7mGIKQNrtv8Ul
n8w/NjOsRA==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAA8D8AAAAAAAAAAPwYQwcs
GPE/vZ76yTNpuz+20b7jA0TyPy79FQmN/8o/DLO8fOiC8z+OsBTp5ezTP82/
7p0w1PQ/DLmPOUIe2j9ADDdRKTf2P8q0jiJJCOA/9KucPRar9z9MGnEnV+Di
P3WTOwsyL/k/2rRN3ciV5T84Z/rLrsL6P5+9SFMuJ+g/kKjOaLZk/D9cpAa5
KpPqP5pGVRNrFP4/UKDAF3XY7D8YTYO759D/Pwz50ADZ9e4/gB+YRKDMAECr
NK2YG3XwP6PPnKxBtgFAZONZFUNa8T8eof4e3KQCQCqHo1/pKfI/yh1zAfGX
A0Ati8xJoOPyPyROY1n/jgRAJLiaSQWH8z+SW1sQhIkFQKs3oKzBE/Q/+qWY
OfqGBkBW7DvGion0P+RkkVjbhgdA0rU4FyLo9D+7Z1Con4gIQCeb9m5VL/U/
fRh/Y76LCUCuQgwG/171P0+H9wyujwpAf5ZSkgV39T+7/be45JMLQGTyT1Rc
d/U/x9i2MA==
      "]], 
     {GrayLevel[0]}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
     StripOnInput -> False], TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 259},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->628254193]
}, Open  ]],

Cell["Plot an inverse function.", "ExampleText",
 CellID->1149581062],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionArrowDiagram", "[", 
  RowBox[{"5", ",", "5", ",", "\"\<Inverse\>\""}], "]"}]], "Input",
 CellLabel->"In[9]:=",
 CellID->1165752543],

Cell[BoxData[
 GraphicsBox[
  {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
   0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
   {RGBColor[0.0692454, 0.645548, 0.968322], 
    DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
   {GrayLevel[0], PointSize[Medium], Opacity[1], 
    PointBox[{{0.4, 0.34641016151377546`}, {
     0.7000000000000001, -0.5196152422706632}, {-0.9, -0.17320508075688773`}, \
{-0.6000000000000001, 0.34641016151377546`}, {
     0.30000000000000004`, -0.8660254037844386}, {
     3.8, -0.34641016151377546`}, {2.6, -0.6928203230275509}, {
     2.9, -1.2124355652982142`}, {3.1, 0.8660254037844386}, {3.2, 
     1.3856406460551018`}}], 
    {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAMzMzMzMzB0DeaNfVImbzv6xS0cAL
TwZAvsEGU0UW9L+U7KKyHmcFQEZKQ010sfS/txLyF+d7BEC53XNtXTf1v90W
F7/hjQNAYrkPprmn9b/Stz/zjJ0CQELW0VhNAva/hDxsOWirAUCSDlt26Eb2
v5ELxgz0twBAKEayl2Z19r8OTOc0Y4f/P/ENlRGvjfa/mGIc+kSe/T/5go4B
tY/2v9Lcue+Qtfs/zXbNVHd79r8gN/ZjSs75P+ZAtsgAUfa/uJAZ43Pp9z9q
6y/lZxD2v2o6Tq4OCPY/ncGv8M659b8M9iAzGiv0P6OXCd5jTfW/wFP5g5NT
8j8Jcg40YMv0v2gW0tF0gvA/uXwG7wg09L8avvHOaXHtP1qDFlyuh/O/iB43
TY3u6T9OV6Tuq8byv0i70Q0xfuY/q77QD2jx8b/I+hAvKCLjP26rIOhTCPG/
APK3DHa43z/4j3Ij6wvwv2BlIFxMXNk/1li/Xmf57b84MzMzMzPTP6hMWOh6
tuu/jtzJOA==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAzczMzMzMBEAiCq2GlSvmv3Kvqp+m
KQRAO+UGBypH579cLT2ucYMDQD4kfWm/ROi/UYLyKIbaAkBezPQgzyPpv2S6
w7A9LwJAxbc/0uLj6b8LHaYn84EBQE5N6ZKUhOq/okdSgALTAEDvEkofjwXr
v7SNeo3IIgBAgWnFB45m67+qAxWhReP+P6BwGdVdp+u/gJITkN5//T9Ezq0j
3Mfrv8EaeXsYHPw/qtzStffH679yAzgpsLj6P0aQ53ywp+u/rxODLWJW+T/L
OmGZF2frvxIkvobq9fc/eie1UU8G678GTb45BJj2P+7iKACLheq/cn+O7mg9
9T+51JL3DuXpv5Iu7Y3Q5vM/FKAZXzAl6b/6T7ff8JTyP7iHBQVVRui/k3h0
Kn1I8T9czrso80jnvxpEN9QlAvA/Eb0APJEt5r/4/wgKMIXtPxaaoZvF9OS/
nobllfoU6z/zaaw/Np/jv1BnZoD2tOg/689fY5gt4r9oZmZmZmbmP5vHASWw
oOC/P3/JZA==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAZmZmZmZmDkAiCq2GlSvWv552iXE/
3gxAaCqYT5FN3L8D2TPlkU0LQHRS+kWR7+C/HAzUWjK1CUC4BRNqqm7jvzNA
H4H5FQhA7Y85+cCi5b9Qohipw3AGQN6tcKWpiue/uAgrUXDGBED0BmqMYSXp
v/wHk67hFwNAeEPjwA5y6r9OXFg1/GUBQHrHR74AcOu/b60sPkxj/z/xt1nG
sB7sv/4+p+GN9/s/q5ewKMJ97L+GPVQAjor4PwON53MCjey/LHMA8B0e9T+I
N2GQaUzsv2iZKboOtPE/euKSxBm8679Yl31KYJzsP5rM1KJf3Oq/JMgYfZ/c
5T+QK8Hgsa3pv1hOV5XfWN4/B3o3GLEw6L+QnX1eux7RP7l7JXInZua/4HmQ
ElxHsD/mLkM7CE/kv3BvMZfBjcG/MpX6Ym/s4b8g6JMUm2TVv9Cmf8lBf96/
2PTBElPg4L9CMFM4EJTYvzCC4ZM56ua/tv9JCG4a0r/QzMzMzMzsvwIKrYaV
K8a/SlLIAg==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAzczMzMzMCECqTFjoerbrP29iG5Fz
tQdAeJB24uh16D++Z8Y465QGQPl3plMPaeU/xpso3MxrBUDQgV6NjJHiP5bB
4CG2OgRACfC7J8Xh3z/6YCvrSAIDQPkBtqLsENs/uRf+/SrDAUDmV0OxHbPW
P7AOEq0FfgBAsOkQb6nK0j/BfBj8Cmf+P7BSWWZFs84/dl3mm7PI+z9IcLfs
ucPIP4iIRehoIvk/sEYgttbJwz87897HknX2P9gQB15/kL8/zS1WmpzD8z9A
DW8JKoK5P2ghe3f0DfE/4JB3CeRrtT/A1BTaFKzsP5jQ+ITYT7M/pMDGdp86
5z+wBxQIJi+zPzzWNiNsyuE/cMQo7N0JtT/wZFolu7zYP2hSoE4E37g/0GNs
FlHlyz8gIZSWkKy+P0ASFy3Q4qk/kB+ERLc3wz/g73VqBYK9v8irk/W/Ecg/
sAy3BUje0b9UJNFYz+HNP2BYqOloN9y/BqEC72dS0j84MzMzMzPjvyAKrYaV
K9Y/Pp/BVQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZCUAiCq2GlSv2P5EexUUf
4whAy+uYN+OG9D/SMQZvQiMIQCRk++I88/I/FXa24mhaB0COku61eHHxP7Gj
8DL9iAZAyG9iYGMC8D9+rAR+bq8FQOHjAlF/Te0/byWCMzDOBEA4da8GjL7q
P4o++Na55QNA6knnZUhZ6D8IyYvAhvYCQHREZ9T5HuY/ZPGE2xUBAkDon+Ts
zhDkPxtn92LpBQFAJSx93t4v4j+RraichgUAQErLlNgofeA/i4CyJOsA/j/M
lN0GJ/PdP08fLJOB7vs/CNqVC9lL2z+ARnH06dT5P9A3ji/QBdk/vMF5h0G1
9z8QOn9GQSLXP4UtKMOokPU/rCLB4yyi1T8e4Dq9QmjzP9hjKNJehtQ/gN3Y
jzQ98T9k1OOnbc/TP3yTGXxJIe4/oP6Vdrp90z8ovQAvdcfpP8Tu05dwkdM/
PJ2WNztv5T/81CSWhQrUPxR0UbvpGuE/WK+OMrno1D+YmZmZmZnZPyQKrYaV
K9Y/eP2/bg==
      "]], 
     {GrayLevel[0]}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"\\!\\(\\*SuperscriptBox[\\(\[ScriptF]\\), \\(-1\\)]\\)(\[ScriptX]) \
\[LeftTeeArrow] \[ScriptX]\"", 18, Bold, StripOnInput -> False], 
    TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 261},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->2038613099]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Options",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(6)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "ArrowStyle",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection", "ExampleSubsection",
 WholeCellGroupOpener->True],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->600017015],

Cell["\<\
Arrows in the diagram can be styled using standard graphics directives.\
\>", "ExampleText",
 CellID->1731213334],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionArrowDiagram", "[", 
  RowBox[{"5", ",", "3", ",", "\"\<Onto\>\"", ",", 
   RowBox[{"ArrowStyle", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"Red", ",", "Dashed"}], "}"}]}]}], "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->246571335],

Cell[BoxData[
 GraphicsBox[
  {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
   0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
   {RGBColor[0.0692454, 0.645548, 0.968322], 
    DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
   {GrayLevel[0], PointSize[Medium], Opacity[1], 
    PointBox[{{0.4, 0.34641016151377546`}, {
     0.7000000000000001, -0.5196152422706632}, {-0.9, -0.17320508075688773`}, \
{-0.6000000000000001, 0.34641016151377546`}, {
     0.30000000000000004`, -0.8660254037844386}, {
     3.8, -0.34641016151377546`}, {2.6, -0.6928203230275509}, {
     2.9, -1.2124355652982142`}}], 
    {RGBColor[1, 0, 0], Dashing[{Small, Small}], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz0z+qTFjoerbrv2s3QsZu
VNo/6pr57TVW6r8WtNqb68ngP9qJgPnXH+m/iRueBsp25D/zYh+5BRTov/IK
CmrfLug/o6vnR00z578dh2eZMvDrP+FxY+Ilfua/V3S1gMW47z9AAVGn7/Tl
vxAcpxlLw/E/FpKiZPOX5b8m50v+z6vzP4IC3XBiZ+W/m9AWOe6U9T9zGOqQ
VmPlv2FWeUOiffc/yjBs6tGL5b8T/DzP6GT5P5mcmgK/4OW/e6IZUL9J+z+Q
R6fJ8GHmv8n45IQkK/0/K5unsiIP578nPRIAGQj/Px7v89f45+e/uu+c189v
AEBSOuorAOzov2aOsDBfWAFA8iT6ta4a6r8TKt0lPz0CQLAX2ttjc+u/+NFL
RvYdA0DHb762aPXsv8gqJVYN+gNADqFldPCf7r/oWtWND9EEQMTj4mEMOfC/
WpsC2IqiBUA6/5AmdTXxv+6MFQ4QbgZAx+KjmqxE8r8yMzMzMzMHQN5o19Ui
ZvO/xVTNBQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAZ2ZmZmZm5j+axwElsKDgv8rkneow
suk/bt4xgMDy37/B6FINuQTtP/66jsBI8d6/tG72wR0u8D8Z7tCngT3evyN6
1S952/E/Sa7ImcrX3b8CrD//ionzP9Z9wo5ZwN2/tKto/m439T92QOT2Ovfd
v+wg0xNB5PY/srmTs1F83r8SPFi3HY/4P7L06SZXT9+/0iDiaiI3+j9EsZWs
7TfgvxAkmTJu2/s/zGYYmqLu4L8iGUQMInv9P/zNUGppy+G/LFecZWEV/z/q
I2r4zM3iv75Wq0ipVABAjDuPKkT147/zllGdDxsBQJheqToyQeW/KkJza3rd
AUD2aY8J57DmvwBCuIqCmwJAkAJ5fJ9D6L8TUnomw1QDQBlNhOSF+Om/FitF
89kIBEC2Ohdwss7rv5I3/mJntwRA+kHgpSvF7b/VJZjXDmAFQDNANenm2u+/
KnA31HYCBkBYI8WDZAfxvyDIrSxJngZAPqJaZ9Mv8r8zMzMzMzMHQN5o19Ui
ZvO/Q+3JVw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAzczMzMzM7L8iCq2GlSvGv+IBWv67
9ue/ciB3DPoQvr/CGn9vKQ3jv0Z2U21fkbG/bUoVyWAl3L8AnK2T83ubvy5c
2DLqE9K/IJGxhQQEgD/AxSjXGq2/vxRy12bHEKI/MGejRC55oj/QHez84mms
PwRh4eXbK8k/7PQ3I2uDsT/Kl1PlnePWP3KzHn4P8rI/szflCF+Z4D/sKPIE
HICyP4eCgBPrvuU/fMwNLs0tsD8Dw6H7t9/qP7DCl6C8+Kc/e+UuLw357z8w
d8/uI7iXP+abMwkbhPI/gACzS8Off782X7G3QQX1PyjOB5/+bqe/XCpacaZ+
9z9A6n9GVUW3v5SPNiL57vk/sHZxVq5Owr+s4GCG7lT8PyZ/+3tA3Mm/Ag7K
2UCv/j9uCzaQryPRvwHDiUJYfgBAt1NMYcnF1b8xL5FjAp4BQHoHLln40dq/
deaILQa2AkCunRxrxyLgv3hxFg3PxQNABCSELNQO47/OzMzMzMwEQCIKrYaV
K+a/zqLFAA==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz478iCq2GlSvWP14nrMl0
xNy/Ob2uibT91z8w2qpE2g/TvzjgulsbX9k/IJ7qVnqbwr9WNxV5Dk/aP4Bk
z7BALX8/JOeRkA7N2j+4rPu1lJTEP+xcI8fY2No/MAoRSM8V1D/kiVPbZnLa
P5BHNE0R2t0/C5+VKO+Z2T/UOfrj8MjjP8OGborkT9g/8JMOWAyc6D/QaYMf
9pTWPwiMEKrLY+0/sZWu7A5q1D+PfyPE0g7xP0cSTGBV0NE/hcJGAIxj8z8A
IQBsVZLNP+gNeerUrvU/3i6UdlSsxj+4Rdrlde/3P0SHIdWh5L0/SJ36/Dwk
+j9Qv/lAkaGpPxj/MYT+S/w/oCF990tnl7/WTqG4lWX+P9jyi39JEbq/ppPE
rfI3AEDsLP1lcOXHvxHU0STsNAFAXtlhesS+0b91ghgHMSkCQB4k0cZj5de/
mme0uD8UA0Ba2w2aUmPev7QzAoGb9QNATSETkY+a4r/OzMzMzMwEQCEKrYaV
K+a/9anGFg==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ2T8iCq2GlSvWP+uzjfAq
oeE/zMMoXf0n2T8zMPVlpoXmPyjBsr0Os9s/wBUhnKZ36z++4IwzcMvdP02+
9OpFOvA/iF9RJAVw3z9+pyZH2LzyP065ejP3T+A/NAb4bzVC9T+f8hhdRa3g
P0ABdfcGyfc/+3MJjbvP4D/OgBWq9U/6PyXVQHpHt+A/bF7XRKrV/D9y71Ue
9mPgP2LLYCvOWP8/Gt4+Xeer3z+xJOUOBuwAQMi1fQkXG94/RODXdggpAkDt
CUvtTxbcP+JPFBnGYgNAmDRRPKSe2T/0xCN9mJgEQDiEhCBjtdY/mWzhPtvJ
BUAOy0wIGFzTP6MetGXs9QZADjlpqRMpzz+gcX+6LBwIQAYXG9BzwcY/JXoe
HAA8CUAY7CVWjAu7P0liO9LNVApA0J2YJYfXmz+M5FfeAGYLQICMZbjfZq2/
GqvbSghvDEB4IqYhdfLCvySh/ndXbw1AhM0fgB9Jz79mZmZmZmYOQCQKrYaV
K9a/j760Dw==
      "]], 
     {GrayLevel[0]}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
     StripOnInput -> False], TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 259},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->1813993131]
}, Open  ]],

Cell["Arrowheads can be included in the style specifications.", "ExampleText",
 CellID->54562559],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionArrowDiagram", "[", 
  RowBox[{"6", ",", "5", ",", "\"\<Constant\>\"", ",", 
   RowBox[{"ArrowStyle", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"Purple", ",", "DotDashed", ",", 
      RowBox[{"Arrowheads", "[", "Small", "]"}]}], "}"}]}]}], "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->57411844],

Cell[BoxData[
 GraphicsBox[
  {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
   0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
   {RGBColor[0.0692454, 0.645548, 0.968322], 
    DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
   {GrayLevel[0], PointSize[Medium], Opacity[1], 
    PointBox[{{-0.6000000000000001, -0.34641016151377546`}, {
      0.8, -0.34641016151377546`}, {
      0, -1.3856406460551018`}, {-0.7000000000000001, -0.8660254037844386}, \
{-0.30000000000000004`, 1.2124355652982142`}, {0, -0.6928203230275509}, {2.7, 
      0.8660254037844386}, {3.4, 0.}, {2.8, -1.3856406460551018`}, {2.5, 
      0.17320508075688773`}, {3.3, 1.2124355652982142`}}], 
    {RGBColor[0.5, 0, 0.5], Dashing[{0, Small, Small, Small}], Arrowheads[
     Small], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAAAAAiCq2GlSv2v18ZF/27
1bA/cGrg6cBM9L8kan722YHBPyOM8NOtevK/EwS9NSFAyz+2WVeRU7bwv88m
ByNE0NI/Y6JPTUQB7r+2A6vVxk7YP2QpZ6IDteq/qui5Z64Y3j/KE6IXpYnn
vzKNjkZ0FeI/TNUGMNeA5L9tXPZMHkHlPzarhxc2nOG/zdMAnKaN6D885J+Q
lbrdv/fl/RdN+es/2HSBdBSL2L+UEGUiQYLvP4h9FjqprNO/mhQuSFGT8T+w
9Uh10kPOvyYPv1RBcvM/6G7V1Hraxb+UPi2X8lz1P/BgHUWEQ7y/gDoRs2BS
9z9AAcPcBHesv+ofepmBUfk/ADdg/iBGer/EKxkWRln7P4ASbUxd9qI/wFMo
X5po/T8waZLJSB2zPwWDv6dmfv8/4G851888uz/kaiXax8wAQKhE+DS66sA/
4IZpuHvcAUCg0dVB23HDPwhBZMS+7QJACKx7tHMyxT8AAAAAAAAEQCgKrYaV
K8Y/71u8Ew==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAZ2ZmZmZm5r+qTFjoerbrvwUCPDeh
CeO/1WO+8nQl6L8yowksZAjfvzUwsq6OvOS/BHoqn+6v17+W1BNFl33hv+QG
9YzHDdC/bl64PY/U3L8cBxap90vAv4QbVfaCCNe/ADDyorgVKj/GgTkjHZrR
v2w6oxE/08A/WK6cBX8Yyb98JK+lpQjRP5A03Wxihr+/woIwAsLb2T8AGnFw
EuWsvwGx2vcib+E/AJjk/N0XcD/HRz/MtAXmP4BbTJX/mq0/s82FRCev6j/Y
NNZzKuq6P8lAvCIBae8/QCGwUoiowj9wHF86YBjyP4DJWbhW/sY/fklp9O2B
9D9oAzCOM3TKP4JUGRPi7/Y/8DN+ykgIzT/cI2KU8mD5P6gJ+i84uc4/0vCm
z9PT+z+4r5kHHIbPPwSXtSU5R/4/KMR2moduzz8W3aHY6lwAQJBkfWuHcs4/
eY7Iey6VAUBQtsgwoZLMPx+4aMzBywJAUHCwjNPPyT8BAAAAAAAEQCAKrYaV
K8Y/gIC+Sw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAAAAAiCq2GlSvmv7xhnqVF
trQ/KyUcyZNQ47+NVnlMnDjFP50e6ZB+lOC/j8P4SVRJ0D+Sk2OpkvHbvyrG
d9ugMdY/z0/2uqz91r/i1AtuEVLcP9tMaxXsT9K/lqqg5bJT4T8o4Ql/mNXL
v4eYONYgl+Q/fPVibUShw784g5tBl/HnP7hyH8lsDri/7Yjsp05h6z8gZvne
2y2kv+sdVEJ05O4/gEFNVFTkhD+m81V9lTzxP0AUf9V+Cqw/939RNMYO8z9g
5bz3Ih24Px4r6unU5/Q/2PuWF21vwD8kGSOdxsb2PyAJpM7fIsQ/581jLZ2q
+D+4dPTp8ibHP5QlT+FXkvo/GMcIwAx6yT/lBvvu83z8Pyz/UI7xGss/DIhA
BW1p/j9AUbcgxAjMP27V7epeKwBAjPb+RgZDzD8VswhQ8CEBQKDGuBeZycs/
qDXJ3ecXAkCAE6oAvZzKP0t7qBHDDANAsBWdpBG9yD8AAAAAAAAEQCAKrYaV
K8Y/LsW/FQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz478iCq2GlSvWvzJ8ZCgw
NN+/MJY+gVWB0L/9Hm+44MHXv1KkI2jeWMa/JLASk2sT0L+2PEBY2764v0hd
vD3IWcC/ULHRRh3dmL8AYJjCgntSv1ANT6ZUtqU/POetJgNvwD9oxDWiam+6
PzYwkjatrNA/sELvuJo+xD/k6YmQikjZPwzG8uKKfso/rDpxIUQD4T8EBini
NPTPP8x64GkBceU/quKTjllO0j98HO+RI+vpP8ZuymvGOtQ/PG5zdkpv7j+q
b/DAW77VPwp/tVGIffE/zCFG50vY1j84Ab1MBsbzPwIHMkYBiNc/1g7ICukP
9j9yCqGiHs3XP9hRAq35Wfg/7Fl+UH+n1z/O4D08AaP6P2i0KEY3F9c/Dr7y
Tcnp/D9c29oRkxzWP0q286gcLf8/LMUMsRe41D/vnMD047UAQIwe40mC6tI/
6ZAxEk3SAUCii9LGx7TQP1VrWcQy6wJAdNNXqSgwzD8AAAAAAAAEQCQKrYaV
K8Y/sEy7Iw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ6T8iCq2GlSvWv9rpJCmz
aOs/Okta5b520r9tYU+r6UztP+IKy6cX2s2/IFXTMzxF7z8ymwnFuSDHv/TG
cJVPqPA/oPB6ffXEwL/7q45t/rbxP+APUARVlLW/y0p7AhvO8j/wP0g/FM6k
vybTiTsR7fM/ACDby//hH7/BIb3VSBP1P8A4pCe+BqM/nPeRtCVA9j/8UEgs
7DWyP1Qc1DQIc/c/UIp+YRELuj8YeFKBTav4P+jZ52JTf8A/rDNF6U/o+T+w
2zMNfobDP4QGODhnKfs/bMMjYm0Zxj9aDUoP6W38P3wwX8DDNsg/9c2TPym1
/T8MXTjsYd3JPypxgyV6/v4/wPUTqGcMyz+pXH+CliQAQJx2ZCs0w8s/MekM
M0nKAEBw6/l3ZgHMPwzlGDn9bwFA5M94jd3Gyz/WpJyoWhUCQEDA3Hq4E8s/
4geDwwm6AkC0sP1NVujJP4zUNiizXQNAYGkg4VVFyD8AAAAAAAAEQCQKrYaV
K8Y/fDW6jQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz07/eaNfVImbzP0/JU6zy
LcW/kf/e6tm28z+gJfnZZeWevyd2+dFm7vM/ALbSRHYIuz+1r40RrAz0P0y0
zFaM684/vivdmZkR9D8sqZZyBSjYP062ic0s/fM/01kPFB9q4D9wkviCcM/z
P6H06R/ruuQ/u2KS/nyI8z8QHY2oHATpP+Lc4uV3KPM/mCZ8gm1D7T9yHZ8r
lK/yPxrs+F9Ou/A/5DSd9BEe8j9WTX/xt83yPzNGy3U+dPE/7iceBtrX9D/B
RTjLc7LwPwGbu5Kf2PY/piqIkDGy7z+Ur9aD+M74P+S2IIJB0e0/217kTdq5
+j9YEp6aFsPrP91JvXpAmPw/4gweCciI6T+nF9EzLWn+P5ThVWiEI+c/4hJr
5NQVAEDscgIekZTkPxYKV5lj7wBAPHdArknd4T963BxJz8ABQF5jUAY+/t0/
OJhs1aiJAkAKHR1SLffXP/aEK6yFSQNAbv6m/5Oo0T8AAAAAAAAEQCAKrYaV
K8Y/ZH/DZA==
      "]], 
     {GrayLevel[0]}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
     StripOnInput -> False], TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 259},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->2104699905]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "FrameLabel",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection", "ExampleSubsection",
 WholeCellGroupOpener->True],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->2065565702],

Cell["Change the labeling for the function.", "ExampleText",
 CellID->1889599759],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionArrowDiagram", "[", 
  RowBox[{"5", ",", "9", ",", "\"\<OneToOne\>\"", ",", 
   RowBox[{"FrameLabel", "\[Rule]", " ", 
    RowBox[{"Style", "[", 
     RowBox[{
     "\"\<x \[RightTeeArrow] \[ScriptG](x)\>\"", ",", "18", ",", "Bold", ",", 
      "Italic"}], "]"}]}]}], "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->1588449004],

Cell[BoxData[
 GraphicsBox[
  {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
   0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
   {RGBColor[0.0692454, 0.645548, 0.968322], 
    DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
   {GrayLevel[0], PointSize[Medium], Opacity[1], 
    PointBox[{{0.5, -1.2124355652982142`}, {-0.5000000000000001, 
      0.17320508075688773`}, {-0.9, -0.17320508075688773`}, {
      0, -1.0392304845413265`}, {0, 1.3856406460551018`}, {
      2.5, -0.8660254037844386}, {2.2, 0.6928203230275509}, {
      3.6, -1.0392304845413265`}, {3.2, 0.34641016151377546`}, {
      3.8, -0.34641016151377546`}, {2.6, 0.34641016151377546`}, {
      2.6, -0.34641016151377546`}, {3.7, 0.8660254037844386}, {2.9, 
      1.2124355652982142`}}], 
    {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAA4D/eaNfVImbzv6Fpibkj
1+M/feJ9szw08r9DAbshG8nnP/7eHV0HGfG/hGtwVc7T6z9AMtwXGRXwv7pD
4FEY9e8/CCL7nfdR7r+oDfULZBXyP2wfcZVZquy/EkTg6lA59D9uS3jbODTr
v32oKAswZfY/GvUb81vw6b/HXeV62pf4P8pBwLNu3+i/OCRNrSXQ+j9eYvTt
AQLov/5kHxnkDP0/ruibHotY57+H3aLY5Uz/P1YCmDBk4+a/MoNypXzHAEDa
rBFNy6Lmv1BT8tr16AFAfjN+uuKW5r8NxKj0RAoDQAl3b8qwv+a/EXnFcNAq
BEDipTnWHx3nvyQ3QzX/SQVArCtwSv6u57/9MyThOGcGQDjAM8H+dOi/5zhM
HeaBB0B6oEMruG7pv+mxzexwmQhAVCe8B6ab6r/VDX/8RK0JQDcyZaoo++u/
GTWu8c+8CkCwA2uQhYztv0NVyLeBxwtAqa9Uw+dO77/NzMzMzMwMQJrHASWw
oPC/SBLI/Q==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAAAACaxwElsKDwv08kcc8f
tLY/z1WYFSCN7r9UBwyIJC/HP7AznLOo+uu/mKFhU3u80T/AdMKFV4vpv6jz
N4yAGNg/tBZOR3dA579FoQobQqjeP6pplV0/G+W/Xkl2XCK04j/WHcgy0xzj
v4qNNcJ5KuY/UGlPm0FG4b+Begb0ULXpP1KiNIwIMd+/7XeyxsZS7T/GqkRw
/ijcv6Df6xZ4gPA/lpbdpgB22b89Djug7F7yP/gvrtN9Gde/pbQXIcND9D8c
hHu3thTVvwiCV1j6LfY/FO3vhb1o07+DqFQqjhz4PxxQHVR1FtK/jTXxKngO
+j8Yw/+fkR7Rv3/H1CiwAvw/NIdA8ZWB0L+qvpq5LPj9P/Tha5PVP9C/s4im
xuPt/z++275pc1nQv4GmKY1l8QBAvFek3GHO0L84CJp27OoBQDZX6+BintG/
y66MOQLjAkD+lLIYCMnSvyTPyDMj2QNAEgH4DbNN1L/MzMzMzMwEQCYKrYaV
K9a/DpLHbQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAzczMzMzM7L8iCq2GlSvGv3Rf883V
sOi/TOMSPCBxs79oM71PsnHkvxB7Snkueo8/WlbeJ6MR4L/kHlbDn725PzZL
gUf1Jde/9nZu0S33xj9g1rY0beDLv8WDzbmwHdA/aNfaMrUbsr+gzwUMQFPU
PyA6yg+hQ7Q/29PKHgka2D/URsgOZqTNP6OUY/gKcNs/BlAH7sW32D9vlSJ2
gFPeP58u/1MwXuE/mGFqnnBh4D/vgDc3WW3mP1RDeERxXuE/Z6PEaq6G6z/E
QwvvOyDiP/Dxcqm9U/A/KoHDymmm4j9el8Cug+byP7gMoKWz8OI/RE3euUt6
9T8InsUU8v7iPxQHd7a3Dfg/OMZoiR3R4j/8Fw7BaZ/6P3aJ0FROZ+I/kr+l
4AQu/T+8Pm+bvMHhPz5K6b8tuP8/3o0YN8Dg4D/pQryyRR4BQHO1yBChid8/
S2B59eNcAkCjsa5uCN3cPxQaMJlIlwNAF8flxiG92T/OzMzMzMwEQCMKrYaV
K9Y/IfrHxg==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAQAAAAAA4L8iCq2GlSvGP0gGr4GB
09m/t6DhGcgw0D/zozCeKW3TvxscbDF7AtU/vgBrRrugyb8xbmRwVYjZP2h7
Hwx/Ari/oaLIe/C/3T9AtS0BWc6PP+opkL6H0+A/lKAvTUdSwD9skzcp0J3i
P0AKyrhK984/jkG/U9495D+W4+JMFfLWP9ai9HvVsuU/5M3wyn2I3j9uQrC8
7/vmP5dgWxlsHeM/NovWdn4Y6D+rUUCGhwLnP5Y1/63qB+k/q9K7G4Dx6j8I
NpJYtcnpP+exiY0/6O4/xqovo3dd6j/kN4a3VXLxP24EPyfjwuo/OJ1+KFNy
8z+MdogUwvnqP25pWnAIc/U/2J3CTfcB6z9ydjqEZXP3PxQ1BXh+2+o/XIkM
iFpy+T/Ephj9a4bqPyjvyl7Ybvs/ckGhAO0C6j++ulY60Wf9P/bNK0hHUek/
3k+gKjlc/z/yPCgW2XHoP/8i6VWDpQBAUhDm9xhl5z+bmZmZmZkBQCQKrYaV
K+Y/BIrBlQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAAAAAiCq2GlSv2PzkQLSHz
nb4/3g4odOkS9z/gj8FlHu7OP8ab6Ca04/c/d1wpR5Jq1z9lVgrWhp34P9DL
sPrNfd8/4gOX6P4/+T+Btxu3PNbjP95d1ynGyvk/0wDZmR755z9ythD3kj36
P0NyAc5aJew/OCaXZiiY+j/h+cxCXSzwPzmJH2hW2vo/xA99lIFI8j+MJEDe
+QP7P57fxsL7ZfQ/jmwSsfwU+z+yE/x+rIP2P/QI7NlVDfs/YQh8XXSg+D83
3ihoCe36P8MUNW40u/o/9pEDfyi0+j/pQ+TUztL8P+GufUzRYvo/NrbCYCfm
/j9FO1z5Lvn5PwsYqBESegBAgUFBknl3+T/GafUC130BQNZw7un13fg/ZVLy
rth9AkBlnsB09Sz4P64jZD+PeQNAiIZ3HdZk9z+8qz8ldXAEQEG9YBMChvY/
mfGGXwdiBUCgPgCS75D1P9Lny8DFTQZANoRUoiCG9D8zMzMzMzMHQN5o19Ui
ZvM/UV2/cA==
      "]], 
     {GrayLevel[0]}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"x \[RightTeeArrow] \[ScriptG](x)\"", 18, Bold, Italic, StripOnInput -> 
     False], TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 263},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->2016250655]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "PointLabels",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection", "ExampleSubsection",
 WholeCellGroupOpener->True],

Cell[TextData[{
 "With PointLabels names can be assigned to the points. Automatic uses ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["a", "TI"], 
    StyleBox["1", "TR"]], ",", " ", 
   SubscriptBox[
    StyleBox["b", "TI"], 
    StyleBox["1", "TR"]], ",", 
   StyleBox["\[Ellipsis]", "TR"]}]], "InlineFormula"],
 " for the points and \[ScriptF] for the function."
}], "ExampleText",
 CellID->2089459734],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionArrowDiagram", "[", 
  RowBox[{"5", ",", "8", ",", "\"\<OneToOne\>\"", ",", 
   RowBox[{"PointLabels", "\[Rule]", " ", "Automatic"}]}], "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1897153594],

Cell[BoxData[
 GraphicsBox[
  {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
   0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
   {RGBColor[0.0692454, 0.645548, 0.968322], 
    DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
   {GrayLevel[0], PointSize[Medium], Opacity[1], 
    PointBox[{{-0.4, -0.6928203230275509}, {-0.7000000000000001, 
      0.5196152422706632}, {-0.30000000000000004`, 0.8660254037844386}, {
      0, -1.3856406460551018`}, {0.1, -0.17320508075688773`}, {3.6, 
      0.34641016151377546`}, {3., -0.34641016151377546`}, {
      3.1, -1.2124355652982142`}, {3.2, 1.0392304845413265`}, {2.7, 
      0.17320508075688773`}, {2.3, -0.17320508075688773`}, {
      3.8, -0.6928203230275509}, {2.3, 0.5196152422706632}}], 
    {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAAAAAiCq2GlSv2vwdNS8wd
ub4/wYNTZK/59L8MBeyGbCTPP0KA8w163vO/B9fgqpyn1z+E07HIi9ryv3OH
wKMw6t8/SDLTf27u8b9QG+oXyCrkP/owjnufGvG/I4jA1aFy6D/7xpEej1/w
v/pQURZgyuw/ojfHVEF777/HXeV62pfwP1KEaxVUau6/OCRNrSXQ8j/mpJ9P
54ztv/5kHxnkDPU/NitHgHDj7L+H3aLY5Uz3P95EQ5JJbuy/ZQblSvmO+T9i
77yusC3sv6Gm5LXr0fs/BnYpHMgh7L8aiFHpiRT+P5G5GiyWSuy/EXnFcNAq
AEBq6OQ3BajsvyQ3QzX/SQFANG4brOM57b/9MyThOGcCQMAC3yLk/+2/5zhM
HeaBA0AC4+6Mnfnuv+mxzexwmQRA7rSztEUT8L/VDX/8RK0FQGA6CAYHw/C/
GTWu8c+8BkAcIwt5tYvxv0NVyLeBxwdAGPl/kuZs8r/NzMzMzMwIQN5o19Ui
ZvO/WZXMrA==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ2b8iCq2GlSvmv+GfSBsb
bdO/T3xn2xae478Xe5Rvhg3Kv50+ok89NeG/4Gc889ynub8lK0xgoOTdv0Dj
odeocXk/tfbnVAWt2b9AUL+5ZJO9P4FFkFPmxdW/YG38GRQfzT98ckF+VTHS
v4Zry8IL4tU/dCxkUnLizb/8SUmze1jdP1CnjrGVDsi/JpqrGHJ34j/4KKCu
LOrCv8uTjkyfUOY/qAsOjOPvvL/dhHO5ujXqP6i4yNKBdLW/3QXvTrMk7j8g
amH8WMyuv45yXmC5DfE/QB6KUjSQpb980R9R7wvzP8AHKSP4cp6/0DYYwuwL
9T/Aw/p5GpeXvwYD9AmiDPc/YNq1UnOQlr8MENQd/wz5P+DyYgqOX5u/9CKm
IfQL+z+AXvqz7gCjv8CIZPhxCP0/oLNwe944q79WVPDTagH/P6D1YwGdKbO/
u/QcYul6AEDAfYCRDiW6v8vvtSJQcgFAaPHIwYdFwb9nZmZmZmYCQBwKrYaV
K8a/xp3LUg==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZuT8iCq2GlSvGvwwxpx7/
Hsk/1bIhWsc/ur8qJes/qevSPwT05jGgmKK/wihVkYN32T+Y9bjH8OiZP95U
a6fKF+A/LHHvrH4LtT8guNDmJojjP7J9FE3eL8E/sVeLaAML5z8Q3qW48TfH
P2fGPT2Dnuo/BKklUMaazD8aQYyhwEDuP8WpMjzAqtA/2cLt/+b38D8rQB/k
zrLSPzcHG3vb1PI/Eei3Lntk1D+FXeavQLb0P0ePuADfvtU/7QPDMBeb9j81
5dGOQsHWP5+phrxdgvg/07wuvxxr1z9lL+nGEWv6P4HtMnITvNc/2xi0ATBU
/D9jGU2y+7PXP+/jWOa0PP4/OfrBytlS1z/qotKfzhEAQOcbZUXhmNY/OR6m
WfMDAUClOkDPdIbVP0N9fSVI9AFAH8c3BCYc1D8eeVh+TeICQH9YyCG1WtI/
+1IJGYXNA0CwCgahEEPQPxO5NydytQRAItEncKmsyz+amZmZmZkFQCIKrYaV
K8Y/WQS/Og==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAZ2ZmZmZm5r+axwElsKDgP4Ufy3Km
yOC/l6sFsgxq4z8XTWAP2BfWv4bEYrU78eU/QGKGzQnNxL9Wv/bJ5TToP6Di
xC2Fx5c/uOn9Xdcz6j8I5w0EohXLP+JhRVYB7es/4pboiR++2T/FXASeeV/t
P3lKnVj1B+M/Si8Qo3uK7j+XC8uI1DzpP74zKb5obe8/Y9GtR2J67z/nmZRD
5APwP+QzFnan3vI/P1ZziqQs8D8KXdT5IwH2PxSDVZTfMPA/BL+j4vwi+T9W
WJIikxDwP3zA5bqIQvw/TvfIsKCX7z/6aopFH17/P3lIpGN3xe4/JRWVLw06
AUDXl6LgGavtP/0hAW9rwQJA3GVk+h1J7D8dly64WkQEQI3Rv4M/oOo/1GF4
vA3CBUDv4hfsX7HoP2GzXvS5OQdAAu++x4V95j9uPf0KmKoIQAyNo0TcBeQ/
ORTPR+QTCkA5CpKLskvhPzQYiPbedAtAiiy9HPag3D/MzMzMzMwMQCIKrYaV
K9Y/aJfAGw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz07+qTFjoerbrP/tdSEDv
I8i/av624r8W7T/0+MO6PEqzv3oPMNcdTe4/UIWuNm05pD9hNpEX8FjvP/Ze
W9uw7sM/2XZkRNQc8D/Sp2jM/nnRP7qTJvdnd/A/RoIEmyQL2T8KzK8UA7zw
P+0EGwBjU+A/nwMHNoHq8D8Wm2TJbCTkP2nL6a/JAvE/Am76Pqn25z9xQOOf
zwTxP455v1MRyOs/RTQi85Hw8D/yxEZrnpbvP17+CmcbxvA/4QiAtiWw8T/j
qISDgoXwPy9fS+uKkfM/FX8Ej+ku8D+No3hmf271PziqvPj8hO8/2UWgFQZG
9z8EX8ak9YDuPzGDx8ckF/k/Y3S2GkdS7T+MuiCy5OD6P6aB1vSR+es/VQr+
8lKi/D+OKfIZjXfqP/W7sBKBWv4/SPhKXAXN6D8ajgjBQgQAQM7R6gzd+uY/
jM41C77VAEDimo6DCwLlPyDASEFDoQFAyNNom5zj4j9mZmZmZmYCQJrHASWw
oOA/vp7E9g==
      "]], 
     {GrayLevel[0], InsetBox[
       StyleBox["\<\"\\!\\(TraditionalForm\\`a\\_1\\)\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], Offset[{-6, 6}, {0, -1.3856406460551018`}]],
       InsetBox[
       StyleBox["\<\"\\!\\(TraditionalForm\\`b\\_1\\)\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-6, 6}, {-0.4, -0.6928203230275509}]], InsetBox[
       StyleBox["\<\"\\!\\(TraditionalForm\\`c\\_1\\)\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-6, 6}, {0.1, -0.17320508075688773`}]], InsetBox[
       StyleBox["\<\"\\!\\(TraditionalForm\\`d\\_1\\)\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-6, 6}, {-0.7000000000000001, 0.5196152422706632}]], InsetBox[
       StyleBox["\<\"\\!\\(TraditionalForm\\`e\\_1\\)\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-6, 6}, {-0.30000000000000004`, 0.8660254037844386}]], InsetBox[
       StyleBox["\<\"f(\\!\\(TraditionalForm\\`a\\_1\\))\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{10, 6}, {3.1, -1.2124355652982142`}]], InsetBox[
       StyleBox["\<\"f(\\!\\(TraditionalForm\\`b\\_1\\))\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{10, 6}, {2.3, -0.17320508075688773`}]], InsetBox[
       StyleBox["\<\"f(\\!\\(TraditionalForm\\`c\\_1\\))\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{10, 6}, {2.7, 0.17320508075688773`}]], InsetBox[
       StyleBox["\<\"f(\\!\\(TraditionalForm\\`d\\_1\\))\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{10, 6}, {3.6, 0.34641016151377546`}]], InsetBox[
       StyleBox["\<\"f(\\!\\(TraditionalForm\\`e\\_1\\))\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{10, 6}, {2.3, 0.5196152422706632}]]}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
     StripOnInput -> False], TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 259},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->1980292572]
}, Open  ]],

Cell[TextData[{
 "The syntax for point names and function name is ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       SubscriptBox[
        StyleBox["pt", "TI"], 
        StyleBox["1", "TR"]], ",", 
       SubscriptBox[
        StyleBox["pt", "TI"], 
        StyleBox["2", "TR"]], ",", 
       StyleBox["\[Ellipsis]", "TR"]}], "}"}], ",", "\"\<\!\(\*
StyleBox[\"fun\", \"TI\"]\)\>\""}], "}"}], " "}]], "InlineFormula"],
 "where the function name must be a string."
}], "ExampleText",
 CellID->1122314517],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionArrowDiagram", "[", 
  RowBox[{"5", ",", "8", ",", "\"\<OneToOne\>\"", ",", 
   RowBox[{"PointLabels", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Delta]", ",", 
        "\[Epsilon]", ",", "\[Phi]", ",", "\[Theta]"}], "}"}], ",", 
      "\"\<\[ScriptG]\>\""}], "}"}]}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", " ", 
    RowBox[{"Style", "[", 
     RowBox[{
     "\"\<x \[RightTeeArrow] \[ScriptG](x)\>\"", ",", "18", ",", "Bold", ",", 
      "Italic"}], "]"}]}]}], "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->1391215618],

Cell[BoxData[
 GraphicsBox[
  {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
   0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
   {RGBColor[0.0692454, 0.645548, 0.968322], 
    DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
   {GrayLevel[0], PointSize[Medium], Opacity[1], 
    PointBox[{{-0.8, 0.34641016151377546`}, {0.1, -1.2124355652982142`}, {
     0.30000000000000004`, 1.2124355652982142`}, {
     0.8, -0.6928203230275509}, {-0.2, 0.6928203230275509}, {3.8, 0.}, {
     3.6, -1.0392304845413265`}, {2.1, -0.5196152422706632}, {2.8, 
     1.0392304845413265`}, {3.1, -0.5196152422706632}, {3.2, 0.}, {
     2.6, -0.6928203230275509}, {3.3, 1.2124355652982142`}}], 
    {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZuT/eaNfVImbzv0lUVDLL
NM4/agzFB2wQ8r9IVAT2SwrYP8J2/ONp1PC/Bzrow62Y4D+W2/AtiGXvvzuC
W4pcReU/SjkUoShY7b+xfupYtwjqP+iQmuTLgeu/MoCGMjfg7j+1XmeLa+Pp
v3cm5zSl5PE/Yp6dc+N96L+NUgL/qmD0P4Rr9lHxUee/SMF8ftvi9j+xmxhN
NGDmv0sllfThafk/9bomqSyp5b8jdq0RZ/T7P+84sIM7LeW/YQd0qxGB/j90
4ymgouzkv/U/BLpDhwBAxQUJRYTn5L9Rwt3Yts0BQFywkynjHeW/esuD+zQT
A0Cdz290oo/lv/TEMG0RVwRA9NTxyoU85r/Fgr9WoJgFQH7TInExJOe/2uzY
GTfXBkD1EW16KkbovxSve6ssEghAlT7VCteh6b8A563t2UgJQE2pnah+Nuu/
rDcpCJp6CkA4MyieSgPtvzM30r/KpgtAgwPja0YH77/MzMzMzMwMQJrHASWw
oPC/JVHMHw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ6T8iCq2GlSvmv74ScpMF
a+w/rFXd8AtX5L9/ryxvBVHvP1KXv3nUo+K/PmdAvAcl8T+8rkUO1hLhv1rj
dPBHqvI/2leO5cpJ379EqvTKdDf0P7jpOKSJtdy/AUwajrvL9T+UujCtRmra
v4qsq7dFZvc/4bUtmjlp2L84dKtyOQb5P3pNpKRys9a/FurtCrqq+j9AUFQV
2knVv2DkM2LoUvw/EvkIyS8t1L9LrI1m4/39PxSby8oKXtO/bQTHiciq/z8e
8L4D2dzSvwV6zhxarABAso7NAN+p0r+Exz4sYYMBQJZ5Ss43xdK/PtRIWwda
AkCsFZfp1C7Tv4QbaMXaLwNABiPWSH7m079rlu31aQQEQCakp3jS69S//tAy
JETXBEDT693PRj7Wv5C3cW/5pwVAcWIQuSfd1798WyEaG3YGQOf45BGZx9m/
8C+4xDtBB0ANut2flvzbv2KRtKfvCAhAtoBsmvR63r/MzMzMzMwIQJrHASWw
oOC/0PXNhA==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ6b8iCq2GlSvWP6RynED6
PuS/rCKG34jh2j92RiWkqZTdvwpDMAR4Gd8/HvBJuhx90r/+zcn4kmjhP2AF
aLXB9Ly/tlTv2EwD4z+Q5BO0jJSwP6tSOrQPXOQ/sMgPfKdHzz8IDZmcJHLl
PyCHOuLHONs/Km9GBfhE5j9YGfwy7W7jPzA9FBEa1OY/SIYUxG5G6T+q8MXN
Ph/nP4SIxCDPH+8//MBcXD4m5z8I367j+XvyPwx2PwYV6eY/vfLabuFl9T8V
yzI/42fmPx8+L2sSTPg/nFYhlO2i5T/T/VEKAy37P/8ZvIacmuQ/Ie06RywH
/j9hBQdWfE/jPyRl1lqFbABATtzuszzC4T86JOOnj9ABQDLtH9Bg598/Nq6e
IPguA0CLqb6/mcnbP0qH7NcEhwRAlr7iWFMt1z9ouV9A/9cFQNTSRtz/FNI/
yooXjTQhB0Cvs9q5pgbJPyCKnhD2YQhAlDWnQwntuT+amZmZmZkJQAAAAAAA
AJC8Eya7aw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZyb8iCq2GlSvmP2BdGVzO
t5m/MGkVdTsV6D8Ytn+v2n7DP1kelI0qvek/DZepg+s91T/QQsjfgSLrP1xa
oirybOA/9JHZ0INE7D8Q5M1vl0bmPyYUE4CWIu0/fgAZ68oo7D/I2ogYRLzt
Pza+ErY2CPE/IHudDzsR7j/470BwLv3zP6AbRlBOIe4/Bcqe/rrx9j+HHvZS
deztP9hrkfRK5Pk/tbglIsxy7T9PLlrzTdP8Pw8OcUuTtOw/BUKGfjW9/z8i
uFe9L7LrP9kyr+c6UAFA9OOukSps6j9aQnNTw70CQDJx4sQw4+g/OOj+DnIm
BEACrCvaEhjnP1tzmLmHiQVAaVXubMQL5T/NZ+jqR+YGQOqtdq9bv+I/FIjx
lvk7CEBbGF3XEDTgPwteQHDniQlAPE6773rW2j9NKh5IYM8KQFLz9pS3zNQ/
+C6UbLcLDECg1ESZF5zMPw10DQRFPg1ASJ/IM6d3vT9mZmZmZmYOQAAAAAAA
AKC8UVq2lA==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQbWIMApft72Vcv6qU9tn+jt2jANF7t+xz1h28u7/o
i72Fj/73V9EP7Rd8C3y0P+OrfZWaTJhq3FN7v1XNMQvcv9lnvTN1nFn80n51
oCXXa77v9kzNMowys97aB/z/rWO277s9l/7mF3uef7Cf+UDskEroD/s5tQ9b
d3N8thfS+7g57fIP+4U/lq5+offVPs2nh2WnxU/708825ipEfrd/NZH/49OW
n/bS7/fvsmn7aR9xxY/v6Paf9pmuc36Ibv1trzjHJs764k97w5C/C5qf/IXz
Ha7m5K/jZHCAqc+4+XmqjQKjA8y82gs/yyJMmRxg9v3jvcOZ78HsAHOPh2lx
xLsIFgeYe3nWtJ5xTWN1gPnHKLC+5lk+mwPMv3qHXcudS9kdYOFx4CcD/6VS
DgdYeH1q2aDEVcjpAAvPNDDgcoCFNwCXh76n
      "]], 
     {GrayLevel[0], InsetBox[
       StyleBox["\<\"\[Alpha]\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-6, 6}, {0.1, -1.2124355652982142`}]], InsetBox[
       StyleBox["\<\"\[Beta]\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-6, 6}, {0.8, -0.6928203230275509}]], InsetBox[
       StyleBox["\<\"\[Gamma]\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-6, 6}, {-0.8, 0.34641016151377546`}]], InsetBox[
       StyleBox["\<\"\[Delta]\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-6, 6}, {-0.2, 0.6928203230275509}]], InsetBox[
       StyleBox["\<\"\[Epsilon]\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-6, 6}, {0.30000000000000004`, 1.2124355652982142`}]], InsetBox[
       StyleBox["\<\"\[ScriptG](\[Alpha])\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{10, 6}, {3.6, -1.0392304845413265`}]], InsetBox[
       StyleBox["\<\"\[ScriptG](\[Beta])\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{10, 6}, {3.1, -0.5196152422706632}]], InsetBox[
       StyleBox["\<\"\[ScriptG](\[Gamma])\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], Offset[{10, 6}, {3.2, 0.}]], InsetBox[
       StyleBox["\<\"\[ScriptG](\[Delta])\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], Offset[{10, 6}, {3.8, 0.}]], InsetBox[
       StyleBox["\<\"\[ScriptG](\[Epsilon])\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{10, 6}, {3.3, 1.2124355652982142`}]]}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"x \[RightTeeArrow] \[ScriptG](x)\"", 18, Bold, Italic, StripOnInput -> 
     False], TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 260},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->1075696398]
}, Open  ]],

Cell["Labels need not be alphanumeric characters.", "ExampleText",
 CellID->938735133],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionArrowDiagram", "[", 
  RowBox[{"6", ",", "4", ",", "\"\<Onto\>\"", ",", 
   RowBox[{"PointLabels", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "\[FilledUpTriangle]", ",", "\[FilledDownTriangle]", ",", 
        "\[FilledDiamond]", ",", "\[DiamondSuit]", ",", "\[ClubSuit]", ",", 
        "\[HeartSuit]", ",", "\[SpadeSuit]"}], "}"}], ",", 
      "\"\<\[HappySmiley]\>\""}], "}"}]}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", " ", 
    RowBox[{"Style", "[", 
     RowBox[{
     "\"\<x \[RightTeeArrow] \[ScriptG](x)\>\"", ",", "18", ",", "Bold", ",", 
      "Italic"}], "]"}]}]}], "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->1906509469],

Cell[BoxData[
 GraphicsBox[
  {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
   0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
   {RGBColor[0.0692454, 0.645548, 0.968322], 
    DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
   {GrayLevel[0], PointSize[Medium], Opacity[1], 
    PointBox[{{-0.30000000000000004`, 0.17320508075688773`}, {
     0.30000000000000004`, 
     1.2124355652982142`}, {-0.5000000000000001, -1.2124355652982142`}, {
     0.30000000000000004`, -0.17320508075688773`}, {-0.30000000000000004`, \
-0.5196152422706632}, {0.6000000000000001, 0.34641016151377546`}, {3.4, 
     0.34641016151377546`}, {2.5, 0.5196152422706632}, {
     2.2, -0.6928203230275509}, {3.5, 0.8660254037844386}}], 
    {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAQAAAAAA4L/eaNfVImbzv0gGr4GB
09m/9KE0gGMf8r/zozCeKW3TvxwDUrr26vC/vgBrRrugyb8s3SdVAJPvv2h7
Hwx/Ari/9MJ1zzJ37b9AtS0BWc6PP1rqyU6jg+u/lKAvTUdSwD/YgCLkWrnp
v0AKyrhK984/ttKauUwZ6L+W4+JMFfLWP25xZZFVpOa/5M3wyn2I3j/X0alQ
O1vlv5dgWxlsHeM/DomDlqw+5L+rUUCGhwLnP67eWl9AT+O/q9K7G4Dx6j88
3se0dY3iv+exiY0/6O4/fmkqarP54b/kN4a3VXLxP9cPG+ZHlOG/OJ1+KFNy
8z+4ndH4aF3hv25pWnAIc/U/bHaXvzNV4b9ydjqEZXP3PzHfVJWse+G/XIkM
iFpy+T+BbUEQv9Dhvyjvyl7Ybvs/09K4DD5U4r++ulY60Wf9P05GLsXjBeO/
3k+gKjlc/z9S1zH3UeXjv/8i6VWDpQBA8wN0FRLy5L+bmZmZmZkBQCEKrYaV
K+a/OErPrw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz07+axwElsKDgv0S/MVXc
Ism/kNT6QlMz3r+CWukxTTe3v5Tpmq1bc9u/ECrP1mlAkT/8bx0Q7wLZv/wD
23nxIMA/snkpvFjj1r/cttM+nEfOP05dBx+5FdW/uERZrUVK1j9st5YoBZvT
v9QfPCgUgN0/TsxXyQV00r/Oo/kWs2DiP2U/zYdXodG/AH80IDEF5j/osm4t
aiPRvzjYO2iVq+k/92BYi4D60L/Ej3wl8FHtP7gm11awJtG/MtS5ySh78D+u
0eMd4qfRv0UI3nxlS/I/V8uTU9F90r8tFEfZtxj0P5KJeHQMqNO/Md4SGCvi
9T8zd9pC9SXVv0ThQYDMpvc/KV+vGsH21r/IAoHnq2X5P9rNH115Gdm/sAmY
Mdwd+z8yW2Lz+4zbv9EZOc5zzvw/G2+n6PtP3r/vce40jXb+P0fqYQ2BsOC/
alVyr6MKAEAWL54AN1/iv0hsKB/j1ABAmX8kRTsz5L+amZmZmZkBQCIKrYaV
K+a/NSPHpw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz0z8iCq2GlSvGv2odNXFp
Zdo/fUSg9yqutb9QPZVw3OvgPwAYTbtwpFa/vHRDAxfD5D+M1xm1T5izP3u1
Xb1atug/9tPS3fEPwz+dgGsLj8PsP3bNQLAqmcs/TiM8y0Z08D8r9uNrpbHR
P3TyCrSRkfI/aSbOHxg11T/g4IgKibj0PxXozzQQVdg/vAMfdwjo9j8RCGs0
5Q/bP82jVhvnHvk/u2fqUSRk3T+k9F0v+Fv7P9fzIC+RUN8/hB2goQue/T9e
eiNCE2rgP3HlG7ju4/8/blNOVQv34D/Ps5FZNhYBQApGxATmTuE/njqXuCc7
AkDCx/uydHHhPx5ctAkwYANAfm/qCaVe4T+aI1LRs4QEQLycvwSBFuE/OpIs
2heoBUA2sJjqLpngP1gOrYfByQZAO0pjdOLN3z8i0PMnF+kHQJ+jOQ1NAN4/
HMRkRYAFCUCzECmKksrbP4iejPdlHgpAzW4CGN8t2T8yMzMzMzMLQCMKrYaV
K9Y/AcyxhQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz078iCq2GlSvGP6wR21R8
Dsm/JpMsD4rGzz8G+2nOYpS2vxr8snK8YdQ/gKcCMGP5lj9mIvCUzo7YP+Q2
CnUZZsE/LM/PLkRo3D/WBtVBcCHQP4DXDV4S7N8/ptoPYc+21z/ycojZLYzh
P/a7KwEkb98/rhofl7j14j97z0K/KqPjPxfFVJLpMeQ/XLykah2c5z9+CNQF
GUDlP1ZoRWtOoOs/+A9tlbcf5j8+7XAynK3vP2ixJppO0OY/OFHXLfDg8T9g
XDNhgFHnPxFjZWh47fM/WHqoXQij5z91+up3UPv1Pyjf3Uy7xOc/ETg/WWEJ
+D/m/GFNh7bnP4s+E+uTFvo/iq126HN45z9rYQ2C0SH8P2KJEQ6iCuc/M72l
fAQq/j+m6WEDTG3mP0DiOmsMFwBAAt3lQ8Wg5T8aTFZdfhYBQMh1HlV6peQ/
rDqtitASAkCm8vqM8HvjP3IQoxF9CwNAbk8Zy8Uk4j8AAAAAAAAEQJrHASWw
oOA/Nvi0vA==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz4z8iCq2GlSvWPzDsR2Lg
iuY/7tGpNDSO2z8S3CQ3CgHqP9K4gnPAUOA/rsA6f9qT7T8WPsEaZbHiP7ik
Keq1oPA/OAgDHsXn5D8QQQ9P5YPyP7Szpviz8uY/5D6kDXty9D9STOsyHNHo
PxJNxbhwa/Y/cq8p9f+B6j8XABRium34P4HQhI55BOw/zs4gKEd4+j9LaMnu
u1ftP0BdNccBivw/mmQ8ExN77j/xQXIs0aH+P9FcHmbkbe8/UBt5hUxfAECi
rliI1xfwPzTJUTmdbwFAFsLKHwZg8D97AUQ0SoECQCYVrKxXj/A/Nt4EQMKT
A0AcUowTs6XwPwZiirpzpgRA3oWWdwyj8D8DSVHjzLgFQPx23EBlh/A/DQWz
KDzKBkBGmJcbzFLwP5HcInUw2gdALPxf8FwF8D9aMCl8GegIQPPUuqqBPu8/
nw/0BmjzCUC25/TwW0HuP156VECO+wpA6yhKBc8T7T8BAAAAAAAMQKpMWOh6
tus/NxqxMQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz0z/eaNfVImbzPyagy2UC
3ds/2wKcVP5L9D+cpPgUL1fiP6Bm9YJgGPU/hASFX0zR5j9qhMnu3Mr1P+ob
9vF4Wus/8ee84xRj9j9mTxuuTPDvPybBcp234PY/0s2IXyxI8j8fwGdygkP3
P6Jy3nEUnPQ/MgdR90CL9z/q7xBeovL2P6du7BrNt/c/z9V3jJhK+T/8VzM6
D8n3P3GQQ6a4ovs/GlnlLP6+9z+ZemY+xPn9P/QXZEqfmfc/yN5EvT4nAEB4
wt1mBln3Pyos2t3TTwFA6aPHyFX99j/IaCUjBHYCQKdsrha+hvY/JDiHdDOZ
A0C/0mU9fvX1P+qvYVHHuARAxD2lTuNJ9T8O7RIjJ9QFQOc0I1hIhPQ/tKzr
jbzqBkAzQkUzFqXzP1jz9sDz+wdAO+19TcOs8j92bWjEOwcJQCBSdmnTm/E/
BuyHxgYMCkBTrCRZ13LwP6Ys8WbKCQtAENvpYdlk7j8AAAAAAAAMQKpMWOh6
tus/Ht7ArQ==
      "]], 
     {GrayLevel[0], InsetBox[
       StyleBox["\<\"\[FilledUpTriangle]\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-6, 6}, {-0.5000000000000001, -1.2124355652982142`}]], InsetBox[
       StyleBox["\<\"\[FilledDownTriangle]\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-6, 6}, {-0.30000000000000004`, -0.5196152422706632}]], 
      InsetBox[
       StyleBox["\<\"\[FilledDiamond]\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-6, 6}, {0.30000000000000004`, -0.17320508075688773`}]], 
      InsetBox[
       StyleBox["\<\"\[DiamondSuit]\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-6, 6}, {-0.30000000000000004`, 0.17320508075688773`}]], 
      InsetBox[
       StyleBox["\<\"\[ClubSuit]\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-6, 6}, {0.6000000000000001, 0.34641016151377546`}]], InsetBox[
       StyleBox["\<\"\[HeartSuit]\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-6, 6}, {0.30000000000000004`, 1.2124355652982142`}]], InsetBox[
       StyleBox["\<\"\[HappySmiley](\[FilledUpTriangle])\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{10, 6}, {2.2, -0.6928203230275509}]], InsetBox[
       StyleBox["\<\"\[HappySmiley](\[FilledDiamond])\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{10, 6}, {3.4, 0.34641016151377546`}]], InsetBox[
       StyleBox["\<\"\[HappySmiley](\[DiamondSuit])\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], Offset[{10, 6}, {2.5, 0.5196152422706632}]],
       InsetBox[
       StyleBox["\<\"\[HappySmiley](\[ClubSuit])\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{10, 6}, {3.5, 0.8660254037844386}]]}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"x \[RightTeeArrow] \[ScriptG](x)\"", 18, Bold, Italic, StripOnInput -> 
     False], TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 260},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->1712020740]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "PointLabelOffset",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection", "ExampleSubsection",
 WholeCellGroupOpener->True],

Cell["\<\
The offset of the labels from the points is controlled using \
PointLabelOffset. Distances are given in printers points as two ordered \
pairs. The first ordered pair controls labels in the domain set and the \
second ordered pair labels in the range set.\
\>", "ExampleText",
 CellID->1176106066],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionArrowDiagram", "[", 
  RowBox[{"5", ",", "8", ",", "\"\<OneToOne\>\"", ",", 
   RowBox[{"PointLabels", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Delta]", ",", 
        "\[Epsilon]", ",", "\[Phi]", ",", "\[Theta]"}], "}"}], ",", 
      "\"\<\[ScriptG]\>\""}], "}"}]}], ",", 
   RowBox[{"PointLabelOffset", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "10"}], ",", 
        RowBox[{"-", "6"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"14", ",", 
        RowBox[{"-", "4"}]}], "}"}]}], "}"}]}]}], "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1166944104],

Cell[BoxData[
 GraphicsBox[
  {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
   0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
   {RGBColor[0.0692454, 0.645548, 0.968322], 
    DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
   {GrayLevel[0], PointSize[Medium], Opacity[1], 
    PointBox[{{0.4, 0}, {0.7000000000000001, 
      0.5196152422706632}, {-0.30000000000000004`, -0.17320508075688773`}, \
{-0.7000000000000001, -0.8660254037844386}, {
      0.30000000000000004`, -1.2124355652982142`}, {
      2.5, -0.8660254037844386}, {3., -1.3856406460551018`}, {
      2.9, -0.17320508075688773`}, {3.5, -0.8660254037844386}, {2.2, 
      0.6928203230275509}, {2.6, 0.34641016151377546`}, {2.8, 
      1.0392304845413265`}, {3.5, 0.8660254037844386}}], 
    {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz0z/eaNfVImbzvwT/pRvU
V9o/K884PreQ8r+8AgBgtdDgP1iLLPJS0PG/LvqLnwyG5D/V/usGXCXxv9z5
TRT4Seg/TSknMy2Q8L8Fuc5JeBrsP8O44p4VEfC/ADipHof17z967+9ysVDv
v8ATu2uM7PE/7MSZK1ys7r8ZGViajuHzP7C+rpmCNe6/1tp+RcDY9T8+gPbM
Y+ztvxKZpW4W0fc/wKOekSbR7b//YBB8hcn5P8m4pFvZ4+2/fyHOxgHB+z9E
Dis/ciTuv7rBvCiAtv0/8Fi89c6S7r9B30iK9qj/P5Rre/C0Lu+/tf7ON67L
AECYWjV30ffvv/IdfsJVwAFALcci6tx28L8wj/QU8LECQFg9msb1B/G/OsV4
/fyfA0A8ePlT5q7xv+bx2Sz+iQRAqKZG/lVr8r/0C3V5d28FQPqXlcngPPO/
HD0WIe9PBkB+6BSHFyP0v6rDkwnuKgdA1FkMEIAd9b8CAAAAAAAIQCIKrYaV
K/a/KLLQ2w==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQnZ4GAs/2r/KJeFG17fX+G+WsOxI5Hu6X3qt3KebY
i/3VSzV1ZYSv79+3f9aPftFn+wPkIh6Ebz6+n0N/5dXGpY/3N1z5XnF9ZtX+
Swfud9SWP9zv4iewyiHzmH3+CzH/MMn7+6Mibec4/L1mz6nnay935vb+8tAF
OXNKHtob1wRHC3Pe3B8oknBNqvK5fR+r0uk1F6/tT3z0XONVx1t7Pt+V88TV
r+63Sg5pyLzw0V7oprCeGdeV/SVO76+duffF/o7w2oXzqy/vj+i6+/X/2+9w
/pTMCp/kP7/g6ufcjtgRzvEPbl5w4rkN8h0MDjD7OALtvO05mRxg7rHefS9o
XxuzA8y9TYlHoy/9Z3GA+SfgkVLQ5mI2B5h/fV5kP5n9kN0BFh5is7nfnPTg
dICFV8yR9Ct1K7gcYOHJAAY8DrDwBgDG48Jp
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQbWIMApf3K3GtbZuqfWz/7Ljw7Fdcx/Yf4XpvcDVv
1/6AQ/P6j+ku3d/xxvvnHYEl+wvYo1ic5m23PzBTYVKo72T7CubZTrusz9p3
SSfoTJHZZH/m/PyVvRXX7T/0eGo4ntlrH9D7Z1O55QN7nd6CapF3h+1T12zJ
kD/yxP7NGdnMKuUT9jaSc/iuRr60X7Ovd5NYy2n7W3OP3df88dZ+3unTU+T4
ztnfzhBxCvX7aC+39oe65YHz9kHau/UKFn22TxZU4z7gc8HeI+9elub3r3D+
foPWcBfvH3D1a722rWyZ+wtu3t8eb6H1b/7A7Ut3tn/aaf4f7p6KlbwP8vYx
OMDce+XSDPHddxkdYP6ZJrNYQ/AnkwPMv5P+rNxzi5/FARYe19y5nzcrsjrA
wmubU1NxtS6bAyw8IeHN7gALbwCUDLf5
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ2T8AAAAAAAAAAD3b1QL7
vN4/fwBW77G0rz+ANnuTlgbiP4IWUGIDwb4/qpL/Jq/D5D8qBYaYC1bGPynj
vDtTlOc/uh+J6RjKzD+KIZ1yBHfqP+CmM4ieXNE/biyS2Dpq7T+kmof/KBDU
P6HLRtsyNvA/Iqxs7Dx+1j8tkg8z9r3xP0mC9TuQpdg/L4wklZdL8z/gN75l
/oTaP06tCwZE3vQ/rhEkB4kb3D9OK0LdJXX2P4mBP2pYaN0/VmmaNmUP+D92
11j4u2reP7o2yWQorPk/buaalyoi3z9CmeVklEr7PzXl0fNCjt8/btqdUs3p
/D/m8Q+yy67fP7Yu5Nz2iP4/2NEbj7OD3z9Vh2pdmhMAQILHmWgRDd8/eJ1M
kNXhAECroukwJEveP0uzApq/rgFAsnrAzVI+3T8l01zA63kCQLvLj+Er59s/
moPBNe5CA0DR7tZ/ZUbaP1YKYVJcCQRAVhWHzNxc2D/NzMzMzMwEQCEKrYaV
K9Y/KRC9bg==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAZ2ZmZmZm5j+axwElsKDgP+YuytVb
uOg/NNMczbzC4j/oIzAejiLrP2paqPAvyeQ/Pr3kO7Wj7T/Jbt149rLmPwIz
e34+HfA/QnnghQx/6D95qtvawnLxPzTZpPh9LOo/oBOrfbLR8j9mtXT0Zrrr
P+sdFTJTOfQ/ikTYV/Qn7T9U46Ym5qj1P35mnCxkdO4/emGOUqgf9z94Hb0O
Bp/vP26pFt3SnPg/2KX+xJ1T8D+C3SmHmx/6P0YG/TY8xvA/dAGhFjWn+z9/
hkaMISfxP5ysKMPPMv0/oDUrWxp28T+u1oCkmcH+P7zvpbz8svE/yZ/ukF8p
AEAaHZhiqN3xP8qsFbG18gBAatDtqAb28T/YEYFeZLwBQIYroaEK/PE/iwMA
lgCGAkAGq5Ybse/xP55ILV4fTwNAsrROpADR8T9n5DEAVhcEQNOCa4QJoPE/
3HBkQDreBECoRA235VzxP+Icp5ZiowVAlAkJ3LgH8T9mZmZmZmYGQJrHASWw
oPA/QrG/sQ==
      "]], 
     {GrayLevel[0], InsetBox[
       StyleBox["\<\"\[Alpha]\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-10, -6}, {0.30000000000000004`, -1.2124355652982142`}]], 
      InsetBox[
       StyleBox["\<\"\[Beta]\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-10, -6}, {-0.7000000000000001, -0.8660254037844386}]], 
      InsetBox[
       StyleBox["\<\"\[Gamma]\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-10, -6}, {-0.30000000000000004`, -0.17320508075688773`}]], 
      InsetBox[
       StyleBox["\<\"\[Delta]\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], Offset[{-10, -6}, {0.4, 0}]], InsetBox[
       StyleBox["\<\"\[Epsilon]\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-10, -6}, {0.7000000000000001, 0.5196152422706632}]], InsetBox[
       StyleBox["\<\"\[ScriptG](\[Alpha])\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{14, -4}, {3., -1.3856406460551018`}]], InsetBox[
       StyleBox["\<\"\[ScriptG](\[Beta])\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{14, -4}, {3.5, -0.8660254037844386}]], InsetBox[
       StyleBox["\<\"\[ScriptG](\[Gamma])\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{14, -4}, {2.9, -0.17320508075688773`}]], InsetBox[
       StyleBox["\<\"\[ScriptG](\[Delta])\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{14, -4}, {2.6, 0.34641016151377546`}]], InsetBox[
       StyleBox["\<\"\[ScriptG](\[Epsilon])\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{14, -4}, {2.8, 1.0392304845413265`}]]}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
     StripOnInput -> False], TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 259},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->1946674674]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "PointLabelStyle",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection", "ExampleSubsection",
 WholeCellGroupOpener->True],

Cell["\<\
Any directives that can be used with Style can be used to set the style of \
the labels.\
\>", "ExampleText",
 CellID->1931171216],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionArrowDiagram", "[", 
  RowBox[{"6", ",", "4", ",", "\"\<Onto\>\"", ",", 
   RowBox[{"PointLabels", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "\[FilledUpTriangle]", ",", "\[FilledDownTriangle]", ",", 
        "\[FilledDiamond]", ",", "\[DiamondSuit]", ",", "\[ClubSuit]", ",", 
        "\[HeartSuit]", ",", "\[SpadeSuit]"}], "}"}], ",", 
      "\"\<\[HappySmiley]\>\""}], "}"}]}], ",", 
   RowBox[{"PointLabelStyle", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"Red", ",", "16"}], "}"}]}]}], "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->126786554],

Cell[BoxData[
 GraphicsBox[
  {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
   0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
   {RGBColor[0.0692454, 0.645548, 0.968322], 
    DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
   {GrayLevel[0], PointSize[Medium], Opacity[1], 
    PointBox[{{0.7000000000000001, 
      0.17320508075688773`}, {-0.5000000000000001, 
      0.17320508075688773`}, {-0.5000000000000001, -1.2124355652982142`}, {
      0, -0.6928203230275509}, {0.2, 1.0392304845413265`}, {
      0.7000000000000001, -0.8660254037844386}, {3.3, -0.8660254037844386}, {
      2.3, 0.8660254037844386}, {2.6, -1.0392304845413265`}, {3.4, 
      1.0392304845413265`}}], 
    {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAQAAAAAA4L/eaNfVImbzv8As7Yy4
Udi/feJ9szw08r98/Ym8yW3Qv/7eHV0HGfG/9lE+qsawwL9AMtwXGRXwvwCP
eD9cz1W/CCL7nfdR7r88bahfIKvAP2wfcZVZquy/RBCBq0Pl0D9uS3jbODTr
v/KhoizAlNk/GvUb81vw6b+Nu8r1tC/hP8pBwLNu3+i/b0iaWkug5T9eYvTt
AQLov/vJPjLIGeo/ruibHotY578Nu0Wxy5nuP1YCmDBk4+a/ZAblSvmO8T/a
rBFNy6Lmv6Cm5LXr0fM/fjN+uuKW5r8aiFHpiRT2Pwl3b8qwv+a/IvKK4aBV
+D/ipTnWHx3nv0huhmr+k/o/rCtwSv6u57/6Z0jCcc78PzjAM8H+dOi/znGY
OswD/z96oEMruG7pv+mxzexwmQBAVCe8B6ab6r/VDX/8RK0BQDcyZaoo++u/
GTWu8c+8AkCwA2uQhYztv0NVyLeBxwNAqa9Uw+dO77/NzMzMzMwEQJrHASWw
oPC/Kb7Wnw==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQnZ4GAs/sV/lEvKja9nr/n+LUqb93vLTfnPtV/9GT
l/vDlF/IPpB/az+PcdnBaqsX+39pX/q8yOeD/Zag43c0dzzb/yU8tmWV4Cf7
C8nVNTvjnu4/z8+XIHbvs733uqevHuo92c9kHGD1acNX+3eer+JdNB/vX/Rf
Kcat87v9H4nFq7P9Hu2/xdypLpH20/5h9I43p+c93L/Ls2lOlttve4eDh4MO
ST3cf0bP676d5l/7osX33kw59GD/mTUT90/h/W/fs760pWjag/2zNM0XmFYy
OMD5L6ImJGQzOsDUK9/et/FUDJMDzDzhOus3n32YHWD26TW8uxhjzeIAc0/9
s+uP2tRZHWDulfjx4/wXATYHmH9YQy7u1frO5gDzL/MrsbZ1N9kdYOGh/b31
B/sODgdYeDEuXiZ1cCKnAyw8wcGdxuUAC28Ac1nVEA==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAAAAAiCq2GlSvmvxdp+A9p
WsE/hFRFVEwV5L9Oswh/vojRP2IW81MdMuK/GpOyveqN2j+w6CrmCIPgvzZx
oAL42+E/xVvki+cR3r+x5+jE+IDmP0uoAx9Nidu/scVrw4Az6z+Siamrmm3Z
vwM7S+4R8e8/fPa0ju6/178PVVAtlFvyP1xr/MMsgda/PBMTzJ3B9D9AHVZt
/rHVv1MGowRgKfc/IC/aeNFS1b+2Em0elJH5PyqGjmbYY9W/xG9xJPP4+z8g
MZstCuXVv3VUv5I2Xv4/gJkVQSLW1r9sU9qBDGAAQOrzX7SgNti/ueBSbquO
AUAqowl/ygXavw/ZyHxXugJAWIkM4KlC3L/C68OrceIDQACgMuAO7N6/jDmy
3lwGBUAYU6/5RwDhvw5URTF+JQZAmCgxXEW/4r8cuKFJPT8HQPoafmsSsuS/
YCc1qQRTCECu0MN9ptfmv/f8CPxBYAlA1Wns990u6b9mZmZmZmYKQKpMWOh6
tuu/OwHAPw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAQAAAAAA4L8iCq2GlSvGP7A+Sywc
0Nm/AKSeg+zF0D+pBuYVll/Tv5IMaOJ0LNY/Moaz4rBjyb/kQqBNhkbbPygE
ALq5Kbe/ai/U4rUI4D8gF32N4i+VP1avhCZNReI/MK/+xE1FwT9EKu8g2Vfk
P9TjtKSqINA/yJMeUEA/5j9eqxO9D8nXP7hreRaA+uc/zPaQ/MWX3z+yX/pD
rYjpP/Xls29UxOM/UJ36kvTo6j9/G+ffwMvnP3aeSxibGuw/r9AkCQXg6z9i
9GSm/hztP8FDOdX2/u8/EHJxI5bv7T/82bxBMxPyPwYNDtLxke4/Krvi6A8q
9D9W25OMuwPvP4KI9ZP1QvY/gLnN8rZE7z+oQGlix1z4PyJXAYrBVO8/3G9m
9md2+j8iqzrP0jPvP2R9NAy6jvw/QBrQO/zh7j/QHJYRoaT+P5rqHDxpX+4/
r8lj3oBbAEB673YYX6ztP7dZZ1FhYgFASKdr0DzJ7D9nZmZmZmYCQKxMWOh6
tus/oH7FyA==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAZ2ZmZmZm5j8iCq2GlSvGP1Vet7DE
9ec/OTbp7Ugwzj8IgjdpFJzpP8Ip1zQT9dI/6LLre3VY6z9iDfNCiqrWP/VD
Th/8Ke0/Vuo+2JE22j+RvmpRsQ/vP40HAihIl90/zeF2rUmE8D8UZLwwcWXg
Pwm/8SvLifE/n4+j0dbn4T+DDO5i0pfyP+ToG/EHUuM/gGaVC9Ct8z+mC2dh
RKPkPw6yeKUwy/Q/IBSbMtna5T/mw9PEXO/1Pz5ClBEh+OY/YmXlYrkZ9z8s
mK2fhPrnPwAWMTCoSfg/4OgRw3rh6D9g5H7oh375P/y4eu+IrOk/ssxsqLS3
+j+CWDZnQ1vqP6I1ZESI9Ps/VrVSdE3t6j/yZ8WgWjT9P/CQzplZYus/Ijga
C4J2/j+S/7a8KbrrP1aOIJRTuv8/AGAbRY/06z/lgj61kX8AQNhPxjZrEew/
8cHymiIiAUASfq5BrhDsP+WzSTmGxAFA1KIWyljy6z9mZmZmZmYCQKpMWOh6
tus/k8DGDw==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQPWsmCJy0n3WcUXXDgg/2mwNCTnX9umJvVHGiyXT/
R/tt6jx67lF37as4ed/NPvrJ/p1Dt8OJz4/sXV/L60Rs/mx/78CyCe/8nttP
z5or8ab9i/2z5+evuO15bb+WMebFM6ev9ivc6m7ulvlg31Llmvr08Vd7o7Co
LfxJn+xrlH4rTcr6Zj+Xx6/91Zov9lsUNNcsvfbN/o3f4/cif77ZBz0qWPFM
/bv9rQwRp1C/n/ZNddd+acZ9tw/U3q1XsOi3vfnbBTeCKr/bu+fdy9L8/hfO
j59xSHuRKoMDTH2Y6u2QQ36MDjDz6o6pdoaXMTnA7Nu8cP6nI7OZHWDuqVzJ
+yBvH4sDzL2XL80Q332X1QHmn6kyizUEf7I5wPw78c/KPbf4ORxg4XHNnft5
syKnAyy8tjk1FVfrcjnAwtMYDLgdYOENALbHw/c=
      "]], 
     {GrayLevel[0], InsetBox[
       StyleBox["\<\"\[FilledUpTriangle]\"\>",
        StripOnInput->False,
        LineColor->RGBColor[1, 0, 0],
        FrontFaceColor->RGBColor[1, 0, 0],
        BackFaceColor->RGBColor[1, 0, 0],
        GraphicsColor->RGBColor[1, 0, 0],
        FontSize->16,
        FontColor->RGBColor[1, 0, 0]], 
       Offset[{-6, 6}, {-0.5000000000000001, -1.2124355652982142`}]], InsetBox[
       StyleBox["\<\"\[FilledDownTriangle]\"\>",
        StripOnInput->False,
        LineColor->RGBColor[1, 0, 0],
        FrontFaceColor->RGBColor[1, 0, 0],
        BackFaceColor->RGBColor[1, 0, 0],
        GraphicsColor->RGBColor[1, 0, 0],
        FontSize->16,
        FontColor->RGBColor[1, 0, 0]], 
       Offset[{-6, 6}, {0.7000000000000001, -0.8660254037844386}]], InsetBox[
       StyleBox["\<\"\[FilledDiamond]\"\>",
        StripOnInput->False,
        LineColor->RGBColor[1, 0, 0],
        FrontFaceColor->RGBColor[1, 0, 0],
        BackFaceColor->RGBColor[1, 0, 0],
        GraphicsColor->RGBColor[1, 0, 0],
        FontSize->16,
        FontColor->RGBColor[1, 0, 0]], 
       Offset[{-6, 6}, {0, -0.6928203230275509}]], InsetBox[
       StyleBox["\<\"\[DiamondSuit]\"\>",
        StripOnInput->False,
        LineColor->RGBColor[1, 0, 0],
        FrontFaceColor->RGBColor[1, 0, 0],
        BackFaceColor->RGBColor[1, 0, 0],
        GraphicsColor->RGBColor[1, 0, 0],
        FontSize->16,
        FontColor->RGBColor[1, 0, 0]], 
       Offset[{-6, 6}, {-0.5000000000000001, 0.17320508075688773`}]], InsetBox[
       StyleBox["\<\"\[ClubSuit]\"\>",
        StripOnInput->False,
        LineColor->RGBColor[1, 0, 0],
        FrontFaceColor->RGBColor[1, 0, 0],
        BackFaceColor->RGBColor[1, 0, 0],
        GraphicsColor->RGBColor[1, 0, 0],
        FontSize->16,
        FontColor->RGBColor[1, 0, 0]], 
       Offset[{-6, 6}, {0.7000000000000001, 0.17320508075688773`}]], InsetBox[
       StyleBox["\<\"\[HeartSuit]\"\>",
        StripOnInput->False,
        LineColor->RGBColor[1, 0, 0],
        FrontFaceColor->RGBColor[1, 0, 0],
        BackFaceColor->RGBColor[1, 0, 0],
        GraphicsColor->RGBColor[1, 0, 0],
        FontSize->16,
        FontColor->RGBColor[1, 0, 0]], 
       Offset[{-6, 6}, {0.2, 1.0392304845413265`}]], InsetBox[
       StyleBox["\<\"\[HappySmiley](\[FilledUpTriangle])\"\>",
        StripOnInput->False,
        LineColor->RGBColor[1, 0, 0],
        FrontFaceColor->RGBColor[1, 0, 0],
        BackFaceColor->RGBColor[1, 0, 0],
        GraphicsColor->RGBColor[1, 0, 0],
        FontSize->16,
        FontColor->RGBColor[1, 0, 0]], 
       Offset[{10, 6}, {2.6, -1.0392304845413265`}]], InsetBox[
       StyleBox["\<\"\[HappySmiley](\[FilledDownTriangle])\"\>",
        StripOnInput->False,
        LineColor->RGBColor[1, 0, 0],
        FrontFaceColor->RGBColor[1, 0, 0],
        BackFaceColor->RGBColor[1, 0, 0],
        GraphicsColor->RGBColor[1, 0, 0],
        FontSize->16,
        FontColor->RGBColor[1, 0, 0]], 
       Offset[{10, 6}, {3.3, -0.8660254037844386}]], InsetBox[
       StyleBox["\<\"\[HappySmiley](\[DiamondSuit])\"\>",
        StripOnInput->False,
        LineColor->RGBColor[1, 0, 0],
        FrontFaceColor->RGBColor[1, 0, 0],
        BackFaceColor->RGBColor[1, 0, 0],
        GraphicsColor->RGBColor[1, 0, 0],
        FontSize->16,
        FontColor->RGBColor[1, 0, 0]], 
       Offset[{10, 6}, {2.3, 0.8660254037844386}]], InsetBox[
       StyleBox["\<\"\[HappySmiley](\[HeartSuit])\"\>",
        StripOnInput->False,
        LineColor->RGBColor[1, 0, 0],
        FrontFaceColor->RGBColor[1, 0, 0],
        BackFaceColor->RGBColor[1, 0, 0],
        GraphicsColor->RGBColor[1, 0, 0],
        FontSize->16,
        FontColor->RGBColor[1, 0, 0]], 
       Offset[{10, 6}, {3.4, 1.0392304845413265`}]]}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
     StripOnInput -> False], TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 259},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->847883873]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "SetsStyle",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection", "ExampleSubsection",
 WholeCellGroupOpener->True],

Cell["\<\
The domain and range sets are drawn using the Disk command. Any graphics \
directive that works with Disk can be used to style the sets.\
\>", "ExampleText",
 CellID->1481494928],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionArrowDiagram", "[", 
  RowBox[{"6", ",", "5", ",", "\"\<Onto\>\"", ",", 
   RowBox[{"SetsStyle", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        TagBox[
         ColorSetterBox[RGBColor[1., 0.4374761577782864, 0.0340886549172198], 
          "Swatch"],
         Setting], ",", 
        RowBox[{"EdgeForm", "[", 
         RowBox[{"{", 
          RowBox[{"Green", ",", "Thick"}], "}"}], "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"Yellow", ",", 
        RowBox[{"EdgeForm", "[", 
         RowBox[{"{", 
          RowBox[{"Blue", ",", "Thick", ",", "Dashed"}], "}"}], "]"}]}], 
       "}"}]}], "}"}]}]}], "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1559851578],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1., 0.4374761577782864, 0.0340886549172198], EdgeForm[{RGBColor[
    0, 1, 0], Thickness[Large]}], 
    DiskBox[{0, 0}, {1.1, 1.6500000000000001`}]}, 
   {RGBColor[1, 1, 0], EdgeForm[{RGBColor[0, 0, 1], Thickness[Large], 
    Dashing[{Small, Small}]}], DiskBox[{3, 0}, {1.1, 1.6500000000000001`}], 
    {GrayLevel[0], PointSize[Medium], Opacity[1], 
     PointBox[{{0.2, -1.0392304845413265`}, {0.7000000000000001, 
      0.5196152422706632}, {0.6000000000000001, 1.0392304845413265`}, {0.1, 
      0.17320508075688773`}, {-0.4, 
      1.0392304845413265`}, {-0.2, -0.6928203230275509}, {
      3., -1.3856406460551018`}, {2.9, 1.2124355652982142`}, {
      3.1, -0.17320508075688773`}, {3.5, 0.5196152422706632}, {
      2.5, -0.8660254037844386}}], 
     {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZyT+axwElsKDwv0JJtD/k
cNQ/YQfso0q9779092J3SDbcPyhkCuIfZu6//jUN8WwM4j8H4eYZljztv2bd
erk0CuY/zXkfK0tB7L8C7BptXRLqPyhto2vEdOu/lKNLYsMi7j9szfNgbtfq
v6j0H0ifHPE/RKmOhpxp6r8k+/PZ0SnzP+hZoyGJK+q/vjhIu+I39T+ldyci
VR3qvyPQzcq6Rfc/dtxcEQg/6r/74VsFQ1L5P2760Q2QkOq/lLz6GWVc+z9l
pd7UwRHrvwl/kP0LY/0/1kaY2VjC678F1eB9JGX/P09OMWn3oey/vOXI6c6w
AEC3kbDcJrDtvxsidBm1qwFAHzzm11fs7r9Gnnetv6ICQO5xvkrxKvC/wPhe
kWuVA0CHNf+fA/bwvyz2SAI4gwRAm7fOK/fW8b9LlDnTpmsFQM2iRpJUzfK/
c+kMsDxOBkBg7NUamdjzv7RK516BKgdAAhmd9Tb49L8BAAAAAAAIQCIKrYaV
K/a/UArKqA==
       "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZyb8iCq2GlSvmv/IDaJGv
oLa/vdZvV76A5L9gVwAArBaaPxZPV7/1/+K/tOgvfjIYwj8QNtboB6rhv7bz
myjwk9A/AItMQap/4L8Jcp2T8DTYP9hThzH2At+//29SPQ7r3z+/T9ybAmDd
v34ndtcY2eM/pPovDVgX3L8yMrA0HcPnPy3uWemkKdu/rLX9ioCx6z9JcelP
Z5favyQyS90sou8/TLg52exg2r//YBB8hcnxP17iRW1Shtq/fyHOxgHB8z9U
jVI0hAfbv7rBvCiAtvU/rCJ1oT3k279B30iK9qj3P/RH85YJHN2/av2db1yX
+T/8JWekQq7ev+Q7/ISrgPs/wMZDrwlN4L9fHukp4GP9PxezMmg7b+G/c4rx
+vk//z/fKPGCHL3iv+bx2Sz+iQBAtoWL1/s15L/0C3V5d28BQFpoKW4R2eW/
HD0WIe9PAkBiCSjpfqXnv6nDkwnuKgNADuwW+0+a6b8BAAAAAAAEQKpMWOh6
tuu/N3fMsA==
       "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZuT8iCq2GlSvGP+yIfQfR
+cs/IA8j0ciSxz/kG9Lvk5nVP48ysp7JSMg/ehBFtF443T80HUFdN03IP5Hu
N92eauI/LH4/sw+gxz8z8Up4EzbmP5I75YCuQcY/w2FJt4n86T/Qo3KvzTLE
P6/4bMwAvO0/MH6LzoR0wT/CmSnQPbnwP1yiwv+QELw/PHYO8ACP8j8gKPdi
09+zP84rg4RQXvQ/uM0anFK2pD+6tHS4Nib2PwBlmKkWGF6/8yaPpcHl9z9w
0awHFiqpv7fBl9QDnPk/JJKEd2n0ub8Ns2y7FEj7P/B7+qKxR8S/CbpnOBHp
/D+ATL1EHC/Mv0Au4gocfv4/DbLHgCFW0r8MPU0kLwMAQK80fWNS3da/8F7e
54PAAECVVa+5uarbv+sMqdqndgFAtACqmOVd4L9RA+5XOiUCQGzmokjrBuO/
pZXxvt7LAkAk0TeTBM/lvwejIaQ8agNAIryhobe06L8BAAAAAAAEQKpMWOh6
tuu/IRnAtQ==
       "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAZ2ZmZmZm5j+axwElsKDgP+kgitqe
8+k/EHajtdBr4T+Y41zgJIntP/GZ3WJZDeI/iJXH0IiS8D+d3P529ITiP/d8
J3G9YvI/b6ySe2LS4j8O+LICOjT0P4uXDVt69eI/61zNiAcG9j9W5Zl2Ke7i
PxyT5dsu1/c/t934r3O84j+o6Zksuab5P/5/c2dzYOI/jiKthrBz+z9Iwdtt
WdrhP2o3h1MgPf0/Rr+l6mwq4T+kmPzbFQL/P+qlJjYLUeA/b4WLZNBgAED6
nxpQT53eP6rmzdFpPQFAC9VdtJZH3D+urwUpYhYCQOYnkL8potk/+ARsSkbr
AkDBILLib67WP4KiSECluwNAAHCTGvpt0z8Rk+F6EIcEQODhHjYExc8/qh4l
CxxNBUAEb17J0hvIP8HP7tteDQZA0Ay2FOHIvz8ELMnpcscGQIA3IRvmjKw/
DYsOefV6B0Dg56mMaBmRv7JSTEqHJwhAsPnGx67Tt7/NzMzMzMwIQBwKrYaV
K8a/fmrC1A==
       "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ2b+axwElsKDwP48mBnqA
z82/6szI4OOo8T+cKFlLjiKwv7aZI1iqkfI/2LDPtd73uz/jdmwIiFrzP266
t9c3JtI/YEBWXRID9D/uLtBlMWrdP7FreenvivQ/UfvcDfVh5D8qQMeV2PH0
P/0gwhmuFuo/iRPKx5U39T8/grDHPNDvPzg/nn4CXPU/UJ8IvMvF8j/EbpRm
C1/1P/pnnMvZovU/gtxy465A9T+E4EGdw334P6cLUBH9APU/HtqMXgVV+z9u
iwa8F6D0P5bbvS0dJ/4/ukxGTTIe9D8R4UDzRXkAQMQNTbGRe/M/ypqe9ura
AUBdVVQyjLjyP8wIPPzBNwNAZGXISonV8T9R8v7rEY8EQFZvYG4B0/A/J+6k
nCTgBUCJWmqU+2LvP2DzajRHKgdAnpDv9y/j7D+PO/OHymwIQOlW34DzJ+o/
PBw3dwOnCUCGBNUyuTLnP5qGU0hL2ApA3lmc1hIF5D8AAAAAAAAMQJrHASWw
oOA/5TbBAA==
       "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz4z+axwElsKDwP1isCy2f
BOY/1aHp7/SK8T8ZScYIn+roPwKBeKuQZPI/FWgaEqnj6z9NdTXhDy3zP05g
g3op7u4/tLb0Lgjk8z8Rd8GXQQTxPz0SSn8YifQ/zhjnWoiY8j8GHgw96Rv1
P1Z5eIQSM/Q/M9/MgSyc9T8EQXg/BtP1P0w5Lz+eCfY/47a614Z39z+bOANj
BGT2PyyxAC+1H/k/Zg4W9i6r9j8YeVozsMr6P+ZlpTX43vY/OtGTVpV3/D+k
kGinRP/2P9fAaQaBJf4//ugkKAMM9z/VW0olj9P/P0auxfQsBfc/pDqvwW3A
AEBAh/Ktxer2P+qBzitBlgFA6sMiVtu89j/S/FNc0GoCQKJjLkqGe/Y/ZDeZ
iqo9A0C20WA06Sb2P/Yd2NVfDgRADzQU+jC/9T/jwYeAgdwEQHEO36OURPU/
VpYeK6KnBUAo3mBAVbf0P8j3Gg5WbwZAfiy9wb0X9D8zMzMzMzMHQN5o19Ui
ZvM/FO628g==
       "]], 
      {GrayLevel[0]}}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
     StripOnInput -> False], TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 259},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->1830255666]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Possible Issues",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->1305812373],

Cell["\<\
When a point in the range set has more than one pre-image only one will be \
identified with the point labels..\
\>", "ExampleText",
 CellID->2113280235],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionArrowDiagram", "[", 
  RowBox[{"7", ",", "3", ",", "\"\<Constant\>\"", ",", 
   RowBox[{"PointLabels", "\[Rule]", " ", "Automatic"}]}], "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->668871912],

Cell[BoxData[
 GraphicsBox[
  {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
   0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
   {RGBColor[0.0692454, 0.645548, 0.968322], 
    DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
   {GrayLevel[0], PointSize[Medium], Opacity[1], 
    PointBox[{{0.1, 0.8660254037844386}, {0.1, -0.8660254037844386}, {
      0, 1.3856406460551018`}, {-0.6000000000000001, 
      1.0392304845413265`}, {-0.5000000000000001, 0.17320508075688773`}, {
      0.5, -0.5196152422706632}, {-0.4, -0.34641016151377546`}, {
      3.7, -0.5196152422706632}, {2.6, 0.34641016151377546`}, {
      2.3, -0.8660254037844386}}], 
    {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQPWsmCOy0X+UT8aJq2+v9WkrpnJu4T9jflExvb9Z5
tf/ZF8fOVccv2otderF52+EX+x8IRX46s+K6/fzbffcu1zzff+pR3QLFZXft
+3ySKjsinu233iNdb3vvof0Fmz2z88Of7v+ce0vc/8sT+5Vmv9cLVj3ZH8h8
MmWR8Av7LfmK2k8OPN6vInfDrd/qtb26UZOfp87j/YGRS6dlpr+zf9c+/3P/
nkf7a/OuMDue+WCvczE46XPho/16Iu1twmmf7D9VxBrO9nm03yLo/v0ghi9w
/svvE5NUZ32Fq9+7U3j2RqPvcPNCwn8dkjv5A27fnieMJlNjfsHdk6Nfecvu
zW+4ez06bixzL/8L98+mtW7/7v75B/fvdYEXUyTtGBxg4THtC9cj218MDrDw
Ejg4R6hiE6MDLDzTwIDJARbeALvL1N4=
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAA4D+axwElsKDgv56R1gnm
iuI/TNwMs1uf378eLxm2oh7lP3RLXzUGOd6/Tkt79te55z9w36/yHQ/dv7pa
4MUjW+o/IfC9/EAi3L8jjTPkIAHtP7QTdAHtctu/BFH2k2eq7z9IqzgIfwHb
v4SGkCzHKvE/7XSRQDPO2r//Y3hclYDyP/xTNOIk2dq/4Pwd/ejV8z98QZYe
TiLbvyo89PIMKvU/Wg4AJIip278+MY7DTHz2P65UJzKLbty/6QZi9fTL9z+a
qkDA7nDdv5usA29TGPk/G+N2tCmw3r99qqTVt2D6P2j3VFbJFeC/NPym6nOk
+z/MYScsMPHgv/JLEejb4vw/Uk2g8dTp4b9rf7PbRhv+PxJspbgz/+K/hjrL
AA9N/z/ABxVVuTDkv6Jj/AvJOwBAe+3cqsN95b/E26reGM0AQEtU/gOi5ea/
Zsk/3ilaAUBSGlFulWfov0L8NDGx4gFAyJjUINEC6r9mZmZmZmYCQKpMWOh6
tuu/f2nK7Q==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ2b8iCq2GlSvWv77kFXzn
+NG/xv06pI640798GJzuYnvEv4OktmcOn9G/iL09sY1go7/6BF8HZMDPv1Ta
HeBG37U/CzeJJc35zL+gGwImktPKP8ShqzPR68q/cFlJIp1l1T+CaPlHh5fJ
v4KNyXERZ90/b8UD8aP9yL8vX5xI87TiP51i7NV4Hsm/1SmNKu605j8cKxKL
9PnJv7PnDwxaseo/tpFPm6KPy7/xxt19GajuP69kxcWr3s2/+KOsCYlL8T87
swg463LQv4IcH8AWPvM//AdapsNR0r9UmRBarSr1P42ZphrhitS/6BOUeUcQ
9z+72B2cFR3Xv4SLs3Xj7fg/SpuI7AMH2r+82Cbjg8L6P473l0EgR92/ZHHK
GjCN/D9KZXOL2G3gv4LEjr30TP4/4z0fDGhh4r9zFs0acgAAQBtY0ZE1feS/
5XosmgvUAEAmgU2xIsDmvy9HnJbWoAFA174SPfwo6b9nZmZmZmYCQKpMWOh6
tuu/PNDQtg==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAQAAAAAA4L8iCq2GlSvGP3ax9iLG
Y9e/t7/pLk6xyD9QvthURnbNv250vWe1bco/OH1g7rwquL/cQF9k31/LP7Bq
zPTKR6U/JyHbpkuHyz+8uJNLcbbGP6R1P0Tl48o/1uZRW3EH1D+4VrbvAnbJ
P3QcB3MJqdw/DNmEzGY+xz+ntiZCtp3iP0iqCAc+PsQ/McAVHAfd5j/Eruo0
IHfAP81xi1k2EOs/qNS2+RzWtz9FNJh8CTXvPzDTLxnKcao/uvTq0qak8T/A
J342psNxP85niF9spfM/wEi2l+vyqL9gfKNQxZv1P3TCFj/2gLu/qCuxGqeG
9z9o85W9JvnFv6oWikcNZfk/PAmWA2Hizr905J0A+jX7PzxbW8O3O9S/kvKi
lXb4/D+POAJYnlnZv/jgev+Tq/4/7S+GNy3I3r/hQoOvNScAQIRd20ZBQuK/
nv7SOw/wAECuAPWgyUXlv1zrkRLsrwFA+g8wShZt6L9mZmZmZmYCQKpMWOh6
tuu/SL/GTw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZuT+qTFjoerbrP508EBxe
5Mw/aZ00k88J6z9onLt0HWzWPxAegwVyLuo/rLYaKPZP3j96bGmk1iTpP5c4
zLbEDOM/puUwXort5z+xwZS9WuLmP5eKf18yieY/lLVDVLSm6j8WzrO7i/jk
P6If8svRV+4/G8yRCGs84z+RfzrV3vnwPwYmd+27VeE/EE852ka88j/MuqJM
AYveP2s0wPgxcvQ/3Aoy9aIZ2j+SoN/Utxr2Pz7+NFG4WdU/nN43Lve09z+a
ULaKxk3QP9a9fVcWQPk/NMa/RfbwxT8kXPqpQ7v6P4hgARmqcrU/0sfI9LUl
/D/AQLxyLlaAv+Ryluesfv0/gJEgceyJur880614ccX+P1iI5ivl/sm/AwsV
Rlb5/z/I1Y6MN5bTv6WIxvjbjABA3MqJAEtj2r9RRaC7/hIBQGiK1pCIseC/
hyl6P8yOAUDsKKSc6Ujkv7hSx9MCAAJAyjdS0eD1579mZmZmZmYCQKxMWOh6
tuu/fiDN1w==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz47+axwElsKDwPx+VxpST
Uty/TVtyt7Js8D/29gtI5ErSv6cEmzloG/A/SMz20lWpwL+A1gWb91nvP8A8
Pxb2rJg/lhu6FFBD7j8UJ4YJm5zGP79k47pt8+w/hCoeRqHw1D89U8nFAmvr
P8S27gOubN4/2mnabN+q6T+MoQO2td7jPzxMMXjxs+c/JDGUE/Ru6D8HEGrC
Q4flP+QXMjym5Ow/UpEKq/0l4z9+Brszt57wP5jryHlikeA/Lvs6En+78j+U
fgZmoZXbPxhBh8mLx/Q/fJCHu4Kn1T+4TDFKx8H2P3DNz+4ct84/HI33+CSp
+D8sZaLNN2rBP6okSz2ifPo/QEQYryV7mz/E3YUKRzv8PyBBRdniY7a/7o2H
Yybk/T/APkRUB3LKv1wRddhedv8/bFT1v2Ik1b9qEyt/jXgAQFilGj/iVt2/
vVkp8MgpAUD898FYFObiv62sKbWDzgFA6DNJmdw/579nZmZmZmYCQKxMWOh6
tuu/qzPEyQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAAAAAiCq2GlSv2PzGjTs6g
DsI/HUaGODWJ9T9/SZ3Bq+7RP7fmpkmzzPQ/Ek+qolix2j/4fqy/c/bzP/lD
FYpYpeE/2v2kSOgG8z/6KPmGEtvlPzkovf6P/vE/SPzHcJ736T/y3s8k993w
Px7nkMzN+O0/u/Pzt21L7z9Oq/9RQO7wP0BptKTprOw/Ppr9UlPQ8j/gk1DW
xeHpP3j5fR+gofQ/c8SjwH3r5j8+1zzUL2H2P37Vnrujy+M/Xi929xQO+D8+
fswt4IPgPyMJ5/Zrp/k/BiJDVuEr2j8wP42gWyz7PxhBIRFOB9M/kd/llRWc
/D8I+nmToTvHPwv+bLnW9f0/oJKrc8Karz/43iSW5zj/P6A+tb1QoK+/GsZ2
YE4yAECAh1SbSSjIv+CWvJkrvABAAH/RH6ho1L+/cVZQwjkBQJQfwYso7dy/
P60g4c+qAUBuqEuMkM7iv+X2vE8YDwJA6Ihv3vo5579lZmZmZmYCQKhMWOh6
tuu/zD/MxQ==
      "]], 
     {GrayLevel[0], InsetBox[
       StyleBox["\<\"\\!\\(TraditionalForm\\`a\\_1\\)\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-6, 6}, {0.1, -0.8660254037844386}]], InsetBox[
       StyleBox["\<\"\\!\\(TraditionalForm\\`b\\_1\\)\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-6, 6}, {0.5, -0.5196152422706632}]], InsetBox[
       StyleBox["\<\"\\!\\(TraditionalForm\\`c\\_1\\)\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-6, 6}, {-0.4, -0.34641016151377546`}]], InsetBox[
       StyleBox["\<\"\\!\\(TraditionalForm\\`d\\_1\\)\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-6, 6}, {-0.5000000000000001, 0.17320508075688773`}]], InsetBox[
       StyleBox["\<\"\\!\\(TraditionalForm\\`e\\_1\\)\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], Offset[{-6, 6}, {0.1, 0.8660254037844386}]],
       InsetBox[
       StyleBox["\<\"\\!\\(TraditionalForm\\`f\\_1\\)\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{-6, 6}, {-0.6000000000000001, 1.0392304845413265`}]], InsetBox[
       StyleBox["\<\"\\!\\(TraditionalForm\\`g\\_1\\)\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], Offset[{-6, 6}, {0, 1.3856406460551018`}]], 
      InsetBox[
       StyleBox["\<\"f(\\!\\(TraditionalForm\\`a\\_1\\))\"\>",
        StripOnInput->False,
        FontSize->14,
        FontSlant->Italic,
        FontColor->GrayLevel[0]], 
       Offset[{10, 6}, {2.3, -0.8660254037844386}]]}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
     StripOnInput -> False], TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 259},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->551218834]
}, Open  ]],

Cell["\<\
Large numbers of points in either the domain or range sets can lead to \
confusing diagrams.\
\>", "ExampleText",
 CellID->1837323731],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionArrowDiagram", "[", 
  RowBox[{"75", ",", "66", ",", "\"\<Onto\>\""}], "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->1232586704],

Cell[BoxData[
 GraphicsBox[
  {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
   0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
   {RGBColor[0.0692454, 0.645548, 0.968322], 
    DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
   {GrayLevel[0], PointSize[Medium], Opacity[1], 
    PointBox[{{0.1, -0.5196152422706632}, {-0.8, 0}, {
      0.7000000000000001, -0.5196152422706632}, {-0.30000000000000004`, \
-1.2124355652982142`}, {-0.7000000000000001, -0.5196152422706632}, \
{-0.5000000000000001, -0.8660254037844386}, {0.30000000000000004`, 
      0.8660254037844386}, {-0.5000000000000001, -1.2124355652982142`}, \
{-0.8, -0.6928203230275509}, {-0.8, 0.6928203230275509}, {0.2, 
      1.3856406460551018`}, {-0.9, -0.17320508075688773`}, {
      0.6000000000000001, 1.0392304845413265`}, {0.7000000000000001, 
      0.5196152422706632}, {0, -1.3856406460551018`}, {-0.6000000000000001, 
      1.0392304845413265`}, {0.1, 1.2124355652982142`}, {
      0.6000000000000001, -1.0392304845413265`}, {-0.4, 
      0.6928203230275509}, {-0.8, 0.34641016151377546`}, {
      0.5, -0.5196152422706632}, {-0.30000000000000004`, 
      0.5196152422706632}, {-0.2, 
       0}, {-0.6000000000000001, -0.6928203230275509}, {0.8, 
      0.34641016151377546`}, {
      0.9, -0.17320508075688773`}, {-0.5000000000000001, 
      0.5196152422706632}, {-0.1, 0.5196152422706632}, {-0.30000000000000004`,
       0.8660254037844386}, {-0.2, -1.0392304845413265`}, {
      0.4, -0.34641016151377546`}, {0.8, 
      0.6928203230275509}, {-0.4, -0.34641016151377546`}, {0.2, 0}, {
      0, 1.3856406460551018`}, {0.6000000000000001, 0}, {0.2, 
      1.0392304845413265`}, {-0.4, -0.6928203230275509}, {-0.9, 
      0.5196152422706632}, {0.9, 0.5196152422706632}, {
      0.7000000000000001, -0.8660254037844386}, {0.30000000000000004`, 
      0.5196152422706632}, {-0.5000000000000001, -0.5196152422706632}, {
      0, -0.6928203230275509}, {0.5, 0.5196152422706632}, {
      0.30000000000000004`, -0.17320508075688773`}, {
      0.6000000000000001, -0.34641016151377546`}, {
      0.8, 0}, {-0.2, -0.34641016151377546`}, {0.2, 
      0.34641016151377546`}, {-0.2, 0.34641016151377546`}, {
      0.2, -1.0392304845413265`}, {
      0.4, -1.0392304845413265`}, {-0.6000000000000001, \
-1.0392304845413265`}, {-0.9, 
      0.17320508075688773`}, {-0.5000000000000001, -0.17320508075688773`}, {
      0.2, -0.34641016151377546`}, {0.5, 0.17320508075688773`}, {-0.4, 0}, {
      0.4, 0.6928203230275509}, {-0.7000000000000001, 0.17320508075688773`}, {
      0.9, 0.17320508075688773`}, {-0.1, -0.8660254037844386}, \
{-0.7000000000000001, -0.8660254037844386}, {0.1, 0.8660254037844386}, {-0.1, 
      1.2124355652982142`}, {
      0.1, -0.17320508075688773`}, {-0.2, -0.6928203230275509}, \
{-0.7000000000000001, 0.5196152422706632}, {
      0.30000000000000004`, -1.2124355652982142`}, {-0.2, \
-1.3856406460551018`}, {-0.5000000000000001, 1.2124355652982142`}, {0.1, 
      0.5196152422706632}, {
      0, 1.0392304845413265`}, {-0.30000000000000004`, -0.5196152422706632}, {
      3.1, -1.2124355652982142`}, {3.2, -0.34641016151377546`}, {2.1, 
      0.17320508075688773`}, {2.5, 0.17320508075688773`}, {3.1, 
      0.8660254037844386}, {3.5, 1.2124355652982142`}, {3., 
      0.34641016151377546`}, {3.2, 1.0392304845413265`}, {
      2.5, -0.8660254037844386}, {2.1, -0.5196152422706632}, {
      3.4, -1.0392304845413265`}, {3.2, -1.0392304845413265`}, {2.8, 
      1.3856406460551018`}, {3.7, -0.17320508075688773`}, {
      3.8, -0.6928203230275509}, {3.5, 0.8660254037844386}, {
      3., -0.34641016151377546`}, {3.5, -0.8660254037844386}, {3.4, 
      1.0392304845413265`}, {3.7, 0.5196152422706632}, {
      3.9, -0.17320508075688773`}, {2.6, 1.0392304845413265`}, {3.1, 
      0.5196152422706632}, {2.3, -0.17320508075688773`}, {
      3.5, -0.17320508075688773`}, {2.9, -1.2124355652982142`}, {3.6, 
      0.34641016151377546`}, {3.4, -0.34641016151377546`}, {
      2.2, -0.6928203230275509}, {3.8, 0.6928203230275509}, {
      2.6, -0.6928203230275509}, {3.1, 1.2124355652982142`}, {
      2.3, -0.5196152422706632}, {2.8, -0.6928203230275509}, {
      3., -0.6928203230275509}, {3.2, -0.6928203230275509}, {
      2.1, -0.17320508075688773`}, {3.8, 0.34641016151377546`}, {2.2, 
      0.34641016151377546`}, {2.9, -0.17320508075688773`}, {
      3.7, -0.5196152422706632}, {2.5, 0.8660254037844386}, {2.7, 
      0.17320508075688773`}, {3.5, 0.17320508075688773`}, {
      2.6, -1.0392304845413265`}, {2.3, 0.8660254037844386}, {
      3.1, -0.8660254037844386}, {3.9, 0.17320508075688773`}, {3.3, 
      1.2124355652982142`}, {3.8, -0.34641016151377546`}, {2.3, 
      0.5196152422706632}, {2.4, 0.34641016151377546`}, {
      3.3, -1.2124355652982142`}, {2.9, -0.5196152422706632}, {
      2.4, -0.6928203230275509}, {3.2, 0.6928203230275509}, {
      2.7, -0.8660254037844386}, {3.7, 0.17320508075688773`}, {
      3.6, -0.6928203230275509}, {3.5, -0.5196152422706632}, {2.3, 
      0.17320508075688773`}, {2.5, -0.17320508075688773`}, {
      3., -1.0392304845413265`}, {2.8, -0.34641016151377546`}, {3.3, 
      0.8660254037844386}, {3.3, 0.17320508075688773`}}], 
    {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZyb8iCq2GlSv2vy3m52YV
erS/wYNTZK/59L/IrUm1SyumP0KA8w163vO/dBQovJ+1xT+E07HIi9ryv6a6
89ZjHdM/SDLTf27u8b/TaQdjw4jbP/owjnufGvG/vCFabzsM4j/7xpEej1/w
v5Tq6q/5Y+Y/ojfHVEF7778oVWSPTsnqP1KEaxVUau6/CuIz9OQ57z/mpJ9P
54ztv8sx7OWw2fE/NitHgHDj7L9Uqm+lshn0P95EQ5JJbuy/MtOxF8Zb9j9i
77yusC3sv25zsYK4nvg/BnYpHMgh7L/nVB62VuH6P5G5GiyWSuy/775Xrm0i
/T9q6OQ3BajsvxU7UzfLYP8/NG4brOM57b9jmopHn80AQMAC3yLk/+2/TZ+y
g0zoAUAC4+6Mnfnuv08YNFPX/wJA7rSztEUT8L87dOViqxMEQGA6CAYHw/C/
f5sUWDYjBUAcIwt5tYvxv6m7Lh7oLQZAGPl/kuZs8r8zMzMzMzMHQN5o19Ui
ZvO/ARTOBg==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAAAAAiCq2GlSv2vwdNS8wd
ub4/wYNTZK/59L8MBeyGbCTPP0KA8w163vO/B9fgqpyn1z+E07HIi9ryv3OH
wKMw6t8/SDLTf27u8b9QG+oXyCrkP/owjnufGvG/I4jA1aFy6D/7xpEej1/w
v/pQURZgyuw/ojfHVEF777/HXeV62pfwP1KEaxVUau6/OCRNrSXQ8j/mpJ9P
54ztv/5kHxnkDPU/NitHgHDj7L+H3aLY5Uz3P95EQ5JJbuy/ZQblSvmO+T9i
77yusC3sv6Gm5LXr0fs/BnYpHMgh7L8aiFHpiRT+P5G5GiyWSuy/EXnFcNAq
AEBq6OQ3BajsvyQ3QzX/SQFANG4brOM57b/9MyThOGcCQMAC3yLk/+2/5zhM
HeaBA0AC4+6Mnfnuv+mxzexwmQRA7rSztEUT8L/VDX/8RK0FQGA6CAYHw/C/
GTWu8c+8BkAcIwt5tYvxv0NVyLeBxwdAGPl/kuZs8r/NzMzMzMwIQN5o19Ui
ZvO/WZXMrA==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQzcgAAg/238u4flUp7fP+BOH3Ses8r+3XKq8ueyL4
ab9UB2fk3/Un9i/hkrrudevD/h/Hnc/vaVy4n832+Kxv9e/3b1m78URbwAH7
JXqynuLtb/f3TbZi+7r6kv259h2zmU+/3p8x54moZ+hd+0scfMbLfV7tl/28
Z5M3/xP7JzPrPpzhebk/+sj+dOHGl/YKcRu75Hhe7L/uqKQg8fedvfunNmMV
7+f7t+WvLrRw+GQfOv+qv8upZ/uXlG3MkmD5ap9xQ8TxcOez/Rph0klbTnyH
80ViFaOn9PyCq3+wevceB7+/cPP8S8t6FNkYHGD2zdPcXjk5idEB5p6bl1Y6
793N5ABz7xfh+UefC7A4wPzT5X37Z2wiqwPMv22liyNz17A5wMLjxb9v1mkf
2R1g4XXJ44jRHH1OB1h4poEBlwMsvAG1rMX5
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQbWIMApf338u4flUp7fP+RbVfX1f9P7zf+FH4mTPK
n/YfmFu706Cxaf+hfasmz/zzYf+vAvv3F0UP2Vczrp7t/ej9/ogZd7X/X79k
z7J4wvF/zO/2axlYHWU9d8de8LxyiVf8m/06Qf13bhY/tterF7vN+/XVfpXS
fx3X+l/YOx61NeA89nL/lPOLHjvteGvv7DnN+u6lF/uTgyrVDUs/2s+7+oWZ
W+bF/v7+D807Bb/YG5lN6Pq88Pn+EC2dc+EbvtnL7O2Yfj3l+f5VVy8brvD/
CefnF/DXuL37DVc/e23bjXNd/+DmSfmIu7+fyOAAs29b0oFYrjuMDjD3lGjf
6eBUZnaAuffXzgh3+zQWB5h/vvi0SDEsZXWA+fcDB8/DRffYHGDhMQ8UXEIc
DrDwuhW+f913e04HWHimgQGXAyy8AQt8zjM=
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz0z/eaNfVImbzv3wl5CYL
1tg/o47vCt578r/+XlneCqLeP3avYE9CovG/e86AeA9K4j8ru6MZw9nwv7TG
6eCPVOU/xHnky8oi8L+IVOmV6W7oP3Y8Hvd0++6/Apg0HHeX6z/kJJp709Xt
vxNZV2+LzO4/iqIY8kzV7L84dKtyOQbxP1fuU3dp+uu/FurtCrqq8j+676sv
nUXrv2DkM2LoUvQ/I0SGCUi36r9LrI1m4/31PySVZ4q1T+q/bQTHiciq9z+p
P+GmHA/qvwr0nDm0WPk/845opZ/16b8Ij31Ywgb7P2UEJwxMA+q/fKiRtg60
/D9wUs2ZGjjqvwc30Iq1X/4/HdlsSe+T6r9rlu31aQQAQK2ZVWGZFuu//tAy
JETXAECEvfCM07/rv5C3cW/5pwFA0viJAUSP7L98WyEaG3YCQA5E9K18hO2/
8C+4xDtBA0CgpPB0+57uv2KRtKfvCARA9Qc4cire77/MzMzMzMwEQJrHASWw
oPC/RybRzA==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQbWIMAo/3zzrOqLphwYf9TY/uZaywuLefRa+sL5Tl
/f46bnXWNzeu7i/hfru94/ub/RuSLFNcp5/af8/9uLWAzOv9D45Oc48I3bTf
yvNM0ouil/sTijbdzBTdaM8n7X/405/n+2+c606tqD9ln6BZc/Lrrmf7p6ma
zeMOvWZfcFPAb87ap/td/rO8Kmu4b89wd4t16dUn+x1YdN0mxD2xPxKp6vzZ
6Ml+GYv+VwLML+1TFKPm5Rx7vJ9r753Dbqvf2v9yPvpIasLj/dULBefFaH2E
8z8l7+gqr/sMV/+g9uWd7Re+ws0zOFDn+kj+B9y+WWVFEnuzfsHdo9Lm91lg
wx+4e2/dSEla+e4f3D/622wOrZrE4ADzr/Y057CSlYwOsPAoeLikQXY3kwMs
vALuSPOKHmd2gIXn2TMgwOIAC28A0tbOcQ==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQPWsmCJzcP+s4o+qGBR/2z1IWtbWu2bSfRa+sL5Tl
/f4XBVMW57Cusy/hfru94/ub/W0Xfp57O/uE/T3349YCMq/3K7/Y3O7MctXe
yvNM0ouil/uNzdgEvty7a88n7X/405/n+1sO6D9blfPYPkGz5uTXXc/2z7j/
fHbQ9+f2BTcF/Oasfbq/wOe8I0fvG3uGu1usS68+2d/+oqZz3dQP9kciVZ0/
Gz3ZX8q07va755/sUxSj5uUce7z/zZGQE5w2X+1/OR99JDXh8f5H7BUshyd8
h/MjT8l9vPvkJ1y9+xN/ZznzP3Dzpqs9Xe3R9g9uX0Nezv6JGQwOMPe4lt1a
Y93N6ABz74L5qU/aVzE5wPyT9DL/692jzA4w/8adLOtcfofFARYei0VubA54
x+oAC69mfn8Hj19sDrDwZAADDgdYeAMArCbOmg==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQPWsmCJy0n3WcUXXDgg/7S29Uv1pXfsVenEXfSk3j
/X4Oox/t+YF37DcU8xmq2bzd7/b5hdInrUf2wTyrbVLrXu83383El+bwzH7W
XvuH59+/3H/O/+XZF2mv7AOSds22nPhi/yMt43luc97Z75vfUhiT/Hw/0xP3
N4r5H+3PslzovZH2bD97PI+Q/9TP9ionZr4xnvN0v6GvxJW2/V/tpW0nC99i
frq/MVRNcfGb7/aOO0RuSs97sv/8aqb4Gulf9us0LC31s5/sn74rmf2V1x84
/6mAg+vhyn9w9bN51E/mBzM4wMzbwKy7qjuD0QFmn5Ejv+2iaiYHmHuqM5Z1
K/YwO8Dc63B/x/+KmSwOMP+ktWxaL7mY1QHm36RvFavKVrA5wMLjs46JE+tK
dgdYeKnueBxqu4TDARaekPDmdICFNwCtlb38
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQPWsmCNy0n3WcUXXDgg/7FwYtuW276IG9OIu+lZrG
+/1Z9Ulacv1P7DcU8xmq2bzdvybSvzNi4gv7YJ7VNql1r/fPU8wsObPsjf2s
vfYPz79/uV/qtvqs5Wkf7AOSds22nPhi/5ITPk1hjZ/s981vKYxJfr7fTLxa
PnTRF/uzLBd6b6Q92281yd616cQ3e5UTM98Yz3m6P6XBm33Xpx/20raThW8x
P92/pSMy5Jr8b3vHHSI3pec92c9033TSer+/9us0LC31s5/sP/V2mpVs3X84
n2eRpU31NQYHmHqTZQeSON8yOsDM85p7zFmVidkBZt/pWyuuWYuyOMDcI8Lk
oLpbldUB5t6bFUEzhYzZHGD++S/r47nZht0B5t/f/UIuAk4cDrDw6Dl29vY8
J04HWHjtC6x9f82WywEWnsZgwO0AC28AW/q6Lg==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz4z+axwElsKDwvxo0M03D
geU/5IalC3i0779rU7Jix+HnP9zsWC/aQu6/xMzV1vxR6j+5vGPTSu3svwBV
O3UY0ew/vpkEM3+067/yvbUhx13vP3KISUYdmeq/X7YdRlf78D8Wz/9pu5vp
v/eUU/Q2TfI/urbrD+C86L/Y5fRUz6PzPxSCcncB/ee/bLMkoGr+9D/fc9xu
hVznv3eKJO1QXPY/VC9QHcHb5r8UeumTyLz3PwIcpNX4eua/12mejxYf+T+H
xh3yXzrmvyaB3+F+gvo/JHoyuRga5r/m+Hn2ROb7P4qIV0s0Gua/D2p7B6xJ
/T8t5uuZsjrmv82BW4H3q/4/TO0/Z4J75r/VeoWzNQYAQN5Du0+B3Oa/PlDZ
Wia1AEB+CRzce13nv5ft9uNwYgFACJ/FnC3+57+EtSVcuQ0CQG6KEE5Bvui/
kGBw4aS2AkCOMogFUZ3pv6Xi3dLZXANAkXH+Z+aa6r8AAAAAAAAEQKpMWOh6
tuu/FdbQiQ==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQnZ4GAs/2r/KJeFG17fX+kiWL0tv9H+2ven1Kp6Dy
5f4nhb3ZQfZ39r864Ri2OOf5/n3izRKcmy7vD2GVuaU18en+PfozPlR8P7p/
A5siI/OLx/sL/M7Vx3Ms3N9yY3Eib9Gj/RIsdYbhqbvtrz27kC5g8HB/Qt38
fK+75+2fTUt9s03pwf57M+Y1C0jdtH9rIrxFfuq9/bwXfwrFaj+0L9H7ckg6
4O7+lywx2++ef2q/ZW9sJV/5nf3XO1dOEK54Zf+AyYQpm+vO/sP8HzjaFN/D
+RGn5D7effIRrt7tib+znPkXuHnT1J6u9mj7BreP4e6NOuWLP+Du6XqzJbJc
7DfcvQ72p0/wh/61h/nnyKX7r3Z3/7eH+TfuZFnn8jsMDrDwWCxyY3PAO0YH
WHg18/s7ePxicoCFJwMYsDjAwhsAQRXJuQ==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQzcgAAg/2r/KJeFG17fV+mZoKJqdL1/dXvT6lU1D5
cr+BV6Mt99Nz+1+dcAxbnPN8/wntm3ofp+zeH8Iqc0tr4tP9D/y4PVgF59pv
YFNkZH7xeP8f1mU7G8OP2rfcWJzIW/Ro/yVfnZlKyy7bX3t2IV3A4OH+P91z
WIp237F/Ni31zTalB/uvbjJdkff5kf1bE+Et8lPv7S82T6g8PPG5fYnel0PS
AXf3+2cfknUxe2O/ZW9sJV/5nf1yP9p/2+R/sH/AZMKUzXVnv4j1avNvhz/B
+T1/A1UEJb7C1VeJN5cFZn2Hm3cqUuJ+9c6fcPtMBHg2hbH8gbtnn/yLnlWe
/+DutbL8X6vowuAA88+fJs7+8nhGB5h/fyQJKDuWMTnAwsN2XaHvq3ZmB1h4
ya58cfPhZBYHWHjOmgkCrA6w8AYAHPzDIw==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQPWsmCOzcv8on4kXVttf7GW4IFlWWzrGven1Kp6Dy
5X72lxu/qvsdsX91wjFscc7z/av8HvvGbrlkH8Iqc0tr4tP9XX5S71Zm3bbf
wKbIyPzi8X5fPzPrtUqP7FtuLE7kLXq0f/vnJMl785/ZX3t2IV3A4OH+M0LS
51u1Xts/m5b6ZpvSg/2L65lKrQ+8t39rIrxFfuq9/QpNYsoe+p/sS/S+HJIO
uLufb457aUfjF/ste2Mr+crv7G+Ne5e4+Oo3+wdMJkzZXHf2Vy8UnBej9RPO
/5S8o6u87jdc/YPal3e2X/gLN08iYb/SR34GB5h9vtaWPXGmjA4w9wg5L6/k
iGBygLk3N8doY0g5swPMP/rbbA6tmsTiAPOv9jTnsJKVrA6w8Ch4uKRBdjeb
Ayy8Au5I84oeZ3eAhefZMyDA4QALbwDMjMMn
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQnZ4GAs/sV/lEvKja9nq/zIbA1sxfL+0XTLdpETz4
cr+u3KcP3kve2ifNbPG49/X5/vNBi5n4Nn6wt+t7fPtK6LP9J6LWH30345N9
wXv2fxcfPtl/J7ox+WjHF3uO9Cqh2bMe77d6suVIV/03+0mXH26/2/Bo/6VC
UxWTmh/2C94+ztaf+nB/wcZTczbX/rLfV3DZmvfGg/1l3r47fjT9sX9xN7Tp
tueD/WpOFgwfu/7Za0h6pJy+en+/uthSdxcvBocNU5lj9pTd31/c/CUw1J8R
zm+rqJVUDGFygKm//7nIVjaC2QFm3iMHk9P7o1gcYPZteFDXXh/N6gBzT626
vfmVSDYHmHvXvL7x3CyU3QHmH9M3B82UfDkcYP41/uA+Q8iR0wEWHvxCsXPb
DLgcYOE1WdfV4r8UtwMsPBnAgMcBFt4AC3DKqg==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQPWsmCLzcr8S1tm2q9rP9x4QPdWkcebqftf5GWbTQ
k/1/njHbeFx/uH/eu+250WqP9pdK6u2SXnV7v2N7SOWsjAf7J3f+emxcf3m/
QOEla64td/efOBB1Iibl2P6aXcc33P12a/+HwLMy06yX7Y8wjYlRnn1jv4JD
3RG5T9vsy/5/miK96Np+ix9JEypWnbFXaWsLusRzdf8Rk8QA6X/X7IUKKhfI
3bm8nyVsZkvPpvv2cWk12osELu+fF86674fZE3sL96OvT6y6tF+v9Djflakv
4PxTig1dbZ/fwNWbSvpNur/sA9y8BPaoEPGLn+D2JTfJVbl8/wJ3z9cLPuLO
Et/tYe6t31f4/6PRT3uYf05zpMYbu/22h/n38JuPIW8D/trDwuNpZEYLV9B/
e1h4qex4HGq7hMEBFp6Q8GZ0gIU3AEzgz60=
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQbWIMAo/3K3GtbZuq/Wz/gajtHi277+1nrb9RFi30
ZL8uo/Xqww+v7Z/3bntutNqj/UEzD92au+vsfsf2kMpZGQ/2y3zeFjPn5K79
AoWXrLm23N3fcOxb57OVM+1rdh3fcPfbrf0yrmlCW7iO2EeYxsQoz76x/0rM
m9qQzkv2Zf8/TZFedG3/C49/IpxKt+1V2tqCLvFc3X+CYTpfydOH9kIFlQvk
7lzenyG4ecUa+2f2cWk12osELu9n2ZetEj/3lb2F+9HXJ1Zd2j/ltm6p1Z93
cL6ES3HFtzUf4eozfBqPCt36DDdvslVvuxfLN7h907YGri3X+gF3jwZLg1eZ
9y+4ezd9XGKkmvYH7p9/1jMmpVX+s4f593e/kIuAE4MDLDx6jp29Pc+J0QEW
XvsCa99fs2VygIUnOLiNmR1g4Q0Aih7IKA==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQPWsmCNzcr8S1tm2q9rP9n3pfVW9/d3E/a/2Nsmih
J/v3ZdzZ81bzyP5577bnRqs92n/j32IWwZYF+x3bQypnZTzYL+NQc21a5g57
gcJL1lxb7u7PKbqRde3cGfuaXcc33P12a3/M+/9X1S5ds48wjYlRnn1j/yLN
nV6KYffty/5/miK96Nr+211PP2R9f2yv0tYWdInn6n799L8lt7xf2AsVVC6Q
u3N5//lySX7hZW/s49JqtBcJXN5vJpR59FHjB3sL96OvT6y6tP+fwqncgI2f
4Hyf8mWrNR98gaufU7/lhyvvd7h5x3IP7Ko1/wm379SLlgerYn/D3RNjvrl2
Zd1fuHufqdxIjZj5H+6fmdufPvK5xuAA8+9UzTV3V95mdICFh1pCeqnFHSYH
WHhFvBbrLLjO7AALz7NnQIDFARbeAGl41c4=
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQPWsmCJzcr8S1tm2q9rP9U1jK96xq37Kftf5GWbTQ
k/0Fh78Urd+31H7eu+250WqP9j+ZWRAxteKovWN7SOWsjAf7r9zJbjJ6fsle
oPCSNdeWu/uZ2SyZLExu29fsOr7h7rdb+0Xi0i5Wnn9oH2EaE6M8+8Z+i9+H
uMsFn9mX/f80RXrRtf1OH73DL8a+sldpawu6xHN1/7SzybcdNr2zFyqoXCB3
5/J+6ff7d9m0fbSPS6vRXiRweX+m65wfols/21u4H319YtWl/YYhfxc0P/kK
59evunkvRvgHXP35Tc+1Kx1+wc37teDz2w1Zf+D2/ZXeIXxvwj+4e9xNiyPe
RTA4wNzLs6b1jGsaowPMP0aB9TXP8pkcYP7VO+xa7lzK7AALj/0/GfgvlbI4
wMLrY8sGJa5CVgdYeKaBAZsDLLwB9q/KTQ==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQDQNKXGvbpmo/279Ab09Zx+p99qz1N8qihZ7sLzsV
VubKed5+3rvtudFqj/bvl2StFO+8bu/YHlI5K+PB/oUrLfz/b75vL1B4yZpr
y939GS+b0psYntjX7Dq+4e63W/urj5wxf2D6wj7CNCZGefaN/fMSNYvulr+x
L/v/aYr0omv7r2jfvCed9MFepa0t6BLP1f3/JJ8uCO75ZC9UULlA7s7l/X5K
n9/m7/xiH5dWo71I4PL+ORXntT1ffLO3cD/6+sSqS/tT2g2ubBP/Cedvussj
NMHtN1w90zOp+DXFf+Hm6V5RU3wy9z/cvhnLSxdzPGJwgLnn4TmeT+2fGB1g
7l3mKp92/z+TA8w/Z15JiDVwsjjA/Hsi5r7AXT5WB1h4RE6etSKbn80BFl7d
cl57FnOxO8DCExLaHA6w8AYAkRO/iw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAZ2ZmZmZm5r+axwElsKDgv8OXo1KC
S+G/hR/f3uk93L9zOIhbhynYv0w2xw2/ste/fs+YuroTy7+PsagDSaLTv5BD
nr1N7aW/dRzZ0a8O0L9QJtnMsWrAP1TvUpKy88m/Mg3DNltJ0z+ifEDwz8rE
v34Dp3B8eN4/gNrGmXSlwL9BVcSzZt7kP9iNIW2nC7u/ecqB2CmI6j+QELEd
K9u2vzANOjRDG/A/eIOQVK27tL8KXmNaPPPyP4yTAnJOrrS/pleuG37K9T9o
PzWOFbO2v86AW5aGn/g/+Cx+dfDIur8QCLoW1XD7P5w17BzadsC/GB9H4+o8
/j+KNqssD4/Ev/xg4AMmgQBAOAB2POuqyb8kFWAPwN8BQICpUlm4x8+/r3q4
jIk5A0CPdYQdHHHTvwWDfQLLjQRA3xFX/lJ7178SMZvmz9sFQCIWTw5cANy/
dN4g/uYiB0ByVbSw6H7gvxQDS7tiYghAFKUWk4Y447+bmZmZmZkJQCIKrYaV
K+a/2m/EeQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAQAAAAAA4L+axwElsKDgv3zqofhb
R9W/YHNth0j227/y607RxafEv9RlhBuEKde/wAugWHWTij+mKOEBn93Sv1Af
wjRoWMg/bmfkqcEpzr8ikAbB4azXP1DSpTCXose/a0+RFkWo4T8uox5QNSnC
v4HM3i6AiOc/JFqUNQeBu7/RFq4EBHTtP8g0+uWx1bS/Kho/O9az8T/k9UGa
9FOwv75TW4gosPQ/MJl3/Wb8q7++EE9dY633P+jbxnxXq6u/ij5csvCp+j+Y
Myy15bSvv9rT09s6pP0/GHapn3YLtL9ctqqwVk0AQIxQabZ9Zrq/8us16trF
AUAAihdpFHTBvxZbClPiOgNAyFJ5PfrFxr+672D9pqsEQArQMLkdJs2/2wcy
PmUXBkAM561sjkjSv820BxVcfQdAISlwAYSB1r/a0GWSzdwIQDiNhwyyO9u/
8JKRPP80CkA27ZI3Szrgvz4ng3I6hQtAAc9d57UU47/OzMzMzMwMQCIKrYaV
K+a/yZTFpA==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz07+axwElsKDgv147qlce
9cC/YHNth0j227+YtiohT8ejP9RlhBuEKde/VJoj79BCyz+mKOEBn93Sv3Tc
LecA+dg/cGfkqcEpzr92rulG1zziP1DSpTCXose/0LX3fKsO6D8yox5QNSnC
v+YyRZXm7u0/LFqUNQeBu7+bPoo1Ne3xP9A0+uWx1bS/XE1ybgnn9D/s9UGa
9FOwv++Gjrtb4/c/QJl3/Wb8q7/vQ4KQluD6P/jbxnxXq6u/u3GP5SPd/T+o
Myy15bSvv4SDgwe3awBAIHapn3YLtL/0T0RK8OYBQJRQabZ9Zrq/ioXPg3Rf
A0AEihdpFHTBv670o+x71ARAzFJ5PfrFxr9SifqWQEUGQA7QMLkdJs2/dKHL
1/6wB0AO561sjkjSv2ZOoa71FglAIilwAYSB1r9yav8rZ3YKQDiNhwyyO9u/
iCwr1pjOC0A27ZI3Szrgv9bAHAzUHg1AAc9d57UU479mZmZmZmYOQCIKrYaV
K+a/LaDI3w==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQPWsmCOy0n3WcUXXDggf71xpu8/zFetx+x9VJ94VK
7+23+K/7+NK+C/YtWemXhZ7e3j+hXGS7gvg1ew4zC4s9k27ur7luNlug87b9
nvJ/GV711/crq5hW3O94YM/yb2Hi9dVX97dyCzLLhT+2f+MfpRkteWX/J7kl
V47pPrNn0BHxX33i0v4fh0JiIrhf2pcYKFZU7bq4/3Yof7zvu9f2GWdyGSPe
X9jvuu3MUdmr7+w/HM3fvD79wv61U85tk7//wV5n49eOk8oX9rfPS6kRDvkE
5wve8AlYcuIzXL1bx+pmHpuvcPN2XORgbV/7DW7fLuWHJ/xlfsDdU7h2leGS
9p9w9yotlCg//O4X3D8htak87wP+wP0bIJrHmr32Lzw80janV9Ux/4eHV73Z
vy+WmgwOsPA8ewYEGBxg4Q0AVHrNGA==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQDQEP7GcdZ1TdsODBfklZv/y7Po/s7zWymG/Zc2//
pIuR2aeWP7H/Y7XqqG/enf1vdm3haud/bv9533/LtPBb+x8bHZl+pumlfU3C
01TJyhv7b0RI2/kxvLHv7KkQSr1ybf/jUpHdO9ve2S+zya7UyLq6fwPzMo6Y
rg/2O5smnVO2vrLfR8pfvvzKR/vdbK9vfne/vH8Gv/Kvfwqf7cUmey5SnXhp
v966OA3n/C/2qz4/0eATv7R/QnWN76R9X+2nVN01/XTz4n6bwIB6K77vcP48
ubwlnbE/4OpN9q68dHb1T7h5jZtq14b++AW3T+ak2pECpz9w99ye6PQ+v/Mv
3L0JCvvXLj3zD+6ftZGlDptZGRxg/l1tvnNxyyIGB1h4MH3qP11sw+gACy+F
+5O49C8zOsDCMw0MmBxg4Q0A/ADKHg==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQnZ4GAs/sZx1nVN2w4MH+BPcTT68WvrBn0SvrC2W5
v1/Rdtu/I52v7Eu4327v+H5n/6rLFcHia9/Y33M/bi0gc3v/oslt1atuvbO3
9jyT9KLo5v7Jk8/6re74YM8n7X/405/r+9/tiWj7vvyjfZJmzcmvu67t33zk
WPHCU5/si24K+M1Ze3X/i/kOd/k+fLZnurvFuvTqlf0Cjt2CS8S/2h+NVHX+
bHRlP/u5xVZHHL7ZpypGzcs5dnm/k/73DRezvtv/cT76SGrC5f17L3D46039
Aef/MIw5ar3/J1z9h30l76Jf/IKbJ5GwX+kj/x+4fT7Wlj1xpn/h7hFyXl7J
EfEP7t6cHKONIeX/4f4Rj5ZLDPVkcID5V/S0ovauKwwOsPDYXhB0oC+W0QEW
Xurvetu6HjM6wMITHNxpTA6w8AYAEMTNYw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ2b8iCq2GlSvWvw5lnRFl
7NC/5p7dIQP/0b+YzCE1tiHAv0VFckJKccy/ADAfgySKjj9+jlGL+LTFv6SR
TtesPMQ/jEbdFEiZv7/INTjwV2jTP6R4WmHedrW/yPE97WfN3D+A++MnKRCq
vzZufiFFJOM/sMV+gJBFmb9R3dONW+rnP8BO2FPPZXW/q1lZy2627D9AOLtV
xmV+P+yfPJ75wvA/IGBbuoEYiz9QrAa4LSvzP+B+0AOk94g/XgkLvoyS9T8A
QHZE1qJxPw7uWCzQ9/c/4OsRTZdRhb9yQE6dsln6P0hOl21ZWKC/DFs/dvC2
/D9QyOTCp9Guv7hLK5NIDv8/3Px9ZVFcuL+PuJB4Pq8AQMArC7PygMG/WQZ/
qynTAUAeOGPZ9KnHv9sgEv5K8gJAHo5qY+qlzr/phG4WCgwEQNQrT1CPONO/
LfQBdtEfBUA8l9p0t4PXv8TJ1cgOLQZAiskraSYy3L8zMzMzMzMHQJrHASWw
oOC/jMzCEQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZyb8iCq2GlSvWvyg+/SgE
z6i/mkb6csBv0b90R4/5gza7P2YDZ3leTMq/7fxuZ3bk0D/UajOEUKLCvz0s
BENFJds/lACmLdDKt796sU9sTsXiPwgJTLXhZKi/JiSYhnwH6D+Av1SbUA2E
v1qdLqhiVu0/YBFMb38JlT8IfaDkl1fxP5BaIjIcK6Y/f+N10IYH9D/g0XPY
kumtP9T5SxORufY/0Nn04gPesD+G3qqKSGz5PxBbuOc50LA/86MzuD4e/D9w
XguX5ZatP/NgFoQFzv4/oGAOhIGhpT9K66j/F70AQCA5SkABiZM/usyqkqkQ
AkDAjx2ZRueHv3cAUE4DYQNAMI4W0xqRqb8j1Pi6cq0EQLDxjuV6e7i/GYfY
dEf1BUAO/ggWvgfDv9PqjInTNwdAIevOb66+yr/fKmrUa3QIQP4ZwEw7r9G/
jcNJWWiqCUBM9lSQQXHWv394rJ0k2QpAPnFIuuOi27/+//////8LQJrHASWw
oOC/XbvC3w==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZyT8iCq2GlSvWv5d5oTTd
/NU/wPJrymG30b/JdenxYl7fP1Wk7F3UXsu/nP3h4jF25D+pfMKHpCvEv0Td
quvmUOk/kKNBIQyyu78u5jUyvTzuP5A+AJ6n1LC/Q8dNH4yb8T9gK7lOfROf
v+RCa/KpHvQ/ANHKRETwYL/YNcXVYqb2P8BQp0difpM/KUgRG18x+T9wnJXR
JdugP+BMxFhFvvs/2EVAGhRBpD9qxVghu0v+P/BqMmgu76M/VKuP3TJsAEBw
JppfQMufP8HTG2x1sQFAsOXJsBtNkD/CBSin+PQCQADHyDVjT3q/IT3l5hA2
BEAgFpa0vGWivyvT8ssTdAVA+KGEZ2+Ssr9axsSZWK4GQJik0iWwvr2/ucYr
kDjkB0AUG7Z32VjFv9eFz0MPFQlAn7ycaUyyzL/lV2z1OkAKQOiih07lc9K/
3dul5xxlC0DExpeCfPrWvyIhQbMZgwxAZB26EYXq27+ZmZmZmZkNQJrHASWw
oOC/KkrGPQ==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQPWsmCNy0V+Ja2zZV+9r+dpv94i9237eP+hGRy3rh
0v4nfS4bbzE+tmfpFAtfUnl+/+aCDINrZs/sHe2WPrJ6fnL/coHaN0bVL+3D
nbaEH359ZP/bt7PlTc++sb+43abyUeeB/f+2yD3uEf1gP+1ou/j19p37lx1w
Pt598qP9l6fMxvvMN+8v66x6ur7xs/2Kb9s7l/uu2W9U/Wozv+1X+wvn/z7q
X79k/xK//o/7f32zXxD2LqjBaP7+ugQe5Ss7f9gfmPJzzzWpWfu3XVITiK/8
Bef3PVnsWGzxB66eaYdHvfK3v3Dz9q3c4du88T/cPred37de3sLgAHOPtD2X
xuU+RgeYe+XmbQr0SWNygPknP7Tq3hkrZgeYf3Nr63mfcLM4wMLjQanHW6Wb
LA6w8OLYtHCazCJWB1h4poEBmwMsvAGQ3dIC
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQbWIMAo/tlbjWtk3VvrZ/FYtbUZTLM/uoHxG5rBcu
7ff5ukrcMeOlPUunWPiSyvP7pa6fm2Yz9429o93SR1bPT+7nK38cNPPhe/tw
py3hh18f2a+lxXj47MyP9he321Q+6jyw3+hFoNgBj8/20462i19v37n/1uey
X/v+fLH/8pTZeJ/55v2r9qyVeLzlm/2Kb9s7l/uu2Z+6Tva5U8EP+wvn/z7q
X79k/43GQyqfdX/ZLwh7F9RgNH//xsn2Yexvf9sfmPJzzzWpWftfskY5T1rz
F84/JH69ZFn2f7j6WV9tI1dPYXCAmVfxrtQhuojRAWbfg6CJ/rl+TA4w92y5
sfhgjgazA8y9O8x9Xj/9z+wA8w/ne+GKtCssDjD/sotLLq9dyuoAC48q/kdt
e4rYHGDhtcjb2mCbNbsDLDwZwIDDARbeABe9ylo=
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAzczMzMzM7L8iCq2GlSvGvwrCHcl6
cOe/86QBA8wqt7/IhyBBvvbhvyB265rSvY+/OpnU7v3E2L/sC18mtG+qP+AT
cf7K2sq/sFnrxtJavD9glzCJfcuev/ZBtNyTtMQ/fAoSJ+Ftwz8YcTu99S3K
P2r/I8Rmd9U/MuePcqeWzj8FSuE3N6fgP/F8Lg0p9tA/h2exqd+a5j+qjBxg
mBbSP6/a6UOEk+w//Rb+pgis0j9Yquj2/EbxP6YuFJcqttI/JNj1S4pD9D+v
gwfQ+DTSP3RtbXXUPfc/nazC3rco0T9SBu/6RjT6PwBsJTLsI88/fnEFbk8l
/T9GikKkFuPKP+Mn1x+vBwBAfMHgzzCRxT+HvC3Kc3gBQHSIUqgaYr4/qNT+
CjLkAkCwGPnPOBivP5qB1OEoSgRAoL/HsJ57db+nnTJfmqkFQFYMahdyQLS/
vV9eCcwBB0CSoPHQAZLEvwv0Tz8HUghAvycdkKz7z7+bmZmZmZkJQCIKrYaV
K9a/8JjG9Q==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAQAAAAAA4L8iCq2GlSvGv7pietg3
ytW/i1XIYFFJuL9O13YddbnGv5DConGYcpi/gF7zDxKih7+cxCIYZEqkP7QI
MipGHsQ/tLZP05ZzuD/yXzmzJQLVP/AkB3t4Y8I//ezHARQL4D+ilxkdW4zH
PyPouZ6kouU/wjmTc7axyz+luyoazUTrP1hNyVZX0c4/bhh0H0h38D/+xUC/
ynTQP2JAbWd2TvM/Q+mIMar80D88kZaNbyb2Pz9lLOoBANE/2orhTrH9+D9J
uh8j0H7QPwK0jsm50vs/xP2a0rLyzj9CO+1JCKT+P6bebfBQ4Ms/JSk9Cw+4
AEC63a7gG8jHP5b6eZ2/GgJAChTk0D+swj+9rvmoWXkDQIjVDmjlHrk/SBRS
JiPTBECUpERJy9OlP54cF5xkJwZA4Hugetf7lL+ryjSAaXUHQAAwiB4aU7e/
DXi6l4C8CECDQXe1d6TFv62c5FT8+wlABkCAn3dF0L80MzMzMzMLQCIKrYaV
K9a/jnm5rQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZuT8iCq2GlSvGv26LE65S
GdA/JQaPvtZnub8QBZcx1ADaPxTl58ojg6C/EBhRzdQL4j9o+swTKEqcP7ev
Gzs8LOc/rBO031qMtD8SS+kF6F7sP+AHWhldEsA/4N4YEIvQ8D8ovvd8wOrE
P3ob5CD+d/M/ToyWdMXMyD/VSW2xZCT2P8SgNZNctss/S7BCnVPU+D+a/sk8
+qXNP6DGGOBdhvs/CncneJeazj9Sq3dXFTn+P6o3iXqyk84/XziAwoV1AEC+
4W/sTpHNP9+WcShpzQFASqKw5/WTyz+wUQ9mfiMDQEZRtq61nMg/ITMR+Q93
BEAmMRsdIa3EP95mtrRpxwVALM3Oo52Ovz+JOl8h2RMHQJQiyyew27M/f+0+
261bCECwYCCFux6ZPzlR8+85nglA/IOHpGNMor9FkdA60toKQLJTpiXCZbq/
8ymwv84QDEB14vyZ7bbGv+XeEgSLPw1ALOzx9hiN0L9kZmZmZmYOQCIKrYaV
K9a/1Hi8Jw==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQbWIMApftlbjWtk3VPrY/Lfe84Lsz1+3L3q9LsFU6
uD/i2jPOE0137FMOf/n9gGvn/rYQvyUKKx/Y21/2Ouu0fsP+ujOxhmkyj+0P
bDr0Uy5n0f6iT1uuP5/51H6DT0HlR8nW/XX860KCFV7YT1hvuWKhbof9zwVP
VwWue2W/omxFozT7fHujc+Y32p3e2rtIC6S9lVluf3rH/d7pt9/bZ+yQVq6/
vMb+T5X2V36bj/ZPIhXKZYM22Md5eErFd3+yd7Czc+5122jPKSfrw377M5yf
/dqqMEzrK1w9U1fZ/FkV3+Dm+dd7VSkd/Q6379XUzxNs+X/C3bMijn+PTdgv
uHv13nZXFc38DfePRsN2X+Mbf+D+VbGxF7km+A8eHpc2vEne4vofHl7trj8L
ployOMDC8+wZEGBwgIU3AKYLvso=
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQffYMCLyxV+Ja2zZV+9j+5YVOMeemvbPnW1C675DD
of0RIZwn75t+sFdx5+7bEbJ3v0mZ3Llg1Y/206Tab0+6sm2/YdUSfRfJT/bR
8yVS2lk27t9V1VunL/jZvub3kezw88v3v7JT15/E88W+Yz+D3k2mefu3tT6/
94zrq/2FAy53XvT37W9VNZq9jPubvcOkKxnd67L3dxR91vDk+27vkPNrlcbW
UvsHb1/4uAr9sF+QsD3ab2eb/ZdHhSKC4j/tFWb8v2+3qsfeIeCgnJLMLzg/
2NXr2VuF33D1aw7Yc71S+QM3bx0v44we9b9w+2rWeof4aPyDu8frCzfLQrX/
cPfa3Am6xSjI4ADzT2PM8cbvPQwOMP82xHVtes/G6AALj7z3IltX1jE6wMLL
4G8H07PPjA6w8EwDAyYHWHgDAL7ev8k=
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ6b8AAAAAAAAAAFR/m1X/
QuW/8Kw9k0mysD/zPxVaOtXgv/6dz5XBy78/GqCAdUil2L/GhQiCMqLGP5Ch
5FuG9s6//nDrAoeKzD9Ij4WlBsW4v/lrVu7dzdA/cMEinXKeqT+yirBhkOnS
P6SFxlLhXck/xx2lfjyX1D8IIQ6CHTvWP+ioXUn+1dU/vBkZ/UPT3z8E9wOg
LKXWPwxzrG8mueQ/JOV/lFkE1z/Si0CjjonpPxqOy6ZS89Y/7kVJr0xY7j8j
477fIHLWP6iH8sVpkfE/w3pEzAiB1T8L2uc2TPPzP1kg+liKINQ/pfTYD4pQ
9j8ZcVCOYFHSP1HlxCzip/g/7IpNLYEU0D+3CruKFvj6P4boTlo41so/S6aX
8Ow//T8n3PYzNq3EP0/bvZUvfv8/UAzfU4Fiuz+2UTvj1tgAQHzy7rIxmKc/
+sDOQp7sAUCQ0djiHoKVv5CWopXb+QJAnv36iUMauL8AAAAAAAAEQCMKrYaV
K8a/Q8fH8w==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ2b8AAAAAAAAAAA5lnRFl
7NC/8Kw9k0mysD+YzCE1tiHAv/6dz5XBy78/ADAfgySKjj/GhQiCMqLGP6SR
TtesPMQ//nDrAoeKzD/INTjwV2jTP/lrVu7dzdA/yPE97WfN3D+yirBhkOnS
PzZufiFFJOM/xx2lfjyX1D9R3dONW+rnP+ioXUn+1dU/q1lZy2627D8E9wOg
LKXWP+yfPJ75wvA/JOV/lFkE1z9QrAa4LSvzPxqOy6ZS89Y/XgkLvoyS9T8j
477fIHLWPw7uWCzQ9/c/w3pEzAiB1T9yQE6dsln6P1kg+liKINQ/DFs/dvC2
/D8ZcVCOYFHSP7hLK5NIDv8/7IpNLYEU0D+PuJB4Pq8AQIboTlo41so/WQZ/
qynTAUAn3PYzNq3EP9sgEv5K8gJAUAzfU4Fiuz/phG4WCgwEQHzy7rIxmKc/
LfQBdtEfBUCQ0djiHoKVv8TJ1cgOLQZAnv36iUMauL8zMzMzMzMHQCMKrYaV
K8a/jfW8HQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZyb8AAAAAAAAAACg+/SgE
z6i/IA7LTlTvsj90R4/5gza7P94Q85PMCsI/7fxuZ3bk0D9wqSaJ2rTJPz0s
BENFJds//YlDe+E40D96sU9sTsXiPwGJA1D5HtM/JiSYhnwH6D8mZNIBK4vV
P1qdLqhiVu0/OMuhfS181z8IfaDkl1fxP3RV8Qz58Ng/f+N10IYH9D9fhLvh
x+jZP9T5SxORufY/l0BqfxZj2j+G3qqKSGz5P+cgmwCkX9o/86MzuD4e/D/w
dY45ct7ZP/NgFoQFzv4/NtYut8Xf2D9K66j/F70AQLStsZolZNc/usyqkqkQ
AkClHeRRW2zVP3cAUE4DYQNAXThKLHL50j8j1Pi6cq0EQLdNSc22DNA/GYfY
dEf1BUA3FlH3bE/JP9PqjInTNwdAJCmLnXyYwT/fKmrUa3QIQJTAs+do8bE/
jcNJWWiqCUBACvtpAmtxv394rJ0k2QpAbZxtzjjdtb/+//////8LQCIKrYaV
K8a/ypS8kA==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZyT8AAAAAAAAAAJd5oTTd
/NU/iV0E8c7QsT/JdenxYl7fP+9vba9W+MA/nP3h4jF25D+bl5eFhivIP0Td
quvmUOk/fEK5/CR+zj8u5jUyvTzuP376LJ9r9tE/Q8dNH4yb8T9sd8GxXTrU
P+RCa/KpHvQ/gHQj/rQJ1j/YNcXVYqb2Py5/J6t7Y9c/KUgRG18x+T+xvd9A
+kbYP+BMxFhFvvs/3hL1Cbiz2D9qxVghu0v+P4BXs1N7qdg/VKuP3TJsAECJ
rKaMSSjYP8HTG2x1sQFAfqi5QWcw1z/CBSin+PQCQAbn1flXwtU/IT3l5hA2
BEBfRxrw3d7TPyvT8ssTdAVApeHLrPmG0T9axsSZWK4GQPrBcPrSd80/ucYr
kDjkB0Ay+aOVUf7GP9eFz0MPFQlATq96R71Jvz/lV2z1OkAKQNQ5K8GBva0/
3dul5xxlC0AQlFd939yJvyIhQbMZgwxAAk00LL77tr+ZmZmZmZkNQCEKrYaV
K8a/yljCFw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz4z8AAAAAAAAAAFsRRYdf
SOc/sfjtaognrz8Ekq2fJnPrPx5cxMzVpr0/GC9qdFKx7z/yc3l+3hjFP2LP
l1lRAPI/gp8dCemWyj9nwZr4Zi/0P+i6/3qgSs8/PMvqcMFk9j/5nZ8Rw5jR
P+zErMg0n/g/EMcm/8Mk0z8TQahRkt36P6PSk+eASNQ/GqvhSake/T9WMCj/
XgPVP5SMTn1HYf8/bLcKH/tU1T9oFif0HNIAQLLE4/kpPdU/1ipIrSbzAUC8
Gdcy+LvUPxznCY0nEwNAC03PVqrR0z9+B/PChjEEQKtZHri8ftI/uOomXKxN
BUDUmoYs48PQP1tFBZQBZwZAamieWxFEzT+Sdykk8XwHQBQPLbqdNMg/xhKf
k+eOCEAev0nSGlzCP6yIIYVTnAlAWGlDYEV7tz+6fj0EpqQKQCyrsqThcqE/
Stkq0VKnC0AQ2QUHzhWevzI/N6vQowxANK7B58g4ub+amZmZmZkNQCIKrYaV
K8a/EKqxsA==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ6T8AAAAAAAAAAMB3q+3F
ru0/sfjtaognrz81/AmDxuzwPx9cxMzVpr0/v0pobdwL8z/xc3l+3hjFP5QC
y4yEM/U/gZ8dCemWyj+a9M0rmmL3P+i6/3qgSs8/bv4dpPSX+T/4nZ8Rw5jR
Px743/tn0vs/D8cm/8Mk0z9FdNuExRD+P6LSk+eASNQ/Jm+KPu4oAEBVMCj/
XgPVP+PfQFg9SgFAarcKH/tU1T8AsMCNtmsCQLLE4/kpPdU/cMThRsCMA0C7
Gdcy+LvUP7SAoybBrARACk3PVqrR0z8XoYxcIMsFQKpZHri8ftI/UITA9UXn
BkDTmoYs48PQP/Teni2bAAhAZ2ieWxFEzT8qEcO9ihYJQBIPLbqdNMg/YKw4
LYEoCkAdv0nSGlzCP0Qiux7tNQtAVmlDYEV7tz9SGNedPz4MQCirsqThcqE/
4nLEauxADUAQ2QUHzhWev8rY0ERqPQ5ANK7B58g4ub8yMzMzMzMPQCIKrYaV
K8a/siC39A==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQffYMCLzZr8S1tm2q9jH7n25f90Z/f7F/mtn/o7UT
ztkbSJnnV3M93S+5x+GrncJl+3f2XgKOHA/3X+5iv3dg9jX7n+8TvGY9vbnf
VePelNc7btqfOjb51JQZF/ffvPdCXqLsjr2HIkPIpuhD+/8mh5RcunjP/sDp
WyuuWbfu/3Dmnmxu6gP7G2E67eG799vP4BTdL2jw0D7u3K+XW6ZfsF80RXpG
9omH9vPeG8mp+dy0r5GcdUlT75F9tsqlrqPMD+35VpYWiSU+sv/tOOX2gqSn
cP70dz7hwQdewtWb/8/42C31Dm7esdwDu2rNP8LtO/Wi5cGq2M/2MPfEmG+u
XVn31R7m3mcqN1IjZn63h/nHKP/00ZlrftrD/KsVHLE7ePtve1h4+Bw496pg
x197WHhtuKYr9HDdf3tYeELCm8EBFt4ABFbRHg==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQnZ4GAs/2K3GtbZuqfcx+8oO+8K8THu2fZvb/aO2E
c/aT0xcKannc3S+5x+GrncJle/7Nx4O3Ol/df7mL/d6B2dfso9w0fs8yPLXf
VePelNc7btr/eC79XV5/8/6b917IS5TdsXd4mCo29/sa+7/JISWXLt6zv/OG
W/+O6wn7D2fuyeamPrBvejR93aLdV+xncIruFzR4aK81+/i2xpS79oumSM/I
PvHQ3vRefWklz2P7GslZlzT1Htlf7LL4qJ313J5vZWmRWOIj+8QVvxzZT76G
8+tW3bwXI/wBrv7cpufalQ6f4Ob9WvD57YasL3D7/kjvEL434RvcPX1Zzzbc
2fgD7l6JCO6Z3Wd+2cP8k7Lo348z9//Yw/wb09b9ru3VP3tYeOz/ycB/qZTB
ARZeH1s2KHEVMjrAwhMc3GlMDrDwBgBpf8q4
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQDQEP7JW41rZN1T5mbxFpFe/d98heqrYv2uDIafvM
M8l2ZjpP7Q+u/GQpUXTBfipHuEbsjef2tvcUr+YduWTfbZEhFTrxlX3nnOgl
D65fsf/wgdF1Tehbe0exHauDV1+z13zzLv2r4Af7yCP/H5+3vWH/oHTHBc/a
j/auL1z31ffdtH90fMIUoTef7C8+tpi6Yc4t+xDBd6K9sV/s77q/4b6cftu+
86CP59cLX+1D/V5FVr+/bS/820S2zfW7vWRySkui8R37tyynH1Xu+gHnl9tt
3sal/wuufs07wVdFi37DzZOzyM9+K/wXbp9Cl7v+tqZ/cPdcFxacwfXuP9y9
R5r3v5uyisEB5p+7H1N3ropmdID597btq60vOJkcYOHxjCe94NMWJgdYeG3K
NNTIjWF2gIUnJLxZHGDhDQBUbr5G
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQffYMCLyxV+Ja2zZV+5j9s5dSNqsk39ufU1m95q3J
aXsDf+E53rs+2E/aFN+/6+J5+7uHD7zWfPfRPig06vKR/xftb9R7bDqj/tn+
1OaS5YL3LttfFipp7U37Yr+n/eGvo01X7SMWfnA+vPKr/fzr7yYzvb1mL5bF
k3/owzf73onH7dhlbthvbtjaetfqh303b76xMf9Ne4ayzoTU9p/2Bo0C2azH
btpPEznat/LqL/s5CqVPZBxu2f94+GjzD5U/9ptm1ly3qL1lv3j7tqcLSv7C
+WytV7g+HP4HV39OsENGipPBAWZeCc8nzrj1DA4w+w5Nd5uaHcLoAHOPYs0V
rdffGR1g7k05LMTVNpPJAeafhz0rip9ZMDvA/Hs/68217VeZHWDhYaZ6+N+y
fBYHWHiFCB2zTWJjdYCF56yZIMDqAAtvAMVYxMk=
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ6b8iCq2GlSvWP9iO6pVH
PeS/9iVDSa122z8tqdobFofdv8EZltq4IeA/1jJuiJde0r9YuGdnq0fiPzCO
SGP8G7y/zzLffQos5D+wc003uuaxPxbYLhzVzeU/oGvv+VYd0D/eo1CULSzn
P8xliirN3ds/ZcGlAVpG6D82fRRratrjPxEGmavEG+k/uJrk3BLO6T/uDRBV
/KvpP94NHXe3xu8/F9OAeLT26T/vQ4KQluDyP+zei3DF++k/u3GP5SPd9T9w
iQWNLLvpPwkHBw9u1/g/5x1jFAw16T/pn4iU4M37P5giizGraeg/FQufB+m+
/j8qahLOdVnnP670o+x71ABA+Pf5WPwE5j9SifqWQEUCQKcYDHrzbOQ/dKHL
1/6wA0AkWQGyM5LiP2ZOoa71FgVAGjig57h14D9yav8rZ3YGQB0MKcRDMdw/
iCwr1pjOB0DqvophX/jWP9bAHAzUHglAUvv0AYpD0T9mZmZmZmYKQCIKrYaV
K8Y/ShrBxw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZyb8iCq2GlSvWPyg+/SgE
z6i/qs1fmmrn2j90R4/5gza7P5GSptD7MN8/7fxuZ3bk0D9tL6BlAYPhPz0s
BENFJds/EEr4gDsy4z96sU9sTsXiP5JJWGtHpeQ/JiSYhnwH6D8ktz9EYNvl
P1qdLqhiVu0/rWonguHT5j8IfaDkl1fxP8svz0lHjuc/f+N10IYH9D9ARzS0
LgroP9T5SxORufY/XKULA1ZH6D+G3qqKSGz5P4QVpMOcReg/86MzuD4e/D8J
wB3gAwXoP/NgFoQFzv4/LPDtnq2F5z9K66j/F70AQOtbr5Ddx+Y/usyqkqkQ
AkDkk0hs+MvlP3cAUE4DYQNAQKF72YOS5D8j1Pi6cq0EQOwr+ykmHOM/GYfY
dEf1BUCfyioBpmnhP9PqjInTNwdAtJ5y1dP33j/fKmrUa3QIQEf6mcDvp9o/
jcNJWWiqCUD5HQV96eXVP394rJ0k2QpAB6MRU0e00D/+//////8LQCIKrYaV
K8Y/1lrBJQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZyT8iCq2GlSvWP5d5oTTd
/NU/hCHuQsmf2j/JdenxYl7fPxrCY97Ap94/nP3h4jF25D/4arxkrCDhP0Td
quvmUOk/sNWEAlS14j8u5jUyvTzuP1AC7ZIAEeQ/Q8dNH4yb8T/HQDec+TLl
P+RCa/KpHvQ/UT9oQqUa5j/YNcXVYqb2P6hE6piIx+Y/KUgRG18x+T/qY8bj
RznnP+BMxFhFvvs/gA5RyKZv5z9qxVghu0v+P9EwMG2Iauc/VKuP3TJsAEBW
26mJ7ynnP8HTG2x1sQFAUFkzZP6t5j/CBSin+PQCQJR4QcD29uU/IT3l5hA2
BEDAqGO7OQXlPyvT8ssTdAVA5HW8mUfZ4z9axsSZWK4GQJC18oG/c+I/ucYr
kDjkB0Beg78oX9XgP9eFz0MPFQlA9rWL2AT+3T/lV2z1OkAKQFxx0r5F49k/
3dul5xxlC0CCTcKKrlzVPyIhQbMZgwxA4vaf+6Vs0D+ZmZmZmZkNQCMKrYaV
K8Y//zbFPQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ6T8iCq2GlSvWP8B3q+3F
ru0/OMkKlIYQ2j81/AmDxuzwPyoh3vlKld0/v0pobdwL8z8N4vRiAlzgP5QC
y4yEM/U/8eydBYW74T+a9M0rmmL3P8tzFuJy6OI/bv4dpPSX+T8NVCZMLOLj
Px743/tn0vs/mOjpwiyo5D9FdNuExRD+P2JuIDcLOuU/Jm+KPu4oAEA8nepC
epflP+PfQFg9SgFAxuDbUkjA5T8AsMCNtmsCQGpnSMBftOU/cMThRsCMA0Du
EcLcxnPlP7SAoybBrARAliu+7p/+5D8XoYxcIMsFQOaxZR8pVeQ/UITA9UXn
BkB60plZvHfjP/Teni2bAAhAKx8+Gs9m4j8qEcO9ihYJQNbI4THyIuE/YKw4
LYEoCkCw6dHvolnfP0Qiux7tNQtAeOS93mYK3D9SGNedPz4MQIdfQ7vxWdg/
4nLEauxADUCRrDymOErUP8rY0ERqPQ5AKj15mca6zz8yMzMzMzMPQCIKrYaV
K8Y/PJK9hw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAzczMzMzM7L+axwElsKDgP6buunig
t+i/JaewqyiT4j/9bVJg2Yzkvx5Tmt6KVeQ/6NCVi61O4L+WJKDE5+blP3rC
oJm6/te/ei9JZ2pG5z/M9Ck7yITOv1S2wUNYc+g/UExT8eizub+WltGtEW3p
P3CdmBWZ5qM/IiuVJBIz6j+UCEKNkuzGP+uwy5jwxOo/ZqyGJ6V61D/F35Wk
XyLrP04yOvUdhd0/UCOHtC1L6z+dWZzQc0jjP/Op8yFFP+s/WasgtZrM5z94
VG0+rP7qP22cJzSeTOw/IG5pUIWJ6j/8DuaFDWPwP3D0EIEO4Ok/cNVNuFib
8j8EFUW7oQLpP7aKCigDzvQ/tGHpe7Tx5z8k71JI4vn2P18LjZPXreY/jCU+
J88d+T9hN5TZtjflP1gRQwqnOPs/xTQK0RiQ4z90/XoITEn9P0zyTD/et+E/
lLJVoqVO/z+iMZNpA2DfPzI/N6vQowBApiMTEC7z2j+amZmZmZkBQCIKrYaV
K9Y/DY/Img==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAZ2ZmZmZm5r+axwElsKDgP0CIVBI6
UeK/JaewqyiT4j8uD9jz5Uzcvx5Tmt6KVeQ/BdVeSo7Q07+WJKDE5+blP1zr
p5nbY8a/ei9JZ2pG5z/QbEGGuqyjv1S2wUNYc+g/4ObfQUp/uT+WltGtEW3p
P/TA/94/k84/IiuVJBIz6j8W0W0TFkPYP+uwy5jwxOo/mbwp+rij4D/F35Wk
XyLrP41/A2H1KOU/UCOHtC1L6z8DwAI32q7pP/Op8yFFP+s/vxGHGwEz7j94
VG0+rP7qP2oBR02CWfE/IG5pUIWJ6j8uQhm5QJbzP3D0EIEO4Ok/ogiB64vO
9T8EFUW7oQLpP+i9PVs2Afg/tGHpe7Tx5z9WIoZ7FS36P18LjZPXreY/vlhx
WgJR/D9hN5TZtjflP4pEdj3aa/4/xTQK0RiQ4z9TGNedPz4AQEzyTD/et+E/
43LEauxAAUCiMZNpA2DfP8vY0ERqPQJApiMTEC7z2j8zMzMzMzMDQCIKrYaV
K9Y/wN28Pw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAQAAAAAA4L+axwElsKDgPzhTK5jv
z9a/S1MiA8ra4j8Mrsa109zKv5Yj3dDF3uQ/wIxGOEgDr7+BrWfGkavmP+i2
IyoEVLc/ORgwZDlA6D8c/z0vW1nPP9lEmPTlm+k/PFBqsGOh2T9Qg+L93r3q
P2EfcH7t1uE/2oETpIql6z9JBSRFX+bmPzGHlfptUuw/6ym8z1f86z9ypnFF
LcTsP6wZkSUSi/A/CFH8KYz67D82kiXuhxjzP1pz285t9ew/dCPshzKl9T/e
HVXr1LTsP050BKW3L/g/2JvexeM47D9Q2Bwbvrb6Px277CHcges/DkeXmu44
/T9J6w4dH5DqPyJzsmT0tP8/bLhn+yxk6T/ALCsAvxQBQBj4neOk/uc/Hy2S
9p5KAkDmxWqKRGDmPz3sNap1ewNAgx3xzeeJ5D9LvtJboaYEQDd7FEGIfOI/
Q0IMToPLBUBJaQynPDngP4iHpxmA6QZA8nv2vnCC2z////////8HQCIKrYaV
K9Y/VvjDAA==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz07+axwElsKDgP9krWxfW
+sG/cf+TWmsi4z9wEhbhIwGXPw70H8MAaOU/tmOaeHgfyD9sNi/IO3DnPyjR
5eHv29Y/+gAXYQg66T9iFgNAeefgP1/TbqVzxOo/ZlMuaHpx5j8LcPNNrA7s
P4xOIAULCew/lNiRIwMY7T8HkUjAmdXwP3hdX1zr3+0/o0unUnuq8z8ibU3m
+mXuP5dzoJqpgfY/xH5xn+qp7j9xxMnAoln5P8I8w3uWq+4/Db4UguQw/D9H
5zyY/WruPzXnwfzsBf8/lMlTO0Lo7T88N5C+nesAQMyByMKpI+0/wMLWpKhR
AkCRwdh+nB3sPzCUEzdZtANAJg/meqXW6j9YSJNC8xIFQNTkrjNyT+k/4q3r
v7xsBkBsVEE70ojnPzi2sDX+wAdARAbYyraD5T9GZM4ZAw8JQCIE3EIyQeM/
qBFUMRpWCkDBOU+Zd8LgP0g2fu6VlQtAPtTZbbMR3D/OzMzMzMwMQCIKrYaV
K9Y/Ho274g==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZub+axwElsKDgPwi3+QgO
e64/cf+TWmsi4z/pW7wVvnnMPw70H8MAaOU/qP4ZCYnc2D9sNi/IO3DnP/tO
WVde1OE/+QAXYQg66T/HfGmm303nP17TbqVzxOo/y7mUzuDX7D8LcPNNrA7s
P3haw7W4N/E/k9iRIwMY7T85xHvzzAj0P3hdX1zr3+0/1X7aha7d9j8hbU3m
+mXuP8mm083ctPk/xH5xn+qp7j+j9/zz1Yz8P8I8w3uWq+4/QPFHtRdk/z9G
5zyY/WruPzSN+heQHAFAk8lTO0Lo7T/U0ClYN4UCQMyByMKpI+0/WFxwPkLr
A0CQwdh+nB3sP8ktrdDyTQVAJQ/meqXW6j/w4SzcjKwGQNPkrjNyT+k/e0eF
WVYGCEBrVEE70ojnP9FPSs+XWglAQwbYyraD5T/e/WeznKgKQCIE3EIyQeM/
QKvtyrPvC0DBOU+Zd8LgP+DPF4gvLw1APtTZbbMR3D9nZmZmZmYOQCIKrYaV
K9Y/2ZTGXA==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQPWsmCOy0n3WcUXXDggf2WkrpnJu4T9hn//rUulzr
kf2zL46dq45ftNdzKowsmfzY/oFQ5KczK67bL7U4rRd+64n9qUd1CxSX3bXf
dvz75EX/ntpb75Gut7330L7k+tzC3f+f2X/OvSXu/+WJ/ey7cbGSd5/bBzKf
TFkk/MJ+whKLR27TXtiryN1w67d6bS/76Pq+h1ov7QMjl07LTH9nH9azS3L2
rJf2tXlXmB3PfLCXcGZbbf70pb2eSHubcNon+6DZf273c72ytwi6fz+I4Quc
//L7xCTVWV/h6vfuFJ690eg73LyQ8F+H5E7+gNu35wmjydSYX3D35OhX3rJ7
8xvuXo+OG8vcy//C/bNprdu/u3/+wf17XeDFFEk7BgdYeEz7wvXI9heDAyy8
BA7OEarYxOgAC880MGBygIU3AII0yt4=
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQbWIMApftZx1nVN2w4IH9vGOchZbRt+yX1Mh2Sk57
ZJ8QqKpSf/2B/aOqq0d8Ep/YX7xRaeE/9Yk9W1uZYRjjM/tDLybuE0h9YX9F
JYg/svS5/atXpr/2uryxt1l7/1f/nhf2l2v94lQkP9hvcqgI9b320j6H+/ze
s6Kf7JeolS38ffCVPZd3ipmv+Bf7tsVtF2XrX9vzP30WOEnmm32MGUu3P+8b
+wO3L87sUvlhv4HdLNo1543986OPNZT0ftnneEbcPzP7jX39giZbF4s/cP6y
rilnbjv+g6tfc8Ce65UKgwPMvHW8jDN61BkdYPbVrPUO8dFgcoC5x+sLN8tC
NWYHmHsrdizdwqzE4gDzD9POPub3kqwOMP8y7BVJvc/L5gALjzv3NLOC/7I5
wMIr4Zcg65nn7A6w8DwDBhwOsPAGAFVktRY=
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQDQEP7GcdZ1TdsOCB/eeDh//dE3tif0qj/8Guu4/s
S9fON9jr9sI+yltie/urJ/Ydt1RuJfa+sf/AZ2fMcOyZvY2ugqhLyQf7KbyW
PBr5L+wPHDAx/7Dqk/0B622rZZ++tC8V9l7W8fKLfZ5u51Ip9df2DMrqjXf0
vtvH1X5RKDF5Y599Uypke+VP+3OVAcazed7a75LesOfhid/2XH/va8uuf2u/
wvQhn5L0P/ss09UXFku/sxf4c7h2njmDwxUhh56vge/su+Zelf6dyAjnH/un
MYmjh8kBpn5vt+Nuja3MDjDznur0vN97h8UBZp8UB1ukNhObA8w99/lL3K6q
sDvA3Gtl+b9W0YXDAeafP02c/eXxnA4w//5IElB2LONygIWH7bpC31ft3A6w
8JJd+eLmw8k8DrDwnDUTBHgdYOENAL5Psq8=
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQnZ4GAs/sZx1nVN2w4IG99Rvb0uvWr+y3B10zebXz
kT2L5J/D2zXe2cs9/mb7aukT+8WVJYKVoh/tq72ijFYnP7OfE8vIbqzw2d5k
5tFeh0/P7V8s/5JWYvzVvu6L1+VrAS/tC6dK+iv7fbcX2N7wd3PdK3umJ+5v
FPN/2jMGmT40r35tzxHPI+Q/9bf9qr7spjuub+yNfCWutO3/a79Zsija7N4b
+6ZQNcXFb/7b9877MPWL81v7F6GN+vt4GR0U9M6YPix7a+9zdyNzqSoTnF/M
sWDRQxtmB5j62TzqJ/ODWRxg5m1g1l3VncHqALfPkd92UTWbA8w91RnLuhV7
2B1g7nW4v+N/xUwOB5h/0lo2rZdczOkA82/St4pVZSu4HGDh8VnHxIl1JbcD
LLxUdzwOtV3C4wALz1kzQYDXARbeALPnuQo=
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAzczMzMzM7D+axwElsKDgP/FFpcY4
nu8/EHzRujl14j9Z8S9RHELxP2o67zFxKOQ/1wDaVaG+8j8AI2mdb7nlP/R8
DorhQ/Q/z6XnOGAn5z/eQ45kDtH1P+FcktmAceg/m+WzJ1Vl9z9ydBZVIpfp
PyRGRVHf//g/zPaX3qiX6j/SDUUM05/6PwCrXFmMcus/sIOHpFNE/D+cqQSh
WCfsP/p9zfuB7P0/NFUqx6217D/mRScAfZf/PzIESUZAHe0/BE+wETGiAECu
Wc8p2V3tP9NGm+kmeQFAZApIK1Z37T9RlAv5LVACQPKUicSpae0/DKEVKNQm
A0DmRuM22zTtP1HoNJKn/ANAOcBDhwbZ7D84Y7rCNtEEQKr/Wm9cVuw/zJ3/
8BCkBUDT279DIq3rP12EPjzGdAZAhKAmz7Hd6j9KKO7m50IHQElVvCJ56Ok/
vfyEkQgOCEC29L9b+s3oPy9egXS81QhAYZF4XsuO5z+amZmZmZkJQCIKrYaV
K+Y/Zhq/gQ==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQPWsmCLzcr8S1tm2q9jP7M0YJ3Dt3PN5/3dXjS0jy
S/tWhQz3y+m397elr83YHfrG/k6t0IFiy3P7121Qnm3B+N7+QcXFs7Fbl+yP
sW/7aLDpg72JU4VIivsR+x4LjZqf+z7ar6j50zGP+7q947Xt/unLPtkzvd4W
qcD7yH7r95wF/9M/25dNN64IXfLCXk10/tZo5i/2UtsepLx3f29fZe2moFnx
xT7kwzQNk1+f7P9573imeeyL/d3lx+5IBX+zV/LuXmry9ou9YoSl8tM1P+H8
1fyZ10+z/oWrr7h0XI1Fj8EBZp6S1OeFr64xOsDss3Od9HJ7DbMDzD1ZBQ/0
euRYHWDuvVO9Y1/nbjYHmH8euZqXXw3icID59546o830x5wOsPAQYHmb8SuH
2wEWXpt4bjjpveNxgIVnGhjwOcDCGwCL0b4p
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ2b8iCq2GlSvmP9W2N+YJ
8dK/vmruc8BH6D/jBJdEJTLIv2pN3YWRPOo/2OQHlVVVtL9y7tsA/wjsP9C5
tvyns6A/FtF5lxSs7T9MeUD1/MzCP+4tFOzzJO8/8OzBvuXW0D/aUWmDajnw
P9p2dVK4Y9g/BsjgX4PK8D9kOPdTegTgP+fhaI53RfE/s3ShHkbh4z9p2cXR
BarxP8OvueEyxuc/u2Ilz/j38T858JalL7HrPxEIbionL/I/OXESOyig7z/O
MjGcc0/yP4oA9CqDyPE/Uz00Ac1Y8j/EvhpU2cDzPxRPiGMuS/I/goBB/wq4
9T/bLyz9nibyP9yF3i0Nrfc/vaw2NDLr8T+c/UQK1p75P3aXi5AHmfE/Gj2y
dF2M+z9w1iCrSjDxP6ycco+ddP0/5mXcFjOx8D+EnNNJk1b/P16QF0MEHPA/
HszMdJ+YAEC2B66vGuLuP+0MWEnSgQFAEICVF1Jh7T9nZmZmZmYCQKpMWOh6
tus/9ifHxQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ2T8iCq2GlSvmP6QTcq2V
ud4/hejDcE7c5z/b51XPzP/hP7YUeRq5buk/BrQRjmy05D8SoTD+/+HqP0D0
eJE6eec/93MfHF417D+k7zXCvkzqPyZY0mIfaO0/xmieOnkt7T+dQLkOoXnu
P/TTMonxDPA/9w2EAFJp7z86YAOXN4jxPzKBEYVZG/A/9IG8iMUH8z9mBCGZ
q3DwP59q39rPivQ/b4AN93G08D9kxm4wiRD2PwMxwKiI5vA/VurlvyKY9z/A
W4Ma1QbxP8xt8cDMIPk/B1scKUYV8T8q967atqn6PxF64yrUEfE/erw5khAy
/D8mz9TzgPzwP54PSbkJuf0/suuX1FfV8D90RKbc0j3/P+rlfpRtnPA/N6of
2c5fAECM639m4FHwPy3sTcPOHgFArG5esq/r7z8aIr3Ug9sBQFvTg4MJEe8/
2uGt7ImVAkACz0lEQhTuP53T4lZ+TANA1v1qFOD17D8AAAAAAAAEQKpMWOh6
tus/MDrE/Q==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ6T8iCq2GlSvmP74ScpMF
a+w/mL58HB8A6D9/ryxvBVHvP/J8mpNWs+k/PmdAvAcl8T+IZRT/VETrP1rj
dPBHqvI/V+iSmkWy7D9EqvTKdDf0P2ifPTtm/O0/AUwajrvL9T/6tsG2ByLv
P4qsq7dFZvc/qpwhIEcR8D84dKtyOQb5P8T2g924fvA/FurtCrqq+j8S9lcB
H9nwP2DkM2LoUvw/3stqlEkg8T9LrI1m4/39P10j+tMSVPE/bQTHiciq/z8a
Tr1FX3TxPwV6zhxarABAdqZ5xh2B8T+Exz4sYYMBQLxrGpNHevE/PtRIWwda
AkC3REdM4F/xP4QbaMXaLwNAYIF39PUx8T9rlu31aQQEQBghg+ig8PA//tAy
JETXBEAtj7XSA5zwP5C3cW/5pwVAhvFomEs08D98WyEaG3YGQNCXZ4Rec+8/
8C+4xDtBB0A+N2u931juP2KRtKfvCAhA6dMjwLAZ7T/MzMzMzMwIQKpMWOh6
tus/gv+9qw==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQbWIMApf3r/KJeFG17bX9otqvr6v+H97PEBX+W836
nf2BubU7DRqb9hdeiNh2zOWD/a8C+/cXRQ/Zl/Hp1VgFfLSPmHFX+//1S/Yb
ba2fJdh/stcysDrKeu6O/arlH/l3C3621wnqv3Oz+LH9HP+KmJvHPturlP7r
uNb/wn7Smicbo+O+2E85v+ix04639pOyLNZ8vP7FPjmoUt2w9KN9igrniSjj
r/b9/R+adwp+sZcq2d2SVvDVPkRL51z4hm/2Sw3s6770fbVfdfWy4Qr/n3B+
fgF/jdu733D1s9e23TjX9Q9unpSPuPv7iQwOMPu2JR2I5brD6ABzT4n2nQ5O
ZWYHmHt/7Yxwt09jcYD554tPixTDUlYHmH8/cPA8XHSPzQEWHvNAwSXE4QAL
r1vh+9d9t+d0gIVnGhhwOcDCGwDll8Ax
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQPWsmCOy0X+UT8aJq22t7saM97Qx65+zfNs/zDxN/
Z/9v287cHX+u2wfLnPhZrvTBPiX6l1Z25wN7a509v/nlP9oHTRZ/4Kz11L6p
mVH9LcMn+9M+zSsK772052TZs3j68U/23XsmTSld/M4+V6SEY1nRZ/uTepcf
Je/+aP9UjHXSXsYv9rMVviW90fpiz/D1aPCkki/2/NtFijjnf7PfYXQp/vmp
L/ZzHmWczBX7ab9GI+55H8tXeyvBWymn+n/b+9ixBmcpfrVP6Nhvcp7zH5xf
Hy3xYqojgwNMvWvhIWGPWkYHmHkNewJnh21ncoDZl2GafFvqPbMDzD2rZFek
v1dmdYC5d9bP1UkGoWwOMP/8eH7sinUTuwPMv19M+ZiKVnE4wMLj7VTR/aZn
OR1g4VXNEHLkzUsuB1h4MoABjwMsvAFiPbX1
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz0z+qTFjoerbrP3wl5CYL
1tg/IAEofgSL7T/+XlneCqLeP3q/RfU7Pu8/e86AeA9K4j8I1F8wnWfwP7TG
6eCPVOU/bxUffpUe8T+IVOmV6W7oP/hwdM6lw/E/AZg0HHeX6z/BfDaMdlby
PxJZV2+LzO4/7j330LnW8j83dKtyOQbxPwiYWY4rRPM/FurtCrqq8j9Wly2y
kZ7zP1/kM2LoUvQ/Im1ARbzl8z9LrI1m4/31P6HEz4SFGfQ/bATHiciq9z9f
75L20Tn0Pwr0nDm0WPk/ukdPd5BG9D8Hj31Ywgb7PwEN8EO6P/Q/e6iRtg60
/D/85Rz9UiX0PwU30Iq1X/4/pSJNpWj38z9qlu31aQQAQF3CWJkTtvM//tAy
JETXAEByMIuDdmHzP4+3cW/5pwFAypI+Sb758j98WyEaG3YCQC1tCfMhf/I/
8C+4xDtBA0DkPIuP4vHxP2KRtKfvCARAOYvnEEtS8T/MzMzMzMwEQJrHASWw
oPA/cFfBaA==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQbWIMAo/3zzrOqLphwQf7pkf3MlZY3NtvVHGiyXT/
R/s6bnXWNzeu7q/i5H03++gn+w1Jlimu00/td30trxOx+bP9g6PT3CNCN+2f
njVX4k37F/uEok03M0U32q9ljHnxzOmr/Y1z3akV9afsW6pcU58+/mo/TdVs
HnfoNfsapd9Kk7K+2bv8Z3lV1nDffouC5pql177ZO7Douk2Ie2If9KhgxTP1
7/YyFv2vBJhf2jfVXfulGffdnmvvncNuq9/am79dcCOo8rt99ULBeTFaH+H8
T8k7usrrPsPVP6h9eWf7ha9w8wwO1Lk+kv8Bt29WWZHE3qxfcPeotPl9Ftjw
B+7eWzdSkla++wf3j/42m0OrJjE4wPyrPc05rGQlowMsPAoeLmmQ3c3kAAuv
gDvSvKLHmR1g4Xn2DAiwOMDCGwCzJcc7
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQDQOzjjOqbljwwf4sO//v6ugD9kYVJ5pM93+0fxVl
n1jVe8G+ipP33eyjn+wFtnpuOSx10971tbxOxObP9hVRDlodLx7YT8+aK/Gm
/Yt9Q2Nm3oPQp/ZrGWNePHP6av9KbZqP4OWX9i1VrqlPH3+1/+/qx7Qz9p19
jdJvpUlZ3+yzb0qFbK/8aL9FQXPN0mvf7HdJb9jz8MRn+6BHBSueqX+3X2H6
kE9J+pt9U921X5px3+3lf7T/tsn/YW/+dsGNoMrv9qLWq82/Hf4F5/f8DVQR
lPgLV18l3lwWmPUfbt5TnZ73e+8wOMDsk+Jgi9RmYnKAuec+f4nbVRVmB5h7
rSz/1yq6sDjA/POnibO/PJ7VAebfH0kCyo5lbA6w8LBdV+j7qp3dARZesitf
3Hw4mcMBFp6zZoIApwMsvAEqf7te
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQPWsmCJy0n3WcUXXDgg/2pTeqX60rv2KvwftGx3ft
R3sOox/t+YF37GNCawx9F3+yd/v8QumT1iN7Ls51q3WbPtub72biS3N4Zp++
ITmywvOL/Tn/l2dfpL2y54lTqzn844v9Iy3jeW5z3tmH2jsKbur/as/0xP2N
Yv5H+3M90sxfeL/Zs8fzCPlP/WyvtHpbsFvxN3tDX4krbfu/2i/7YOlQvP+b
fWOomuLiN9/tRYwrY4M+fbM/v5opvkb6l/3dX2lrT3B/t5++K5n9ldcfOP+p
gIPr4cp/cPWzedRP5gczOMDM28Csu6o7g9EBZp+RI7/tomomB5h7qjOWdSv2
MDvA3Otwf8f/ipksDjD/pLVsWi+5mNUB5t+kbxWrylawOcDC47OOiRPrSnYH
WHip7ngcaruEwwEWnpDw5nSAhTcA1yq0iA==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQbWIMAo/tZx1nVN2w4IP9KQejpBOtz+zFzI0bazo/
2it/2PJqzZuX9gpvo3asj/tkf0ilcItQ2lv7qS6nVt2Q/2wfve/wK4GKD/bs
Nh9uTTjz2d6srORYjcsn+1NiuxfXpHyxz5y5bUGE+Bf7HycsMr2ff7HnSus8
tOb9V/vCxJjD20O+2jtkve+tO/Pd/v/+i4va13y1V9zwgunimp/2Abxn8nve
fbU3nrrBdWn/b3uzOfuUFkt9szcW/qH6u/ivvZbY57AtBt/sZ++8suxixH84
/5x03+HJ8xgcYOrD+D49+dvP6AAzz31jnrx6A5MDzL6kzYqsE/KYHWDu2STJ
LbYrisUB5l4fbR1mDRdWB5h/LNtZPsZqsTnA/Gsu67XzIQ+7Ayw84rV2alu/
ZneAhZfptZu+X45wOMDCc9ZMEOB0gIU3AG7Nvv8=
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAQAAAAAA4L/eaNfVImbzP7pietg3
ytW/yoSgcACn9D9O13YddbnGvxh/5iTLyfU/gF7zDxKih79HIG6n6M32P7QI
MipGHsQ/jgXi886y9z/yXzmzJQLVP8DuDZYEePg//ezHARQL4D8WPVDqIB35
PyPouZ6kouU/WnEfVcyh+T+luyoazUTrP80zhnHABfo/bhh0H0h38D+iO302
yEj6P2JAbWd2TvM/c0QPE8Bq+j88kZaNbyb2P3IjOAGWa/o/2orhTrH9+D+0
+HSPSUv6PwK0jsm50vs/22kA4esJ+j9CO+1JCKT+P/fFuqSfp/k/JSk9Cw+4
AEDa5cICmST5P5b6eZ2/GgJApIzJgB2B+D+9rvmoWXkDQHv3Ld2Dvfc/SBRS
JiPTBEBHL/fgM9r2P54cF5xkJwZAM4jCKKbX9T+ryjSAaXUHQCKHxORjtvQ/
DXi6l4C8CEDyIf6PBnfzP62c5FT8+wlAIfrMnjca8j80MzMzMzMLQJrHASWw
oPA/HGy7zQ==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQPWsmCOzcfy/j+lWltM/2DDcEiypL59iXSc4zXtHy
xZ795cav6n5H7PetejSPr/ur/Sq/x76xWy7Zd/Z8vXOq4pt9l5/Uu5VZt+1v
cxefjPP9bu/rZ2a9VumR/cdFhjMjOX7Yb/+cJHlv/jP7E9LSYhErf9ifEZI+
36r12v7A4QuXWQx+2i+uZyq1PvDe/sfBfzESc37aKzSJKXvof7Kf1uwWGfn2
pz3fHPfSjsYv9sfk16yeo/zLvjXuXeLiq9/sq/vKOo/a/bKvXig4L0brJ5z/
KXlHV3ndb7j6B7Uv72y/8BdunkTCfqWP/AwOMPt8rS174kwZHWDuEXJeXskR
weQAc29ujtHGkHJmB5h/9LfZHFo1icUB5l/tac5hJStZHWDhUfBwSYPsbjYH
WHgF3JHmFT3O7gALz7NnQIDDARbeAM8XwZE=
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQPWsmCOy0v5dx/apS2mf7mVduH/fQOGtfJjnPeEXL
F/uAg0uPP/h8zX7fqkfz+Lq/2pdJb5DSarxv39nz9c6pim/2q3uLY3dLP7G/
zV18Ms73u/2mLXMWCne+sP+4yHBmJMcPe5mok/UubG/tT0hLi0Ws/GE/c88B
qbITH+wPHL5wmcXgpz1L8Za3AcKf7X8c/BcjMeenffBWz7DqpK/205rdIiPf
/rR3PF+1YveW7/bH5NesnqP8y37HRMWp1zh+2Vf3lXUetftlv+6Ky8X+2D9w
vur013tXbfwHV98ZwdfxtZHBAWbexp8Re7pWMjrA7Ht25ZLqj3NMDjD3rLnj
ILzoI7MDzL1sbGe83gmwOsD8c8L/WrSLDpsDzL9H9O++53Vmd4CFB2e1ndT2
EA4HWHi9LN26bF0CpwMsPNPAgMsBFt4ArVXHGw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAAAAAiCq2GlSv2P5RZqc8g
YMI/+0+9daJI9z/8qBwllpHSPyC4mlygSvg/ai73+JYf3D8V/V9XBjH5P952
RIWA6uI/cjJEJlr7+T/If8/LVtbnP8JIeG4wqfo/Hyg12LHQ7D/9Zx3zLDr7
P7EPOL926/A/Quc1xgKu+z+lApKiL3PzP+7pdnF0BPw/9hTe5yv+9T+O+eQW
VD38P60ZkSUSi/g/2k4qiYNY/D83kiXuhxj7PwLgmVv0Vfw/dSPshzKl/T9E
tdbppzX8Pyc6gtLbFwBAQnQbV6/3+z8obI4NX1sBQOSDIoUrnPs/h6NLTXec
AkD6m7MCTSP7P5E5WTJ62gNAjALg8VON+j/ALCsAvxQFQGIi++WP2vk/Hy2S
9p5KBkBIiWG5Xwv5Pz3sNap1ewdAF7UkWzEg+D9LvtJboaYIQPFjtpSBGfc/
Q0IMToPLCUD6WrLH2/f1P4iHpxmA6QpAUsXpo9m79D////////8LQN5o19Ui
ZvM/Xbq8sg==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQPWsmCJy0V+Ja2zZV+5v9j+entL4XXrafW3nvUaLI
d/tVbhesD9jfsl+hzyY19eV3+y/hS567znxgL9teyrNv1Q/7F+t3cYQqP7Hv
r5ttUxb+036evMwBnT3P7YMi01iT3v+0Z0lbUPIk8bV9A/eTUwZFv+xd/rvu
6BF6b/9zMbvq3Pu/7G8yd6pLpH20b2OqZck1/22/07NpTpbbZ/vr/uUXCyt/
25/W87pvp/nVft+9zJaOpUD+mon7p/B+t98YMW/HzN2/7Y2D8xyyP/1A8C9u
VTh47Rdcveu22uSpO//AzVP7U3bj+cx/cPv0Gt5djLFmcIC5p/7Z9Udt6owO
MPdK/Phx/osAkwPMP6whF/dqfWdygPmX+ZVY27qbzA6w8ND+3vqDfQeLAyy8
GBcvkzo4kdUBFp5pYMDmAAtvAG3Kwnw=
      "]], 
     {GrayLevel[0]}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
     StripOnInput -> False], TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 268},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->90489880]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Neat Examples",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->589267740],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1172163540],

Cell["Have students infer a function from the diagram.", "ExampleText",
 CellID->1685122849],

Cell[BoxData[
 RowBox[{
  RowBox[{"ptSet", "=", 
   RowBox[{"N", "@", 
    RowBox[{"Transpose", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"x", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"x", ",", 
           RowBox[{
            SuperscriptBox["x", "2"], "-", "3"}]}], "}"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", 
         RowBox[{"-", "3"}], ",", "3"}], "}"}]}], "]"}], "]"}]}]}], 
  ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->659214597],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionArrowDiagram", "[", 
  RowBox[{
   RowBox[{"ptSet", "[", 
    RowBox[{"[", "1", "]"}], "]"}], ",", 
   RowBox[{"ptSet", "[", 
    RowBox[{"[", "2", "]"}], "]"}], ",", "\"\<OneToOne\>\"", ",", 
   RowBox[{"Axes", "\[Rule]", " ", "True"}], ",", 
   RowBox[{"Epilog", "\[Rule]", " ", 
    RowBox[{"Line", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"3", ",", 
         RowBox[{"-", "1.75"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "1.75"}], "}"}]}], "}"}], "]"}]}], ",", 
   RowBox[{"PlotLabel", "\[Rule]", " ", 
    RowBox[{"Style", "[", 
     RowBox[{
     "\"\<Describe a function that might be represented by the diagram\>\"", 
      ",", "14"}], "]"}]}]}], "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->845386507],

Cell[BoxData[
 GraphicsBox[
  {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
   0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
   {RGBColor[0.0692454, 0.645548, 0.968322], 
    DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
   {GrayLevel[0], PointSize[Medium], Opacity[1], 
    PointBox[{{-1., 0.}, {-0.6666666666666666, 0.}, {-0.3333333333333333, 
     0.}, {0., 0.}, {0.3333333333333333, 0.}, {0.6666666666666666, 0.}, {1., 
     0.}, {2.552786404500042, 1.341640786499874}, {2.701857603000028, 
     0.223606797749979}, {2.850928801500014, -0.447213595499958}, {
     3., -0.670820393249937}, {3.149071198499986, -0.447213595499958}, {
     3.298142396999972, 0.223606797749979}, {3.447213595499958, 
     1.341640786499874}}], 
    {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAA8L8AAAAAAAAAAAbOUpH4
Zey/DTiPjobMwD++L7iZ9pvov+ks45xddtA/Pj6ZyPyj5L94aMfKByzYP725
YTEmgOC/rjhslyqD3z9iGO9aSmXYv+RlunjwO+M/uP5jxwj3zr9eVF0APYPm
PwDZ/8fUHrm/TonRjb2V6T+g+BLlCYqpP49c2s/Qcew/ENa7crDUyT+mejlW
8hXvP6yhvw5n3tY/8uXkr13A8D+Eab7eEoTgPz8kbcxx2PE/1MDo5Bex5T9k
3LHkoNLyPxIN/ouD8+o/L3leOWau8z8eeZS1RSTwP0/yPC5Na/Q/HBTYnK3W
8j+lWhWI8Qj1P2I2oieLj/U/XqLcof+G9T8E7F+UbE34P7RUFpk05fU/YidE
eN0O+z/jxFFxXiP2P4Zv4oRn0v0/utWvLlxB9j9bFWWnSUsAQCJJY+cdP/Y/
0ZFcivSsAUBCTiLMpBz2P9nLfcN4DQNAldOEJwPa9T9FAkhHG2wEQGTyT1Rc
d/U/BrPDVg==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV5b8AAAAAAAAAAHode15f
L+G/caaBmPsitT95eolydNfZvy3XqBPXXsQ/IXwNP1IZ0b9eiO1g2GLNPyBF
d1/3UcC/WjH6iVzM0j/AffBXIZqOP/3vzfUOftY/4IwHQXt2xD+WHb6qjcTZ
P55oiTNepdM/qf4ywRue3D/OJx30My7dP8DkQR82Cd8/Vb2041do4z8YOQkj
SoLgPzWCUnvaQ+g//58VgJRH4T9vzrzyDSftP2C1JHgR1OE/lN6MM60H8T9s
nTuAdifiP1A+kaASffM/vnuTWJdB4j8+TGpLafL1P2z1EyRmIuI/on/rRmNm
+D9/Vq5v88nhPz5c616z1/o/5nOVKW444T+uog/JDUX9P0zyVogjbuA/1obZ
1Cit/z9IUcbD/dbeP6xBSs1eBwFAZLYy5RJi3D9JfGVURDQCQDEO7SjUftk/
wUbyV6VcA0DwwhrNyS7WP+ShyZXkfwRAVAftybVz0j8trNqEZ50FQNntv8Ul
n8w/pDG86g==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV1b8AAAAAAAAAAN14NcRv
I8m/RINkILR+qj8QhSQGt06tv2EnfVm44rg/bM4s7E92tT+omR8B93HBP+LE
b4N8/8w/2TdfCNaexT/BCGlSz7fXP7JTHR7C9cg/WJOodnGA4D+6hGOj9XTL
PzJ1D+YFK+U/BfFVbx0bzT8QMJwfK9npPy5+KINZ580/nH8uZ2WI7j+62viA
PdnNP4tr0jYcm/E/Uk9O5dDwzD/JMT5RFPDzP73WIAOPLss/3PSFQt5B9j9b
lmnCZpPIP9lHU/o+j/g/WnFfIbogxT+bHHq//db6Pxr4onhd2MA/DQfn1eQX
/T9ounoFLXm3P9WK9yLCUP8/0DHZnWxApz/7n/LnM8AAQEDqcov+PX2/NdD2
dNbSAUD0Yogz8tuwvxnC8P+23wJATdyCZAO4wL8iWWTeRuYDQGCOueCow8m/
Tryjv/rlBEDdZC1wDsbRv5/7uvZK3gVABquMQxcG178YVm3Cs84GQNrtv8Ul
n9y/o6nD2A==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQjQ449t+87WZ20N6vOYEj8M5ie6a4e42vAy7ab5Dj
55rqscl+8yRx2Z0yt+xbOb3eXOPaaX+w9/SODzUP7WsfGkpEGO6z59jNZnfj
41P7NZ9yq9bdOWC/SiD64NasV/aSetqvmy4ctK9xf3Ba8+k7e2cGsX/LPx+0
/8dYy561/qO94YRwXm63g/Yacn3Bq159tteZfThPYMJ++03p2uvVlL/Z/7GS
jbnyZZd9yZR/ZRvDf9gbeKtNu7xni73V4nh5+fZf9goijCu2XltpL6Hsetlt
wx/7BRytTQ0qPfY6rw4LMl/6Z3/B6McKu1Nz9+uEPlWZWcrg0OM06ZnRl837
+RMf74yuZXSYUrYz9/afA/vfJhaE7K1ncvhRXVxxde+J/XxWB26b1DA79Gpt
0imdc2E/x86UKJsiFod9YnO5z5pc3f98daJrYCKrw7LoAL0m1Vv7iz8VHPzi
weaQZuff/jX7/n72m4HPVmuwO8Q/0XFd2/RoPzSUHVI++YfElD/dDwCohK/i

      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV1T8AAAAAAAAAAOSYIDPT
J90/BE336M1epj+8qK9aaYziP0iUBzlE77Q/lNCLkw6S5j9pk/t/azq9P1QE
8e+2ouo/bI0hHzsGwj/uZdZDOrzuP+e8jQDjsMQ/ma0s2DVu8T9MN9Msw5vG
P5mVm2WNgPM/ESumLtfFxz/YBxRkCpT1P5f/EN2ALsg/u/m+0ZKn9z/9vF+v
iNXHP5g1tKYMuvk/i2qX2h27xj85j59qXsr7PyLCaTjW38Q/tukZym/X/T/A
gbL3rUTCPxXQZisq4P8/KBlRLQ7WvT982iOhvPEAQLy0E05RqbU/wzRL0SXw
AUB4epdy9w6oPw+BrKjJ6gJAgKChsTm1bj8rFwEqI+EDQPiSr6uJDKe/9wDJ
nq/SBECIkKdusGW5v3gmptzuvgVAu4bATZ5QxL+WdF2JY6UGQFJe2yFqmMy/
MOlZXZOFB0CiyUp468LSvxs6jmQHXwhAzlF6ChSK17/oqZI9TDEJQNntv8Ul
n9y/G/2/Aw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV5T8AAAAAAAAAANk3wLYr
iug/NXAUYRUDsT/gJN9c79brPxhEM/Nia8A/fq+FDOA57z9/6KneVbnHP3oq
JuOYWPE/4Q15QYNnzj+MwlHdhh3zPzRZPtgvOdI/ernRocnq9D8s0C00W+vU
PywQCnJsv/Y/ujiDgNVI1z+YKZOldpr4Py5mKnldUNk/itIpLux6+j95VHl0
3wDbPzzVbB3OX/w/PFt09XVZ3D8eIiEsG0j+PytWkiVqWd0/J2RaIWgZAEBE
JsA1NADeP2gnXIF0DwFAuRJwpntN3j+AvoIosAUCQCLNjnYXQd4/TiIlcJj7
AkBsLkU5DtvdP2wl192q8ANA3iF7EpYb3T+4v6RoZeQEQC6XHZoUA9w/8SQR
vkbWBUB+ujamHpLaP2wKtYbOxQZA+gv1+3fJ2D80s1eqfbIHQEI+y+cSqtY/
iqBek9abCEBm+d+8DzXUP98ecHFdgQlAxLkRPLxr0T/SUyV7mGIKQNrtv8Ul
n8w/NjOsRA==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAA8D8AAAAAAAAAAPwYQwcs
GPE/vZ76yTNpuz+20b7jA0TyPy79FQmN/8o/DLO8fOiC8z+OsBTp5ezTP82/
7p0w1PQ/DLmPOUIe2j9ADDdRKTf2P8q0jiJJCOA/9KucPRar9z9MGnEnV+Di
P3WTOwsyL/k/2rRN3ciV5T84Z/rLrsL6P5+9SFMuJ+g/kKjOaLZk/D9cpAa5
KpPqP5pGVRNrFP4/UKDAF3XY7D8YTYO759D/Pwz50ADZ9e4/gB+YRKDMAECr
NK2YG3XwP6PPnKxBtgFAZONZFUNa8T8eof4e3KQCQCqHo1/pKfI/yh1zAfGX
A0Ati8xJoOPyPyROY1n/jgRAJLiaSQWH8z+SW1sQhIkFQKs3oKzBE/Q/+qWY
OfqGBkBW7DvGion0P+RkkVjbhgdA0rU4FyLo9D+7Z1Con4gIQCeb9m5VL/U/
fRh/Y76LCUCuQgwG/171P0+H9wyujwpAf5ZSkgV39T+7/be45JMLQGTyT1Rc
d/U/x9i2MA==
      "]], 
     {GrayLevel[0]}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->True,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->LineBox[{{3, -1.75}, {3, 1.75}}],
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
     StripOnInput -> False], TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->FormBox[
    StyleBox[
    "\"Describe a function that might be represented by the diagram\"", 14, 
     StripOnInput -> False], TraditionalForm],
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 275},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->444991084]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 WholeCellGroupOpener->True,
 CellID->250839057],

Cell[TextData[ButtonBox["Building an Understanding of Functions",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/tutorial/BuildingAnUnderstandingOfFunctions"]], \
"Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 WholeCellGroupOpener->True,
 CellID->38303248],

Cell[TextData[ButtonBox["Classroom Utilities",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/guide/ClassroomUtilities"]], "MoreAbout",
 CellID->539642637],

Cell[TextData[ButtonBox["Understanding Functions",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/guide/UnderstandingFunctions"]], "MoreAbout",
 CellID->101302551]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"FunctionArrowDiagram - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "SymbolColor", "LinkTrails" -> 
  GridBox[{{
      RowBox[{
        ButtonBox[
        "Classroom Utilities", ButtonData -> 
         "paclet:ClassroomUtilities/guide/ClassroomUtilities", 
         BaseStyle -> {"Link", "DockedLinkTrail"}], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 26, 55.3708624}", 
    "history" -> {"XX", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Symbol", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "FunctionArrowDiagram[dompts, rngpts, type, opts] creates an arrow \
diagram representing various types of functions such as one to one or \
constant.", "synonyms" -> {}, "title" -> "FunctionArrowDiagram", 
    "titlemodifier" -> "", "windowtitle" -> "FunctionArrowDiagram", "type" -> 
    "Symbol", "uri" -> "ClassroomUtilities/ref/FunctionArrowDiagram"}, 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[9879, 288, 490, 14, 45, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->486231370]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 191612, 4558}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[613, 21, 2988, 70, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3626, 95, 63, 1, 51, "ObjectName",
 CellID->1224892054],
Cell[3692, 98, 587, 16, 83, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4316, 119, 609, 18, 39, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->295067813],
Cell[4928, 139, 81, 1, 70, "ExampleText",
 CellID->1805848246],
Cell[5012, 142, 161, 4, 70, "ExampleText",
 CellID->281283540],
Cell[5176, 148, 118, 3, 70, "ExampleText",
 CellID->1868009087],
Cell[5297, 153, 128, 3, 70, "ExampleText",
 CellID->1619558343],
Cell[5428, 158, 135, 4, 70, "ExampleText",
 CellID->2034052995],
Cell[5566, 164, 161, 4, 70, "ExampleText",
 CellID->536510372],
Cell[5730, 170, 135, 4, 70, "ExampleText",
 CellID->1993472098],
Cell[5868, 176, 138, 4, 70, "ExampleText",
 CellID->1393174239],
Cell[6009, 182, 127, 3, 70, "Notes"],
Cell[6139, 187, 3703, 96, 70, "3ColumnTableMod"]
}, Closed]],
Cell[CellGroupData[{
Cell[9879, 288, 490, 14, 45, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->486231370],
Cell[CellGroupData[{
Cell[10394, 306, 195, 6, 31, "ExampleSection",
 CellID->308284301],
Cell[10592, 314, 127, 3, 28, "Input",
 CellID->950986302],
Cell[10722, 319, 72, 1, 32, "ExampleText",
 CellID->1423066363],
Cell[CellGroupData[{
Cell[10819, 324, 166, 4, 28, "Input",
 CellID->1653476740],
Cell[10988, 330, 6253, 139, 282, "Output",
 CellID->2070927553]
}, Open  ]],
Cell[17256, 472, 119, 3, 70, "ExampleText",
 CellID->270095829],
Cell[CellGroupData[{
Cell[17400, 479, 169, 4, 70, "Input",
 CellID->1867651892],
Cell[17572, 485, 5520, 127, 280, "Output",
 CellID->1811916342]
}, Open  ]],
Cell[23107, 615, 118, 3, 70, "ExampleText",
 CellID->1922168802],
Cell[CellGroupData[{
Cell[23250, 622, 165, 4, 70, "Input",
 CellID->1294553393],
Cell[23418, 628, 5526, 127, 280, "Output",
 CellID->1559845534]
}, Open  ]],
Cell[28959, 758, 90, 1, 70, "ExampleText",
 CellID->2036783074],
Cell[CellGroupData[{
Cell[29074, 763, 225, 6, 70, "Input",
 CellID->1323791538],
Cell[29302, 771, 6065, 136, 282, "Output",
 CellID->1991011009]
}, Open  ]],
Cell[CellGroupData[{
Cell[35404, 912, 225, 6, 70, "Input",
 CellID->1870802292],
Cell[35632, 920, 6064, 136, 282, "Output",
 CellID->831840709]
}, Open  ]],
Cell[41711, 1059, 96, 1, 70, "ExampleText",
 CellID->1065231632],
Cell[41810, 1062, 579, 20, 70, "Input",
 CellID->1924357632],
Cell[CellGroupData[{
Cell[42414, 1086, 281, 8, 70, "Input",
 CellID->1574997204],
Cell[42698, 1096, 6182, 138, 280, "Output",
 CellID->628254193]
}, Open  ]],
Cell[48895, 1237, 69, 1, 70, "ExampleText",
 CellID->1149581062],
Cell[CellGroupData[{
Cell[48989, 1242, 164, 4, 70, "Input",
 CellID->1165752543],
Cell[49156, 1248, 4963, 117, 282, "Output",
 CellID->2038613099]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[54168, 1371, 249, 7, 70, "ExampleSection"],
Cell[CellGroupData[{
Cell[54442, 1382, 261, 7, 70, "ExampleSubsection"],
Cell[54706, 1391, 127, 3, 70, "Input",
 CellID->600017015],
Cell[54836, 1396, 123, 3, 70, "ExampleText",
 CellID->1731213334],
Cell[CellGroupData[{
Cell[54984, 1403, 272, 7, 70, "Input",
 CellID->246571335],
Cell[55259, 1412, 4859, 115, 280, "Output",
 CellID->1813993131]
}, Open  ]],
Cell[60133, 1530, 97, 1, 70, "ExampleText",
 CellID->54562559],
Cell[CellGroupData[{
Cell[60255, 1535, 336, 8, 70, "Input",
 CellID->57411844],
Cell[60594, 1545, 5566, 127, 280, "Output",
 CellID->2104699905]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[66209, 1678, 261, 7, 70, "ExampleSubsection"],
Cell[66473, 1687, 128, 3, 70, "Input",
 CellID->2065565702],
Cell[66604, 1692, 81, 1, 70, "ExampleText",
 CellID->1889599759],
Cell[CellGroupData[{
Cell[66710, 1697, 360, 9, 70, "Input",
 CellID->1588449004],
Cell[67073, 1708, 5006, 117, 284, "Output",
 CellID->2016250655]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[72128, 1831, 262, 7, 70, "ExampleSubsection"],
Cell[72393, 1840, 417, 13, 70, "ExampleText",
 CellID->2089459734],
Cell[CellGroupData[{
Cell[72835, 1857, 228, 5, 70, "Input",
 CellID->1897153594],
Cell[73066, 1864, 7409, 176, 280, "Output",
 CellID->1980292572]
}, Open  ]],
Cell[80490, 2043, 552, 18, 70, "ExampleText",
 CellID->1122314517],
Cell[CellGroupData[{
Cell[81067, 2065, 660, 17, 70, "Input",
 CellID->1391215618],
Cell[81730, 2084, 6978, 171, 281, "Output",
 CellID->1075696398]
}, Open  ]],
Cell[88723, 2258, 86, 1, 70, "ExampleText",
 CellID->938735133],
Cell[CellGroupData[{
Cell[88834, 2263, 719, 18, 70, "Input",
 CellID->1906509469],
Cell[89556, 2283, 7942, 190, 281, "Output",
 CellID->1712020740]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[97547, 2479, 267, 7, 70, "ExampleSubsection"],
Cell[97817, 2488, 307, 6, 70, "ExampleText",
 CellID->1176106066],
Cell[CellGroupData[{
Cell[98149, 2498, 760, 22, 70, "Input",
 CellID->1166944104],
Cell[98912, 2522, 7074, 175, 280, "Output",
 CellID->1946674674]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[106035, 2703, 266, 7, 70, "ExampleSubsection"],
Cell[106304, 2712, 140, 4, 70, "ExampleText",
 CellID->1931171216],
Cell[CellGroupData[{
Cell[106469, 2720, 636, 16, 70, "Input",
 CellID->126786554],
Cell[107108, 2738, 9109, 213, 280, "Output",
 CellID->847883873]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[116266, 2957, 260, 7, 70, "ExampleSubsection"],
Cell[116529, 2966, 188, 4, 70, "ExampleText",
 CellID->1481494928],
Cell[CellGroupData[{
Cell[116742, 2974, 753, 22, 70, "Input",
 CellID->1559851578],
Cell[117498, 2998, 5633, 128, 280, "Output",
 CellID->1830255666]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[123192, 3133, 278, 8, 70, "ExampleSection",
 CellID->1305812373],
Cell[123473, 3143, 163, 4, 70, "ExampleText",
 CellID->2113280235],
Cell[CellGroupData[{
Cell[123661, 3151, 227, 5, 70, "Input",
 CellID->668871912],
Cell[123891, 3158, 7922, 183, 280, "Output",
 CellID->551218834]
}, Open  ]],
Cell[131828, 3344, 144, 4, 70, "ExampleText",
 CellID->1837323731],
Cell[CellGroupData[{
Cell[131997, 3352, 163, 4, 70, "Input",
 CellID->1232586704],
Cell[132163, 3358, 47544, 878, 289, "Output",
 CellID->90489880]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[179756, 4242, 275, 8, 70, "ExampleSection",
 CellID->589267740],
Cell[180034, 4252, 128, 3, 70, "Input",
 CellID->1172163540],
Cell[180165, 4257, 92, 1, 70, "ExampleText",
 CellID->1685122849],
Cell[180260, 4260, 578, 20, 70, "Input",
 CellID->659214597],
Cell[CellGroupData[{
Cell[180863, 4284, 806, 23, 70, "Input",
 CellID->845386507],
Cell[181672, 4309, 6352, 141, 296, "Output",
 CellID->444991084]
}, Open  ]]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[188085, 4457, 448, 13, 70, "TutorialsSection",
 CellID->250839057],
Cell[188536, 4472, 211, 5, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[CellGroupData[{
Cell[188784, 4482, 448, 13, 70, "MoreAboutSection",
 CellID->38303248],
Cell[189235, 4497, 171, 4, 70, "MoreAbout",
 CellID->539642637],
Cell[189409, 4503, 179, 4, 70, "MoreAbout",
 CellID->101302551]
}, Open  ]],
Cell[189603, 4510, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

