(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     14112,        461]
NotebookOptionsPosition[     10425,        348]
NotebookOutlinePosition[     12624,        406]
CellTagsIndexPosition[     12539,        401]
WindowTitle->GetSummands - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Series Solutions of ODEs\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/tutorial/SeriesSolutionsOfODEs"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"TutorialsSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Calculus Applications\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/guide/CalculusApplications"], \
"\<\"Classroom Utilities\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/guide/ClassroomUtilities"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"MoreAboutSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"ClassroomUtilities/ref/GetSummands\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["ClassroomUtilities/ref/GetSummands"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/ClassroomUtilities/\
ref/GetSummands.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "ClassroomUtilities/ref/GetSummands", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["GetSummands", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"GetSummands", "[", 
       RowBox[{
        StyleBox[
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"n", "=", 
           SubscriptBox[
            StyleBox["n", "TI"], 
            StyleBox["0", "TR"]]}], "\[Infinity]"], "TI"], 
        RowBox[{
         RowBox[{
          StyleBox["a", "TI"], "[", 
          StyleBox["n", "TI"], "]"}], " ", 
         SuperscriptBox[
          StyleBox["x", "TI"], 
          StyleBox["n", "TI"]]}]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]returns the general term a[i] ",
     Cell[BoxData[
      RowBox[{
       SuperscriptBox[
        StyleBox["x", "TI"], 
        StyleBox["i", "TI"]], " "}]]],
     "from the symbolic sum ."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"GetSummands", "[", 
       RowBox[{
        SubscriptBox[
         StyleBox["sum", "TI"], 
         StyleBox["1", "TR"]], "+", 
        SubscriptBox[
         StyleBox["sum", "TI"], 
         StyleBox["2", "TR"]], "+", " ", 
        StyleBox["\[Ellipsis]", "TR"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]returns a sum of general terms"
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {None, {None}}, 
   "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 WholeCellGroupOpener->True,
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->665895204],

Cell["\<\
For summations with closed form sums the general term will not be recognized.\
\>", "Notes",
 CellID->1067943069],

Cell["\<\
This function is used to find general terms in series solutions of ODE's.\
\>", "Notes",
 CellID->1761596084]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 WholeCellGroupOpener->True,
 CellTags->"PrimaryExamplesSection",
 CellID->145846841],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->275746715],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1935826246],

Cell["This gets the general terms out of two series.", "ExampleText",
 CellID->1054464472],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GetSummands", "[", 
  RowBox[{
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"n", "=", "0"}], "\[Infinity]"], 
    RowBox[{
     SuperscriptBox["x", "n"], " ", 
     RowBox[{"a", "[", "n", "]"}]}]}], "+", 
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"n", "=", "0"}], "\[Infinity]"], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "+", "n"}], ")"}], " ", 
     SuperscriptBox["x", "n"], " ", 
     RowBox[{"a", "[", 
      RowBox[{"1", "+", "n"}], "]"}]}]}]}], "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->1349325860],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["x", "n"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"a", "[", "n", "]"}], "+", 
    RowBox[{"a", "[", 
     RowBox[{"1", "+", "n"}], "]"}], "+", 
    RowBox[{"n", " ", 
     RowBox[{"a", "[", 
      RowBox[{"1", "+", "n"}], "]"}]}]}], ")"}]}]], "Output",
 ImageSize->{194, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->692416700]
}, Open  ]],

Cell["Use the general terms to find a recurrence relation.", "ExampleText",
 CellID->233593807],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     SuperscriptBox["x", "n"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"a", "[", "n", "]"}], "+", 
       RowBox[{"a", "[", 
        RowBox[{"1", "+", "n"}], "]"}], "+", 
       RowBox[{"n", " ", 
        RowBox[{"a", "[", 
         RowBox[{"1", "+", "n"}], "]"}]}]}], ")"}]}], "\[Equal]", "0"}], ",", 
   RowBox[{"a", "[", 
    RowBox[{"n", "+", "1"}], "]"}]}], "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->411007533],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", "[", 
     RowBox[{"1", "+", "n"}], "]"}], "\[Rule]", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{"a", "[", "n", "]"}], 
      RowBox[{"1", "+", "n"}]]}]}], "}"}], "}"}]], "Output",
 ImageSize->{138, 32},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->852659996]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 WholeCellGroupOpener->True,
 CellID->250839057],

Cell[TextData[ButtonBox["Series Solutions of ODEs",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/tutorial/SeriesSolutionsOfODEs"]], "Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 WholeCellGroupOpener->True,
 CellID->38303248],

Cell[TextData[ButtonBox["Calculus Applications",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/guide/CalculusApplications"]], "MoreAbout",
 CellID->115313532],

Cell[TextData[ButtonBox["Classroom Utilities",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/guide/ClassroomUtilities"]], "MoreAbout",
 CellID->238654608]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"GetSummands - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "SymbolColor", "LinkTrails" -> 
  GridBox[{{
      RowBox[{
        ButtonBox[
        "Classroom Utilities", ButtonData -> 
         "paclet:ClassroomUtilities/guide/ClassroomUtilities", 
         BaseStyle -> {"Link", "DockedLinkTrail"}], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 27, 14.4184624}", 
    "history" -> {"XX", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Symbol", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "GetSummands[\\[Sum]n = n 0 \\[Infinity] a[n] x^n] returns the general \
term a[i] x^i from the symbolic sum .GetSummands[sum_1 + sum_2 + ...] returns \
a sum of general terms", "synonyms" -> {}, "title" -> "GetSummands", 
    "titlemodifier" -> "", "windowtitle" -> "GetSummands", "type" -> "Symbol",
     "uri" -> "ClassroomUtilities/ref/GetSummands"}, "SearchTextTranslated" -> 
  ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[5840, 174, 490, 14, 45, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->145846841]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 12396, 394}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[604, 21, 2873, 65, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3502, 90, 54, 1, 51, "ObjectName",
 CellID->1224892054],
Cell[3559, 93, 1350, 43, 148, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4946, 141, 609, 18, 39, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->665895204],
Cell[5558, 161, 123, 3, 70, "Notes",
 CellID->1067943069],
Cell[5684, 166, 119, 3, 70, "Notes",
 CellID->1761596084]
}, Closed]],
Cell[CellGroupData[{
Cell[5840, 174, 490, 14, 45, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->145846841],
Cell[CellGroupData[{
Cell[6355, 192, 195, 6, 31, "ExampleSection",
 CellID->275746715],
Cell[6553, 200, 128, 3, 28, "Input",
 CellID->1935826246],
Cell[6684, 205, 90, 1, 32, "ExampleText",
 CellID->1054464472],
Cell[CellGroupData[{
Cell[6799, 210, 577, 19, 50, "Input",
 CellID->1349325860],
Cell[7379, 231, 431, 15, 36, "Output",
 CellID->692416700]
}, Open  ]],
Cell[7825, 249, 95, 1, 32, "ExampleText",
 CellID->233593807],
Cell[CellGroupData[{
Cell[7945, 254, 511, 17, 30, "Input",
 CellID->411007533],
Cell[8459, 273, 393, 14, 53, "Output",
 CellID->852659996]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[8913, 294, 448, 13, 70, "TutorialsSection",
 CellID->250839057],
Cell[9364, 309, 182, 4, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[CellGroupData[{
Cell[9583, 318, 448, 13, 70, "MoreAboutSection",
 CellID->38303248],
Cell[10034, 333, 175, 4, 70, "MoreAbout",
 CellID->115313532],
Cell[10212, 339, 171, 4, 70, "MoreAbout",
 CellID->238654608]
}, Open  ]],
Cell[10398, 346, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

