(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     12773,        411]
NotebookOptionsPosition[      9214,        303]
NotebookOutlinePosition[     11390,        360]
CellTagsIndexPosition[     11305,        355]
WindowTitle->MakeSeries - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Series Solutions of ODEs\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/tutorial/SeriesSolutionsOfODEs"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"TutorialsSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Calculus Applications\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/guide/CalculusApplications"], \
"\<\"Classroom Utilities\"\>":>Documentation`HelpLookup[
       "paclet:ClassroomUtilities/guide/ClassroomUtilities"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"MoreAboutSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"ClassroomUtilities/ref/MakeSeries\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["ClassroomUtilities/ref/MakeSeries"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/ClassroomUtilities/\
ref/MakeSeries.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "ClassroomUtilities/ref/MakeSeries", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["MakeSeries", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"MakeSeries", "[", 
       RowBox[{
        StyleBox["x", "TI"], ",", " ", 
        StyleBox["a", "TI"], ",", " ", 
        StyleBox["n", "TI"], ",", " ", 
        StyleBox["ord", "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]creates a series summation for of the form ",
     Cell[BoxData[
      RowBox[{
       ButtonBox["Sum",
        BaseStyle->"Link",
        ButtonData->"paclet:ref/Sum"], "[", " ", 
       RowBox[{
        RowBox[{
         RowBox[{
          StyleBox["a", "TI"], "[", 
          StyleBox["n", "TI"], "]"}], " ", 
         RowBox[{
          StyleBox["x", "TI"], "^", 
          StyleBox["n", "TI"]}]}], ",", " ", 
        RowBox[{"{", 
         RowBox[{
          StyleBox["n", "TI"], ",", "0", ",", 
          ButtonBox["Infinity",
           BaseStyle->"Link",
           ButtonData->"paclet:ref/Infinity"]}], "}"}]}], "]"}]], 
      "InlineFormula"],
     " together with derivative of the series up to order ",
     Cell[BoxData[
      StyleBox["ord", "TI"]], "InlineFormula"],
     "."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 WholeCellGroupOpener->True,
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->177100871],

Cell["\<\
This is intended for use in solving ODEs using a series solution approach.\
\>", "Notes",
 CellID->1067943069],

Cell[TextData[{
 "The order ",
 Cell[BoxData[
  StyleBox["ord", "TI"]], "InlineFormula"],
 " should be given as an integer."
}], "Notes",
 CellID->1163593359],

Cell["All other arguments should be symbols.", "Notes",
 CellID->1591150479],

Cell["", "Notes",
 CellID->1527495131]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 WholeCellGroupOpener->True,
 CellTags->"PrimaryExamplesSection",
 CellID->138676619],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->22481800],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1999728109],

Cell["Make a symbolic series and its first derivative.", "ExampleText",
 CellID->1311252413],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MakeSeries", "[", 
  RowBox[{"x", ",", "a", ",", "n", ",", "1"}], "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->742119777],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"n", "=", "0"}], "\[Infinity]"], 
    RowBox[{
     SuperscriptBox["x", "n"], " ", 
     RowBox[{"a", "[", "n", "]"}]}]}], ",", 
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"n", "=", "0"}], "\[Infinity]"], 
    RowBox[{"n", " ", 
     SuperscriptBox["x", 
      RowBox[{
       RowBox[{"-", "1"}], "+", "n"}]], " ", 
     RowBox[{"a", "[", "n", "]"}]}]}]}], "}"}]], "Output",
 ImageSize->{176, 38},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->222070263]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 WholeCellGroupOpener->True,
 CellID->250839057],

Cell[TextData[ButtonBox["Series Solutions of ODEs",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/tutorial/SeriesSolutionsOfODEs"]], "Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 WholeCellGroupOpener->True,
 CellID->38303248],

Cell[TextData[ButtonBox["Calculus Applications",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/guide/CalculusApplications"]], "MoreAbout",
 CellID->335265824],

Cell[TextData[ButtonBox["Classroom Utilities",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/guide/ClassroomUtilities"]], "MoreAbout",
 CellID->205312961]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"MakeSeries - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "SymbolColor", "LinkTrails" -> 
  GridBox[{{
      RowBox[{
        ButtonBox[
        "Classroom Utilities", ButtonData -> 
         "paclet:ClassroomUtilities/guide/ClassroomUtilities", 
         BaseStyle -> {"Link", "DockedLinkTrail"}], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 28, 1.9672624}", 
    "history" -> {"XX", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Symbol", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "MakeSeries[x, a, n, ord] creates a series summation for of the form \
Sum[a[n] x^n, {n, 0, Infinity}] together with derivative of the series up to \
order ord.", "synonyms" -> {}, "title" -> "MakeSeries", "titlemodifier" -> "",
     "windowtitle" -> "MakeSeries", "type" -> "Symbol", "uri" -> 
    "ClassroomUtilities/ref/MakeSeries"}, "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[5909, 179, 490, 14, 45, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->138676619]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 11162, 348}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[603, 21, 2869, 65, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3497, 90, 53, 1, 51, "ObjectName",
 CellID->1224892054],
Cell[3553, 93, 1269, 39, 101, "Usage"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4859, 137, 609, 18, 39, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->177100871],
Cell[5471, 157, 120, 3, 70, "Notes",
 CellID->1067943069],
Cell[5594, 162, 158, 6, 70, "Notes",
 CellID->1163593359],
Cell[5755, 170, 76, 1, 70, "Notes",
 CellID->1591150479],
Cell[5834, 173, 38, 1, 70, "Notes",
 CellID->1527495131]
}, Closed]],
Cell[CellGroupData[{
Cell[5909, 179, 490, 14, 45, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->138676619],
Cell[CellGroupData[{
Cell[6424, 197, 194, 6, 31, "ExampleSection",
 CellID->22481800],
Cell[6621, 205, 128, 3, 28, "Input",
 CellID->1999728109],
Cell[6752, 210, 92, 1, 32, "ExampleText",
 CellID->1311252413],
Cell[CellGroupData[{
Cell[6869, 215, 149, 4, 28, "Input",
 CellID->742119777],
Cell[7021, 221, 620, 21, 59, "Output",
 CellID->222070263]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[7702, 249, 448, 13, 45, "TutorialsSection",
 CellID->250839057],
Cell[8153, 264, 182, 4, 17, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[CellGroupData[{
Cell[8372, 273, 448, 13, 45, "MoreAboutSection",
 CellID->38303248],
Cell[8823, 288, 175, 4, 17, "MoreAbout",
 CellID->335265824],
Cell[9001, 294, 171, 4, 70, "MoreAbout",
 CellID->205312961]
}, Open  ]],
Cell[9187, 301, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

