(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     23578,        780]
NotebookOptionsPosition[     18053,        603]
NotebookOutlinePosition[     20503,        667]
CellTagsIndexPosition[     20383,        661]
WindowTitle->PolygonInteriorQ - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Problems in Analytic Geometry\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/tutorial/ProblemsInAnalyticGeometry"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"TutorialsSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Classroom Utilities/ref/PolygonInteriorQ\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Classroom Utilities/ref/PolygonInteriorQ"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Classroom \
Utilities/ref/PolygonInteriorQ.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Classroom Utilities/ref/PolygonInteriorQ", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell["PolygonInteriorQ", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"PolygonInteriorQ", "[", 
       RowBox[{
        StyleBox["pt", "TI"], ",", 
        StyleBox["poly", "TI"]}], "]"}]], "InlineFormula"],
     " \[LineSeparator]returns True if the point ",
     Cell[BoxData[
      StyleBox["pt", "TI"]], "InlineFormula"],
     " is located in the interior of the polygon formed by the vertices ",
     Cell[BoxData[
      StyleBox["poly", "TI"]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"PolygonInteriorQ", "[", 
       RowBox[{
        StyleBox["ptlis", "TI"], ",", 
        StyleBox["poly", "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]returns a list of ordered pairs of the form \
{pt,testval} for the points in ",
     Cell[BoxData[
      StyleBox["ptlis", "TI"]], "InlineFormula"],
     "."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {None, {None}}, 
   "RowsIndexed" -> {}}},
 CellID->982511436]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 WholeCellGroupOpener->True,
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->43795294],

Cell[TextData[{
 "The ",
 Cell[BoxData[
  StyleBox["pt", "TI"]], "InlineFormula"],
 " value can either be an ordered pair or a list of ordered pairs."
}], "Notes",
 CellID->1067943069],

Cell["\<\
The polygon should be specified as a list of ordered pairs in the order to \
correctly render the polygon.\
\>", "Notes",
 CellID->1238034398],

Cell[TextData[{
 "If you are testing a single point and want the point returned with the test \
value put it in a list. {",
 Cell[BoxData[
  StyleBox["pt", "TI"]], "InlineFormula"],
 "}."
}], "Notes",
 CellID->675071403]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 WholeCellGroupOpener->True,
 CellTags->"PrimaryExamplesSection",
 CellID->132073712],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->471860901],

Cell["this loads the package.", "ExampleText",
 CellID->768963120],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->406570194],

Cell["\<\
Create a list of points and see if a given point is on the interior of the \
corresponding polygon.\
\>", "ExampleText",
 CellID->2079769451],

Cell[BoxData[
 RowBox[{
  RowBox[{"pts1", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"-", 
        FractionBox["1", "2"]}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", 
       SqrtBox["3"]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "3"}], ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       FractionBox["1", "2"]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "3"}], ",", 
       FractionBox["1", "2"]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "3"}], ",", 
       RowBox[{"-", 
        FractionBox["1", "2"]}]}], "}"}]}], "}"}]}], ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->93840738],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolygonInteriorQ", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{"1", "/", "2"}]}], "}"}], ",", "pts1"}], "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->928286668],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->1854338902]
}, Open  ]],

Cell["Visualize the point and polygon.", "ExampleText",
 CellID->1591240650],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graphics", "[", 
  RowBox[{"{", 
   RowBox[{"Yellow", ",", 
    RowBox[{"Polygon", "[", "pts1", "]"}], ",", "Red", ",", 
    RowBox[{"PointSize", "[", ".03", "]"}], ",", 
    RowBox[{"Point", "[", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"1", "/", "2"}]}], "}"}], "]"}]}], "}"}], "]"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->1367748459],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 1, 0], 
    PolygonBox[
     NCache[{{1, Rational[-1, 2]}, {1, 3^Rational[1, 2]}, {-3, 
       3}, {-1, Rational[1, 2]}, {-3, Rational[1, 2]}, {-3, 
        Rational[-1, 2]}}, {{1, -0.5}, {1, 1.7320508075688772`}, {-3, 
       3}, {-1, 0.5}, {-3, 0.5}, {-3, -0.5}}]]}, 
   {RGBColor[1, 0, 0], PointSize[0.03], 
    PointBox[NCache[{0, Rational[1, 2]}, {0, 0.5}]]}}]], "Output",
 ImageSize->{184, 158},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->95466802]
}, Open  ]],

Cell["Check several points at once.", "ExampleText",
 CellID->340256905],

Cell[BoxData[
 RowBox[{
  RowBox[{"pts2", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1.363"}], ",", "2.066"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0.2928", ",", "1.471"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2.088"}], ",", "1.062"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0.4788", ",", "2.661"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2.386"}], ",", "0.2061"}], "}"}]}], "}"}]}], 
  ";"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->2096334021],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"pttst", "=", 
  RowBox[{"PolygonInteriorQ", "[", 
   RowBox[{"pts2", ",", "pts1"}], "]"}]}]], "Input",
 CellLabel->"In[6]:=",
 CellID->1469178265],

Cell[BoxData[
 RowBox[{"PolygonInteriorQ", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1.363`"}], ",", "2.066`"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0.2928`", ",", "1.471`"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2.088`"}], ",", "1.062`"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0.4788`", ",", "2.661`"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2.386`"}], ",", "0.2061`"}], "}"}]}], "}"}], ",", 
   "pts1"}], "]"}]], "Output",
 ImageSize->{409, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->710549539]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graphics", "[", 
  RowBox[{"{", 
   RowBox[{"Yellow", ",", 
    RowBox[{"Polygon", "[", "pts1", "]"}], ",", "Red", ",", 
    RowBox[{"PointSize", "[", ".02", "]"}], ",", 
    RowBox[{"Point", "[", "pts2", "]"}]}], "}"}], "]"}]], "Input",
 CellLabel->"In[7]:=",
 CellID->786225012],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 1, 0], 
    PolygonBox[
     NCache[{{1, Rational[-1, 2]}, {1, 3^Rational[1, 2]}, {-3, 
       3}, {-1, Rational[1, 2]}, {-3, Rational[1, 2]}, {-3, 
        Rational[-1, 2]}}, {{1, -0.5}, {1, 1.7320508075688772`}, {-3, 
       3}, {-1, 0.5}, {-3, 0.5}, {-3, -0.5}}]]}, 
   {RGBColor[1, 0, 0], PointSize[0.02], 
    PointBox[{{-1.363, 2.066}, {0.2928, 1.471}, {-2.088, 1.062}, {0.4788, 
     2.661}, {-2.386, 0.2061}}]}}]], "Output",
 ImageSize->{184, 158},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->1559516453]
}, Open  ]],

Cell["Get the points on the interior of the polygon.", "ExampleText",
 CellID->755113936],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases", "[", 
  RowBox[{"pttst", ",", 
   RowBox[{"{", 
    RowBox[{"_", ",", "True"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[8]:=",
 CellID->1719052229],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1.363`"}], ",", "2.066`"}], "}"}], ",", "True"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0.2928`", ",", "1.471`"}], "}"}], ",", "True"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2.386`"}], ",", "0.2061`"}], "}"}], ",", "True"}], 
    "}"}]}], "}"}]], "Output",
 ImageSize->{514, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->2013309712]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Applications",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->258228157],

Cell["This loads the package.", "ExampleText",
 CellID->1020732261],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1347738391],

Cell["\<\
Check to see if the coordinates you have for a city are inside the country.\
\>", "ExampleText",
 CellID->1095332335],

Cell[BoxData[
 RowBox[{
  RowBox[{"CN", "=", 
   RowBox[{
    RowBox[{"CountryData", "[", 
     RowBox[{"\"\<China\>\"", ",", "\"\<Polygon\>\""}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "1"}], "]"}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->1929112628],

Cell[BoxData[
 RowBox[{
  RowBox[{"snghai", "=", 
   RowBox[{"CityData", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"\"\<Shanghai\>\"", ",", "\"\<China\>\""}], "}"}], ",", 
     "\"\<Coordinates\>\""}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->2035320037],

Cell["Latitude and longitude are reversed.", "ExampleText",
 CellID->777655459],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolygonInteriorQ", "[", 
  RowBox[{"snghai", ",", "CN"}], "]"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->356848796],

Cell[BoxData["False"], "Output",
 ImageSize->{39, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->1783371948]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolygonInteriorQ", "[", 
  RowBox[{
   RowBox[{"Reverse", "@", "snghai"}], ",", "CN"}], "]"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->703068857],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->1023253805]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graphics", "[", 
  RowBox[{"{", 
   RowBox[{"Yellow", ",", 
    RowBox[{"Polygon", "[", "CN", "]"}], ",", "Blue", ",", 
    RowBox[{"Point", "[", 
     RowBox[{"Reverse", "@", "snghai"}], "]"}]}], "}"}], "]"}]], "Input",
 CellLabel->"In[6]:=",
 CellID->469823771],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztXS2Q4koQpupMJDI2EhmJjUQiI496CvEEEhsZicQicbcyFolExiIjI+fl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    "], {{0, 101}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 101},
  PlotRange->{{0, 180}, {0, 101}}]], "Output",
 ImageSize->{184, 101},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[6]=",
 CellID->23001315]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 WholeCellGroupOpener->True,
 CellID->250839057],

Cell[TextData[ButtonBox["Problems in Analytic Geometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/tutorial/ProblemsInAnalyticGeometry"]], \
"Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"PolygonInteriorQ - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "SymbolColor", "LinkTrails" -> 
  GridBox[{{
      RowBox[{
        StyleBox[
         ButtonBox[
         "Mathematica", ButtonData -> "paclet:guide/Mathematica", 
          BaseStyle -> {"Link", "DockedLinkTrail"}], FontSlant -> "Italic"], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 29, 55.4416624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Symbol", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "PolygonInteriorQ[pt, poly] returns True if the point pt is located in \
the interior of the polygon formed by the vertices \
poly.PolygonInteriorQ[ptlis, poly] returns a list of ordered pairs of the \
form {pt,testval} for the points in ptlis.", "synonyms" -> {}, "title" -> 
    "PolygonInteriorQ", "titlemodifier" -> "", "windowtitle" -> 
    "PolygonInteriorQ", "type" -> "Symbol", "uri" -> 
    "Classroom Utilities/ref/PolygonInteriorQ"}, "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[5502, 168, 490, 14, 45, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->132073712]},
 "RasterizedOutput"->{
  Cell[14587, 518, 2718, 53, 70, "Output",
   CellTags->"RasterizedOutput",
   CellID->23001315]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 20118, 650},
 {"RasterizedOutput", 20261, 654}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[609, 21, 2496, 57, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3130, 82, 59, 1, 51, "ObjectName",
 CellID->1224892054],
Cell[3192, 85, 1063, 32, 146, "Usage",
 CellID->982511436]
}, Open  ]],
Cell[CellGroupData[{
Cell[4292, 122, 608, 18, 39, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->43795294],
Cell[4903, 142, 184, 6, 70, "Notes",
 CellID->1067943069],
Cell[5090, 150, 152, 4, 70, "Notes",
 CellID->1238034398],
Cell[5245, 156, 220, 7, 70, "Notes",
 CellID->675071403]
}, Closed]],
Cell[CellGroupData[{
Cell[5502, 168, 490, 14, 45, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->132073712],
Cell[CellGroupData[{
Cell[6017, 186, 195, 6, 31, "ExampleSection",
 CellID->471860901],
Cell[6215, 194, 66, 1, 32, "ExampleText",
 CellID->768963120],
Cell[6284, 197, 127, 3, 28, "Input",
 CellID->406570194],
Cell[6414, 202, 151, 4, 32, "ExampleText",
 CellID->2079769451],
Cell[6568, 208, 780, 29, 44, "Input",
 CellID->93840738],
Cell[CellGroupData[{
Cell[7373, 241, 213, 7, 28, "Input",
 CellID->928286668],
Cell[7589, 250, 162, 5, 36, "Output",
 CellID->1854338902]
}, Open  ]],
Cell[7766, 258, 76, 1, 32, "ExampleText",
 CellID->1591240650],
Cell[CellGroupData[{
Cell[7867, 263, 383, 11, 70, "Input",
 CellID->1367748459],
Cell[8253, 276, 553, 14, 179, "Output",
 CellID->95466802]
}, Open  ]],
Cell[8821, 293, 72, 1, 70, "ExampleText",
 CellID->340256905],
Cell[8896, 296, 573, 20, 70, "Input",
 CellID->2096334021],
Cell[CellGroupData[{
Cell[9494, 320, 170, 5, 70, "Input",
 CellID->1469178265],
Cell[9667, 327, 697, 23, 54, "Output",
 CellID->710549539]
}, Open  ]],
Cell[CellGroupData[{
Cell[10401, 355, 304, 8, 70, "Input",
 CellID->786225012],
Cell[10708, 365, 610, 15, 179, "Output",
 CellID->1559516453]
}, Open  ]],
Cell[11333, 383, 89, 1, 70, "ExampleText",
 CellID->755113936],
Cell[CellGroupData[{
Cell[11447, 388, 181, 6, 70, "Input",
 CellID->1719052229],
Cell[11631, 396, 617, 23, 36, "Output",
 CellID->2013309712]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[12297, 425, 274, 8, 70, "ExampleSection",
 CellID->258228157],
Cell[12574, 435, 67, 1, 70, "ExampleText",
 CellID->1020732261],
Cell[12644, 438, 128, 3, 70, "Input",
 CellID->1347738391],
Cell[12775, 443, 127, 3, 70, "ExampleText",
 CellID->1095332335],
Cell[12905, 448, 284, 9, 70, "Input",
 CellID->1929112628],
Cell[13192, 459, 281, 9, 70, "Input",
 CellID->2035320037],
Cell[13476, 470, 79, 1, 70, "ExampleText",
 CellID->777655459],
Cell[CellGroupData[{
Cell[13580, 475, 141, 4, 70, "Input",
 CellID->356848796],
Cell[13724, 481, 163, 5, 36, "Output",
 CellID->1783371948]
}, Open  ]],
Cell[CellGroupData[{
Cell[13924, 491, 171, 5, 70, "Input",
 CellID->703068857],
Cell[14098, 498, 162, 5, 36, "Output",
 CellID->1023253805]
}, Open  ]],
Cell[CellGroupData[{
Cell[14297, 508, 287, 8, 70, "Input",
 CellID->469823771],
Cell[14587, 518, 2718, 53, 70, "Output",
 CellTags->"RasterizedOutput",
 CellID->23001315]
}, Open  ]]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[17366, 578, 448, 13, 70, "TutorialsSection",
 CellID->250839057],
Cell[17817, 593, 194, 5, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[18026, 601, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

