(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     94771,       2438]
NotebookOptionsPosition[     85595,       2153]
NotebookOutlinePosition[     88785,       2228]
CellTagsIndexPosition[     88663,       2222]
WindowTitle->RiemannSumPlot3D - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Working with Riemann Sums\"\>":>
       Documentation`HelpLookup[
       "paclet:Classroom Utilities/tutorial/WorkingWithRiemannSums"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"TutorialsSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Classroom Utilities/ref/RiemannSumPlot3D\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Classroom Utilities/ref/RiemannSumPlot3D"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Classroom \
Utilities/ref/RiemannSumPlot3D.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Classroom Utilities/ref/RiemannSumPlot3D", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell["RiemannSumPlot3D", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"RiemannSumPlot3D", "[", 
       RowBox[{
        StyleBox["f", "TI"], ",", 
        RowBox[{"{", 
         RowBox[{
          StyleBox["x", "TI"], ",", 
          SubscriptBox[
           StyleBox["x", "TI"], 
           StyleBox["min", "TI"]], ",", 
          SubscriptBox[
           StyleBox["x", "TI"], 
           StyleBox["max", "TI"]]}], "}"}], ",", 
        StyleBox["n", "TI"], ",", "\"\<Disk\>\""}], "]"}]], "InlineFormula"],
     " \[LineSeparator]creates a stack of ",
     Cell[BoxData[
      StyleBox["n", "TI"]], "InlineFormula"],
     " disks with radius determined by the value of x at the point f(x) to \
approximate the volume obtained by revolving ",
     Cell[BoxData[
      StyleBox["f", "TI"]], "InlineFormula"],
     " around the vertical axis."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"RiemannSumPlot3D", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          SubscriptBox[
           StyleBox["f", "TI"], 
           StyleBox["1", "TR"]], ",", 
          SubscriptBox[
           StyleBox["f", "TI"], 
           StyleBox["2", "TR"]]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          StyleBox["x", "TI"], ",", 
          SubscriptBox[
           StyleBox["x", "TI"], 
           StyleBox["min", "TI"]], ",", 
          SubscriptBox[
           StyleBox["x", "TI"], 
           StyleBox["max", "TI"]]}], "}"}], ",", 
        StyleBox["n", "TI"], ",", "\"\<Shell\>\""}], "]"}]], "InlineFormula"],
     "\[LineSeparator]creates ",
     Cell[BoxData[
      StyleBox["n", "TI"]], "InlineFormula"],
     " shells of height ",
     Cell[BoxData[
      RowBox[{
       RowBox[{
        SubscriptBox[
         StyleBox["f", "TI"], 
         StyleBox["2", "TR"]], 
        RowBox[{"(", 
         StyleBox["x", "TI"], ")"}]}], "-", 
       RowBox[{
        SubscriptBox[
         StyleBox["f", "TI"], 
         StyleBox["1", "TR"]], 
        RowBox[{"(", 
         StyleBox["x", "TI"], ")"}], " ", "with", " ", "radii", " ", "taken", 
        " ", "from", " ", "the", " ", "interval", " ", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox[
           StyleBox["x", "TI"], 
           StyleBox["min", "TI"]], ",", 
          SubscriptBox[
           StyleBox["x", "TI"], 
           StyleBox["max", "TI"]]}], ")"}]}]}]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"RiemannSumPlot3D", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          SubscriptBox[
           StyleBox["f", "TI"], 
           StyleBox["1", "TR"]], ",", 
          SubscriptBox[
           StyleBox["f", "TI"], 
           StyleBox["2", "TR"]]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          StyleBox["x", "TI"], ",", 
          SubscriptBox[
           StyleBox["x", "TI"], 
           StyleBox["min", "TI"]], ",", 
          SubscriptBox[
           StyleBox["x", "TI"], 
           StyleBox["max", "TI"]]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          StyleBox["z", "TI"], ",", 
          SubscriptBox[
           StyleBox["z", "TI"], 
           StyleBox["min", "TI"]], ",", 
          SubscriptBox[
           StyleBox["z", "TI"], 
           StyleBox["max", "TI"]]}], "}"}], ",", 
        StyleBox["n", "TI"], ",", "\"\<Washer\>\""}], "]"}]], "InlineFormula"],
     "\[LineSeparator]creates a stack of ",
     Cell[BoxData[
      StyleBox["n", "TI"]], "InlineFormula"],
     " washers with radii ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["x", "TI"], 
       StyleBox["i", "TI"]]], "InlineFormula"],
     " and ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["x", "TI"], 
       StyleBox["j", "TI"]]], "InlineFormula"],
     " where ",
     Cell[BoxData[
      RowBox[{
       RowBox[{
        SubscriptBox[
         StyleBox["f", "TI"], 
         StyleBox["1", "TR"]], 
        RowBox[{"(", 
         SubscriptBox[
          StyleBox["x", "TI"], 
          StyleBox["i", "TI"]], ")"}]}], " ", "=", " ", 
       RowBox[{
        SubscriptBox[
         StyleBox["f", "TI"], 
         StyleBox["2", "TR"]], 
        RowBox[{"(", 
         SubscriptBox[
          StyleBox["x", "TI"], 
          StyleBox["j", "TI"]], ")"}]}]}]], "InlineFormula"],
     " with thickness (",
     Cell[BoxData[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox[
          StyleBox["z", "TI"], 
          StyleBox["max", "TI"]], "-", 
         SubscriptBox[
          StyleBox["z", "TI"], 
          StyleBox["min", "TI"]]}], ")"}], "/", 
       StyleBox["n", "TI"]}]], "InlineFormula"],
     "."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"RiemannSumPlot3D", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            SubscriptBox[
             StyleBox["f", "TI"], 
             StyleBox["i", "TI"]], ",", 
            StyleBox["\[Ellipsis]", "TR"]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            SubscriptBox[
             StyleBox["g", "TI"], 
             StyleBox["j", "TI"]], ",", 
            StyleBox["\[Ellipsis]", "TR"]}], "}"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          StyleBox["x", "TI"], ",", 
          SubscriptBox[
           StyleBox["x", "TI"], 
           StyleBox["min", "TI"]], ",", 
          SubscriptBox[
           StyleBox["x", "TI"], 
           StyleBox["m1", "TI"]], ",", 
          SubscriptBox[
           StyleBox["x", "TI"], 
           StyleBox["m2", "TI"]], ",", 
          SubscriptBox[
           StyleBox["x", "TI"], 
           StyleBox["max", "TI"]]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          StyleBox["z", "TI"], ",", 
          SubscriptBox[
           StyleBox["z", "TI"], 
           StyleBox["min", "TI"]], ",", 
          SubscriptBox[
           StyleBox["z", "TI"], 
           StyleBox["max", "TI"]]}], "}"}], ",", 
        StyleBox["n", "TI"], ",", "\"\<Washer\>\""}], "]"}]], "InlineFormula"],
     "\[LineSeparator]",
     "evaluates the functions ",
     Cell[BoxData[
      RowBox[{
       SubscriptBox[
        StyleBox["f", "TI"], 
        StyleBox["i", "TI"]], " "}]], "InlineFormula"],
     "and ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["g", "TI"], 
       StyleBox["i", "TI"]]], "InlineFormula"],
     " on the intervals ",
     Cell[BoxData[
      RowBox[{"(", 
       RowBox[{
        SubscriptBox[
         StyleBox["x", "TI"], 
         StyleBox["min", "TI"]], ",", 
        SubscriptBox[
         StyleBox["x", "TI"], 
         StyleBox["mid1", "TI"]]}], ")"}]], "InlineFormula"],
     " and  creates n washers of thickness ",
     Cell[BoxData[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox[
          StyleBox["z", "TI"], 
          StyleBox["max", "TI"]], "-", 
         SubscriptBox[
          StyleBox["z", "TI"], 
          StyleBox["min", "TI"]]}], ")"}], "/", 
       StyleBox["n", "TI"]}]], "InlineFormula"],
     " with inner radius determined by ",
     Cell[BoxData[
      StyleBox["f", "TI"]], "InlineFormula"],
     " and the outer radius by ",
     Cell[BoxData[
      StyleBox["g", "TI"]], "InlineFormula"],
     "."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, 
   "Rows" -> {None, None, None, {None}}, "RowsIndexed" -> {}}},
 CellID->982511436]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 WholeCellGroupOpener->True,
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->86187868],

Cell[TextData[{
 "Functions in the list ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{
    SubscriptBox[
     StyleBox["f", "TI"], 
     StyleBox["i", "TI"]], ",", 
    StyleBox["\[Ellipsis]", "TR"]}], "}"}]], "InlineFormula"],
 " should collectively satisfy the horizontal line test when plotted on the \
plane."
}], "Notes",
 CellID->1067943069],

Cell[TextData[{
 "The same holds for the functions in the list ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{
    SubscriptBox[
     StyleBox["g", "TI"], 
     StyleBox["i", "TI"]], ",", " ", 
    StyleBox["\[Ellipsis]", "TR"]}], "}"}]], "InlineFormula"],
 "."
}], "Notes",
 CellID->2000816085],

Cell["\<\
All functions in the given list must be strictly monotone on the specified \
interval.\
\>", "Notes",
 CellID->1105379435],

Cell["The Shell function is used to create the washers.", "Notes",
 CellID->269512222],

Cell["The FullForm of any shell is a GraphicsComplex.", "Notes",
 CellID->648903631],

Cell["Multiple shells are given as a list of GraphicsComplexes.", "Notes",
 CellID->1201366751],

Cell[TextData[{
 "Full disks are created using the graphics primitive, ",
 ButtonBox["Cylinder",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cylinder"],
 "."
}], "Notes",
 CellID->729734963],

Cell["\<\
Multiple disks take advantage of being able to represent many cylinders in a \
single expression.\
\>", "Notes",
 CellID->376415124],

Cell[TextData[{
 "When partial disks or shells are used the angle used is of the form ",
 Cell[BoxData[
  RowBox[{
   StyleBox["k", "TI"], " ", 
   RowBox[{
    StyleBox["\[Pi]", "TR"], "/", "72"}]}]], "InlineFormula"],
 "nearest the given angle."
}], "Notes",
 CellID->2069648653],

Cell["\<\
When using washers two functions need to be given, that can include a \
vertical segment.\
\>", "Notes",
 CellID->258152475]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 WholeCellGroupOpener->True,
 CellTags->"PrimaryExamplesSection",
 CellID->620116192],

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(0)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->248693115],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"],
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->1962642188],

Cell["This loads the package.", "ExampleText",
 CellID->602544230],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1575478005],

Cell["\<\
The simplest case uses disks to approximate the volume obtained by revolving \
a curve around the vertical axis. This uses twelve disks.\
\>", "ExampleText",
 CellID->585462525],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSumPlot3D", "[", 
  RowBox[{
   SuperscriptBox["x", "2"], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", "12", ",", 
   "\"\<Disk\>\""}], "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->98587121],

Cell[BoxData[
 Graphics3DBox[{{}, 
   CylinderBox[
    NCache[{{{0, 0, 0}, {0, 0, Rational[1, 12]}}, {{0, 0, Rational[1, 12]}, {
       0, 0, Rational[1, 6]}}, {{0, 0, Rational[1, 6]}, {
       0, 0, Rational[1, 4]}}, {{0, 0, Rational[1, 4]}, {
       0, 0, Rational[1, 3]}}, {{0, 0, Rational[1, 3]}, {
       0, 0, Rational[5, 12]}}, {{0, 0, Rational[5, 12]}, {
       0, 0, Rational[1, 2]}}, {{0, 0, Rational[1, 2]}, {
       0, 0, Rational[7, 12]}}, {{0, 0, Rational[7, 12]}, {
       0, 0, Rational[2, 3]}}, {{0, 0, Rational[2, 3]}, {
       0, 0, Rational[3, 4]}}, {{0, 0, Rational[3, 4]}, {
       0, 0, Rational[5, 6]}}, {{0, 0, Rational[5, 6]}, {
       0, 0, Rational[11, 12]}}, {{0, 0, Rational[11, 12]}, {0, 0, 1}}}, {{{0,
        0, 0}, {0, 0, 0.08333333333333333}}, {{0, 0, 0.08333333333333333}, {
       0, 0, 0.16666666666666666`}}, {{0, 0, 0.16666666666666666`}, {
       0, 0, 0.25}}, {{0, 0, 0.25}, {0, 0, 0.3333333333333333}}, {{
       0, 0, 0.3333333333333333}, {0, 0, 0.4166666666666667}}, {{
       0, 0, 0.4166666666666667}, {0, 0, 0.5}}, {{0, 0, 0.5}, {
       0, 0, 0.5833333333333334}}, {{0, 0, 0.5833333333333334}, {
       0, 0, 0.6666666666666666}}, {{0, 0, 0.6666666666666666}, {
       0, 0, 0.75}}, {{0, 0, 0.75}, {0, 0, 0.8333333333333334}}, {{
       0, 0, 0.8333333333333334}, {0, 0, 0.9166666666666666}}, {{
       0, 0, 0.9166666666666666}, {0, 0, 1}}}], {0.2041241841597826, 
    0.3535533974754966, 0.45643546817774155`, 0.5400617248631179, 
    0.6123724365159084, 0.6770032013113849, 0.735980073114286, 
    0.7905694150680643, 0.8416254117066694, 0.8897565210105393, 
    0.9354143471647444, 0.9789450102721969}]}]], "Output",
 ImageSize->{184, 156},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->1243850810,
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell["\<\
Multiple curves require the use of shells or washers. This gives the volume \
generated by the area between two curves.\
\>", "ExampleText",
 CellID->1794743772],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSumPlot3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Sin", "[", "x", "]"}], ",", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"2", 
        RowBox[{"x", "/", 
         RowBox[{"(", "\[Pi]", ")"}]}]}], ")"}], "2"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", 
     RowBox[{"\[Pi]", "/", "2"}]}], "}"}], ",", "8", ",", "\"\<Shell\>\""}], 
  "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->703270242],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnb2OJLcRxw/wk/gNBsqdbOJ0IcCBAUUCvLBg4+SDpFtpcdbnCYIdKHJg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    "], {{0, 138}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 138},
  PlotRange->{{0, 180}, {0, 138}}]], "Output",
 ImageSize->{184, 138},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->778435796]
}, Open  ]],

Cell["\<\
The result is easier to understand if a partial revolution is used.\
\>", "ExampleText",
 CellID->213866294],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSumPlot3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Sin", "[", "x", "]"}], ",", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"2", 
        RowBox[{"x", "/", 
         RowBox[{"(", "\[Pi]", ")"}]}]}], ")"}], "2"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", 
     RowBox[{"\[Pi]", "/", "2"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"8", ",", 
     RowBox[{"3", " ", 
      RowBox[{"\[Pi]", "/", "2"}]}]}], "}"}], ",", "\"\<Shell\>\""}], 
  "]"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->1710661827],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnb2OJDUQx0/iEXgCHgA0QqQkm5CukAiQiJBYQKCDEx8Lq+P7EIKAiAAR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    "], {{0, 138}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 138},
  PlotRange->{{0, 180}, {0, 138}}]], "Output",
 ImageSize->{184, 138},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->700045848]
}, Open  ]],

Cell["\<\
The same volume can be approximated using washers, but an additional argument \
specifying the range over which the washers should be generated.\
\>", "ExampleText",
 CellID->2114257139],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSumPlot3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Sin", "[", "x", "]"}], ",", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"2", 
        RowBox[{"x", "/", 
         RowBox[{"(", "\[Pi]", ")"}]}]}], ")"}], "2"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", 
     RowBox[{"\[Pi]", "/", "2"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"z", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"8", ",", 
     RowBox[{"3", " ", 
      RowBox[{"\[Pi]", "/", "2"}]}]}], "}"}], ",", "\"\<Washer\>\""}], 
  "]"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->2055311911],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnc2u5UYRx0fKI/AEPADoCLHN5m6yHUViEYlVJEIi0GSiASa5mpBPFEVC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    "], {{0, 140}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 140},
  PlotRange->{{0, 180}, {0, 140}}]], "Output",
 ImageSize->{184, 140},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->1755149070]
}, Open  ]],

Cell["\<\
Using washers more complicated curves can be used including vertical lines. \
Here is an example. First some function definitions.\
\>", "ExampleText",
 CellID->1540270085],

Cell[BoxData[
 RowBox[{
  RowBox[{"f1", "[", "x_", "]"}], ":=", " ", "x"}]], "Input",
 CellLabel->"In[6]:=",
 CellID->1551437657],

Cell[BoxData[
 RowBox[{
  RowBox[{"f2", "[", "x_", "]"}], ":=", " ", 
  RowBox[{"2", "-", "x"}]}]], "Input",
 CellLabel->"In[7]:=",
 CellID->266259325],

Cell[BoxData[
 RowBox[{
  RowBox[{"f3", "[", "x_", "]"}], ":=", " ", 
  RowBox[{"4", "-", " ", 
   SqrtBox["x"]}]}]], "Input",
 CellLabel->"In[8]:=",
 CellID->1812038574],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"vrtLn", "=", 
  RowBox[{"MakeVerticalSegment", "[", 
   RowBox[{".5", ",", 
    RowBox[{"{", 
     RowBox[{"z", ",", "2", ",", "3"}], "}"}]}], "]"}]}]], "Input",
 CellLabel->"In[9]:=",
 CellID->1466715111],

Cell[BoxData[
 RowBox[{"VerticalSegment", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"{", 
     RowBox[{"2", ",", "3"}], "}"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.5`", ",", 
     RowBox[{"2", "\[LessEqual]", "z", "<", "3"}]}], "}"}]}], "]"}]], "Output",
 ImageSize->{285, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->617356451]
}, Open  ]],

Cell["\<\
This function is not needed for the final plot but is used to help visualize \
the cross-section.\
\>", "ExampleText",
 CellID->600596883],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"invFn1", "=", 
  RowBox[{"MakeRadiusFunction", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"f1", "[", 
       RowBox[{"x", "-", 
        RowBox[{"1", "/", "2"}]}], "]"}], ",", 
      RowBox[{"f2", "[", 
       RowBox[{"x", "-", 
        RowBox[{"1", "/", "2"}]}], "]"}], ",", 
      RowBox[{"f3", "[", 
       RowBox[{"x", "-", 
        RowBox[{"1", "/", "2"}]}], "]"}], ",", "vrtLn"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", 
      RowBox[{"1", "/", "2"}], ",", 
      RowBox[{"3", "/", "2"}]}], "}"}]}], "]"}]}]], "Input",
 CellLabel->"In[10]:=",
 CellID->116086664],

Cell[BoxData[
 RowBox[{
  TagBox[GridBox[{
     {"\[Piecewise]", GridBox[{
        {
         RowBox[{
          TagBox[
           RowBox[{"InterpolatingFunction", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"{", 
               RowBox[{"0", ",", "1"}], "}"}], "}"}], ",", "\<\"<>\"\>"}], 
            "]"}],
           False,
           Editable->False], "[", "#1", "]"}], 
         RowBox[{"0", "\[LessEqual]", "#1", "<", "1"}]},
        {
         RowBox[{
          TagBox[
           RowBox[{"InterpolatingFunction", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"{", 
               RowBox[{"1", ",", "2"}], "}"}], "}"}], ",", "\<\"<>\"\>"}], 
            "]"}],
           False,
           Editable->False], "[", "#1", "]"}], 
         RowBox[{"1", "\[LessEqual]", "#1", "<", "2"}]},
        {
         RowBox[{
          TagBox[
           RowBox[{"InterpolatingFunction", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"{", 
               RowBox[{"3", ",", "4"}], "}"}], "}"}], ",", "\<\"<>\"\>"}], 
            "]"}],
           False,
           Editable->False], "[", "#1", "]"}], 
         RowBox[{"3", "\[LessEqual]", "#1", "<", "4"}]},
        {"0.5`", 
         RowBox[{"2", "\[LessEqual]", "#1", "<", "3"}]},
        {"0", 
         TagBox["True",
          "PiecewiseDefault",
          AutoDelete->True]}
       },
       AllowedDimensions->{2, Automatic},
       Editable->True,
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxItemSize->{
        "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
         "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.84]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.4]}, 
           Offset[0.2]}, "RowsIndexed" -> {}},
       Selectable->True]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxItemSize->{
     "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.35]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}],
   "Piecewise",
   DeleteWithContents->True,
   Editable->False,
   SelectWithContents->True,
   Selectable->False], "&"}]], "Output",
 ImageSize->{366, 73},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[10]=",
 CellID->1333077181]
}, Open  ]],

Cell["Create a polygon to revolve around the vertical axis.", "ExampleText",
 CellID->1058671681],

Cell[BoxData[
 RowBox[{
  RowBox[{"pts", "=", " ", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"invFn1", "[", "x", "]"}], ",", "x"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", ".05", ",", "3.95", ",", ".05"}], "}"}]}], "]"}]}], 
  ";"}]], "Input",
 CellLabel->"In[11]:=",
 CellID->1153410372],

Cell[BoxData[
 RowBox[{
  RowBox[{"pts2", "=", " ", 
   RowBox[{"Reverse", "[", 
    RowBox[{
     RowBox[{"TranslationTransform", "[", 
      RowBox[{"{", 
       RowBox[{"2.5", ",", "0"}], "}"}], "]"}], "[", "pts", "]"}], "]"}]}], 
  ";"}]], "Input",
 CellLabel->"In[12]:=",
 CellID->33240735],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graphics", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"LightGray", ",", 
     RowBox[{"Polygon", "[", 
      RowBox[{"Join", "[", 
       RowBox[{"pts", ",", "pts2"}], "]"}], "]"}]}], "}"}], ",", 
   RowBox[{"Axes", "\[Rule]", " ", "True"}], ",", 
   RowBox[{"AxesOrigin", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0"}], "}"}]}]}], "]"}]], "Input",
 CellLabel->"In[13]:=",
 CellID->331284925],

Cell[BoxData[
 GraphicsBox[
  {GrayLevel[0.85], PolygonBox[CompressedData["
1:eJxdlj1MFEEUx5fj7tzb+8ACCxuMnZXRcIUNmddhC4mdjUagsFBbYmGDra1o
QmNlYmgsxFYLQ7IaEhKNCYUHeGj41ANE5MPb9/7/3exsspm82Znf/ufNe/Pm
4u17o2OFIAhmum/SPp9OnpZ7pu1LN6jPCuy37mOcPKuuqf3v3dh48rTxfd6d
6rPmAn0W0P8TnEW0625C530GbwPjvro72r+F/iVwdtB+Q/8OxlPnL+hZAfe3
+5TT2UF/G/o6rkd5P8DbTXVau4d51LkP3gb6/6B/C7oOUp2D+E6d45hPndPg
U2eM/1PncU5f1k+d45hHnU/Bpc4Y/6VO422Dd4D59O9Bui7j/XVXcus+TP1i
vH/grYF3BF4bvCP8fxXzjt3V3L6cYP0t8E7TfbU2ENrGzWzjZ7b5uSe1jZvZ
1hY8XsHjZbbxelPb9Ge2tUWPV/R4RY9X8nglj1f2eGWPV/Z4ZzwebcZfKA+G
oi8v7u+DF8repZtP3u3tghdK1+j2dMCriA4fYjxVxPKD+RbJ6+HWZP/sNniR
TK2PzA1MbYIXyWO118GryisdzzyqSv48qMnD/tnuiDZ4NbmgwO/g1WRLcSvg
1SWhDbeWwauLxd0yeA25bgPAa8ibgQRI/zXkRoIbof/65Hzy+376r8/0l0Kh
fU6/h8Lxal4OhbxR5YXC/6mMR6FQj36eI69u6z0krw69FeF6df0zFaE/6Hf6
S917NxL6c1mXGwn9rbhbVeF+bKr/qsL9Mn0ZV+UN1IX7rXo+1IXxoPjJhjBe
dD3X+oTxZJyzwnhjm49Hrjez8/FM/5U9TtnjlTxeyeMVPV7R4xU9Xq/H6/V4
BY9X8HiZnT9vyMvs/HlFXuDxgtSvPA9j+J3nJfeF5yn3jeet5VdF8udxJDyv
Y8QFz/Mm4obnPeOK9cB4VcnXi5qwnsSIW9YbxjXrEeOe9epEeczb3TRPWddi
5BXrIfOO9ZJ5yXpqbmP87aTfWY+bmM+6xrxlPef/WdeM0xDeB6if94Um1sf7
BPOW9w36h/cR4zHfWJfo36X0nOR9h3nL+xD3z8YtpuewtQvgMR7m0/jgfYx5
y/vaBOKL97n/Bm2/xg==
    "]]},
  Axes->True,
  AxesOrigin->{0, 0}]], "Output",
 ImageSize->{184, 189},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[13]=",
 CellID->331291916]
}, Open  ]],

Cell["\<\
Finally, the solid using the polygon as a cross-section of the solid. \
\>", "ExampleText",
 CellID->1192927172],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSumPlot3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"f1", "[", 
        RowBox[{"x", "-", 
         RowBox[{"1", "/", "2"}]}], "]"}], ",", 
       RowBox[{"f2", "[", 
        RowBox[{"x", "-", 
         RowBox[{"1", "/", "2"}]}], "]"}], ",", 
       RowBox[{"f3", "[", 
        RowBox[{"x", "-", 
         RowBox[{"1", "/", "2"}]}], "]"}], ",", "vrtLn"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"f1", "[", 
        RowBox[{"x", "-", "3"}], "]"}], ",", 
       RowBox[{"f2", "[", 
        RowBox[{"x", "-", "3"}], "]"}], ",", 
       RowBox[{"f3", "[", 
        RowBox[{"x", "-", "3"}], "]"}], ",", 
       RowBox[{"MakeVerticalSegment", "[", 
        RowBox[{"3.0", ",", 
         RowBox[{"{", 
          RowBox[{"z", ",", "2", ",", "3"}], "}"}]}], "]"}]}], "}"}]}], "}"}],
    ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"1", "/", "2"}], ",", 
     RowBox[{"3", "/", "2"}], ",", "3", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", "0", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"18", ",", 
     RowBox[{"3", " ", 
      RowBox[{"\[Pi]", "/", "2"}]}]}], "}"}], ",", "\"\<Washer\>\""}], 
  "]"}]], "Input",
 CellLabel->"In[14]:=",
 CellID->1628121451],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnLuuJEkRhkfiEXgCHmDQEcJdpx3c0UoYSFgrzUE4u6BldJjRWYa5Ca2D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    "], {{0, 155}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 155},
  PlotRange->{{0, 180}, {0, 155}}]], "Output",
 ImageSize->{184, 155},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[14]=",
 CellID->955044298]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Options",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->2061341341],

Cell[BoxData[
 InterpretationBox[Cell["CapStyle", "ExampleSubsection",
   WholeCellGroupOpener->True],
  $Line = 0; Null]], "ExampleText",
 CellID->1757724783],

Cell["This loads the package.", "ExampleText",
 CellID->92732157],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1623718624],

Cell["\<\
The flat surfaces of shells and washers can be modified using the option \
CapStyle.\
\>", "ExampleText",
 CellID->1764211139],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSumPlot3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Sin", "[", "x", "]"}], ",", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"2", 
        RowBox[{"x", "/", 
         RowBox[{"(", "\[Pi]", ")"}]}]}], ")"}], "2"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", 
     RowBox[{"\[Pi]", "/", "2"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"z", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"8", ",", 
     RowBox[{"3", " ", 
      RowBox[{"\[Pi]", "/", "2"}]}]}], "}"}], ",", "\"\<Washer\>\"", ",", 
   RowBox[{"CapStyle", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"Red", ",", 
      RowBox[{"EdgeForm", "[", 
       RowBox[{"{", 
        RowBox[{"Thin", ",", "Yellow"}], "}"}], "]"}]}], "}"}]}]}], 
  "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->789899044],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnc/u5DgRxyPxJLxBiztnrqO9IXFaiRUr0OyMBnZgNMv+RSskxIkD2gOH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    "], {{0, 140}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 140},
  PlotRange->{{0, 180}, {0, 140}}]], "Output",
 ImageSize->{184, 140},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->1136738808]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Possible Issues",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->1305812373],

Cell["\<\
Large numbers of partitions can take awhile to render and present unclear \
images.\
\>", "ExampleText",
 CellID->1381194325],

Cell["This loads the package.", "ExampleText",
 CellID->1582279852],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1087471763],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSumPlot3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Sin", "[", "x", "]"}], ",", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"2", 
        RowBox[{"x", "/", 
         RowBox[{"(", "\[Pi]", ")"}]}]}], ")"}], "2"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", 
     RowBox[{"\[Pi]", "/", "2"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"z", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"128", ",", 
     RowBox[{"3", " ", 
      RowBox[{"\[Pi]", "/", "2"}]}]}], "}"}], ",", "\"\<Washer\>\""}], 
  "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->2052084771],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztm12OZCcMRlvKSrKVLCFSNjAP/ZrnrCH7YAXZ300lV0IMGGOM+f+OpqWa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    "], {{0, 139}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 139},
  PlotRange->{{0, 180}, {0, 139}}]], "Output",
 ImageSize->{184, 139},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->1154622344]
}, Open  ]],

Cell["\<\
A list of functions that does not pass the horizontal line test cannot be \
used for specifying the radius of washers. This is a list of functions that \
is not allowed on the specified interval.\
\>", "ExampleText",
 CellID->1577166995],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"2", "-", "x"}], ",", 
     SuperscriptBox["x", "2"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"1", "/", "2"}], ",", 
     RowBox[{"3", "/", "2"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->1055030044],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxF1GswFXgYBvCTcCTJCnWSzanISrFJJdWTy9jSJkxSQhxlksZx2ZZ1S6HL
VHRfXUxUK5diRGiSba2QXJJEadu/w3FwnPvlX5H27MzO7Id33i/Pp2fm+bE5
XL/9WgwGY5vm/v2RRKnPYBC05WR9/vqVYvN0sUuGFkHQlB6nforCxlpwSFeH
YPzQmecpXyiGo952GOgTzPK6mPvpM8U++uQCy4TAW7tgtUJFEWp4Yr6jDcHL
xLq4YSHFzvUs2wgfgp4wxVhbL4VH7oYtbfkEHxzD+q+WU1wx/ay/cPsAnFTH
kmJ2UcgWthwekg1AT2r+bBaTwqrVIjXxPA/i+bF/uNeo0eksYF7cOIh54QYp
1aFqZKxucLEkg6gf+pYnma1G2UcjhWPOEIozEnQjalWoCBR4mrnwUeR35v1P
HBXqt3S6R/Xzwd4zL8TLWAXXKdtGz1PDIOljeUGPlehArujSMgFYxPGU00El
VrHPVla0ChDBdba+O18JoxbV5I6EEXw0zzrq36iAeoIrOThvFDpam0+2xivg
pwrez2weRYCO77aUBQqcyO5OiIsbQ91as8amJjmaqgIdfYyFyPvNb7j3ZzmS
8/3jqn4XosG+p8nKUo51LE7g6bBxhCs6/NZ1ysAN4D0bMRJh82vDvNMJMqQ6
jbWbPBHBja9/9TBbhsv5049UBotRbZ+2ybpLisVd5yr7ZkoQ/euawjlJUjzO
tj8fUybBrbUHapptpDBVl2QJraV4M6dEafNAgpPMRG1OqhRS6WzqoiNB+t/t
vd90SMFw3efzPlyMVdf+Gs+0kiH3hYe1ukqE10u+99mbLMOu6ry9rjNE+HR4
ll78CxkMTfgSVuQ43hbfnua1SA7ve7WTUY+F2HinYXtgvByZNUmhW1lC5ERb
jXc1yMFb9HBpUNIYrpu7tRqwFGhtSY4abh+FXY5gMj1SgaIBB90pu1Ek8j1j
a58qYOQUZ5GTNoItWvbrvjNWInqi4PT9fgGYkkDfaRwl5Gyv4C8uAqTXbyq5
UasEIyOttCl7GGk7ygNiZ6igHVDgYyHiY017r+eMMBUu7PeJE23g472pR6Ze
jUrTbwIVnx+CrpWzmStTDeXDs92pskEIzUZG14eoYfvK3u64+yD8Oxcax5Wr
sTPsab95AQ/3Aq6fNPyqRpHPczFvGg/1DS83RftTZCaFpC32H4C/Mnsbs1Cz
g7m8fbJKgoYONwfuXYoUTtJNdgWBQ7HauLeIIv++8TvfMgKDkJC+wlKKEXf3
7Q+KCBqbl4d7VlAkxtxxjs8jWHm9LTGrjuLG8whD1XECIzf9O9O7KUiysObj
boK0BfXHo15TaDdnyG0CNLtVx0Z292h2bLxg+e4dBC2lb1fc7qOIKfrx1iNv
giOmxY/cPlBo9dw/k+ROIB75ofPoKMWSFTFhk3YaB/6cqBgd07jwi94NO1uC
1rzyS77jFIca898ELSUo9JsbyJZQVAV2bX3CJgiuGx56qqTwPLFydZqZJn/l
WrONWpN/1RpTNodgbax3yTlKcc4ivPSDEYGJdTU3VONIX+VFS8wkOMY46Ncy
QTE5tWwPV49A+s7CyUHjjqVX4+WbGodCHnbNzdW45HE56GWn1v9uHfjPsX8A
n0h/Pg==
     "]]}, 
   {Hue[0.9060679774997897, 0.6, 0.6], LineBox[CompressedData["
1:eJwVjXs01HkAxedn8kySg1JKdpuOsoVkQ+X+UinZSiLbFCaqLdoGbYuKKHm0
eaV1ZGq3E2tDKbGrh+RMD47nyFvUkIwxGMPMfPPIrP3jnvvH/dx7zf25HsfU
GAzG7ln97yeFch0GQ4jOCrERgyHATubIxstqQrid+NiloS6AxUrRKQ11Idiy
5p/1dAToD+qo19WZze+9HDYzFOAoeXHdxFCIuLz0Yz9YCMDRi19sayHEvgk7
uxp3AQ5sMll93F0I+/oAT85dAbZlbnatvSOE3+6j1wd2NCLDaFLHbG8P1PVM
HfOevoPMrOpsn6wHdakBEXuWN4NVvTQyPK0XnksGSyKutaDBQaSZ7vQJCVcT
JWmsNlz+nr9xufATvLilhqmP2lH4RX/cNqUP2mJJKjOoE0VskYvxxs+I3ZRY
KLfsQrlrw9ag95+xKsHq9rP33dgys/q1S2I/PG4mnzAt/4h6ZA7fsBQhISYq
uOetEOvNk4qLqkUovRx8MOBRD/SrFNOeYQMYqwy6UFbSC+UUVxq4SIzRc5JS
h8JP8FD4HNOsFCNu+5HQxld9iE9uCgsNHUSYmmh/+uzv2xK2rbuBBDbtBY2y
/n6cv+MVWvJSgra+GeX0/AE4mvizfzsyBIMIX7tuSzG43r1vBvSHMcie11p4
fBCRdoN1hi+GYXpcY9wgW4Lf7zAvFvuMYKn4p2G+YAjfNqYWt8+VojX2T6cv
y0bwPNkqLbhQiqw5TGaRjxRGyvwrkpWjMH7FK7XbM4oEzfA5/pGj2H6wPiNr
ZhTRH+vaFtSPwiprxfOYXBnWZ3UPxbJkSH92OtSAM4bmFTbufudleGy5oLXH
ZBwTZ+dpnamRQUu+NYhbPY6OvGxq1zdj6ItNUnGT5HDK4e9lnxkDx7mwrdpF
gZTTrKFG/hgCp0qib6kU4C1xrtad3Ylz9cv8q0aJ71JE09Enx3HA5oH2riSC
8M8uIU8qxkFtUPhV+n+Bq5qV4yoDOaLz7xaeXzMBTSl7H+Uvh9vcBN4hMoHo
cjr/1hM5OPcl/I6WSUR5PvQO0VYgrKZxUcO9KWyoa3PRPqKAXfOvnXPSptFl
tC1Wq1QBvzattTnhX6HBcjDeoqlE9GGTHcr9M5AYD4g3+SpRltSb4rFdBa8G
M4PQh0qsybLgzxgy6PvevAQ9lRLmvpGSnDgGXc4X0Ke9CBa7cVIfMynaS568
WzOXQLdlq0TzHEXz652tuX8T3DYJFC25QNHWeUqDtnsETHsrpnUURev6+rbn
FhDsVMu4+eMlin5duSbApYigtLZlS8FVil7Hqw2/UkawwmFZ9AEeRes76+Qw
mwhcg98Vvy6j6CjT8rigZoLN0qbsrnKKHlKGnGxqIVDf+UuxvIKiqwo61ma3
E8yLT7NhvaHoi0Z5T50/EKgcnfLi6yh6ZGBHQ4yYIOIcJ87jA0UffjVVJB4k
+CNjb/ApIUVX3354Y98QQca17oi4XorO9VjINpfO9tVbJc/6KdqnrL+vQk5Q
stbBjSWd5TOyKi2UBMEmZutoGUXbh+zJTyUE1Ids+0PjFG248l8uZ5KAJ0/M
SSMUfYkR6FE1RZDjItF+MEHRo51L7ay/EsSESpKrpija95/GhZkzBNbhV237
vlJ0bcqVSZWKoNi7iqhUFP0fH4Vvug==
     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesOrigin->{0.5, 0.30000000000000004`},
  PlotRange->NCache[{{
      Rational[1, 2], 
      Rational[3, 2]}, {0.2500000204081637, 2.249999938775511}}, {{0.5, 
    1.5}, {0.2500000204081637, 2.249999938775511}}],
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 ImageSize->{184, 114},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->1855001756]
}, Open  ]],

Cell["\<\
The same set of functions is acceptable on a different interval.\
\>", "ExampleText",
 CellID->885509210],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"2", "-", "x"}], ",", 
     SuperscriptBox["x", "2"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->1994409593],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnS+XqzoUxbvW+yTvY4ytHFlZeSsrnkA8UYtEIrFI3IvERiIjsUhkZN9O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    "], {{0, 119}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 119},
  PlotRange->{{0, 180}, {0, 119}}]], "Output",
 ImageSize->{184, 119},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[4]=",
 CellID->188979205]
}, Open  ]],

Cell["This fails with a warning the intervals overlap.", "ExampleText",
 CellID->1140254108],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSumPlot3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"2", "-", "x"}], ",", 
       SuperscriptBox["x", "2"]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"MakeVerticalSegment", "[", 
       RowBox[{"4", ",", 
        RowBox[{"{", 
         RowBox[{"z", ",", "0", ",", "2"}], "}"}]}], "]"}], "}"}]}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"1", "/", "2"}], ",", 
     RowBox[{"3", "/", "2"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"z", ",", "0", ",", "2"}], "}"}], ",", "8", ",", 
   "\"\<Washer\>\""}], "]"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->106727328],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"MakeRadiusFunction", "::", "ovrlp"}], "MessageName"], ":", 
  " ", "\<\"Two or more of the inverse function support intervals for the \
\[NoBreak]\\!\\(\\\"inner radius\\\"\\)\[NoBreak] are not disjoint. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:Classroom \
Utilities/ref/MakeRadiusFunction\\\", ButtonNote -> \
\\\"ClassroomUtilities`MakeRadiusFunction::ovrlp\\\"]\\)\"\>"}]], "Message", \
"MSG",
 CellLabel->"During evaluation of In[5]:=",
 CellID->1416373300],

Cell[BoxData[
 RowBox[{"RiemannSumPlot3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"2", "-", "x"}], ",", 
       SuperscriptBox["x", "2"]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"VerticalSegment", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{"0", ",", "2"}], "}"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"4", ",", 
          RowBox[{"0", "\[LessEqual]", "z", "<", "2"}]}], "}"}]}], "]"}], 
      "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     FractionBox["1", "2"], ",", 
     FractionBox["3", "2"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"z", ",", "0", ",", "2"}], "}"}], ",", "8", 
   ",", "\<\"Washer\"\>"}], "]"}]], "Output",
 ImageSize->{501, 53},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->129826105]
}, Open  ]],

Cell["\<\
The radii of the washers are calculated using an inverse function defined \
using Piecewise. If one of the functions does not have values in the \
designated region the value will be assumed to be zero. In the following 1 + \
x does not take values in the rectangle given by 0<x<\[Pi] and 0 <z <1 and is \
ignored when creating the washers.\
\>", "ExampleText",
 CellID->143961383],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Sin", "[", "x", "]"}], ",", 
     RowBox[{"1", "+", " ", "x"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "\[Pi]"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[6]:=",
 CellID->1492377559],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnS2Xs7wWhlnr/JJXjuxfQFZGVj5IxBGod2EjkUhsZNyJxEZGRmKRyMie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    "], {{0, 121}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 121},
  PlotRange->{{0, 180}, {0, 121}}]], "Output",
 ImageSize->{184, 121},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[6]=",
 CellID->397536586]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSumPlot3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Sin", "[", "x", "]"}], ",", 
     RowBox[{"1", "+", " ", "x"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", 
     RowBox[{"\[Pi]", "/", "2"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"6", ",", 
     RowBox[{"3", " ", 
      RowBox[{"\[Pi]", "/", "2"}]}]}], "}"}], ",", "\"\<Washer\>\"", ",", 
   RowBox[{"Axes", "\[Rule]", " ", "True"}]}], "]"}]], "Input",
 CellLabel->"In[7]:=",
 CellID->654622185],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnb+LFEkUxxcuM5P7CxY0MLhjkEvEbP+C8eCCMzRwvWR3ddXFZfW4A4OV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    "], {{0, 140}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 140},
  PlotRange->{{0, 180}, {0, 140}}]], "Output",
 ImageSize->{184, 140},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->580761213]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 WholeCellGroupOpener->True,
 CellID->1255426704],

Cell[TextData[StyleBox[ButtonBox["MakeVerticalSegment",
 BaseStyle->"Link",
 ButtonData->"paclet:Classroom Utilities/ref/MakeVerticalSegment"],
 FontFamily->"Verdana"]], "RelatedLinks",
 CellID->1588619492],

Cell[TextData[StyleBox[ButtonBox["VerticalSegment",
 BaseStyle->"Link",
 ButtonData->"paclet:Classroom Utilities/ref/VerticalSegment"],
 FontFamily->"Verdana"]], "RelatedLinks",
 CellID->263618893]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 WholeCellGroupOpener->True,
 CellID->250839057],

Cell[TextData[ButtonBox["Working with Riemann Sums",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Classroom Utilities/tutorial/WorkingWithRiemannSums"]], "Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"RiemannSumPlot3D - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "SymbolColor", "LinkTrails" -> 
  GridBox[{{
      RowBox[{
        StyleBox[
         ButtonBox[
         "Mathematica", ButtonData -> "paclet:guide/Mathematica", 
          BaseStyle -> {"Link", "DockedLinkTrail"}], FontSlant -> "Italic"], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 30, 17.5156624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Symbol", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "RiemannSumPlot3D[f, {x, x_min, x_max}, n, \"Disk\"] creates a stack of n \
disks with radius determined by the value of x at the point f(x) to \
approximate the volume obtained by revolving f around the vertical \
axis.RiemannSumPlot3D[{f_1, f_2}, {x, x_min, x_max}, n, \"Shell\"] creates n \
shells of height f_2 (x) - f_1 (x) with radii taken from the interval (x_min, \
x_max).RiemannSumPlot3D[{f_1, f_2}, {x, x_min, x_max}, {z, z_min, z_max}, n, \
\"Washer\"] creates a stack of n washers with radii x_i and x_j where f_1 \
(x_i) = f_2 (x_j) with thickness ((z_max - z_min)/n.RiemannSumPlot3D[{{f_i, \
...}, {g_j, ...}}, {x, x_min, x_m1, x_m2, x_max}, {z, z_min, z_max}, n, \
\"Washer\"] evaluates the functions f_i and g_i on the intervals (x_min, \
x_mid1) and creates n washers of thickness (z_max - z_min)/n with inner \
radius determined by f and the outer radius by g.", "synonyms" -> {}, "title" -> 
    "RiemannSumPlot3D", "titlemodifier" -> "", "windowtitle" -> 
    "RiemannSumPlot3D", "type" -> "Symbol", "uri" -> 
    "Classroom Utilities/ref/RiemannSumPlot3D"}, "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[13160, 430, 490, 14, 45, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->620116192]},
 "RasterizedOutput"->{
  Cell[68169, 1743, 3377, 63, 70, "Output",
   CellTags->"RasterizedOutput",
   CellID->188979205],
  Cell[74596, 1909, 4341, 79, 70, "Output",
   CellTags->"RasterizedOutput",
   CellID->397536586]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 88295, 2208},
 {"RasterizedOutput", 88439, 2212}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[609, 21, 2489, 57, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3123, 82, 59, 1, 51, "ObjectName",
 CellID->1224892054],
Cell[3185, 85, 7477, 243, 322, "Usage",
 CellID->982511436]
}, Open  ]],
Cell[CellGroupData[{
Cell[10699, 333, 608, 18, 39, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->86187868],
Cell[11310, 353, 346, 12, 70, "Notes",
 CellID->1067943069],
Cell[11659, 367, 293, 11, 70, "Notes",
 CellID->2000816085],
Cell[11955, 380, 132, 4, 70, "Notes",
 CellID->1105379435],
Cell[12090, 386, 86, 1, 70, "Notes",
 CellID->269512222],
Cell[12179, 389, 84, 1, 70, "Notes",
 CellID->648903631],
Cell[12266, 392, 95, 1, 70, "Notes",
 CellID->1201366751],
Cell[12364, 395, 193, 7, 70, "Notes",
 CellID->729734963],
Cell[12560, 404, 142, 4, 70, "Notes",
 CellID->376415124],
Cell[12705, 410, 281, 9, 70, "Notes",
 CellID->2069648653],
Cell[12989, 421, 134, 4, 70, "Notes",
 CellID->258152475]
}, Closed]],
Cell[CellGroupData[{
Cell[13160, 430, 490, 14, 45, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->620116192],
Cell[13653, 446, 195, 6, 31, "ExampleSection",
 CellID->248693115],
Cell[CellGroupData[{
Cell[13873, 456, 269, 8, 17, "ExampleSection",
 CellID->1962642188],
Cell[14145, 466, 66, 1, 32, "ExampleText",
 CellID->602544230],
Cell[14214, 469, 128, 3, 70, "Input",
 CellID->1575478005],
Cell[14345, 474, 187, 4, 70, "ExampleText",
 CellID->585462525],
Cell[CellGroupData[{
Cell[14557, 482, 254, 8, 70, "Input",
 CellID->98587121],
Cell[14814, 492, 5638, 96, 177, 1669, 27, "CachedBoxData", "BoxData", "Output",
 CellID->1243850810]
}, Open  ]],
Cell[20467, 591, 171, 4, 70, "ExampleText",
 CellID->1794743772],
Cell[CellGroupData[{
Cell[20663, 599, 481, 16, 70, "Input",
 CellID->703270242],
Cell[21147, 617, 5034, 90, 159, "Output",
 CellID->778435796]
}, Open  ]],
Cell[26196, 710, 118, 3, 70, "ExampleText",
 CellID->213866294],
Cell[CellGroupData[{
Cell[26339, 717, 588, 20, 70, "Input",
 CellID->1710661827],
Cell[26930, 739, 4924, 88, 159, "Output",
 CellID->700045848]
}, Open  ]],
Cell[31869, 830, 196, 4, 70, "ExampleText",
 CellID->2114257139],
Cell[CellGroupData[{
Cell[32090, 838, 658, 22, 70, "Input",
 CellID->2055311911],
Cell[32751, 862, 4901, 88, 161, "Output",
 CellID->1755149070]
}, Open  ]],
Cell[37667, 953, 182, 4, 70, "ExampleText",
 CellID->1540270085],
Cell[37852, 959, 129, 4, 70, "Input",
 CellID->1551437657],
Cell[37984, 965, 151, 5, 70, "Input",
 CellID->266259325],
Cell[38138, 972, 170, 6, 70, "Input",
 CellID->1812038574],
Cell[CellGroupData[{
Cell[38333, 982, 229, 7, 70, "Input",
 CellID->1466715111],
Cell[38565, 991, 397, 13, 36, "Output",
 CellID->617356451]
}, Open  ]],
Cell[38977, 1007, 148, 4, 70, "ExampleText",
 CellID->600596883],
Cell[CellGroupData[{
Cell[39150, 1015, 627, 20, 70, "Input",
 CellID->116086664],
Cell[39780, 1037, 2881, 86, 94, "Output",
 CellID->1333077181]
}, Open  ]],
Cell[42676, 1126, 97, 1, 70, "ExampleText",
 CellID->1058671681],
Cell[42776, 1129, 351, 12, 70, "Input",
 CellID->1153410372],
Cell[43130, 1143, 295, 10, 70, "Input",
 CellID->33240735],
Cell[CellGroupData[{
Cell[43450, 1157, 436, 13, 70, "Input",
 CellID->331284925],
Cell[43889, 1172, 1387, 29, 210, "Output",
 CellID->331291916]
}, Open  ]],
Cell[45291, 1204, 122, 3, 70, "ExampleText",
 CellID->1192927172],
Cell[CellGroupData[{
Cell[45438, 1211, 1304, 41, 70, "Input",
 CellID->1628121451],
Cell[46745, 1254, 5531, 98, 176, "Output",
 CellID->955044298]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[52325, 1358, 270, 8, 70, "ExampleSection",
 CellID->2061341341],
Cell[52598, 1368, 159, 4, 70, "ExampleText",
 CellID->1757724783],
Cell[52760, 1374, 65, 1, 70, "ExampleText",
 CellID->92732157],
Cell[52828, 1377, 128, 3, 70, "Input",
 CellID->1623718624],
Cell[52959, 1382, 136, 4, 70, "ExampleText",
 CellID->1764211139],
Cell[CellGroupData[{
Cell[53120, 1390, 866, 28, 70, "Input",
 CellID->789899044],
Cell[53989, 1420, 4836, 87, 161, "Output",
 CellID->1136738808]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[58874, 1513, 278, 8, 70, "ExampleSection",
 CellID->1305812373],
Cell[59155, 1523, 135, 4, 70, "ExampleText",
 CellID->1381194325],
Cell[59293, 1529, 67, 1, 70, "ExampleText",
 CellID->1582279852],
Cell[59363, 1532, 128, 3, 70, "Input",
 CellID->1087471763],
Cell[CellGroupData[{
Cell[59516, 1539, 660, 22, 70, "Input",
 CellID->2052084771],
Cell[60179, 1563, 2884, 55, 160, "Output",
 CellID->1154622344]
}, Open  ]],
Cell[63078, 1621, 247, 5, 70, "ExampleText",
 CellID->1577166995],
Cell[CellGroupData[{
Cell[63350, 1630, 337, 12, 70, "Input",
 CellID->1055030044],
Cell[63690, 1644, 4036, 77, 135, "Output",
 CellID->1855001756]
}, Open  ]],
Cell[67741, 1724, 115, 3, 70, "ExampleText",
 CellID->885509210],
Cell[CellGroupData[{
Cell[67881, 1731, 285, 10, 70, "Input",
 CellID->1994409593],
Cell[68169, 1743, 3377, 63, 70, "Output",
 CellTags->"RasterizedOutput",
 CellID->188979205]
}, Open  ]],
Cell[71561, 1809, 92, 1, 70, "ExampleText",
 CellID->1140254108],
Cell[CellGroupData[{
Cell[71678, 1814, 688, 23, 70, "Input",
 CellID->106727328],
Cell[72369, 1839, 586, 12, 70, "Message",
 CellID->1416373300],
Cell[72958, 1853, 904, 30, 74, "Output",
 CellID->129826105]
}, Open  ]],
Cell[73877, 1886, 391, 7, 70, "ExampleText",
 CellID->143961383],
Cell[CellGroupData[{
Cell[74293, 1897, 300, 10, 70, "Input",
 CellID->1492377559],
Cell[74596, 1909, 4341, 79, 70, "Output",
 CellTags->"RasterizedOutput",
 CellID->397536586]
}, Open  ]],
Cell[CellGroupData[{
Cell[78974, 1993, 596, 18, 70, "Input",
 CellID->654622185],
Cell[79573, 2013, 4391, 79, 161, "Output",
 CellID->580761213]
}, Open  ]]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[84025, 2099, 446, 13, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[84474, 2114, 206, 4, 70, "RelatedLinks",
 CellID->1588619492],
Cell[84683, 2120, 197, 4, 70, "RelatedLinks",
 CellID->263618893]
}, Open  ]],
Cell[CellGroupData[{
Cell[84917, 2129, 448, 13, 70, "TutorialsSection",
 CellID->250839057],
Cell[85368, 2144, 185, 4, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[85568, 2151, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

