(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     45978,       1282]
NotebookOptionsPosition[     41044,       1127]
NotebookOutlinePosition[     43584,       1194]
CellTagsIndexPosition[     43462,       1188]
WindowTitle->TrapezoidalDistribution - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Visualizing Statistical Concepts\"\>":>
       Documentation`HelpLookup[
       "paclet:Classroom Utilities/tutorial/VisualizingStatisticalConcepts"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"TutorialsSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"TriangularDistribution\"\>":>
       Documentation`HelpLookup["paclet:ref/TriangularDistribution"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"SeeAlsoSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Classroom Utilities/ref/TrapezoidalDistribution\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Classroom Utilities/ref/TrapezoidalDistribution"]}, 
            Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Classroom \
Utilities/ref/TrapezoidalDistribution.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Classroom Utilities/ref/TrapezoidalDistribution", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["TrapezoidalDistribution", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"TrapezoidalDistribution", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          StyleBox["min", "TI"], ",", 
          StyleBox["max", "TI"]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          SubscriptBox[
           StyleBox["mode", "TI"], 
           StyleBox["min", "TI"]], ",", 
          SubscriptBox[
           StyleBox["mode", "TI"], 
           StyleBox["max", "TI"]]}], "}"}]}], "]"}]], "InlineFormula"],
     " \[LineSeparator]represents a trapezoidal distribution taking values on \
the interval ",
     Cell[BoxData[
      RowBox[{"(", 
       RowBox[{
        StyleBox["min", "TI"], ",", " ", 
        StyleBox["max", "TI"]}]}]], "InlineFormula"],
     ") achieving its maximum on the interval (",
     Cell[BoxData[
      RowBox[{
       SubscriptBox[
        StyleBox["mode", "TI"], 
        StyleBox["min", "TI"]], ",", " ", 
       SubscriptBox[
        StyleBox["mode", "TI"], 
        StyleBox["max", "TI"]]}]], "InlineFormula"],
     " )"
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}},
 CellID->982511436]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 WholeCellGroupOpener->True,
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->275260772],

Cell[TextData[{
 "TrapezoidalDistribution allows min, max, mode$min and mode$max to be any \
real numbers such that ",
 Cell[BoxData[
  RowBox[{
   StyleBox["min", "TI"], " ", "\[LessEqual]", " ", 
   SubscriptBox[
    StyleBox["mode", "TI"], 
    StyleBox["min", "TI"]], " ", "<", " ", 
   SubscriptBox[
    StyleBox["mode", "TI"], 
    StyleBox["max", "TI"]], " ", "\[LessEqual]", " ", 
   StyleBox["max", "TI"]}]], "InlineFormula"]
}], "Notes",
 CellID->1067943069],

Cell[TextData[{
 "TrapezoidalDistribution can be used with such functions a ",
 ButtonBox["Mean",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Mean"],
 ", ",
 ButtonBox["CDF",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/CDF"],
 " and ",
 ButtonBox["RandomVariate",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomVariate"],
 "."
}], "Notes",
 CellID->2093070604]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 WholeCellGroupOpener->True,
 CellTags->"PrimaryExamplesSection",
 CellID->76199832],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(3)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->355917],

Cell["This loads the package.", "ExampleText",
 CellID->308019921],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1455222977],

Cell["Probability density function.", "ExampleText",
 CellID->241318756],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"PDF", "[", 
      RowBox[{
       RowBox[{"TrapezoidalDistribution", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"0", ",", "6"}], "}"}], ",", "m"}], "]"}], ",", "x"}], 
      "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"m", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"1", ",", "5"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"1", ",", "4"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "3"}], "}"}]}], "}"}]}], "}"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "1"}], ",", "7"}], "}"}], ",", 
   RowBox[{"Exclusions", "\[Rule]", " ", "None"}], ",", 
   RowBox[{"Filling", "\[Rule]", " ", "Axis"}], ",", 
   RowBox[{"Evaluated", "\[Rule]", " ", "True"}]}], "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->1395206556],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztXS23q7wS3mvdX3J/xrFbIpHIF4m4AsmVyMjI2Mi4GxkbGRkZi0RW9k4S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    "], {{0, 124}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 124},
  PlotRange->{{0, 180}, {0, 124}}]], "Output",
 ImageSize->{184, 124},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[2]=",
 CellID->229758313]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell["\t", "ExampleDelimiter"],
  $Line = 0; Null]], "ExampleDelimiter",
 CellID->1138557463],

Cell["Cumulative distribution function.", "ExampleText",
 CellID->89381188],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"CDF", "[", 
      RowBox[{
       RowBox[{"TrapezoidalDistribution", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"0", ",", "6"}], "}"}], ",", "m"}], "]"}], ",", "x"}], 
      "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"m", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"1", ",", "5"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"1", ",", "4"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "3"}], "}"}]}], "}"}]}], "}"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "1"}], ",", "7"}], "}"}], ",", 
   RowBox[{"Exclusions", "\[Rule]", " ", "None"}], ",", 
   RowBox[{"Filling", "\[Rule]", " ", "Axis"}], ",", 
   RowBox[{"Evaluated", "\[Rule]", " ", "True"}]}], "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1655985871],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztXSGT4zwSnar7Jfczli5cGBj4GRocMDQ5ICgoKCoodg2bCgoKihoaGuZa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    "], {{0, 128}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 128},
  PlotRange->{{0, 180}, {0, 128}}]], "Output",
 ImageSize->{184, 128},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[1]=",
 CellID->10138667]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell["\t", "ExampleDelimiter"],
  $Line = 0; Null]], "ExampleDelimiter",
 CellID->1433820333],

Cell["Mean and variance.", "ExampleText",
 CellID->1304333744],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Mean", "[", 
    RowBox[{"TrapezoidalDistribution", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "5"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"2", ",", "3"}], "}"}]}], "]"}], "]"}], ",", 
   RowBox[{"Variance", "[", 
    RowBox[{"TrapezoidalDistribution", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "5"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"2", ",", "3"}], "}"}]}], "]"}], "]"}]}], "}"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->424166832],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["14", "5"], ",", 
   FractionBox["109", "150"]}], "}"}]], "Output",
 ImageSize->{73, 32},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->1900467166]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Scope",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->1293636265],

Cell[BoxData[
 RowBox[{
  RowBox[{"data", "=", 
   RowBox[{"RandomVariate", "[", 
    RowBox[{
     RowBox[{"TrapezoidalDistribution", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "4"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3"}], "}"}]}], "]"}], ",", 
     RowBox[{"10", "^", "4"}]}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->2126097146],

Cell["Compare its histogram to the PDF: ", "ExampleText",
 CellID->1334330389],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"Histogram", "[", 
    RowBox[{"data", ",", "20", ",", "\"\<PDF\>\""}], "]"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"Plot", "[", 
    RowBox[{
     RowBox[{"PDF", "[", 
      RowBox[{
       RowBox[{"TrapezoidalDistribution", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "1"}], ",", "4"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "3"}], "}"}]}], "]"}], ",", "x"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"-", "1"}], ",", "4"}], "}"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", "Thick"}]}], "]"}]}], "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->1621417849],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0.798413061722744, 0.824719615472648, 0.968322270542458], 
    EdgeForm[Opacity[0.7]], {}, 
    {RGBColor[0.798413061722744, 0.824719615472648, 0.968322270542458], 
     EdgeForm[Opacity[0.7]], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{-1., 0.}, {-0.75, 0.0136}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{20.500000000000007`, 29.233877551020413`}, {
          34.71506313757733, 39.1875668236084}}],
        StatusArea[#, 0.0136]& ,
        TagBoxNote->"0.0136"],
       StyleBox["0.0136`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.0136, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{-0.75, 0.}, {-0.5, 0.0392}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{28.233877551020413`, 36.967755102040826`}, {
          28.178585610930615`, 39.1875668236084}}],
        StatusArea[#, 0.0392]& ,
        TagBoxNote->"0.0392"],
       StyleBox["0.0392`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.0392, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{-0.5, 0.}, {-0.25, 0.0764}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{35.967755102040826`, 44.70163265306123}, {
          18.680266705022106`, 39.1875668236084}}],
        StatusArea[#, 0.0764]& ,
        TagBoxNote->"0.0764"],
       StyleBox["0.0764`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.0764, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{-0.25, 0.}, {0., 0.0924}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{43.70163265306123, 52.43551020408164}, {
          14.594968250867907`, 39.1875668236084}}],
        StatusArea[#, 0.0924]& ,
        TagBoxNote->"0.0924"],
       StyleBox["0.0924`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.0924, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{0., 0.}, {0.25, 0.1352}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{51.43551020408164, 60.16938775510204}, {
          3.666794886005434, 39.1875668236084}}],
        StatusArea[#, 0.1352]& ,
        TagBoxNote->"0.1352"],
       StyleBox["0.1352`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.1352, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{0.25, 0.}, {0.5, 0.1536}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{59.16938775510204, 
          67.90326530612245}, {-1.0312983362718953`, 39.1875668236084}}],
        StatusArea[#, 0.1536]& ,
        TagBoxNote->"0.1536"],
       StyleBox["0.1536`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.1536, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{0.5, 0.}, {0.75, 0.1584}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{66.90326530612245, 
          75.63714285714286}, {-2.2568878725181634`, 39.1875668236084}}],
        StatusArea[#, 0.1584]& ,
        TagBoxNote->"0.1584"],
       StyleBox["0.1584`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.1584, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{0.75, 0.}, {1., 0.2088}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{74.63714285714286, 
          83.37102040816328}, {-15.125578003103882`, 39.1875668236084}}],
        StatusArea[#, 0.2088]& ,
        TagBoxNote->"0.2088"],
       StyleBox["0.2088`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.2088, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{1., 0.}, {1.25, 0.2216}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{82.37102040816328, 
          91.10489795918367}, {-18.393816766427236`, 39.1875668236084}}],
        StatusArea[#, 0.2216]& ,
        TagBoxNote->"0.2216"],
       StyleBox["0.2216`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.2216, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{1.25, 0.}, {1.5, 0.266}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{90.10489795918367, 
          98.83877551020409}, {-29.730519976705146`, 39.1875668236084}}],
        StatusArea[#, 0.266]& ,
        TagBoxNote->"0.266"],
       StyleBox["0.266`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.266, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{1.5, 0.}, {1.75, 0.2828}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{97.83877551020409, 
          106.57265306122449`}, {-34.02008335356704, 39.1875668236084}}],
        StatusArea[#, 0.2828]& ,
        TagBoxNote->"0.2828"],
       StyleBox["0.2828`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.2828, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{1.75, 0.}, {2., 0.3136}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{105.57265306122449`, 
          114.3065306122449}, {-41.88428287781388, 39.1875668236084}}],
        StatusArea[#, 0.3136]& ,
        TagBoxNote->"0.3136"],
       StyleBox["0.3136`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.3136, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{2., 0.}, {2.25, 0.3412}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{113.3065306122449, 
          122.04040816326531`}, {-48.931422711229864`, 39.1875668236084}}],
        StatusArea[#, 0.3412]& ,
        TagBoxNote->"0.3412"],
       StyleBox["0.3412`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.3412, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{2.25, 0.}, {2.5, 0.3364}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{121.04040816326531`, 
          129.77428571428572`}, {-47.7058331749836, 39.1875668236084}}],
        StatusArea[#, 0.3364]& ,
        TagBoxNote->"0.3364"],
       StyleBox["0.3364`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.3364, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{2.5, 0.}, {2.75, 0.3396}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{128.77428571428572`, 
          137.5081632653061}, {-48.52289286581445, 39.1875668236084}}],
        StatusArea[#, 0.3396]& ,
        TagBoxNote->"0.3396"],
       StyleBox["0.3396`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.3396, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{2.75, 0.}, {3., 0.3432}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{136.5081632653061, 
          145.24204081632655`}, {-49.44208501799914, 39.1875668236084}}],
        StatusArea[#, 0.3432]& ,
        TagBoxNote->"0.3432"],
       StyleBox["0.3432`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.3432, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{3., 0.}, {3.25, 0.2912}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{144.24204081632655`, 
          152.97591836734694`}, {-36.164865041998006`, 39.1875668236084}}],
        StatusArea[#, 0.2912]& ,
        TagBoxNote->"0.2912"],
       StyleBox["0.2912`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.2912, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{3.25, 0.}, {3.5, 0.2176}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{151.97591836734694`, 
          160.70979591836735`}, {-17.37249215288869, 39.1875668236084}}],
        StatusArea[#, 0.2176]& ,
        TagBoxNote->"0.2176"],
       StyleBox["0.2176`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.2176, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{3.5, 0.}, {3.75, 0.13}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{159.70979591836735`, 168.44367346938776`}, {
          4.994516883605542, 39.1875668236084}}],
        StatusArea[#, 0.13]& ,
        TagBoxNote->"0.13"],
       StyleBox["0.13`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.13, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{3.75, 0.}, {4., 0.0392}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{167.44367346938776`, 176.17755102040815`}, {
          28.178585610930615`, 39.1875668236084}}],
        StatusArea[#, 0.0392]& ,
        TagBoxNote->"0.0392"],
       StyleBox["0.0392`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.0392, {
         GrayLevel[0]}], "Tooltip"]& ]}, {}, {}}, {{{}, {}, 
     {Hue[0.67, 0.6, 0.6], Thickness[Large], LineBox[CompressedData["
1:eJwVin041AcAx4+nNC01zUtpGJmXeTy1LQ4d313nJWQT7hZqPGiMHBlWlDFL
j9ps10Pe5Rqr0LqGXOz8vOThvNPvx1hR3tIlZwxXufvN/vg+n+fzfL4mobG+
J9UZDIbzxv7ncglTStNyInvK3nRHwGHnq/IAMnZZTnRo8xauc81hzz43MTEr
Jxz6mKx4rh1Sp5tXiN6Nf5XlVADXDVutPU3Tija6gfCeN5cHU/HxFAZzox89
UebO/Qo+Q+k2avwFghtzMNeFm4RqjR6B+qN5wiGlcgeHmwndHENpeMJzghOW
vOueax68Q/RrstxmiR7/HeN1HkJ4akVnV1hOESFz/er7yN/wUrImYmdPEBnV
x7Umh6sxM/51pcR5lEhNqx2Id7gLlgcvXRExRKi1lf0k4NTCUTafeCi0g2hw
2pTwHerxTXJxhFN9HVGaZnFN960GsObGCx17ctH5935J5zYJ9BVSY9f+JhjR
r4zNxgko8zS2TFLd4NoY8m74tCBvoODp0VsknCNEul4DrRCMBAfa/DsG1oVf
1w7ZPcDVLuq12dQTBN/R9PNoaEdpp8kXLNY0ihKtOUxmB0YutjMEQc+wYHyb
e7awEyGm1Qczm2QIGxY+le7pgkGqrE387CXkcas/8zO6cWWsbsG1chGl+k31
Jpq9qL0gVa89toSIQHbCnxf7wNfOknwqW4Zruplf1Xw/BrbKmktKVrA3UMIl
6wfQPb0atfvAGkQnd717/tQgwvt4xQWtCnhgsLDAcAjlHxzTk8W/xnIpJ1jy
eAjnT2xmBmis49bizIG3BQ9Bq/uX5TcqcbqT7R7iR4LM0hN5tyqheac2KohH
4uY7o4S6VImyXItsXgAJH6MvJ04NK9EXqkUdCSZx3SHSiP2PEtbKv0Idokm4
xZ0rfm6uwuxHcak7M0hkPy7Pd7yiQlBRad2DP0iEhkfclOersJSuPUrUkWC+
sBKXX1PhUuQP6w1iEk8Uv49sv63Cfdsozt0mEh/r3Neb6lBBb8B2sKSLxLBn
b85lpQqDm3rnkyZJGIlXfnkUScON/2av1U4KiQu+e4xiaVTo1MgXdSj0mIkq
ghNpbG6MahTrUzgriGqYTKfRtmXM97AhBTJqYmqukAZbKE6PsKRgLWTxPxTS
ELrHee+zpvD9SIEi+gYNtQWL3Ws2FPa7+GvJa2g0O+aJMj+hcNlQarfSQ8OZ
TJyocKYw6WfeYveQRkmyTVUMm4L9pQyvM6M01t+fSbJ1oTC76hTyZoZGY4z/
9nYPCk42RTLWPA0DnW1jPx6hkBOmSEhdopHc0Fbh/zmFFwVcmlDQGAtJOf2e
L4XPWrqtzryikcP41ou/4f8B1zME1g==
       "]], LineBox[CompressedData["
1:eJxTTMoPSmViYGCQBGIQHSkv9dqZmcEhFAyu2lfMM/M/xIrgL7ZMFXnMgeCf
uzL5JjMPgq/K/THZVQjBr3H0e9cmgeAbfzF3nOyF4L9apjhlfg2Cfyhd1kfn
JYI/U0OSZWcoI5w/dVba8UQFJjg/fkrsOZ7nCP5fl6U/5+9jhvNfPHWqONTO
AucvtzWUPpvICudvai0VTNdlg/N7QpO9mb8i+FZ3Fq5sP8sO569S3v1E4xaC
L5l9Rf7UMwT/Al/iXqm3CH7HprdR2Z8R/BSWLSsnf0PwHcKqfuz+ieAvf/BU
ZM8fBB8AAKmfHg==
       "]], LineBox[CompressedData["
1:eJwBAQP+/CFib1JlAgAAAC8AAAACAAAAWR8a60MDCEASV7o3oEzVP7CdkzNd
BghA1bDLy1xE1T9oQSDRrQwIQOqmVNKFM9U/2Ig5DE8ZCEAVk2bf1xHVP7cX
bIKRMghAbWuK+XvO1D+SaTLRuTUIQHrmznwQxtQ/brv4H+I4CECFYRMApb3U
PyZfhb0yPwhAmlecBs6s1D+Wpp7400sIQMVDrhMgi9Q/djXRbhZlCEAaHNIt
xEfUPzRTNlublwhAyswZYgzB0z/r541KB5sIQI2VhY7st9M/onzlOXOeCEBQ
XvG6zK7TPxGmlBhLpQhA0u/IE42c0z/u+PLV+rIIQNoSeMUNeNM/qJ6vUFrO
CEDqWNYoDy/TPxvqKEYZBQlADeWS7xGd0j8CgRsxl3IJQFD9C30XedE/nzaq
SpVJCkBah8lxjnfOP4jcjrciEgtA1WdbgvFJyj9FOcRImOsLQOUjlCYpwsU/
ToZPLZ22DEC1M61jZIfBPyZbhFWkfQ1Aat4mxdHDuj/S5gmik1UOQJUMQeqD
xLE/ymLlQRIfD0CAxDeCer6iP90Ep/iVIg9AQJi/nIBzoj/wpmivGSYPQABs
R7eGKKI/FevrHCEtD0CVE1fskpKhP2Bz8vcvOw9AqmJ2VqtmoD/3g/+tTVcP
QIABalW4HZw/JKUZGomPD0AAemSme76SPzdH29AMkw9AgCF024cokj9K6ZyH
kJYPQADJgxCUkpE/by0g9ZedD0AqGKN6rGaQP7q1JtCmqw9AqmzDnbodjD9Q
xjOGxMcPQFXlve59voI/Y2j1PEjLD0BVNN1YlpKBP3YKt/PLzg9AVYP8wq5m
gD+bTjph09UPQFVDdi6/HXw/5tZAPOLjD0AAvHB/gr5yP/l4AvNl5w9AAFqv
U7NmcD8MG8Sp6eoPQADw20/IHWw/Ml9HF/HxD0AAaNagi75iP0UBCc509Q9A
AEinktodXD9Yo8qE+PgPQADAoeOdvlI/akWMO3z8D0BVdThpwr5CP33nTfL/
/w9AVVVVWctCYj4TRVJT
       "]]}, {}}, {{}, {}, {}}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, True},
  AxesOrigin->{-1., 0.},
  FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
  GridLines->{None, None},
  PlotRange->{{-1., 4.}, {All, All}},
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.02], 
     Scaled[0.1]}},
  Ticks->{Automatic, Automatic}]], "Output",
 ImageSize->{184, 114},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->1627377131]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 WholeCellGroupOpener->True,
 CellID->1255426704],

Cell[TextData[StyleBox[ButtonBox["TriangularDistribution",
 BaseStyle->"Link",
 ButtonData->"paclet:ref/TriangularDistribution"],
 FontFamily->"Verdana",
 FontWeight->"Bold"]], "SeeAlso",
 CellID->929782353]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 WholeCellGroupOpener->True,
 CellID->250839057],

Cell[TextData[StyleBox[ButtonBox["Visualizing Statistical Concepts",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Classroom Utilities/tutorial/VisualizingStatisticalConcepts"],
 FontWeight->"Bold"]], "Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"TrapezoidalDistribution - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "SymbolColor", "LinkTrails" -> 
  GridBox[{{
      RowBox[{
        StyleBox[
         ButtonBox[
         "Mathematica", ButtonData -> "paclet:guide/Mathematica", 
          BaseStyle -> {"Link", "DockedLinkTrail"}], FontSlant -> "Italic"], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 31, 59.1340624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Symbol", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "TrapezoidalDistribution[{min, max}, {mode_min, mode_max}] represents a \
trapezoidal distribution taking values on the interval (min, max) achieving \
its maximum on the interval (mode_min, mode_max )", "synonyms" -> {}, "title" -> 
    "TrapezoidalDistribution", "titlemodifier" -> "", "windowtitle" -> 
    "TrapezoidalDistribution", "type" -> "Symbol", "uri" -> 
    "Classroom Utilities/ref/TrapezoidalDistribution"}, 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[6321, 194, 489, 14, 45, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->76199832]},
 "RasterizedOutput"->{
  Cell[8274, 264, 5793, 103, 145, "Output",
   CellTags->"RasterizedOutput",
   CellID->229758313],
  Cell[15280, 413, 4218, 77, 149, "Output",
   CellTags->"RasterizedOutput",
   CellID->10138667]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 43097, 1174},
 {"RasterizedOutput", 43239, 1178}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[616, 21, 2827, 65, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3468, 90, 66, 1, 51, "ObjectName",
 CellID->1224892054],
Cell[3537, 93, 1255, 40, 101, "Usage",
 CellID->982511436]
}, Open  ]],
Cell[CellGroupData[{
Cell[4829, 138, 609, 18, 39, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->275260772],
Cell[5441, 158, 468, 14, 70, "Notes",
 CellID->1067943069],
Cell[5912, 174, 372, 15, 70, "Notes",
 CellID->2093070604]
}, Closed]],
Cell[CellGroupData[{
Cell[6321, 194, 489, 14, 45, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->76199832],
Cell[CellGroupData[{
Cell[6835, 212, 192, 6, 31, "ExampleSection",
 CellID->355917],
Cell[7030, 220, 66, 1, 32, "ExampleText",
 CellID->308019921],
Cell[7099, 223, 128, 3, 28, "Input",
 CellID->1455222977],
Cell[7230, 228, 72, 1, 32, "ExampleText",
 CellID->241318756],
Cell[CellGroupData[{
Cell[7327, 233, 944, 29, 47, "Input",
 CellID->1395206556],
Cell[8274, 264, 5793, 103, 145, "Output",
 CellTags->"RasterizedOutput",
 CellID->229758313]
}, Open  ]],
Cell[CellGroupData[{
Cell[14104, 372, 126, 3, 70, "ExampleDelimiter",
 CellID->1138557463],
Cell[14233, 377, 75, 1, 70, "ExampleText",
 CellID->89381188],
Cell[CellGroupData[{
Cell[14333, 382, 944, 29, 65, "Input",
 CellID->1655985871],
Cell[15280, 413, 4218, 77, 149, "Output",
 CellTags->"RasterizedOutput",
 CellID->10138667]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[19547, 496, 126, 3, 70, "ExampleDelimiter",
 CellID->1433820333],
Cell[19676, 501, 62, 1, 70, "ExampleText",
 CellID->1304333744],
Cell[CellGroupData[{
Cell[19763, 506, 559, 18, 70, "Input",
 CellID->424166832],
Cell[20325, 526, 253, 9, 53, "Output",
 CellID->1900467166]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[20639, 542, 268, 8, 70, "ExampleSection",
 CellID->1293636265],
Cell[20910, 552, 426, 14, 70, "Input",
 CellID->2126097146],
Cell[21339, 568, 78, 1, 70, "ExampleText",
 CellID->1334330389],
Cell[CellGroupData[{
Cell[21442, 573, 750, 22, 70, "Input",
 CellID->1621417849],
Cell[22195, 597, 17371, 473, 70, "Output",
 CellID->1627377131]
}, Open  ]]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[39627, 1077, 446, 13, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[40076, 1092, 207, 5, 70, "SeeAlso",
 CellID->929782353]
}, Open  ]],
Cell[CellGroupData[{
Cell[40320, 1102, 448, 13, 70, "TutorialsSection",
 CellID->250839057],
Cell[40771, 1117, 231, 5, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[41017, 1125, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

