(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    136991,       3926]
NotebookOptionsPosition[    127190,       3603]
NotebookOutlinePosition[    129930,       3673]
CellTagsIndexPosition[    129849,       3668]
WindowTitle->Building an Understanding of Functions - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Classroom Utilities\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/guide/ClassroomUtilities"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"TutorialMoreAboutSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Functions \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"FunctionArrowDiagram\"\>":>
       Documentation`HelpLookup[
       "paclet:UnderstandingFunctions/ref/FunctionArrowDiagram"], \
"\<\"FunctionTable\"\>":>Documentation`HelpLookup[
       "paclet:UnderstandingFunctions/ref/FunctionTable"], "\<\"PlotGrid\"\>":>
       Documentation`HelpLookup["paclet:UnderstandingFunctions/ref/PlotGrid"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"TutorialFunction"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"ClassroomUtilities/tutorial/\
BuildingAnUnderstandingOfFunctions\"\>":>None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
             "ClassroomUtilities/tutorial/BuildingAnUnderstandingOfFunctions"]\
}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/ClassroomUtilities/\
tutorial/BuildingAnUnderstandingOfFunctions.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "ClassroomUtilities/tutorial/BuildingAnUnderstandingOfFunctions",
              ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["Building an Understanding of Functions", "Title",
 CellID->509267359],

Cell["\<\
Some of the functions defined in the ClassroomUtilities` package are intended \
to assist in presenting functions in a pre-calculus or calculus class. The \
results from these functions can be used in slideshow presentations in class \
or in homework assignments or quizzes.\
\>", "Text",
 CellID->1534169418],

Cell["This loads the package.", "MathCaption",
 CellID->1135936937],

Cell[BoxData[
 RowBox[{"<<", "ClassroomUtilities`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->562478329],

Cell[CellGroupData[{

Cell["The Functions Defined in the Package", "Section",
 CellID->350261409],

Cell[BoxData[GridBox[{
   {
    ButtonBox["FunctionTable",
     BaseStyle->"Link",
     ButtonData->"paclet:UnderstandingFunctions/ref/FunctionTable"], 
    RowBox[{
    "create", " ", "a", " ", "table", " ", "of", " ", "function", " ", 
     "values"}]},
   {
    ButtonBox["FunctionArrowDiagram",
     BaseStyle->"Link",
     ButtonData->"paclet:UnderstandingFunctions/ref/FunctionArrowDiagram"], 
    Cell["\<\
represent a function as two sets of points with the values indicated by arrows\
\>", "TableText"]},
   {
    ButtonBox["PlotGrid",
     BaseStyle->"Link",
     ButtonData->"paclet:UnderstandingFunctions/ref/PlotGrid"], Cell[
    "create grids for students to use in plotting given functions", 
     "TableText"]}
  }]], "DefinitionBox",
 CellID->2096742444],

Cell["Working with functions.", "Caption",
 CellID->1891092685]
}, Open  ]],

Cell[CellGroupData[{

Cell["Representing a Function as a Table of Values", "Section",
 CellID->992940646],

Cell["\<\
The algebraic form of the function can be used to generate the table.\
\>", "MathCaption",
 CellID->836781195],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionTable", "[", 
  RowBox[{
   RowBox[{"x", " ", 
    RowBox[{"Cos", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "4", ",", ".4"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->2058623809],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      ItemBox["x",
       Alignment->Center,
       BaseStyle->{FontWeight -> "Bold"},
       StripOnInput->False], 
      ItemBox[
       InterpretationBox["\<\"\\!\\(TraditionalForm\\`\\*TagBox[\\(\
TraditionalForm\\`\\\"\\\\[ScriptF]\\\"\\), TraditionalForm, Rule[Editable, \
True]]\\)(\\!\\(TraditionalForm\\`\\*TagBox[\\(TraditionalForm\\`x\\), \
TraditionalForm, Rule[Editable, True]]\\)) = \
\\!\\(TraditionalForm\\`\\*TagBox[\\(TraditionalForm\\`\\(x\\\\ \
\\(\\(cos(x)\\)\\)\\)\\), TraditionalForm, Rule[Editable, True]]\\)\"\>",
        StringForm["`3`(`2`) = `1`", 
         TraditionalForm[$CellContext`x Cos[$CellContext`x]], 
         TraditionalForm[$CellContext`x], 
         TraditionalForm["\[ScriptF]"]],
        Editable->False],
       Alignment->Center,
       BaseStyle->{FontWeight -> "Bold"},
       StripOnInput->False]},
     {"0.`", 
      TagBox[
       InterpretationBox["\<\"0.\"\>",
        0.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"0.4`", 
      TagBox[
       InterpretationBox["\<\"0.36842\"\>",
        0.3684243976011541,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"0.8`", 
      TagBox[
       InterpretationBox["\<\"0.55737\"\>",
        0.5573653674777325,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"1.2000000000000002`", 
      TagBox[
       InterpretationBox["\<\"0.43483\"\>",
        0.43482930537200815`,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"1.6`", 
      TagBox[
       InterpretationBox["\<\"-0.046719\"\>",
        -0.046719235682062106`,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"2.`", 
      TagBox[
       InterpretationBox["\<\"-0.83229\"\>",
        -0.8322936730942848,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"2.4000000000000004`", 
      TagBox[
       InterpretationBox["\<\"-1.7697\"\>",
        -1.76974491729899,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"2.8000000000000003`", 
      TagBox[
       InterpretationBox["\<\"-2.6382\"\>",
        -2.6382225538722435`,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"3.2`", 
      TagBox[
       InterpretationBox["\<\"-3.1945\"\>",
        -3.19454328254321,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"3.6`", 
      TagBox[
       InterpretationBox["\<\"-3.2283\"\>",
        -3.228330298802929,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"4.`", 
      TagBox[
       InterpretationBox["\<\"-2.6146\"\>",
        -2.6145744834544478`,
        AutoDelete->True],
       NumberForm[#, 5]& ]}
    },
    AutoDelete->False,
    BaseStyle->{"Menu"},
    GridBoxAlignment->{"Columns" -> {Left, "."}, "Rows" -> {{Automatic}}},
    GridBoxBackground->{"Columns" -> {{
         RGBColor[1, 1, 0.85]}}},
    GridBoxDividers->{"Columns" -> {
        AbsoluteThickness[1.5], None, 
        AbsoluteThickness[1.5]}, "Rows" -> {{True}}, 
      "RowsIndexed" -> {2 -> Directive[
          AbsoluteThickness[1.5]], 1 -> Directive[
          AbsoluteThickness[1.5]], -1 -> Directive[
          AbsoluteThickness[1.5]]}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
    GridBoxSpacings->{"Columns" -> {{1}}, "Rows" -> {{0.62}}}],
   "Grid"], TraditionalForm]], "Output",
 ImageSize->{169, 233},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->340725475]
}, Open  ]],

Cell["\<\
If all values are givne without decimal points, the function values can be \
exact.\
\>", "MathCaption",
 CellID->590824636],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionTable", "[", 
  RowBox[{
   RowBox[{"x", " ", 
    RowBox[{"Cos", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "\[Pi]", ",", 
     RowBox[{"\[Pi]", "/", "6"}]}], "}"}], ",", 
   RowBox[{"FunctionValues", "\[Rule]", " ", "Exact"}]}], "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->25273524],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      ItemBox["x",
       Alignment->Center,
       BaseStyle->{FontWeight -> "Bold"},
       StripOnInput->False], 
      ItemBox[
       InterpretationBox["\<\"\\!\\(TraditionalForm\\`\\*TagBox[\\(\
TraditionalForm\\`\\\"\\\\[ScriptF]\\\"\\), TraditionalForm, Rule[Editable, \
True]]\\)(\\!\\(TraditionalForm\\`\\*TagBox[\\(TraditionalForm\\`x\\), \
TraditionalForm, Rule[Editable, True]]\\)) = \
\\!\\(TraditionalForm\\`\\*TagBox[\\(TraditionalForm\\`\\(x\\\\ \
\\(\\(cos(x)\\)\\)\\)\\), TraditionalForm, Rule[Editable, True]]\\)\"\>",
        StringForm["`3`(`2`) = `1`", 
         TraditionalForm[$CellContext`x Cos[$CellContext`x]], 
         TraditionalForm[$CellContext`x], 
         TraditionalForm["\[ScriptF]"]],
        Editable->False],
       Alignment->Center,
       BaseStyle->{FontWeight -> "Bold"},
       StripOnInput->False]},
     {"0", "0"},
     {
      FractionBox["\[Pi]", "6"], 
      FractionBox["\[Pi]", 
       RowBox[{"4", " ", 
        SqrtBox["3"]}]]},
     {
      FractionBox["\[Pi]", "3"], 
      FractionBox["\[Pi]", "6"]},
     {
      FractionBox["\[Pi]", "2"], "0"},
     {
      FractionBox[
       RowBox[{"2", " ", "\[Pi]"}], "3"], 
      RowBox[{"-", 
       FractionBox["\[Pi]", "3"]}]},
     {
      FractionBox[
       RowBox[{"5", " ", "\[Pi]"}], "6"], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{"5", " ", "\[Pi]"}], 
        RowBox[{"4", " ", 
         SqrtBox["3"]}]]}]},
     {"\[Pi]", 
      RowBox[{"-", "\[Pi]"}]}
    },
    AutoDelete->False,
    BaseStyle->{"Menu"},
    GridBoxAlignment->{"Columns" -> {Left, "."}, "Rows" -> {{Automatic}}},
    GridBoxBackground->{"Columns" -> {{
         RGBColor[1, 1, 0.85]}}},
    GridBoxDividers->{"Columns" -> {
        AbsoluteThickness[1.5], None, 
        AbsoluteThickness[1.5]}, "Rows" -> {{True}}, 
      "RowsIndexed" -> {2 -> Directive[
          AbsoluteThickness[1.5]], 1 -> Directive[
          AbsoluteThickness[1.5]], -1 -> Directive[
          AbsoluteThickness[1.5]]}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
    GridBoxSpacings->{"Columns" -> {{1}}, "Rows" -> {{0.62}}}],
   "Grid"], TraditionalForm]], "Output",
 ImageSize->{131, 216},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->1381655423]
}, Open  ]],

Cell["You can define the function and use it in the table.", "MathCaption",
 CellID->713580102],

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", "x_", "]"}], ":=", " ", 
  RowBox[{
   SuperscriptBox["x", "2"], "-", "2"}]}]], "Input",
 CellLabel->"In[4]:=",
 CellID->1592089466],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionTable", "[", 
  RowBox[{
   RowBox[{"f", "[", "t", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", "6"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->601413442],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      ItemBox["t",
       Alignment->Center,
       BaseStyle->{FontWeight -> "Bold"},
       StripOnInput->False], 
      ItemBox[
       InterpretationBox["\<\"\\!\\(TraditionalForm\\`\\*TagBox[\\(\
TraditionalForm\\`f\\), TraditionalForm, Rule[Editable, \
True]]\\)(\\!\\(TraditionalForm\\`\\*TagBox[\\(TraditionalForm\\`t\\), \
TraditionalForm, Rule[Editable, True]]\\)) = \
\\!\\(TraditionalForm\\`\\*TagBox[\\(TraditionalForm\\`\\(t\\^2 - 2\\)\\), \
TraditionalForm, Rule[Editable, True]]\\)\"\>",
        StringForm["`3`(`2`) = `1`", 
         TraditionalForm[-2 + $CellContext`t^2], 
         TraditionalForm[$CellContext`t], 
         TraditionalForm[$CellContext`f]],
        Editable->False],
       Alignment->Center,
       BaseStyle->{FontWeight -> "Bold"},
       StripOnInput->False]},
     {"0", 
      TagBox[
       InterpretationBox["\<\"-2.\"\>",
        -2.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"1", 
      TagBox[
       InterpretationBox["\<\"-1.\"\>",
        -1.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"2", 
      TagBox[
       InterpretationBox["\<\"2.\"\>",
        2.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"3", 
      TagBox[
       InterpretationBox["\<\"7.\"\>",
        7.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"4", 
      TagBox[
       InterpretationBox["\<\"14.\"\>",
        14.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"5", 
      TagBox[
       InterpretationBox["\<\"23.\"\>",
        23.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"6", 
      TagBox[
       InterpretationBox["\<\"34.\"\>",
        34.,
        AutoDelete->True],
       NumberForm[#, 5]& ]}
    },
    AutoDelete->False,
    BaseStyle->{"Menu"},
    GridBoxAlignment->{"Columns" -> {Left, "."}, "Rows" -> {{Automatic}}},
    GridBoxBackground->{"Columns" -> {{
         RGBColor[1, 1, 0.85]}}},
    GridBoxDividers->{"Columns" -> {
        AbsoluteThickness[1.5], None, 
        AbsoluteThickness[1.5]}, "Rows" -> {{True}}, 
      "RowsIndexed" -> {2 -> Directive[
          AbsoluteThickness[1.5]], 1 -> Directive[
          AbsoluteThickness[1.5]], -1 -> Directive[
          AbsoluteThickness[1.5]]}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
    GridBoxSpacings->{"Columns" -> {{1}}, "Rows" -> {{0.62}}}],
   "Grid"], TraditionalForm]], "Output",
 ImageSize->{102, 159},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->742019350]
}, Open  ]],

Cell["The function does not need to be called f.", "MathCaption",
 CellID->1976218387],

Cell[BoxData[
 RowBox[{
  RowBox[{"g", "[", "r_", "]"}], ":=", " ", 
  RowBox[{
   SuperscriptBox["r", "2"], "-", 
   RowBox[{"2", " ", "r"}], " ", "+", "3"}]}]], "Input",
 CellLabel->"In[6]:=",
 CellID->1320345807],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionTable", "[", 
  RowBox[{
   RowBox[{"g", "[", "\[Theta]", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Theta]", ",", "0", ",", "7"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[7]:=",
 CellID->1103453798],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      ItemBox["\[Theta]",
       Alignment->Center,
       BaseStyle->{FontWeight -> "Bold"},
       StripOnInput->False], 
      ItemBox[
       InterpretationBox["\<\"\\!\\(TraditionalForm\\`\\*TagBox[\\(\
TraditionalForm\\`g\\), TraditionalForm, Rule[Editable, \
True]]\\)(\\!\\(TraditionalForm\\`\\*TagBox[\\(TraditionalForm\\`\[Theta]\\), \
TraditionalForm, Rule[Editable, True]]\\)) = \
\\!\\(TraditionalForm\\`\\*TagBox[\\(TraditionalForm\\`\\(\[Theta]\\^2 - \
\\(\\(2\\\\ \[Theta]\\)\\) + 3\\)\\), TraditionalForm, Rule[Editable, \
True]]\\)\"\>",
        StringForm["`3`(`2`) = `1`", 
         TraditionalForm[
         3 - 2 $CellContext`\[Theta] + $CellContext`\[Theta]^2], 
         TraditionalForm[$CellContext`\[Theta]], 
         TraditionalForm[$CellContext`g]],
        Editable->False],
       Alignment->Center,
       BaseStyle->{FontWeight -> "Bold"},
       StripOnInput->False]},
     {"0", 
      TagBox[
       InterpretationBox["\<\"3.\"\>",
        3.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"1", 
      TagBox[
       InterpretationBox["\<\"2.\"\>",
        2.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"2", 
      TagBox[
       InterpretationBox["\<\"3.\"\>",
        3.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"3", 
      TagBox[
       InterpretationBox["\<\"6.\"\>",
        6.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"4", 
      TagBox[
       InterpretationBox["\<\"11.\"\>",
        11.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"5", 
      TagBox[
       InterpretationBox["\<\"18.\"\>",
        18.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"6", 
      TagBox[
       InterpretationBox["\<\"27.\"\>",
        27.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"7", 
      TagBox[
       InterpretationBox["\<\"38.\"\>",
        38.,
        AutoDelete->True],
       NumberForm[#, 5]& ]}
    },
    AutoDelete->False,
    BaseStyle->{"Menu"},
    GridBoxAlignment->{"Columns" -> {Left, "."}, "Rows" -> {{Automatic}}},
    GridBoxBackground->{"Columns" -> {{
         RGBColor[1, 1, 0.85]}}},
    GridBoxDividers->{"Columns" -> {
        AbsoluteThickness[1.5], None, 
        AbsoluteThickness[1.5]}, "Rows" -> {{True}}, 
      "RowsIndexed" -> {2 -> Directive[
          AbsoluteThickness[1.5]], 1 -> Directive[
          AbsoluteThickness[1.5]], -1 -> Directive[
          AbsoluteThickness[1.5]]}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
    GridBoxSpacings->{"Columns" -> {{1}}, "Rows" -> {{0.62}}}],
   "Grid"], TraditionalForm]], "Output",
 ImageSize->{137, 179},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->681677321]
}, Open  ]],

Cell["\<\
Values can be omitted from the second column for students to fill in by \
indicating the position to be left blank.\
\>", "MathCaption",
 CellID->1213592640],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionTable", "[", 
  RowBox[{
   RowBox[{"g", "[", "\[Phi]", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Phi]", ",", "0", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "4"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[8]:=",
 CellID->51821274],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      ItemBox["\[Phi]",
       Alignment->Center,
       BaseStyle->{FontWeight -> "Bold"},
       StripOnInput->False], 
      ItemBox[
       InterpretationBox["\<\"\\!\\(TraditionalForm\\`\\*TagBox[\\(\
TraditionalForm\\`g\\), TraditionalForm, Rule[Editable, \
True]]\\)(\\!\\(TraditionalForm\\`\\*TagBox[\\(TraditionalForm\\`\[Phi]\\), \
TraditionalForm, Rule[Editable, True]]\\)) = \
\\!\\(TraditionalForm\\`\\*TagBox[\\(TraditionalForm\\`\\(\[Phi]\\^2 - \
\\(\\(2\\\\ \[Phi]\\)\\) + 3\\)\\), TraditionalForm, Rule[Editable, True]]\\)\
\"\>",
        StringForm["`3`(`2`) = `1`", 
         TraditionalForm[3 - 2 $CellContext`\[Phi] + $CellContext`\[Phi]^2], 
         TraditionalForm[$CellContext`\[Phi]], 
         TraditionalForm[$CellContext`g]],
        Editable->False],
       Alignment->Center,
       BaseStyle->{FontWeight -> "Bold"},
       StripOnInput->False]},
     {"0", 
      TagBox[
       InterpretationBox["\<\"3.\"\>",
        3.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"1", "\<\"   \"\>"},
     {"2", 
      TagBox[
       InterpretationBox["\<\"3.\"\>",
        3.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"3", "\<\"   \"\>"},
     {"4", 
      TagBox[
       InterpretationBox["\<\"11.\"\>",
        11.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"5", 
      TagBox[
       InterpretationBox["\<\"18.\"\>",
        18.,
        AutoDelete->True],
       NumberForm[#, 5]& ]}
    },
    AutoDelete->False,
    BaseStyle->{"Menu"},
    GridBoxAlignment->{"Columns" -> {Left, "."}, "Rows" -> {{Automatic}}},
    GridBoxBackground->{"Columns" -> {{
         RGBColor[1, 1, 0.85]}}},
    GridBoxDividers->{"Columns" -> {
        AbsoluteThickness[1.5], None, 
        AbsoluteThickness[1.5]}, "Rows" -> {{True}}, 
      "RowsIndexed" -> {2 -> Directive[
          AbsoluteThickness[1.5]], 1 -> Directive[
          AbsoluteThickness[1.5]], -1 -> Directive[
          AbsoluteThickness[1.5]]}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
    GridBoxSpacings->{"Columns" -> {{1}}, "Rows" -> {{0.62}}}],
   "Grid"], TraditionalForm]], "Output",
 ImageSize->{137, 143},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->125477302]
}, Open  ]],

Cell[TextData[{
 "The function can be evaluated at a list of points as in the ",
 ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"],
 " function."
}], "MathCaption",
 CellID->1633635841],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionTable", "[", 
  RowBox[{
   SuperscriptBox["x", "2"], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "3", ",", "4", ",", "7"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[9]:=",
 CellID->939152792],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      ItemBox["x",
       Alignment->Center,
       BaseStyle->{FontWeight -> "Bold"},
       StripOnInput->False], 
      ItemBox[
       InterpretationBox["\<\"\\!\\(TraditionalForm\\`\\*TagBox[\\(\
TraditionalForm\\`\\\"\\\\[ScriptF]\\\"\\), TraditionalForm, Rule[Editable, \
True]]\\)(\\!\\(TraditionalForm\\`\\*TagBox[\\(TraditionalForm\\`x\\), \
TraditionalForm, Rule[Editable, True]]\\)) = \
\\!\\(TraditionalForm\\`\\*TagBox[\\(TraditionalForm\\`x\\^2\\), \
TraditionalForm, Rule[Editable, True]]\\)\"\>",
        StringForm["`3`(`2`) = `1`", 
         TraditionalForm[$CellContext`x^2], 
         TraditionalForm[$CellContext`x], 
         TraditionalForm["\[ScriptF]"]],
        Editable->False],
       Alignment->Center,
       BaseStyle->{FontWeight -> "Bold"},
       StripOnInput->False]},
     {"1", "\<\"   \"\>"},
     {"3", 
      TagBox[
       InterpretationBox["\<\"9.\"\>",
        9.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"4", "\<\"   \"\>"},
     {"7", 
      TagBox[
       InterpretationBox["\<\"49.\"\>",
        49.,
        AutoDelete->True],
       NumberForm[#, 5]& ]}
    },
    AutoDelete->False,
    BaseStyle->{"Menu"},
    GridBoxAlignment->{"Columns" -> {Left, "."}, "Rows" -> {{Automatic}}},
    GridBoxBackground->{"Columns" -> {{
         RGBColor[1, 1, 0.85]}}},
    GridBoxDividers->{"Columns" -> {
        AbsoluteThickness[1.5], None, 
        AbsoluteThickness[1.5]}, "Rows" -> {{True}}, 
      "RowsIndexed" -> {2 -> Directive[
          AbsoluteThickness[1.5]], 1 -> Directive[
          AbsoluteThickness[1.5]], -1 -> Directive[
          AbsoluteThickness[1.5]]}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
    GridBoxSpacings->{"Columns" -> {{1}}, "Rows" -> {{0.62}}}],
   "Grid"], TraditionalForm]], "Output",
 ImageSize->{96, 104},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->68073360]
}, Open  ]],

Cell["\<\
Blanks can be inserted in the first column by using a two list form for the \
exclusions.\
\>", "MathCaption",
 CellID->2122055448],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionTable", "[", 
  RowBox[{
   SuperscriptBox["x", "2"], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "3", ",", "4", ",", "7"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", "2", "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "3"}], "}"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[10]:=",
 CellID->2124988807],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      ItemBox["x",
       Alignment->Center,
       BaseStyle->{FontWeight -> "Bold"},
       StripOnInput->False], 
      ItemBox[
       InterpretationBox["\<\"\\!\\(TraditionalForm\\`\\*TagBox[\\(\
TraditionalForm\\`\\\"\\\\[ScriptF]\\\"\\), TraditionalForm, Rule[Editable, \
True]]\\)(\\!\\(TraditionalForm\\`\\*TagBox[\\(TraditionalForm\\`x\\), \
TraditionalForm, Rule[Editable, True]]\\)) = \
\\!\\(TraditionalForm\\`\\*TagBox[\\(TraditionalForm\\`x\\^2\\), \
TraditionalForm, Rule[Editable, True]]\\)\"\>",
        StringForm["`3`(`2`) = `1`", 
         TraditionalForm[$CellContext`x^2], 
         TraditionalForm[$CellContext`x], 
         TraditionalForm["\[ScriptF]"]],
        Editable->False],
       Alignment->Center,
       BaseStyle->{FontWeight -> "Bold"},
       StripOnInput->False]},
     {"1", "\<\"   \"\>"},
     {"\<\"   \"\>", 
      TagBox[
       InterpretationBox["\<\"9.\"\>",
        9.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"4", "\<\"   \"\>"},
     {"7", 
      TagBox[
       InterpretationBox["\<\"49.\"\>",
        49.,
        AutoDelete->True],
       NumberForm[#, 5]& ]}
    },
    AutoDelete->False,
    BaseStyle->{"Menu"},
    GridBoxAlignment->{"Columns" -> {Left, "."}, "Rows" -> {{Automatic}}},
    GridBoxBackground->{"Columns" -> {{
         RGBColor[1, 1, 0.85]}}},
    GridBoxDividers->{"Columns" -> {
        AbsoluteThickness[1.5], None, 
        AbsoluteThickness[1.5]}, "Rows" -> {{True}}, 
      "RowsIndexed" -> {2 -> Directive[
          AbsoluteThickness[1.5]], 1 -> Directive[
          AbsoluteThickness[1.5]], -1 -> Directive[
          AbsoluteThickness[1.5]]}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
    GridBoxSpacings->{"Columns" -> {{1}}, "Rows" -> {{0.62}}}],
   "Grid"], TraditionalForm]], "Output",
 ImageSize->{96, 105},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[10]=",
 CellID->1647661316]
}, Open  ]],

Cell["\<\
To have all values included in the second column use {} in the second slot.\
\>", "MathCaption",
 CellID->1077533374],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionTable", "[", 
  RowBox[{
   RowBox[{"g", "[", "r", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", 
     RowBox[{"-", "2"}], ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"2", ",", "3", ",", "5"}], "}"}], ",", 
     RowBox[{"{", "}"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[11]:=",
 CellID->373821667],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      ItemBox["r",
       Alignment->Center,
       BaseStyle->{FontWeight -> "Bold"},
       StripOnInput->False], 
      ItemBox[
       InterpretationBox["\<\"\\!\\(TraditionalForm\\`\\*TagBox[\\(\
TraditionalForm\\`g\\), TraditionalForm, Rule[Editable, \
True]]\\)(\\!\\(TraditionalForm\\`\\*TagBox[\\(TraditionalForm\\`r\\), \
TraditionalForm, Rule[Editable, True]]\\)) = \
\\!\\(TraditionalForm\\`\\*TagBox[\\(TraditionalForm\\`\\(r\\^2 - \\(\\(2\\\\ \
r\\)\\) + 3\\)\\), TraditionalForm, Rule[Editable, True]]\\)\"\>",
        StringForm["`3`(`2`) = `1`", 
         TraditionalForm[3 - 2 $CellContext`r + $CellContext`r^2], 
         TraditionalForm[$CellContext`r], 
         TraditionalForm[$CellContext`g]],
        Editable->False],
       Alignment->Center,
       BaseStyle->{FontWeight -> "Bold"},
       StripOnInput->False]},
     {
      RowBox[{"-", "2"}], 
      TagBox[
       InterpretationBox["\<\"11.\"\>",
        11.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"\<\"   \"\>", 
      TagBox[
       InterpretationBox["\<\"6.\"\>",
        6.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"\<\"   \"\>", 
      TagBox[
       InterpretationBox["\<\"3.\"\>",
        3.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"1", 
      TagBox[
       InterpretationBox["\<\"2.\"\>",
        2.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"\<\"   \"\>", 
      TagBox[
       InterpretationBox["\<\"3.\"\>",
        3.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"3", 
      TagBox[
       InterpretationBox["\<\"6.\"\>",
        6.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"4", 
      TagBox[
       InterpretationBox["\<\"11.\"\>",
        11.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"5", 
      TagBox[
       InterpretationBox["\<\"18.\"\>",
        18.,
        AutoDelete->True],
       NumberForm[#, 5]& ]}
    },
    AutoDelete->False,
    BaseStyle->{"Menu"},
    GridBoxAlignment->{"Columns" -> {Left, "."}, "Rows" -> {{Automatic}}},
    GridBoxBackground->{"Columns" -> {{
         RGBColor[1, 1, 0.85]}}},
    GridBoxDividers->{"Columns" -> {
        AbsoluteThickness[1.5], None, 
        AbsoluteThickness[1.5]}, "Rows" -> {{True}}, 
      "RowsIndexed" -> {2 -> Directive[
          AbsoluteThickness[1.5]], 1 -> Directive[
          AbsoluteThickness[1.5]], -1 -> Directive[
          AbsoluteThickness[1.5]]}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
    GridBoxSpacings->{"Columns" -> {{1}}, "Rows" -> {{0.62}}}],
   "Grid"], TraditionalForm]], "Output",
 ImageSize->{142, 182},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->442109947]
}, Open  ]],

Cell["\<\
The position of the table can be set using Text Alignment from the Format \
menu after rendering the table.\
\>", "MathCaption",
 CellID->1060121871],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionTable", "[", 
  RowBox[{
   RowBox[{"g", "[", "r", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", 
     RowBox[{"-", "2"}], ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"2", ",", "5"}], "}"}], ",", 
     RowBox[{"{", "}"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[12]:=",
 CellID->1943950385],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      ItemBox["r",
       Alignment->Center,
       BaseStyle->{FontWeight -> "Bold"},
       StripOnInput->False], 
      ItemBox[
       InterpretationBox["\<\"\\!\\(TraditionalForm\\`\\*TagBox[\\(\
TraditionalForm\\`g\\), TraditionalForm, Rule[Editable, \
True]]\\)(\\!\\(TraditionalForm\\`\\*TagBox[\\(TraditionalForm\\`r\\), \
TraditionalForm, Rule[Editable, True]]\\)) = \
\\!\\(TraditionalForm\\`\\*TagBox[\\(TraditionalForm\\`\\(r\\^2 - \\(\\(2\\\\ \
r\\)\\) + 3\\)\\), TraditionalForm, Rule[Editable, True]]\\)\"\>",
        StringForm["`3`(`2`) = `1`", 
         TraditionalForm[3 - 2 $CellContext`r + $CellContext`r^2], 
         TraditionalForm[$CellContext`r], 
         TraditionalForm[$CellContext`g]],
        Editable->False],
       Alignment->Center,
       BaseStyle->{FontWeight -> "Bold"},
       StripOnInput->False]},
     {
      RowBox[{"-", "2"}], 
      TagBox[
       InterpretationBox["\<\"11.\"\>",
        11.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"\<\"   \"\>", 
      TagBox[
       InterpretationBox["\<\"6.\"\>",
        6.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"0", 
      TagBox[
       InterpretationBox["\<\"3.\"\>",
        3.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"1", 
      TagBox[
       InterpretationBox["\<\"2.\"\>",
        2.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"\<\"   \"\>", 
      TagBox[
       InterpretationBox["\<\"3.\"\>",
        3.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"3", 
      TagBox[
       InterpretationBox["\<\"6.\"\>",
        6.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"4", 
      TagBox[
       InterpretationBox["\<\"11.\"\>",
        11.,
        AutoDelete->True],
       NumberForm[#, 5]& ]},
     {"5", 
      TagBox[
       InterpretationBox["\<\"18.\"\>",
        18.,
        AutoDelete->True],
       NumberForm[#, 5]& ]}
    },
    AutoDelete->False,
    BaseStyle->{"Menu"},
    GridBoxAlignment->{"Columns" -> {Left, "."}, "Rows" -> {{Automatic}}},
    GridBoxBackground->{"Columns" -> {{
         RGBColor[1, 1, 0.85]}}},
    GridBoxDividers->{"Columns" -> {
        AbsoluteThickness[1.5], None, 
        AbsoluteThickness[1.5]}, "Rows" -> {{True}}, 
      "RowsIndexed" -> {2 -> Directive[
          AbsoluteThickness[1.5]], 1 -> Directive[
          AbsoluteThickness[1.5]], -1 -> Directive[
          AbsoluteThickness[1.5]]}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
    GridBoxSpacings->{"Columns" -> {{1}}, "Rows" -> {{0.62}}}],
   "Grid"], TraditionalForm]], "Output",
 TextAlignment->-0.5,
 ImageSize->{142, 181},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->248460609]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Using the FunctionArrowDiagram", "Section",
 CellID->105144541],

Cell[TextData[{
 "The ",
 Cell[BoxData[
  ButtonBox["FunctionArrowDiagram",
   BaseStyle->"Link",
   ButtonData->"paclet:ClassroomUtilities/ref/FunctionArrowDiagram"]], 
  "InlineFormula"],
 " function is used to represent a function by two sets of points with arrows \
indicating where points in the first set end up in the second set. With the \
default setting the sets are essenrially dimensionless with no exact position \
indicated for the points. Various properties such as one-to-one and onto can \
be included. It is also possible to include relations which are not functions \
by have one or more points from the first set with multiple images in the \
second set."
}], "Text",
 CellID->1932742753],

Cell["A basic diagram illustrating a one-to one map.", "MathCaption",
 CellID->769805149],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionArrowDiagram", "[", 
  RowBox[{"5", ",", "8", ",", "\"\<OneToOne\>\""}], "]"}]], "Input",
 CellLabel->"In[114]:=",
 CellID->1673649715],

Cell[BoxData[
 GraphicsBox[
  {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
   0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
   {RGBColor[0.0692454, 0.645548, 0.968322], 
    DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
   {GrayLevel[0], PointSize[Medium], Opacity[1], 
    PointBox[{{0.8, 
      0.6928203230275509}, {-0.5000000000000001, -0.17320508075688773`}, \
{-0.4, 0.6928203230275509}, {-0.9, 0.17320508075688773`}, {
      0, -0.6928203230275509}, {2.4, -1.0392304845413265`}, {3.5, 
      0.5196152422706632}, {2.7, 0.17320508075688773`}, {2.8, 
      0.6928203230275509}, {2.2, 0.6928203230275509}, {
      3., -0.34641016151377546`}, {2.5, 1.2124355652982142`}, {
      3.6, -0.34641016151377546`}}], 
    {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAAAAAiCq2GlSvmv3OKeKJD
u7w/zlHUuAeh479CVFlL5C3NP2Jm3TX/QOG//vh3oXU01j8E7vg5fRnev/RZ
APNQA94/aeH5pOIK2r8etWrBr//iP1AzzMFVWNa/XOnqXjIS5z+0xo7NzAPT
v/VbVCoHN+s/QA30IgwP0L9CxF9D+2vvP8JxepRK98q/tizqnWnX8T9Yg1lI
7JXGv1KUXSOm/vM/WJfCYFD8wr/vwr21jir2Pzza9t5fLMC/ttEeXvxZ+D/A
XIiFMU+8v7dPd0fGi/o/sNupfxreub+6+B9cwr78P1hRZFjGBrm/e4r948Xx
/j8QJYtQp8m5vy5mgBHTkQBAKJ4TAVYmvL8aWNR7nKkBQLAz+cjIDcC/lDiZ
vKq/AkAIbywyoNPCv9bgkUpq0wNAKO37lzhjxr+xuO3VSOQEQLhJWEmuusq/
9rvqlbXxBUAkx++Is9fPv2oBqJUh+wZAVU4M48jb0r8AAAAAAAAIQCEKrYaV
K9a/EBnJhQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAQAAAAAA4L8iCq2GlSvGv3zqofhb
R9W/86QBA8wqt7/y607RxafEvyB265rSvY+/wAugWHWTij/sC18mtG+qP1Af
wjRoWMg/sFnrxtJavD8ikAbB4azXP/ZBtNyTtMQ/a0+RFkWo4T8YcTu99S3K
P4HM3i6AiOc/MuePcqeWzj/RFq4EBHTtP/F8Lg0p9tA/Kho/O9az8T+qjBxg
mBbSP75TW4gosPQ//Rb+pgis0j++EE9dY633P6YuFJcqttI/ij5csvCp+j+v
gwfQ+DTSP9rT09s6pP0/nazC3rco0T9ctqqwVk0AQABsJTLsI88/8us16trF
AUBGikKkFuPKPxZbClPiOgNAfMHgzzCRxT+672D9pqsEQHSIUqgaYr4/2wcy
PmUXBkCwGPnPOBivP820BxVcfQdAoL/HsJ57db/a0GWSzdwIQFYMahdyQLS/
8JKRPP80CkCSoPHQAZLEvz4ng3I6hQtAvycdkKz7z7/OzMzMzMwMQCIKrYaV
K9a/7o/ANw==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAzczMzMzM7L8iCq2GlSvGP7LXy62x
M+m/A/nEyArA0D8YKdH1iXrlvwq98RXxFNY/q/EiY0+j4b8GgPNzqRHbP9Zh
EVIXYNu/HnSWdY6z3z+Sy/W1rkXTv264Zm8V/OE/KH+4P2P3xb8mgEh5ne7j
P+AXCOUvNKS/Rpg901aw5T/I2lp1XVW4Pxx5vt1SQOc/TIReH6qmzT8aCYxd
vZ3oP2bCjaDPrtc/7rhL7NzH6T/jglX4iVHgP/z+5loTvuo/o9TZ3LDV5D9u
/3kF3n/rP3tk0Ql3Yek/gNikGNYM7D/XaOD/cvLtPxrLGsiwZOw/CkL7PRxD
8T/STFJ2P4fsPwyFNeAsjfM/kPRAzW907D8CFHFvNNb1P84hFshLLOw/QvEl
gfwc+D9INe+t+a7rP37pJtxPYPo/MCqI/bv86j8SbbQc+578P+BW80nxFeo/
BlWWV83X/j9qDWsIFPvoP+8E813MhABAdrxXT7qs5z+amZmZmZkBQCIKrYaV
K+Y/r2jOvw==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQPWsmCNzcr8S1tm2q9jP7LaJCc25nX9wflG3ltCDj
hX1S7ZYCO4lD+x/1HpbILX1lf0DpZ3Dl1vr9FYEvpzwNeGO/xuxc6du8A/Yy
AU9yeX+9tU/JtHRWF7po71GXyMtc8N7eYsuk/x08t+x/mc9iZoj8YG9Y2Wj6
ZsJD+0/3/Q+uuf/Bnv1Zxu2Fas/ste7Veh3w/mi/NGvymm9HXtmfnH/UjXHu
R/vGeV8DyjPf2/+01p7pcvmj/Y6JilOvcXyyX7fqa1nuu4/26664XOyP/QLn
q0x/vXfVxm9w9UIbZQReM/+Em5f0eWOFaPBvuH1nVi71/jD3L9w9ETsb1Vwe
/4e7l43tjNc7AUYHmH9O+F+LdtFhcoD594j+3fe8zswOsPDgrLaT2h7C4gAL
r5elW5etS2B1gIVnGhiwOcDCGwBWucLb
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ6T8iCq2GlSvmP9rpJCmz
aOs/lmlW1wAG6D9tYU+r6UztP3rMEGDayuk/IFXTMzxF7z9mKMHYMXnrP/TG
cJVPqPA/C9Ok6iIQ7T/7q45t/rbxPzeNeanVju4/y0p7AhvO8j80Cw8Gf/Tv
PybTiTsR7fM/LZgCnTCg8D/BIb3VSBP1P2DpPhbmOPE/nPeRtCVA9j+pTMbn
DsTxP1Qc1DQIc/c/PrAZO2FB8j8YeFKBTav4P9bCXpGasPI/rDNF6U/o+T8Q
Q6jmfxHzP4QGODhnKfs/B0BG0d1j8z9aDUoP6W38P6mtDZ2Ip/M/9c2TPym1
/T8704hiXNzzPypxgyV6/v4/UkYEGj0C9D+pXH+CliQAQG1WbqoWGfQ/MekM
M0nKAEAIBQH03CD0PwzlGDn9bwFAluGw1osZ9D/WpJyoWhUCQKJfXTQnA/Q/
4geDwwm6AkCwfcHuut3zP4zUNiizXQNAxtQl4Vqp8z8AAAAAAAAEQN5o19Ui
ZvM/AQu3Nw==
      "]], 
     {GrayLevel[0]}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
     StripOnInput -> False], TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 259},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[114]=",
 CellID->291713427]
}, Open  ]],

Cell["\<\
The points are randomly selected from a predefined set of points so each \
diagram will be distinct.\
\>", "MathCaption",
 CellID->1369107928],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionArrowDiagram", "[", 
  RowBox[{"5", ",", "8", ",", "\"\<OneToOne\>\""}], "]"}]], "Input",
 CellLabel->"In[115]:=",
 CellID->309668291],

Cell[BoxData[
 GraphicsBox[
  {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
   0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
   {RGBColor[0.0692454, 0.645548, 0.968322], 
    DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
   {GrayLevel[0], PointSize[Medium], Opacity[1], 
    PointBox[{{0, 0.34641016151377546`}, {
      0.8, -0.34641016151377546`}, {-0.6000000000000001, 
      0.34641016151377546`}, {-0.6000000000000001, -0.6928203230275509}, {
      0, -1.0392304845413265`}, {3.9, 0.17320508075688773`}, {3.6, 
      0.6928203230275509}, {3.5, -0.8660254037844386}, {
      2.1, -0.5196152422706632}, {3., -0.6928203230275509}, {
      2.6, -1.0392304845413265`}, {3.6, -0.34641016151377546`}, {3.2, 
      0.34641016151377546`}}], 
    {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQDQOzjjOqbljwYf/anB/lK6fssrfdsGDi8fz3+/d8
Zrv37Okpew2XQOXEo2/3yz6uZtqfesXebmqRHb/zm/3Mk1e43K28bR+5TO3e
vJev9lvs3Bp5LPSh/dUPE3yO73y5f95/S76630/sK3qmHtTc8mL//Rn3A9XW
vLBvi/bjDbz5fL/vwtUvTqa+sc+al+y3UfP5/nYx/0xz4Q/2J5nztyxf+mz/
zN9ia7i/fbT//bTTr8oXyK+M60m89dle9JPCs3DFZ/sZFax5LfZ/RfDnNb3t
Xfwdrl642csgqe0n3LwvJ5yXbkn7DbdP6+zKgjbnv3D3nJ5ZM+mW9H+4e+vj
4s2iyxkcYP7J2WWuMjGW0QHm36yA2jciDkwOsPCYFPsmLk+O2QEWXhmcvI3q
P5gdYOF59gwIsDjAwhsA/vrJ3A==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz478iCq2GlSvmv+NQCF/C
rdu/OPxhpQmG479h3A1PSbrQv3J1bW+nH+G/KJiF7ipHtr+2rTfFafPdvwCl
6s82F7c/yLhImKsq2r+4KaBae0bRPzhuqVsW59a/bDi8xiPq3D+p1mVrZSrU
v0yZq/tMVeQ/m5u7kAz20b+c43rR0EDqP0QS1Tw3S9C/j4CloTwa8D8WBc7T
j1XOvyK6we6OFvM/cPAKRq8qzb8id7XDyRP2Px7B3mVrFs2/7qTCGFcQ+T8K
F/jzzhjOvzw6OkKhCvw/mOJAa6gY0L8c07vHEwH/PzXZ8DBqr9G/JB9pHQ75
AEASSuL31M/Tv0iOPYYVbgJAdi4T4sd41r/sIpQw2t4DQBft7p/ZqNm/DTtl
cZhKBUAebAQwWV7dv//nOkiPsAZAGVdjYqfL4L8LBJnFABAIQCQJ72e+KOO/
IcbEbzJoCUC+Lz6ZMMXlv29atqVtuApAiREJSZuf6L////////8LQKpMWOh6
tuu/3RfEhg==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ6T8iCq2GlSvWv3ejSINb
p+w/7v7SQE76079nvsb1E8HvP3Ilg43FENK/HMCs5Y5y8T+JSMor/2/Qv9L/
YDLnCPM/IjMzf7Axzr92K81Fu6L0PzDj4LgNGMy/vleRqzE/9j+DmYBJM5TK
v7I3golv3fc/1YR8/O6myb/6i8gTmXz5P7pElLa+UMm/JM2AAdIb+z8cXhAz
0JHJv60vnQE+uvw/qVt+6wBqyr8Q/csvAVf+P5p5Aire2Mu/GTskiUDx/z90
JTVGpd3Nv4xcLTARxABAlAI2BqI70L8c7aBoZ40BQGPcm6cs0tG/HmqrGThU
AkD6kWTRmrHTv2PNj8UZGANAImjtIO7Y1b8YqA59pNgDQJ550g0CR9i/0OqL
FnKVBEBjbSaFjPrav2n6Q2QeTgVAZoSVmB7y3b+KTnNpRwIGQNiIi6ASluC/
lGVUjo2xBkBSB2MadVPiv8x32NKTWwdAGqq3Z0ow5L8AAAAAAAAIQCIKrYaV
K+a/bGrAJg==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz478iCq2GlSvWP4DI0ROj
Sd6/EbwW7fS71z/yHZyuzRzWv0hf7jVk7dg/eMfOVnnIy79KcfpQQb/ZPyga
NOIykba/GOm84xwx2j/ATCWTDPWkPza8iIS6Qto/zIWADljAxT9i8Y/aEPTZ
PwrMpjTxG9M/Vj/ZoklF2T9cGBG5kk3bP7KVGprBNtg//J0cylu44T9PUIRL
CMnWP0ad1DqHwOU/uDCXxN/81D/my5oDKL3pP9CHLS4809I/5EwWmSCs7T8A
Me5KQ03QP0x3gVetxfA/qDnctpfYyj9pU9gpZKzyP/Q9ptC5Y8Q/aCqKgzKJ
9D+c+mHLs366PyfbamYbW/Y/UAgbYQi9pT8Yk2ScJyH4P6Ds18FCQ5i/Vlij
Omba+T8Y5fiETka4vzylaSHthfs/AMRx9SXcxb/kQUp42SL9P5qFReQdF9C/
XnGFJlCw/j+05zhs94nVv7jKpCO/FgBAIBlwd7tD27/NzMzMzMwAQJvHASWw
oOC/oXDGkg==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQDQNKXGvbpmpfs98g5i9wMu+gvU39Zo2p4nfs9bpf
iFduvGjflthVb3nugf3HU67BVS9v2U9M3bR+QfRj+4+2OYxOuo/s5/5+ULR0
01P71Fq5ayf/P7MP9o7/xHnpuf0KAfe5R1++tv/QF3p77Y6X9l0tiW99Sj7Y
r+A6/r0v9bX901cljBf+fLLX2TJ5+8nrb+xfGnp/FO79ap98w2zlTL539knn
1Nv2KP+w/7qW6VEk53v7MsfnKln7f9kfaFsY2nr8vf2pN30du+L+2t+RCO+L
VvlgvzBceb8KA4ODFo98aKjbB/uJbjoXIwIZHdrCFX7M9fxgv5L3ZZ/bIiaH
f6UBO431PthvaTc0sv7E7NA8V87T68V7exFprsoWB1aH+KX3+JXM39v7C1uy
FPawOVh67jrm4/nOPm1l8taAq+wOHB/j5nvIv7V3ON3AUyrF6aB6KmLX5p2v
7RfzFLQYRnM5yF/Tvuwt+cpeYWHzu9vTuB2cXi2//dbuhf25MyDA4wAJ72f2
ABHytq4=
      "]], 
     {GrayLevel[0]}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
     StripOnInput -> False], TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 259},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[115]=",
 CellID->122401488]
}, Open  ]],

Cell["Diagrams can be repeated by using SeedRandom.", "MathCaption",
 CellID->120719405],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"SeedRandom", "[", "1234", "]"}], ";", 
    RowBox[{"FunctionArrowDiagram", "[", 
     RowBox[{"8", ",", "6", ",", "\"\<Onto\>\""}], "]"}]}], ",", 
   RowBox[{
    RowBox[{"SeedRandom", "[", "1234", "]"}], ";", 
    RowBox[{"FunctionArrowDiagram", "[", 
     RowBox[{"8", ",", "6", ",", "\"\<Onto\>\""}], "]"}]}]}], "}"}]], "Input",
 CellLabel->"In[116]:=",
 CellID->1706477067],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[
    {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
     0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
     {RGBColor[0.0692454, 0.645548, 0.968322], 
      DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
     {GrayLevel[0], PointSize[Medium], Opacity[1], 
      PointBox[{{0.4, 
        0.34641016151377546`}, {-0.2, 
         0}, {-0.9, -0.17320508075688773`}, {-0.6000000000000001, \
-1.0392304845413265`}, {0.1, -0.5196152422706632}, {-0.30000000000000004`, 
        1.2124355652982142`}, {0.8, -0.34641016151377546`}, {0.4, 
        1.0392304845413265`}, {2.2, 0.}, {3.3, -0.8660254037844386}, {3.2, 
        0.6928203230275509}, {2.3, 0.5196152422706632}, {
        3.9, -0.17320508075688773`}, {2.7, 1.2124355652982142`}}], 
      {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz47+axwElsKDwvyyy8XOv
rNy/JiptVlFO7r8njaCK0bDSv4TaCnSzlOu/OETmnDHwwL8iEH7K+BXpv4Dk
pCflgJ8/Ag6CQnTT5r/sIUUBNDTJP9m/GNJYzuS/VL1l62Vo1z9yL6vZuAfj
v0avsSObLuE/IAV0koWA4b8M9aRsXbnmP7RSgY6OOeC/GiO/BYlR7D/yJCOV
Amfev80YPgMT+vA/YpUMpNHd3L/1Qet9G8/zP/xZOupa2Nu/lDs2P12m9j8E
ry0jKVfbv2yMX2VWfvk/AivR24Ba279gtFithFX8P0ZOGU5g4tu//G63P2Yq
/z+YbfVhf+7cv1/stz69/QBAYneQ009+3r8oa/RloWMCQDmkZr9+SOC/aTr/
ryHGA0DmQOtnt5Lhv9tFwwOCJAVASxNDrCId47/GWXZ4CH4GQNjdKkXv5uS/
3NM9uP3RB0A2IDpKKu/mv76ydWGtHwlA0hTGsr806b9nZmZmZmYKQKpMWOh6
tuu/OETHMA==
        "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZuT+axwElsKDgv/nTzJxN
Qc8/9G2LUL8N279C52IktyDZP9MFWd9HR9W/VGZ4esZw4T+SigjfFOLPv8p2
Q7f9buY/unIP3Lcbxr+wSDgQW4jrP7TQbexQgbq/JNOh8hRd8D/gdaRPZlal
v1SQDU/UAPM/AFZ6yxMUij8VMlAQBa71P0AqBQLYgK4/Ps2vpjtj+D9weGOK
LEi5P1R7r0AIH/s/kOCT0TiqwD+PWjmO99/9PyB79A57sMM/Cp3RwklSAED4
JicrQrXFP1m0EhWytQFAFIqB8nu3xj8ZLK0peBkDQOQWWWGftsY/Q52uOt98
BEDMKbbsrLLFPwG1jrQq3wVA0PAVgi6swz/dKD6anj8HQEQ8Oz43pMA/rtPl
6H+dCEB4Hmq2xTi5P2EOIfsU+AlA0IuhV1Nbrj87nn7rpU4LQAB8ELl0Y4k/
UvQT9nygDEAI5bDAGYmlv3347tjm7A1AtGK8B+Odur8zMzMzMzMPQCIKrYaV
K8a/TRu4Bw==
        "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ6T8iCq2GlSvWv+/Jho6S
Jes/qvqIpoRT07/wFyAUtMHsP2FGJHdAoNC/1ChDfSNt7j9wVhCDbybMv6zM
/P5+E/A/gjoIC4pbx7/8xueRLPfwP6g6p4Nb4sK/wLccqSHh8T+YiaQdh3q9
v0hpYyHi0PI/hOPmYOrctb85Qm/E7sXzP5AH947W362/p+tejMW/9D/AZ+8i
bW6hv0vGuejhvfU/QOu1q8qwib+uk8YEvb/2PwBz1PwLCHk/+gAWD87E9z+Q
e8rgO2iWP74dG4KKzPg/MCvJ2NnIoT/MOattZtb5P+ACrk2326Y/Yyo+wdTh
+j9gddwhBWqqPxCRx5ZH7vs/sGarU+BxrD8ibQF+MPv8P8BTSA018qw/vAQA
yAAI/j/A8xI3v+qrP38L59IpFP8/qGy+mwpcqT+b7cuqjg8AQHjvJJ5yR6U/
PkuY1SaUAEAALeoBQ16fP0ITGg+XFwFA0Kk1gR4skT+amZmZmZkBQAAAAAAA
AJA8lTbEXg==
        "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAzczMzMzM7L8iCq2GlSvGvyxjQxOp
9ei/NK7B58g4ub+KyxGrsQPlvwjZBQfOFZ6/SmFcd/744L8wq7Kk4XKhPycS
2fVor9m/WGlDYEV7tz/6YsVpCUTRvx6/SdIaXMI/qBIx3Ktowb8UDy26nTTI
PwCY3tuzZTO/amieWxFEzT8Eu/ejoIvBP9Sahizjw9A/RveaG/2m0T+rWR64
vH7SP17648r2mdo/C03PVqrR0z9B7njk/szhP70Z1zL4u9Q//T/9yCVR5j+z
xOP5KT3VP3WA/J4K1+o/a7cKH/tU1T9nQ9YFR1zvP1UwKP9eA9U/vIskezrv
8T+j0pPngEjUP+IHIASYLfQ/EMcm/8Mk0z+UAeJbC2j2P/mdnxHDmNE/aAsy
1GWd+D/ouv96oErPP2z9NHN7zPo/gp8dCemWyj9EtZeSI/T8P/JzeX7eGMU/
zAP2fDkT/z8gXMTM1aa9PxAilYROlABAsPjtaognrz+amZmZmZkBQAAAAAAA
AAAAfPXH5g==
        "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZyb8AAAAAAAAAAEwtUbmo
jLy//r1l/G9Nsz/gZhYR4SyUvyKNpffqxsI/DF7kUqVEsz9mwIxWwVvLPxKS
xtUEK8Y/zzSLu1aw0T+pcG/t6YbRPzxA8wi3aNU/nT44sPQh2D8MtQc/CNXY
P49vTm8h494/UKepZHnz2z82sIdVbePiP1LqyNJiwt4/Ev17R7xk5j+MHJeK
I6DgP3rTGHGh8+k/zrFw2um14T8+7GiAOY7tP1gO4PXwoeI/bwIwd02Z8D/K
DnOgu2PjP+aFTIFrb/I/X9HJBuP64z820+psfUj0P4zeJfUWZ+Q/ZeRZN4gj
9j/gSPgBHqjkPzR1BtKP//c/8i9YrNW95D9AwTiol9v5Pyx8UW4yqOQ//m4Z
JaO2+z/2HALDP2fkP5WLtjm2j/0/iIqCICD74z8ne8Hi1WX/P/bGnOUMZOM/
b3PfVgScAEAtkls7VqLiP9H+sJ6rggFAtPeB6mK24T9lZmZmZmYCQJnHASWw
oOA/e8TG0A==
        "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ2T8iCq2GlSvWP/yhL1HH
IuA/pM7syg/52j8GJ5kM2pPjPyuBCTaHd98/pHvmfzEe5z+8UyOsTNLhPyD0
qMPsv+o/HyoTeQe/4z/T6VCHHnfuP0kuspDugOU/3il3C+cg8T97tTM7Exfn
PzIifjP8DvM/N13K+J2A6D/ymtSSyAT1P6AHAPTOvOk/YpGF6EEB9z8HS39n
/srqP17n1WhaA/k/gFIY95yq6z/UqWtMAQr7P/Hz0fszW+w/bIQKYSMU/T/o
nt7CZdzsP0aWmJurIP8/4bxTv+0t7T/VFo/VQZcAQLEhia6gT+0/ozU5Rkqe
AUBvPw2vbEHtP+A4I4/jpAJAE/AhSlkD7T9PSqBagqoDQOrLvG+Hlew/M3js
15uuBEAvLA1lMfjrP9p71ASmsAVAih+Rpaor6z+15e/2F7AGQFC4ybZfMOo/
RtRGJGqsB0AuNabu1QbpPwyqPKsWpQhA9pHELKuv5z+bmZmZmZkJQCIKrYaV
K+Y/w0295w==
        "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ2T+axwElsKDwPwiLQIbY
nuA/hIsN+Lpi8T8g4heiioHkPyBd/lhQDvI/ZpxzV9Ny6D+wHhA9FaPyP85D
4R+bcOw/TtJztLog8z80qcDpYTzwP6DYMRT+hvM//gRZ5JRE8j9+qIkZqdXz
P9wnU3vST/Q/kjq8BpIM9D9YLicNBV32P0DiMbmbK/Q/8mt77hVr+D9h0++4
tTL0P1YDAf7tePo/+SBVQdwh9D8wFY84doX8P/2RGkMY+fM/yO8tTZiP/j+B
PJRff7jzPx/ZYZgfSwBAyGs33TNg8z8dBIrYK0wBQAzoapVk8PI/Vn9ig2hK
AkBYRqvbTGnyP7W7DbNORQNAJHEQXjTL8T/fNxFHWTwEQEYdRf9uFvE/WZL4
KgUvBUCtWQSqXEvwP8aP4pvRHAZAMK9pPNLU7j/lLdNsQAUHQMzYeW8X6Ow/
DIOmSdbnB0CmRVtejtHqP07kgPgaxAhAYuzMqFKS6D+bmZmZmZkJQCIKrYaV
K+Y/Ssa4CQ==
        "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQbWIMApf338u4flUp7bO9hD+H9iKB4/tz1h28u7/o
i/2JfLsDzSVr9y/4Fvhof8ZXe51Z7uITwzfa+61qjlng/s3+1hsuixmMJ+1X
B1pyveb7bj9n/qXZF89esQ/4/1vHbN93+0OhaTa9FnftZz4QO6QS+sOe5UT/
Ded7j+yF9D5uTrv8w55vn4TKPK3n9mk+PSw7LX7aJ80yWMFb/9r+1UT+j09b
ftozrfZxcrv53j7iih/f0e0/7c+vZoqvkf5krzjHJs764k/76buS2V95fYHz
nwo4uB6u/AZXbyrpN+n+sh9w8xLYo0LEL/6C25fcJFfl8v0P3D1fL/iIO0v8
h7vX4f6O/xUzGRxg/klr2bRecjGjA8y/Sd8qVpWtYHKAhccnHRMn1pXMDrDw
UtnxONR2CYsDLDxnzQQBVgdYeAMAaR3AsQ==
        "]], 
       {GrayLevel[0]}}}},
    AlignmentPoint->Center,
    AspectRatio->Automatic,
    Axes->False,
    AxesLabel->None,
    AxesOrigin->Automatic,
    AxesStyle->{},
    Background->None,
    BaseStyle->{},
    BaselinePosition->Automatic,
    ColorOutput->Automatic,
    ContentSelectable->Automatic,
    CoordinatesToolOptions:>Automatic,
    DisplayFunction:>$DisplayFunction,
    Epilog->{},
    FormatType:>TraditionalForm,
    Frame->True,
    FrameLabel->FormBox[
      StyleBox[
      "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
       StripOnInput -> False], TraditionalForm],
    FrameStyle->{},
    FrameTicks->None,
    FrameTicksStyle->{},
    GridLines->None,
    GridLinesStyle->{},
    ImageMargins->0.,
    ImagePadding->All,
    ImageSize->Automatic,
    ImageSizeRaw->Automatic,
    LabelStyle->{},
    Method->Automatic,
    PlotLabel->None,
    PlotRange->All,
    PlotRangeClipping->False,
    PlotRangePadding->Automatic,
    PlotRegion->Automatic,
    PreserveImageOptions->Automatic,
    Prolog->{},
    RotateLabel->True,
    Ticks->Automatic,
    TicksStyle->{}], ",", 
   GraphicsBox[
    {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
     0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
     {RGBColor[0.0692454, 0.645548, 0.968322], 
      DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
     {GrayLevel[0], PointSize[Medium], Opacity[1], 
      PointBox[{{0.4, 
        0.34641016151377546`}, {-0.2, 
         0}, {-0.9, -0.17320508075688773`}, {-0.6000000000000001, \
-1.0392304845413265`}, {0.1, -0.5196152422706632}, {-0.30000000000000004`, 
        1.2124355652982142`}, {0.8, -0.34641016151377546`}, {0.4, 
        1.0392304845413265`}, {2.2, 0.}, {3.3, -0.8660254037844386}, {3.2, 
        0.6928203230275509}, {2.3, 0.5196152422706632}, {
        3.9, -0.17320508075688773`}, {2.7, 1.2124355652982142`}}], 
      {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAANDMzMzMz47+axwElsKDwvyyy8XOv
rNy/JiptVlFO7r8njaCK0bDSv4TaCnSzlOu/OETmnDHwwL8iEH7K+BXpv4Dk
pCflgJ8/Ag6CQnTT5r/sIUUBNDTJP9m/GNJYzuS/VL1l62Vo1z9yL6vZuAfj
v0avsSObLuE/IAV0koWA4b8M9aRsXbnmP7RSgY6OOeC/GiO/BYlR7D/yJCOV
Amfev80YPgMT+vA/YpUMpNHd3L/1Qet9G8/zP/xZOupa2Nu/lDs2P12m9j8E
ry0jKVfbv2yMX2VWfvk/AivR24Ba279gtFithFX8P0ZOGU5g4tu//G63P2Yq
/z+YbfVhf+7cv1/stz69/QBAYneQ009+3r8oa/RloWMCQDmkZr9+SOC/aTr/
ryHGA0DmQOtnt5Lhv9tFwwOCJAVASxNDrCId47/GWXZ4CH4GQNjdKkXv5uS/
3NM9uP3RB0A2IDpKKu/mv76ydWGtHwlA0hTGsr806b9nZmZmZmYKQKpMWOh6
tuu/OETHMA==
        "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZuT+axwElsKDgv/nTzJxN
Qc8/9G2LUL8N279C52IktyDZP9MFWd9HR9W/VGZ4esZw4T+SigjfFOLPv8p2
Q7f9buY/unIP3Lcbxr+wSDgQW4jrP7TQbexQgbq/JNOh8hRd8D/gdaRPZlal
v1SQDU/UAPM/AFZ6yxMUij8VMlAQBa71P0AqBQLYgK4/Ps2vpjtj+D9weGOK
LEi5P1R7r0AIH/s/kOCT0TiqwD+PWjmO99/9PyB79A57sMM/Cp3RwklSAED4
JicrQrXFP1m0EhWytQFAFIqB8nu3xj8ZLK0peBkDQOQWWWGftsY/Q52uOt98
BEDMKbbsrLLFPwG1jrQq3wVA0PAVgi6swz/dKD6anj8HQEQ8Oz43pMA/rtPl
6H+dCEB4Hmq2xTi5P2EOIfsU+AlA0IuhV1Nbrj87nn7rpU4LQAB8ELl0Y4k/
UvQT9nygDEAI5bDAGYmlv3347tjm7A1AtGK8B+Odur8zMzMzMzMPQCIKrYaV
K8a/TRu4Bw==
        "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ6T8iCq2GlSvWv+/Jho6S
Jes/qvqIpoRT07/wFyAUtMHsP2FGJHdAoNC/1ChDfSNt7j9wVhCDbybMv6zM
/P5+E/A/gjoIC4pbx7/8xueRLPfwP6g6p4Nb4sK/wLccqSHh8T+YiaQdh3q9
v0hpYyHi0PI/hOPmYOrctb85Qm/E7sXzP5AH947W362/p+tejMW/9D/AZ+8i
bW6hv0vGuejhvfU/QOu1q8qwib+uk8YEvb/2PwBz1PwLCHk/+gAWD87E9z+Q
e8rgO2iWP74dG4KKzPg/MCvJ2NnIoT/MOattZtb5P+ACrk2326Y/Yyo+wdTh
+j9gddwhBWqqPxCRx5ZH7vs/sGarU+BxrD8ibQF+MPv8P8BTSA018qw/vAQA
yAAI/j/A8xI3v+qrP38L59IpFP8/qGy+mwpcqT+b7cuqjg8AQHjvJJ5yR6U/
PkuY1SaUAEAALeoBQ16fP0ITGg+XFwFA0Kk1gR4skT+amZmZmZkBQAAAAAAA
AJA8lTbEXg==
        "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAzczMzMzM7L8iCq2GlSvGvyxjQxOp
9ei/NK7B58g4ub+KyxGrsQPlvwjZBQfOFZ6/SmFcd/744L8wq7Kk4XKhPycS
2fVor9m/WGlDYEV7tz/6YsVpCUTRvx6/SdIaXMI/qBIx3Ktowb8UDy26nTTI
PwCY3tuzZTO/amieWxFEzT8Eu/ejoIvBP9Sahizjw9A/RveaG/2m0T+rWR64
vH7SP17648r2mdo/C03PVqrR0z9B7njk/szhP70Z1zL4u9Q//T/9yCVR5j+z
xOP5KT3VP3WA/J4K1+o/a7cKH/tU1T9nQ9YFR1zvP1UwKP9eA9U/vIskezrv
8T+j0pPngEjUP+IHIASYLfQ/EMcm/8Mk0z+UAeJbC2j2P/mdnxHDmNE/aAsy
1GWd+D/ouv96oErPP2z9NHN7zPo/gp8dCemWyj9EtZeSI/T8P/JzeX7eGMU/
zAP2fDkT/z8gXMTM1aa9PxAilYROlABAsPjtaognrz+amZmZmZkBQAAAAAAA
AAAAfPXH5g==
        "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZyb8AAAAAAAAAAEwtUbmo
jLy//r1l/G9Nsz/gZhYR4SyUvyKNpffqxsI/DF7kUqVEsz9mwIxWwVvLPxKS
xtUEK8Y/zzSLu1aw0T+pcG/t6YbRPzxA8wi3aNU/nT44sPQh2D8MtQc/CNXY
P49vTm8h494/UKepZHnz2z82sIdVbePiP1LqyNJiwt4/Ev17R7xk5j+MHJeK
I6DgP3rTGHGh8+k/zrFw2um14T8+7GiAOY7tP1gO4PXwoeI/bwIwd02Z8D/K
DnOgu2PjP+aFTIFrb/I/X9HJBuP64z820+psfUj0P4zeJfUWZ+Q/ZeRZN4gj
9j/gSPgBHqjkPzR1BtKP//c/8i9YrNW95D9AwTiol9v5Pyx8UW4yqOQ//m4Z
JaO2+z/2HALDP2fkP5WLtjm2j/0/iIqCICD74z8ne8Hi1WX/P/bGnOUMZOM/
b3PfVgScAEAtkls7VqLiP9H+sJ6rggFAtPeB6mK24T9lZmZmZmYCQJnHASWw
oOA/e8TG0A==
        "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ2T8iCq2GlSvWP/yhL1HH
IuA/pM7syg/52j8GJ5kM2pPjPyuBCTaHd98/pHvmfzEe5z+8UyOsTNLhPyD0
qMPsv+o/HyoTeQe/4z/T6VCHHnfuP0kuspDugOU/3il3C+cg8T97tTM7Exfn
PzIifjP8DvM/N13K+J2A6D/ymtSSyAT1P6AHAPTOvOk/YpGF6EEB9z8HS39n
/srqP17n1WhaA/k/gFIY95yq6z/UqWtMAQr7P/Hz0fszW+w/bIQKYSMU/T/o
nt7CZdzsP0aWmJurIP8/4bxTv+0t7T/VFo/VQZcAQLEhia6gT+0/ozU5Rkqe
AUBvPw2vbEHtP+A4I4/jpAJAE/AhSlkD7T9PSqBagqoDQOrLvG+Hlew/M3js
15uuBEAvLA1lMfjrP9p71ASmsAVAih+Rpaor6z+15e/2F7AGQFC4ybZfMOo/
RtRGJGqsB0AuNabu1QbpPwyqPKsWpQhA9pHELKuv5z+bmZmZmZkJQCIKrYaV
K+Y/w0295w==
        "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAmpmZmZmZ2T+axwElsKDwPwiLQIbY
nuA/hIsN+Lpi8T8g4heiioHkPyBd/lhQDvI/ZpxzV9Ny6D+wHhA9FaPyP85D
4R+bcOw/TtJztLog8z80qcDpYTzwP6DYMRT+hvM//gRZ5JRE8j9+qIkZqdXz
P9wnU3vST/Q/kjq8BpIM9D9YLicNBV32P0DiMbmbK/Q/8mt77hVr+D9h0++4
tTL0P1YDAf7tePo/+SBVQdwh9D8wFY84doX8P/2RGkMY+fM/yO8tTZiP/j+B
PJRff7jzPx/ZYZgfSwBAyGs33TNg8z8dBIrYK0wBQAzoapVk8PI/Vn9ig2hK
AkBYRqvbTGnyP7W7DbNORQNAJHEQXjTL8T/fNxFHWTwEQEYdRf9uFvE/WZL4
KgUvBUCtWQSqXEvwP8aP4pvRHAZAMK9pPNLU7j/lLdNsQAUHQMzYeW8X6Ow/
DIOmSdbnB0CmRVtejtHqP07kgPgaxAhAYuzMqFKS6D+bmZmZmZkJQCIKrYaV
K+Y/Ssa4CQ==
        "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQbWIMApf338u4flUp7bO9hD+H9iKB4/tz1h28u7/o
i/2JfLsDzSVr9y/4Fvhof8ZXe51Z7uITwzfa+61qjlng/s3+1hsuixmMJ+1X
B1pyveb7bj9n/qXZF89esQ/4/1vHbN93+0OhaTa9FnftZz4QO6QS+sOe5UT/
Ded7j+yF9D5uTrv8w55vn4TKPK3n9mk+PSw7LX7aJ80yWMFb/9r+1UT+j09b
ftozrfZxcrv53j7iih/f0e0/7c+vZoqvkf5krzjHJs764k/76buS2V95fYHz
nwo4uB6u/AZXbyrpN+n+sh9w8xLYo0LEL/6C25fcJFfl8v0P3D1fL/iIO0v8
h7vX4f6O/xUzGRxg/klr2bRecjGjA8y/Sd8qVpWtYHKAhccnHRMn1pXMDrDw
UtnxONR2CYsDLDxnzQQBVgdYeAMAaR3AsQ==
        "]], 
       {GrayLevel[0]}}}},
    AlignmentPoint->Center,
    AspectRatio->Automatic,
    Axes->False,
    AxesLabel->None,
    AxesOrigin->Automatic,
    AxesStyle->{},
    Background->None,
    BaseStyle->{},
    BaselinePosition->Automatic,
    ColorOutput->Automatic,
    ContentSelectable->Automatic,
    CoordinatesToolOptions:>Automatic,
    DisplayFunction:>$DisplayFunction,
    Epilog->{},
    FormatType:>TraditionalForm,
    Frame->True,
    FrameLabel->FormBox[
      StyleBox[
      "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
       StripOnInput -> False], TraditionalForm],
    FrameStyle->{},
    FrameTicks->None,
    FrameTicksStyle->{},
    GridLines->None,
    GridLinesStyle->{},
    ImageMargins->0.,
    ImagePadding->All,
    ImageSize->Automatic,
    ImageSizeRaw->Automatic,
    LabelStyle->{},
    Method->Automatic,
    PlotLabel->None,
    PlotRange->All,
    PlotRangeClipping->False,
    PlotRangePadding->Automatic,
    PlotRegion->Automatic,
    PreserveImageOptions->Automatic,
    Prolog->{},
    RotateLabel->True,
    Ticks->Automatic,
    TicksStyle->{}]}], "}"}]], "Output",
 ImageSize->{389, 145},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[116]=",
 CellID->2059958637]
}, Open  ]],

Cell["\<\
For some purposes it is convenient to specify the points used to illustrate \
the function. The points will be scaled to fit in the set frames but the \
general configuration will be preserved.\
\>", "Text",
 CellID->1954677637],

Cell["An example with the points in specified locations.", "MathCaption",
 CellID->1487800726],

Cell[BoxData[
 RowBox[{
  RowBox[{"cubeSet", "=", " ", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"x", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"x", ",", 
          RowBox[{
           SuperscriptBox["x", "3"], "/", "5"}]}], "}"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", 
        RowBox[{"-", "3."}], ",", "3"}], "}"}]}], "]"}], "]"}]}], 
  ";"}]], "Input",
 CellLabel->"In[117]:=",
 CellID->62213253],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionArrowDiagram", "[", 
  RowBox[{
   RowBox[{"cubeSet", "[", 
    RowBox[{"[", "1", "]"}], "]"}], ",", 
   RowBox[{"cubeSet", "[", 
    RowBox[{"[", "2", "]"}], "]"}], ",", "\"\<OneToOne\>\""}], "]"}]], "Input",
 CellLabel->"In[118]:=",
 CellID->167216214],

Cell[BoxData[
 GraphicsBox[
  {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[GrayLevel[
   0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
   {RGBColor[0.0692454, 0.645548, 0.968322], 
    DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
   {GrayLevel[0], PointSize[Medium], Opacity[1], 
    PointBox[{{-1., 0.}, {-0.6666666666666666, 0.}, {-0.3333333333333333, 
     0.}, {0., 0.}, {0.3333333333333333, 0.}, {0.6666666666666666, 0.}, {1., 
     0.}, {2.514357068821368, -1.3112359141823067`}, {
     2.676238045880912, -0.3885143449429056}, {
     2.838119022940456, -0.0485642931178632}, {3., 0.}, {3.161880977059544, 
     0.0485642931178632}, {3.323761954119088, 0.3885143449429056}, {
     3.485642931178632, 1.3112359141823067`}}], 
    {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAA8L8AAAAAAAAAAOP7ND96
l+q/lieeNMoLmT9w76cqRSflv+R+AjU2GKU/vGYe7YZk379E6KgCZ7KpP2RQ
Esa0dtS/UCOoXQZSqj/oqDbLwBXDv5j/8JO/9qY/gBw21auSlT/Qhd8ntUSf
P/hnxppKX8g/gOWF6utigT/I5cgnz/LWP8AUY/lDwJW/POWwNHnO4D8Ysf2l
KACuv0BrMDszFOY/vH3Hlpl+ur/yAHh9yUfrP5glnL7D8sO/fonLuDwz8D/s
ctHJ4JbLv3qFPdLFtvI/I6XEAcsT0r+GR3iVKi31Pwebbvwr0Na/qFT/hRyV
9z8Q6mYLEP7bv4jJ7dFU7fk/nKI61VvN4L+mZl//lDT8P57j+EeU0eO/BkBs
ladp/j8Ch56gFwrnv3izLmCwRQBAFGnIYjB16r+utmF4T0wBQAAiITsOEe6/
aSyNuiVIAkD79d964+3wvyfA1IatOANA70Mhwavp8r/+big9Zx0EQAgPjILS
+vS/IjLGgQ==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV5b8AAAAAAAAAABXxDlAj
yuC/nXMfJ4B4rT/ZDAx1X1XYv+wjDGYNzbs/rhy6E7rlzb8Rvt2bvJXDPzjT
D47oxrW/7FUnor1oyD8o9ZTj+KOwP9gfInv6XMw/zE1C5mawyz+MuCYPWnDP
P7KFBhTCltc/exJRS53Q0D+VKL4/v6/gP1od4Ts5d9E/wwxk2ZuW5T9t4C5y
qKvRP/0C3QJdfeo/rj/iG89t0T9PJKD2aGHvP8q7IgrOvdA/wMd6rxMg8j+O
mVtABTjPP9yfOluBi/Q/gXKTQw0SzD9S2IPTtPH2P59oXsNfC8g/NglXM2hR
+T8SGSqgHybDP5J+rAhZqfs/oH1opsvJuj+0EYv/SPj9P3Be1DYAK6s/I913
RX8eAEAABJC8HJp0v0cclsUiOwFAdFKS/7XLsb8TSbL4d1ECQDzatBL988G/
zVDqL+tgA0CPGNQeacrLv4JXRGPsaARAwHUzb/Mx07+p9Bp+72gFQAigY0Vr
3di/UKO5Fg==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV1b8AAAAAAAAAADrK37XA
Ucq/OzYTJlZAsT9wbtQGWDGzvw60Gc6Sf8A/OM5TjM/erT8/xOnRCZrHP8q5
lCJR1sg/V5JDPsvrzT/bLz8u+jzVP93D+1i+uNE/XUJHyaAs3j+FpCgJGBTU
P7w9iUevmuM/E3SMjLIG1j+QLBguNCnoP5nAJlSFj9c/XiQXJHS/7D8MgHLx
v63YP2U3mw2ArfA/mAv+hMpg2T9H2NMas/zyP52FAA9GqNk/d7oZiRlM9T+8
58GhDITZP0ckjox5mvc/y/q6dTH02D8qYY/kmeb5P4mKYt8A+dc/6+TugUIv
/D9+Pq0mAJPWP6NchSw9c/4/sJVVQO3C1D8raeYTq1gAQHaKEWm+idI/X8YU
ay50AUC8s9ZFQ9HPP9mDRS2SiwJAc4IFKvjByT9woQccQp4DQDVxF5zS6MI/
G9mpd6yrBEB63byn6ZK2P8srkExCswVAhBAesFE/lz9Veg2/d7QGQAmgY0Vr
3ai/FRi2dg==
      "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQjQ4W6O0p61i9z/5FxJK6Cyc32JedCitz5TxvL5B3
JfmFyAH7/ZKsleKd1+1buJNM33Aft1+40sL//+b79hlu/Isd3c7aZ7xsSm9i
eGJ/InJSjG/CRfvqI2fMH5i+sH9z7+8G9j2X7eclahbdLX9jf04kvYJ/61X7
K9o370knfbBX6Lu8K8Lruv0/yacLgns+2RsteZDDU33D3k/p89v8nV/sn629
+7DD7ab9nIrz2p4vvtnznJ2imLTmpn1Ku8GVbeI/4fxNd3mEJrj9hqtneiYV
v6b4L9w83Stqik/m/ofbN2N56WKORwwOMPc8PMfzqf0TowPMvctc5dPu/2dy
gPnnzCsJsQZOFgeYf0/E3Be4y8fqAAuPyMmzVmTzsznAwqtbzmvPYi52B1h4
QkKZwwEW3gAvYa5l
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV1T8AAAAAAAAAAG38TZak
ftw/lXs110pTsD8I8wW8zerhPyNQHvN7VL8/29w/gCqr5T/IUtqYzn3GPy57
kwZrfuk/efraB7igzD/iOua3h2LtP7TvKGDcB9E/5PN6Bbiq8D9TGdNY92PT
P8HCZM6FqvI/iLVBZGxj1T9gQaamna/0P6dbgCEsBdc/zaG+L+249j9YyE7o
WEjYPzYNS85fxfg/NFC9Pkcs2T/yGMs839P6P3sUKTR+sNk/T1RHH1Tj/D9c
smeht9TZPx6Xi5em8v4/AWEATuCY2T9tNdNsX4AAQNu9XvoX/dg/t2Ev4EKG
AUDC3flOsQHYP9+gqLByigJA65N4sDGn1j+XNDPQZIwDQGIz6vhQ7tQ/7cQp
YZCLBECzWTkW+dfSP71R7P5thwVAL4UMjkVl0D8gGLcFeH8GQNKlrM4FL8s/
mE2L2SpzB0BjbcrzW+DEP0oRBCwFYghAsC1BkczDuz+rhfJAiEsJQAmgY0Vr
3ag/ur262g==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV5T8AAAAAAAAAAL38vW0F
deg/Afi46WDXsj+na7F1Uq/rPyjKohRKQ8I/6qDz/YUC7z/2WObOG4LKPwNS
fTBuNvE/cZv70eIR0T8VmknZQvbyP6Kjk3sdktQ/wHoEW9O/9D+RYWha4r/X
PzHAJ+0skvY/IBd9pYGZ2j9cQ9MdWGz4P+P+xTl4Hd0/mi0YVVlN+j/2Z5Jn
cErfPxrDd1oxNPw/Do7sU6GP4D8Q307c3R/+PyZvgxx7TeE/gyNz/KwHAECo
5n3RYN7hP8SNb2TPAAFARdH4kAVC4j/Y+Vj10foBQAoKCXw0eOI/kU97BzD1
AkDaB8nS0IDiP+UPlcJk7wNAGgWaA9Zb4j/nFFRk6+gEQASZkK1XCeI/Ka7G
hj/hBUDPeQuWgYnhP0+5m2bd1wZAgu15kZfc4D98cAwpQswHQFE8Xl/1AuA/
49VKIey9CEBYej7zHPrdP5PlTxVbrAlANKKFrtuW2z9VC+WBEJcKQAigY0Vr
3dg/iRa38A==
      "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAA8D8AAAAAAAAAAMYDci4u
IfE/6ichcK5Quz9Wquny81XyP2Y8aDrD48o/jibKd62d8z987gxVn9XTP+no
INms9/Q//0Fa1wD82T+FvumAOmP2P56TpsbC4d8/COx3iJXf9z++4r4H4sHi
P5+U0B70a/k/OP28yINv5T81HsDzgwf7P8amntpa+Oc/XsFxp2qx/D9Ef0D3
DlvqP/oOTj7GaP4/np22FFyW7D9feHHMVhYAQHPWOxETqe4/pcLKdxj+AEDk
lAQqDUnwP/Ya7igt6wFAPABgMTco8T8/wl4UF90CQNv4izWR8fE/2Skn3lXT
A0CVgUNgsKTyP2pq9N1mzQRAttvQpjVB8z9u6mlkxcoFQEZhe/zNxvM/SmuI
AevKBkBmcJh+MjX0Px4iA8xPzQdAJQYomiiM9D/jAl2patEIQOIT6SqCy/Q/
SfqnlrHWCUCAEdWTHfP0P+58v/GZ3ApADM/10OUC9T8BkdfCmOILQAkPjILS
+vQ/dty+5Q==
      "]], 
     {GrayLevel[0]}}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->FormBox[
    StyleBox[
    "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
     StripOnInput -> False], TraditionalForm],
  FrameStyle->{},
  FrameTicks->None,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 259},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[118]=",
 CellID->2140938772]
}, Open  ]],

Cell[TextData[{
 "This could be combined with the ",
 Cell[BoxData[
  ButtonBox["FunctionTable",
   BaseStyle->"Link",
   ButtonData->"paclet:ClassroomUtilities/ref/FunctionTable"]], 
  "InlineFormula"],
 " to build an exercise."
}], "MathCaption",
 CellID->549119355],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Grid", "[", 
  RowBox[{"{", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"FunctionArrowDiagram", "[", 
      RowBox[{
       RowBox[{"cubeSet", "[", 
        RowBox[{"[", "1", "]"}], "]"}], ",", 
       RowBox[{"cubeSet", "[", 
        RowBox[{"[", "2", "]"}], "]"}], ",", "\"\<OneToOne\>\"", ",", 
       RowBox[{"ImageSize", "\[Rule]", " ", "Medium"}]}], "]"}], ",", 
     RowBox[{"FunctionTable", "[", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "3"], "/", "5"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", 
         RowBox[{"-", "3"}], ",", "3"}], "}"}]}], "]"}]}], "}"}], "}"}], 
  "]"}]], "Input",
 CellLabel->"In[119]:=",
 CellID->1311768326],

Cell[BoxData[
 TagBox[GridBox[{
    {
     GraphicsBox[
      {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[
       GrayLevel[0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
       {RGBColor[0.0692454, 0.645548, 0.968322], 
        DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
       {GrayLevel[0], PointSize[Medium], Opacity[1], 
        PointBox[{{-1., 0.}, {-0.6666666666666666, 0.}, {-0.3333333333333333, 
         0.}, {0., 0.}, {0.3333333333333333, 0.}, {0.6666666666666666, 0.}, {
         1., 0.}, {2.514357068821368, -1.3112359141823067`}, {
         2.676238045880912, -0.3885143449429056}, {
         2.838119022940456, -0.0485642931178632}, {3., 0.}, {
         3.161880977059544, 0.0485642931178632}, {3.323761954119088, 
         0.3885143449429056}, {3.485642931178632, 1.3112359141823067`}}], 
        {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAA8L8AAAAAAAAAAOP7ND96
l+q/lieeNMoLmT9w76cqRSflv+R+AjU2GKU/vGYe7YZk379E6KgCZ7KpP2RQ
Esa0dtS/UCOoXQZSqj/oqDbLwBXDv5j/8JO/9qY/gBw21auSlT/Qhd8ntUSf
P/hnxppKX8g/gOWF6utigT/I5cgnz/LWP8AUY/lDwJW/POWwNHnO4D8Ysf2l
KACuv0BrMDszFOY/vH3Hlpl+ur/yAHh9yUfrP5glnL7D8sO/fonLuDwz8D/s
ctHJ4JbLv3qFPdLFtvI/I6XEAcsT0r+GR3iVKi31Pwebbvwr0Na/qFT/hRyV
9z8Q6mYLEP7bv4jJ7dFU7fk/nKI61VvN4L+mZl//lDT8P57j+EeU0eO/BkBs
ladp/j8Ch56gFwrnv3izLmCwRQBAFGnIYjB16r+utmF4T0wBQAAiITsOEe6/
aSyNuiVIAkD79d964+3wvyfA1IatOANA70Mhwavp8r/+big9Zx0EQAgPjILS
+vS/IjLGgQ==
          "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV5b8AAAAAAAAAABXxDlAj
yuC/nXMfJ4B4rT/ZDAx1X1XYv+wjDGYNzbs/rhy6E7rlzb8Rvt2bvJXDPzjT
D47oxrW/7FUnor1oyD8o9ZTj+KOwP9gfInv6XMw/zE1C5mawyz+MuCYPWnDP
P7KFBhTCltc/exJRS53Q0D+VKL4/v6/gP1od4Ts5d9E/wwxk2ZuW5T9t4C5y
qKvRP/0C3QJdfeo/rj/iG89t0T9PJKD2aGHvP8q7IgrOvdA/wMd6rxMg8j+O
mVtABTjPP9yfOluBi/Q/gXKTQw0SzD9S2IPTtPH2P59oXsNfC8g/NglXM2hR
+T8SGSqgHybDP5J+rAhZqfs/oH1opsvJuj+0EYv/SPj9P3Be1DYAK6s/I913
RX8eAEAABJC8HJp0v0cclsUiOwFAdFKS/7XLsb8TSbL4d1ECQDzatBL988G/
zVDqL+tgA0CPGNQeacrLv4JXRGPsaARAwHUzb/Mx07+p9Bp+72gFQAigY0Vr
3di/UKO5Fg==
          "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV1b8AAAAAAAAAADrK37XA
Ucq/OzYTJlZAsT9wbtQGWDGzvw60Gc6Sf8A/OM5TjM/erT8/xOnRCZrHP8q5
lCJR1sg/V5JDPsvrzT/bLz8u+jzVP93D+1i+uNE/XUJHyaAs3j+FpCgJGBTU
P7w9iUevmuM/E3SMjLIG1j+QLBguNCnoP5nAJlSFj9c/XiQXJHS/7D8MgHLx
v63YP2U3mw2ArfA/mAv+hMpg2T9H2NMas/zyP52FAA9GqNk/d7oZiRlM9T+8
58GhDITZP0ckjox5mvc/y/q6dTH02D8qYY/kmeb5P4mKYt8A+dc/6+TugUIv
/D9+Pq0mAJPWP6NchSw9c/4/sJVVQO3C1D8raeYTq1gAQHaKEWm+idI/X8YU
ay50AUC8s9ZFQ9HPP9mDRS2SiwJAc4IFKvjByT9woQccQp4DQDVxF5zS6MI/
G9mpd6yrBEB63byn6ZK2P8srkExCswVAhBAesFE/lz9Veg2/d7QGQAmgY0Vr
3ai/FRi2dg==
          "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQjQ4W6O0p61i9z/5FxJK6Cyc32JedCitz5TxvL5B3
JfmFyAH7/ZKsleKd1+1buJNM33Aft1+40sL//+b79hlu/Isd3c7aZ7xsSm9i
eGJ/InJSjG/CRfvqI2fMH5i+sH9z7+8G9j2X7eclahbdLX9jf04kvYJ/61X7
K9o370knfbBX6Lu8K8Lruv0/yacLgns+2RsteZDDU33D3k/p89v8nV/sn629
+7DD7ab9nIrz2p4vvtnznJ2imLTmpn1Ku8GVbeI/4fxNd3mEJrj9hqtneiYV
v6b4L9w83Stqik/m/ofbN2N56WKORwwOMPc8PMfzqf0TowPMvctc5dPu/2dy
gPnnzCsJsQZOFgeYf0/E3Be4y8fqAAuPyMmzVmTzsznAwqtbzmvPYi52B1h4
QkKZwwEW3gAvYa5l
          "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV1T8AAAAAAAAAAG38TZak
ftw/lXs110pTsD8I8wW8zerhPyNQHvN7VL8/29w/gCqr5T/IUtqYzn3GPy57
kwZrfuk/efraB7igzD/iOua3h2LtP7TvKGDcB9E/5PN6Bbiq8D9TGdNY92PT
P8HCZM6FqvI/iLVBZGxj1T9gQaamna/0P6dbgCEsBdc/zaG+L+249j9YyE7o
WEjYPzYNS85fxfg/NFC9Pkcs2T/yGMs839P6P3sUKTR+sNk/T1RHH1Tj/D9c
smeht9TZPx6Xi5em8v4/AWEATuCY2T9tNdNsX4AAQNu9XvoX/dg/t2Ev4EKG
AUDC3flOsQHYP9+gqLByigJA65N4sDGn1j+XNDPQZIwDQGIz6vhQ7tQ/7cQp
YZCLBECzWTkW+dfSP71R7P5thwVAL4UMjkVl0D8gGLcFeH8GQNKlrM4FL8s/
mE2L2SpzB0BjbcrzW+DEP0oRBCwFYghAsC1BkczDuz+rhfJAiEsJQAmgY0Vr
3ag/ur262g==
          "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV5T8AAAAAAAAAAL38vW0F
deg/Afi46WDXsj+na7F1Uq/rPyjKohRKQ8I/6qDz/YUC7z/2WObOG4LKPwNS
fTBuNvE/cZv70eIR0T8VmknZQvbyP6Kjk3sdktQ/wHoEW9O/9D+RYWha4r/X
PzHAJ+0skvY/IBd9pYGZ2j9cQ9MdWGz4P+P+xTl4Hd0/mi0YVVlN+j/2Z5Jn
cErfPxrDd1oxNPw/Do7sU6GP4D8Q307c3R/+PyZvgxx7TeE/gyNz/KwHAECo
5n3RYN7hP8SNb2TPAAFARdH4kAVC4j/Y+Vj10foBQAoKCXw0eOI/kU97BzD1
AkDaB8nS0IDiP+UPlcJk7wNAGgWaA9Zb4j/nFFRk6+gEQASZkK1XCeI/Ka7G
hj/hBUDPeQuWgYnhP0+5m2bd1wZAgu15kZfc4D98cAwpQswHQFE8Xl/1AuA/
49VKIey9CEBYej7zHPrdP5PlTxVbrAlANKKFrtuW2z9VC+WBEJcKQAigY0Vr
3dg/iRa38A==
          "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAA8D8AAAAAAAAAAMYDci4u
IfE/6ichcK5Quz9Wquny81XyP2Y8aDrD48o/jibKd62d8z987gxVn9XTP+no
INms9/Q//0Fa1wD82T+FvumAOmP2P56TpsbC4d8/COx3iJXf9z++4r4H4sHi
P5+U0B70a/k/OP28yINv5T81HsDzgwf7P8amntpa+Oc/XsFxp2qx/D9Ef0D3
DlvqP/oOTj7GaP4/np22FFyW7D9feHHMVhYAQHPWOxETqe4/pcLKdxj+AEDk
lAQqDUnwP/Ya7igt6wFAPABgMTco8T8/wl4UF90CQNv4izWR8fE/2Skn3lXT
A0CVgUNgsKTyP2pq9N1mzQRAttvQpjVB8z9u6mlkxcoFQEZhe/zNxvM/SmuI
AevKBkBmcJh+MjX0Px4iA8xPzQdAJQYomiiM9D/jAl2patEIQOIT6SqCy/Q/
SfqnlrHWCUCAEdWTHfP0P+58v/GZ3ApADM/10OUC9T8BkdfCmOILQAkPjILS
+vQ/dty+5Q==
          "]], 
         {GrayLevel[0]}}}},
      AlignmentPoint->Center,
      AspectRatio->Automatic,
      Axes->False,
      AxesLabel->None,
      AxesOrigin->Automatic,
      AxesStyle->{},
      Background->None,
      BaseStyle->{},
      BaselinePosition->Automatic,
      ColorOutput->Automatic,
      ContentSelectable->Automatic,
      CoordinatesToolOptions:>Automatic,
      DisplayFunction:>$DisplayFunction,
      Epilog->{},
      FormatType:>TraditionalForm,
      Frame->True,
      FrameLabel->FormBox[
        StyleBox[
        "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
         StripOnInput -> False], TraditionalForm],
      FrameStyle->{},
      FrameTicks->None,
      FrameTicksStyle->{},
      GridLines->None,
      GridLinesStyle->{},
      ImageMargins->0.,
      ImagePadding->All,
      ImageSize->Medium,
      ImageSizeRaw->Automatic,
      LabelStyle->{},
      Method->Automatic,
      PlotLabel->None,
      PlotRange->All,
      PlotRangeClipping->False,
      PlotRangePadding->Automatic,
      PlotRegion->Automatic,
      PreserveImageOptions->Automatic,
      Prolog->{},
      RotateLabel->True,
      Ticks->Automatic,
      TicksStyle->{}], 
     TagBox[GridBox[{
        {
         ItemBox[
          TagBox[
           FormBox["x",
            TraditionalForm],
           TraditionalForm,
           Editable->True],
          Alignment->Center,
          BaseStyle->{FontWeight -> "Bold"},
          StripOnInput->False], 
         ItemBox[
          
          InterpretationBox["\<\"\\!\\(\\*TagBox[\\(TraditionalForm\\`\\\"\\\\\
[ScriptF]\\\"\\), TraditionalForm, Rule[Editable, \
True]]\\)(\\!\\(\\*TagBox[\\(TraditionalForm\\`x\\), TraditionalForm, \
Rule[Editable, True]]\\)) = \
\\!\\(\\*TagBox[\\(TraditionalForm\\`x\\^3\\/5\\), TraditionalForm, \
Rule[Editable, True]]\\)\"\>",
           StringForm["`3`(`2`) = `1`", 
            TraditionalForm[Rational[1, 5] $CellContext`x^3], 
            TraditionalForm[$CellContext`x], 
            TraditionalForm["\[ScriptF]"]],
           Editable->False],
          Alignment->Center,
          BaseStyle->{FontWeight -> "Bold"},
          StripOnInput->False]},
        {
         RowBox[{"-", "3"}], 
         TagBox[
          InterpretationBox["\<\"-5.4\"\>",
           -5.4,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {
         RowBox[{"-", "2"}], 
         TagBox[
          InterpretationBox["\<\"-1.6\"\>",
           -1.6,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {
         RowBox[{"-", "1"}], 
         TagBox[
          InterpretationBox["\<\"-0.2\"\>",
           -0.2,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {"0", 
         TagBox[
          InterpretationBox["\<\"0.\"\>",
           0.,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {"1", 
         TagBox[
          InterpretationBox["\<\"0.2\"\>",
           0.2,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {"2", 
         TagBox[
          InterpretationBox["\<\"1.6\"\>",
           1.6,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {"3", 
         TagBox[
          InterpretationBox["\<\"5.4\"\>",
           5.4,
           AutoDelete->True],
          NumberForm[#, 5]& ]}
       },
       AutoDelete->False,
       BaseStyle->{"Menu"},
       GridBoxAlignment->{"Columns" -> {Left, "."}, "Rows" -> {{Automatic}}},
       GridBoxBackground->{"Columns" -> {{
            RGBColor[1, 1, 0.85]}}},
       GridBoxDividers->{"Columns" -> {
           AbsoluteThickness[1.5], None, 
           AbsoluteThickness[1.5]}, "Rows" -> {{True}}, 
         "RowsIndexed" -> {2 -> Directive[
             AbsoluteThickness[1.5]], 1 -> Directive[
             AbsoluteThickness[1.5]], -1 -> Directive[
             AbsoluteThickness[1.5]]}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
       GridBoxSpacings->{"Columns" -> {{1}}, "Rows" -> {{0.62}}}],
      "Grid"]}
   },
   AutoDelete->False,
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Grid"]], "Output",
 ImageSize->{478, 261},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[119]=",
 CellID->1669435519]
}, Open  ]],

Cell[TextData[{
 "Unlabeled axes can be included to assist in understanding. The second \
ellipse is centered on the point {3,0} so you will need to add the \"axis\" \
there using ",
 ButtonBox["Line",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Line"],
 "."
}], "MathCaption",
 CellID->1659794903],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Grid", "[", 
  RowBox[{"{", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"FunctionArrowDiagram", "[", 
      RowBox[{
       RowBox[{"cubeSet", "[", 
        RowBox[{"[", "1", "]"}], "]"}], ",", 
       RowBox[{"cubeSet", "[", 
        RowBox[{"[", "2", "]"}], "]"}], ",", "\"\<OneToOne\>\"", ",", 
       RowBox[{"ImageSize", "\[Rule]", " ", "Medium"}], ",", 
       RowBox[{"Axes", "\[Rule]", " ", "True"}], ",", 
       RowBox[{"Epilog", "\[Rule]", " ", 
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"3", ",", 
             RowBox[{"-", "1.75"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"3", ",", "1.75"}], "}"}]}], "}"}], "]"}]}]}], "]"}], ",", 
     RowBox[{"FunctionTable", "[", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "3"], "/", "5"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", 
         RowBox[{"-", "3"}], ",", "3"}], "}"}]}], "]"}]}], "}"}], "}"}], 
  "]"}]], "Input",
 CellLabel->"In[120]:=",
 CellID->360360191],

Cell[BoxData[
 TagBox[GridBox[{
    {
     GraphicsBox[
      {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[
       GrayLevel[0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
       {RGBColor[0.0692454, 0.645548, 0.968322], 
        DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
       {GrayLevel[0], PointSize[Medium], Opacity[1], 
        PointBox[{{-1., 0.}, {-0.6666666666666666, 0.}, {-0.3333333333333333, 
         0.}, {0., 0.}, {0.3333333333333333, 0.}, {0.6666666666666666, 0.}, {
         1., 0.}, {2.514357068821368, -1.3112359141823067`}, {
         2.676238045880912, -0.3885143449429056}, {
         2.838119022940456, -0.0485642931178632}, {3., 0.}, {
         3.161880977059544, 0.0485642931178632}, {3.323761954119088, 
         0.3885143449429056}, {3.485642931178632, 1.3112359141823067`}}], 
        {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAA8L8AAAAAAAAAAOP7ND96
l+q/lieeNMoLmT9w76cqRSflv+R+AjU2GKU/vGYe7YZk379E6KgCZ7KpP2RQ
Esa0dtS/UCOoXQZSqj/oqDbLwBXDv5j/8JO/9qY/gBw21auSlT/Qhd8ntUSf
P/hnxppKX8g/gOWF6utigT/I5cgnz/LWP8AUY/lDwJW/POWwNHnO4D8Ysf2l
KACuv0BrMDszFOY/vH3Hlpl+ur/yAHh9yUfrP5glnL7D8sO/fonLuDwz8D/s
ctHJ4JbLv3qFPdLFtvI/I6XEAcsT0r+GR3iVKi31Pwebbvwr0Na/qFT/hRyV
9z8Q6mYLEP7bv4jJ7dFU7fk/nKI61VvN4L+mZl//lDT8P57j+EeU0eO/BkBs
ladp/j8Ch56gFwrnv3izLmCwRQBAFGnIYjB16r+utmF4T0wBQAAiITsOEe6/
aSyNuiVIAkD79d964+3wvyfA1IatOANA70Mhwavp8r/+big9Zx0EQAgPjILS
+vS/IjLGgQ==
          "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV5b8AAAAAAAAAABXxDlAj
yuC/nXMfJ4B4rT/ZDAx1X1XYv+wjDGYNzbs/rhy6E7rlzb8Rvt2bvJXDPzjT
D47oxrW/7FUnor1oyD8o9ZTj+KOwP9gfInv6XMw/zE1C5mawyz+MuCYPWnDP
P7KFBhTCltc/exJRS53Q0D+VKL4/v6/gP1od4Ts5d9E/wwxk2ZuW5T9t4C5y
qKvRP/0C3QJdfeo/rj/iG89t0T9PJKD2aGHvP8q7IgrOvdA/wMd6rxMg8j+O
mVtABTjPP9yfOluBi/Q/gXKTQw0SzD9S2IPTtPH2P59oXsNfC8g/NglXM2hR
+T8SGSqgHybDP5J+rAhZqfs/oH1opsvJuj+0EYv/SPj9P3Be1DYAK6s/I913
RX8eAEAABJC8HJp0v0cclsUiOwFAdFKS/7XLsb8TSbL4d1ECQDzatBL988G/
zVDqL+tgA0CPGNQeacrLv4JXRGPsaARAwHUzb/Mx07+p9Bp+72gFQAigY0Vr
3di/UKO5Fg==
          "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV1b8AAAAAAAAAADrK37XA
Ucq/OzYTJlZAsT9wbtQGWDGzvw60Gc6Sf8A/OM5TjM/erT8/xOnRCZrHP8q5
lCJR1sg/V5JDPsvrzT/bLz8u+jzVP93D+1i+uNE/XUJHyaAs3j+FpCgJGBTU
P7w9iUevmuM/E3SMjLIG1j+QLBguNCnoP5nAJlSFj9c/XiQXJHS/7D8MgHLx
v63YP2U3mw2ArfA/mAv+hMpg2T9H2NMas/zyP52FAA9GqNk/d7oZiRlM9T+8
58GhDITZP0ckjox5mvc/y/q6dTH02D8qYY/kmeb5P4mKYt8A+dc/6+TugUIv
/D9+Pq0mAJPWP6NchSw9c/4/sJVVQO3C1D8raeYTq1gAQHaKEWm+idI/X8YU
ay50AUC8s9ZFQ9HPP9mDRS2SiwJAc4IFKvjByT9woQccQp4DQDVxF5zS6MI/
G9mpd6yrBEB63byn6ZK2P8srkExCswVAhBAesFE/lz9Veg2/d7QGQAmgY0Vr
3ai/FRi2dg==
          "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQjQ4W6O0p61i9z/5FxJK6Cyc32JedCitz5TxvL5B3
JfmFyAH7/ZKsleKd1+1buJNM33Aft1+40sL//+b79hlu/Isd3c7aZ7xsSm9i
eGJ/InJSjG/CRfvqI2fMH5i+sH9z7+8G9j2X7eclahbdLX9jf04kvYJ/61X7
K9o370knfbBX6Lu8K8Lruv0/yacLgns+2RsteZDDU33D3k/p89v8nV/sn629
+7DD7ab9nIrz2p4vvtnznJ2imLTmpn1Ku8GVbeI/4fxNd3mEJrj9hqtneiYV
v6b4L9w83Stqik/m/ofbN2N56WKORwwOMPc8PMfzqf0TowPMvctc5dPu/2dy
gPnnzCsJsQZOFgeYf0/E3Be4y8fqAAuPyMmzVmTzsznAwqtbzmvPYi52B1h4
QkKZwwEW3gAvYa5l
          "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV1T8AAAAAAAAAAG38TZak
ftw/lXs110pTsD8I8wW8zerhPyNQHvN7VL8/29w/gCqr5T/IUtqYzn3GPy57
kwZrfuk/efraB7igzD/iOua3h2LtP7TvKGDcB9E/5PN6Bbiq8D9TGdNY92PT
P8HCZM6FqvI/iLVBZGxj1T9gQaamna/0P6dbgCEsBdc/zaG+L+249j9YyE7o
WEjYPzYNS85fxfg/NFC9Pkcs2T/yGMs839P6P3sUKTR+sNk/T1RHH1Tj/D9c
smeht9TZPx6Xi5em8v4/AWEATuCY2T9tNdNsX4AAQNu9XvoX/dg/t2Ev4EKG
AUDC3flOsQHYP9+gqLByigJA65N4sDGn1j+XNDPQZIwDQGIz6vhQ7tQ/7cQp
YZCLBECzWTkW+dfSP71R7P5thwVAL4UMjkVl0D8gGLcFeH8GQNKlrM4FL8s/
mE2L2SpzB0BjbcrzW+DEP0oRBCwFYghAsC1BkczDuz+rhfJAiEsJQAmgY0Vr
3ag/ur262g==
          "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV5T8AAAAAAAAAAL38vW0F
deg/Afi46WDXsj+na7F1Uq/rPyjKohRKQ8I/6qDz/YUC7z/2WObOG4LKPwNS
fTBuNvE/cZv70eIR0T8VmknZQvbyP6Kjk3sdktQ/wHoEW9O/9D+RYWha4r/X
PzHAJ+0skvY/IBd9pYGZ2j9cQ9MdWGz4P+P+xTl4Hd0/mi0YVVlN+j/2Z5Jn
cErfPxrDd1oxNPw/Do7sU6GP4D8Q307c3R/+PyZvgxx7TeE/gyNz/KwHAECo
5n3RYN7hP8SNb2TPAAFARdH4kAVC4j/Y+Vj10foBQAoKCXw0eOI/kU97BzD1
AkDaB8nS0IDiP+UPlcJk7wNAGgWaA9Zb4j/nFFRk6+gEQASZkK1XCeI/Ka7G
hj/hBUDPeQuWgYnhP0+5m2bd1wZAgu15kZfc4D98cAwpQswHQFE8Xl/1AuA/
49VKIey9CEBYej7zHPrdP5PlTxVbrAlANKKFrtuW2z9VC+WBEJcKQAigY0Vr
3dg/iRa38A==
          "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAA8D8AAAAAAAAAAMYDci4u
IfE/6ichcK5Quz9Wquny81XyP2Y8aDrD48o/jibKd62d8z987gxVn9XTP+no
INms9/Q//0Fa1wD82T+FvumAOmP2P56TpsbC4d8/COx3iJXf9z++4r4H4sHi
P5+U0B70a/k/OP28yINv5T81HsDzgwf7P8amntpa+Oc/XsFxp2qx/D9Ef0D3
DlvqP/oOTj7GaP4/np22FFyW7D9feHHMVhYAQHPWOxETqe4/pcLKdxj+AEDk
lAQqDUnwP/Ya7igt6wFAPABgMTco8T8/wl4UF90CQNv4izWR8fE/2Skn3lXT
A0CVgUNgsKTyP2pq9N1mzQRAttvQpjVB8z9u6mlkxcoFQEZhe/zNxvM/SmuI
AevKBkBmcJh+MjX0Px4iA8xPzQdAJQYomiiM9D/jAl2patEIQOIT6SqCy/Q/
SfqnlrHWCUCAEdWTHfP0P+58v/GZ3ApADM/10OUC9T8BkdfCmOILQAkPjILS
+vQ/dty+5Q==
          "]], 
         {GrayLevel[0]}}}},
      AlignmentPoint->Center,
      AspectRatio->Automatic,
      Axes->True,
      AxesLabel->None,
      AxesOrigin->Automatic,
      AxesStyle->{},
      Background->None,
      BaseStyle->{},
      BaselinePosition->Automatic,
      ColorOutput->Automatic,
      ContentSelectable->Automatic,
      CoordinatesToolOptions:>Automatic,
      DisplayFunction:>$DisplayFunction,
      Epilog->LineBox[{{3, -1.75}, {3, 1.75}}],
      FormatType:>TraditionalForm,
      Frame->True,
      FrameLabel->FormBox[
        StyleBox[
        "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
         StripOnInput -> False], TraditionalForm],
      FrameStyle->{},
      FrameTicks->None,
      FrameTicksStyle->{},
      GridLines->None,
      GridLinesStyle->{},
      ImageMargins->0.,
      ImagePadding->All,
      ImageSize->Medium,
      ImageSizeRaw->Automatic,
      LabelStyle->{},
      Method->Automatic,
      PlotLabel->None,
      PlotRange->All,
      PlotRangeClipping->False,
      PlotRangePadding->Automatic,
      PlotRegion->Automatic,
      PreserveImageOptions->Automatic,
      Prolog->{},
      RotateLabel->True,
      Ticks->Automatic,
      TicksStyle->{}], 
     TagBox[GridBox[{
        {
         ItemBox[
          TagBox[
           FormBox["x",
            TraditionalForm],
           TraditionalForm,
           Editable->True],
          Alignment->Center,
          BaseStyle->{FontWeight -> "Bold"},
          StripOnInput->False], 
         ItemBox[
          
          InterpretationBox["\<\"\\!\\(\\*TagBox[\\(TraditionalForm\\`\\\"\\\\\
[ScriptF]\\\"\\), TraditionalForm, Rule[Editable, \
True]]\\)(\\!\\(\\*TagBox[\\(TraditionalForm\\`x\\), TraditionalForm, \
Rule[Editable, True]]\\)) = \
\\!\\(\\*TagBox[\\(TraditionalForm\\`x\\^3\\/5\\), TraditionalForm, \
Rule[Editable, True]]\\)\"\>",
           StringForm["`3`(`2`) = `1`", 
            TraditionalForm[Rational[1, 5] $CellContext`x^3], 
            TraditionalForm[$CellContext`x], 
            TraditionalForm["\[ScriptF]"]],
           Editable->False],
          Alignment->Center,
          BaseStyle->{FontWeight -> "Bold"},
          StripOnInput->False]},
        {
         RowBox[{"-", "3"}], 
         TagBox[
          InterpretationBox["\<\"-5.4\"\>",
           -5.4,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {
         RowBox[{"-", "2"}], 
         TagBox[
          InterpretationBox["\<\"-1.6\"\>",
           -1.6,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {
         RowBox[{"-", "1"}], 
         TagBox[
          InterpretationBox["\<\"-0.2\"\>",
           -0.2,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {"0", 
         TagBox[
          InterpretationBox["\<\"0.\"\>",
           0.,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {"1", 
         TagBox[
          InterpretationBox["\<\"0.2\"\>",
           0.2,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {"2", 
         TagBox[
          InterpretationBox["\<\"1.6\"\>",
           1.6,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {"3", 
         TagBox[
          InterpretationBox["\<\"5.4\"\>",
           5.4,
           AutoDelete->True],
          NumberForm[#, 5]& ]}
       },
       AutoDelete->False,
       BaseStyle->{"Menu"},
       GridBoxAlignment->{"Columns" -> {Left, "."}, "Rows" -> {{Automatic}}},
       GridBoxBackground->{"Columns" -> {{
            RGBColor[1, 1, 0.85]}}},
       GridBoxDividers->{"Columns" -> {
           AbsoluteThickness[1.5], None, 
           AbsoluteThickness[1.5]}, "Rows" -> {{True}}, 
         "RowsIndexed" -> {2 -> Directive[
             AbsoluteThickness[1.5]], 1 -> Directive[
             AbsoluteThickness[1.5]], -1 -> Directive[
             AbsoluteThickness[1.5]]}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
       GridBoxSpacings->{"Columns" -> {{1}}, "Rows" -> {{0.62}}}],
      "Grid"]}
   },
   AutoDelete->False,
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Grid"]], "Output",
 ImageSize->{478, 261},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[120]=",
 CellID->1328919778]
}, Open  ]],

Cell["More than one FunctionTable could be used.", "MathCaption",
 CellID->985671205],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Grid", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"FunctionTable", "[", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "3"], "/", "5"}], ",", 
        RowBox[{"{", 
         RowBox[{"x", ",", 
          RowBox[{"-", "3"}], ",", "3"}], "}"}]}], "]"}], ",", 
      RowBox[{"FunctionTable", "[", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", "2"], "/", "2"}], ",", 
        RowBox[{"{", 
         RowBox[{"x", ",", 
          RowBox[{"-", "3"}], ",", "3"}], "}"}]}], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"FunctionArrowDiagram", "[", 
       RowBox[{
        RowBox[{"cubeSet", "[", 
         RowBox[{"[", "1", "]"}], "]"}], ",", 
        RowBox[{"cubeSet", "[", 
         RowBox[{"[", "2", "]"}], "]"}], ",", "\"\<OneToOne\>\"", ",", 
        RowBox[{"ImageSize", "\[Rule]", " ", "Medium"}], ",", 
        RowBox[{"Axes", "\[Rule]", " ", "True"}], ",", 
        RowBox[{"Epilog", "\[Rule]", " ", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"3", ",", 
              RowBox[{"-", "1.75"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"3", ",", "1.75"}], "}"}]}], "}"}], "]"}]}]}], "]"}], 
      ",", "SpanFromLeft"}], "}"}]}], "}"}], "]"}]], "Input",
 CellLabel->"In[121]:=",
 CellID->487089477],

Cell[BoxData[
 TagBox[GridBox[{
    {
     TagBox[GridBox[{
        {
         ItemBox[
          TagBox[
           FormBox["x",
            TraditionalForm],
           TraditionalForm,
           Editable->True],
          Alignment->Center,
          BaseStyle->{FontWeight -> "Bold"},
          StripOnInput->False], 
         ItemBox[
          
          InterpretationBox["\<\"\\!\\(\\*TagBox[\\(TraditionalForm\\`\\\"\\\\\
[ScriptF]\\\"\\), TraditionalForm, Rule[Editable, \
True]]\\)(\\!\\(\\*TagBox[\\(TraditionalForm\\`x\\), TraditionalForm, \
Rule[Editable, True]]\\)) = \
\\!\\(\\*TagBox[\\(TraditionalForm\\`x\\^3\\/5\\), TraditionalForm, \
Rule[Editable, True]]\\)\"\>",
           StringForm["`3`(`2`) = `1`", 
            TraditionalForm[Rational[1, 5] $CellContext`x^3], 
            TraditionalForm[$CellContext`x], 
            TraditionalForm["\[ScriptF]"]],
           Editable->False],
          Alignment->Center,
          BaseStyle->{FontWeight -> "Bold"},
          StripOnInput->False]},
        {
         RowBox[{"-", "3"}], 
         TagBox[
          InterpretationBox["\<\"-5.4\"\>",
           -5.4,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {
         RowBox[{"-", "2"}], 
         TagBox[
          InterpretationBox["\<\"-1.6\"\>",
           -1.6,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {
         RowBox[{"-", "1"}], 
         TagBox[
          InterpretationBox["\<\"-0.2\"\>",
           -0.2,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {"0", 
         TagBox[
          InterpretationBox["\<\"0.\"\>",
           0.,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {"1", 
         TagBox[
          InterpretationBox["\<\"0.2\"\>",
           0.2,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {"2", 
         TagBox[
          InterpretationBox["\<\"1.6\"\>",
           1.6,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {"3", 
         TagBox[
          InterpretationBox["\<\"5.4\"\>",
           5.4,
           AutoDelete->True],
          NumberForm[#, 5]& ]}
       },
       AutoDelete->False,
       BaseStyle->{"Menu"},
       GridBoxAlignment->{"Columns" -> {Left, "."}, "Rows" -> {{Automatic}}},
       GridBoxBackground->{"Columns" -> {{
            RGBColor[1, 1, 0.85]}}},
       GridBoxDividers->{"Columns" -> {
           AbsoluteThickness[1.5], None, 
           AbsoluteThickness[1.5]}, "Rows" -> {{True}}, 
         "RowsIndexed" -> {2 -> Directive[
             AbsoluteThickness[1.5]], 1 -> Directive[
             AbsoluteThickness[1.5]], -1 -> Directive[
             AbsoluteThickness[1.5]]}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
       GridBoxSpacings->{"Columns" -> {{1}}, "Rows" -> {{0.62}}}],
      "Grid"], 
     TagBox[GridBox[{
        {
         ItemBox[
          TagBox[
           FormBox["x",
            TraditionalForm],
           TraditionalForm,
           Editable->True],
          Alignment->Center,
          BaseStyle->{FontWeight -> "Bold"},
          StripOnInput->False], 
         ItemBox[
          
          InterpretationBox["\<\"\\!\\(\\*TagBox[\\(TraditionalForm\\`\\\"\\\\\
[ScriptF]\\\"\\), TraditionalForm, Rule[Editable, \
True]]\\)(\\!\\(\\*TagBox[\\(TraditionalForm\\`x\\), TraditionalForm, \
Rule[Editable, True]]\\)) = \
\\!\\(\\*TagBox[\\(TraditionalForm\\`x\\^2\\/2\\), TraditionalForm, \
Rule[Editable, True]]\\)\"\>",
           StringForm["`3`(`2`) = `1`", 
            TraditionalForm[Rational[1, 2] $CellContext`x^2], 
            TraditionalForm[$CellContext`x], 
            TraditionalForm["\[ScriptF]"]],
           Editable->False],
          Alignment->Center,
          BaseStyle->{FontWeight -> "Bold"},
          StripOnInput->False]},
        {
         RowBox[{"-", "3"}], 
         TagBox[
          InterpretationBox["\<\"4.5\"\>",
           4.5,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {
         RowBox[{"-", "2"}], 
         TagBox[
          InterpretationBox["\<\"2.\"\>",
           2.,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {
         RowBox[{"-", "1"}], 
         TagBox[
          InterpretationBox["\<\"0.5\"\>",
           0.5,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {"0", 
         TagBox[
          InterpretationBox["\<\"0.\"\>",
           0.,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {"1", 
         TagBox[
          InterpretationBox["\<\"0.5\"\>",
           0.5,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {"2", 
         TagBox[
          InterpretationBox["\<\"2.\"\>",
           2.,
           AutoDelete->True],
          NumberForm[#, 5]& ]},
        {"3", 
         TagBox[
          InterpretationBox["\<\"4.5\"\>",
           4.5,
           AutoDelete->True],
          NumberForm[#, 5]& ]}
       },
       AutoDelete->False,
       BaseStyle->{"Menu"},
       GridBoxAlignment->{"Columns" -> {Left, "."}, "Rows" -> {{Automatic}}},
       GridBoxBackground->{"Columns" -> {{
            RGBColor[1, 1, 0.85]}}},
       GridBoxDividers->{"Columns" -> {
           AbsoluteThickness[1.5], None, 
           AbsoluteThickness[1.5]}, "Rows" -> {{True}}, 
         "RowsIndexed" -> {2 -> Directive[
             AbsoluteThickness[1.5]], 1 -> Directive[
             AbsoluteThickness[1.5]], -1 -> Directive[
             AbsoluteThickness[1.5]]}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
       GridBoxSpacings->{"Columns" -> {{1}}, "Rows" -> {{0.62}}}],
      "Grid"]},
    {
     GraphicsBox[
      {RGBColor[0.986419, 0.970581, 0.342397], Opacity[0.2], EdgeForm[
       GrayLevel[0]], DiskBox[{0, 0}, {1.1, 1.6500000000000001`}], 
       {RGBColor[0.0692454, 0.645548, 0.968322], 
        DiskBox[{3, 0}, {1.1, 1.6500000000000001`}]}, 
       {GrayLevel[0], PointSize[Medium], Opacity[1], 
        PointBox[{{-1., 0.}, {-0.6666666666666666, 0.}, {-0.3333333333333333, 
         0.}, {0., 0.}, {0.3333333333333333, 0.}, {0.6666666666666666, 0.}, {
         1., 0.}, {2.514357068821368, -1.3112359141823067`}, {
         2.676238045880912, -0.3885143449429056}, {
         2.838119022940456, -0.0485642931178632}, {3., 0.}, {
         3.161880977059544, 0.0485642931178632}, {3.323761954119088, 
         0.3885143449429056}, {3.485642931178632, 1.3112359141823067`}}], 
        {RGBColor[0.5, 0, 0.5], Arrowheads[Medium], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAA8L8AAAAAAAAAAOP7ND96
l+q/lieeNMoLmT9w76cqRSflv+R+AjU2GKU/vGYe7YZk379E6KgCZ7KpP2RQ
Esa0dtS/UCOoXQZSqj/oqDbLwBXDv5j/8JO/9qY/gBw21auSlT/Qhd8ntUSf
P/hnxppKX8g/gOWF6utigT/I5cgnz/LWP8AUY/lDwJW/POWwNHnO4D8Ysf2l
KACuv0BrMDszFOY/vH3Hlpl+ur/yAHh9yUfrP5glnL7D8sO/fonLuDwz8D/s
ctHJ4JbLv3qFPdLFtvI/I6XEAcsT0r+GR3iVKi31Pwebbvwr0Na/qFT/hRyV
9z8Q6mYLEP7bv4jJ7dFU7fk/nKI61VvN4L+mZl//lDT8P57j+EeU0eO/BkBs
ladp/j8Ch56gFwrnv3izLmCwRQBAFGnIYjB16r+utmF4T0wBQAAiITsOEe6/
aSyNuiVIAkD79d964+3wvyfA1IatOANA70Mhwavp8r/+big9Zx0EQAgPjILS
+vS/IjLGgQ==
          "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV5b8AAAAAAAAAABXxDlAj
yuC/nXMfJ4B4rT/ZDAx1X1XYv+wjDGYNzbs/rhy6E7rlzb8Rvt2bvJXDPzjT
D47oxrW/7FUnor1oyD8o9ZTj+KOwP9gfInv6XMw/zE1C5mawyz+MuCYPWnDP
P7KFBhTCltc/exJRS53Q0D+VKL4/v6/gP1od4Ts5d9E/wwxk2ZuW5T9t4C5y
qKvRP/0C3QJdfeo/rj/iG89t0T9PJKD2aGHvP8q7IgrOvdA/wMd6rxMg8j+O
mVtABTjPP9yfOluBi/Q/gXKTQw0SzD9S2IPTtPH2P59oXsNfC8g/NglXM2hR
+T8SGSqgHybDP5J+rAhZqfs/oH1opsvJuj+0EYv/SPj9P3Be1DYAK6s/I913
RX8eAEAABJC8HJp0v0cclsUiOwFAdFKS/7XLsb8TSbL4d1ECQDzatBL988G/
zVDqL+tgA0CPGNQeacrLv4JXRGPsaARAwHUzb/Mx07+p9Bp+72gFQAigY0Vr
3di/UKO5Fg==
          "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV1b8AAAAAAAAAADrK37XA
Ucq/OzYTJlZAsT9wbtQGWDGzvw60Gc6Sf8A/OM5TjM/erT8/xOnRCZrHP8q5
lCJR1sg/V5JDPsvrzT/bLz8u+jzVP93D+1i+uNE/XUJHyaAs3j+FpCgJGBTU
P7w9iUevmuM/E3SMjLIG1j+QLBguNCnoP5nAJlSFj9c/XiQXJHS/7D8MgHLx
v63YP2U3mw2ArfA/mAv+hMpg2T9H2NMas/zyP52FAA9GqNk/d7oZiRlM9T+8
58GhDITZP0ckjox5mvc/y/q6dTH02D8qYY/kmeb5P4mKYt8A+dc/6+TugUIv
/D9+Pq0mAJPWP6NchSw9c/4/sJVVQO3C1D8raeYTq1gAQHaKEWm+idI/X8YU
ay50AUC8s9ZFQ9HPP9mDRS2SiwJAc4IFKvjByT9woQccQp4DQDVxF5zS6MI/
G9mpd6yrBEB63byn6ZK2P8srkExCswVAhBAesFE/lz9Veg2/d7QGQAmgY0Vr
3ai/FRi2dg==
          "]], ArrowBox[CompressedData["
1:eJxTTMoPSmViYGCQAGIQjQ4W6O0p61i9z/5FxJK6Cyc32JedCitz5TxvL5B3
JfmFyAH7/ZKsleKd1+1buJNM33Aft1+40sL//+b79hlu/Isd3c7aZ7xsSm9i
eGJ/InJSjG/CRfvqI2fMH5i+sH9z7+8G9j2X7eclahbdLX9jf04kvYJ/61X7
K9o370knfbBX6Lu8K8Lruv0/yacLgns+2RsteZDDU33D3k/p89v8nV/sn629
+7DD7ab9nIrz2p4vvtnznJ2imLTmpn1Ku8GVbeI/4fxNd3mEJrj9hqtneiYV
v6b4L9w83Stqik/m/ofbN2N56WKORwwOMPc8PMfzqf0TowPMvctc5dPu/2dy
gPnnzCsJsQZOFgeYf0/E3Be4y8fqAAuPyMmzVmTzsznAwqtbzmvPYi52B1h4
QkKZwwEW3gAvYa5l
          "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV1T8AAAAAAAAAAG38TZak
ftw/lXs110pTsD8I8wW8zerhPyNQHvN7VL8/29w/gCqr5T/IUtqYzn3GPy57
kwZrfuk/efraB7igzD/iOua3h2LtP7TvKGDcB9E/5PN6Bbiq8D9TGdNY92PT
P8HCZM6FqvI/iLVBZGxj1T9gQaamna/0P6dbgCEsBdc/zaG+L+249j9YyE7o
WEjYPzYNS85fxfg/NFC9Pkcs2T/yGMs839P6P3sUKTR+sNk/T1RHH1Tj/D9c
smeht9TZPx6Xi5em8v4/AWEATuCY2T9tNdNsX4AAQNu9XvoX/dg/t2Ev4EKG
AUDC3flOsQHYP9+gqLByigJA65N4sDGn1j+XNDPQZIwDQGIz6vhQ7tQ/7cQp
YZCLBECzWTkW+dfSP71R7P5thwVAL4UMjkVl0D8gGLcFeH8GQNKlrM4FL8s/
mE2L2SpzB0BjbcrzW+DEP0oRBCwFYghAsC1BkczDuz+rhfJAiEsJQAmgY0Vr
3ag/ur262g==
          "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAVVVVVVVV5T8AAAAAAAAAAL38vW0F
deg/Afi46WDXsj+na7F1Uq/rPyjKohRKQ8I/6qDz/YUC7z/2WObOG4LKPwNS
fTBuNvE/cZv70eIR0T8VmknZQvbyP6Kjk3sdktQ/wHoEW9O/9D+RYWha4r/X
PzHAJ+0skvY/IBd9pYGZ2j9cQ9MdWGz4P+P+xTl4Hd0/mi0YVVlN+j/2Z5Jn
cErfPxrDd1oxNPw/Do7sU6GP4D8Q307c3R/+PyZvgxx7TeE/gyNz/KwHAECo
5n3RYN7hP8SNb2TPAAFARdH4kAVC4j/Y+Vj10foBQAoKCXw0eOI/kU97BzD1
AkDaB8nS0IDiP+UPlcJk7wNAGgWaA9Zb4j/nFFRk6+gEQASZkK1XCeI/Ka7G
hj/hBUDPeQuWgYnhP0+5m2bd1wZAgu15kZfc4D98cAwpQswHQFE8Xl/1AuA/
49VKIey9CEBYej7zHPrdP5PlTxVbrAlANKKFrtuW2z9VC+WBEJcKQAigY0Vr
3dg/iRa38A==
          "]], ArrowBox[CompressedData["
1:eJwBkQFu/iFib1JlAgAAABgAAAACAAAAAAAAAAAA8D8AAAAAAAAAAMYDci4u
IfE/6ichcK5Quz9Wquny81XyP2Y8aDrD48o/jibKd62d8z987gxVn9XTP+no
INms9/Q//0Fa1wD82T+FvumAOmP2P56TpsbC4d8/COx3iJXf9z++4r4H4sHi
P5+U0B70a/k/OP28yINv5T81HsDzgwf7P8amntpa+Oc/XsFxp2qx/D9Ef0D3
DlvqP/oOTj7GaP4/np22FFyW7D9feHHMVhYAQHPWOxETqe4/pcLKdxj+AEDk
lAQqDUnwP/Ya7igt6wFAPABgMTco8T8/wl4UF90CQNv4izWR8fE/2Skn3lXT
A0CVgUNgsKTyP2pq9N1mzQRAttvQpjVB8z9u6mlkxcoFQEZhe/zNxvM/SmuI
AevKBkBmcJh+MjX0Px4iA8xPzQdAJQYomiiM9D/jAl2patEIQOIT6SqCy/Q/
SfqnlrHWCUCAEdWTHfP0P+58v/GZ3ApADM/10OUC9T8BkdfCmOILQAkPjILS
+vQ/dty+5Q==
          "]], 
         {GrayLevel[0]}}}},
      AlignmentPoint->Center,
      AspectRatio->Automatic,
      Axes->True,
      AxesLabel->None,
      AxesOrigin->Automatic,
      AxesStyle->{},
      Background->None,
      BaseStyle->{},
      BaselinePosition->Automatic,
      ColorOutput->Automatic,
      ContentSelectable->Automatic,
      CoordinatesToolOptions:>Automatic,
      DisplayFunction:>$DisplayFunction,
      Epilog->LineBox[{{3, -1.75}, {3, 1.75}}],
      FormatType:>TraditionalForm,
      Frame->True,
      FrameLabel->FormBox[
        StyleBox[
        "\"\[ScriptX] \[RightTeeArrow] \[ScriptF](\[ScriptX])\"", 18, Bold, 
         StripOnInput -> False], TraditionalForm],
      FrameStyle->{},
      FrameTicks->None,
      FrameTicksStyle->{},
      GridLines->None,
      GridLinesStyle->{},
      ImageMargins->0.,
      ImagePadding->All,
      ImageSize->Medium,
      ImageSizeRaw->Automatic,
      LabelStyle->{},
      Method->Automatic,
      PlotLabel->None,
      PlotRange->All,
      PlotRangeClipping->False,
      PlotRangePadding->Automatic,
      PlotRegion->Automatic,
      PreserveImageOptions->Automatic,
      Prolog->{},
      RotateLabel->True,
      Ticks->Automatic,
      TicksStyle->{}], "\[SpanFromLeft]"}
   },
   AutoDelete->False,
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Grid"]], "Output",
 ImageSize->{364, 437},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[121]=",
 CellID->1324780046]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Creating Plot Grids", "Section",
 CellID->1090019249],

Cell[TextData[{
 "The ",
 Cell[BoxData[
  ButtonBox["PlotGrid",
   BaseStyle->"Link",
   ButtonData->"paclet:ClassroomUtilities/ref/PlotGrid"]], "InlineFormula"],
 " function is designed to make it easy to create a grid for plotting. You \
can easily restrict it to one or two quadrants or you can have a grid that is \
symmetric about the origin. You can also render the grid lines in different \
ways depending on your needs. Using the Epilog option allows you to include \
forms on the grid."
}], "Text",
 CellID->1918009033],

Cell["A default plot grid.", "MathCaption",
 CellID->100792514],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotGrid", "[", "]"}]], "Input",
 CellLabel->"In[21]:=",
 CellID->1802668962],

Cell[BoxData[
 FormBox[
  GraphicsBox[{},
   AlignmentPoint->Center,
   AspectRatio->Automatic,
   Axes->True,
   AxesLabel->None,
   AxesOrigin->Automatic,
   AxesStyle->{},
   Background->None,
   BaseStyle->{14, Italic},
   BaselinePosition->Automatic,
   ColorOutput->Automatic,
   ContentSelectable->Automatic,
   CoordinatesToolOptions:>Automatic,
   DisplayFunction:>$DisplayFunction,
   Epilog->{},
   FormatType:>TraditionalForm,
   Frame->False,
   FrameLabel->None,
   FrameStyle->{},
   FrameTicks->Automatic,
   FrameTicksStyle->{},
   GridLines->{{-10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 
    7, 8, 9, 10}, {-10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 
    6, 7, 8, 9, 10}},
   GridLinesStyle->{},
   ImageMargins->0.,
   ImagePadding->All,
   ImageSize->432,
   ImageSizeRaw->Automatic,
   LabelStyle->{},
   Method->Automatic,
   PlotLabel->None,
   PlotRange->{{-10, 10}, {-10, 10}},
   PlotRangeClipping->False,
   PlotRangePadding->Automatic,
   PlotRegion->Automatic,
   PreserveImageOptions->Automatic,
   Prolog->{},
   RotateLabel->True,
   Ticks->Automatic,
   TicksStyle->{}], TraditionalForm]], "Output",
 ImageSize->{436, 429},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[21]=",
 CellID->2108217122]
}, Open  ]],

Cell["Using the argument All returns the same as no argument.", "MathCaption",
 CellID->462296947],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotGrid", "[", "All", "]"}]], "Input",
 CellLabel->"In[22]:=",
 CellID->228953383],

Cell[BoxData[
 FormBox[
  GraphicsBox[{},
   AlignmentPoint->Center,
   AspectRatio->Automatic,
   Axes->True,
   AxesLabel->None,
   AxesOrigin->Automatic,
   AxesStyle->{},
   Background->None,
   BaseStyle->{14, Italic},
   BaselinePosition->Automatic,
   ColorOutput->Automatic,
   ContentSelectable->Automatic,
   CoordinatesToolOptions:>Automatic,
   DisplayFunction:>$DisplayFunction,
   Epilog->{},
   FormatType:>TraditionalForm,
   Frame->False,
   FrameLabel->None,
   FrameStyle->{},
   FrameTicks->Automatic,
   FrameTicksStyle->{},
   GridLines->{{-10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 
    7, 8, 9, 10}, {-10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 
    6, 7, 8, 9, 10}},
   GridLinesStyle->{},
   ImageMargins->0.,
   ImagePadding->All,
   ImageSize->432,
   ImageSizeRaw->Automatic,
   LabelStyle->{},
   Method->Automatic,
   PlotLabel->None,
   PlotRange->{{-10, 10}, {-10, 10}},
   PlotRangeClipping->False,
   PlotRangePadding->Automatic,
   PlotRegion->Automatic,
   PreserveImageOptions->Automatic,
   Prolog->{},
   RotateLabel->True,
   Ticks->Automatic,
   TicksStyle->{}], TraditionalForm]], "Output",
 ImageSize->{436, 429},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[22]=",
 CellID->1472477601]
}, Open  ]],

Cell["A grid covering only the first and second quadrants.", "MathCaption",
 CellID->33967602],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotGrid", "[", 
  RowBox[{"{", 
   RowBox[{"QuadrantI", ",", "QuadrantII"}], "}"}], "]"}]], "Input",
 CellLabel->"In[23]:=",
 CellID->1810320359],

Cell[BoxData[
 FormBox[
  GraphicsBox[{},
   AlignmentPoint->Center,
   AspectRatio->Automatic,
   Axes->True,
   AxesLabel->None,
   AxesOrigin->Automatic,
   AxesStyle->{},
   Background->None,
   BaseStyle->{14, Italic},
   BaselinePosition->Automatic,
   ColorOutput->Automatic,
   ContentSelectable->Automatic,
   CoordinatesToolOptions:>Automatic,
   DisplayFunction:>$DisplayFunction,
   Epilog->{},
   FormatType:>TraditionalForm,
   Frame->False,
   FrameLabel->None,
   FrameStyle->{},
   FrameTicks->Automatic,
   FrameTicksStyle->{},
   GridLines->{{-10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 
    7, 8, 9, 10}, {-10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 
    6, 7, 8, 9, 10}},
   GridLinesStyle->{},
   ImageMargins->0.,
   ImagePadding->All,
   ImageSize->432,
   ImageSizeRaw->Automatic,
   LabelStyle->{},
   Method->Automatic,
   PlotLabel->None,
   PlotRange->{{-10, 10}, {0, 10}},
   PlotRangeClipping->False,
   PlotRangePadding->Automatic,
   PlotRegion->Automatic,
   PreserveImageOptions->Automatic,
   Prolog->{},
   RotateLabel->True,
   Ticks->Automatic,
   TicksStyle->{}], TraditionalForm]], "Output",
 ImageSize->{436, 234},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[23]=",
 CellID->329491979]
}, Open  ]],

Cell["Domain and range values can be given explicitly.", "MathCaption",
 CellID->2068892072],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotGrid", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "2"}], ",", "5"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "1"}], ",", "6"}], "}"}]}], "}"}], "]"}]], "Input",
 CellLabel->"In[24]:=",
 CellID->1961325527],

Cell[BoxData[
 FormBox[
  GraphicsBox[{},
   AlignmentPoint->Center,
   AspectRatio->Automatic,
   Axes->True,
   AxesLabel->None,
   AxesOrigin->Automatic,
   AxesStyle->{},
   Background->None,
   BaseStyle->{14, Italic},
   BaselinePosition->Automatic,
   ColorOutput->Automatic,
   ContentSelectable->Automatic,
   CoordinatesToolOptions:>Automatic,
   DisplayFunction:>$DisplayFunction,
   Epilog->{},
   FormatType:>TraditionalForm,
   Frame->False,
   FrameLabel->None,
   FrameStyle->{},
   FrameTicks->Automatic,
   FrameTicksStyle->{},
   GridLines->{{-2, -1, 0, 1, 2, 3, 4, 5}, {-1, 0, 1, 2, 3, 4, 5, 6}},
   GridLinesStyle->{},
   ImageMargins->0.,
   ImagePadding->All,
   ImageSize->432,
   ImageSizeRaw->Automatic,
   LabelStyle->{},
   Method->Automatic,
   PlotLabel->None,
   PlotRange->{{-2, 5}, {-1, 6}},
   PlotRangeClipping->False,
   PlotRangePadding->Automatic,
   PlotRegion->Automatic,
   PreserveImageOptions->Automatic,
   Prolog->{},
   RotateLabel->True,
   Ticks->Automatic,
   TicksStyle->{}], TraditionalForm]], "Output",
 ImageSize->{436, 436},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[24]=",
 CellID->1558849087]
}, Open  ]],

Cell["\<\
The use of GridLinesStyle allows the grid to be rendered only on lattice \
points.\
\>", "MathCaption",
 CellID->297261601],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotGrid", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"QuadrantI", ",", "QuadrantII"}], "}"}], ",", 
   RowBox[{"GridLinesStyle", "\[Rule]", " ", "XGrid"}]}], "]"}]], "Input",
 CellLabel->"In[25]:=",
 CellID->1028765085],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3SFQY08Sx3GuziAjsZHIyFhkZCTynow4ERmLfBIZG0nVmSdjkchILFVn
kMjcjzTMToDdS/dk83i876fqT4X9b2eG6UnnJVmmh//69/Rf/zw7O/uv/vvP
P87OXm9vAQAAAAAAcHKbenwm43qT/WFTne1UTWvzAoA2NFVeDl8rpBVClUUq
IoA+2St7r9XwvTrq9v51IwD8aJ/KYfpO/4d6iE5pdsLh8/n85eUlFvv09HRz
cxMeWrG6h1is5qyZh4deLpcPDw/h8KqqwrEaV6OHw4+fr9+Xw71vWs1X4Sb/
Ufk6WMkm72K+7u/vzzKr1co7aIodDAbPz8+Hxz4+Pp6fn6fwxWLhGlp/P8Xq
fnRvh8dqnpptCvcunVYpXzStoSv86uoqxV5eXrpiv2e+9l4T5y+Ws2/yfCUH
Dq1HlhYqRWkBD5/2tq/50qKVbPKSx1d382Vub2/rug4EmtFo5MpUTs8+0+k0
PLRiw89fmrNmHh5az313d3fh8IuLi3Csxi155j12vvbfIsyvCPdq4ytt0bby
pR2ufR4O72i+NpvNZDIJDz2bzcIXad3NV7vCrwUKYxHzcc0/fGKSffupHG4P
vybEsfD4Ak7m0wfI728mfq6G1EMAP1aT3t7aK3xvf/zF58rUQwAw1EMAMNRD
ADDUQwAw1EMAMNRDADDe30wBgJ+KSggAhnoIAIZ6CACGeggAhnoIAIZ6CACG
eggAhnqITig5+DF8eG95eItDFx6VWTjzjuarsB6SrxOH9/DxdX9/X+3ohncO
T09P6/V6OBze3d0FzoFX7HK5HI/HGtrbZkh/X1GK1T3ofrxDa7aas2auWO/Q
WiUNPZ1O5/O5t7nDdteIxNpq6Otm4249pyiNq9G7mC/91LF8aaGaprm4uFCs
q5fH9j1fs9nMNrl36O7mSyNqtUejkW54F83yNZlMbm5u+pMvLfLV1dVwRze8
vRK0Voo6Pz/XVy2dK91acEUpWVo03fA2INPfV5RidQ+64Xp4ap6abZq5twGZ
VsmGvry81A3vRlWireWNvnp7x1i+NK4tWufylX5fz1tOtVBp5lpAV6zlK23y
nuTLNnnKl3eTW760YrbJe5Ivc78TCDR1XYcvbnVZW9I1RrHhC2PNuaSLlp6y
vU98uZIukBq3pJVVi/nSLm0rX4WbvJ/5Wq1W4U3e3XwBp8HnKQBgqIcAYKiH
AGCohwBgqIcAYKiHAGCohwBgqIcAYM4ybc8FANpEGQQAQz0EAEM9BABDPQTQ
P5t6bB+bjOvsCDrqIYC+aar3Oqhb1a9D/6iHAPomq4JNlV0hUg8B9M3rq2Ur
g02VXR5SD9EB6/W68HzscOzz87P35PmcYkt61hSejx3oppEsFotwrMYtPG85
HHtwvt7fP6z2TsgvOR97WzZz7fBAH5Dkp+fra4WHwHcuX/ph81YF3qqoQVOX
BG+rAmUq7+/gPdVcfz/1T9H9eB9leX8H79JplfL+Kd4NU1VV6u+g265Yy1fq
x+F9lJ0oX29Xh78uEre7R1beP8X7Kyrl+Qpv8hbzZZu8lceX5Sv1T/Fu8hbz
ZZs8lq/UykpUjb1tGqyVlaa9Wq0K+3/F+rUpVvcQuLjVbDVnzTzQekyrpLVS
xrXygaZpGjr1//IumuVL42p0zcH7LHCKfKkKpiL4WhHfLhA11dRfL5YvPUL1
0A7nSw8NbXKFexetxXzZJi/JlwJV0AKPL8uXNaLqVr50NR7OVz6HWOCW/rAn
H7pw9L+6aHk5/PjpMv2X2xiaxxfQml//1Ca/PHzD5ykAeiX9U+wPxXBLPQSA
d9RDADDUQwAw1EMAMNRDADDUQwAwNE8BAEMlBABDPQQAQz0EAEM9BABDPQQA
Qz0EAEM9BABDPQQAQz3EN3d/fz/fCZwKboeKX15ertfrQHMlO7TfWiR4j8O1
k9gVGztKXbPVnO0o9cBRuoq6vr6u6zqwaHYIfOwodRta42r0LuZL9TCWLy1U
ypf36Ptt2Sbvbr4UkvLl3eSWL9vk/cmX5py3bgn3JxqPx7rheoxYq53RaDQY
DML9pBSre/D2u9E8FaI5x/rdWKud1E/K2wUs70/kbRJk+SrvJ9VWvtLv6wX6
E6V8eZsEpdZItsl7ki/b5Clf3k2e+rWF+0l1MV+maRo9jwQC0wTCzQ60ziWd
HBUb7oeoOXszldOGKWnSGsuU0bjecpRrMV+BSpgU5ks7XPs8HN7RfOliSVdZ
4aFjF4emu/kCToP3DwHAUA8BwFAPAcBQDwHAUA8BwFAPAcBQDwHAUA8BwJxl
2p4LALSJMggAhnoIAIZ6CACGegign5rq9XOTKjukhHoIoHd2pbD6dFwT9RBA
r2zq8dm4/vI0Z+ohgB55rYZvPhdF6iG+v6enp0B3hsR7oHfO2jSEwwONPHIl
M9eKBRpDJCVn/37nfL2+UH57naybHyui6mFJvkoWrZ/5KgwvfHx1Ll+Pj483
NzdXO4vFwpu11Wql8MFgMJvN6rp2xWqdFavAi4sL3fDOX39fUYqtqko3vFnT
bDW0Zq5Yb68ErZLWajQaTadThXsPwNfQilJl0Ffvolm+NK5G1xy8TyV/P197
NVDfpLcQNVX7qWO/olKeL9vkCvcegN9ivmyTt/L4snxdXl7aJveWphbztVwu
w/myRZtMJja096dO+dL8Ax0iNFtVs+FwqDvxbhXLl2J1D96fervrfqI5W768
W0WrpCjt8Ovr60AptqJk+fJuFcuXxtXouuF9/vrr+dKL5eySMK+Hmqr91G3l
Sztc+zywyVvMl23yVh5flq/xeGz10LvJW8yXbfJYvoyexUouTUte8G53XSxb
id2WzVwbJtCoNCl5FaNxS15//cV85fXw9Y3Ej58wF75/WDJz7fBwK6vtT83X
/6Npl7xe7mi+gCNJl4Rff8jM5ykAesT+EfZXHy5vqYcA8I56CACGeggAhnoI
AIZ6CACGeggAhnoIAIZmUgBgqIQAYKiHAGCohwBgqIcAYKiHAGCohwBgqIcA
YKiH+P5WO+Hw6XQaPin68fFxNpuFh1Zs+BBdzVkzDw8dOIM9d3V1FY61Y+TD
4S3mS/WwrXwVbvJ+5ivQ8yXpYr70w+b9fbwTqKoqxQ4GA9fR4k9PT/nQ3g3z
oTmRq9+B5qnZplj9FK6htUr50N4No0yl2OFw6Ir9MfnyXihqoVKgd6v3M1+F
m7zk8bXtbL7McrkMdKtJJpNJ+Plrs9l4M5VTbLgzheasmYeH1oa5u7sLh49G
o3Csxi253mgxX9qi4XzpAV5ykaYdrn0eDu9ovnR9eH19HR56Pp+Hrw+7my/g
NLyXhQDwU1EPAcBQDwHAUA8BwFAPAcBQDwHAUA8BwFAPAcDEfjkFAH4eKiEA
GOohABjqIQAY6iGA/trU46pJ31EPAfSUiuHZ2bj+dcAX9RBAH6kYjqtqnJdD
6iGA/nkthvXGvmZ/TD3E9/fy8uI9DzwXPvG4PLzFobVirvPbjzj0d89XKoNN
dZa9ebgtrofkK+Dh4aGtoTuXL414c3MzmUyurq50w5s1a5QzGAwWi0Wg40Bd
11VVDYdD3Yl3/vr7ilKs7kH34x1as9WcNfNAWyitkqJGo9H19bVueLPWNI01
p9BXb0MNy5fG1ejfNF/ZNWFeDi1f4RYqWqiULy2ga9qWL+1w7fNe5UshKV/e
TW75Go/H0+k08PjqaL7ST2310Lto6aeez+cKd3V50E+t2JQvby8S60mh2Nls
pvtx7TTNU7Gas+0076LZLtUOt3x5K7l2acqXd6dZvjSuPb6+Yb52H6Hk3iqi
5UvfB/KlR4QWKuXLWxksX9rh2uf9yZcWLc+XtxeJ5UvTVj3sT77MZicQaEra
KmnpvE8fOcWWXFSXzFxZK3kRVNJLUeOWtDo9Ub4+vXm43b1ebitfhZv85+fr
K4WbvKP5Ao7v05uHWz5PAdBLX5VD6iEAvKEeAoChHgKAoR4CgKEeAoChHgKA
oR4CgKGfFAAYKiEAGOohABjqIQAY6iEAGOohABjqIQAY6iEAGOohvr9mJxw+
n8/Dhx5br4Tw0IGeFInmrJmHh14ulyWdeqqqCsdqXO/x77kW86V62Fa+Cjd5
P/NVssm7mK/NZnN1dTXc0Y31eu0K18+rqPPz8/F4PJlMXFl7fn5W7Gg0GgwG
1r3FNbR1WFCs7kE3vK0lNFvNWTNXrDdrWiVFXVxcXF5e6ob3VPPr62tFqTLo
q267Yi1fGlej64b3UPQW86VHlv3UgV9ROUq+wpu8xXzVdd1uvmKbvN182SaP
5Wu7a5Gge5jNZoEWD9YYQtPWnAPNDux5U9PWnbgK2naXbkUpNvYUptlqzsp1
rEOEorTU2q6BRXt8fFSUdpq+6nZgaI2r0XXDe9nQYr40VUVpl7aVL+1w7fNu
5cs2eUm+VIFVl8L5Cm/yFvNlmzycr9aVNBgqiUVMyZrz/uHp8fgCvifqIQAY
6iEAGOohABjqIQAY6iEAGOohABjqIQAY+qcAgKESAoChHgKAoR4CgKEeAuif
prIPTsZ1fiwL9RBAz2zqcdW83diriNRDAL2lgkg9BIBtdqVoqIf48QoPjWzx
vMqODl0YfsKhm+qo7x+2eDxpP/L1jYYuFB76/v6+qirrFxA4f369Xg+Hw6Zp
AofAK3a5XNp55t5Dxe1UcMWuVitvh4Xt7ih1zdlOYvcObUepT6fTxWIRPn/+
/PxcXwOHwCtK42r075+vpvp1bWj5Uj2M5UsLlfLlPQTe8qUdrn3eq3xpxJQv
76JZvjTtm5ub/uRLi5y3bvE2tLKmTtY1xtvvRgtu/W6s1Y63AVnqd2P9pFw1
zfrdpJl7W+1olfJWO96NqkSnfjfab65Yy1fqT/Sd8/XhlfLnflKua0UtVJq5
FvDwwO17vtIm70m+bJOnfMX6SWnFbJN7rxk6mi+jQlrS2k9L7e1Wk2id67oO
D63YcP9KzbmkFePd3V1Jv9GSVowaV6OHw0+Qrw8foySF/UZ1oRWL3e4eZYGL
jaSj+Src5Le3t+GWTN3NF3BEv4rhpq72qyKfpwDoj/d/h22qD686qIcAYKiH
AGCohwBgqIcAYKiHAGCohwBgqIcAYGK/nAIAPw9lEAAM9RAADPUQAAz1EAAM
9RAADPUQAAz1EAAM9RDf38PDQ8lRtN6j/nPPz88lRwcrNnx08LZs5uv1OtBN
Iyk5FVzjBlpaJC3mS/WwrXxph5ecZ97RfL28vKxWq3B40zTh88y3HczX4+Nj
3qrAWxUXi0XqkuBtVaDHRd7f4fb21hWuv5/6p+h+vI+yvL+D91RzrVLeP8V7
oHpVVam/g267Yi1fqR+HtzS1mC9NNe+fcvp8hTd5i/myTd5ivlL/FO8mbzFf
tslj+bJWVtZfT09h3v5f1spKC67noHD/Ly14uL+eYnUPgYtbzVZz1swDrce0
SlorbTbrr+ddNLsaV74Cz4CWL+vXpjl4q0qL+dJU7aduK1/VjsK9i9ZivmyT
t5gvq2bdypeuxsP5ShMo6ZRackVdGN7i0CWv+wqHLhy9xUUrfP+wZOh2N3lH
89XiJm83X8AJ8HkKABjqIQAY6iEAGOohABjqIQAY6iEAGOohABiapwCAoRIC
gKEeAoChHgKAoR4CgKEeAoChHuL7Y5cGsGgBLFoAixZQsmgseEDhovVzzbv7
U7c48+4uWotarIcthvdz6MJwhu5WOEOfMtzO1p7P54FTwdMh8LHmSnaouJ3s
7T0L105iV2zsKHVr8WNDBw4BtqHrug4smh0Cb0N7j1K3oTWuhXtju5svLVTK
V+Do45JN3t18KSTly7vJU740+f7kS3O21i1nAPCDBJpwmaZpSpoSVlUV7nSg
Au5tv5VTbOAp22jO3vZbudvb25ImrbFMGY3rbZeW62e+tMO1z8PhHc2XLpZ0
lRUeOnZxaLqbLwAAAAAA8Dubevz6LuS4zj/Haip7b7KKv5Lvh/eF+rB++IW9
FMC+OtzxK1hT5Xf2ev92N7pTdvGfNDXb9Y/YSyHsK6ejVrC9oNf7er9v3eb5
6Q/Ytn/GXophXzkdtYJ9urP0nf4Pe/j3eCX4Z+ylGPaV01Er2O/vbO8bfMne
vGCVvsBeKsC+OtxRK9jeFWV+qbn/DX6Ld8a+xF4qxL46zMEV7O2Dlw/213j/
BXZeT/u+hQ9ZvR327ZfYS4XYVwc5agX78H5j9i1b+EBaKLbtF9hLZdhXBzlq
Bfv08cv7cxI7+M/Scze79rfYS37sK6fjVbD0Tz73w97+mA38R/x72UOwl7zY
Vw5UMAAAAAAAAAAn8vT0FOjOkKzX63CstWkIhwcaeeRKZq4VCzSGSErO/u1t
vkoWrZ/5Kgz/efn6H6/iODM=
    "], {{0, 234}, {432, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{432, 234},
  PlotRange->{{0, 432}, {0, 234}}]], "Output",
 ImageSize->{436, 234},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[25]=",
 CellID->190568837]
}, Open  ]],

Cell["\<\
The size of the grid can be given as Tiny, Small, Medium or Large.\
\>", "MathCaption",
 CellID->1628175589],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotGrid", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "4"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3", ",", "8"}], "}"}]}], "}"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "Small"}]}], "]"}]], "Input",
 CellLabel->"In[26]:=",
 CellID->1510494249],

Cell[BoxData[
 FormBox[
  GraphicsBox[{},
   AlignmentPoint->Center,
   AspectRatio->Automatic,
   Axes->True,
   AxesLabel->None,
   AxesOrigin->Automatic,
   AxesStyle->{},
   Background->None,
   BaseStyle->{14, Italic},
   BaselinePosition->Automatic,
   ColorOutput->Automatic,
   ContentSelectable->Automatic,
   CoordinatesToolOptions:>Automatic,
   DisplayFunction:>$DisplayFunction,
   Epilog->{},
   FormatType:>TraditionalForm,
   Frame->False,
   FrameLabel->None,
   FrameStyle->{},
   FrameTicks->Automatic,
   FrameTicksStyle->{},
   GridLines->{{-2, -1, 0, 1, 2, 3, 4}, {3, 4, 5, 6, 7, 8}},
   GridLinesStyle->{},
   ImageMargins->0.,
   ImagePadding->All,
   ImageSize->Small,
   ImageSizeRaw->Automatic,
   LabelStyle->{},
   Method->Automatic,
   PlotLabel->None,
   PlotRange->{{-2, 4}, {3, 8}},
   PlotRangeClipping->False,
   PlotRangePadding->Automatic,
   PlotRegion->Automatic,
   PreserveImageOptions->Automatic,
   Prolog->{},
   RotateLabel->True,
   Ticks->Automatic,
   TicksStyle->{}], TraditionalForm]], "Output",
 ImageSize->{184, 167},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[26]=",
 CellID->2046496343]
}, Open  ]],

Cell["\<\
The size can also be specified using the size in inches multiplied by 72.\
\>", "MathCaption",
 CellID->1231499100],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotGrid", "[", 
  RowBox[{"All", ",", 
   RowBox[{"ImageSize", "\[Rule]", " ", 
    RowBox[{"3", " ", "72"}]}]}], "]"}]], "Input",
 CellLabel->"In[27]:=",
 CellID->357983914],

Cell[BoxData[
 FormBox[
  GraphicsBox[{},
   AlignmentPoint->Center,
   AspectRatio->Automatic,
   Axes->True,
   AxesLabel->None,
   AxesOrigin->Automatic,
   AxesStyle->{},
   Background->None,
   BaseStyle->{14, Italic},
   BaselinePosition->Automatic,
   ColorOutput->Automatic,
   ContentSelectable->Automatic,
   CoordinatesToolOptions:>Automatic,
   DisplayFunction:>$DisplayFunction,
   Epilog->{},
   FormatType:>TraditionalForm,
   Frame->False,
   FrameLabel->None,
   FrameStyle->{},
   FrameTicks->Automatic,
   FrameTicksStyle->{},
   GridLines->{{-10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 
    7, 8, 9, 10}, {-10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 
    6, 7, 8, 9, 10}},
   GridLinesStyle->{},
   ImageMargins->0.,
   ImagePadding->All,
   ImageSize->216,
   ImageSizeRaw->Automatic,
   LabelStyle->{},
   Method->Automatic,
   PlotLabel->None,
   PlotRange->{{-10, 10}, {-10, 10}},
   PlotRangeClipping->False,
   PlotRangePadding->Automatic,
   PlotRegion->Automatic,
   PreserveImageOptions->Automatic,
   Prolog->{},
   RotateLabel->True,
   Ticks->Automatic,
   TicksStyle->{}], TraditionalForm]], "Output",
 ImageSize->{220, 213},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[27]=",
 CellID->1365946449]
}, Open  ]],

Cell["\<\
Adding a circle or other geometric shape can be accomplished using Epilog.\
\>", "MathCaption",
 CellID->1717592664],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotGrid", "[", 
  RowBox[{"All", ",", 
   RowBox[{"Epilog", "\[Rule]", " ", 
    RowBox[{"Circle", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "0"}], "}"}], ",", "4"}], "]"}]}]}], "]"}]], "Input",
 CellLabel->"In[28]:=",
 CellID->1104720874],

Cell[BoxData[
 FormBox[
  GraphicsBox[{},
   AlignmentPoint->Center,
   AspectRatio->Automatic,
   Axes->True,
   AxesLabel->None,
   AxesOrigin->Automatic,
   AxesStyle->{},
   Background->None,
   BaseStyle->{14, Italic},
   BaselinePosition->Automatic,
   ColorOutput->Automatic,
   ContentSelectable->Automatic,
   CoordinatesToolOptions:>Automatic,
   DisplayFunction:>$DisplayFunction,
   Epilog->CircleBox[{0, 0}, 4],
   FormatType:>TraditionalForm,
   Frame->False,
   FrameLabel->None,
   FrameStyle->{},
   FrameTicks->Automatic,
   FrameTicksStyle->{},
   GridLines->{{-10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 
    7, 8, 9, 10}, {-10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 
    6, 7, 8, 9, 10}},
   GridLinesStyle->{},
   ImageMargins->0.,
   ImagePadding->All,
   ImageSize->432,
   ImageSizeRaw->Automatic,
   LabelStyle->{},
   Method->Automatic,
   PlotLabel->None,
   PlotRange->{{-10, 10}, {-10, 10}},
   PlotRangeClipping->False,
   PlotRangePadding->Automatic,
   PlotRegion->Automatic,
   PreserveImageOptions->Automatic,
   Prolog->{},
   RotateLabel->True,
   Ticks->Automatic,
   TicksStyle->{}], TraditionalForm]], "Output",
 ImageSize->{436, 429},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[28]=",
 CellID->861781913]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotGrid", "[", 
  RowBox[{"All", ",", 
   RowBox[{"Epilog", "\[Rule]", " ", 
    RowBox[{"Line", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"4", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Sin", "[", "t", "]"}], ",", 
          RowBox[{"Cos", "[", "t", "]"}]}], "}"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"t", ",", "0", ",", 
         RowBox[{"2", " ", "\[Pi]"}], ",", 
         RowBox[{"2", " ", 
          RowBox[{"\[Pi]", "/", "5"}]}]}], "}"}]}], "]"}], "]"}]}]}], 
  "]"}]], "Input",
 CellLabel->"In[29]:=",
 CellID->2006498565],

Cell[BoxData[
 FormBox[
  GraphicsBox[{},
   AlignmentPoint->Center,
   AspectRatio->Automatic,
   Axes->True,
   AxesLabel->None,
   AxesOrigin->Automatic,
   AxesStyle->{},
   Background->None,
   BaseStyle->{14, Italic},
   BaselinePosition->Automatic,
   ColorOutput->Automatic,
   ContentSelectable->Automatic,
   CoordinatesToolOptions:>Automatic,
   DisplayFunction:>$DisplayFunction,
   Epilog->LineBox[
     NCache[{{0, 4}, {
       4 (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
         Rational[1, 2], -1 + 5^Rational[1, 2]}, {
       4 (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
         Rational[1, 2], -1 - 5^
        Rational[
         1, 2]}, {(-4) (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
         Rational[1, 2], -1 - 5^
        Rational[
         1, 2]}, {(-4) (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
         Rational[1, 2], -1 + 5^Rational[1, 2]}, {0, 4}}, {{0, 4}, {
       3.804226065180614, 1.2360679774997898`}, {
       2.3511410091698925`, -3.23606797749979}, {-2.3511410091698925`, \
-3.23606797749979}, {-3.804226065180614, 1.2360679774997898`}, {0, 4}}]],
   FormatType:>TraditionalForm,
   Frame->False,
   FrameLabel->None,
   FrameStyle->{},
   FrameTicks->Automatic,
   FrameTicksStyle->{},
   GridLines->{{-10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 
    7, 8, 9, 10}, {-10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 
    6, 7, 8, 9, 10}},
   GridLinesStyle->{},
   ImageMargins->0.,
   ImagePadding->All,
   ImageSize->432,
   ImageSizeRaw->Automatic,
   LabelStyle->{},
   Method->Automatic,
   PlotLabel->None,
   PlotRange->{{-10, 10}, {-10, 10}},
   PlotRangeClipping->False,
   PlotRangePadding->Automatic,
   PlotRegion->Automatic,
   PreserveImageOptions->Automatic,
   Prolog->{},
   RotateLabel->True,
   Ticks->Automatic,
   TicksStyle->{}], TraditionalForm]], "Output",
 ImageSize->{436, 429},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[29]=",
 CellID->1338896423]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["Classroom Utilities",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/guide/ClassroomUtilities"]], "TutorialMoreAbout",
 CellID->1161018363]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED WOLFRAM EDUCATION GROUP COURSES", "TutorialRelatedLinksSection",
 WholeCellGroupOpener->True,
 CellID->415694148],

Cell[TextData[{
 ButtonBox["M50",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[":",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["An",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["Introduction",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["to",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["Mathematica",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["in",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["the",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["Classroom",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"]
}], "TutorialRelatedLinks",
 CellID->415694149]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Building an Understanding of Functions - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "TutorialColor", 
  "LinkTrails" -> GridBox[{{
      RowBox[{
        ButtonBox[
        "Classroom Utilities", ButtonData -> 
         "paclet:ClassroomUtilities/guide/ClassroomUtilities", 
         BaseStyle -> {"Link", "DockedLinkTrail"}], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 32, 27.6196624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Tutorial", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "Some of the functions defined in the ClassroomUtilities` package are \
intended to assist in presenting functions in a pre-calculus or calculus \
class. The results from these functions can be used in slideshow \
presentations in class or in homework assignments or quizzes. This loads the \
package. Working with functions.", "synonyms" -> {}, "title" -> 
    "Building an Understanding of Functions", "titlemodifier" -> "", 
    "windowtitle" -> "Building an Understanding of Functions", "type" -> 
    "Tutorial", "uri" -> 
    "ClassroomUtilities/tutorial/BuildingAnUnderstandingOfFunctions"}, 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> Notebook], 
   Cell[
    StyleData["Output"], CellContext -> Notebook], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "RasterizedOutput"->{
  Cell[108417, 3082, 6953, 122, 255, "Output",
   CellTags->"RasterizedOutput",
   CellID->190568837]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"RasterizedOutput", 129722, 3661}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[631, 21, 3110, 70, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3766, 95, 75, 1, 51, "Title",
 CellID->509267359],
Cell[3844, 98, 319, 6, 70, "Text",
 CellID->1534169418],
Cell[4166, 106, 67, 1, 31, "MathCaption",
 CellID->1135936937],
Cell[4236, 109, 106, 3, 28, "Input",
 CellID->562478329],
Cell[CellGroupData[{
Cell[4367, 116, 75, 1, 51, "Section",
 CellID->350261409],
Cell[4445, 119, 771, 22, 111, "DefinitionBox",
 CellID->2096742444],
Cell[5219, 143, 63, 1, 31, "Caption",
 CellID->1891092685]
}, Open  ]],
Cell[CellGroupData[{
Cell[5319, 149, 83, 1, 51, "Section",
 CellID->992940646],
Cell[5405, 152, 120, 3, 31, "MathCaption",
 CellID->836781195],
Cell[CellGroupData[{
Cell[5550, 159, 259, 8, 28, "Input",
 CellID->2058623809],
Cell[5812, 169, 3483, 109, 254, "Output",
 CellID->340725475]
}, Open  ]],
Cell[9310, 281, 134, 4, 70, "MathCaption",
 CellID->590824636],
Cell[CellGroupData[{
Cell[9469, 289, 352, 10, 70, "Input",
 CellID->25273524],
Cell[9824, 301, 2359, 69, 237, "Output",
 CellID->1381655423]
}, Open  ]],
Cell[12198, 373, 95, 1, 70, "MathCaption",
 CellID->713580102],
Cell[12296, 376, 176, 6, 70, "Input",
 CellID->1592089466],
Cell[CellGroupData[{
Cell[12497, 386, 220, 7, 70, "Input",
 CellID->601413442],
Cell[12720, 395, 2618, 85, 180, "Output",
 CellID->742019350]
}, Open  ]],
Cell[15353, 483, 86, 1, 70, "MathCaption",
 CellID->1976218387],
Cell[15442, 486, 215, 7, 70, "Input",
 CellID->1320345807],
Cell[CellGroupData[{
Cell[15682, 497, 235, 7, 70, "Input",
 CellID->1103453798],
Cell[15920, 506, 2850, 93, 200, "Output",
 CellID->681677321]
}, Open  ]],
Cell[18785, 602, 167, 4, 70, "MathCaption",
 CellID->1213592640],
Cell[CellGroupData[{
Cell[18977, 610, 288, 9, 70, "Input",
 CellID->51821274],
Cell[19268, 621, 2344, 70, 164, "Output",
 CellID->125477302]
}, Open  ]],
Cell[21627, 694, 210, 7, 70, "MathCaption",
 CellID->1633635841],
Cell[CellGroupData[{
Cell[21862, 705, 338, 11, 70, "Input",
 CellID->939152792],
Cell[22203, 718, 2000, 57, 125, "Output",
 CellID->68073360]
}, Open  ]],
Cell[24218, 778, 141, 4, 70, "MathCaption",
 CellID->2122055448],
Cell[CellGroupData[{
Cell[24384, 786, 419, 14, 70, "Input",
 CellID->2124988807],
Cell[24806, 802, 2013, 57, 126, "Output",
 CellID->1647661316]
}, Open  ]],
Cell[26834, 862, 127, 3, 70, "MathCaption",
 CellID->1077533374],
Cell[CellGroupData[{
Cell[26986, 869, 385, 13, 70, "Input",
 CellID->373821667],
Cell[27374, 884, 2840, 92, 203, "Output",
 CellID->442109947]
}, Open  ]],
Cell[30229, 979, 159, 4, 70, "MathCaption",
 CellID->1060121871],
Cell[CellGroupData[{
Cell[30413, 987, 376, 13, 70, "Input",
 CellID->1943950385],
Cell[30792, 1002, 2852, 93, 202, "Output",
 CellID->248460609]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[33693, 1101, 69, 1, 70, "Section",
 CellID->105144541],
Cell[33765, 1104, 708, 15, 70, "Text",
 CellID->1932742753],
Cell[34476, 1121, 89, 1, 70, "MathCaption",
 CellID->769805149],
Cell[CellGroupData[{
Cell[34590, 1126, 167, 4, 70, "Input",
 CellID->1673649715],
Cell[34760, 1132, 4878, 115, 280, "Output",
 CellID->291713427]
}, Open  ]],
Cell[39653, 1250, 152, 4, 70, "MathCaption",
 CellID->1369107928],
Cell[CellGroupData[{
Cell[39830, 1258, 166, 4, 70, "Input",
 CellID->309668291],
Cell[39999, 1264, 4892, 115, 280, "Output",
 CellID->122401488]
}, Open  ]],
Cell[44906, 1382, 88, 1, 70, "MathCaption",
 CellID->120719405],
Cell[CellGroupData[{
Cell[45019, 1387, 442, 12, 70, "Input",
 CellID->1706477067],
Cell[45464, 1401, 13544, 293, 166, "Output",
 CellID->2059958637]
}, Open  ]],
Cell[59023, 1697, 238, 5, 70, "Text",
 CellID->1954677637],
Cell[59264, 1704, 94, 1, 70, "MathCaption",
 CellID->1487800726],
Cell[59361, 1707, 549, 19, 70, "Input",
 CellID->62213253],
Cell[CellGroupData[{
Cell[59935, 1730, 286, 8, 70, "Input",
 CellID->167216214],
Cell[60224, 1740, 6072, 136, 280, "Output",
 CellID->2140938772]
}, Open  ]],
Cell[66311, 1879, 268, 9, 70, "MathCaption",
 CellID->549119355],
Cell[CellGroupData[{
Cell[66604, 1892, 698, 21, 70, "Input",
 CellID->1311768326],
Cell[67305, 1915, 9303, 230, 282, "Output",
 CellID->1669435519]
}, Open  ]],
Cell[76623, 2148, 301, 9, 70, "MathCaption",
 CellID->1659794903],
Cell[CellGroupData[{
Cell[76949, 2161, 1069, 31, 70, "Input",
 CellID->360360191],
Cell[78021, 2194, 9332, 230, 282, "Output",
 CellID->1328919778]
}, Open  ]],
Cell[87368, 2427, 85, 1, 70, "MathCaption",
 CellID->985671205],
Cell[CellGroupData[{
Cell[87478, 2432, 1412, 41, 70, "Input",
 CellID->487089477],
Cell[88893, 2475, 12190, 319, 458, "Output",
 CellID->1324780046]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[101132, 2800, 59, 1, 70, "Section",
 CellID->1090019249],
Cell[101194, 2803, 528, 12, 70, "Text",
 CellID->1918009033],
Cell[101725, 2817, 63, 1, 70, "MathCaption",
 CellID->100792514],
Cell[CellGroupData[{
Cell[101813, 2822, 101, 3, 70, "Input",
 CellID->1802668962],
Cell[101917, 2827, 1301, 47, 450, "Output",
 CellID->2108217122]
}, Open  ]],
Cell[103233, 2877, 98, 1, 70, "MathCaption",
 CellID->462296947],
Cell[CellGroupData[{
Cell[103356, 2882, 107, 3, 70, "Input",
 CellID->228953383],
Cell[103466, 2887, 1301, 47, 450, "Output",
 CellID->1472477601]
}, Open  ]],
Cell[104782, 2937, 94, 1, 70, "MathCaption",
 CellID->33967602],
Cell[CellGroupData[{
Cell[104901, 2942, 170, 5, 70, "Input",
 CellID->1810320359],
Cell[105074, 2949, 1298, 47, 255, "Output",
 CellID->329491979]
}, Open  ]],
Cell[106387, 2999, 92, 1, 70, "MathCaption",
 CellID->2068892072],
Cell[CellGroupData[{
Cell[106504, 3004, 299, 11, 70, "Input",
 CellID->1961325527],
Cell[106806, 3017, 1188, 45, 457, "Output",
 CellID->1558849087]
}, Open  ]],
Cell[108009, 3065, 133, 4, 70, "MathCaption",
 CellID->297261601],
Cell[CellGroupData[{
Cell[108167, 3073, 247, 7, 28, "Input",
 CellID->1028765085],
Cell[108417, 3082, 6953, 122, 255, "Output",
 CellTags->"RasterizedOutput",
 CellID->190568837]
}, Open  ]],
Cell[115385, 3207, 118, 3, 70, "MathCaption",
 CellID->1628175589],
Cell[CellGroupData[{
Cell[115528, 3214, 349, 12, 70, "Input",
 CellID->1510494249],
Cell[115880, 3228, 1179, 45, 188, "Output",
 CellID->2046496343]
}, Open  ]],
Cell[117074, 3276, 125, 3, 70, "MathCaption",
 CellID->1231499100],
Cell[CellGroupData[{
Cell[117224, 3283, 199, 6, 70, "Input",
 CellID->357983914],
Cell[117426, 3291, 1301, 47, 234, "Output",
 CellID->1365946449]
}, Open  ]],
Cell[118742, 3341, 126, 3, 70, "MathCaption",
 CellID->1717592664],
Cell[CellGroupData[{
Cell[118893, 3348, 287, 9, 70, "Input",
 CellID->1104720874],
Cell[119183, 3359, 1318, 47, 450, "Output",
 CellID->861781913]
}, Open  ]],
Cell[CellGroupData[{
Cell[120538, 3411, 603, 19, 70, "Input",
 CellID->2006498565],
Cell[121144, 3432, 2022, 61, 450, "Output",
 CellID->1338896423]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[123215, 3499, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[123283, 3502, 180, 4, 70, "TutorialMoreAbout",
 CellID->1161018363]
}, Open  ]],
Cell[CellGroupData[{
Cell[123500, 3511, 127, 2, 70, "TutorialRelatedLinksSection",
 CellID->415694148],
Cell[123630, 3515, 3506, 82, 70, "TutorialRelatedLinks",
 CellID->415694149]
}, Open  ]]
}, Open  ]],
Cell[127163, 3601, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

