(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     59118,       2049]
NotebookOptionsPosition[     49404,       1728]
NotebookOutlinePosition[     52159,       1793]
CellTagsIndexPosition[     52116,       1790]
WindowTitle->Elementary Matrix Operations - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Solving Linear Systems\"\>":>
       Documentation`HelpLookup["paclet:tutorial/SolvingLinearSystems"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"RelatedTutorialsSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"ClassroomUtilities/tutorial/ElementaryMatrixOperations\
\"\>":>None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["ClassroomUtilities/tutorial/ElementaryMatrixOperations"]}, 
            Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/ClassroomUtilities/\
tutorial/ElementaryMatrixOperations.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "ClassroomUtilities/tutorial/ElementaryMatrixOperations", 
             ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell["Elementary Matrix Operations", "Title",
 CellID->509267359],

Cell["\<\
Some of the functions in the ClassroomUtilities` package provide support for \
learning the concepts in solving systems of linear equations using the \
methods of linear algebra. There are three functions to perform elementary \
row operations. These are designed so the student needs to know what to do at \
each step while the computer does the arithmetic. Additional functions are \
included for converting systems of equations to matrix form and converting \
matrices to systems of equations.\
\>", "Text",
 CellID->1534169418],

Cell["This loads the package.", "MathCaption",
 CellID->66330686],

Cell[BoxData[
 RowBox[{"<<", "ClassroomUtilities`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->275826809],

Cell[BoxData[GridBox[{
   {
    RowBox[{"SwitchRows", "[", 
     RowBox[{
      StyleBox["mat", "TI"], ",", 
      SubscriptBox[
       StyleBox["row", "TI"], 
       StyleBox["i", "TI"]], ",", 
      SubscriptBox[
       StyleBox["row", "TI"], 
       StyleBox["j", "TI"]]}], "]"}], Cell[TextData[{
     "switch rows ",
     Cell[BoxData[
      StyleBox["i", "TI"]], "InlineFormula"],
     " and ",
     Cell[BoxData[
      StyleBox["j", "TI"]], "InlineFormula"],
     " in the matrix ",
     Cell[BoxData[
      StyleBox["mat", "TI"]], "InlineFormula"]
    }], "TableText"]},
   {
    RowBox[{"MultiplyARow", "[", 
     RowBox[{
      StyleBox["mat", "TI"], ",", 
      SubscriptBox[
       StyleBox["row", "TI"], 
       StyleBox["i", "TI"]], ",", 
      StyleBox["const", "TI"]}], "]"}], 
    RowBox[{"multiply", " ", "row", " ", 
     StyleBox["i", "TI"], " ", "by", " ", "the", " ", "number", " ", 
     StyleBox["const", "TI"]}]},
   {
    RowBox[{"AddToRow", "[", 
     RowBox[{
      StyleBox["mat", "TI"], ",", " ", 
      SubscriptBox[
       StyleBox["row", "TI"], 
       StyleBox["i", "TI"]], ",", 
      SubscriptBox[
       StyleBox["row", "TI"], 
       StyleBox["j", "TI"]], ",", 
      StyleBox["const", "TI"]}], "]"}], Cell[TextData[{
     "add ",
     Cell[BoxData[
      RowBox[{
       StyleBox["const", "TI"], " ", 
       SubscriptBox[
        StyleBox["row", "TI"], 
        StyleBox["j", "TI"]]}]], "InlineFormula"],
     " to ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["row", "TI"], 
       StyleBox["i", "TI"]]], "InlineFormula"]
    }], "TableText"]}
  }]], "DefinitionBox",
 CellID->2096742444],

Cell["Elementary row operators.", "Caption",
 CellID->1891092685],

Cell[CellGroupData[{

Cell["Some Preliminaries", "Section",
 CellID->1551468637],

Cell[TextData[{
 "Before you begin to use these functions you may find it useful to set your \
preferences to have all output rendered in ",
 ButtonBox["TraditionalForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TraditionalForm"],
 ". You will find the Preferences located under the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " menu item on a Macintosh computer and under the Edit menu on a Windows \
computer. Open the Preferences and select Evaluation from  the tabs at the \
top. Change Format type of new output cells to ",
 ButtonBox["TraditionalForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TraditionalForm"],
 " and close the Preferences dialog. With this setting all matrices will be \
displayed as matrices and not lists of lists."
}], "Text",
 CellID->786940185],

Cell["\<\
You will also find it convenient to have the Basic Math Assistant palette \
open so you can easily copy output from a previous calculation into your \
current calculation. You will find the palette at the top of the Palettes \
menu and you will want the Calculator/Advanced feature of the palette. You \
will find a template for entering a matrix there in addition to the Output \
from Above button.\
\>", "Text",
 CellID->1899391069],

Cell["\<\
Now that we have our system set to make things easy we will use the \
elementary row operators to row reduce a matrix.\
\>", "Text",
 CellID->721284028],

Cell["\<\
To enter a matrix, choose the matrix template button from the palette and \
select the first selection placeholder. Type in a value and tab to the next \
position in the matrix.\
\>", "MathCaption",
 CellID->836781195],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"mat", "=", 
  RowBox[{"(", GridBox[{
     {"3", "4"},
     {"7", "2"}
    }], ")"}]}]], "Input",
 CellLabel->"In[3]:=",
 CellID->2058623809],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"3", "4"},
     {"7", "2"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 ImageSize->{40, 31},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->75471281]
}, Open  ]],

Cell[TextData[{
 "Let's change the entry in the upper left hand corner to 1 using the ",
 Cell[BoxData[
  ButtonBox["MultiplyARow",
   BaseStyle->"Link",
   ButtonData->"paclet:ClassroomUtilities/ref/MultiplyARow"]], 
  "InlineFormula"],
 " function."
}], "MathCaption",
 CellID->1430468037],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MultiplyARow", "[", 
  RowBox[{"mat", ",", "1", ",", 
   RowBox[{"1", "/", "3"}]}], "]"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->796869694],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", 
      FractionBox["4", "3"]},
     {"7", "2"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 ImageSize->{43, 42},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->1280924366]
}, Open  ]],

Cell["\<\
Next we get a 0 in the lower left by adding -7 times the first row to the \
second. Here is where the Output from Above button comes into play.\
\>", "MathCaption",
 CellID->2119741834],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AddToRow", "[", 
  RowBox[{
   FormBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1", 
        FractionBox["4", "3"]},
       {"7", "2"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    TraditionalForm], ",", "2", ",", "1", ",", 
   RowBox[{"-", "7"}]}], "]"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->1894783955],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", 
      FractionBox["4", "3"]},
     {"0", 
      RowBox[{"-", 
       FractionBox["22", "3"]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 ImageSize->{57, 53},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->598930322]
}, Open  ]],

Cell[TextData[{
 "Now we can multiply the second row by ",
 Cell[BoxData[
  FractionBox[
   RowBox[{"-", "3"}], "22"]], "InlineFormula"],
 " to have a 1 in the lower right hand corner."
}], "MathCaption",
 CellID->966068365],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MultiplyARow", "[", 
  RowBox[{
   FormBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1", 
        FractionBox["4", "3"]},
       {"0", 
        RowBox[{"-", 
         FractionBox["22", "3"]}]}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    TraditionalForm], ",", "2", ",", 
   RowBox[{
    RowBox[{"-", "3"}], "/", "22"}]}], "]"}]], "Input",
 CellLabel->"In[6]:=",
 CellID->1734639235],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", 
      FractionBox["4", "3"]},
     {"0", "1"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 ImageSize->{43, 42},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->1991114620]
}, Open  ]],

Cell[TextData[{
 "We finish by adding ",
 Cell[BoxData[
  FractionBox[
   RowBox[{"-", "4"}], "3"]], "InlineFormula"],
 " row two to row one."
}], "MathCaption",
 CellID->425747139],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AddToRow", "[", 
  RowBox[{
   FormBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1", 
        FractionBox["4", "3"]},
       {"0", "1"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    TraditionalForm], ",", "1", ",", "2", ",", 
   RowBox[{
    RowBox[{"-", "4"}], "/", "3"}]}], "]"}]], "Input",
 CellLabel->"In[9]:=",
 CellID->116653369],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0"},
     {"0", "1"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 ImageSize->{40, 31},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->1005840755]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Working with Systems of Equations", "Section",
 CellID->2093220672],

Cell["\<\
There are some other functions in the ClassroomUtilities` package we can use \
to make our work easier. These functions allow the conversion of a system of \
equations to matrix form and from matrix form to a system of equations.\
\>", "Text",
 CellID->174300101],

Cell[BoxData[GridBox[{
   {
    RowBox[{"AugmentIdentityMatrix", "[", 
     StyleBox["mat", "TI"], "]"}], 
    RowBox[{
    "augment", " ", "the", " ", "matrix", " ", "mat", " ", "by", " ", "an", 
     " ", "identity", " ", "matrix"}]},
   {
    RowBox[{"CreateAugmentedMatrix", "[", 
     RowBox[{
      StyleBox["eqs", "TI"], ",", 
      StyleBox["vars", "TI"]}], "]"}], 
    RowBox[{
    "create", " ", "an", " ", "augmented", " ", "matrix", " ", "from", " ", 
     "a", " ", "system", " ", "of", " ", "equations", " ", 
     StyleBox["eqs", "TI"]}]},
   {
    RowBox[{"CreateEquations", "[", 
     RowBox[{
      StyleBox["mat", "TI"], ",", 
      StyleBox["vars", "TI"]}], "]"}], 
    RowBox[{
    "create", " ", "a", " ", "system", " ", "of", " ", "equations", " ", "in",
      " ", "the", " ", "variables", " ", 
     StyleBox["vars", "TI"]}]},
   {
    RowBox[{"GetColumns", "[", 
     RowBox[{
      StyleBox["mat", "TI"], ",", 
      SubscriptBox[
       StyleBox["col", "TI"], 
       StyleBox["i", "TI"]]}], "]"}], 
    RowBox[{"get", " ", "the", " ", 
     SuperscriptBox[
      StyleBox["i", "TI"], "th"], " ", "column", " ", "from", " ", "the", " ",
      "matrix", " ", 
     StyleBox["mat", "TI"]}]},
   {
    RowBox[{"GetColumns", "[", 
     RowBox[{
      StyleBox["mat", "TI"], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox[
         StyleBox["col", "TI"], 
         StyleBox["i", "TI"]], ",", 
        SubscriptBox[
         StyleBox["col", "TI"], 
         StyleBox["j", "TI"]], ",", 
        StyleBox["\[Ellipsis]", "TR"]}], "}"}]}], "]"}], 
    RowBox[{
    "get", " ", "several", " ", "columns", " ", "from", " ", "the", " ", 
     "matrix", " ", 
     StyleBox["mat", "TI"]}]}
  }]], "DefinitionBox",
 CellID->294102999],

Cell["Functions for converting systems of equations.", "Caption",
 CellID->1629232446],

Cell[CellGroupData[{

Cell["Solving a System of  Linear Equations", "Subsection",
 CellID->682729150],

Cell["\<\
We begin by typing in our system of equations and giving it a name. You may \
use any name you wish, but it is important to remember the name is assigned \
using a single equal sign and two equal signs are required in the equations.\
\>", "Text",
 CellID->623698129],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"eqs", " ", "=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"2", " ", "x"}], " ", "+", " ", "y"}], " ", "\[Equal]", "7"}], 
    ",", 
    RowBox[{
     RowBox[{"x", "-", "y"}], " ", "\[Equal]", " ", "5"}]}], "}"}]}]], "Input",
 CellLabel->"In[10]:=",
 CellID->626773096],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"2", " ", "x"}], "+", "y"}], "\[LongEqual]", "7"}], ",", 
    RowBox[{
     RowBox[{"x", "-", "y"}], "\[LongEqual]", "5"}]}], "}"}], 
  TraditionalForm]], "Output",
 ImageSize->{121, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[10]=",
 CellID->768135937]
}, Open  ]],

Cell["First we convert the system to an augmented matrix.", "MathCaption",
 CellID->1536491042],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CreateAugmentedMatrix", "[", 
  RowBox[{"eqs", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[11]:=",
 CellID->314087719],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"2", "1", "7"},
     {"1", 
      RowBox[{"-", "1"}], "5"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 ImageSize->{63, 31},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->357488907]
}, Open  ]],

Cell["\<\
Now we can use row reduction to find a solution. We begin by switching rows. \
Remember the Output from Above button on the palette.\
\>", "MathCaption",
 CellID->1804299290],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SwitchRows", "[", 
  RowBox[{
   FormBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"2", "1", "7"},
       {"1", 
        RowBox[{"-", "1"}], "5"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    TraditionalForm], ",", "1", ",", "2"}], "]"}]], "Input",
 CellLabel->"In[12]:=",
 CellID->241587743],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", 
      RowBox[{"-", "1"}], "5"},
     {"2", "1", "7"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 ImageSize->{63, 31},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->1621103771]
}, Open  ]],

Cell["\<\
We continue with the row reduction remembering to use the Output from Above \
at each step.\
\>", "MathCaption",
 CellID->766049708],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AddToRow", "[", 
  RowBox[{
   FormBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1", 
        RowBox[{"-", "1"}], "5"},
       {"2", "1", "7"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    TraditionalForm], ",", "2", ",", "1", ",", 
   RowBox[{"-", "2"}]}], "]"}]], "Input",
 CellLabel->"In[13]:=",
 CellID->832978591],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", 
      RowBox[{"-", "1"}], "5"},
     {"0", "3", 
      RowBox[{"-", "3"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 ImageSize->{72, 31},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[13]=",
 CellID->425284144]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MultiplyARow", "[", 
  RowBox[{
   FormBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1", 
        RowBox[{"-", "1"}], "5"},
       {"0", "3", 
        RowBox[{"-", "3"}]}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    TraditionalForm], ",", "2", ",", 
   RowBox[{"1", "/", "3"}]}], "]"}]], "Input",
 CellLabel->"In[14]:=",
 CellID->925030992],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", 
      RowBox[{"-", "1"}], "5"},
     {"0", "1", 
      RowBox[{"-", "1"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 ImageSize->{72, 31},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[14]=",
 CellID->1361761911]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AddToRow", "[", 
  RowBox[{
   FormBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1", 
        RowBox[{"-", "1"}], "5"},
       {"0", "1", 
        RowBox[{"-", "1"}]}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    TraditionalForm], ",", "1", ",", "2", ",", "1"}], "]"}]], "Input",
 CellLabel->"In[15]:=",
 CellID->1399212098],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "4"},
     {"0", "1", 
      RowBox[{"-", "1"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 ImageSize->{63, 31},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[15]=",
 CellID->1611796436]
}, Open  ]],

Cell[TextData[{
 "We can now use the ",
 Cell[BoxData[
  ButtonBox["CreateEquations",
   BaseStyle->"Link",
   ButtonData->"paclet:ClassroomUtilities/ref/CreateEquations"]], 
  "InlineFormula"],
 " function to see the final form of the row reduction."
}], "MathCaption",
 CellID->1103976594],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CreateEquations", "[", 
  RowBox[{
   FormBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1", "0", "4"},
       {"0", "1", 
        RowBox[{"-", "1"}]}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    TraditionalForm], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[16]:=",
 CellID->275905278],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"x", "\[LongEqual]", "4"}], ",", 
    RowBox[{"y", "\[LongEqual]", 
     RowBox[{"-", "1"}]}]}], "}"}], TraditionalForm]], "Output",
 ImageSize->{82, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[16]=",
 CellID->722813473]
}, Open  ]],

Cell["\<\
We can check our solution in several ways. First we use rules, substituting 4 \
for x and -1 for y.\
\>", "MathCaption",
 CellID->11049632],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"eqs", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"x", "\[Rule]", " ", "4"}], ",", " ", 
    RowBox[{"y", "\[Rule]", " ", 
     RowBox[{"-", "1"}]}]}], "}"}]}]], "Input",
 CellLabel->"In[17]:=",
 CellID->1027571382],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"True", ",", "True"}], "}"}], TraditionalForm]], "Output",
 ImageSize->{65, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[17]=",
 CellID->328728100]
}, Open  ]],

Cell[TextData[{
 "Here we use the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function ",
 ButtonBox["LinearSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/LinearSolve"],
 " to check the solution."
}], "MathCaption",
 CellID->1605236616],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LinearSolve", "[", 
  RowBox[{
   RowBox[{"(", GridBox[{
      {"2", "1"},
      {"1", 
       RowBox[{"-", "1"}]}
     }], ")"}], ",", 
   RowBox[{"(", GridBox[{
      {"7"},
      {"5"}
     }], ")"}]}], "]"}]], "Input",
 CellLabel->"In[18]:=",
 CellID->374747851],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"4"},
     {
      RowBox[{"-", "1"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 ImageSize->{35, 31},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[18]=",
 CellID->1965078753]
}, Open  ]],

Cell[TextData[{
 "We can also use the ",
 ButtonBox["Solve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Solve"],
 " function to obtain a solution. This gives our solution as a set of rules \
we can use in subsequent calculations."
}], "MathCaption",
 CellID->927594458],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"soln", "=", 
  RowBox[{"Solve", "[", 
   RowBox[{"eqs", ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}]], "Input",
 CellLabel->"In[20]:=",
 CellID->1496262033],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"x", "\[Rule]", "4"}], ",", 
     RowBox[{"y", "\[Rule]", 
      RowBox[{"-", "1"}]}]}], "}"}], "}"}], TraditionalForm]], "Output",
 ImageSize->{92, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[20]=",
 CellID->651583511]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"eqs", "/.", "soln"}]], "Input",
 CellLabel->"In[21]:=",
 CellID->1144953324],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"True", "True"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 ImageSize->{72, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[21]=",
 CellID->1777039348]
}, Open  ]],

Cell["\<\
We can also find an inverse for our coefficient matrix and obtain the \
solution by multiplying the right hand side by the inverse.\
\>", "MathCaption",
 CellID->1645243007],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"invmat", "=", " ", 
  RowBox[{"Inverse", "[", 
   RowBox[{"(", GridBox[{
      {"2", "1"},
      {"1", 
       RowBox[{"-", "1"}]}
     }], ")"}], "]"}]}]], "Input",
 CellLabel->"In[22]:=",
 CellID->599779877],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "3"], 
      FractionBox["1", "3"]},
     {
      FractionBox["1", "3"], 
      RowBox[{"-", 
       FractionBox["2", "3"]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 ImageSize->{55, 53},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[22]=",
 CellID->212153721]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"invmat", ".", 
  RowBox[{"(", GridBox[{
     {"7"},
     {"5"}
    }], ")"}]}]], "Input",
 CellLabel->"In[23]:=",
 CellID->844734827],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"4"},
     {
      RowBox[{"-", "1"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 ImageSize->{35, 31},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[23]=",
 CellID->443847072]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Verifying Kramer's Rule for a 2 \[Times] 2 System\
\>", "Subsection",
 CellID->1527033073],

Cell["\<\
Here we start with a system of equations with symbolic coefficients. Remember \
to put spaces between the coefficients and the variables to make sure you \
have multiplication.\
\>", "Text",
 CellID->267792614],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"eqs2", "=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"a", " ", "x"}], " ", "+", " ", 
      RowBox[{"b", " ", "y"}]}], " ", "\[Equal]", "r"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"c", " ", "x"}], " ", "+", " ", 
      RowBox[{"d", " ", "y"}]}], " ", "\[Equal]", "s"}]}], "}"}]}]], "Input",
 CellLabel->"In[24]:=",
 CellID->1812223033],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"a", " ", "x"}], "+", 
      RowBox[{"b", " ", "y"}]}], "\[LongEqual]", "r"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"c", " ", "x"}], "+", 
      RowBox[{"d", " ", "y"}]}], "\[LongEqual]", "s"}]}], "}"}], 
  TraditionalForm]], "Output",
 ImageSize->{145, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[24]=",
 CellID->271950230]
}, Open  ]],

Cell["As before we begin by converting to a matrix.", "MathCaption",
 CellID->822314794],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CreateAugmentedMatrix", "[", 
  RowBox[{"eqs2", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[25]:=",
 CellID->1587474313],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"a", "b", "r"},
     {"c", "d", "s"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 ImageSize->{56, 31},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[25]=",
 CellID->343948263]
}, Open  ]],

Cell["Now we use row reduction to find the solution.", "MathCaption",
 CellID->1859962164],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MultiplyARow", "[", 
  RowBox[{
   FormBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"a", "b", "r"},
       {"c", "d", "s"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    TraditionalForm], ",", "1", ",", 
   RowBox[{"1", "/", "a"}]}], "]"}]], "Input",
 CellLabel->"In[26]:=",
 CellID->1612124616],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", 
      FractionBox["b", "a"], 
      FractionBox["r", "a"]},
     {"c", "d", "s"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 ImageSize->{62, 42},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[26]=",
 CellID->177246913]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AddToRow", "[", 
  RowBox[{
   FormBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1", 
        FractionBox["b", "a"], 
        FractionBox["r", "a"]},
       {"c", "d", "s"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    TraditionalForm], ",", "2", ",", "1", ",", 
   RowBox[{"-", "c"}]}], "]"}]], "Input",
 CellLabel->"In[27]:=",
 CellID->429048402],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", 
      FractionBox["b", "a"], 
      FractionBox["r", "a"]},
     {"0", 
      RowBox[{"d", "-", 
       FractionBox[
        RowBox[{"b", " ", "c"}], "a"]}], 
      RowBox[{"s", "-", 
       FractionBox[
        RowBox[{"c", " ", "r"}], "a"]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 ImageSize->{112, 53},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[27]=",
 CellID->1913889435]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MultiplyARow", "[", 
  RowBox[{
   FormBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1", 
        FractionBox["b", "a"], 
        FractionBox["r", "a"]},
       {"0", 
        RowBox[{"d", "-", 
         FractionBox[
          RowBox[{"b", " ", "c"}], "a"]}], 
        RowBox[{"s", "-", 
         FractionBox[
          RowBox[{"c", " ", "r"}], "a"]}]}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    TraditionalForm], ",", "2", ",", 
   FractionBox["1", 
    RowBox[{"d", "-", 
     FractionBox[
      RowBox[{"b", " ", "c"}], "a"]}]]}], "]"}]], "Input",
 CellLabel->"In[28]:=",
 CellID->1729669279],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", 
      FractionBox["b", "a"], 
      FractionBox["r", "a"]},
     {"0", "1", 
      FractionBox[
       RowBox[{"s", "-", 
        FractionBox[
         RowBox[{"c", " ", "r"}], "a"]}], 
       RowBox[{"d", "-", 
        FractionBox[
         RowBox[{"b", " ", "c"}], "a"]}]]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 ImageSize->{82, 71},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[28]=",
 CellID->65442714]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AddToRow", "[", 
  RowBox[{
   FormBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1", 
        FractionBox["b", "a"], 
        FractionBox["r", "a"]},
       {"0", "1", 
        FractionBox[
         RowBox[{"s", "-", 
          FractionBox[
           RowBox[{"c", " ", "r"}], "a"]}], 
         RowBox[{"d", "-", 
          FractionBox[
           RowBox[{"b", " ", "c"}], "a"]}]]}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    TraditionalForm], ",", "1", ",", "2", ",", 
   FractionBox[
    RowBox[{"-", "b"}], "a"]}], "]"}]], "Input",
 CellLabel->"In[29]:=",
 CellID->384066934],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", 
      RowBox[{
       FractionBox["r", "a"], "-", 
       FractionBox[
        RowBox[{"b", " ", 
         RowBox[{"(", 
          RowBox[{"s", "-", 
           FractionBox[
            RowBox[{"c", " ", "r"}], "a"]}], ")"}]}], 
        RowBox[{"a", " ", 
         RowBox[{"(", 
          RowBox[{"d", "-", 
           FractionBox[
            RowBox[{"b", " ", "c"}], "a"]}], ")"}]}]]}]},
     {"0", "1", 
      FractionBox[
       RowBox[{"s", "-", 
        FractionBox[
         RowBox[{"c", " ", "r"}], "a"]}], 
       RowBox[{"d", "-", 
        FractionBox[
         RowBox[{"b", " ", "c"}], "a"]}]]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 ImageSize->{116, 89},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[29]=",
 CellID->1824600503]
}, Open  ]],

Cell[TextData[{
 "This looks pretty messy, but we can use ",
 ButtonBox["Simplify",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Simplify"],
 " to make it look more like what we expect."
}], "Text",
 CellID->2141658408],

Cell["Simplifying the solution matrix.", "MathCaption",
 CellID->879455889],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  FormBox[
   RowBox[{"(", "\[NoBreak]", GridBox[{
      {"1", "0", 
       RowBox[{
        FractionBox["r", "a"], "-", 
        FractionBox[
         RowBox[{"b", " ", 
          RowBox[{"(", 
           RowBox[{"s", "-", 
            FractionBox[
             RowBox[{"c", " ", "r"}], "a"]}], ")"}]}], 
         RowBox[{"a", " ", 
          RowBox[{"(", 
           RowBox[{"d", "-", 
            FractionBox[
             RowBox[{"b", " ", "c"}], "a"]}], ")"}]}]]}]},
      {"0", "1", 
       FractionBox[
        RowBox[{"s", "-", 
         FractionBox[
          RowBox[{"c", " ", "r"}], "a"]}], 
        RowBox[{"d", "-", 
         FractionBox[
          RowBox[{"b", " ", "c"}], "a"]}]]}
     },
     GridBoxAlignment->{
      "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
        "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.7]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
   TraditionalForm], "]"}]], "Input",
 CellLabel->"In[30]:=",
 CellID->2032667769],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", 
      FractionBox[
       RowBox[{
        RowBox[{"d", " ", "r"}], "-", 
        RowBox[{"b", " ", "s"}]}], 
       RowBox[{
        RowBox[{"a", " ", "d"}], "-", 
        RowBox[{"b", " ", "c"}]}]]},
     {"0", "1", 
      FractionBox[
       RowBox[{
        RowBox[{"c", " ", "r"}], "-", 
        RowBox[{"a", " ", "s"}]}], 
       RowBox[{
        RowBox[{"b", " ", "c"}], "-", 
        RowBox[{"a", " ", "d"}]}]]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 ImageSize->{83, 53},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[30]=",
 CellID->877337107]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GetColumns", "[", 
  RowBox[{
   FormBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1", "0", 
        FractionBox[
         RowBox[{
          RowBox[{"d", " ", "r"}], "-", 
          RowBox[{"b", " ", "s"}]}], 
         RowBox[{
          RowBox[{"a", " ", "d"}], "-", 
          RowBox[{"b", " ", "c"}]}]]},
       {"0", "1", 
        FractionBox[
         RowBox[{
          RowBox[{"c", " ", "r"}], "-", 
          RowBox[{"a", " ", "s"}]}], 
         RowBox[{
          RowBox[{"b", " ", "c"}], "-", 
          RowBox[{"a", " ", "d"}]}]]}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    TraditionalForm], ",", "3"}], "]"}]], "Input",
 CellLabel->"In[33]:=",
 CellID->1665534008],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"d", " ", "r"}], "-", 
      RowBox[{"b", " ", "s"}]}], 
     RowBox[{
      RowBox[{"a", " ", "d"}], "-", 
      RowBox[{"b", " ", "c"}]}]], ",", 
    FractionBox[
     RowBox[{
      RowBox[{"c", " ", "r"}], "-", 
      RowBox[{"a", " ", "s"}]}], 
     RowBox[{
      RowBox[{"b", " ", "c"}], "-", 
      RowBox[{"a", " ", "d"}]}]]}], "}"}], TraditionalForm]], "Output",
 ImageSize->{117, 34},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[33]=",
 CellID->180733811]
}, Open  ]],

Cell["\<\
We can compare this with what we get using Kramer's rule. We calculate three \
determinants.\
\>", "MathCaption",
 CellID->782037047],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Det", "[", 
     RowBox[{"(", GridBox[{
        {"r", "b"},
        {"s", "d"}
       }], ")"}], "]"}], ",", 
    RowBox[{"Det", "[", 
     RowBox[{"(", GridBox[{
        {"a", "r"},
        {"c", "s"}
       }], ")"}], "]"}]}], "}"}], "/", 
  RowBox[{"Det", "[", 
   RowBox[{"(", GridBox[{
      {"a", "b"},
      {"c", "d"}
     }], ")"}], "]"}]}]], "Input",
 CellLabel->"In[32]:=",
 CellID->1096729289],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"d", " ", "r"}], "-", 
      RowBox[{"b", " ", "s"}]}], 
     RowBox[{
      RowBox[{"a", " ", "d"}], "-", 
      RowBox[{"b", " ", "c"}]}]], ",", 
    FractionBox[
     RowBox[{
      RowBox[{"a", " ", "s"}], "-", 
      RowBox[{"c", " ", "r"}]}], 
     RowBox[{
      RowBox[{"a", " ", "d"}], "-", 
      RowBox[{"b", " ", "c"}]}]]}], "}"}], TraditionalForm]], "Output",
 ImageSize->{117, 34},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[32]=",
 CellID->1861626393]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 WholeCellGroupOpener->True,
 CellID->415694126],

Cell[TextData[ButtonBox["Solving Linear Systems",
 BaseStyle->"Link",
 ButtonData->"paclet:tutorial/SolvingLinearSystems"]], "RelatedTutorials",
 CellID->806871991]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED WOLFRAM EDUCATION GROUP COURSES", "TutorialRelatedLinksSection",
 WholeCellGroupOpener->True,
 CellID->415694148],

Cell[TextData[ButtonBox["M50: An Introduction to Mathematica in the Classroom",
 BaseStyle->"Hyperlink",
 ButtonData->{
   URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
 ButtonNote->
  "http://www.wolfram.com/services/education/courses/m50.html"]], \
"TutorialRelatedLinks",
 CellID->415694149]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Elementary Matrix Operations - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "TutorialColor", 
  "LinkTrails" -> GridBox[{{
      RowBox[{
        ButtonBox[
        "Classroom Utilities", ButtonData -> 
         "paclet:ClassroomUtilities/guide/ClassroomUtilities", 
         BaseStyle -> {"Link", "DockedLinkTrail"}], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 32, 32.7988624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Tutorial", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "Some of the functions in the ClassroomUtilities` package provide support \
for learning the concepts in solving systems of linear equations using the \
methods of linear algebra. There are three functions to perform elementary \
row operations. These are designed so the student needs to know what to do at \
each step while the computer does the arithmetic. Additional functions are \
included for converting systems of equations to matrix form and converting \
matrices to systems of equations. This loads the package. Elementary row \
operators.", "synonyms" -> {}, "title" -> "Elementary Matrix Operations", 
    "titlemodifier" -> "", "windowtitle" -> "Elementary Matrix Operations", 
    "type" -> "Tutorial", "uri" -> 
    "ClassroomUtilities/tutorial/ElementaryMatrixOperations"}, 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> Notebook], 
   Cell[
    StyleData["Output"], CellContext -> Notebook], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[621, 21, 2536, 57, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3182, 82, 65, 1, 51, "Title",
 CellID->509267359],
Cell[3250, 85, 541, 9, 130, "Text",
 CellID->1534169418],
Cell[3794, 96, 65, 1, 31, "MathCaption",
 CellID->66330686],
Cell[3862, 99, 106, 3, 28, "Input",
 CellID->275826809],
Cell[3971, 104, 1640, 57, 101, "DefinitionBox",
 CellID->2096742444],
Cell[5614, 163, 65, 1, 31, "Caption",
 CellID->1891092685],
Cell[CellGroupData[{
Cell[5704, 168, 58, 1, 51, "Section",
 CellID->1551468637],
Cell[5765, 171, 796, 18, 130, "Text",
 CellID->786940185],
Cell[6564, 191, 444, 8, 70, "Text",
 CellID->1899391069],
Cell[7011, 201, 162, 4, 70, "Text",
 CellID->721284028],
Cell[7176, 207, 228, 5, 70, "MathCaption",
 CellID->836781195],
Cell[CellGroupData[{
Cell[7429, 216, 164, 7, 70, "Input",
 CellID->2058623809],
Cell[7596, 225, 683, 21, 52, "Output",
 CellID->75471281]
}, Open  ]],
Cell[8294, 249, 291, 9, 70, "MathCaption",
 CellID->1430468037],
Cell[CellGroupData[{
Cell[8610, 262, 167, 5, 70, "Input",
 CellID->796869694],
Cell[8780, 269, 710, 22, 63, "Output",
 CellID->1280924366]
}, Open  ]],
Cell[9505, 294, 195, 4, 70, "MathCaption",
 CellID->2119741834],
Cell[CellGroupData[{
Cell[9725, 302, 751, 22, 70, "Input",
 CellID->1894783955],
Cell[10479, 326, 758, 24, 74, "Output",
 CellID->598930322]
}, Open  ]],
Cell[11252, 353, 224, 7, 70, "MathCaption",
 CellID->966068365],
Cell[CellGroupData[{
Cell[11501, 364, 824, 25, 70, "Input",
 CellID->1734639235],
Cell[12328, 391, 710, 22, 63, "Output",
 CellID->1991114620]
}, Open  ]],
Cell[13053, 416, 181, 7, 70, "MathCaption",
 CellID->425747139],
Cell[CellGroupData[{
Cell[13259, 427, 775, 23, 70, "Input",
 CellID->116653369],
Cell[14037, 452, 685, 21, 52, "Output",
 CellID->1005840755]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[14771, 479, 73, 1, 70, "Section",
 CellID->2093220672],
Cell[14847, 482, 273, 5, 70, "Text",
 CellID->174300101],
Cell[15123, 489, 1763, 55, 70, "DefinitionBox",
 CellID->294102999],
Cell[16889, 546, 86, 1, 70, "Caption",
 CellID->1629232446],
Cell[CellGroupData[{
Cell[17000, 551, 79, 1, 70, "Subsection",
 CellID->682729150],
Cell[17082, 554, 276, 5, 70, "Text",
 CellID->623698129],
Cell[CellGroupData[{
Cell[17383, 563, 325, 11, 70, "Input",
 CellID->626773096],
Cell[17711, 576, 389, 14, 36, "Output",
 CellID->768135937]
}, Open  ]],
Cell[18115, 593, 95, 1, 70, "MathCaption",
 CellID->1536491042],
Cell[CellGroupData[{
Cell[18235, 598, 192, 6, 70, "Input",
 CellID->314087719],
Cell[18430, 606, 717, 22, 52, "Output",
 CellID->357488907]
}, Open  ]],
Cell[19162, 631, 184, 4, 70, "MathCaption",
 CellID->1804299290],
Cell[CellGroupData[{
Cell[19371, 639, 731, 21, 70, "Input",
 CellID->241587743],
Cell[20105, 662, 718, 22, 52, "Output",
 CellID->1621103771]
}, Open  ]],
Cell[20838, 687, 142, 4, 70, "MathCaption",
 CellID->766049708],
Cell[CellGroupData[{
Cell[21005, 695, 758, 22, 70, "Input",
 CellID->832978591],
Cell[21766, 719, 739, 23, 52, "Output",
 CellID->425284144]
}, Open  ]],
Cell[CellGroupData[{
Cell[22542, 747, 781, 23, 70, "Input",
 CellID->925030992],
Cell[23326, 772, 740, 23, 52, "Output",
 CellID->1361761911]
}, Open  ]],
Cell[CellGroupData[{
Cell[24103, 800, 764, 22, 70, "Input",
 CellID->1399212098],
Cell[24870, 824, 718, 22, 52, "Output",
 CellID->1611796436]
}, Open  ]],
Cell[25603, 849, 291, 9, 70, "MathCaption",
 CellID->1103976594],
Cell[CellGroupData[{
Cell[25919, 862, 775, 23, 70, "Input",
 CellID->275905278],
Cell[26697, 887, 327, 11, 36, "Output",
 CellID->722813473]
}, Open  ]],
Cell[27039, 901, 149, 4, 70, "MathCaption",
 CellID->11049632],
Cell[CellGroupData[{
Cell[27213, 909, 242, 8, 70, "Input",
 CellID->1027571382],
Cell[27458, 919, 240, 8, 36, "Output",
 CellID->328728100]
}, Open  ]],
Cell[27713, 930, 255, 10, 70, "MathCaption",
 CellID->1605236616],
Cell[CellGroupData[{
Cell[27993, 944, 290, 13, 70, "Input",
 CellID->374747851],
Cell[28286, 959, 698, 22, 52, "Output",
 CellID->1965078753]
}, Open  ]],
Cell[28999, 984, 273, 8, 70, "MathCaption",
 CellID->927594458],
Cell[CellGroupData[{
Cell[29297, 996, 206, 7, 70, "Input",
 CellID->1496262033],
Cell[29506, 1005, 345, 12, 36, "Output",
 CellID->651583511]
}, Open  ]],
Cell[CellGroupData[{
Cell[29888, 1022, 100, 3, 70, "Input",
 CellID->1144953324],
Cell[29991, 1027, 675, 20, 36, "Output",
 CellID->1777039348]
}, Open  ]],
Cell[30681, 1050, 183, 4, 70, "MathCaption",
 CellID->1645243007],
Cell[CellGroupData[{
Cell[30889, 1058, 233, 9, 70, "Input",
 CellID->599779877],
Cell[31125, 1069, 808, 26, 74, "Output",
 CellID->212153721]
}, Open  ]],
Cell[CellGroupData[{
Cell[31970, 1100, 157, 7, 70, "Input",
 CellID->844734827],
Cell[32130, 1109, 697, 22, 52, "Output",
 CellID->443847072]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[32876, 1137, 100, 3, 70, "Subsection",
 CellID->1527033073],
Cell[32979, 1142, 220, 5, 70, "Text",
 CellID->267792614],
Cell[CellGroupData[{
Cell[33224, 1151, 403, 13, 70, "Input",
 CellID->1812223033],
Cell[33630, 1166, 470, 17, 36, "Output",
 CellID->271950230]
}, Open  ]],
Cell[34115, 1186, 88, 1, 70, "MathCaption",
 CellID->822314794],
Cell[CellGroupData[{
Cell[34228, 1191, 194, 6, 70, "Input",
 CellID->1587474313],
Cell[34425, 1199, 695, 21, 52, "Output",
 CellID->343948263]
}, Open  ]],
Cell[35135, 1223, 90, 1, 70, "MathCaption",
 CellID->1859962164],
Cell[CellGroupData[{
Cell[35250, 1228, 734, 21, 70, "Input",
 CellID->1612124616],
Cell[35987, 1251, 745, 23, 63, "Output",
 CellID->177246913]
}, Open  ]],
Cell[CellGroupData[{
Cell[36769, 1279, 788, 23, 70, "Input",
 CellID->429048402],
Cell[37560, 1304, 911, 29, 74, "Output",
 CellID->1913889435]
}, Open  ]],
Cell[CellGroupData[{
Cell[38508, 1338, 1038, 32, 70, "Input",
 CellID->1729669279],
Cell[39549, 1372, 939, 30, 92, "Output",
 CellID->65442714]
}, Open  ]],
Cell[CellGroupData[{
Cell[40525, 1407, 1020, 31, 70, "Input",
 CellID->384066934],
Cell[41548, 1440, 1277, 41, 110, "Output",
 CellID->1824600503]
}, Open  ]],
Cell[42840, 1484, 221, 7, 70, "Text",
 CellID->2141658408],
Cell[43064, 1493, 75, 1, 70, "MathCaption",
 CellID->879455889],
Cell[CellGroupData[{
Cell[43164, 1498, 1257, 39, 70, "Input",
 CellID->2032667769],
Cell[44424, 1539, 1089, 35, 74, "Output",
 CellID->877337107]
}, Open  ]],
Cell[CellGroupData[{
Cell[45550, 1579, 1120, 34, 70, "Input",
 CellID->1665534008],
Cell[46673, 1615, 601, 22, 55, "Output",
 CellID->180733811]
}, Open  ]],
Cell[47289, 1640, 143, 4, 70, "MathCaption",
 CellID->782037047],
Cell[CellGroupData[{
Cell[47457, 1648, 470, 20, 70, "Input",
 CellID->1096729289],
Cell[47930, 1670, 602, 22, 55, "Output",
 CellID->1861626393]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[48593, 1699, 101, 2, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[48697, 1703, 164, 3, 70, "RelatedTutorials",
 CellID->806871991]
}, Open  ]],
Cell[CellGroupData[{
Cell[48898, 1711, 127, 2, 70, "TutorialRelatedLinksSection",
 CellID->415694148],
Cell[49028, 1715, 322, 7, 70, "TutorialRelatedLinks",
 CellID->415694149]
}, Open  ]]
}, Open  ]],
Cell[49377, 1726, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

