(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     31190,        883]
NotebookOptionsPosition[     25540,        707]
NotebookOutlinePosition[     28319,        780]
CellTagsIndexPosition[     28240,        775]
WindowTitle->Falling Body Problems - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Classroom Utilities\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/guide/ClassroomUtilities"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"TutorialMoreAboutSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"ClassroomUtilities/tutorial/Falling Body Problems\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["ClassroomUtilities/tutorial/Falling Body Problems"]}, 
            Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/ClassroomUtilities/\
tutorial/Falling Body Problems.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "ClassroomUtilities/tutorial/Falling Body Problems", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell["Falling Body Problems", "Title",
 CellID->509267359],

Cell["\<\
The Trajectory functions in the ClassroomUtilities` package are intended to \
allow students to compare what happens to object traveling through air when \
we consider the affects of air resistance. \
\>", "Text",
 CellID->1534169418],

Cell["This loads the package.", "MathCaption",
 CellID->100270670],

Cell[BoxData[
 RowBox[{"<<", "ClassroomUtilities`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1621971442],

Cell[BoxData[GridBox[{
   {
    RowBox[{"Trajectory", "[", 
     RowBox[{
      StyleBox["t", "TI"], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox[
         StyleBox["v", "TI"], 
         StyleBox["0", "TR"]], ",", 
        StyleBox["\[Theta]", "TR"], ",", 
        StyleBox["g", "TI"]}], "}"}]}], "]"}], Cell[TextData[{
     "gives a parametric representation of a trajectory with no air \
resistance, initial velocity ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["v", "TI"], 
       StyleBox["0", "TR"]]], "InlineFormula"],
     " launched at an angle \[Theta] with the horizontal."
    }], "TableText"]},
   {
    RowBox[{"Trajectory", "[", 
     RowBox[{
      StyleBox["t", "TI"], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox[
         StyleBox["v", "TI"], 
         StyleBox["0", "TR"]], ",", 
        StyleBox["\[Theta]", "TR"], ",", 
        StyleBox["g", "TI"], ",", 
        StyleBox["k", "TI"]}], "}"}]}], "]"}], Cell[TextData[Cell[BoxData[
    Cell[TextData[{
      "gives a parametric representation of a trajectory with air resistance \
given by ",
      Cell[BoxData[
       RowBox[{
        StyleBox["k", "TI"], "\[Times]", 
        SuperscriptBox[
         SubscriptBox[
          StyleBox["v", "TI"], 
          StyleBox["t", "TI"]], "2"]}]], "InlineFormula"],
      ", initial velocity ",
      Cell[BoxData[
       SubscriptBox[
        StyleBox["v", "TI"], 
        StyleBox["0", "TR"]]], "InlineFormula"],
      " launched at an angle \[Theta] with the horizontal "
     }], "TableText"]]]], "TableText"]},
   {
    RowBox[{"Trajectory", "[", 
     RowBox[{
      StyleBox["t", "TI"], ",", 
      StyleBox["params", "TI"], ",", "\"\<\!\(\*
StyleBox[\"component\", \"TI\"]\)\>\""}], "]"}], 
    RowBox[{
    "returns", " ", "the", " ", "\"\<Horizontal\>\"", " ", "or", " ", 
     "\"\<Vertical\>\"", " ", "component", " ", "of", " ", "the", " ", 
     RowBox[{"function", "."}]}]}
  }]], "DefinitionBox",
 CellID->2096742444],

Cell["Trajectory functions.", "Caption",
 CellID->1891092685],

Cell["\<\
Here is a plot of the path of a ball with initial velocity 45 MPH launched at \
an angle of 42\[Degree] to the ground and no air resistance.\
\>", "MathCaption",
 CellID->836781195],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ParametricPlot", "[", 
  RowBox[{
   RowBox[{"Trajectory", "[", 
    RowBox[{"t", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"45", " ", "MPH"}], ",", 
       RowBox[{"42", " ", "Degree"}], ",", "32"}], "}"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", "3"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->2058623809],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztXKGyozAU7cx+ycqVT+3UPlmJfHKRT6xAYiuRlbVI3EbGIpHIWCSysnsg
QCltIQFCCM2ZfV3K9JJwc3Jyb0j4+eev8+fHbrf7jb9/+CuOrxYWFhYWFkqR
BvtdgX2Q8hPEvftq8b5I05IExOVsADNc0vpu8fZIg4Dw//clNbrk2EYom6Z5
/S/DJ6UsilJ+3JzPc921XBOqUaWSi90rbnBoqqMc0MpxnIXn5HRKvr/J4RB+
7s/4xLHnEa/4pDgOgvh8Tvhxc95xws/P4sc49n0ahikh4E9+uei+K22oSGGo
bmTZhRCGtkb7omXR1scjb1Y2ullhmCRZGBYEA3OcQ/jlRMdjjDOMvZW4pIHb
F2+sUDTQ4pQwNBZazf2KQAzOBHUl5vkljhko910zEEPShnnCs5J2XvI0T1kP
MdBA6LloF4g/+BBTpkvtwUMMSbeaxJmeeuiGdt0AJdAQrht9fUXouavqrZWC
+fRwOL8hSTQSA9kE+ibGegg44gpd1RAEaothDqEsopT113YWLK8bcCzci1gC
PVFpFKECUBJkN1A5sNpkGakijv550SWJgewAeYHrEsSWixWqCGA1MiaIHmIk
47LgetILjCjpoDVPARkQTvg+NU4o+lHGzykPWU0caDgZdM1vcFbAdXCj0oL0
AtFIfZu6qyKBQsOvvfOiigpGhAl3nU6xcZI7GmAI1xAjOkI9svTpxuzDCoI0
DMTITN+HFW2AIeiPuH3dFekD+MBDjQCfr+ONOUtMcz6t/Z6saIPL5jqj7npe
tHna9jJPmYUeIMMpiEGMjUWbU9DyiZHZ7izEICvuI9rBWO551EQtnagbUAmM
rafTqsfWNaCIwUzrPlOIwR9IZZkdRESBrK30mBkKMk43oJPoBQi3VFRp2+Dh
ehTqdR1f9lVlrtf55sy5XBiRwq8WteRq9OFtVmOWOXPcC+4o1Mz5jQDa67oa
I5AbN6bPmZfPH8mq1ldsAIhAfJ/qEOE2IyatJfY9apMRRUiSTMccyAy6kbEL
pA/1V1ZJi+J5rufRMFyy902NNyhhZdipuJoWJRCgYnxZpqzO4h7ZPAWDSBDE
iutocYckznTnLzc81Q0ucWZN4m0GPBlcwyD+SIxicsa1j8w0g++801uHDjco
ZVAM454KbRIITY9HnWN6e0xBrm0DjFUhjpnGKeiGGBAxG2CsEMUj/m89U45c
N4rgZ9Rui4nLP/Saa6+AoHmZGjyJTlWvA+fEaJKmx+I6Z6S+qv79totrn0EE
CGGfWJwgdhbvgRHcmEgtC10Iz+liK9ins8tiYURRGiyS21pimAhK2QKzDVY3
DEU5S0mGfzcB8sQQ2p4/gPvnw7LmfLFj82pMGfPuS1eFzQcWWA5dp2Mu68Nu
6fzcLyfkz20VvTlWVjcEt+f3oryVZj+VpHm9Wa++kpR5zcnKTs789YIHoeu0
19KM8GGr9Mq28GGSZL5zkG8CVWuJ66v1bc/vNXRJbSZtXvf7Xe/LD/sKr0gx
wvzlQimx63Qa9/bzEeZtHxLv48Ohwnfx7IIzv39jYHv+gF2n/wqbN6LBu5x0
6denoiXNjY6h2HUeuSHlw7um7PjQCbJicBnLjXnfvzG4Pf91fW7TMsI97s5e
vsO2wd1aible3ZD0YfOr5z5MkxyDyzhuzPheYpHt+aL1kjWvh+jqAiPM67Fs
L1352eKN6xgfPsjOgw8D5+PTfVhS2N4zfxdSzK8bgtvzBzAhT6kqMNK86XS3
LiNo3r/AcvA6bfMRPuwmNtfnPozjzPcEV5yqizcsVgpKmciCsYlriS0sLCws
LCwsGvwHy98Jxw==
    "], {{0, 55}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 55},
  PlotRange->{{0, 180}, {0, 55}}]], "Output",
 ImageSize->{184, 55},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[2]=",
 CellID->617193793]
}, Open  ]],

Cell["\<\
Adding air resistance shortens the path. For a reasonable simulation of a \
baseball path k should have a value around .002.\
\>", "MathCaption",
 CellID->50578928],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ParametricPlot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Trajectory", "[", 
      RowBox[{"t", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"45", " ", "MPH"}], ",", 
         RowBox[{"42", " ", "Degree"}], ",", "32"}], "}"}]}], "]"}], ",", 
     RowBox[{"Trajectory", "[", 
      RowBox[{"t", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"45", " ", "MPH"}], ",", 
         RowBox[{"42", " ", "Degree"}], ",", "32", ",", ".002"}], "}"}]}], 
      "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", "3"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->1824002331],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztXC2XszgY7Tn7S1aOHDkrKyuRlS+yYgUSi4yMjI2M28jYyMhILBKJ7N40
0On0C0JpgSn3vMPbcnggDZf7fCTh7z//Rn/+Wq1W/+DvP/y5z/sFCxYsWLDg
qbDka+XwRazfIeMfXxe8L6w9kEDGng1gRixPvtdYwpX3hSVE+v+/DtQ4J8fv
4Ia1ZfOvwFapXAjrPx/3l+XYrZwSaq9Sy8XqFjc8RmpjGHCXtS44M5Sa3U5u
Nnz9xbDF5ySRidsqfCZEM2b85+P+KOLrtTsYn9NUcW6lBH/Kqhr7V42GmhQz
1Y2iqKTMca9xf3Fn01iQRAiiRCoUlYYpy5XhP7alLQqb78ubtxx8MKbg3BEM
zIk2fBuJLNPYk+dvJS6WxC3xxtTogQdZgQ+ZiUGGiNEt5TFl65StE7mjKmEq
444GnhgXW38MjzIcj61KGfYX2la32VKWldY5lGR3YCC2cEm/mCc+KznNS67m
KdMhBm6Q4CaLebohLCKOCQnTROQK4ULe96RVYXIQRqVcRJnYZv6E+7t+BMIC
lwSGwA1BsuDFel595hhdN0AJRnQWUeKUIQUfrNBl/pTbAenIpdEZ55vDhaTe
3402vIJlqdps2BuSZERiKGnJltF1yiOiqXwSH24BMYmh0jmdhOXKtB6P3Acx
CUJZRCmIgl7QwtHxet1Ax7JUOJXYZIrIqhjZrcPpwNFASTQVVVuiCyVBdhPH
Ah5nzjJSRxz366KvJIbROXGBRMp37MUq0QVwMWJLEJkg02k/2JZZppDgILuZ
XRbcFL3AiAMdRs1TkHWSTQb3oVmLix8dyGh8pgM9aT0YwRKyGx+yztHReDKM
Vd+QVNN1xrbUZQfzATIj0EMmtKO+IRrZbgUYMqvqq4zb6qJPurAgkkIrtqSc
baEAzgUagrzmTm3kFGCI15Cy2/HjovEs93RjcLeiuEGoyWPaxXFPH1A8GRPL
VcfjwRA8j4y15z4jAnzwoQbB9iXjsEZZuiE0yuarFddRVYYp4TSwK9uFsPAy
Uvat2j0TTV30ONp2M08Zhh7VnsUUTkSLST8vjwCeBS4GJOl6fLWnRCPbtfPU
z0GIITKJHETU7uuXww3ZxF1jVCDPyyRRSHinnZ9dwYO6YXQOrWAxG7BJ00dV
lBAQV3LvDK2LybqYW3iEGGzHQAyr55ScDghDpcr4/WG7M1DqXMxcKiH9dMPq
HJmIyMQzmjQj+EJZxwzXw9rSzQHg4z5QftpXfAwChqqZsx2HXBTTK3qPAu9f
ikDx9HMARhWQ76rGIDXz3BZgBU/eXS4uoVJuRUD4sT/EqHE8YgTyzY3Ha+aC
CBDDzKr0/UoYJjUJfmoQgaSpGqOOesqIh+YS04i8WzLSA7lyE4dCrYwpxqiB
DKAbuXJhp+Zhgvm2KIyVCQsda4ZuJIni/JVlw0fjDXkYLyvzeeRcE0FpC+Xo
EWyIABX+5QktuoKzyT2heYqrgW/pk9v4O1HmxSG3DR5UMroYO3/5xlXdKPKS
RUSQF3H4VwK5LdQjqPThAWKAHghCntGqIFwSw0i75CODwJU+HD36DEn7lXeD
NykIZ9xwAUZEerB9wVWAGP3UA0BommVjpgCnPgVZKmKMERvzK+HUA73aaxRW
6xz+ZaxZZEdiQC7kewyyvx4ITftlLns/xL+ToyzA9Lrh5uT0qmA8OP1jXPNX
NqAwuUrPi4cdzQ/VjyvR6bPngXtiFM11Ly93tifo67OPn9flQI/e54dHQnT6
YPM6YrXgPdCDGw9Sa0Fv+PeE9DbnzL5sBvvj7FoQCveeB9n//gphyUty24UY
owBpyyOLd5Ry7zIasD1XsejGWBDxQzVG0APJy4DtuUQ4MTotz2/Bz/HhUHM/
2fH4aswQ8/OXrnY2b5lg2XaeM/O6D//+SnuZ1/s+Iu7HbZ/05thQ3ei4PP8u
Dj/luJ4q0LxZrNecKci84WRtF2Z+e8JDp/OczqWp+zBafex2PNS8aYHrQ2OK
NNqE34JnzSVuznZvef5dw1g2ZsHmzXO/uvvyw3sXr0nRw/zmRKlu5zm7ufXh
SSpl/BFqftqHMvn8jFTnX3HthAO/f6NleX6L3dnz29n8KBpetoKvvr8qWsHc
ODPsdp5LbtR9mH2strQIML/ow4gUzrn05caw799oXZ5/uz3fZZnOT9wP+/AH
9hS+W2uHOK5uHPvQZJ/rDXWjLZ3Mr/ehNSWcSz9uDPhe4i7L87u2K9S8CXPq
E/Qwb3zZV3DjB4s39pd9uKEq5SHxxpU+JNHnOr6oqp2umf8RUgyvGx2X57fg
gTylbkBP8+ND9/3IdDS/P8Gy9Tyn5rf68DO+OUf9PDncX+9DrYs06Vh0fV68
sWB4yF3A8vxbUCrvMmHswbnEC16MqqxETMZuxTcWbkwKh9VPU1lSuviUqeEw
1DKJ13QsxJganGfZkim8hXXRjQkiV3YKnmUhxjShMh76To/BsejGdDGEV/kf
HwSd3A==
    "], {{0, 58}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 58},
  PlotRange->{{0, 180}, {0, 58}}]], "Output",
 ImageSize->{184, 58},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[3]=",
 CellID->800251831]
}, Open  ]],

Cell["\<\
Typical questions would include \"How high did the ball go?\", \"At what time \
did it reach its highest point?\" and \"How long was it in the air?\".\
\>", "Text",
 CellID->403124315],

Cell["Find the derivative of the vertical component.", "MathCaption",
 CellID->883716184],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"Trajectory", "[", 
    RowBox[{"t", ",", " ", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"45", " ", "MPH"}], ",", " ", 
       RowBox[{"42", " ", "Degree"}], ",", " ", "32", ",", " ", ".002"}], 
      "}"}], ",", "\"\<Vertical\>\""}], "]"}], ",", "t"}], "]"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->1593065675],

Cell[BoxData[
 RowBox[{"126.4911064067352`", " ", 
  RowBox[{"Tan", "[", 
   RowBox[{"0.33590505029475204`", "\[InvisibleSpace]", "-", 
    RowBox[{"0.2529822128134704`", " ", "t"}]}], "]"}]}]], "Output",
 ImageSize->{223, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->1976172576]
}, Open  ]],

Cell["Set the derivative equal to zero.", "MathCaption",
 CellID->621001836],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sol1", "=", 
  RowBox[{"Solve", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{"Trajectory", "[", 
        RowBox[{"t", ",", " ", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"45", " ", "MPH"}], ",", " ", 
           RowBox[{"42", " ", "Degree"}], ",", " ", "32", ",", " ", ".002"}], 
          "}"}], ",", "\"\<Vertical\>\""}], "]"}], ",", "t"}], "]"}], 
     "\[Equal]", "0"}], ",", "t"}], "]"}]}]], "Input",
 CellLabel->"In[5]:=",
 CellID->1366278681],

Cell[BoxData[
 RowBox[{
  RowBox[{"Solve", "::", "\<\"ifun\"\>"}], ":", 
  " ", "\<\"\\!\\(\\*StyleBox[\\\"\\\\\\\"Inverse functions are being used by \
\\\\\\\"\\\", \\\"MT\\\"]\\)\[NoBreak]\\!\\(\\*StyleBox[\\!\\(Solve\\), \
\\\"MT\\\"]\\)\[NoBreak]\\!\\(\\*StyleBox[\\\"\\\\\\\", so some solutions may \
not be found; use Reduce for complete solution information.\\\\\\\"\\\", \
\\\"MT\\\"]\\) \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/Solve/ifun\\\", ButtonNote -> \
\\\"Solve::ifun\\\"]\\)\"\>"}]], "Message", "MSG",
 CellLabel->"During evaluation of In[5]:=",
 CellID->2072772739],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"t", "\[Rule]", "1.3277812956060377`"}], "}"}], "}"}]], "Output",
 ImageSize->{105, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->611702739]
}, Open  ]],

Cell["\<\
Substitute the value into the vertical component to find the maximum height.\
\>", "MathCaption",
 CellID->407333232],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Trajectory", "[", 
   RowBox[{"t", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"45", " ", "MPH"}], ",", " ", 
      RowBox[{"42", " ", "Degree"}], ",", " ", "32", ",", " ", ".002"}], 
     "}"}], ",", "\"\<Vertical\>\""}], "]"}], "/.", "sol1"}]], "Input",
 CellLabel->"In[6]:=",
 CellID->2019247235],

Cell[BoxData[
 RowBox[{"{", "28.75504182041602`", "}"}]], "Output",
 ImageSize->{60, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->804205104]
}, Open  ]],

Cell["\<\
And a good question to see if students really understand what is going on is \
to ask something that does not require taking a derivative. This brings us to \
\"How long was it in  the air?\".\
\>", "Text",
 CellID->129125345],

Cell["Finding when the ball returns to the ground.", "MathCaption",
 CellID->1114352678],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sol2", "=", 
  RowBox[{"Solve", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Trajectory", "[", 
      RowBox[{"t", ",", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"45", " ", "MPH"}], ",", " ", 
         RowBox[{"42", " ", "Degree"}], ",", " ", "32", ",", " ", ".002"}], 
        "}"}], ",", "\"\<Vertical\>\""}], "]"}], "\[Equal]", "0"}], ",", 
    "t"}], "]"}]}]], "Input",
 CellLabel->"In[7]:=",
 CellID->1251189945],

Cell[BoxData[
 RowBox[{
  RowBox[{"Solve", "::", "\<\"ifun\"\>"}], ":", 
  " ", "\<\"\\!\\(\\*StyleBox[\\\"\\\\\\\"Inverse functions are being used by \
\\\\\\\"\\\", \\\"MT\\\"]\\)\[NoBreak]\\!\\(\\*StyleBox[\\!\\(Solve\\), \
\\\"MT\\\"]\\)\[NoBreak]\\!\\(\\*StyleBox[\\\"\\\\\\\", so some solutions may \
not be found; use Reduce for complete solution information.\\\\\\\"\\\", \
\\\"MT\\\"]\\) \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/Solve/ifun\\\", ButtonNote -> \
\\\"Solve::ifun\\\"]\\)\"\>"}]], "Message", "MSG",
 CellLabel->"During evaluation of In[7]:=",
 CellID->720573659],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"t", "\[Rule]", "8.126569997615275`*^-16"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", "\[Rule]", "2.6555625912120746`"}], "}"}]}], "}"}]], "Output",
 ImageSize->{244, 18},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->1021201380]
}, Open  ]],

Cell["\<\
Using the second solution with the horizontal component tell us how far the \
ball traveled.\
\>", "MathCaption",
 CellID->642286836],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Trajectory", "[", 
   RowBox[{"t", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"45", " ", "MPH"}], ",", " ", 
      RowBox[{"42", " ", "Degree"}], ",", " ", "32", ",", " ", ".002"}], 
     "}"}], ",", "\"\<Horizontal\>\""}], "]"}], "/.", 
  RowBox[{"sol2", "[", 
   RowBox[{"[", "2", "]"}], "]"}]}]], "Input",
 CellLabel->"In[8]:=",
 CellID->1662042830],

Cell[BoxData["115.75333060415693`"], "Output",
 ImageSize->{53, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->1634047281]
}, Open  ]],

Cell["These results fit nicely with a plot of the trajectory.", "MathCaption",
 CellID->1411773931],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ParametricPlot", "[", 
  RowBox[{
   RowBox[{"Trajectory", "[", 
    RowBox[{"t", ",", " ", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"45", " ", "MPH"}], ",", " ", 
       RowBox[{"42", " ", "Degree"}], ",", " ", "32", ",", " ", ".002"}], 
      "}"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", "2.66"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[9]:=",
 CellID->137642614],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztXKuWo0AQzTn7JfsZsZEjIyMHiViBjEVGImORuG2JRSJbYpHIkdnbPLKB
PIZH0Y9M3TOvZbI1ne5bt6q6Gn5//tl//tpsNlt8/sWn+vkyDuNfyfhp2NQw
PYoVIWWVZWUcyzjOwzANAhEE6ut+n+y2590u3u3O6mN79jxx/W0U5Xh9mhZS
llX1ZfpNmMGbcQNMSBJ5OmVY4o8PteiBL8Jjej5jrSVIghfkufp6v+JlWeG6
rH8rBLgkT6GyczgksON5SWMH//3LebIIr1737Um2F+Rp279wcZ8bWKY8r7Bk
WESsoO8rl08F/L2i/UNl+dXoD/7Qfh+DMGGYgUIuqgqcR6jvYEhLBnG6JUUL
R4kB5wUf4MtYJoQAaAWu6BwAKIGIAx6CJN4hiaKsKIjZqAHCq7nRioYn+r91
SzewHPBW6MPxmCaxhC+bHpEChiFEAclCDoNwRi5Zq0F4t3RQDOnRY9NB+8Am
QFHiqPIHzDx+Nj2cp4CYdCSJIWuWh5sustxe8Qb5ht4RTUAhK5AB84ycELHe
9HAmAKxAmEO4gb7ZSWaIRB1OeolGX0dsjClIL5uJhQMiqzQ9nEVAFoQguP+I
kcTaU920VcqmTTLadOMu4bCKGwjcEIrDPoEgW5JLkACsiGOVtSJ3tYMhwxq2
+fe2X6xYwg34F+QXs2enApOg1UOkIpHhVGRYw+J7LRlt2dLBODFABq+Oy5qL
UINoIia00fRAWjIgprTRpE8Og7qBXKLZQXqn8DEeYAjSbHw1N4Q29+xk4zE3
NNMDEgFWILWwvMrTAGQgWCAjmnmtYV/ohs7xYBIwFYggP1MrHqIsK0wIPnSm
qb0a9nm+oYceIMNRtThTF/eWNaDRUj0hZlDDXszVKQgctXImPyfbnA0kqBCQ
9UNts6Vxs59hog8bxxJBRDi+haUTddjVUMv/zzIu2vuwUlZNpWbHbo9LwIzV
R1DEmlN3ww2NfdjrW+MyZAnyXDmXzFdKz3q60V5YuQ+L8OEdEtf7IJYAXgYX
W2eX7I4ba/Zh8UZQifCuBTnUYTOfPL484MZKfVjkThBArkRWApI3rBvhqaHb
Ztt6fVioBMouyMWisTK+A+Y5CFKiim9Yw66xv5FlJW9c6EQUZURueBNTVujD
YpBhmHGJqhnI85HXLTbznxu0fdiiqN77rIXlgFD7/sItgltGkPVhk0T69Gkz
YxpADKzCgpblKN0Yb05VqfW9XXPHw6CE2v3wxdze5ah8YyQ9ClVGcdppHYJA
5NP3TgcHRpfUKUJI3gO3FgR3Osztw6IeiSKOI1Zj4V1dM/qwiGjqJhGuR1zA
KZxLj+l9WCQ5SDD48J5DWKQeo/uwWVZyoeoi5qvHuD5sfJZhuHznjWEGs9Xj
2z5seEzjmDNPtzFJPcb0YVWnzxdu3bHOeIblz6O4cgOZpz9/k41hI9SdpAs8
vSEGtILwZNEa55NdsWkbgmC+vze6gRxjcPHhK/micxfh77hYFhPcfsP4wRhP
kvGMYjiKoviS3CRlMBgMBoPxthj13Lk56J9cI7HZbBE3dihsdlvOL89QTTH1
9LYRslnViJHPnZuOejKuTw+hsNk9r6Yzv9xmx97W2FKbz49xks2qGbx+7tx0
a57obNHYvHaV6lkmGueVFCQ2nx7/pppVQ/jmuXNzjA28cpnNq2g0AkczzstD
eSPgxsAU2WhN4Nvnzk0z1nl4E2FJvGYdT2xI3MZT1o17jHnu3Ey7hPlGlxC1
VqlsdlFvS/De3y3fGPncuTmgrlPaoVLaHB6NWWLz9W0jLtYpDAaDwXhv/AP9
beaL
    "], {{0, 57}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 57},
  PlotRange->{{0, 180}, {0, 57}}]], "Output",
 ImageSize->{184, 57},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[9]=",
 CellID->585115497]
}, Open  ]],

Cell["", "Text",
 CellID->550400330],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["Classroom Utilities",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/guide/ClassroomUtilities"]], "TutorialMoreAbout",
 CellID->1199210299]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED WOLFRAM EDUCATION GROUP COURSES", "TutorialRelatedLinksSection",
 WholeCellGroupOpener->True,
 CellID->415694148],

Cell[TextData[{
 ButtonBox["M50",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[":",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["An",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["Introduction",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["to",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["Mathematica",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["in",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["the",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["Classroom",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"]
}], "TutorialRelatedLinks",
 CellID->790177859]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Falling Body Problems - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "TutorialColor", 
  "LinkTrails" -> GridBox[{{
      RowBox[{
        StyleBox[
         ButtonBox[
         "Mathematica", ButtonData -> "paclet:guide/Mathematica", 
          BaseStyle -> {"Link", "DockedLinkTrail"}], FontSlant -> "Italic"], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 32, 36.6364624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Tutorial", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "The Trajectory functions in the ClassroomUtilities` package are intended \
to allow students to compare what happens to object traveling through air \
when we consider the affects of air resistance. This loads the package. \
Trajectory functions.", "synonyms" -> {}, "title" -> "Falling Body Problems", 
    "titlemodifier" -> "", "windowtitle" -> "Falling Body Problems", "type" -> 
    "Tutorial", "uri" -> "ClassroomUtilities/tutorial/Falling Body Problems"},
   "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> Notebook], 
   Cell[
    StyleData["Output"], CellContext -> Notebook], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "RasterizedOutput"->{
  Cell[6324, 188, 2365, 47, 76, "Output",
   CellTags->"RasterizedOutput",
   CellID->617193793],
  Cell[9576, 269, 3028, 58, 79, "Output",
   CellTags->"RasterizedOutput",
   CellID->800251831],
  Cell[19136, 548, 2353, 47, 78, "Output",
   CellTags->"RasterizedOutput",
   CellID->585115497]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"RasterizedOutput", 27921, 762}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[614, 21, 2533, 58, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3172, 83, 58, 1, 51, "Title",
 CellID->509267359],
Cell[3233, 86, 244, 5, 70, "Text",
 CellID->1534169418],
Cell[3480, 93, 66, 1, 31, "MathCaption",
 CellID->100270670],
Cell[3549, 96, 107, 3, 28, "Input",
 CellID->1621971442],
Cell[3659, 101, 1994, 60, 172, "DefinitionBox",
 CellID->2096742444],
Cell[5656, 163, 61, 1, 31, "Caption",
 CellID->1891092685],
Cell[5720, 166, 191, 4, 31, "MathCaption",
 CellID->836781195],
Cell[CellGroupData[{
Cell[5936, 174, 385, 12, 28, "Input",
 CellID->2058623809],
Cell[6324, 188, 2365, 47, 76, "Output",
 CellTags->"RasterizedOutput",
 CellID->617193793]
}, Open  ]],
Cell[8704, 238, 174, 4, 31, "MathCaption",
 CellID->50578928],
Cell[CellGroupData[{
Cell[8903, 246, 670, 21, 47, "Input",
 CellID->1824002331],
Cell[9576, 269, 3028, 58, 79, "Output",
 CellTags->"RasterizedOutput",
 CellID->800251831]
}, Open  ]],
Cell[12619, 330, 194, 4, 70, "Text",
 CellID->403124315],
Cell[12816, 336, 89, 1, 70, "MathCaption",
 CellID->883716184],
Cell[CellGroupData[{
Cell[12930, 341, 378, 11, 70, "Input",
 CellID->1593065675],
Cell[13311, 354, 336, 9, 36, "Output",
 CellID->1976172576]
}, Open  ]],
Cell[13662, 366, 76, 1, 70, "MathCaption",
 CellID->621001836],
Cell[CellGroupData[{
Cell[13763, 371, 533, 16, 70, "Input",
 CellID->1366278681],
Cell[14299, 389, 674, 12, 70, "Message",
 CellID->2072772739],
Cell[14976, 403, 252, 8, 36, "Output",
 CellID->611702739]
}, Open  ]],
Cell[15243, 414, 127, 3, 70, "MathCaption",
 CellID->407333232],
Cell[CellGroupData[{
Cell[15395, 421, 347, 10, 70, "Input",
 CellID->2019247235],
Cell[15745, 433, 197, 6, 36, "Output",
 CellID->804205104]
}, Open  ]],
Cell[15957, 442, 236, 5, 70, "Text",
 CellID->129125345],
Cell[16196, 449, 88, 1, 70, "MathCaption",
 CellID->1114352678],
Cell[CellGroupData[{
Cell[16309, 454, 460, 14, 70, "Input",
 CellID->1251189945],
Cell[16772, 470, 673, 12, 70, "Message",
 CellID->720573659],
Cell[17448, 484, 355, 11, 39, "Output",
 CellID->1021201380]
}, Open  ]],
Cell[17818, 498, 143, 4, 70, "MathCaption",
 CellID->642286836],
Cell[CellGroupData[{
Cell[17986, 506, 401, 12, 70, "Input",
 CellID->1662042830],
Cell[18390, 520, 177, 5, 36, "Output",
 CellID->1634047281]
}, Open  ]],
Cell[18582, 528, 99, 1, 70, "MathCaption",
 CellID->1411773931],
Cell[CellGroupData[{
Cell[18706, 533, 427, 13, 28, "Input",
 CellID->137642614],
Cell[19136, 548, 2353, 47, 78, "Output",
 CellTags->"RasterizedOutput",
 CellID->585115497]
}, Open  ]],
Cell[21504, 598, 36, 1, 70, "Text",
 CellID->550400330],
Cell[CellGroupData[{
Cell[21565, 603, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[21633, 606, 180, 4, 70, "TutorialMoreAbout",
 CellID->1199210299]
}, Open  ]],
Cell[CellGroupData[{
Cell[21850, 615, 127, 2, 70, "TutorialRelatedLinksSection",
 CellID->415694148],
Cell[21980, 619, 3506, 82, 70, "TutorialRelatedLinks",
 CellID->790177859]
}, Open  ]]
}, Open  ]],
Cell[25513, 705, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

