(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     65653,       1614]
NotebookOptionsPosition[     59406,       1422]
NotebookOutlinePosition[     62359,       1496]
CellTagsIndexPosition[     62279,       1491]
WindowTitle->Integrals As Limits - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Classroom Utilities\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/guide/ClassroomUtilities"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"TutorialMoreAboutSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Functions \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"RiemannSum\"\>":>Documentation`HelpLookup[
       "paclet:IntegrationTechniques/ref/IntegralEst"], "\<\"RiemannSumPlot\"\
\>":>Documentation`HelpLookup["paclet:IntegrationTechniques/ref/IPlot"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"TutorialFunction"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"ClassroomUtilities/tutorial/IntegralsAsLimits\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["ClassroomUtilities/tutorial/IntegralsAsLimits"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/ClassroomUtilities/\
tutorial/IntegralsAsLimits.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "ClassroomUtilities/tutorial/IntegralsAsLimits", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["Integrals As Limits", "Title",
 CellID->509267359],

Cell[TextData[{
 "Functions for developing an understanding of integrals as limiting cases of \
known geometric forms are included in the package ",
 Cell[BoxData["ClassroomUtilities`"], "InlineFormula"],
 ". Rectangles and trapezoids of uniform and random widths are included in a \
single function ",
 Cell[BoxData[
  ButtonBox["RiemannSumPlot",
   BaseStyle->"Link",
   ButtonData->"paclet:ClassroomUtilities/ref/IPlot"]], "InlineFormula"],
 ". "
}], "Text",
 CellID->1534169418],

Cell["This load the package:", "MathCaption",
 CellID->1086341370],

Cell[BoxData[
 RowBox[{"<<", "ClassroomUtilities`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->233438796],

Cell[CellGroupData[{

Cell["Visualizing Integrals as Limits", "Subsection",
 CellID->2100704950],

Cell[TextData[{
 "The function ",
 Cell[BoxData[
  ButtonBox["RiemannSumPlot",
   BaseStyle->"Link",
   ButtonData->"paclet:ClassroomUtilities/ref/IPlot"]], "InlineFormula"],
 " plots the function along with a number of polygons used to approximate the \
area between the function and the axis. ",
 Cell[BoxData[
  ButtonBox["RiemannSum",
   BaseStyle->"Link",
   ButtonData->"paclet:ClassroomUtilities/ref/IntegralEst"]], "InlineFormula"],
 " can be used with ",
 Cell[BoxData[
  ButtonBox["RiemannSumPlot",
   BaseStyle->"Link",
   ButtonData->"paclet:ClassroomUtilities/ref/IPlot"]], "InlineFormula"],
 " to give the area of the polygons in the plot. "
}], "Text",
 CellID->1916937970],

Cell[BoxData[GridBox[{
   {
    RowBox[{
     ButtonBox["RiemannSumPlot",
      BaseStyle->"Link",
      ButtonData->"paclet:IntegrationTechniques/ref/IPlot"], "[", 
     RowBox[{
      StyleBox["fun", "TI"], ",", 
      RowBox[{"{", 
       RowBox[{
        StyleBox["var", "TI"], ",", 
        SubscriptBox[
         StyleBox["var", "TI"], 
         StyleBox["min", "TI"]], ",", 
        SubscriptBox[
         StyleBox["var", "TI"], 
         StyleBox["max", "TI"]]}], "}"}], ",", 
      StyleBox["n", "TI"], ",", "\"\<\!\(\*
StyleBox[\"type\", \"TI\"]\)\>\""}], "]"}], 
    RowBox[{"plot", " ", "the", " ", "function", " ", 
     StyleBox["fun", "TI"], " ", "over", " ", 
     SubscriptBox[
      StyleBox["var", "TI"], 
      StyleBox["min", "TI"]], " ", "to", " ", 
     SubscriptBox[
      StyleBox["var", "TI"], 
      StyleBox["max", "TI"]], " ", "with", " ", 
     StyleBox["n", "TI"], " ", "polygons", " ", "of", " ", "the", " ", "form",
      " ", "\"\<\!\(\*
StyleBox[\"type\", \"TI\"]\)\>\""}]},
   {
    RowBox[{
     ButtonBox["RiemannSumPlot",
      BaseStyle->"Link",
      ButtonData->"paclet:IntegrationTechniques/ref/IPlot"], "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        SubscriptBox[
         StyleBox["f", "TI"], 
         StyleBox["1", "TR"]], ",", 
        SubscriptBox[
         StyleBox["f", "TI"], 
         StyleBox["2", "TR"]]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        StyleBox["var", "TI"], ",", 
        SubscriptBox[
         StyleBox["var", "TI"], 
         StyleBox["min", "TI"]], ",", 
        SubscriptBox[
         StyleBox["var", "TI"], 
         StyleBox["max", "TI"]]}], "}"}], ",", 
      StyleBox["n", "TI"], ",", "\"\<\!\(\*
StyleBox[\"type\", \"TI\"]\)\>\"", ",", 
      StyleBox["opts", "TI"]}], "]"}], Cell[TextData[{
     "plot the functions ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["f", "TI"], 
       StyleBox["1", "TR"]]], "InlineFormula"],
     " and ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["f", "TI"], 
       StyleBox["2", "TR"]]], "InlineFormula"],
     " approximating the area between the curves approximated by ",
     Cell[BoxData[
      StyleBox["n", "TI"]], "InlineFormula"],
     " polygons of ",
     Cell[BoxData["\"\<\!\(\*
StyleBox[\"type\", \"TI\"]\)\>\""], "InlineFormula"],
     "."
    }], "TableText"]},
   {
    RowBox[{
     ButtonBox["RiemannSum",
      BaseStyle->"Link",
      ButtonData->"paclet:IntegrationTechniques/ref/IntegralEst"], "[", 
     RowBox[{
      StyleBox["fn", "TI"], ",", 
      RowBox[{"{", 
       RowBox[{
        StyleBox["var", "TI"], ",", 
        SubscriptBox[
         StyleBox["var", "TI"], 
         StyleBox["min", "TI"]], ",", 
        SubscriptBox[
         StyleBox["var", "TI"], 
         StyleBox["max", "TI"]]}], "}"}], ",", 
      StyleBox["n", "TI"], ",", "\"\<\!\(\*
StyleBox[\"type\", \"TI\"]\)\>\"", ",", 
      StyleBox["opts", "TI"]}], "]"}], Cell["\<\
give the area of the polygons used to approximate the area under a curve.\
\>", "TableText"]},
   {
    RowBox[{
     ButtonBox["RiemannSum",
      BaseStyle->"Link",
      ButtonData->"paclet:IntegrationTechniques/ref/IntegralEst"], "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        SubscriptBox[
         StyleBox["f", "TI"], 
         StyleBox["1", "TR"]], ",", 
        SubscriptBox[
         StyleBox["f", "TI"], 
         StyleBox["2", "TR"]]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        StyleBox["var", "TI"], ",", 
        SubscriptBox[
         StyleBox["var", "TI"], 
         StyleBox["min", "TI"]], ",", 
        SubscriptBox[
         StyleBox["var", "TI"], 
         StyleBox["max", "TI"]]}], "}"}], ",", 
      StyleBox["n", "TI"], ",", "\"\<\!\(\*
StyleBox[\"type\", \"TI\"]\)\>\"", ",", 
      StyleBox["opts", "TI"]}], "]"}], Cell["\<\
give the area of the polygons used to estimate the area between two functions.\
\>", "TableText"]}
  }]], "DefinitionBox",
 CellID->2096742444],

Cell["Plotting approximations to integrals.", "Caption",
 CellID->1891092685],

Cell["\<\
For functions that are strictly positive or strictly negative all polygons \
will have the same color. For functions that take positive and negative \
values the polygons contributing positive area will have one color and those \
contributing negative area will have a second color.\
\>", "Text",
 CellID->1354038939],

Cell["\<\
A strictly positive function using the left end of the sub-intervals to \
determine height of the rectangles.\
\>", "MathCaption",
 CellID->836781195],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSumPlot", "[", 
  RowBox[{
   RowBox[{"1", " ", "+", " ", 
    RowBox[{
     SuperscriptBox["x", "2"], "/", "3"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "4"}], "}"}], ",", "8", ",", "\"\<Left\>\""}],
   "]"}]], "Input",
 CellLabel->"In[12]:=",
 CellID->2058623809],

Cell[BoxData[
 GraphicsBox[{{
    {RGBColor[1, 0.95, 0], Opacity[0.4], EdgeForm[{GrayLevel[0.5], 
     AbsoluteThickness[0.4]}], RectangleBox[{0., 0}, {0.5, 1.}], 
     RectangleBox[{0.5, 0}, {1., 1.0833333333333333`}], 
     RectangleBox[{1., 0}, {1.5, 1.3333333333333333`}], 
     RectangleBox[{1.5, 0}, {2., 1.75}], 
     RectangleBox[{2., 0}, {2.5, 2.333333333333333}], 
     RectangleBox[{2.5, 0}, {3., 3.083333333333333}], 
     RectangleBox[{3., 0}, {3.5, 4.}], 
     RectangleBox[{3.5, 0}, {4., 5.083333333333333}]}, 
    {RGBColor[0.9, 0, 1.], Opacity[0.4], EdgeForm[{GrayLevel[0.5], 
     AbsoluteThickness[0.4]}]}}, {{}, {}, 
    {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwV03k01OsbAHDLYBBfRM1MIWslTVqElOeNboQWS0nLRLRNhUpFinRR3Wzd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      "]]}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->True,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{278, 432},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->988531026]
}, Open  ]],

Cell["\<\
A function taking both positive and negative values using the center of the \
sub-intervals to determine the size of the polygons.\
\>", "MathCaption",
 CellID->1648657431],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSumPlot", "[", 
  RowBox[{
   RowBox[{
    SuperscriptBox["x", "3"], "/", "2"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "2"}], ",", "2"}], "}"}], ",", "11", ",", 
   "\"\<Center\>\""}], "]"}]], "Input",
 CellLabel->"In[13]:=",
 CellID->3945576],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnS+Xs8qehc+s+STnY7SNjEQiLxIxAolFlkSWRbLWmJJYJBI3WCQSyRSh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    "], {{0, 432}, {221, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{221, 432},
  PlotRange->{{0, 221}, {0, 432}}]], "Output",
 ImageSize->{225, 432},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[13]=",
 CellID->100483496]
}, Open  ]],

Cell["\<\
With the \"type\" set to Trapezoids some trapezoids may be split into two \
triangles where the function crosses the axis.\
\>", "MathCaption",
 CellID->674464701],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSumPlot", "[", 
  RowBox[{
   RowBox[{"Sin", "[", "t", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", 
     RowBox[{"\[Pi]", "/", "2"}], ",", 
     RowBox[{"3", " ", 
      RowBox[{"\[Pi]", "/", "2"}]}]}], "}"}], ",", "5", ",", 
   "\"\<Trapezoid\>\""}], "]"}]], "Input",
 CellLabel->"In[14]:=",
 CellID->1309540841],

Cell[BoxData[
 GraphicsBox[{{
    {RGBColor[1, 0.95, 0], Opacity[0.4], EdgeForm[{GrayLevel[0.5], 
     AbsoluteThickness[0.4]}], 
     PolygonBox[{{{1.5707963267948966`, 0}, {1.5707963267948966`, 1.}, {
        2.199114857512855, 0.8090169943749475}, {2.199114857512855, 0}}, {{
        2.199114857512855, 0}, {2.199114857512855, 0.8090169943749475}, {
        2.827433388230814, 0.3090169943749475}, {2.827433388230814, 0}}, {{
        2.827433388230814, 0}, {2.827433388230814, 0.3090169943749475}, {
        3.141592653589793, 0}}}]}, 
    {RGBColor[0.9, 0, 1.], Opacity[0.4], EdgeForm[{GrayLevel[0.5], 
     AbsoluteThickness[0.4]}], 
     PolygonBox[{{{3.141592653589793, 0}, {
        3.455751918948773, -0.30901699437494773`}, {3.455751918948773, 0}}, {{
        3.455751918948773, 0}, {3.455751918948773, -0.30901699437494773`}, {
        4.084070449666731, -0.8090169943749473}, {4.084070449666731, 0}}, {{
        4.084070449666731, 0}, {4.084070449666731, -0.8090169943749473}, {
        4.71238898038469, -1.}, {4.71238898038469, 0}}}]}}, {{}, {}, 
    {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwt13k0Vtv/B3BUpgbOxm2m0KAkIqK0d2QsiXRJriEplaJS94YoLsqXiFQk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      "]]}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->True,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 234},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[14]=",
 CellID->1440821426]
}, Open  ]],

Cell["\<\
If total area between the curve and the axis is sought then BarStyles \[Rule] \
MonoColor will make all the polygons the same color.\
\>", "MathCaption",
 CellID->1261763501],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSumPlot", "[", 
  RowBox[{
   RowBox[{"Sin", "[", "t", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", 
     RowBox[{"\[Pi]", "/", "2"}], ",", 
     RowBox[{"3", " ", 
      RowBox[{"\[Pi]", "/", "2"}]}]}], "}"}], ",", "5", ",", 
   "\"\<Trapezoid\>\"", ",", 
   RowBox[{"BarStyles", "\[Rule]", " ", "MonoColor"}]}], "]"}]], "Input",
 CellLabel->"In[15]:=",
 CellID->58147182],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0.95, 0], Opacity[0.4], EdgeForm[{GrayLevel[0.5], 
    AbsoluteThickness[0.4]}], 
    PolygonBox[{{{1.5707963267948966`, 0}, {1.5707963267948966`, 1.}, {
       2.199114857512855, 0.8090169943749475}, {2.199114857512855, 0}}, {{
       2.199114857512855, 0}, {2.199114857512855, 0.8090169943749475}, {
       2.827433388230814, 0.3090169943749475}, {2.827433388230814, 0}}, {{
       2.827433388230814, 0}, {2.827433388230814, 0.3090169943749475}, {
       3.455751918948773, -0.30901699437494773`}, {3.455751918948773, 0}}, {{
       3.455751918948773, 0}, {3.455751918948773, -0.30901699437494773`}, {
       4.084070449666731, -0.8090169943749473}, {4.084070449666731, 0}}, {{
       4.084070449666731, 0}, {4.084070449666731, -0.8090169943749473}, {
       4.71238898038469, -1.}, {4.71238898038469, 0}}}]}, {{}, {}, 
    {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwt13k0Vtv/B3BUpgbOxm2m0KAkIqK0d2QsiXRJriEplaJS94YoLsqXiFQk
qUiKMk89tUskUkkqNBkyhbMRcZ7n8fjtu9bvrHXWWa8/zvns/f589h9n8R5f
ey8JMTGxAXr/9+ySD/ASLObgwOR/F4Gj6dNLB5ZwsPh8heR/nmp4XbZVg4Pm
OvPnC6kVXmv91bCKg3PUZY3HqFU9nj6s1OGgKCfWZ5BaZ9ROomQtB+cZZmd1
U6PzHQ5Zhhxst3Ef+UK9feGJzGvGHFRJyN5aT+2WJ8mP2UTrv76Y/4z6iNnV
rWfMOPioTX5pHnVws8aNY1YctKpfkZlKHXP40dBeGw5ml7WujaJOEbfZ7GjH
wa1fdBuOU99P/HbZaicHWy8tCdpNXa7h17t+Fwf9XUt0N1E32SVcWOTBQZ+7
z15JUnd3qrcBLw6eLTbO7hIR+PtUse60A3Q/g+7JldSKt5qbev3ofhwybgRQ
q+kfWvnFn4NCYW2xPfWaWuHpN/9w8FVD2ufl1NuHVdQKznBQb8f+bW8n6P4i
8k5k/MvBVL+/Um5QH5ln+vLKOQ5eTJs1fpg62mTf4aA4Di479KBpKnVNfHaJ
WRoHLfMhnC+k6126UXZdOgeHLx8QfBYQ2FP+1mXFXQ6elD5YlUwt2TEsLpdL
+xErCgXUJrqGW5swB71L+tN/cgTaVdem1j7noO0jVH+N2n23yxCvmoOqzhdk
tlAHh4VcvvmWg06rV9y5PU7zel/VevA7By0sRyONx+h69jvq/tXBwUK3O2rf
f9P1CHrCbbs56FIQ+DqY+rfajJV6hIMJN+5tKR0lUNff/sSEiKN9rhpWGCEw
W/G7TJwyH85Jyh46PEig35uZTp9U+fD3seTXnYRAvXMb7igv40Nrx0733dSP
BUkmOav5kBEvz4EsgW/aHQJrER/Otg5u6+8jcCi3tn+qJx/qOI+VN3cRWHyQ
M9q6nw+7lIqYVdQB6svPJxziw/CGs4uCOwmUuBq+RNWfD1VDA/bN/UH7GYJc
YTgfrjW3S9RrI1DfpvjtqUw+lMn1iJpoIZAv2bnw2X0+PD9k4a1J/eSpgo90
Lh8W982ZdGom0ELvqPSVUj6cll3ieP8TgU7zNTcV1vDhsZ6kYr1GWr/3Zj7b
x4fdPpKGLXUE4vCYRE9tAXz3Jr/Gj0dgspThtj16AhjtG5tx4BGB/ud/SHqs
E8Crn7kI93ICV8Ss/8cVCeBp+3Jny1ICL13qdd61XQDNbJ5aiBUS6H3bbJGt
nwAORwvNmfsEgmcT94xyBfCWxIVdpVcI7DfJ8jQsFMCO7TEyxy4TWF3psGBd
qQBaHfR+ppFIYNDL+zFrnwrgBcEUq4R4Ov9vd/lp1wtg5h6l8m0xBPK+Fa1d
OiiAfbe19PeHErhPePgpoy2ExWOKOiqHCGwr33JAqCuEoXMf5Nw8QKDLKQ3Q
bSCECxd4W6p6E7jj9w9PHhTCpqrkpoVe9PwP7pbabyuEt2TD94lcCZz7w2ob
74gQLucr23jZE1j3asmXfTlCmAu+DdYa0HyjpoTb5Qnhmd7N7ow+gRWWbas2
FAlhXGrH4E49AsuqUkKYx0LoI6N7slmbwDtYQY1XJ4RPh6QVazUIPFMgfpDp
F0LL+P2FdvPpPF77OvZoxQQ8dDN0q0DIwr0hTikCrQloqp0e+ZnPwkue79F6
3Qk4qyouoWychSMra86Xr5+Ai9W8lH1HWFjMK5xftnUChpoYTX3ax0Kjb/+D
xUcm4Fitgi7XzEK02CgyN38CFpw6a1hbyEKbzMt/ZBiJYAwoshr0pN/fcWFJ
ykYR9P57SdJxDxa2TIbrJZiIYK/zsOyoKwu9nU7Yn7UWwUVXN/45vIuFoTIO
F/5yFsFxzeHYj7YsLD3ESP4RIIIHOx7mm61nobp29GhEmQjeamtswPIs5Jee
bfReNwnDv/GqNYsHIH/+Z17REjGUuZqnSJr64f8IT6koWwzlTX6adPreB7uq
DJ7cXCGO9CQdpyW0/ITd9o4tqwrE0bGZ7p6nbvXC/HY588w1EkhavWfd8709
ME5SvjT1oQQ6su1RV71SN1RsK7y+1mAKstt569mFi51wz5PhhrKiKaj4ctVj
H+YH/NP+Q+DXjVORotsctV/O7dA1YHJFSNVUpHS6cIHoSivUzqjQmGU6DVWw
YfL5VV/h3YgVKRteTUMWkVYbVw41w58HV7o6WUsij8ic1V6mH2HDtHcJCo2S
KFmD9duk1QBf6z15u3W3FHpyMCzTenUdjN+3effxZin0z+rKVMHRSjixzdfZ
cI80UsrUHLYkJXCNkW5Wf5s0YtaY3pmUSIOBZRtFC91kkMFe18CVcan4IGFf
sF0ySNa9OaX0UilWW77ue7aXLCoykz0cu6QSh85eekiWyKJTmaK8mvg6PJK8
e6LzxHS0+VeQq5NLA9ZevOL7g1/T0bW+K+0tf3zEwPW63uygGWjGp/jPFTIt
+Mn1rGYfwQykN0Pw82PqV3z66MS/6J+ZaMFgZ2FJaCvuEIzeLpicieTvddoh
q3ascD4SKAbPQr4mPcv7vnXg3rrHIwpScuhe2uz7z8904vMKMUvFI+WQg5Zb
0+PBLqw8ZrdaX0Ieiczfvjc37cG1eHX4enN59EL1p4Hy1V78WaYltCRcHul5
NOVp8X5i+PBPo7BaecStrLwz2tKHc5THRmbOYtDzYJc4gw/9+IzcXkc5GwZ9
TguZj28O4NwZH64t2sagtIhL+8bSB3CbjHmrji2DLnisTVx1dwCbTF1+wMGO
Qe+7tVMSHgzgaVxfQNJOBtka5VmbPxrAUe3HU9VcGdSR5TVm1jiAE4rCOg18
GWQaLT41ZyqL7+y+fdz9IoOWOmfNmOXG4i9RBfFu8Qxq7XkxGOvBYlD+PM81
gUEbNi4altvL4pA5P4hLIoMMjndvkT3AYqcP6j67khiUzvzt2HOMxTK2d/ba
32RQDXci2yqCxT4mWTs35zEoUX3EJO0+i28dLfM3zWeQm29TWGcOi5vSahJM
Chi0MWL91+W5LN482fsOFTEIubJNWYUsXshbsc24jEEipDJ46TGL36zNNtd/
xqCPFz4Ey71lsY7GQ4Pl7xikOjl9VeIgi+O6X9mrNjDoQXTrtIRhFrMZPYcX
vGfQuRdKkrEjLL6nqpou/4FBa9U0gs6Os1h9QaI810Tr9WSt3y5G8B9ygb01
rTS/s2rsJTmCBSPm1w4N0voWJo7lmgQ7F3gWew0xaCTyakeQFsGlR8+8cxtm
UMY/vARjbYL92TIphxEGFf0t4VamSzDbrem/YZxB9v5pqclGBLe2AJuZYgB9
+m1zf4olwVVPv4keyAHUlxFv5etBcNzLdqcn8gDtt/9tNNuTYJf6rvzXDED2
FirjvL0Ej3xn9/UrABTJs/4g7k2w6qTojcYcgPJdG2cfPUJwiLHKjfRFAGWP
X03vDCDYsNwNJusA5CqOFiyNJ3hqhWdS1hqAxIr6bSMTCK6v2f+rVBcg1V9j
W7ouEezd7Jv5aS1A39OqU65fIfjyeIickhFA0qdupw6lEPxLP+17rAlA+8pi
Zq3OJDinoDUkfAdAC7o9xCfLCA4eCCsMdQDIzXiKleQjgrcvW9YbshOgi/L8
G9N5BA8nH7YPcAQo1ONuoPwTgvVD+eq+uwGa/9i2R6KC4KfbFWudPAFaFX34
fFQNwRejSkQ79wK0495KfKyW4D2Vzro7vADKbbaZsusVwdPW3Uyx2U/zMowo
VnlNsLWKlq/JIYBiTTZXJ9YT3Dhgoah5HKCgGNMI9U8EZyzrs9TwB2jhh75r
A9QnPS6cXnoCoN9aS3BhE8FzPzR2Lfqb5hk2T8u4hWBXnkeZUiDNy6wiW/8r
wd1Rga6TobRevUi1rp32v1I5QRgG0OA7sxknOwiOEj2r5v4FyMH9gITyD4I1
j0nrjEQAJKO1UMm7k2C/XYlTeqMAWh2m8v5HN8H8ZQ/vNsQDFDe8JutMP8Ei
8XHbgQSAivQeuMwfIFj8CxqTSgRoj6zfzQJq6bgGM+MrAJUZJH37wtJ5HR9t
z7wG0EFfy6VgiOB5DRujKlIAqvv4eE8S9cLsSJ2v1wFyWaOqrTxMsLr73DMg
DSCjoNjTi34RrPtyw8LT6XT+SvvbpEZpP26FV17JACji/cv0QGqjoDeH8u8A
tGax5Xg/tYm2e3n3XYBOCWbavfxN+3s1zNE+h75/pf6lxzjBDsfqRD4PAKr8
+U2ugtpxq9KdyIe0Xuz2bhWO5imWOcLLA+jXnk0WjdQ+3rUXlxUDZF2YUqsi
oPmYKBialAC08oXUJR/q4wtcWl1KAZqncWugmDqgfkArvhygJu9tdpuFdN7u
6X/MfgTQx5CduyOpz/4bcrqaB9Aml5Oj1dTn1jF1wicApf0Yat40QXA0cD4+
+ylAMy1M1QOpY/tvzVvzDKDmz9dEedSX0/QO7H8O0AlFe885IoKTA07Lh1YC
BPoyByyoUx1elKRU0fO1a1TqBHWGtJNkQzVAvr4H+qup77an5fS/BKiq6nzx
AHU2r9dBqhagoyBRnpkkOPfyGuHiVwD13j7H16Eu9Au8vaEOIK1xD//t1KXW
ldaOrwFq4CkH+1A/Up85fPQNQMNveLMjqJ+IdiZFvwVo3XqjTdepK5pSUWY9
QF+NE/n0fw2/yO/ufvYOoEyTOtNK6ppo7dgvDQD1//i6oJG6bt8p/bH3AMWQ
l+Ft1PWo4ivzASC+S3RwP3XjvOnhmh8B0nVcJjVK/Wlkh6bFJ7ofgyQVAXXL
m5T3Hk0A1Ua2VYuov93tDAhqBuj//1/x/wGn3M/7
      "]]}}},
  AlignmentPoint->Center,
  AspectRatio->Automatic,
  Axes->True,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  CoordinatesToolOptions:>Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  ImageSizeRaw->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->All,
  PlotRangeClipping->False,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageSize->{364, 234},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[15]=",
 CellID->860103210]
}, Open  ]],

Cell["\<\
When two functions are used the color of the polygons will be determined by \
the position of the first function relative to the second.\
\>", "MathCaption",
 CellID->196193630],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSumPlot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Sin", "[", "t", "]"}], ",", 
     RowBox[{"Cos", "[", "t", "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", 
     RowBox[{"\[Pi]", "/", "4"}], ",", 
     RowBox[{"7", " ", 
      RowBox[{"\[Pi]", "/", "4"}]}]}], "}"}], ",", "9", ",", 
   "\"\<Center\>\""}], "]"}]], "Input",
 CellLabel->"In[16]:=",
 CellID->1038941265],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnS23szq3hp8zzp/Ydv+ELZddsrKycmPOGIhXILGRkZExRyCRkbHIyEgs
EonknYF+0BZo+IZ0XiO7m4dFSaHNPe9MQvj73/+c//3fP3/+/D/893//8+eP
WS7nYK79IAjyPaBuIAgyFNQNBEHKUtMfEANPvP1BeLD+zw/VzZWoGwiCVIB0
vOkGqEa1TnhPyoG6gSBIRYtuPFY9CwfqBoIgFS26cbMbqBsIgrQyzG80GVcf
6YQGAQ0D5nnM99j5DIXXr6cT//2Nfn/hlTfXXy7M96HQMDSle89k7MlBEKSV
tfMbVSvmULIskoIzFvm+UQYoQWBKFPGIc6UipbjWXCuzZf0WKFo/1gvBAcZ4
4IOG8J8f7nmcEB7HZpuiuL4LdQNB5qW+bnK7cNLsoCxyPSVNc3AIIA6gEtDS
KY2g7YMUFAU08Ic4jF5OU54kMYhJEERQhedFRlUClufFxE+OIMg4RuuGlCml
yekU+Z64XCi4hUaTX7BkGXiPulMTex7oiQLhmvecIAjSz1DdALkgxMgFvMJy
vfLWT5nBXVgu1/2ULCuiSPs+CJcREPjn/CcIQZA3LHVDqQzcxfkcwWuSpC9/
vec3Visv+Q3os8SxBvsBAiJEil0YBFmUft0oilKKFBpjGMq7u3jnWTfWcB1d
eVGwHKBsl3NMiFQK+y8IsghdV2DTtKAEDEYMzfBj+N7cb7yTJFkQCHAgQuj+
LREEGcq7YlQtTkKL6zEYL2yV3/gI2A/GFHSvokgX2HdBkJlo6obWue8L6JJo
lQ3ayQ79xgucqzp3iuqBIO/0DLBsBXQDXoOA1iM2+4do9o7eXDu/MRzq+6wa
AcJgedQxIoibkIGhH3Qj8M2QTqUmRv/1/ca490ZxbManAc9yNLPDQZADUf28
raJ2lkFbiEA3tI6nR//R7x23PN3hRFF0OZuRqEOOEUHcxC4QRxBtPQ9aDa/y
G+MC90ssnrqT9WssiogQ7vvcZrAr6gbiMI0G1R49tYrBpcdxvSZq6MaW0X/o
8owOB0Qj8I2A3G6am3QFB0GOSE8gznMehiaVUd1xdi2N8RvjA/dB/UazgPWC
bstNTrtqRBA36chvQG/e3EnaSH5Gd93YVfS3XF7G4Zi7bn0/qrotmN9Avoj3
QJymJpUBwtEaRr85v9FaQDR8v+VqC+oG4jDN/Ab0Rzg3U+jkeedVA8xvtC5D
hwXEtnmlCXUDcZh7E4a46Zn854fwin6jq2SZkVxKr+kg1A3EYaqf97Wfnuef
IyzmN/qXpTR5oVSjbiAuQwLqe59txr2g3/hYQH7BeHgXtvV3iyC2FFkOxX57
0I34EqaSWkbVxnXYA0T/+/L6DqeaZV3glGLI/kmljj1aDpm9yjSogieEJNXC
x2CKfsO+xmoGZmE/vQCCrE9CIihD33Ufv5FKJvywyD5EUsxvDDlGA6VGlfF+
fGRv5DoTHgWzMeK9zQZVZBykIxW0J4yi3xhS4xWwHGA8cB51ZD9okciAD8pp
NHm7PyVKKPRZmtKB4zdGH+ODLCuwz4LsBBlGir0+aW0QrYE4ldT0WXL2/if0
G0NqfD/bCWPJlO8LQaYABkP4LEumTqLbNf9GnjLhk1yzl/WY3xhyjC0IkQaB
wHQHsj6ZSkE0ijme+tEXiAsuAqLjp3QH3g87pMZ20jT3vBjTHcia6EjKkM+1
t4/zbyhO5cOTYH5j0DF2An4DXIcUmO5AlqcoJYlUJGfcpU0gzpLqEq0ZhY75
jUE1foBSTHcgy5LPlNB4wXJ+0SKLYi/MU8xvTM1vvCCEDgKJ6Q5kCa4JjbEX
W3uAn7fwTL7ih75FZH3+MX/5m9bTaRZmdAf6jSE1WqFUhiPSkdkxIzRCXi4T
k8jlnz+eD79z4d304RYxBT3r10gam/nMr5nSw0X/leoaqhtlNboj8IXWmClF
5kExMXGERj/+6S9PVD918fNkOa5m40c8h1FYpThJ3s3J4Fg8c3zfZY0DqDOl
+HRaZDpgM3S8bN4M7MY/l+rpYpd//jr5r48cA1m5/b3mT4ORj2qrWCHib+5w
RuB55gb8CWcVcYRxPwBKKD+HLFj8x6Ppj1fbGeH90JZgp6nXXH1/Pmx1UwzL
R6VcyLf4jTFvjGN474sQ2deIuAB8lfW96YNeC5bHZ1bQfMR7B/94hPenEo4O
2YD1XrOb1HyudJEXwmcgICNOy0t0XnqZHMrhJEkcBvc1M1/BQfaP+SpvX2/R
+Kp7lnOWiTMrWWG5/cvyiB/P7XqKvv3LCAX4kLoz4j0nV5q6YSgKGfB04NXh
A0X/rWqsp0nP8wGuA3XDGYb6jTRU4sLtt39/XfrH86obFZJEqVT2OzlW9N/K
4eR5DNKRZbauA3XDGcxXae0W0jBJvNh++9blTXQDSGisuO1A1sNF/61qBNMZ
+FwpK9eBuuEM9n5D+SLxhM2W/a9b6UZZ3TgD6mGzkyNG/+0cjsl1SPnZdaBu
OINlfiPxYx0k/dssl98YRI9ulGaCU2Vzz91Bo/+GNVLCxac551E3nOGz32CF
9CIdqr5thryuoBu3++jbMUPigw+PADhy9N/M4TB2f/gm5jcc50N+A0TjwjOS
9m0zcHlbv1GTp5nwWdk9Lv7o0X+rGiPOKe3MdaBuOEOP38hZLs4so1nrX/fs
N2w2q6cj63oigwPR/+PyQsco5V06ML/hLF35DRANcBrgN8r5nMYe8htNaulo
nZTMjei/VY1ScBJ21Yi4QKvfAI8BTgOko5zVaZR78htXqgGl77f/OxP9e5YX
PUaleBhGT0MDUTcc4j2/kdNMViO7yrmdRr28L90ozYBSIx3Pc2m6FP23qlEr
HgS8KF5qRFzgxW+kJAWn8eieOO83bpjbWNRjLk3Hon/r8grHmKYgHQ/Xgbrh
DM38BohG4kXlvZnP7TRWy2/0X4ftQoY8u0mHe9F/qxpBOnyPZ9m9RsQF7n5D
h0p60UIeo9y936iRYVRLh5PR/2V5tWPMc+M64BV1wxnq/AaIRuLH5b2BL+M0
6uU960ZZuY5UKlej/1Y1VtLBwxB1wxHgx2NEw4tXcBrl7v1GTUJjFjgb/Tt0
Yw3XcT5zfJaTG1A/XM1p7G38Rg/sEqay76n3sxe3/UZdwG8EgUTpODqKS2gg
qzmN8iB+o6xsWEKJjprPrXYq+pcbOZxqcmOUjgOjIqkjOWj+jW/Ib9TUbUpx
kI6VXMc3+I36q0fpOC7gNHT1NMZx84tOed3tddgm9zalIrqO63A+v1E2rsPW
0qHU4HlfkQ1JaKxvj3AdMb/oN+Q3mm1qHdfxPX6jBqSDhCgdhyEhcSoeM206
6Tem7+Ql+q/gOr4kv/F2nlE6DgA4jVQ+zeyN+Y1W3mPx0q7j2/zG41Sj69g3
xmm8zemNfqOV1ui/qOv4qvzG29lG6dgp706jBvMbrXTF4uVcx9f6jesJR9ex
P1qdRg36jVZ6ov9CruM78xvP5xylY0d0OY0azG+00h+Lpz/v/r18ud+4nnZ0
HfugvkurZwMn/caM4zeq0hI9U0kTSvu3GbT8zfmN5zOP0rEx/U6jBvMbrdjE
4lTQGV0H+o3HyUfXsR09OY0mTvqN6TuxjP4zug7Mbzyff5SODbBxGjWY32jF
PhbP5TrQb7x+Beg61sXSadSg32hlUPSfxXXYRP/q0Ebu/315V/mN1m/NDdcx
yw9yaeydRk2d3zCHtuP8xqAzP8vG77H41mbby4vr6N+4dbc20d9+tzYb32uc
d7c9Ww7VjbLDddh/xQv9ctb/QS6626bTsNxt7TdgY3vP8BCZUVt2/Xh6PvB2
uhF1tIIPrqPa2CoKd+hG+/YfP0Nz+eNnuB9jY7efP7P9Z3h3RyN0o2xzHZs3
cJd048Vp2OvGtYFbO4eryFhsX+vGy/oe3UAQBBmKvX72MNd+xgF+DwR8xBu/
ZvzG6PeaZ8AtdwXn0Iy7uiG9OCXafvvX6L/WlZQDjd+YQirGSMfXXE+Z8PaQ
V9IxtEb3GXF1w4hGqKZc3cDrKbMzwnUcK/qPW55+jOA6pBx6jO4zNPpPcRqP
6I/jNxZgqOs4XPTfqkZCuBCDanSfQdF/otMo0W8szCDXccToP3R5rmOk1NZ1
fI9ulGs5jXoZ8xuLouPEUjp6Y7HvVXnjn1lvwt2F39Dnn1GHBtJh4zq21w1N
fzyxdCWW0V9eoulOo34l/umf6nurv1I3/MaMV2emY+k6eqK/pmdhFkA9/qa6
fZsRy7twOLoOfoH3c9YD92njOrbWDWEEfxXd+Ow0LlFGZ3Aa9fLldNKY31gY
m1yHTfQXXlM3ppZd+I266DO1Tlk0y0fXsa1uCM8T+/Ab4sIzks4W/Qn9608l
iJjfWJiPrsMi+vue53/aZsDyTvIbmv5d/Qb9JXIdW+qGUY31+illr9PIafq+
fmr0p/TnLh2Y31iMftfxMfrfeiuzlR35DdMF+xl9dD2uY0PdoD+PAY0/dMAt
YyPo8RszO43ba12jPnsu5TcW3f8UelxHf/SHoFxlDgP6yB86kd+4Fkq9wfmN
5nKX69g6v7Gi31jLaTyiPy9jODbMb6xCl+voif7Ce0Qub1QeoLXswW/cDm2G
vE2r6/ge3VjNacCrul4EM51LzG+sBrgOGUYvK/cU/VfNb8y4/O46tteNVXjP
byznNOplHL+xCZnSL9Kxh+jvQI0gHZy/1Og+L35jOafxkt9Ybv+tNS56Dvc2
fqOLTKUJeUiHe9H/fXmdY4wMzWN0n2b0B9FY1Gk08xvrOI16Gf3GnabrcDL6
b1WjEQ5+r9F97tFfnFlOs3Wiv3t+Y9H9z8vddbga/ZvLax5j7Tq+Rzdq0Sh4
Xt6b29LRH/Mbm1K7Doej/1Y1gnB4Ht36610DSogRDZqvGf3Rb2wOuA5+Ca/n
yBTXon+bbqxRr+8xSpOtv97FYedwNaeB+Y1dQYNQBKQhHYsX5/1GXaOUKSHO
SkeRF8JnNNwg+qPf2ANwWvKUySfpcC36b+RwSpAON10HiEbAQDq2if7O5TcO
cR32hbpNgXQIPyxy1mgFS5Uv8Rv16a1ch9z2K56XIs3BaYBolBs9PQ39xh64
t6ki5+A64PX243cs+q9UV+MYryRJFoaOSEeR5eA0yko0yq2eDo/5jR3QbFNF
xoQXgvdotIL5y1f5jRqlXJCOXKfgNJpr0G9M5+C68ZBacB0N6XAm+q9UV6tu
lEY6ct9f/Eaz5chVKgP+shLzG9M5uG40SsGFH2ZqKdfxhX6jJksLkI78ZvIP
RJZoGfL39eg3puOEbjwiZkJoKhdxHd+W32iSZUUQHEw6Wm+jrsH8xnSc0I2n
khCSCtr119Hla/1GDYgGuI40PYZ06EgmpG+yOPQbEzn0ddiu6JlQqiL6vv6A
0X+luj7qRg1Ih1b5Ol/xaBQXKupL52J+YzqHU4yaj7FYRzRhzj0/ZY0aPxCG
Usp0ha94HAmNdfxh0Br6jek4oRvt0TMVLHFzftG18xsvUJoIsUfpkCRKpfq4
GeY3puOEbnSWVFIRhM8NZGRBv9GEsYTzzy10PYpCBDxTVmqGfqOVevbalsnk
dT3Z/NNfDq4bnyNprsz12esJPV70X6muoboBRJHeyW0s9d1quc4st8f8Rgug
GtWM0MJ7VQ5BW55LcXDdsCpFaqSjyGy3by3oN96B3koYymLTayx5mpkbT7IB
2Vr0G+88JpJ/EY6r2XidZd4J3bCIqnkE0pFrZrv9XqL/SnUNym80USoLgs2k
Azom97vV7MH8xjs3u9FmOMpaPZ6kwwndsCsFF0HYGBU2rKDf6CJNc8/bYFSY
FknrcNCPfLvfaD5h6Ja36PQb97NNvZf8xhGHcJBRUErZJWTeuHd/RX5jHGFI
z2ceBHT0HoZSfY/h6LeXmN94oTu/cfu7C35jCtVgwmjou8i3+I2R7y0KHoa8
/3n3XZUOjafyEukgOVb035Hf6OD5espVR/Ttwb0vCY4v1I3S3LygZcAH9YvJ
t+Q3Ju2HsYixYe8dGv2rxyTp40X/9R3OknynbpRmWgaTh8/TIRfvHr/2NcpB
a4xj6LDwZ3H4UKllDC1oHs/xxBP0G9P5Wt0ob9f9wXvYbHzE6L+Vw9Eq9n2e
ZZHN9pbRP6NanFnJiqNG/53nNwbyzbpRk5BI8c+zWh00+m9VI4gGSIfWn12H
TSxWvpCXqH8b+1f0G9NB3SirTKn8lCk9bvTfzuHE0GER4oPr+Bj9E89kQfu3
OUD0X9/hLMkRL8IuQZZoMwtl9wCmQ0f/DWtkDHbLi6Kv0q64mdMc+iZpqFv/
eqzoj37DVYosjy8077g36vjRfzOHk0ju+1GWtW/TFf1BLsSFFSx/WX/U6I/5
DacRAddts764Ef23qhFEw/e5lO2VvkfMxBfQPSlYUXbE0ymv6Demg7rxjoqk
mQT7ucviTPTvWV70GKGrQkg9uqM3v8GKqm+SlPeG4Eb0X9/hLAnqRiuZSk2f
pXFrtkvRf8MaRWyMRzPd0YzFGdHxmU4fodH/in5jOqgbXdSjO+6TzjkW/VuX
1zlGraPLhSsVPyqtNlG+NBdbXY3+mN/4JhQTwvRZCiej/3Y1RmHIKb1WmjNz
3UQFsnQ3+rvnN/A6bD951WdhgZvRv7m8ssOR1XUW6hEQjYxmjkf/9R3OkqBi
WFGU7ByaWY6L+49/8eK037gWEYTkRKJg5hEa/a/O+42UFuczT9MFn1iBumEJ
fPWppLFXP0rSwehfrutwcsXjizmZUCnoRniRhdvRf6264GT6FxGGdNHmgLph
Sd2mipzLkCRkDePhsN9QrJpnvrhWCv9XJLucYhFo96L/ajVmrPDPgvuqwH7K
bmhG/1QyiJUQMRtt4djRfzWHAwZD+ETHj6fpNaM/8xXEypQVzkT/1RxOHGjv
HKfV8NoSdWM3vMbiarZS8B7gQJ7Wz1dc8xuFeXQvnLE8e5ri9SUWa5pD0IwC
VXZE1emvjvkNRcwZA8l9qXHR5oC6YUlr9M8SJryn6Pm+zehll/IbtUNLn8aZ
R49K36InmG1oCzkvXtbPsuxSfoNXDg3EtuUYlwSvw1rSGYuLqrc++fks78UN
vwHuAk5O0r3nrlic0vxyiiGM5qzlr1Ne3fAbSZURijsyQug3dkJ/9M/TCOKp
4vQpJHRv/x35jbhW1OqxNZ3b90d/aBeXs2kdPdvsJ/p3Lc/rcMBdBBdBvL4r
UKgbO8EmFkOHBdSj0W2ZVA7tN+AkROfA5lR8jMXgN8B1eOcY2stuo/86NRas
pF4CPThF8o81LtocUDcssY3+0G3hRj2yhFtt37180PyGFvRqvey2t4z+GS/q
IJtNvtpy0PxGBNbrFEuS2h7jkqBuWDIoFpthHoSI6yAx23e9lMP5jSwxyU/o
mwy6xjQoFsswA+MBMRci7+bRf7Uaa8Xg/oBrTKgbO4EMh4aEn0N2DunYB4s1
2tdK+Y1x0OB2pGSFR7ZR32enE/c8RsZWt35+YxxBMP5IF20OqBtLU89cKgOe
JlYPXLhDtvEbg48u9qgMI/sn0cxFHOvLJaYkGfpoWnKEu1ProyPDj24dUDfW
IdepJFF8oZaPayk3m2HMllQqOBw4qCJb8P6pj0iZGvWgida2H4Ps+O7Uoiij
yFxC4lztUzFqcPzGmkATS4x6EB3Jj0+Z3KffgENQXEanMKHxtorRRIrU9wUU
kJGPG+/Tb6RpDu4CNDDiqnta/b2AirE+oBiaS1APGfIe+0G2yG/0fGzokkBv
CzxG67zNewCaHhgPCNaMJlnW+0iL9cdvdJDnlcG4xJaitxNQNzYkUynYD4jd
ion3/MBO/Ib5kDSOzwS6JFnHQyJ2BQRrIUznxfPiKGqJ3TvxG/Ahg0BczhF0
SXpUbp+gbmxPUeo4ET4DB6K4uLfNbfMbD7kIeCpUebDftQEaIzTJ8zmCUB7H
Or91qrbNb4CpoCSBT1XlZNZOJnej6Q+IgSfe/iA8k8f4oU/GGHVjP5j+S5xA
54WfQvAhdIF7Xj74jZCARJgP8BuYPpRQH5MwhwD6L4wpiOx1BjXwabbMk1m6
XsOAgoJ5njidIkKSvfZHQDredANUo1onvCflQN3YJ6nU3CPxJYzPoSREx6y6
y2N+pwG7VRGVIYlOAa+ynYfojIwjz4uqC8Mup9irRlLFgb7PWTGj08h5mYRp
fVfv7290PjMwPItO3DcHLbrxWPUsHKgbu4XcZhjLEqY4EYFp2iAjwjdjMhWn
0OQzxewHZ8L20PfQEU0o9D5IdA6i3wD2BmvMuNZizPiNI1LnN8BygGhUd75A
0+agJMRLaiVRJNM0t7wPF7aE7aHAe0l188gJhAJ8hZdEga7vqTnIiW3RjZvd
aNGNJmt+SqSf1kGAtBqGSr2Q+YSdQw7lBM0/hJ6FKT9BPW6zfn1a/1utv4Ss
ei8N2gcgbn3Qa9AxvpL6PvU9Bm7kfObgEE6//PeHg1sAVTmdzJrr+tNjPZT7
engv7CEM28d2bn3QDeqExZ2HHKDf+F6gl5HrtHrdT87t8EAHR+tcqQxe9zwc
axqY30AQZBhXG3KVh2YHBa+nIAgyA6gbCIIMBXUDQZChoG4gCDIU1A0E+Sr+
C8AC/sA=
    "], {{0, 161}, {360, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{360, 161},
  PlotRange->{{0, 360}, {0, 161}}]], "Output",
 ImageSize->{364, 161},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[16]=",
 CellID->86384514]
}, Open  ]],

Cell["This gives the estimate for the integral above.", "MathCaption",
 CellID->743339996],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSum", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Sin", "[", "t", "]"}], ",", 
     RowBox[{"Cos", "[", "t", "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", 
     RowBox[{"\[Pi]", "/", "4"}], ",", 
     RowBox[{"7", " ", 
      RowBox[{"\[Pi]", "/", "4"}]}]}], "}"}], ",", "9", ",", 
   "\"\<Center\>\""}], "]"}]], "Input",
 CellLabel->"In[17]:=",
 CellID->1402938173],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "6"], " ", 
  RowBox[{"(", 
   RowBox[{"1", "+", 
    SqrtBox["3"]}], ")"}], " ", "\[Pi]"}]], "Output",
 ImageSize->{87, 32},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[17]=",
 CellID->1757151059]
}, Open  ]],

Cell["This gives the total area of the polygons.", "MathCaption",
 CellID->382326629],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"est", "=", 
  RowBox[{"RiemannSum", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Sin", "[", "t", "]"}], ",", 
      RowBox[{"Cos", "[", "t", "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", 
      RowBox[{"\[Pi]", "/", "4"}], ",", 
      RowBox[{"7", " ", 
       RowBox[{"\[Pi]", "/", "4"}]}]}], "}"}], ",", "9", ",", 
    "\"\<Center\>\"", ",", 
    RowBox[{"Estimate", "\[Rule]", " ", "Area"}]}], "]"}]}]], "Input",
 CellLabel->"In[18]:=",
 CellID->492581779],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  RowBox[{"(", 
   RowBox[{"1", "+", 
    SqrtBox["3"]}], ")"}], " ", "\[Pi]"}]], "Output",
 ImageSize->{87, 32},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[18]=",
 CellID->242735069]
}, Open  ]],

Cell["This is the area between the curves.", "MathCaption",
 CellID->1480230272],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"area", "=", 
  RowBox[{"Integrate", "[", 
   RowBox[{
    RowBox[{"Abs", "[", 
     RowBox[{
      RowBox[{"Sin", "[", "t", "]"}], "-", 
      RowBox[{"Cos", "[", "t", "]"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", 
      RowBox[{"\[Pi]", "/", "4"}], ",", 
      RowBox[{"7", " ", 
       RowBox[{"\[Pi]", "/", "4"}]}]}], "}"}]}], "]"}]}]], "Input",
 CellLabel->"In[19]:=",
 CellID->1779614656],

Cell[BoxData[
 RowBox[{"3", " ", 
  SqrtBox["2"]}]], "Output",
 ImageSize->{38, 18},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[19]=",
 CellID->1576859030]
}, Open  ]],

Cell["Here is the error in our estimate.", "MathCaption",
 CellID->1604822736],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", 
  RowBox[{"area", "-", "est"}], "]"}]], "Input",
 CellLabel->"In[20]:=",
 CellID->2008601279],

Cell[BoxData[
 RowBox[{"-", "0.04885468602693699`"}]], "Output",
 ImageSize->{75, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[20]=",
 CellID->174029583]
}, Open  ]],

Cell["\<\
The functions RiemannSumPlot and RiemannSum work well in Manipulate. The \
uniformity of the syntax makes it easy to enter both functions.\
\>", "Text",
 CellID->1245864415],

Cell["A dynamic example.", "MathCaption",
 CellID->1277664655],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Manipulate", "[", 
  RowBox[{
   RowBox[{"Grid", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"RiemannSum", "[", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "2"], "/", "2"}], ",", 
         RowBox[{"{", 
          RowBox[{"x", ",", "0.", ",", "4"}], "}"}], ",", "n", ",", 
         "\"\<Center\>\""}], "]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"RiemannSumPlot", "[", 
        RowBox[{
         RowBox[{
          SuperscriptBox["x", "2"], "/", "2"}], ",", 
         RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "4"}], "}"}], ",", "n", ",", 
         "\"\<Center\>\""}], "]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"StringForm", "[", 
        RowBox[{"\"\<The value of the integral is ``\>\"", ",", 
         RowBox[{"Integrate", "[", 
          RowBox[{
           RowBox[{
            SuperscriptBox["x", "2"], "/", "2"}], ",", 
           RowBox[{"{", 
            RowBox[{"x", ",", "0", ",", "4"}], "}"}]}], "]"}]}], "]"}], 
       "}"}]}], "}"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"n", ",", "5", ",", "95", ",", "1"}], "}"}], ",", 
   RowBox[{"SaveDefinitions", "\[Rule]", " ", "True"}]}], "]"}]], "Input",
 CellLabel->"In[16]:=",
 CellID->1020870232],

Cell[BoxData[
 TagBox[
  StyleBox[
   DynamicModuleBox[{$CellContext`n$$ = 5, Typeset`show$$ = True, 
    Typeset`bookmarkList$$ = {}, Typeset`bookmarkMode$$ = "Menu", 
    Typeset`animator$$, Typeset`animvar$$ = 1, Typeset`name$$ = 
    "\"untitled\"", Typeset`specs$$ = {{
      Hold[$CellContext`n$$], 5, 95, 1}}, Typeset`size$$ = {
    224., {125.84375, 131.15625}}, Typeset`update$$ = 0, Typeset`initDone$$, 
    Typeset`skipInitDone$$ = True, $CellContext`n$6664$$ = 0}, 
    DynamicBox[Manipulate`ManipulateBoxes[
     1, StandardForm, "Variables" :> {$CellContext`n$$ = 5}, 
      "ControllerVariables" :> {
        Hold[$CellContext`n$$, $CellContext`n$6664$$, 0]}, 
      "OtherVariables" :> {
       Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
        Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
        Typeset`specs$$, Typeset`size$$, Typeset`update$$, Typeset`initDone$$,
         Typeset`skipInitDone$$}, "Body" :> Grid[{{
          
          ClassroomUtilities`RiemannSum[$CellContext`x^2/
           2, {$CellContext`x, 0., 4}, $CellContext`n$$, "Center"]}, {
          
          ClassroomUtilities`RiemannSumPlot[$CellContext`x^2/
           2, {$CellContext`x, 0, 4}, $CellContext`n$$, "Center"]}, {
          StringForm["The value of the integral is ``", 
           Integrate[$CellContext`x^2/2, {$CellContext`x, 0, 4}]]}}], 
      "Specifications" :> {{$CellContext`n$$, 5, 95, 1}}, "Options" :> {}, 
      "DefaultOptions" :> {}],
     ImageSizeCache->{503., {157., 162.}},
     SingleEvaluation->True],
    Deinitialization:>None,
    DynamicModuleValues:>{},
    SynchronousInitialization->True,
    UnsavedVariables:>{Typeset`initDone$$},
    UntrackedVariables:>{Typeset`size$$}], "Manipulate",
   Deployed->True,
   StripOnInput->False],
  Manipulate`InterpretManipulate[1]]], "Output",
 ImageSize->{507, 321},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[16]=",
 CellID->993468198]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["Classroom Utilities",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/guide/ClassroomUtilities"]], "TutorialMoreAbout",
 CellID->534648361]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED WOLFRAM EDUCATION GROUP COURSES", "TutorialRelatedLinksSection",
 WholeCellGroupOpener->True,
 CellID->480403165],

Cell[TextData[{
 ButtonBox["M50",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[":",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["An",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["Introduction",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["to",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["Mathematica",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["in",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["the",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["Classroom",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"]
}], "TutorialRelatedLinks",
 CellID->1744965082]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Integrals As Limits - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "TutorialColor", 
  "LinkTrails" -> GridBox[{{
      RowBox[{
        ButtonBox[
        "Classroom Utilities", ButtonData -> 
         "paclet:ClassroomUtilities/guide/ClassroomUtilities", 
         BaseStyle -> {"Link", "DockedLinkTrail"}], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 32, 40.3492624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Tutorial", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "Functions for developing an understanding of integrals as limiting cases \
of known geometric forms are included in the package ClassroomUtilities`. \
Rectangles and trapezoids of uniform and random widths are included in a \
single function RiemannSumPlot. This load the package: The function \
RiemannSumPlot plots the function along with a number of polygons used to \
approximate the area between the function and the axis. RiemannSum can be \
used with RiemannSumPlot to give the area of the polygons in the plot. ", 
    "synonyms" -> {}, "title" -> "Integrals As Limits", "titlemodifier" -> "",
     "windowtitle" -> "Integrals As Limits", "type" -> "Tutorial", "uri" -> 
    "ClassroomUtilities/tutorial/IntegralsAsLimits"}, "SearchTextTranslated" -> 
  ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> Notebook], 
   Cell[
    StyleData["Output"], CellContext -> Notebook], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "RasterizedOutput"->{
  Cell[15452, 433, 8547, 148, 453, "Output",
   CellTags->"RasterizedOutput",
   CellID->100483496],
  Cell[39689, 937, 9193, 159, 182, "Output",
   CellTags->"RasterizedOutput",
   CellID->86384514]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"RasterizedOutput", 62054, 1481}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[612, 21, 2900, 66, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3537, 91, 56, 1, 51, "Title",
 CellID->509267359],
Cell[3596, 94, 482, 12, 74, "Text",
 CellID->1534169418],
Cell[4081, 108, 66, 1, 31, "MathCaption",
 CellID->1086341370],
Cell[4150, 111, 106, 3, 28, "Input",
 CellID->233438796],
Cell[CellGroupData[{
Cell[4281, 118, 74, 1, 51, "Subsection",
 CellID->2100704950],
Cell[4358, 121, 688, 19, 76, "Text",
 CellID->1916937970],
Cell[5049, 142, 3970, 124, 185, "DefinitionBox",
 CellID->2096742444],
Cell[9022, 268, 77, 1, 31, "Caption",
 CellID->1891092685],
Cell[9102, 271, 327, 6, 70, "Text",
 CellID->1354038939],
Cell[9432, 279, 160, 4, 70, "MathCaption",
 CellID->836781195],
Cell[CellGroupData[{
Cell[9617, 287, 314, 10, 70, "Input",
 CellID->2058623809],
Cell[9934, 299, 4993, 111, 453, "Output",
 CellID->988531026]
}, Open  ]],
Cell[14942, 413, 182, 4, 70, "MathCaption",
 CellID->1648657431],
Cell[CellGroupData[{
Cell[15149, 421, 300, 10, 30, "Input",
 CellID->3945576],
Cell[15452, 433, 8547, 148, 453, "Output",
 CellTags->"RasterizedOutput",
 CellID->100483496]
}, Open  ]],
Cell[24014, 584, 173, 4, 70, "MathCaption",
 CellID->674464701],
Cell[CellGroupData[{
Cell[24212, 592, 352, 11, 70, "Input",
 CellID->1309540841],
Cell[24567, 605, 7008, 142, 255, "Output",
 CellID->1440821426]
}, Open  ]],
Cell[31590, 750, 184, 4, 70, "MathCaption",
 CellID->1261763501],
Cell[CellGroupData[{
Cell[31799, 758, 411, 12, 70, "Input",
 CellID->58147182],
Cell[32213, 772, 6813, 138, 255, "Output",
 CellID->860103210]
}, Open  ]],
Cell[39041, 913, 187, 4, 70, "MathCaption",
 CellID->196193630],
Cell[CellGroupData[{
Cell[39253, 921, 433, 14, 28, "Input",
 CellID->1038941265],
Cell[39689, 937, 9193, 159, 182, "Output",
 CellTags->"RasterizedOutput",
 CellID->86384514]
}, Open  ]],
Cell[48897, 1099, 90, 1, 70, "MathCaption",
 CellID->743339996],
Cell[CellGroupData[{
Cell[49012, 1104, 429, 14, 70, "Input",
 CellID->1402938173],
Cell[49444, 1120, 278, 10, 53, "Output",
 CellID->1757151059]
}, Open  ]],
Cell[49737, 1133, 85, 1, 70, "MathCaption",
 CellID->382326629],
Cell[CellGroupData[{
Cell[49847, 1138, 520, 16, 70, "Input",
 CellID->492581779],
Cell[50370, 1156, 277, 10, 53, "Output",
 CellID->242735069]
}, Open  ]],
Cell[50662, 1169, 80, 1, 70, "MathCaption",
 CellID->1480230272],
Cell[CellGroupData[{
Cell[50767, 1174, 434, 14, 70, "Input",
 CellID->1779614656],
Cell[51204, 1190, 194, 7, 39, "Output",
 CellID->1576859030]
}, Open  ]],
Cell[51413, 1200, 78, 1, 70, "MathCaption",
 CellID->1604822736],
Cell[CellGroupData[{
Cell[51516, 1205, 127, 4, 70, "Input",
 CellID->2008601279],
Cell[51646, 1211, 195, 6, 36, "Output",
 CellID->174029583]
}, Open  ]],
Cell[51856, 1220, 183, 4, 70, "Text",
 CellID->1245864415],
Cell[52042, 1226, 62, 1, 70, "MathCaption",
 CellID->1277664655],
Cell[CellGroupData[{
Cell[52129, 1231, 1268, 36, 70, "Input",
 CellID->1020870232],
Cell[53400, 1269, 1982, 43, 70, "Output",
 CellID->993468198]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[55431, 1318, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[55499, 1321, 179, 4, 70, "TutorialMoreAbout",
 CellID->534648361]
}, Open  ]],
Cell[CellGroupData[{
Cell[55715, 1330, 127, 2, 70, "TutorialRelatedLinksSection",
 CellID->480403165],
Cell[55845, 1334, 3507, 82, 70, "TutorialRelatedLinks",
 CellID->1744965082]
}, Open  ]]
}, Open  ]],
Cell[59379, 1420, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

