(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     17181,        477]
NotebookOptionsPosition[     13521,        376]
NotebookOutlinePosition[     16209,        440]
CellTagsIndexPosition[     16166,        437]
WindowTitle->Integrate By Substitution - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES TUTORIAL", "PacletNameCell"], Cell[TextData[
    Cell[BoxData[
     ActionMenuBox[
      FrameBox["\<\"URL \[RightGuillemet]\"\>",
       StripOnInput->
        False], {"\<\"Classroom \
Utilities/tutorial/IntegrateBySubstitution\"\>":>
      None, "\<\"Copy Mathematica URL\"\>":>
      Module[{DocumentationSearch`Private`nb$}, 
       DocumentationSearch`Private`nb$ = NotebookPut[
          Notebook[{
            Cell["Classroom Utilities/tutorial/IntegrateBySubstitution"]}, 
           Visible -> False]]; 
       SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
       FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
       NotebookClose[DocumentationSearch`Private`nb$]; Null], 
      Delimiter, "\<\"Copy web URL\"\>":>
      Module[{DocumentationSearch`Private`nb$}, 
       DocumentationSearch`Private`nb$ = NotebookPut[
          Notebook[{
            Cell[
             BoxData[
              MakeBoxes[
               Hyperlink[
               "http://reference.wolfram.com/mathematica/Classroom \
Utilities/tutorial/IntegrateBySubstitution.html"], StandardForm]], "Input", 
             TextClipboardType -> "PlainText"]}, Visible -> False]]; 
       SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
       FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
       NotebookClose[DocumentationSearch`Private`nb$]; 
       Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
        NotebookLocate[{
          URL[
           StringJoin[
           "http://reference.wolfram.com/mathematica/", 
            "Classroom Utilities/tutorial/IntegrateBySubstitution", ".html"]],
           None}]}]},
      Appearance->None,
      MenuAppearance->Automatic]],
     LineSpacing->{1.4, 0}]], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell["Integrate By Substitution", "Title",
 CellID->509267359],

Cell[TextData[{
 "This notebook contains an alternate interface for using the ",
 StyleBox[ButtonBox["SubstituteIntegrate",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/SubstituteIntegrate"], "MR"],
 " function. This was constructed using the ",
 StyleBox[ButtonBox["Interpretation",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Interpretation"], "MR"],
 " function. To use this interface you need to enter values in the input \
fields, tabbing from one to another. When all the fields are filled in with \
the correct values you can obtain the results by using \[ShiftKey]\[KeyBar]\
\[EnterKey]."
}], "Text",
 CellID->1534169418],

Cell[CellGroupData[{

Cell["Using the Interface", "Section",
 CellID->1324364347],

Cell["Make sure you have loaded the ClassroomUtilities package.", "Text",
 CellID->800099412],

Cell["Enter the integrand in the field labeled Integrand.", "Text",
 CellID->1932839871],

Cell["Tab to the Variable field and enter the variable", "Text",
 CellID->254583728],

Cell[TextData[{
 "Tab to the next field and enter the expression to be replaced by ",
 StyleBox["u",
  FontSlant->"Italic"]
}], "Text",
 CellID->1001291672],

Cell[TextData[{
 "Tab to the next field and enter ",
 StyleBox["u",
  FontSlant->"Italic"],
 " or whatever variable you want to use for the substitution"
}], "Text",
 CellID->1027273412],

Cell[TextData[{
 "Solve for \[DifferentialD]",
 StyleBox["x",
  FontSlant->"Italic"],
 " and enter the expression in the next field"
}], "Text",
 CellID->1763197251],

Cell["\<\
Use \[ShiftKey]\[KeyBar]\[EnterKey] to evaluate the input. The result will \
appear with the integral unevaluated.\
\>", "Text",
 CellID->929352604],

Cell["\<\
You can evaluate the output by putting your cursor in the output cell and \
using \[ShiftKey]\[KeyBar]\[EnterKey].\
\>", "Text",
 CellID->980650797],

Cell[BoxData[
 RowBox[{"Interpretation", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"intgrnd", " ", "=", " ", 
      RowBox[{"(", 
       RowBox[{"x", " ", 
        RowBox[{"Sqrt", "[", 
         RowBox[{"1", " ", "-", " ", 
          RowBox[{"x", "^", "2"}]}], "]"}]}], ")"}]}], ",", " ", 
     RowBox[{"var", " ", "=", " ", "x"}], ",", " ", 
     RowBox[{"expr", " ", "=", " ", 
      RowBox[{"(", 
       RowBox[{"1", " ", "-", " ", 
        RowBox[{"x", "^", "2"}]}], ")"}]}], ",", " ", 
     RowBox[{"newvar", " ", "=", " ", "u"}], ",", " ", 
     RowBox[{"coef", " ", "=", " ", 
      RowBox[{"-", 
       RowBox[{"(", 
        RowBox[{"1", "/", 
         RowBox[{"(", 
          RowBox[{"2", " ", "x"}], ")"}]}], ")"}]}]}]}], "}"}], ",", "\n", 
   " ", 
   RowBox[{"Framed", "[", "\n", "  ", "\n", "  ", 
    RowBox[{
     RowBox[{"Panel", "[", "\n", "   ", "\n", "   ", 
      RowBox[{
       RowBox[{"Grid", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Item", "[", 
              RowBox[{
               RowBox[{"Style", "[", 
                RowBox[{
                "\"\<Integration by substitution\>\"", ",", " ", 
                 "\"\<Subsection\>\""}], "]"}], ",", " ", 
               RowBox[{"Alignment", " ", "->", " ", "Center"}]}], "]"}], ",", 
             " ", "SpanFromLeft", ",", " ", "SpanFromLeft", ",", " ", 
             "SpanFromLeft"}], "}"}], ",", "\n", "     ", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Style", "[", 
              RowBox[{"\"\<Integrand\>\"", ",", " ", "\"\<Text\>\""}], "]"}], 
             ",", " ", 
             RowBox[{"InputField", "[", 
              RowBox[{"Dynamic", "[", "intgrnd", "]"}], "]"}], ",", " ", 
             "SpanFromLeft", ",", " ", "SpanFromLeft"}], "}"}], ",", "\n", 
           "     ", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Style", "[", 
              RowBox[{"\"\<Variable\>\"", ",", " ", "\"\<Text\>\""}], "]"}], 
             ",", " ", 
             RowBox[{"InputField", "[", 
              RowBox[{"Dynamic", "[", "var", "]"}], "]"}], ",", " ", 
             "SpanFromLeft", ",", " ", "SpanFromLeft"}], "}"}], ",", "\n", 
           "     ", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Style", "[", 
              RowBox[{"\"\<Expression\>\"", ",", " ", "\"\<Text\>\""}], "]"}],
              ",", " ", 
             RowBox[{"InputField", "[", 
              RowBox[{
               RowBox[{"Dynamic", "[", "expr", "]"}], ",", " ", 
               RowBox[{"FieldSize", " ", "->", " ", "6"}]}], "]"}], ",", " ", 
             RowBox[{"Style", "[", 
              RowBox[{"\"\<\[Rule]\>\"", ",", " ", "\"\<Subsubsection\>\""}], 
              "]"}], ",", " ", 
             RowBox[{"InputField", "[", 
              RowBox[{"Dynamic", "[", "newvar", "]"}], "]"}]}], "}"}], ",", 
           "\n", "     ", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"StringForm", "[", 
              RowBox[{"\"\<\[DifferentialD]`1`\[Rule]\>\"", ",", " ", "var"}],
               "]"}], ",", " ", 
             RowBox[{"InputField", "[", 
              RowBox[{
               RowBox[{"Dynamic", "[", "coef", "]"}], ",", " ", 
               RowBox[{"FieldSize", " ", "->", " ", "6"}]}], "]"}], ",", " ", 
             RowBox[{"Style", "[", 
              RowBox[{
               RowBox[{"StringForm", "[", 
                RowBox[{"\"\<\[DifferentialD]`1`\>\"", ",", " ", "newvar"}], 
                "]"}], ",", " ", "\"\<Text\>\""}], "]"}], ",", " ", 
             "SpanFromLeft"}], "}"}]}], "}"}], ",", "\n", "    ", "\n", 
         "    ", 
         RowBox[{"Alignment", " ", "->", " ", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"Right", ",", " ", 
              RowBox[{"{", "Left", "}"}]}], "}"}], ",", " ", "Automatic"}], 
           "}"}]}]}], "]"}], ",", " ", 
       RowBox[{"Background", " ", "->", " ", "LightBrown"}]}], "]"}], ",", 
     "\n", "  ", "\n", "  ", 
     RowBox[{"RoundingRadius", " ", "->", " ", "8"}], ",", " ", 
     RowBox[{"FrameStyle", " ", "->", " ", 
      RowBox[{"Directive", "[", 
       RowBox[{
        RowBox[{"AbsoluteThickness", "[", "18", "]"}], ",", " ", "Brown"}], 
       "]"}]}], ",", " ", 
     RowBox[{"Background", " ", "->", " ", "Brown"}]}], "]"}], ",", "\n", " ",
    "\n", " ", 
   RowBox[{"ClassroomUtilities`SubstituteIntegrate", "[", 
    RowBox[{"intgrnd", ",", " ", "var", ",", " ", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"expr", " ", "->", " ", "newvar"}], ",", " ", 
       RowBox[{
        RowBox[{"\[DifferentialD]", "var"}], " ", "->", " ", 
        RowBox[{"coef", " ", 
         RowBox[{"\[DifferentialD]", "newvar"}]}]}]}], "}"}]}], "]"}]}], "\n",
   " ", "]"}]], "Input",
 CellOpen->False,
 CellID->761520622],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellID->53712336],

Cell[BoxData[
 TagBox[
  DynamicModuleBox[{$CellContext`intgrnd$$ = $CellContext`x (
     1 - $CellContext`x^2)^
    Rational[1, 2], $CellContext`var$$ = $CellContext`x, $CellContext`expr$$ =
    1 - $CellContext`x^2, $CellContext`newvar$$ = $CellContext`u, \
$CellContext`coef$$ = Rational[-1, 2]/$CellContext`x}, 
   InterpretationBox[
    FrameBox[
     PanelBox[
      TagBox[GridBox[{
         {
          ItemBox[
           StyleBox["\<\"Integration by substitution\"\>", "Subsection",
            StripOnInput->False],
           Alignment->Center,
           StripOnInput->False], "\[SpanFromLeft]", "\[SpanFromLeft]", 
          "\[SpanFromLeft]"},
         {
          StyleBox["\<\"Integrand\"\>", "Text",
           StripOnInput->False], 
          InputFieldBox[Dynamic[$CellContext`intgrnd$$]], "\[SpanFromLeft]", 
          "\[SpanFromLeft]"},
         {
          StyleBox["\<\"Variable\"\>", "Text",
           StripOnInput->False], 
          InputFieldBox[Dynamic[$CellContext`var$$]], "\[SpanFromLeft]", 
          "\[SpanFromLeft]"},
         {
          StyleBox["\<\"Expression\"\>", "Text",
           StripOnInput->False], 
          InputFieldBox[Dynamic[$CellContext`expr$$],
           FieldSize->6], 
          StyleBox["\<\"\[Rule]\"\>", "Subsubsection",
           StripOnInput->False], 
          InputFieldBox[Dynamic[$CellContext`newvar$$]]},
         {
          InterpretationBox["\<\"\[DifferentialD]\\!\\(x\\)\[Rule]\"\>",
           StringForm["\[DifferentialD]`1`\[Rule]", $CellContext`x],
           Editable->False], 
          InputFieldBox[Dynamic[$CellContext`coef$$],
           FieldSize->6], 
          StyleBox[
           InterpretationBox["\<\"\[DifferentialD]\\!\\(u\\)\"\>",
            StringForm["\[DifferentialD]`1`", $CellContext`u],
            Editable->False], "Text",
           StripOnInput->False], "\[SpanFromLeft]"}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, {Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{
         "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
       "Grid"],
      Background->RGBColor[0.94, 0.91, 0.88]],
     Background->RGBColor[0.6, 0.4, 0.2],
     FrameStyle->Directive[
       AbsoluteThickness[18], 
       RGBColor[0.6, 0.4, 0.2]],
     RoundingRadius->8,
     StripOnInput->False],
    ClassroomUtilities`SubstituteIntegrate[$CellContext`intgrnd$$, \
$CellContext`var$$, {$CellContext`expr$$ -> $CellContext`newvar$$, 
      DifferentialD[$CellContext`var$$] -> $CellContext`coef$$ 
       DifferentialD[$CellContext`newvar$$]}]],
   DynamicModuleValues:>{}],
  Setting[#, {0}]& ]], "Input",
 CellLabel->"In[13]:=",
 CellID->2008438101],

Cell[BoxData[
 RowBox[{"\[Integral]", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     SqrtBox["u"], "2"]}], 
   RowBox[{"\[DifferentialD]", "u"}]}]}]], "Input",
 CellLabel->"In[57]:=",
 CellID->1671199961],

Cell[TextData[{
 "After evaluating the above cell click the button ",
 Cell[BoxData[
  TagBox[
   ButtonBox[
    StyleBox["\<\"back substitute\"\>",
     StripOnInput->False,
     FontSize->14,
     FontColor->RGBColor[0.6, 0.4, 0.2],
     Background->RGBColor[0.94, 0.91, 0.88]],
    Appearance->"Palette",
    Background->RGBColor[0.94, 0.91, 0.88],
    ButtonFunction:>(SelectionMove[
       SelectedNotebook[], Previous, CellContents]; NotebookApply[
       SelectedNotebook[], 
       RowBox[{"\[SelectionPlaceholder]", "/.", "\[InvisibleSpace]", 
         RowBox[{"\[SelectionPlaceholder]", "\[Rule]", "\[Placeholder]"}]}]]),
    Evaluator->Automatic,
    FrameMargins->5,
    Method->"Preemptive"],
   DisplayForm]], "Output"],
 " . After entering the variable you wish to replace tab to the next \
placeholder and type in the substitution. You can then evaluate the cell and \
get the desired answer."
}], "Text",
 CellID->596989683]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Integrate By Substitution - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "TutorialColor", 
  "LinkTrails" -> GridBox[{{
      RowBox[{
        StyleBox[
         ButtonBox[
         "Mathematica", ButtonData -> "paclet:guide/Mathematica", 
          BaseStyle -> {"Link", "DockedLinkTrail"}], FontSlant -> "Italic"], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 32, 43.0168624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Tutorial", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "This notebook contains an alternate interface for using the \
SubstituteIntegrate function. This was constructed using the Interpretation \
function. To use this interface you need to enter values in the input fields, \
tabbing from one to another. When all the fields are filled in with the \
correct values you can obtain the results by using \[ShiftKey]\[KeyBar]\
\[EnterKey]. Make sure you have loaded the ClassroomUtilities package. Enter \
the integrand in the field labeled Integrand.", "synonyms" -> {}, "title" -> 
    "Integrate By Substitution", "titlemodifier" -> "", "windowtitle" -> 
    "Integrate By Substitution", "type" -> "Tutorial", "uri" -> 
    "Classroom Utilities/tutorial/IntegrateBySubstitution"}, 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> Notebook], 
   Cell[
    StyleData["Output"], CellContext -> Notebook], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[618, 21, 2022, 46, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[2665, 71, 62, 1, 51, "Title",
 CellID->509267359],
Cell[2730, 74, 653, 14, 92, "Text",
 CellID->1534169418],
Cell[CellGroupData[{
Cell[3408, 92, 59, 1, 51, "Section",
 CellID->1324364347],
Cell[3470, 95, 93, 1, 30, "Text",
 CellID->800099412],
Cell[3566, 98, 88, 1, 30, "Text",
 CellID->1932839871],
Cell[3657, 101, 84, 1, 30, "Text",
 CellID->254583728],
Cell[3744, 104, 156, 5, 30, "Text",
 CellID->1001291672],
Cell[3903, 111, 186, 6, 30, "Text",
 CellID->1027273412],
Cell[4092, 119, 165, 6, 30, "Text",
 CellID->1763197251],
Cell[4260, 127, 158, 4, 30, "Text",
 CellID->929352604],
Cell[4421, 133, 158, 4, 30, "Text",
 CellID->980650797],
Cell[4582, 139, 4915, 120, 19, "Input",
 CellOpen->False,
 CellID->761520622],
Cell[9500, 261, 103, 2, 28, "Input",
 CellID->53712336],
Cell[9606, 265, 2706, 68, 196, "Input",
 CellID->2008438101],
Cell[12315, 335, 208, 8, 70, "Input",
 CellID->1671199961],
Cell[12526, 345, 941, 25, 70, "Text",
 CellID->596989683]
}, Open  ]]
}, Open  ]],
Cell[13494, 374, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

