(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     10315,        308]
NotebookOptionsPosition[      6839,        212]
NotebookOutlinePosition[      9502,        276]
CellTagsIndexPosition[      9459,        273]
WindowTitle->Integration By Parts - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES TUTORIAL", "PacletNameCell"], Cell[TextData[
    Cell[BoxData[
     ActionMenuBox[
      FrameBox["\<\"URL \[RightGuillemet]\"\>",
       StripOnInput->
        False], {"\<\"Classroom Utilities/tutorial/IntegrationByParts\"\>":>
      None, "\<\"Copy Mathematica URL\"\>":>
      Module[{DocumentationSearch`Private`nb$}, 
       DocumentationSearch`Private`nb$ = NotebookPut[
          Notebook[{
            Cell["Classroom Utilities/tutorial/IntegrationByParts"]}, Visible -> 
           False]]; SelectionMove[
        DocumentationSearch`Private`nb$, All, Notebook]; 
       FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
       NotebookClose[DocumentationSearch`Private`nb$]; Null], 
      Delimiter, "\<\"Copy web URL\"\>":>
      Module[{DocumentationSearch`Private`nb$}, 
       DocumentationSearch`Private`nb$ = NotebookPut[
          Notebook[{
            Cell[
             BoxData[
              MakeBoxes[
               Hyperlink[
               "http://reference.wolfram.com/mathematica/Classroom \
Utilities/tutorial/IntegrationByParts.html"], StandardForm]], "Input", 
             TextClipboardType -> "PlainText"]}, Visible -> False]]; 
       SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
       FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
       NotebookClose[DocumentationSearch`Private`nb$]; 
       Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
        NotebookLocate[{
          URL[
           StringJoin[
           "http://reference.wolfram.com/mathematica/", 
            "Classroom Utilities/tutorial/IntegrationByParts", ".html"]], 
          None}]}]},
      Appearance->None,
      MenuAppearance->Automatic]],
     LineSpacing->{1.4, 0}]], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell["Integration By Parts", "Title",
 CellID->509267359],

Cell[TextData[{
 "This notebook contains an alternate interface for using the ",
 StyleBox[ButtonBox["PartsIntegrate",
  BaseStyle->"Link",
  ButtonData->"paclet:ClassroomUtilities/ref/PartsIntegrate"], "MR"],
 " function. This was constructed using the ",
 StyleBox[ButtonBox["Interpretation",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Interpretation"], "MR"],
 " function. To use this interface you need to enter values in the input \
fields, tabbing from one to another. When all the fields are filled in with \
the correct values you can obtain the results by using \[ShiftKey]\[KeyBar]\
\[EnterKey]."
}], "Text",
 CellID->1534169418],

Cell[CellGroupData[{

Cell["Using the Interface", "Section",
 CellID->2100806306],

Cell["Make sure you have loaded the ClassroomUtilities package.", "Text",
 CellID->1932839871],

Cell["Enter the integrand in the field labeled Integrand.", "Text",
 CellID->836781195],

Cell["Tab to the Variable field and enter the variable", "Text",
 CellID->1472944615],

Cell[TextData[{
 "Tab to the next field and enter the expression for ",
 StyleBox["u",
  FontSlant->"Italic"]
}], "Text",
 CellID->1736145336],

Cell[TextData[{
 "Tab to the next field and enter the expression for \[DifferentialD]",
 StyleBox["v",
  FontSlant->"Italic"],
 " without the differential"
}], "Text",
 CellID->1534350363],

Cell["\<\
Use \[ShiftKey]\[KeyBar]\[EnterKey] to evaluate the input. The result will \
appear with the integral unevaluated.\
\>", "Text",
 CellID->842917765],

Cell["\<\
You can evaluate the output by putting your cursor in the output cell and \
using \[ShiftKey]\[KeyBar]\[EnterKey].\
\>", "Text",
 CellID->2116740743],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellID->1875703617],

Cell[CellGroupData[{

Cell[BoxData[
 TagBox[
  DynamicModuleBox[{$CellContext`intgrnd$$ = $CellContext`x 
   Cos[$CellContext`x], $CellContext`var$$ = $CellContext`x, $CellContext`u$$ \
= $CellContext`x, $CellContext`dvar$$ = Cos[$CellContext`x]}, 
   InterpretationBox[
    FrameBox[
     PanelBox[
      TagBox[GridBox[{
         {
          ItemBox[
           
           StyleBox["\<\"\[Integral]u \[DifferentialD]v = u v -\[Integral]v\
\[DifferentialD]u\"\>", "Subsection",
            StripOnInput->False],
           Alignment->Center,
           StripOnInput->False], "\[SpanFromLeft]"},
         {
          StyleBox["\<\"Integrand\"\>", "Text",
           StripOnInput->False], 
          InputFieldBox[Dynamic[$CellContext`intgrnd$$]]},
         {
          StyleBox["\<\"Variable\"\>", "Text",
           StripOnInput->False], 
          InputFieldBox[Dynamic[$CellContext`var$$]]},
         {
          StyleBox["\<\"u\"\>", "Text",
           StripOnInput->False], 
          InputFieldBox[Dynamic[$CellContext`u$$],
           FieldSize->6]},
         {
          StyleBox["\<\"dv\"\>", "Text",
           StripOnInput->False], 
          InputFieldBox[Dynamic[$CellContext`dvar$$],
           FieldSize->6]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, {Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{
         "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
       "Grid"],
      Background->RGBColor[0.94, 0.91, 0.88]],
     Background->RGBColor[0.6, 0.4, 0.2],
     FrameStyle->Directive[
       AbsoluteThickness[18], 
       RGBColor[0.6, 0.4, 0.2]],
     RoundingRadius->8,
     StripOnInput->False],
    ClassroomUtilities`PartsIntegrate[$CellContext`intgrnd$$, \
$CellContext`var$$, {$CellContext`u$$, $CellContext`dvar$$}]],
   DynamicModuleValues:>{}],
  Setting[#, {0}]& ]], "Input",
 CellLabel->"In[9]:=",
 CellID->2058623809],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   RowBox[{"\[Integral]", 
    RowBox[{
     RowBox[{"Sin", "[", "x", "]"}], 
     RowBox[{"\[DifferentialD]", "x"}]}]}]}], "+", 
  RowBox[{"x", " ", 
   RowBox[{"Sin", "[", "x", "]"}]}]}]], "Output",
 ImageSize->{149, 28},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->1374151455]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Integration By Parts - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "TutorialColor", 
  "LinkTrails" -> GridBox[{{
      RowBox[{
        StyleBox[
         ButtonBox[
         "Mathematica", ButtonData -> "paclet:guide/Mathematica", 
          BaseStyle -> {"Link", "DockedLinkTrail"}], FontSlant -> "Italic"], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 32, 45.3100624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Tutorial", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "This notebook contains an alternate interface for using the \
PartsIntegrate function. This was constructed using the Interpretation \
function. To use this interface you need to enter values in the input fields, \
tabbing from one to another. When all the fields are filled in with the \
correct values you can obtain the results by using \[ShiftKey]\[KeyBar]\
\[EnterKey]. Make sure you have loaded the ClassroomUtilities package. Enter \
the integrand in the field labeled Integrand.", "synonyms" -> {}, "title" -> 
    "Integration By Parts", "titlemodifier" -> "", "windowtitle" -> 
    "Integration By Parts", "type" -> "Tutorial", "uri" -> 
    "Classroom Utilities/tutorial/IntegrationByParts"}, 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> Notebook], 
   Cell[
    StyleData["Output"], CellContext -> Notebook], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[613, 21, 2001, 45, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[2639, 70, 57, 1, 51, "Title",
 CellID->509267359],
Cell[2699, 73, 643, 14, 92, "Text",
 CellID->1534169418],
Cell[CellGroupData[{
Cell[3367, 91, 59, 1, 51, "Section",
 CellID->2100806306],
Cell[3429, 94, 94, 1, 30, "Text",
 CellID->1932839871],
Cell[3526, 97, 87, 1, 30, "Text",
 CellID->836781195],
Cell[3616, 100, 85, 1, 30, "Text",
 CellID->1472944615],
Cell[3704, 103, 142, 5, 30, "Text",
 CellID->1736145336],
Cell[3849, 110, 188, 6, 30, "Text",
 CellID->1534350363],
Cell[4040, 118, 158, 4, 30, "Text",
 CellID->842917765],
Cell[4201, 124, 159, 4, 30, "Text",
 CellID->2116740743],
Cell[4363, 130, 105, 2, 28, "Input",
 CellID->1875703617],
Cell[CellGroupData[{
Cell[4493, 136, 1904, 54, 164, "Input",
 CellID->2058623809],
Cell[6400, 192, 373, 13, 49, "Output",
 CellID->1374151455]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[6812, 210, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

