(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     77179,       1914]
NotebookOptionsPosition[     69471,       1672]
NotebookOutlinePosition[     72833,       1758]
CellTagsIndexPosition[     72753,       1753]
WindowTitle->Plotting Piecewise Continuous Functions - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Classroom Utilities\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/guide/ClassroomUtilities"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"TutorialMoreAboutSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Functions \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PiecewisePlot\"\>":>
       Documentation`HelpLookup[
       "paclet:PiecewisePlot/ref/PiecewisePlot"], \
"\<\"PiecewisePolarPlot\"\>":>Documentation`HelpLookup[
       "paclet:PiecewisePlot/ref/PiecewisePolarPlot"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"TutorialFunction"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"ClassroomUtilities/tutorial/\
PlottingPiecewiseContinuousFunctions\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
             "ClassroomUtilities/tutorial/\
PlottingPiecewiseContinuousFunctions"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/ClassroomUtilities/\
tutorial/PlottingPiecewiseContinuousFunctions.html"], StandardForm]], "Input",
               TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "ClassroomUtilities/tutorial/\
PlottingPiecewiseContinuousFunctions", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["Plotting Piecewise Continuous Functions", "Title",
 CellID->509267359],

Cell["\<\
The functions defined in the PiecewisePlots` package provide a means of \
plotting piecewise continuous functions together with continuous functions in \
the manner usually found in standard text books. Point of discontinuity are \
represented by filled and open circles with filled circles used for values \
the function takes and open circles indicating values the function does not \
take.\
\>", "Text",
 CellID->1534169418],

Cell[BoxData[GridBox[{
   {
    RowBox[{
     ButtonBox["PiecewisePlot",
      BaseStyle->"Link",
      ButtonData->"paclet:PiecewisePlot/ref/PiecewisePlot"], "[", 
     RowBox[{
      StyleBox["fn", "TI"], ",", 
      RowBox[{"{", 
       RowBox[{
        StyleBox["x", "TI"], ",", 
        SubscriptBox[
         StyleBox["x", "TI"], 
         StyleBox["min", "TI"]], ",", 
        SubscriptBox[
         StyleBox["x", "TI"], 
         StyleBox["max", "TI"]]}], "}"}], ",", 
      StyleBox["opts", "TI"]}], "]"}], Cell[TextData[{
     "create a plot of ",
     Cell[BoxData[
      StyleBox["fn", "TI"]], "InlineFormula"],
     " on the interval ",
     Cell[BoxData[
      RowBox[{"(", 
       RowBox[{
        SubscriptBox[
         StyleBox["x", "TI"], 
         StyleBox["min", "TI"]], ",", " ", 
        SubscriptBox[
         StyleBox["x", "TI"], 
         StyleBox["max", "TI"]]}], ")"}]], "InlineFormula"],
     "."
    }], "TableText"]},
   {
    RowBox[{
     ButtonBox["PiecewisePlot",
      BaseStyle->"Link",
      ButtonData->"paclet:PiecewisePlot/ref/PiecewisePlot"], "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        SubscriptBox[
         StyleBox["fn", "TI"], 
         StyleBox["1", "TR"]], ",", 
        SubscriptBox[
         StyleBox["fn", "TI"], 
         StyleBox["2", "TR"]], ",", 
        StyleBox["\[Ellipsis]", "TR"]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        StyleBox["x", "TI"], ",", " ", 
        SubscriptBox[
         StyleBox["x", "TI"], 
         StyleBox["min", "TI"]], ",", 
        SubscriptBox[
         StyleBox["x", "TI"], 
         StyleBox["max", "TI"]]}], "}"}], ",", 
      StyleBox["opts", "TI"]}], "]"}], Cell[
    "create a plot of several functions.", "TableText"]},
   {
    RowBox[{
     ButtonBox["PiecewisePolarPlot",
      BaseStyle->"Link",
      ButtonData->"paclet:PiecewisePlot/ref/PiecewisePolarPlot"], "[", 
     RowBox[{
      StyleBox["fn", "TI"], ",", 
      RowBox[{"{", 
       RowBox[{
        StyleBox["\[Theta]", "TR"], ",", 
        SubscriptBox[
         StyleBox["\[Theta]", "TR"], 
         StyleBox["min", "TI"]], ",", 
        SubscriptBox[
         StyleBox["\[Theta]", "TR"], 
         StyleBox["max", "TI"]]}], "}"}], ",", 
      StyleBox["ops", "TI"]}], "]"}], Cell[TextData[{
     "create a polar plot for \[Theta] in the interval ",
     Cell[BoxData[
      RowBox[{"(", 
       RowBox[{
        SubscriptBox[
         StyleBox["\[Theta]", "TR"], 
         StyleBox["min", "TI"]], ",", " ", 
        SubscriptBox[
         StyleBox["\[Theta]", "TR"], 
         StyleBox["max", "TI"]]}], ")"}]], "InlineFormula"]
    }], "TableText"]},
   {
    RowBox[{
     ButtonBox["PiecewisePolarPlot",
      BaseStyle->"Link",
      ButtonData->"paclet:PiecewisePlot/ref/PiecewisePolarPlot"], "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        SubscriptBox[
         StyleBox["fn", "TI"], 
         StyleBox["1", "TR"]], ",", 
        SubscriptBox[
         StyleBox["fn", "TI"], 
         StyleBox["2", "TR"]], ",", 
        StyleBox["\[Ellipsis]", "TR"]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        StyleBox["\[Theta]", "TR"], ",", 
        SubscriptBox[
         StyleBox["\[CapitalTheta]", "TR"], 
         StyleBox["min", "TI"]], ",", 
        SubscriptBox[
         StyleBox["\[Theta]", "TR"], 
         StyleBox["max", "TI"]]}], "}"}], ",", " ", 
      StyleBox["opts", "TI"]}], "]"}], 
    RowBox[{
    "create", " ", "a", " ", "polar", " ", "plot", " ", "for", " ", "several",
      " ", 
     RowBox[{"functions", "."}]}]}
  }]], "DefinitionBox",
 CellID->2096742444],

Cell["Piecewise plotting functions.", "Caption",
 CellID->1891092685],

Cell["This loads the package.", "MathCaption",
 CellID->1009911057],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1553316875],

Cell[CellGroupData[{

Cell["Working with Built-in Functions", "Section",
 CellID->1800518439],

Cell[TextData[{
 "Many of ",
 StyleBox["Mathematica's ",
  FontSlant->"Italic"],
 "built-in functions, ",
 ButtonBox["Abs",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Abs"],
 ", ",
 ButtonBox["Floor",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Floor"],
 " and ",
 ButtonBox["Ceiling",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Ceiling"],
 " for example, have piecewise continuous definitions. These are \
automatically handled by the ",
 Cell[BoxData[
  ButtonBox["PiecewisePlot",
   BaseStyle->"Link",
   ButtonData->"paclet:ClassroomUtilities/ref/PiecewisePlot"]], 
  "InlineFormula"],
 " function."
}], "Text",
 CellID->1121334396],

Cell["Plotting absolute value.", "MathCaption",
 CellID->836781195],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PiecewisePlot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Abs", "[", 
     RowBox[{"x", "-", "1"}], "]"}], "+", 
    RowBox[{"1", "/", "2"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "1"}], ",", "3"}], "}"}], ",", 
   RowBox[{"AxesOrigin", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0"}], "}"}]}]}], "]"}]], "Input",
 CellLabel->"In[27]:=",
 CellID->2058623809],

Cell[BoxData[
 FormBox[
  GraphicsBox[{{{}, {}, 
     {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJw11Xs81Pkex/ExP/wmOhZLKnLZdV+XVXIJ+/kuFUm7bqWzyQhFcst2RMdt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       "]]}}, {
     {RGBColor[0.24720000000000014`, 0.24, 0.6], AbsolutePointSize[6], 
      PointBox[{{1., 0.5}}]}, 
     {RGBColor[1, 1, 1], AbsolutePointSize[4], PointBox[{}]}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->True,
   AxesOrigin->{0, 0},
   PlotRange->{{-1, 3}, {0, 2.499999918367347}},
   PlotRangeClipping->True,
   PlotRangePadding->{
     Scaled[0.02], 
     Scaled[0.02]}], TraditionalForm]], "Output",
 ImageSize->{184, 127},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[27]=",
 CellID->2098191147]
}, Open  ]],

Cell["Visualizing the ceiling function.", "MathCaption",
 CellID->469640242],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PiecewisePlot", "[", 
  RowBox[{
   RowBox[{"Ceiling", "[", "x", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "8"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[28]:=",
 CellID->1241144569],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnSGX6joUhVnr/ZL3M2pHIpHIW4l4opL1HBKJrI3EvchaZGVkbSUSOS9p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    "], {{0, 125}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 125},
  PlotRange->{{0, 180}, {0, 125}}]], "Output",
 ImageSize->{184, 125},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[28]=",
 CellID->72957386]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has many built-in functions that are used in acoustical analysis such as ",
 ButtonBox["SquareWave",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SquareWave"],
 " and ",
 ButtonBox["SawtoothWave",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SawtoothWave"],
 ". These can be used to create many interesting examples of piecewise \
continuous functions."
}], "Text",
 CellID->799214764],

Cell["Plotting a square wave with period four.", "MathCaption",
 CellID->2039810210],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PiecewisePlot", "[", 
  RowBox[{
   RowBox[{"SquareWave", "[", 
    RowBox[{"x", "/", "4"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "8"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[29]:=",
 CellID->661258762],

Cell[BoxData[
 FormBox[
  GraphicsBox[{{{{}, {}, 
      {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxTTMoPSmViYGAQAWIQ7ZX64uKBl612DGDwwX6BqH9bl+QWexi/kWOzeqfk
ETg/InLh+g8fr8D53P8bY9zl7sP5qoHvXhzufgLnl5+ztJ/x5SWcL75F8B4j
43s4f9sj5zPnbT7B+Yey5kpHvvkC539Vs+Vfm/sdzmf6uVpAWvUXnG/2P2Fb
8Lo/cD7jIS+n3jd/4fwzLSbnjmn9h/OtbrwokmhE8FdozxXPuo7gi20z9jF5
hOC3OJxq+PcGwa/NWMPf+xXBBwBTPGuP
        "]], LineBox[CompressedData["
1:eJxTTMoPSmViYGAQA2IQffC0ZoUlK4MDAxh82C9nOmmlMA+C371f495dMQQ/
7Uqoe5EOgu/0smkDmxOCv8HGPDRiBoIvP+H1z5XvEHzWI6lv9vczwvmvv9+/
d9WACc5f8mr3Kb4+ZjifT3b5+ydPWOD8nsytaQtU2OD8bw6rn2TUs8P5JUZH
togd5oDz76870n1RkQvO/511ZbZmFTecr+3I/i9tNw+c775IMX2jOB9Cf/wB
F89dCH65bLzSwxh+ON/Apyh+yhoE397kWvuxXwg+AEmpcFw=
        "]], LineBox[CompressedData["
1:eJxTTMoPSmViYGCQAWIQ/fCpyJ45TAIODGDwwT7hgzCTPDOCf63zz+w5rAi+
t/JTUykOBN8sbFumKC+Cr9TZPldACMH/+U6TlV0Gwb8QcibnryqCX6cgaPPK
EsHXeRPKfd0XwV/wV6nLdwmCL8z/gePILwSfZf/s9cpzBOH8bJPdNzm9heD8
1Z5JC/Z9Q/C/vpYSeDZNGM7fGuzOPMVFBM6v92idv+UZgq941+vG4lZRON/K
vGWKpJEYnB8jd/4n42UEf0WJXp6lljjCvlO9jwoaEfy4m4zumqsQ/GPPS1c/
vITg/3q3/q7tcwRf/9tL/lm/EXx5qdfOyX8RfABLQXyt
        "]], LineBox[CompressedData["
1:eJxTTMoPSmViYGAQBWIQ/fCpyJ45TBIODGDwYf/jHfsX2TAj+Kd+smmeYEXw
p1dPNb3PheCn/FV2+saP4P9jcIxWkULwZ7DElQSpIPhhHzu7tUoQfOF7Wxcx
HUHwWWsbQywLJeF8hpVx6x8JScH5RbPW9wodQvD3andwCOZJw/mCXkU6TXwy
cP4d3QuXK/cg+Bvm2as8iZOF82PXmwnK/kPwG1oXNt5ZJQfnr30hO5vPRx7O
V2oR8WBfhuBPl+f++v8/gg8A6QBlTA==
        "]]}, {}}, {{}, {}, {}}}, {
     {RGBColor[0.24720000000000014`, 0.24, 0.6], AbsolutePointSize[6], 
      PointBox[{{0., -1.}, {0., 0.}, {0., 1.}, {2., -1.}, {2., 0.}, {2., 
       1.}, {4., -1.}, {4., 0.}, {4., 1.}, {6., -1.}, {6., 0.}, {6., 1.}, {
       8., -1.}, {8., 0.}, {8., 1.}}]}, 
     {RGBColor[1, 1, 1], AbsolutePointSize[4], 
      PointBox[{{0., -1.}, {0., 1.}, {2., -1.}, {2., 1.}, {4., -1.}, {4., 
       1.}, {6., -1.}, {6., 1.}, {8., -1.}, {8., 1.}}]}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->True,
   AxesOrigin->{0, 0},
   PlotRange->{{0, 8}, {-1., 1.}},
   PlotRangeClipping->True,
   PlotRangePadding->{
     Scaled[0.02], 
     Scaled[0.02]}], TraditionalForm]], "Output",
 ImageSize->{184, 107},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[29]=",
 CellID->548139709]
}, Open  ]],

Cell["\<\
Adding a style for excluded values to make the function easier to understand.\
\>", "MathCaption",
 CellID->2026727180],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PiecewisePlot", "[", 
  RowBox[{
   RowBox[{"SquareWave", "[", 
    RowBox[{"x", "/", "4"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "8.1"}], "}"}], ",", 
   RowBox[{"ExclusionsStyle", "\[Rule]", " ", "Dashed"}]}], "]"}]], "Input",
 CellLabel->"In[31]:=",
 CellID->1965589193],

Cell[BoxData[
 FormBox[
  GraphicsBox[{{{{}, {}, 
      {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxTTMoPSmViYGAQBmIQ/cVnN/sD/TY7BjD4YC9ROc3xTuQWexh/TsWMe7ci
j8D5dyawPf1mchXOv5otUHK78T6c/6VuiUbz2Sdwvl2I+edl7q/g/L1vCr0s
Ut4j+B0X/2mXf4LzH5/4mu2r/xXOX/Iqd/7SHd/h/JaghwxHK37B+f9nr9CQ
5voL5y8QD2Lz9/kH5ztO/v24qe8/nC9RPuGS6S4E/12U6sEXTxH8J5NP1Vz9
gOAbTbg3P/srgg8AD0xsiQ==
        "]], LineBox[CompressedData["
1:eJxTTMoPSmViYGCQBGIQ/Xy7gIEXK4MDAxh82J8kf/f6WQ4EPyKiafIuYQTf
d6K6/3I5BF+5dI9tlBeC/zMiUIe3DMFfplDNWXwWwQ96seTZDRVGOF+woDfo
03EE/8L30n3c2UxwftwqF91FZ5jh/PPGN1MPqrPC+XzrpjPWZLPB+bGng01m
nWKH8ytvWp4KFuKE85d2yX4vKOWC87cIHX8rdokbzp/QPcs9QpwXzq9Znm3x
t5wPzr+pm5k0V4YfzjfbktZjexDBLzjkJJd1FcFfeUFuw7SXCP6x6zw/13xG
8CU93p/f+gvBBwB5OXxV
        "]], LineBox[CompressedData["
1:eJxTTMoPSmViYGCQAmIQ/TmaQ2Ipk4ADAxh8sGcoLjn6hhnBP/5lfXgRG4Lf
V/b65XcOBP9Ey/FqFVkEn2nyYt5AQwS/bH204apIBP/laeFQ5iYEf0quc1Pf
TQTfnr94vaShIJz/xyt9d+0VBN8p4takya1CcP7PAIPzucbCcL6Yk/TeqVcR
/D0srWX5rSII9+5vfjhNUxTOd0tL3dh0CsH3uXfUR6FSDM7f6bd7JreMOJy/
Of+ralQJgr92gv6mFWcQ/B/bC7jdHiL4Lvc3pjz+iuDrsdziL/+B4ItrMu3m
+o3g8y75/jzqL4IPAIH0dRs=
        "]], LineBox[CompressedData["
1:eJxTTMoPSmViYGCQBmIQ/TmaQ2Ipk4QDAxh82L9nz6L0SFYEv5/t894GLgTf
ZNbkrPMiCP4tXRNxOXkEv6Bm28kIDwSf/bRFzeRCBP/t3XXC2YcQ/MsfVFY7
CUvC+eov+pcuPoTgP7reHxiUJgXnq83kO7lJXBrOv+1asPT6UQT/26s0Y68M
GTh/VuKdnjwJWTh/6YVn0x/sQfAVtK+wi8fJwfkq4Q7O/3nk4fyolwcsZ8Uh
+P3VjgamGxD8V1KLQ38fQ/BddrFUH7yL4H/s3Nqr8hLBnxeZtqD9C4JfcnCf
puUvBN9bU3zzq/+4+QCkgInX
        "]], 
       LineBox[{{8.002698775576667, 1.}, {8.005478805837159, 1.}, {
        8.00825883609765, 1.}, {8.011038866358142, 1.}, {8.055519350526009, 
        1.}, {8.099999834693877, 1.}}]}, 
      {Dashing[{Small, Small}], 
       LineBox[{{3.9974170918367347`, -1.}, {4.002582908163265, 1.}}], 
       LineBox[{{7.999918745316175, -1.}, {8.000390624354273, 1.}}], 
       LineBox[{{1.9974170918367347`, 1.}, {2.0025829081632653`, -1.}}], 
       LineBox[{{5.997417091836735, 1.}, {
        6.002582908163265, -1.}}]}}, {{}, {}, {}}}, {
     {RGBColor[0.24720000000000014`, 0.24, 0.6], AbsolutePointSize[6], 
      PointBox[{{0., -1.}, {0., 0.}, {0., 1.}, {2., -1.}, {2., 0.}, {2., 
       1.}, {4., -1.}, {4., 0.}, {4., 1.}, {6., -1.}, {6., 0.}, {6., 1.}, {
       8., -1.}, {8., 0.}, {8., 1.}}]}, 
     {RGBColor[1, 1, 1], AbsolutePointSize[4], 
      PointBox[{{0., -1.}, {0., 1.}, {2., -1.}, {2., 1.}, {4., -1.}, {4., 
       1.}, {6., -1.}, {6., 1.}, {8., -1.}, {8., 1.}}]}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->True,
   AxesOrigin->{0, 0},
   PlotRange->{{0, 8.1}, {-1., 1.}},
   PlotRangeClipping->True,
   PlotRangePadding->{
     Scaled[0.02], 
     Scaled[0.02]}], TraditionalForm]], "Output",
 ImageSize->{184, 107},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[31]=",
 CellID->1815334719]
}, Open  ]],

Cell["Comparing a saw tooth wave to a square wave.", "MathCaption",
 CellID->1030449304],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PiecewisePlot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"SquareWave", "[", 
      RowBox[{"x", "/", "4"}], "]"}], ",", 
     RowBox[{
      RowBox[{"SawtoothWave", "[", 
       RowBox[{"x", "/", "4"}], "]"}], "-", 
      RowBox[{"1", "/", "2"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "8.1"}], "}"}], ",", 
   RowBox[{"ExclusionsStyle", "\[Rule]", " ", "Dashed"}]}], "]"}]], "Input",
 CellLabel->"In[34]:=",
 CellID->1453322920],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztXSGUqzwT7TmfWfnk2pWVK2uRyErkh0T8AomtRCKxkbgvEluJjMQikUj+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    "], {{0, 107}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 107},
  PlotRange->{{0, 180}, {0, 107}}]], "Output",
 ImageSize->{184, 107},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[34]=",
 CellID->36364065]
}, Open  ]],

Cell["\<\
Partitioning a continuous function using a piecewise continuous function.\
\>", "MathCaption",
 CellID->2116071404],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PiecewisePlot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", 
     RowBox[{
      RowBox[{"x", "/", "4"}], " ", "\[Pi]"}], "]"}], 
    RowBox[{"SquareWave", "[", 
     RowBox[{"x", "/", "4"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "16.05"}], "}"}], ",", 
   RowBox[{"ExclusionsStyle", "\[Rule]", " ", "Dashed"}]}], "]"}]], "Input",
 CellLabel->"In[9]:=",
 CellID->335409061],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztXa938zoP3jn3L3nhhReOFhYWDt7CgQsKRwcHC0sDy77A0sLCwNLCwsF+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    "], {{0, 107}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 107},
  PlotRange->{{0, 180}, {0, 107}}]], "Output",
 ImageSize->{184, 107},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[11]=",
 CellID->13311170]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["User Defined Functions", "Section",
 CellID->1297162916],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions such as Sinc have removable Singularities. These discontinuities \
will not be recognized and therefore will not be automatically included in \
piecewise plots. You can define your own function using the ",
 ButtonBox["Piecewise",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Piecewise"],
 " function to include the singularities."
}], "Text",
 CellID->1346377783],

Cell[TextData[{
 ButtonBox["Piecewise",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Piecewise"],
 " plotting the sinc function."
}], "MathCaption",
 CellID->536210442],

Cell[BoxData[
 RowBox[{
  RowBox[{"sinc", "[", "x_", "]"}], ":=", " ", 
  RowBox[{"\[Piecewise]", GridBox[{
     {
      RowBox[{"Sinc", "[", "x", "]"}], 
      RowBox[{"x", "\[NotEqual]", "0"}]},
     {"Indeterminate", 
      RowBox[{"x", "\[Equal]", "0"}]}
    }]}]}]], "Input",
 CellLabel->"In[37]:=",
 CellID->1247515882],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PiecewisePlot", "[", 
  RowBox[{
   RowBox[{"sinc", "[", 
    RowBox[{"x", "-", 
     RowBox[{"\[Pi]", "/", "2"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{
      RowBox[{"-", "3"}], " ", "\[Pi]"}], " ", ",", 
     RowBox[{"3", " ", "\[Pi]"}]}], "}"}], ",", 
   RowBox[{"Ticks", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Range", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "3"}], " ", "\[Pi]"}], ",", 
        RowBox[{"3", " ", "\[Pi]"}], ",", "\[Pi]"}], "]"}], ",", "None"}], 
     "}"}]}]}], "]"}]], "Input",
 CellLabel->"In[42]:=",
 CellID->1116583113],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnSGXqzgUx+ec/ST7McY+WYmsfLitWIGsraxE1iJxG4lFIiOxSGQl+wem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    "], {{0, 106}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 106},
  PlotRange->{{0, 180}, {0, 106}}]], "Output",
 ImageSize->{184, 106},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[42]=",
 CellID->76535670]
}, Open  ]],

Cell["Indeterminate is automatically generated with derivatives.", \
"MathCaption",
 CellID->868317803],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"abs", "[", "x_", "]"}], "=", 
  RowBox[{"PiecewiseExpand", "[", 
   RowBox[{
    RowBox[{"Abs", "[", "x", "]"}], ",", 
    RowBox[{"x", "\[Element]", "Reals"}]}], "]"}]}]], "Input",
 CellLabel->"In[52]:=",
 CellID->1329453543],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Piecewise]", GridBox[{
     {
      RowBox[{"-", "x"}], 
      RowBox[{"x", "<", "0"}]},
     {"x", 
      TagBox["True",
       "PiecewiseDefault",
       AutoDelete->False,
       DeletionWarning->True]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxItemSize->{
     "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.84]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}]}], TraditionalForm]], "Output",
 ImageSize->{66, 31},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[52]=",
 CellID->1514886135]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"abs", "'"}], "[", "x", "]"}]], "Input",
 CellLabel->"In[54]:=",
 CellID->1045072778],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Piecewise]", GridBox[{
     {
      RowBox[{"-", "1"}], 
      RowBox[{"x", "<", "0"}]},
     {"1", 
      RowBox[{"x", ">", "0"}]},
     {"Indeterminate", 
      TagBox["True",
       "PiecewiseDefault",
       AutoDelete->False,
       DeletionWarning->True]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxItemSize->{
     "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.84]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}]}], TraditionalForm]], "Output",
 ImageSize->{117, 48},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[54]=",
 CellID->269431033]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PiecewisePlot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"abs", "'"}], "[", 
    RowBox[{"x", "-", 
     RowBox[{"1", "/", "2"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "2"}], ",", "4"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[55]:=",
 CellID->1927215496],

Cell[BoxData[
 FormBox[
  GraphicsBox[{{{{}, {}, 
      {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxTTMoPSmViYGCQAWIQfTRO8f7////3M4DBh/1/vLY0nuVD8E3M3VVmy/6D
8x32C95fJPQHztdfsHxekNYvOH/KwmUd07x/wPnTgrxeuLl8g/OrYnb6zI//
Auf/Fj2rlhL0Cc7/9N5Te6/fBzh/uqtKZs3tN3B+M9e/tPATL+B808a8vKZf
T+D8yxYbT121fwjnh86vUmHJuAXnTzA4cOzPrUtwfjWDYb1f+FE4X7pvlYCw
6FI4f90TxoWc+ZvtYXx/bvYdr36chPNn6s206Zh0Bc6/sXjWwR3rbsD54pJz
3F6eugPn6zxbHqR99i6c77B5c1zu+Xtw/qmZO1P6Au7D+aEN+7PWX0TwKyxZ
9Zc/QPABnMarew==
        "]], LineBox[CompressedData["
1:eJxTTMoPSmViYGBQBWIQ7ZJcm7GG/4E9Axh8sJ/3baG4iySCX7A5ySJDE8EX
0n1SucETwd+6LHX3j0wE/1Rx4XGrnIdw/j2H2ss1eY/gfNWmk88iAp7A+TmH
RX+ZGD2D8w88SzEVcX8F5/tHyKYvFnsH51ceYVRqffkBztfVvSDl8+ETnP/P
5jJf9pUvcL7RXMNe2Yff4PwPd9QeWj/5AeefORxgsOnILzif+/dVzj1n/8D5
6Tf778lu+QfnL/MuOFLqweAA458OOhan5s0I59ev3fatyYYJzk/ZcCVY04kZ
zr/B1ruu2owFzhc8K/CFWY0Vzq+xPvTvqiYbnF/6l5c9RIYdzl+Q3n5FWpkD
zp/geKZHUpUTzufLX+10WJQLzt/nsmPvNiluOP9vbrxZGB8PnM/Ifmmm9l8E
38yj6JAXAx+c/+vOmheinxB8vWNyb7gq+eH85PUT3v//j+ADAE6As84=
        "]]}, {}}, {{}, {}, {}}}, {
     {RGBColor[0.24720000000000014`, 0.24, 0.6], AbsolutePointSize[6], 
      PointBox[{{0.5, -1.}, {0.5, 1.}}]}, 
     {RGBColor[1, 1, 1], AbsolutePointSize[4], 
      PointBox[{{0.5, -1.}, {0.5, 1.}}]}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->True,
   AxesOrigin->{0, 0},
   PlotRange->{{-2, 4}, {-1., 1.}},
   PlotRangeClipping->True,
   PlotRangePadding->{
     Scaled[0.02], 
     Scaled[0.02]}], TraditionalForm]], "Output",
 ImageSize->{184, 118},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[55]=",
 CellID->2100017634]
}, Open  ]],

Cell[TextData[{
 "To properly represent functions with removable singularities the function \
should be defined using ",
 ButtonBox["Piecewise",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Piecewise"],
 "."
}], "Text",
 CellID->2075938875],

Cell["Visualizing a removable singularity.", "MathCaption",
 CellID->137859316],

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", "x_", "]"}], ":=", " ", 
  RowBox[{"\[Piecewise]", GridBox[{
     {
      FractionBox[
       RowBox[{
        SuperscriptBox["x", "2"], "-", 
        RowBox[{"2", " ", "x"}], " ", "-", "3"}], 
       RowBox[{"x", "+", "1"}]], 
      RowBox[{"x", "\[NotEqual]", 
       RowBox[{"-", "1"}]}]},
     {"Indeterminate", 
      RowBox[{"x", "\[Equal]", 
       RowBox[{"-", "1"}]}]}
    }]}]}]], "Input",
 CellLabel->"In[72]:=",
 CellID->2014152546],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PiecewisePlot", "[", 
  RowBox[{
   RowBox[{"f", "[", "x", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "3"}], ",", "2"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[73]:=",
 CellID->1077112961],

Cell[BoxData[
 FormBox[
  GraphicsBox[{{{}, {}, 
     {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwB4QQe+yFib1JlAgAAAE0AAAACAAAAfedN8v//B8C/8yb5//8XwOZ7FvXb
/AfA8z2L+m3+F8BOEN/3t/kHwCeI7/vb/BfAIDlw/W/zB8CQHLj+t/kXwMKK
kgjg5gfAYUVJBHDzF8AILtcewM0HwASXaw/g5hfAlHRgS4CbB8BKOrAlwM0X
wKsBc6QANwfA1YA5UoCbF8AFZUcyGV0GwIOyI5mMLhfAE9jFbKKRBcAK7GI2
0cgWwFLDmmMpygTAqWHNsRRlFsC99x42yPEDwN97Dxvk+BXA3DtNtdcnA8Du
naba65MVwCfJKhD/TALAlGQViH8mFcCjzl4nJHYBwFJnrxMSuxTA0+M867mt
AMDpcZ713FYUwF6ElBXPqP+/FyFlxTPqE8B+YAOuCxP+vx/YgOvChBPAAC0f
v0OF/L9Ay8fvUCETwNqLmYer1fq/92Lm4Wq1EsAcCmip9EL5v4cCWiq9UBLA
thqVgm2O97+tRqVgm+MRwLIbb9Th4fW/7cYbdXh4EcAWPJ1/N1L0vwZP59+N
FBHA0u4p4ryg8r+0e4o4L6gQwPbACp4jDPG/PLCC5whDEMDiSpQidKvuv7US
pQjdqg/AnfRs+pdO678oPZv+pdMOwCfd7YR+K+i/S3c7od8KDsBh6it+xMTk
v5j6ih8xMQ3AajYSKs2X4b+ZjYRK82UMwHDGpI6Z9dy/zZjUMbOeC8BsaZ+m
VzTWvy3t0/SKxgrADRTVRzbNz79AUX1k0/wJwALoX/06I8K/gf7VrzMiCcAq
/EjhbeSiv/EjhbeRSwjAwLppCOUnrz8VWd5rYIMHwKuJkkGQW8U/Zdfm+0aq
BsC11DQ7DgPRP2plmTie3wXACqPscmY41z+fa6Ix8xgFwP8nKs3/9N0/Abta
BmBBBMCqlwvhCR/iPxQavYc9eAPApvbEbDSH5T9Xws7kMp4CwNIW1kWcteg/
S3qK7pjSAcA5Vo0tDdTrP3KqnLT8CgHA8XCHph427z/EI15WeDIAwG2mbLY2
L/E/k1mTScnQ/r8KAjdiLuXyP/b9yJ3RGv2/Q21UlSqT9D+9kqtq1Wz7vxa5
HW9FJPY/60bikLrb+b+QcoiRMNf3P3CNd27PKPi/ogyfWjpt+T9f82ClxZL2
v1K2CKtI+/o/rkn3VLcE9b+qzRNEJ6v8P1Yy7LvYVPO/msXKgyQ+/j9mOjV8
28Hxv8AJTvErRf4/QPaxDtS68b/lTdFeM0z+PxuyLqHMs/G/MNbXOUJa/j/Q
KSjGvaXxv8bm5O9fdv4/ORkbEKCJ8b/yB/9bm67+Pw74AKRkUfG/SkozNBIf
/z+2tczL7eDwv3COtqEZJv8/kHFJXubZ8L+V0jkPIS3/P2stxvDe0vC/4FpA
6i87/z8gpb8V0MTwv3ZrTaBNV/8/ipSyX7Ko8L+ijGcMiY//P19zmPN2cPC/
yNDqeZCW/z84LxWGb2nwv+0UbueXnf8/E+uRGGhi8L84nXTCpqv/P8hiiz1Z
VPC/zq2BeMTH/z8yUn6HOzjwv/TxBObLzv8/DA77GTQx8L8ZNohT09X/P+fJ
d6wsKvC/ZL6OLuLj/z+cQXHRHRzwv4oCEpzp6v8/dv3tYxYV8L+vRpUJ8fH/
P1G5avYODvC/1IoYd/j4/z8sdeeIBwfwv/rOm+T///8/BjFkGwAA8L9g0pEa

       "]]}}, {
     {RGBColor[0.24720000000000014`, 0.24, 0.6], AbsolutePointSize[6], 
      PointBox[{{-1., -4.}}]}, 
     {RGBColor[1, 1, 1], AbsolutePointSize[4], PointBox[{{-1., -4.}}]}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->True,
   AxesOrigin->{0, -6.},
   PlotRange->{{-3, 2}, {-5.999999897959184, -1.0000001020408162`}},
   PlotRangeClipping->True,
   PlotRangePadding->{
     Scaled[0.02], 
     Scaled[0.02]}], TraditionalForm]], "Output",
 ImageSize->{184, 127},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[73]=",
 CellID->1897874964]
}, Open  ]],

Cell["\<\
Functions can also be plotted with polar coordinates using PiecewisePolarPlot.\
\>", "Text",
 CellID->1847896481],

Cell["A function using polar coordinates.", "MathCaption",
 CellID->687653253],

Cell[BoxData[
 RowBox[{
  RowBox[{"p", "[", "x_", "]"}], ":=", 
  RowBox[{"\[Piecewise]", GridBox[{
     {
      RowBox[{"2", " ", 
       RowBox[{"Sin", "[", 
        RowBox[{"5", " ", "x"}], "]"}]}], 
      RowBox[{"0", "\[LessEqual]", "x", "<", 
       RowBox[{"\[Pi]", "/", "3"}]}]},
     {
      RowBox[{"Sin", "[", 
       RowBox[{"7", " ", "x"}], "]"}], 
      RowBox[{
       RowBox[{"\[Pi]", "/", "3"}], "\[LessEqual]", "x", "<", 
       RowBox[{"2", " ", 
        RowBox[{"\[Pi]", "/", "3"}]}]}]},
     {
      RowBox[{"1", "-", 
       RowBox[{"2", 
        RowBox[{"Cos", "[", " ", "x", "]"}]}]}], 
      RowBox[{
       RowBox[{"2", " ", 
        RowBox[{"\[Pi]", "/", "3"}]}], "\[LessEqual]", " ", "x", "<", 
       RowBox[{"2", " ", "\[Pi]"}]}]}
    }]}]}]], "Input",
 CellLabel->"In[26]:=",
 CellID->1778718664],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PiecewisePolarPlot", "[", 
  RowBox[{
   RowBox[{"p", "[", "\[Theta]", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Theta]", ",", "0", ",", 
     RowBox[{"2", " ", "\[Pi]"}]}], "}"}], ",", 
   RowBox[{"PolarGridLines", "\[Rule]", " ", "True"}], ",", 
   RowBox[{"Axes", "\[Rule]", " ", "None"}]}], "]"}]], "Input",
 CellLabel->"In[50]:=",
 CellID->1952219608],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztXS13MkuzPWvdX3J/xrGRkUjki0RcgYxFIpFYJO4isUgkMjYyMpJ3T2+o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    "], {{0, 180}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 180},
  PlotRange->{{0, 180}, {0, 180}}]], "Output",
 ImageSize->{184, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[50]=",
 CellID->40693698]
}, Open  ]],

Cell["\<\
The functions can be a mix of piecewise continuous and continuous functions.\
\>", "MathCaption",
 CellID->2103919694],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PiecewisePolarPlot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"2", "-", " ", 
      RowBox[{"4", " ", 
       RowBox[{"Cos", "[", "\[Theta]", "]"}]}]}], ",", 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{
       RowBox[{"SawtoothWave", "[", 
        RowBox[{"3", " ", 
         RowBox[{"\[Theta]", "/", "\[Pi]"}]}], "]"}], "+", "1"}]]}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\[Theta]", ",", "0", ",", " ", 
     RowBox[{"2", " ", "\[Pi]"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[29]:=",
 CellID->112565556],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztXSu3q0gWvmvNHxjZtuXII4+NjERGNvKIEchYJBKJReKmJBaJLIlFIktm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    "], {{0, 156}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 156},
  PlotRange->{{0, 180}, {0, 156}}]], "Output",
 ImageSize->{184, 156},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[29]=",
 CellID->73773030]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["Classroom Utilities",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/guide/ClassroomUtilities"]], "TutorialMoreAbout",
 CellID->1785505193]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED WOLFRAM EDUCATION GROUP COURSES", "TutorialRelatedLinksSection",
 WholeCellGroupOpener->True,
 CellID->415694148],

Cell[TextData[{
 ButtonBox["M50",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[":",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["An",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["Introduction",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["to",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["Mathematica",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["in",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["the",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["Classroom",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"]
}], "TutorialRelatedLinks",
 CellID->415694149]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Plotting Piecewise Continuous Functions - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "TutorialColor", 
  "LinkTrails" -> GridBox[{{
      RowBox[{
        ButtonBox[
        "Classroom Utilities", ButtonData -> 
         "paclet:ClassroomUtilities/guide/ClassroomUtilities", 
         BaseStyle -> {"Link", "DockedLinkTrail"}], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 32, 48.8512624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Tutorial", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "The functions defined in the PiecewisePlots` package provide a means of \
plotting piecewise continuous functions together with continuous functions in \
the manner usually found in standard text books. Point of discontinuity are \
represented by filled and open circles with filled circles used for values \
the function takes and open circles indicating values the function does not \
take. Piecewise plotting functions. This loads the package.", 
    "synonyms" -> {}, "title" -> "Plotting Piecewise Continuous Functions", 
    "titlemodifier" -> "", "windowtitle" -> 
    "Plotting Piecewise Continuous Functions", "type" -> "Tutorial", "uri" -> 
    "ClassroomUtilities/tutorial/PlottingPiecewiseContinuousFunctions"}, 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> Notebook], 
   Cell[
    StyleData["Output"], CellContext -> Notebook], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "RasterizedOutput"->{
  Cell[12703, 367, 2800, 54, 146, "Output",
   CellTags->"RasterizedOutput",
   CellID->72957386],
  Cell[22623, 608, 4305, 79, 128, "Output",
   CellTags->"RasterizedOutput",
   CellID->36364065],
  Cell[27528, 712, 4589, 83, 128, "Output",
   CellTags->"RasterizedOutput",
   CellID->13311170],
  Cell[33849, 863, 3312, 62, 127, "Output",
   CellTags->"RasterizedOutput",
   CellID->76535670],
  Cell[46650, 1220, 11263, 193, 201, "Output",
   CellTags->"RasterizedOutput",
   CellID->40693698],
  Cell[58658, 1443, 6789, 119, 177, "Output",
   CellTags->"RasterizedOutput",
   CellID->73773030]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"RasterizedOutput", 72130, 1731}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[632, 21, 2992, 70, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3649, 95, 76, 1, 51, "Title",
 CellID->509267359],
Cell[3728, 98, 437, 8, 110, "Text",
 CellID->1534169418],
Cell[4168, 108, 3605, 117, 169, "DefinitionBox",
 CellID->2096742444],
Cell[7776, 227, 69, 1, 31, "Caption",
 CellID->1891092685],
Cell[7848, 230, 67, 1, 31, "MathCaption",
 CellID->1009911057],
Cell[7918, 233, 128, 3, 28, "Input",
 CellID->1553316875],
Cell[CellGroupData[{
Cell[8071, 240, 71, 1, 51, "Section",
 CellID->1800518439],
Cell[8145, 243, 647, 25, 52, "Text",
 CellID->1121334396],
Cell[8795, 270, 67, 1, 31, "MathCaption",
 CellID->836781195],
Cell[CellGroupData[{
Cell[8887, 275, 426, 14, 70, "Input",
 CellID->2058623809],
Cell[9316, 291, 3040, 59, 148, "Output",
 CellID->2098191147]
}, Open  ]],
Cell[12371, 353, 76, 1, 70, "MathCaption",
 CellID->469640242],
Cell[CellGroupData[{
Cell[12472, 358, 228, 7, 28, "Input",
 CellID->1241144569],
Cell[12703, 367, 2800, 54, 146, "Output",
 CellTags->"RasterizedOutput",
 CellID->72957386]
}, Open  ]],
Cell[15518, 424, 457, 14, 70, "Text",
 CellID->799214764],
Cell[15978, 440, 84, 1, 70, "MathCaption",
 CellID->2039810210],
Cell[CellGroupData[{
Cell[16087, 445, 255, 8, 70, "Input",
 CellID->661258762],
Cell[16345, 455, 2291, 48, 128, "Output",
 CellID->548139709]
}, Open  ]],
Cell[18651, 506, 129, 3, 70, "MathCaption",
 CellID->2026727180],
Cell[CellGroupData[{
Cell[18805, 513, 322, 9, 70, "Input",
 CellID->1965589193],
Cell[19130, 524, 2863, 59, 128, "Output",
 CellID->1815334719]
}, Open  ]],
Cell[22008, 586, 88, 1, 70, "MathCaption",
 CellID->1030449304],
Cell[CellGroupData[{
Cell[22121, 591, 499, 15, 47, "Input",
 CellID->1453322920],
Cell[22623, 608, 4305, 79, 128, "Output",
 CellTags->"RasterizedOutput",
 CellID->36364065]
}, Open  ]],
Cell[26943, 690, 125, 3, 70, "MathCaption",
 CellID->2116071404],
Cell[CellGroupData[{
Cell[27093, 697, 432, 13, 28, "Input",
 CellID->335409061],
Cell[27528, 712, 4589, 83, 128, "Output",
 CellTags->"RasterizedOutput",
 CellID->13311170]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[32166, 801, 62, 1, 70, "Section",
 CellID->1297162916],
Cell[32231, 804, 444, 11, 70, "Text",
 CellID->1346377783],
Cell[32678, 817, 170, 6, 70, "MathCaption",
 CellID->536210442],
Cell[32851, 825, 325, 11, 70, "Input",
 CellID->1247515882],
Cell[CellGroupData[{
Cell[33201, 840, 645, 21, 28, "Input",
 CellID->1116583113],
Cell[33849, 863, 3312, 62, 127, "Output",
 CellTags->"RasterizedOutput",
 CellID->76535670]
}, Open  ]],
Cell[37176, 928, 103, 2, 70, "MathCaption",
 CellID->868317803],
Cell[CellGroupData[{
Cell[37304, 934, 261, 8, 70, "Input",
 CellID->1329453543],
Cell[37568, 944, 937, 29, 52, "Output",
 CellID->1514886135]
}, Open  ]],
Cell[CellGroupData[{
Cell[38542, 978, 119, 4, 70, "Input",
 CellID->1045072778],
Cell[38664, 984, 993, 31, 69, "Output",
 CellID->269431033]
}, Open  ]],
Cell[CellGroupData[{
Cell[39694, 1020, 316, 11, 70, "Input",
 CellID->1927215496],
Cell[40013, 1033, 1645, 37, 139, "Output",
 CellID->2100017634]
}, Open  ]],
Cell[41673, 1073, 242, 8, 70, "Text",
 CellID->2075938875],
Cell[41918, 1083, 79, 1, 70, "MathCaption",
 CellID->137859316],
Cell[42000, 1086, 488, 17, 70, "Input",
 CellID->2014152546],
Cell[CellGroupData[{
Cell[42513, 1107, 243, 8, 70, "Input",
 CellID->1077112961],
Cell[42759, 1117, 2431, 49, 148, "Output",
 CellID->1897874964]
}, Open  ]],
Cell[45205, 1169, 123, 3, 70, "Text",
 CellID->1847896481],
Cell[45331, 1174, 78, 1, 70, "MathCaption",
 CellID->687653253],
Cell[45412, 1177, 827, 27, 70, "Input",
 CellID->1778718664],
Cell[CellGroupData[{
Cell[46264, 1208, 383, 10, 28, "Input",
 CellID->1952219608],
Cell[46650, 1220, 11263, 193, 201, "Output",
 CellTags->"RasterizedOutput",
 CellID->40693698]
}, Open  ]],
Cell[57928, 1416, 128, 3, 70, "MathCaption",
 CellID->2103919694],
Cell[CellGroupData[{
Cell[58081, 1423, 574, 18, 30, "Input",
 CellID->112565556],
Cell[58658, 1443, 6789, 119, 177, "Output",
 CellTags->"RasterizedOutput",
 CellID->73773030]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[65496, 1568, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[65564, 1571, 180, 4, 70, "TutorialMoreAbout",
 CellID->1785505193]
}, Open  ]],
Cell[CellGroupData[{
Cell[65781, 1580, 127, 2, 70, "TutorialRelatedLinksSection",
 CellID->415694148],
Cell[65911, 1584, 3506, 82, 70, "TutorialRelatedLinks",
 CellID->415694149]
}, Open  ]]
}, Open  ]],
Cell[69444, 1670, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

