(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     64422,       1930]
NotebookOptionsPosition[     55794,       1643]
NotebookOutlinePosition[     58264,       1704]
CellTagsIndexPosition[     58221,       1701]
WindowTitle->Problems In Analytic Geometry - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Functions \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Distance\"\>":>Documentation`HelpLookup[
       "paclet:AnalyticGeometry/ref/Distance"], "\<\"MakeCircle\"\>":>
       Documentation`HelpLookup[
       "paclet:AnalyticGeometry/ref/MakeCircle"], "\<\"PerpendicularLine\"\>":>
       Documentation`HelpLookup[
       "paclet:AnalyticGeometry/ref/PerpendicularLine"], \
"\<\"PointSlopeLine\"\>":>Documentation`HelpLookup[
       "paclet:AnalyticGeometry/ref/PointSlopeLine"], "\<\"Slope\"\>":>
       Documentation`HelpLookup[
       "paclet:AnalyticGeometry/ref/Slope"], "\<\"TwoPointLine\"\>":>
       Documentation`HelpLookup["paclet:AnalyticGeometry/ref/TwoPointLine"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"TutorialFunction"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"ClassroomUtilities/tutorial/ProblemsInAnalyticGeometry\
\"\>":>None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["ClassroomUtilities/tutorial/ProblemsInAnalyticGeometry"]}, 
            Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                
                "http://reference.wolfram.com/mathematica/ClassroomUtilities/\
tutorial/ProblemsInAnalyticGeometry.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "ClassroomUtilities/tutorial/ProblemsInAnalyticGeometry", 
             ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell["Problems In Analytic Geometry", "Title",
 CellID->509267359],

Cell["\<\
Some of the functions in the ClassroomUtilities` package are intended to \
facilitate the solutions of problems found in a typical high school algebra \
or geometry class.\
\>", "Text",
 CellID->1534169418],

Cell[BoxData[GridBox[{
   {
    RowBox[{
     ButtonBox["Distance",
      BaseStyle->"Link",
      ButtonData->"paclet:AnalyticGeometry/ref/Distance"], "[", 
     RowBox[{
      SubscriptBox[
       StyleBox["pt", "TI"], 
       StyleBox["1", "TR"]], ",", 
      SubscriptBox[
       StyleBox["pt", "TI"], 
       StyleBox["2", "TR"]]}], "]"}], Cell[
    "find the distance between two points", "TableText"]},
   {
    RowBox[{
     ButtonBox["Slope",
      BaseStyle->"Link",
      ButtonData->"paclet:AnalyticGeometry/ref/Slope"], "[", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox[
        StyleBox["pt", "TI"], 
        StyleBox["1", "TR"]], ",", 
       SubscriptBox[
        StyleBox["pt", "TI"], 
        StyleBox["2", "TR"]]}], "}"}], "]"}], 
    RowBox[{
    "find", " ", "the", " ", "slope", " ", "of", " ", "a", " ", "line", " ", 
     "through", " ", "two", " ", "given", " ", "points"}]},
   {
    RowBox[{
     ButtonBox["TwoPointLine",
      BaseStyle->"Link",
      ButtonData->"paclet:AnalyticGeometry/ref/TwoPointLine"], "[", 
     RowBox[{
      SubscriptBox[
       StyleBox["pt", "TI"], 
       StyleBox["1", "TR"]], ",", 
      SubscriptBox[
       StyleBox["pt", "TI"], 
       StyleBox["2", "TR"]], ",", 
      StyleBox["var", "TI"]}], "]"}], Cell[
    "construct the expression for a line through two given points", 
     "TableText"]},
   {
    RowBox[{
     ButtonBox["PointSlopeLine",
      BaseStyle->"Link",
      ButtonData->"paclet:AnalyticGeometry/ref/PointSlopeLine"], "[", 
     RowBox[{
      SubscriptBox[
       StyleBox["pt", "TI"], 
       StyleBox["1", "TR"]], ",", 
      StyleBox["slp", "TI"], ",", 
      StyleBox["var", "TI"]}], "]"}], Cell["\<\
construct the expression for a line given the slope and a point\
\>", "TableText"]},
   {
    RowBox[{
     ButtonBox["PerpendicularLine",
      BaseStyle->"Link",
      ButtonData->"paclet:AnalyticGeometry/ref/PerpendicularLine"], "[", 
     RowBox[{
      SubscriptBox[
       StyleBox["pt", "TI"], 
       StyleBox["1", "TR"]], ",", 
      StyleBox["eq", "TI"], ",", 
      StyleBox["var", "TI"]}], "]"}], 
    RowBox[{
    "construct", " ", "a", " ", "line", " ", "perpendicular", " ", "to", " ", 
     "a", " ", "given", " ", "line"}]},
   {
    RowBox[{
     ButtonBox["MakeCircle",
      BaseStyle->"Link",
      ButtonData->"paclet:AnalyticGeometry/ref/MakeCircle"], "[", 
     RowBox[{
      StyleBox["cnt", "TI"], ",", 
      StyleBox["rad", "TI"], ",", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox[
         StyleBox["var", "TI"], 
         StyleBox["1", "TR"]], ",", 
        SubscriptBox[
         StyleBox["var", "TI"], 
         StyleBox["2", "TR"]]}], "}"}]}], "]"}], 
    RowBox[{
    "Make", " ", "an", " ", "expression", " ", "for", " ", "a", " ", 
     "circle"}]}
  }]], "DefinitionBox",
 CellID->2096742444],

Cell["Analytic Geometry functions", "Caption",
 CellID->1891092685],

Cell[CellGroupData[{

Cell["Some Basic Examples", "Section",
 CellID->865725525],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellID->442055860],

Cell["Create the expression for a line through two given points.", \
"MathCaption",
 CellID->836781195],

Cell[BoxData[
 RowBox[{"Clear", "[", "ln", "]"}]], "Input",
 CellLabel->"In[74]:=",
 CellID->2086258710],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ln", "[", "t_", "]"}], "=", 
  RowBox[{"TwoPointLine", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"4", ",", "1"}], "}"}], ",", "t"}], "]"}]}]], "Input",
 CellLabel->"In[79]:=",
 CellID->2058623809],

Cell[BoxData[
 RowBox[{
  FractionBox["7", "3"], "-", 
  FractionBox["t", "3"]}]], "Output",
 ImageSize->{40, 32},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[79]=",
 CellID->1526859018]
}, Open  ]],

Cell["Create the expression for a circle.", "MathCaption",
 CellID->446279958],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"circ", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], "=", " ", 
  RowBox[{"MakeCircle", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1", "/", "2"}], ",", "1"}], "}"}], ",", "2", ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}]], "Input",
 CellLabel->"In[72]:=",
 CellID->1614358782],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "4"}], "+", 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     FractionBox["1", "2"], "-", "x"}], ")"}], "2"], "+", 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"1", "-", "y"}], ")"}], "2"]}]], "Output",
 ImageSize->{143, 32},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[72]=",
 CellID->1219652876]
}, Open  ]],

Cell["Find the points of intersection of the line and the circle.", \
"MathCaption",
 CellID->974476317],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sol", "=", " ", 
  RowBox[{"Solve", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"circ", "[", 
        RowBox[{"x", ",", "y"}], "]"}], "\[Equal]", "0"}], ",", 
      RowBox[{"y", "\[Equal]", 
       RowBox[{"ln", "[", "x", "]"}]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}]], "Input",
 CellLabel->"In[80]:=",
 CellID->1313167797],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"x", "\[Rule]", 
      RowBox[{
       FractionBox["1", "20"], " ", 
       RowBox[{"(", 
        RowBox[{"17", "-", 
         RowBox[{"3", " ", 
          SqrtBox["111"]}]}], ")"}]}]}], ",", 
     RowBox[{"y", "\[Rule]", 
      RowBox[{
       FractionBox["1", "20"], " ", 
       RowBox[{"(", 
        RowBox[{"41", "+", 
         SqrtBox["111"]}], ")"}]}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"x", "\[Rule]", 
      RowBox[{
       FractionBox["1", "20"], " ", 
       RowBox[{"(", 
        RowBox[{"17", "+", 
         RowBox[{"3", " ", 
          SqrtBox["111"]}]}], ")"}]}]}], ",", 
     RowBox[{"y", "\[Rule]", 
      RowBox[{
       FractionBox["1", "20"], " ", 
       RowBox[{"(", 
        RowBox[{"41", "-", 
         SqrtBox["111"]}], ")"}]}]}]}], "}"}]}], "}"}]], "Output",
 ImageSize->{302, 63},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[80]=",
 CellID->1637417523]
}, Open  ]],

Cell["Visualize the solution.", "MathCaption",
 CellID->1026957464],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ContourPlot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"y", "\[Equal]", 
      RowBox[{"ln", "[", "x", "]"}]}], ",", 
     RowBox[{
      RowBox[{"circ", "[", 
       RowBox[{"x", ",", "y"}], "]"}], "\[Equal]", "0"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "2"}], ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", "2"}], ",", "3"}], "}"}], ",", 
   RowBox[{"Epilog", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"Red", ",", 
      RowBox[{"Point", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"x", ",", "y"}], "}"}], "/.", "sol"}], "]"}]}], "}"}]}], ",", 
   RowBox[{"Axes", "\[Rule]", " ", "True"}]}], "]"}]], "Input",
 CellLabel->"In[83]:=",
 CellID->748026509],

Cell[BoxData[
 GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyFWAlUTVsfv/ee6vTMSSgqHomERMayt5CZokIyvJSXuSRRRA8lxTMVknko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   "], {{}, {}, 
    TagBox[
     TooltipBox[
      {Hue[0.67, 0.6, 0.6], 
       LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18,
         19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 
        36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 
        53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 
        70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 
        87, 88}]},
      RowBox[{"y", "\[Equal]", 
        RowBox[{
          FractionBox["7", "3"], "-", 
          FractionBox["x", "3"]}]}]],
     Annotation[#, $CellContext`y == 
      Rational[7, 3] + Rational[-1, 3] $CellContext`x, "Tooltip"]& ], 
    TagBox[
     TooltipBox[
      {Hue[0.9060679774997897, 0.6, 0.6], LineBox[CompressedData["
1:eJwl1GV7EAQUBeCNLukQkC5plA6lu5GSZjRr2OgYHUoqnQqIdEopoSghjdJK
d3cICC8PH95z/8A5N1tQeKOwwICAgPziS7cFLWlFa9rQlna0J4gOdKQTnelC
V7rRnWBCCCWMcCKIpAc9iSKaXvSmD33pR38GMJBBDCaGIQxlGMMZwUhGMZox
jOUrvmYc45nARCYxmW/4lilMZRrTmcFMZjGbOcxlHvP5ju9ZwEIW8QOL+ZEl
LGUZy1nBSlaxmjWsZR3r+YkNbGQTm9nCz/zCVraxnR38ym/s5Hf+YBe72cNe
/mQf+znAQQ5xmCMc5S/+5hjHOcFJTnGaM/zDv5zlHOe5wEUucZkrXOUa17nB
TW5xmzvc5R73ecBDHvGYJzzlGc95wX+85BWv+Z83BOhnILGITRziEo/4JCAh
iUhMEj4gKclITgpSkorUpCEt6fiQ9GQgIx+RicxkISvZyE4OcpKL3OThY/KS
L/D9lgpQkEIUpgif8ClFKUZxSlCSUpSmDGUpx2d8TnkqUJFKVKYKValGdWpQ
k1rUpg51qUd9GtCQRnxBY5rQlGY0f7d/WtCSVrSmDW1pR3uC6EBHOtGZLnSl
G90JJoRQ3j2XcCKIpAc9iSKaXvSmD33pR38GMJBBDCaGIQxlGG8B2SmUXg==

        "]]},
      RowBox[{
        RowBox[{
          RowBox[{"-", "4"}], "+", 
          SuperscriptBox[
           RowBox[{"(", 
             RowBox[{
               FractionBox["1", "2"], "-", "x"}], ")"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{"(", 
             RowBox[{"1", "-", "y"}], ")"}], "2"]}], "\[Equal]", "0"}]],
     Annotation[#, -4 + (Rational[1, 2] - $CellContext`x)^2 + (
         1 - $CellContext`y)^2 == 0, "Tooltip"]& ]}],
  AspectRatio->1,
  Axes->True,
  Epilog->{
    RGBColor[1, 0, 0], 
    PointBox[
     NCache[{{
       Rational[1, 20] (17 - 3 111^Rational[1, 2]), 
        Rational[1, 20] (41 + 111^Rational[1, 2])}, {
       Rational[1, 20] (17 + 3 111^Rational[1, 2]), 
        Rational[1, 20] (41 - 111^Rational[1, 2])}}, {{-0.7303480629279108, 
      2.576782687642637}, {2.430348062927911, 1.5232173123573631`}}]]},
  Frame->True,
  PlotRange->{{-2, 3}, {-2, 3}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 ImageSize->{184, 183},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[83]=",
 CellID->73461614]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solving Problems", "Section",
 CellID->1047190250],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ClassroomUtilities`\>\"", "]"}]], "Input",
 CellID->1486904047],

Cell[CellGroupData[{

Cell["Problem 1", "Subsection",
 CellID->1798068103],

Cell["\<\
Given a pair of parallel lines and a point between them find a circle that is \
tangent to both lines and passes through the point.\
\>", "Text",
 CellID->1498044585],

Cell["The given lines and point.", "MathCaption",
 CellID->537062667],

Cell[BoxData[
 RowBox[{
  RowBox[{"ln1", "[", "x_", "]"}], ":=", " ", 
  RowBox[{"x", "/", "2"}]}]], "Input",
 CellLabel->"In[45]:=",
 CellID->538475928],

Cell[BoxData[
 RowBox[{
  RowBox[{"ln2", "[", "x_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"x", "/", "2"}], " ", "+", " ", "3"}]}]], "Input",
 CellLabel->"In[46]:=",
 CellID->668833438],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"pt", " ", "=", " ", 
  RowBox[{"{", 
   RowBox[{"3", ",", " ", 
    RowBox[{"7", "/", "2"}]}], "}"}]}]], "Input",
 CellLabel->"In[47]:=",
 CellID->1154184068],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", 
   FractionBox["7", "2"]}], "}"}]], "Output",
 ImageSize->{48, 32},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[47]=",
 CellID->151847525]
}, Open  ]],

Cell["Visualizing the given elements.", "MathCaption",
 CellID->890780039],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ln1", "[", "x", "]"}], ",", 
     RowBox[{"ln2", "[", "x", "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "6"}], "}"}], ",", 
   RowBox[{"Epilog", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"Red", ",", 
      RowBox[{"Point", "[", "pt", "]"}]}], "}"}]}], ",", 
   RowBox[{"AspectRatio", "\[Rule]", " ", "Automatic"}]}], "]"}]], "Input",
 CellLabel->"In[48]:=",
 CellID->1231612236],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwtxX1MzWscAPBfnepJpOJ0Hhc1L/WTzDJu9+qavl8vY16Sl6kUh151u91T
t3ktJXG85JyzXtgkSn80Ws0mk3mLUB2LzDTNqG4RKm4vJ6XOPR3s+f7x2Wdm
VNLmWFtJkoJ++Pm3dZqLG5MyA8XJgcdj5vg8l9Ug3gDP/nufoJc1tBr6rq/Q
JcoZtAayVKbp22UDnQEN9Ws7t8hFtAGSLu17EixfpYugRqe7EyRX01fB3iGF
dSU8p+/DmsOR2hshzXQ93DwY21L6opVuhNfhf/ml7H9HvwS31fq4GvxIN4Nr
U2zl7xE99BuYPSOxTBvVS7fB4pa/G5bl9dPtUDOaub6p0ES/g/j7RmN11le6
EwqiL0913zVMf4T5Ye1DY7EjdBe0moIbcY2Z7oGBtl8WqLZa6C+QU566OzTU
SveC7OIYdKBQQnEfVCVnlhuKbVDcD03e+WxTni2KB8Dr6EufHQUKFJsgYGPb
r8WFdigehJZkbaBeZ4/ir1AZsM25NtcBxUOgCql/fUXLUDwM0CHlPtzjiOJv
UKCKd5YPjEPxCGQV/XnCP8EJxaNwsfFcnSF5PIrNkK7J7/NImYDi/6En0mfK
tihnFFvgtKXbrjVuIorH4Pbn1b0Tw11QbIVX3os7/vByRbGE8XOj/fJLXWkJ
a0Ii0pZ5utE2uCisu0xZ4kbbYOLRVc1Lp02ibfFknOOoTf4k2hbveURau5WT
aQXeGjuu8DVMphXYeH1Ft95NSdth7Yy4Ol2ekrbDYze3lrQ7udP2uH3WW3WE
3p22R3OV37R5TEU74CFF19OzGSraAVVeAZHeEqcZ7teEDDod5DTD6b/FGOU0
TjN8YPnnwvJ0TjN00utXph7hNMOi8sdnuk5xmmHdp4X+xvOcZqiMdtmnvctp
hrd8PdaWVHOaoXrA1/PeA04zvHJkVd3gY04zXHIpXRXzjNMMo1p6qpa3cpoh
Kx3JVv/LaYYViWxnagenGQ6ZZ7HKD5xmmD01PHRmL6cZ+nXsnre0n9MMm8r2
WsNMnGboGZB7OWeY0wwfScVpFSOcZhhvrAg2mjnNcELO7dnvLZxmeC3UOGy1
cprhd3bOJbc=
     "]]}, 
   {Hue[0.9060679774997897, 0.6, 0.6], LineBox[CompressedData["
1:eJwVjns01HkYxkeDGZcZc/9a4Sia2hQdKnTxe1QUW7G1xjUb47ZY4SQhudcm
I5dsK0XOpki2aNOmpXTRqC1ppSSSrIqoSLcJ6/fHe55z3vM8n+eZFbBtU9AM
BoOxYfpo/fRd5FG3bSn2rhU2bAaDjT2Bc+e1Sv2oHWmjEX1qbNwZ6Q9TSCOp
Et/qtutMNt6eW50dId1NDfNMi/ex2EiTjBn6SnOo/fFcC6EeG7dvuvy3WVpC
KWkxZmPbsR0trtI/KIfh/NL1K9hoys6+tEHaSFlbjSwt2smGhmYM61VYK7Uo
d/k6DxUbzsn+medlD6lxIvMMj9XChfig7vK2HqrYo2OLwbAWOr3DLWPinlM+
4cOOI5Ha4K9VBDfhBdVX0bZA/6U2eO1BtTY+Q1Ra/foPweE6MDWJqMwMeEOp
Zyw87/ZaB7bdP992yH9HbYzriskN1EXTl5T17cVj1Fl7E+H4gC5CLyuVjWnj
1GNLq9ozWzkokp80EG/9SA08qnTSeMTBQs9nHyaDPlM6sr8akr246BlzvQtn
FfXpa4G1230uRp9+s0jiPkF9jdPMdnTXQ25VQoiHxxTVf6qn88EDPUj12Bt2
FjNwi7lK2CPloS4qpSqnVA0Whr2nnofw0D6ngPV9/gzEtvQ7HDzOg1n6v/O2
FDHR0rvYxWOABzu3p4tLi9XBbbe580LKR3dUpr0iWwM2vmWupnI+au28ODfy
NBGxUbY6+jgfEtnNzopMFvY8iF4a9owPqo+Rd3U7G7m2OL7ISIAiSShHulML
JzS807X8BEgr+WnvkjBtHAt57K84LMDRu78150Tp4KT1iRrTJwIkRRa8NYrR
RWmX2a69hkIM+c/T9wrgIFFf2+lXTyH2Twyq9wRzIb+8LPVCkRD1r9e+4Xrr
wULVdim9Q4iOObZ9y8x4GBIYRhsJRAj9Vm5ZUM7DnwdrDrVsEqFJ5pPoYMzH
1qrqBpMDIlh7DlaKyvgwWDD+Q9Y9ESLSnR6unClAncPvQyUcMX4JZn9RKxBg
jbX6RIqzGA1G/lODIiF6cs4mx2aJcXFyD3N+jhC+ydfUxDfFuHtu9aCCL8KL
zlnjzWwJbpgEN2fnixDAs5Y7OUuQccG97Jm2GFe7Tt2LypTAd/YTPx+FGBb8
yDPVzRKo6ixnmrMk2N11SFHIJNjFfPVP4W4JGlO3D7hQBBIzO/85DAKd/CNn
BSkEcZGy99rxBCtszBMy/yYwXBqolCYScGdaBRY0EFyZiD6yKomgd9J2Y1kj
gbZCsSYhlSCz2Wl2wxWCkqrrB1/tI2h1D7j1/jpB80urJcrDBPLtRQaBdwhE
cr0dNDerhnVpVQ/BxflGLjTHt5Bb7vaUwG90vjHNsYwXH/DrJahIdWqmOe0w
lSf0ESw/liShOcb37HVqBwgCuofqaE7NSKzPrDcErPLPWXQu4/4uR8u3BKcj
WD/SOVlduuXKdwQfVLNZdE6VlMf0GpveYeDtQeccOdVVuR+ne/tCzGmf/rtz
hSWfpnsrY6c8p32D7fXJpz9P99rlnaR9B44oNytVBNcYpYn0PyCldWXHV4JQ
5WlX+r84sGNu/wSBbm69Ka2a67r5o5PTOz2UH6emCB6Z96to/R+ccA95
     "]]}},
  Axes->True,
  AxesOrigin->{0, 0},
  Epilog->{
    RGBColor[1, 0, 0], 
    PointBox[
     NCache[{3, 
       Rational[7, 2]}, {3, 3.5}]]},
  PlotRange->{{0, 6}, {0., 5.99999993877551}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 ImageSize->{184, 189},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[48]=",
 CellID->111977019]
}, Open  ]],

Cell["\<\
Our first task is to find the distance between the lines. This is done using \
PointLineDistance after noting the point {0,3} lies on the second line.\
\>", "Text",
 CellID->1891145988],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PointLineDistance", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "3"}], "}"}], ",", 
   RowBox[{"y", "\[Equal]", 
    RowBox[{"ln1", "[", "x", "]"}]}], ",", "x"}], "]"}]], "Input",
 CellLabel->"In[50]:=",
 CellID->1732644716],

Cell[BoxData[
 FractionBox["6", 
  SqrtBox["5"]]], "Output",
 ImageSize->{33, 35},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[50]=",
 CellID->1246795593]
}, Open  ]],

Cell["\<\
The radius of our circle will be half the distance between the lines.\
\>", "Text",
 CellID->1895587772],

Cell["Finding the radius of the circle.", "MathCaption",
 CellID->1223780759],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rad", " ", "=", " ", 
  RowBox[{
   RowBox[{"PointLineDistance", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "3"}], "}"}], ",", 
     RowBox[{"y", "\[Equal]", 
      RowBox[{"ln1", "[", "x", "]"}]}], ",", "x"}], "]"}], "/", 
   "2"}]}]], "Input",
 CellLabel->"In[51]:=",
 CellID->1632329194],

Cell[BoxData[
 FractionBox["3", 
  SqrtBox["5"]]], "Output",
 ImageSize->{33, 35},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[51]=",
 CellID->453173133]
}, Open  ]],

Cell[TextData[{
 "Next we note the center of the circle must lie on the line ",
 StyleBox["y = ",
  FontSlant->"Italic"],
 Cell[BoxData[
  FractionBox["x", "2"]], "InlineFormula"],
 "+ ",
 Cell[BoxData[
  FractionBox["3", "2"]], "InlineFormula"],
 ". A circle with center ",
 StyleBox["pt",
  FontSlant->"Italic"],
 " and radius ",
 StyleBox["rad",
  FontSlant->"Italic"],
 " will intersect the line of the circle center at the desired locations for \
centers of the circles we are seeking."
}], "Text",
 CellID->1503834069],

Cell["Finding centers of circles satisfying the given conditions.", \
"MathCaption",
 CellID->1204156140],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sol", "=", 
  RowBox[{"Solve", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"MakeCircle", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"3", ",", 
          RowBox[{"7", "/", "2"}]}], "}"}], ",", 
        RowBox[{"3", "/", 
         SqrtBox["5"]}], ",", 
        RowBox[{"{", 
         RowBox[{"x", ",", "y"}], "}"}]}], "]"}], "\[Equal]", "0"}], "&&", 
     RowBox[{"y", "\[Equal]", 
      RowBox[{
       RowBox[{"x", "/", "2"}], "+", 
       RowBox[{"3", "/", "2"}]}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}]], "Input",
 CellLabel->"In[52]:=",
 CellID->366355652],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"x", "\[Rule]", 
      RowBox[{
       FractionBox["4", "5"], " ", 
       RowBox[{"(", 
        RowBox[{"4", "-", 
         SqrtBox["2"]}], ")"}]}]}], ",", 
     RowBox[{"y", "\[Rule]", 
      RowBox[{
       FractionBox["1", "10"], " ", 
       RowBox[{"(", 
        RowBox[{"31", "-", 
         RowBox[{"4", " ", 
          SqrtBox["2"]}]}], ")"}]}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"x", "\[Rule]", 
      RowBox[{
       FractionBox["4", "5"], " ", 
       RowBox[{"(", 
        RowBox[{"4", "+", 
         SqrtBox["2"]}], ")"}]}]}], ",", 
     RowBox[{"y", "\[Rule]", 
      RowBox[{
       FractionBox["1", "10"], " ", 
       RowBox[{"(", 
        RowBox[{"31", "+", 
         RowBox[{"4", " ", 
          SqrtBox["2"]}]}], ")"}]}]}]}], "}"}]}], "}"}]], "Output",
 ImageSize->{512, 32},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[52]=",
 CellID->503827215]
}, Open  ]],

Cell["\<\
Visualizing our solutions. The construction circle is shown in gray and the \
solution circles are shown in blue.\
\>", "MathCaption",
 CellID->295982384],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ln1", "[", "x", "]"}], ",", 
     RowBox[{"ln2", "[", "x", "]"}], ",", 
     RowBox[{
      RowBox[{"x", "/", "2"}], "+", 
      RowBox[{"3", "/", "2"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "6"}], "}"}], ",", 
   RowBox[{"Epilog", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"Red", ",", 
      RowBox[{"Point", "[", "pt", "]"}], ",", "Gray", ",", 
      RowBox[{"Circle", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"3", ",", 
          RowBox[{"7", "/", "2"}]}], "}"}], ",", 
        RowBox[{"3", "/", 
         SqrtBox["5"]}]}], "]"}], ",", "Blue", ",", 
      RowBox[{
       RowBox[{"Circle", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"x", ",", "y"}], "}"}], ",", 
         RowBox[{"3", "/", 
          SqrtBox["5"]}]}], "]"}], "/.", "sol"}]}], "}"}]}], ",", 
   RowBox[{"AspectRatio", "\[Rule]", " ", "Automatic"}]}], "]"}]], "Input",
 CellLabel->"In[53]:=",
 CellID->1003801436],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwtxX1MzWscAPBfnepJpOJ0Hhc1L/WTzDJu9+qavl8vY16Sl6kUh151u91T
t3ktJXG85JyzXtgkSn80Ws0mk3mLUB2LzDTNqG4RKm4vJ6XOPR3s+f7x2Wdm
VNLmWFtJkoJ++Pm3dZqLG5MyA8XJgcdj5vg8l9Ug3gDP/nufoJc1tBr6rq/Q
JcoZtAayVKbp22UDnQEN9Ws7t8hFtAGSLu17EixfpYugRqe7EyRX01fB3iGF
dSU8p+/DmsOR2hshzXQ93DwY21L6opVuhNfhf/ml7H9HvwS31fq4GvxIN4Nr
U2zl7xE99BuYPSOxTBvVS7fB4pa/G5bl9dPtUDOaub6p0ES/g/j7RmN11le6
EwqiL0913zVMf4T5Ye1DY7EjdBe0moIbcY2Z7oGBtl8WqLZa6C+QU566OzTU
SveC7OIYdKBQQnEfVCVnlhuKbVDcD03e+WxTni2KB8Dr6EufHQUKFJsgYGPb
r8WFdigehJZkbaBeZ4/ir1AZsM25NtcBxUOgCql/fUXLUDwM0CHlPtzjiOJv
UKCKd5YPjEPxCGQV/XnCP8EJxaNwsfFcnSF5PIrNkK7J7/NImYDi/6En0mfK
tihnFFvgtKXbrjVuIorH4Pbn1b0Tw11QbIVX3os7/vByRbGE8XOj/fJLXWkJ
a0Ii0pZ5utE2uCisu0xZ4kbbYOLRVc1Lp02ibfFknOOoTf4k2hbveURau5WT
aQXeGjuu8DVMphXYeH1Ft95NSdth7Yy4Ol2ekrbDYze3lrQ7udP2uH3WW3WE
3p22R3OV37R5TEU74CFF19OzGSraAVVeAZHeEqcZ7teEDDod5DTD6b/FGOU0
TjN8YPnnwvJ0TjN00utXph7hNMOi8sdnuk5xmmHdp4X+xvOcZqiMdtmnvctp
hrd8PdaWVHOaoXrA1/PeA04zvHJkVd3gY04zXHIpXRXzjNMMo1p6qpa3cpoh
Kx3JVv/LaYYViWxnagenGQ6ZZ7HKD5xmmD01PHRmL6cZ+nXsnre0n9MMm8r2
WsNMnGboGZB7OWeY0wwfScVpFSOcZhhvrAg2mjnNcELO7dnvLZxmeC3UOGy1
cprhd3bOJbc=
     "]]}, 
   {Hue[0.9060679774997897, 0.6, 0.6], LineBox[CompressedData["
1:eJwVjns01HkYxkeDGZcZc/9a4Sia2hQdKnTxe1QUW7G1xjUb47ZY4SQhudcm
I5dsK0XOpki2aNOmpXTRqC1ppSSSrIqoSLcJ6/fHe55z3vM8n+eZFbBtU9AM
BoOxYfpo/fRd5FG3bSn2rhU2bAaDjT2Bc+e1Sv2oHWmjEX1qbNwZ6Q9TSCOp
Et/qtutMNt6eW50dId1NDfNMi/ex2EiTjBn6SnOo/fFcC6EeG7dvuvy3WVpC
KWkxZmPbsR0trtI/KIfh/NL1K9hoys6+tEHaSFlbjSwt2smGhmYM61VYK7Uo
d/k6DxUbzsn+medlD6lxIvMMj9XChfig7vK2HqrYo2OLwbAWOr3DLWPinlM+
4cOOI5Ha4K9VBDfhBdVX0bZA/6U2eO1BtTY+Q1Ra/foPweE6MDWJqMwMeEOp
Zyw87/ZaB7bdP992yH9HbYzriskN1EXTl5T17cVj1Fl7E+H4gC5CLyuVjWnj
1GNLq9ozWzkokp80EG/9SA08qnTSeMTBQs9nHyaDPlM6sr8akr246BlzvQtn
FfXpa4G1230uRp9+s0jiPkF9jdPMdnTXQ25VQoiHxxTVf6qn88EDPUj12Bt2
FjNwi7lK2CPloS4qpSqnVA0Whr2nnofw0D6ngPV9/gzEtvQ7HDzOg1n6v/O2
FDHR0rvYxWOABzu3p4tLi9XBbbe580LKR3dUpr0iWwM2vmWupnI+au28ODfy
NBGxUbY6+jgfEtnNzopMFvY8iF4a9owPqo+Rd3U7G7m2OL7ISIAiSShHulML
JzS807X8BEgr+WnvkjBtHAt57K84LMDRu78150Tp4KT1iRrTJwIkRRa8NYrR
RWmX2a69hkIM+c/T9wrgIFFf2+lXTyH2Twyq9wRzIb+8LPVCkRD1r9e+4Xrr
wULVdim9Q4iOObZ9y8x4GBIYRhsJRAj9Vm5ZUM7DnwdrDrVsEqFJ5pPoYMzH
1qrqBpMDIlh7DlaKyvgwWDD+Q9Y9ESLSnR6unClAncPvQyUcMX4JZn9RKxBg
jbX6RIqzGA1G/lODIiF6cs4mx2aJcXFyD3N+jhC+ydfUxDfFuHtu9aCCL8KL
zlnjzWwJbpgEN2fnixDAs5Y7OUuQccG97Jm2GFe7Tt2LypTAd/YTPx+FGBb8
yDPVzRKo6ixnmrMk2N11SFHIJNjFfPVP4W4JGlO3D7hQBBIzO/85DAKd/CNn
BSkEcZGy99rxBCtszBMy/yYwXBqolCYScGdaBRY0EFyZiD6yKomgd9J2Y1kj
gbZCsSYhlSCz2Wl2wxWCkqrrB1/tI2h1D7j1/jpB80urJcrDBPLtRQaBdwhE
cr0dNDerhnVpVQ/BxflGLjTHt5Bb7vaUwG90vjHNsYwXH/DrJahIdWqmOe0w
lSf0ESw/liShOcb37HVqBwgCuofqaE7NSKzPrDcErPLPWXQu4/4uR8u3BKcj
WD/SOVlduuXKdwQfVLNZdE6VlMf0GpveYeDtQeccOdVVuR+ne/tCzGmf/rtz
hSWfpnsrY6c8p32D7fXJpz9P99rlnaR9B44oNytVBNcYpYn0PyCldWXHV4JQ
5WlX+r84sGNu/wSBbm69Ka2a67r5o5PTOz2UH6emCB6Z96to/R+ccA95
     "]]}, 
   {Hue[0.1421359549995791, 0.6, 0.6], LineBox[CompressedData["
1:eJwtx30w1HkAx/FlsVjWfsP+OpWhsrnc3Tb0QBffT7qUjtI1COWyluSETFKK
hO2u2tUic6dEpjEl2wN1esJR2rYH6ToJRSXpqEtSrBZ7NbN/fN6v+TiKE36K
MmSxWP6f90XNj/FHAxIyvD6H//nRvZJZzs3CcPqf0cv2RraGNr3tiZUL4+mi
y6HH93E09N35JbI4YTrtcPSZb22loZmCoanrhDmU+YK9ht65ueLlGmExVRTc
PuG3SEMTjm27tUp4hmZ2ONHC7RraIJNd9RfW0WjJk82ugxpqbJLE6Yttphsc
UhRnhkep7+4I6Z9Bj2ifSHmutf8TvbgjqrPs7y664Kzy6twOLW0P/UWUlPKC
Vrd2HHjSOUbJMnl0A17RxtXHQ+bcH6f8lqiqBWGvqVFy4Zum1gk6wyGuXCoe
oGamYbGlbTrq3rn5zuK8Qboy5XGSQsJCw6cMv5YjQ/Scl4P1x14WYv5Sq+sy
P9IOkWvV2Q0GKIw8YWe7YYT2tpX7GLcZ4Nu1z4cnokYpN+hS7e4QQ3QNrboH
Xy3VjOW7BTwwxPunX80RBI7TsRQT2dJANhQVqRuDg3W051RX+8OHbAitTP23
H2Ehn5MtrPE3QnViRkVOiQHC3OsDbzYbocUpn7M6zxD9ebeynH2NMTPrH+f1
hWzkNWw6dOauMTwCns4tOWKEqScTDjf7m6AzUeollxkj/pJ3iY/KBFUeIZY3
ck3QXqTc272cA0HQzfaTUg64FS/iWq5xQLtZude2msJhS8byiIWmKBTEWAq3
m8GPe8HEusYUmcWbfp0Xa47FzReu280zw9F7f6hyErlYudEvctklM6TF57+b
lmQBr4u+Gp6HOV5HOE8OEVtieI6N7JtKcxwY7zfqiuZBlZ5smODGxZU3ywZ4
oVZYP3w8hXOai1Yn9+6FM/l6LRDzdaQov4yv1wINQWE7F9sTvZZwW9tfblNK
9FoiLsvnkeeUSXp5+C3a9JNB/iS9PNROi9D121jrtcLlib3s2TnWeq1w7/yS
fjmxwat2x48qUz5uOESrZHk2EPPdIn18+ci+GFj63NwW1x6fup8o5WPd9Cfh
YXJbfEfiz55W8aGtFk1x4QiQ/vh3eQGbYBe7725BugB1e7b2rqAEgpkeEU4s
Bty8onOTMghS4oM+mO9gsGiBS6q0hmDqfIlauJMBb4qrJL+WoH58S5F3GoNn
E+4rS+sIzOXyH1L3MJCqfKbX1hMUVzQe6tvHoDlQfPtDI4HqX9d56sMMIrcW
2kmaCGwirbZJaxjsr+Rc9e4iuDx72orSOgbrCnhlAU8Jwt/Ptq+tZyDaYXsw
/BnByT0+qg+NDFowIzK1m+D7Y2kCSRMD+/te3KpeAnHn62rvLgaVb5PDHAcI
OGWj+8OfMch+sGup6B2BMo7zc2o3g6DqLJHnIMGwdjqnqpeBNi2XHTJEsN8u
NNhxgMFSy9MVihECUfdGF89BBpMHzxcUawhaypN1a4cY9Ldc2a0cJbD3yD2h
GGFwsEi9Rq0luM4q2akcZSDOaPZsHSOIUStXqbUM5kpaZ/WME1gorszoGWdg
sryTvJ8gqAxWj+h0DNpcerQ6HcH/vtwdxw==
     "]]}},
  Axes->True,
  AxesOrigin->{0, 0},
  Epilog->{
    RGBColor[1, 0, 0], 
    PointBox[
     NCache[{3, 
       Rational[7, 2]}, {3, 3.5}]], 
    GrayLevel[0.5], 
    CircleBox[
     NCache[{3, 
       Rational[7, 2]}, {3, 3.5}], 
     NCache[3 5^Rational[-1, 2], 1.3416407864998738`]], 
    RGBColor[0, 0, 1], {
     CircleBox[
      NCache[{
       Rational[4, 5] (4 - 2^Rational[1, 2]), 
        Rational[1, 10] (31 - 4 2^Rational[1, 2])}, {2.068629150101524, 
       2.5343145750507623`}], 
      NCache[3 5^Rational[-1, 2], 1.3416407864998738`]], 
     CircleBox[
      NCache[{
       Rational[4, 5] (4 + 2^Rational[1, 2]), 
        Rational[1, 10] (31 + 4 2^Rational[1, 2])}, {4.331370849898476, 
       3.6656854249492383`}], 
      NCache[3 5^Rational[-1, 2], 1.3416407864998738`]]}},
  PlotRange->{{0, 6}, {0., 5.99999993877551}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 ImageSize->{184, 189},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[53]=",
 CellID->259204020]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Problem 2", "Subsection",
 CellID->740726537],

Cell["\<\
Given two points in the plane show that two circles with radius greater than \
half the distance between the points and centers on the given points \
intersect at two points both of which lie on the perpendicular bisector of \
the line segment between the given points.\
\>", "Text",
 CellID->428205700],

Cell["The given points.", "MathCaption",
 CellID->635498803],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"pt1", ",", "pt2"}], "}"}], "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"5", ",", "2"}], "}"}]}], "}"}]}]], "Input",
 CellID->1392205490],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "2"}], "}"}]}], "}"}]], "Output",
 ImageSize->{107, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellID->1963336365]
}, Open  ]],

Cell["Finding the distance between the points.", "MathCaption",
 CellID->1708050400],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Distance", "[", 
  RowBox[{"pt1", ",", "pt2"}], "]"}]], "Input",
 CellID->1789163351],

Cell[BoxData[
 SqrtBox["17"]], "Output",
 ImageSize->{35, 18},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellID->1402414963]
}, Open  ]],

Cell["Finding the expression for the line between the points.", "MathCaption",
 CellID->1755257316],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ln1", "[", "x_", "]"}], "=", " ", 
  RowBox[{"TwoPointLine", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"5", ",", "2"}], "}"}], ",", "x"}], "]"}]}]], "Input",
 CellID->626851173],

Cell[BoxData[
 RowBox[{
  FractionBox["3", "4"], "+", 
  FractionBox["x", "4"]}]], "Output",
 ImageSize->{40, 32},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellID->2080376076]
}, Open  ]],

Cell["\<\
Finding the expression for the perpendicular bisector of the line segment.\
\>", "MathCaption",
 CellID->1699452525],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"midpt", "=", 
  RowBox[{"IntermediatePoint", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"5", ",", "2"}], "}"}], ",", 
    RowBox[{"1", "/", "2"}]}], "]"}]}]], "Input",
 CellID->1166082575],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", 
   FractionBox["3", "2"]}], "}"}]], "Output",
 ImageSize->{48, 32},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellID->414075503]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"perpln", "[", "x_", "]"}], "=", " ", 
  RowBox[{"PerpendicularLine", "[", 
   RowBox[{"midpt", ",", 
    RowBox[{"y", "\[Equal]", 
     RowBox[{"ln1", "[", "x", "]"}]}], ",", "x"}], "]"}]}]], "Input",
 CellID->1519962641],

Cell[BoxData[
 RowBox[{
  FractionBox["27", "2"], "-", 
  RowBox[{"4", " ", "x"}]}]], "Output",
 ImageSize->{53, 32},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellID->1694126222]
}, Open  ]],

Cell["Visualizing our lines.", "MathCaption",
 CellID->1841221228],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ln1", "[", "x", "]"}], ",", 
     RowBox[{"perpln", "[", "x", "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "6"}], "}"}], ",", 
   RowBox[{"Epilog", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"Red", ",", 
      RowBox[{"Point", "[", 
       RowBox[{"{", 
        RowBox[{"pt1", ",", "pt2"}], "}"}], "]"}]}], "}"}]}], ",", 
   RowBox[{"AspectRatio", "\[Rule]", " ", "Automatic"}]}], "]"}]], "Input",
 CellID->165825726],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwlxX8803kcwPF9IhP5FfbtVB5UlsvdrQf9oIv3O10rHaXrgVAu20hOyCMp
RcK6qzaN5XGnRB49PErWD+r0C0dprR9a1y2hqCTNVMyYHzO7euyP1+vpykn6
JWYKjUYL+tJXR39OPB2clOX3ZbY0mhIO8xa4y5lR8Mn0fVuTiRKaP3fHC5mJ
sOJmxNkjdCUMXF0lSGBmQrsre6m9jRKyGZrZW5h5QH3FWQmP7q97v4lZAqLC
h+cCVygh6cyeBxuYlyC73Q2K9iqhUSC4HcSsh1jeq52eaiVMNUuh98bLYZtL
muiSthcCDkbz/w59Ab0syZUWlQqu74vpKP+3E5Zdltxe3N4HbRG/sVLS3kFN
S/uxVx0fwW6NMLYRP0DTxrPhi55+AltFTPWyyD4wTS362NzyGea5JFTwOf0w
zTwyvqy1H7w7dj5aWaCG9WkvU0S8AWgczwpUnNLAFT8X++GeAYj7Ryarzx6G
dpZn9eVtaijinnNy3DYCPa0V7Kmtavh+81vtZMwYWIbeqDsYPgidmg1PMEAH
oxNir+BngzD4+ptFjBA9TKSZCVaHaEBUmb49LMwA3Rc6254/1wDTxjxo7yka
ium5zNqgIahJzqrMKyUY6d0Qcl8+BAo3MX1jwRRUFTzIcQ8Yhvk5/7lvLTLB
gsYdJy49Hgaf4NeLS0+Z4uzzSSflQVroSOb7CQVTMfGGfylbqoVqn3Cre/lm
2FYsOdy1dgQYoffbzvPpaFn5LkFxZwSgi5Z/Z7c5uuzKWhu9fBSKGHFWzL3T
MNDympl97Shkl+z4fUm8Ba6UX7vrtGQMTj/5S5qXbInrtwdy19wYg4xE8cCc
lOnodz1g1NpnHPqi3WeGc6xQu8hB8F3VOBzTq0w7Y61Rmpk6JclLB7c+rum3
jrDBrdqzafSLOmhx8+5aPt8WjU5A3LdclrjcFo1OQGNo5P6VznZoVA9em1UV
DmV2aFQPCTnsF76zZqDRSfgj1nyciGeg0UmomxNtUDnYo1ED3Jw8bLIwzx6N
GuDJ1VUqoZ0DfmhzHZaa0/CeS6xUUOCAHFsvLjuAhrnXQ8reWjjinZcXnibz
abhl7quoSKEj/mCXePmilIa6GtYsDzoDM1/+KSw0IXjApPdxYSYD6w/t7lkH
BBnzfaLdaBRaFhRfmZFFMC0xdMhiH4Urlnmk82sJzl7KkzH3U2g9y5MnriPY
oN9V7J9B4ZtJ7/Vl9QQthMKf0g9RyJey59Y1ECypbDrRe4RCeQjn4VATQanS
c4nsJIXc3UVOvGaCDlybPfxaCo9W0W/7dxK8uXDOurJ6CrcUWpcHvyYYNbjQ
ua6BQtY+x+NRbwieP8SWDjVRqMB53PQugj+eyWDwmil0fupnWd1DkNPRV+Pf
SWHV59RI136C9PKxo1FvKMx9dmA1a4CgJIH+a3oXhaE1OSxfNUGtbi69uodC
XUa+SbiG4FGniDDXfgpXW12sFI0QZHVt9/BVUzhTfbWwZJSgoiLVsFlDoUpx
66BkjKCzT/450QiFx4tlm2Q6gndppfslYxRysuS+LRME42SSDTIdhYt5LQu6
9QSni27N69ZTaLa2w25wkmBVmGzEYKCw1aNbZzAQ/B8wzDjX
     "]]}, 
   {Hue[0.9060679774997897, 0.6, 0.6], LineBox[CompressedData["
1:eJwdlH8803kcxydpiPm17ctot9l3Y+POVUdT8n2nTpfymHJmSsrPnBM7j1Jx
nGhKbR5D3SNW0eNO5nh0Uad0sY+S5pB+eJyHLnOk8usRmTNZ2nX3x+vx/Pf5
/OfFjkvflbiMRCKFftx/XNiedjEsPS8w/lfltMkkgMIED89eXgzhZGTwNrwX
QM+b0RQFL41oD66OzlwQwMz1zfJUXi7B0zV3Ts0IIJ+ud4vmFRPjNi9+GhgW
QNeDkJfhvEuENMVXcv2eANKrMjtFvKtELvcZSjopgDa5/PdQXitRoeKU9lAE
YLEigzye0ktUDZ1K32nFh20/xMp+E/cT2KNwEZ/mCTePJQ5WP9YRnWHhW7q4
HjCw+1ufjCMviBA+9xDHnQcOWxVJbfCaCLgafaXXhwv2fYmN6/ZMEu9PJzmt
4ePAYaXWyuKmCYPh53MxHhwQDh7s2lT6lkhw2MYY8nOHtsW8HX0qPVGVrxmg
rWFDskarbc3/h9hRpVJVBbKgPL6GQdtvIA5ckLycqGTCp5Lh+Q+J74inNUwr
swo30OlFD2GbkXC3ubZa08SA2SGXz+kRSwSF9ap3/BdnUNZlHYiMNBHSr2Xn
1Wo68OwsQ4+qSHBjeDrRrpMKTdK8uuJKMzjLNKlHkCP0ccvIO0uXgeTGwMze
v+wBL3jqubfcHKT6hugzFbbgHzb0RaVqObhp8r0LSqxgUCoLVMgtQB4wYdHV
YwGN/lG290tWgNBF9cfkLTOgix8MqGVkCBPdX938xEgQI6SSu4cswfLNwpHJ
qDminJ5syztqBdIfc/mVOWNE/qVvTvqmWAPy6XYUNT8hLj4831EsXQlQs/ZK
A6dbk5NWNrMqwwYevfazGmK/1kzGejpHxdmCxG5zm9ZRrzmzNLFcl0QByVSk
x4yfUXN7aus0ZbcdjDV+0rIh2Qz9yRWOrMft4dpnbQ79bAuUzI/3Kau2B9ju
kl3ubYXaxHuyNzEdwLLp2bOVszZorWSilnrZAbr319bsOmGPUguC+ze6OsI1
W8ZeQ6ojOpVkuWhW5gisCvYYeQcVtayKNU1QnWAu88RSXTgdNX8oNBcUO8Ho
oitFLXRGD69vnlA4UEHYS6ldF8xA91lJHfJSKlALzrusDXFDJ25GXB62poG9
Tiwr9GKiaPfnMXsUNLi15VZLE4OFjE0+rl5kOswf3OcnprDR9+bj3edy6bCw
4km51xIb0XH/WC4JA++vMu6GkDjoSJp4zvoYBm+e14/RZjnIzS9By8vGQJiq
VYbOcxBa+u5CUA4GecYXQtkiB1krFFuyjmNAZbgWzS3D0aW69rPjRRgERBbx
nzrhqGNsja+2AgP548QUpS+OqPF2mbI7GHh2MKesj+GoWbAq5HIrBhkR688G
5eAoZlbAbEEY3BmNCMg6jiP18eCOuXYMwswV8vEiHG2oyqEn9GCQCUZvbQWO
4gYnm4J0GLTf7j8ou4MjcvW70zF/Y0AJ0dNaEI7qU8n7skYwkAxQWufacTRv
dCc3vsJgyvAlJaEHR6cZuyPZ0x/9fW9cDdLhyGfkgNfGtxjEtPeKs0Zw1Fd7
2CTRY6AOn/zQ8ApHTP+SGqXhY1+Gu4g9jaN7pMrs+ncYFJoFGiR6HCVr60Va
IwaPlFGVSgOObJS3OaNLGDizDm/VGnHUEKk1mEwYxP3/Xzj6F+0FMDQ=
     "]]}},
  Axes->True,
  AxesOrigin->{0, 0},
  Epilog->{
    RGBColor[1, 0, 0], 
    PointBox[{{1, 1}, {5, 2}}]},
  PlotRange->{{0, 6}, {-10.49999951020408, 13.499999510204082`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 ImageSize->{79, 216},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellID->1746686723]
}, Open  ]],

Cell[TextData[{
 "Next we want to construct the circles with centers on the given points \
having radius greater than ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SqrtBox["17"], "/", "2.", " "}], TraditionalForm]]],
 " Let's take the radius to be 4."
}], "Text",
 CellID->462298971],

Cell["The two circles.", "MathCaption",
 CellID->262970751],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"circ1", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], "=", " ", 
  RowBox[{"MakeCircle", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "1"}], "}"}], ",", "4", ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}]], "Input",
 CellID->228066531],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "16"}], "+", 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"1", "-", "x"}], ")"}], "2"], "+", 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"1", "-", "y"}], ")"}], "2"]}]], "Output",
 ImageSize->{146, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellID->2003360269]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"circ2", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], "=", " ", 
  RowBox[{"MakeCircle", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"5", ",", "2"}], "}"}], ",", "4", ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}]], "Input",
 CellID->79939770],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "16"}], "+", 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"5", "-", "x"}], ")"}], "2"], "+", 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"2", "-", "y"}], ")"}], "2"]}]], "Output",
 ImageSize->{146, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellID->1370435015]
}, Open  ]],

Cell["\<\
Finding the points of intersection for the two circles. We intersect each \
circle with the perpendicular bisector and note the solutions are the same.\
\>", "MathCaption",
 CellID->1139183030],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"circOne", "=", 
  RowBox[{"Solve", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"circ1", "[", 
       RowBox[{"x", ",", "y"}], "]"}], "\[Equal]", "0"}], "&&", 
     RowBox[{"y", "\[Equal]", 
      RowBox[{"perpln", "[", "x", "]"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}]], "Input",
 CellID->1432472205],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"x", "\[Rule]", 
      RowBox[{
       FractionBox["1", "34"], " ", 
       RowBox[{"(", 
        RowBox[{"102", "-", 
         SqrtBox["799"]}], ")"}]}]}], ",", 
     RowBox[{"y", "\[Rule]", 
      RowBox[{
       FractionBox["1", "34"], " ", 
       RowBox[{"(", 
        RowBox[{"51", "+", 
         RowBox[{"4", " ", 
          SqrtBox["799"]}]}], ")"}]}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"x", "\[Rule]", 
      RowBox[{
       FractionBox["1", "34"], " ", 
       RowBox[{"(", 
        RowBox[{"102", "+", 
         SqrtBox["799"]}], ")"}]}]}], ",", 
     RowBox[{"y", "\[Rule]", 
      RowBox[{
       FractionBox["1", "34"], " ", 
       RowBox[{"(", 
        RowBox[{"51", "-", 
         RowBox[{"4", " ", 
          SqrtBox["799"]}]}], ")"}]}]}]}], "}"}]}], "}"}]], "Output",
 ImageSize->{309, 63},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellID->121182918]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"circTwo", "=", 
  RowBox[{"Solve", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"circ2", "[", 
       RowBox[{"x", ",", "y"}], "]"}], "\[Equal]", "0"}], "&&", 
     RowBox[{"y", "\[Equal]", 
      RowBox[{"perpln", "[", "x", "]"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}]], "Input",
 CellID->906805470],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"x", "\[Rule]", 
      RowBox[{
       FractionBox["1", "34"], " ", 
       RowBox[{"(", 
        RowBox[{"102", "-", 
         SqrtBox["799"]}], ")"}]}]}], ",", 
     RowBox[{"y", "\[Rule]", 
      RowBox[{
       FractionBox["1", "34"], " ", 
       RowBox[{"(", 
        RowBox[{"51", "+", 
         RowBox[{"4", " ", 
          SqrtBox["799"]}]}], ")"}]}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"x", "\[Rule]", 
      RowBox[{
       FractionBox["1", "34"], " ", 
       RowBox[{"(", 
        RowBox[{"102", "+", 
         SqrtBox["799"]}], ")"}]}]}], ",", 
     RowBox[{"y", "\[Rule]", 
      RowBox[{
       FractionBox["1", "34"], " ", 
       RowBox[{"(", 
        RowBox[{"51", "-", 
         RowBox[{"4", " ", 
          SqrtBox["799"]}]}], ")"}]}]}]}], "}"}]}], "}"}]], "Output",
 ImageSize->{309, 63},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellID->2135575178]
}, Open  ]],

Cell["\<\
We could also set the two expressions for the circle equal to zero and solve \
the system of equations. \
\>", "MathCaption",
 CellID->1717824335],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"circ1", "[", 
       RowBox[{"x", ",", "y"}], "]"}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{"circ2", "[", 
       RowBox[{"x", ",", "y"}], "]"}], "\[Equal]", "0"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellID->703448077],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"x", "\[Rule]", 
      RowBox[{
       FractionBox["1", "34"], " ", 
       RowBox[{"(", 
        RowBox[{"102", "-", 
         SqrtBox["799"]}], ")"}]}]}], ",", 
     RowBox[{"y", "\[Rule]", 
      RowBox[{
       FractionBox["1", "34"], " ", 
       RowBox[{"(", 
        RowBox[{"51", "+", 
         RowBox[{"4", " ", 
          SqrtBox["799"]}]}], ")"}]}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"x", "\[Rule]", 
      RowBox[{
       FractionBox["1", "34"], " ", 
       RowBox[{"(", 
        RowBox[{"102", "+", 
         SqrtBox["799"]}], ")"}]}]}], ",", 
     RowBox[{"y", "\[Rule]", 
      RowBox[{
       FractionBox["1", "34"], " ", 
       RowBox[{"(", 
        RowBox[{"51", "-", 
         RowBox[{"4", " ", 
          SqrtBox["799"]}]}], ")"}]}]}]}], "}"}]}], "}"}]], "Output",
 ImageSize->{309, 63},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellID->954353404]
}, Open  ]],

Cell["Checking that both solutions are the same.", "MathCaption",
 CellID->262301585],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"x", ",", "y"}], "}"}], "/.", "circOne"}], ")"}], "\[Equal]", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"x", ",", "y"}], "}"}], "/.", "circTwo"}], ")"}]}]], "Input",
 CellID->1866295451],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellID->681496875]
}, Open  ]],

Cell["A visual check of our solutions", "MathCaption",
 CellID->1387688674],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ln1", "[", "x", "]"}], ",", 
     RowBox[{"perpln", "[", "x", "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "6"}], "}"}], ",", 
   RowBox[{"Epilog", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"Red", ",", 
      RowBox[{"Point", "[", 
       RowBox[{"{", 
        RowBox[{"pt1", ",", "pt2"}], "}"}], "]"}], ",", "Blue", ",", 
      RowBox[{"Circle", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", "1"}], "}"}], ",", "4"}], "]"}], ",", 
      RowBox[{"Circle", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"5", ",", "2"}], "}"}], ",", "4"}], "]"}], ",", "Black", ",", 
      RowBox[{"Point", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"x", ",", "y"}], "}"}], "/.", "circOne"}], "]"}]}], "}"}]}], 
   ",", 
   RowBox[{"AspectRatio", "\[Rule]", " ", "Automatic"}]}], "]"}]], "Input",
 CellID->1211137498],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwlxX8803kcwPF9IhP5FfbtVB5UlsvdrQf9oIv3O10rHaXrgVAu20hOyCMp
RcK6qzaN5XGnRB49PErWD+r0C0dprR9a1y2hqCTNVMyYHzO7euyP1+vpykn6
JWYKjUYL+tJXR39OPB2clOX3ZbY0mhIO8xa4y5lR8Mn0fVuTiRKaP3fHC5mJ
sOJmxNkjdCUMXF0lSGBmQrsre6m9jRKyGZrZW5h5QH3FWQmP7q97v4lZAqLC
h+cCVygh6cyeBxuYlyC73Q2K9iqhUSC4HcSsh1jeq52eaiVMNUuh98bLYZtL
muiSthcCDkbz/w59Ab0syZUWlQqu74vpKP+3E5Zdltxe3N4HbRG/sVLS3kFN
S/uxVx0fwW6NMLYRP0DTxrPhi55+AltFTPWyyD4wTS362NzyGea5JFTwOf0w
zTwyvqy1H7w7dj5aWaCG9WkvU0S8AWgczwpUnNLAFT8X++GeAYj7Ryarzx6G
dpZn9eVtaijinnNy3DYCPa0V7Kmtavh+81vtZMwYWIbeqDsYPgidmg1PMEAH
oxNir+BngzD4+ptFjBA9TKSZCVaHaEBUmb49LMwA3Rc6254/1wDTxjxo7yka
ium5zNqgIahJzqrMKyUY6d0Qcl8+BAo3MX1jwRRUFTzIcQ8Yhvk5/7lvLTLB
gsYdJy49Hgaf4NeLS0+Z4uzzSSflQVroSOb7CQVTMfGGfylbqoVqn3Cre/lm
2FYsOdy1dgQYoffbzvPpaFn5LkFxZwSgi5Z/Z7c5uuzKWhu9fBSKGHFWzL3T
MNDympl97Shkl+z4fUm8Ba6UX7vrtGQMTj/5S5qXbInrtwdy19wYg4xE8cCc
lOnodz1g1NpnHPqi3WeGc6xQu8hB8F3VOBzTq0w7Y61Rmpk6JclLB7c+rum3
jrDBrdqzafSLOmhx8+5aPt8WjU5A3LdclrjcFo1OQGNo5P6VznZoVA9em1UV
DmV2aFQPCTnsF76zZqDRSfgj1nyciGeg0UmomxNtUDnYo1ED3Jw8bLIwzx6N
GuDJ1VUqoZ0DfmhzHZaa0/CeS6xUUOCAHFsvLjuAhrnXQ8reWjjinZcXnibz
abhl7quoSKEj/mCXePmilIa6GtYsDzoDM1/+KSw0IXjApPdxYSYD6w/t7lkH
BBnzfaLdaBRaFhRfmZFFMC0xdMhiH4Urlnmk82sJzl7KkzH3U2g9y5MnriPY
oN9V7J9B4ZtJ7/Vl9QQthMKf0g9RyJey59Y1ECypbDrRe4RCeQjn4VATQanS
c4nsJIXc3UVOvGaCDlybPfxaCo9W0W/7dxK8uXDOurJ6CrcUWpcHvyYYNbjQ
ua6BQtY+x+NRbwieP8SWDjVRqMB53PQugj+eyWDwmil0fupnWd1DkNPRV+Pf
SWHV59RI136C9PKxo1FvKMx9dmA1a4CgJIH+a3oXhaE1OSxfNUGtbi69uodC
XUa+SbiG4FGniDDXfgpXW12sFI0QZHVt9/BVUzhTfbWwZJSgoiLVsFlDoUpx
66BkjKCzT/450QiFx4tlm2Q6gndppfslYxRysuS+LRME42SSDTIdhYt5LQu6
9QSni27N69ZTaLa2w25wkmBVmGzEYKCw1aNbZzAQ/B8wzDjX
     "]]}, 
   {Hue[0.9060679774997897, 0.6, 0.6], LineBox[CompressedData["
1:eJwdlH8803kcxydpiPm17ctot9l3Y+POVUdT8n2nTpfymHJmSsrPnBM7j1Jx
nGhKbR5D3SNW0eNO5nh0Uad0sY+S5pB+eJyHLnOk8usRmTNZ2nX3x+vx/Pf5
/OfFjkvflbiMRCKFftx/XNiedjEsPS8w/lfltMkkgMIED89eXgzhZGTwNrwX
QM+b0RQFL41oD66OzlwQwMz1zfJUXi7B0zV3Ts0IIJ+ud4vmFRPjNi9+GhgW
QNeDkJfhvEuENMVXcv2eANKrMjtFvKtELvcZSjopgDa5/PdQXitRoeKU9lAE
YLEigzye0ktUDZ1K32nFh20/xMp+E/cT2KNwEZ/mCTePJQ5WP9YRnWHhW7q4
HjCw+1ufjCMviBA+9xDHnQcOWxVJbfCaCLgafaXXhwv2fYmN6/ZMEu9PJzmt
4ePAYaXWyuKmCYPh53MxHhwQDh7s2lT6lkhw2MYY8nOHtsW8HX0qPVGVrxmg
rWFDskarbc3/h9hRpVJVBbKgPL6GQdtvIA5ckLycqGTCp5Lh+Q+J74inNUwr
swo30OlFD2GbkXC3ubZa08SA2SGXz+kRSwSF9ap3/BdnUNZlHYiMNBHSr2Xn
1Wo68OwsQ4+qSHBjeDrRrpMKTdK8uuJKMzjLNKlHkCP0ccvIO0uXgeTGwMze
v+wBL3jqubfcHKT6hugzFbbgHzb0RaVqObhp8r0LSqxgUCoLVMgtQB4wYdHV
YwGN/lG290tWgNBF9cfkLTOgix8MqGVkCBPdX938xEgQI6SSu4cswfLNwpHJ
qDminJ5syztqBdIfc/mVOWNE/qVvTvqmWAPy6XYUNT8hLj4831EsXQlQs/ZK
A6dbk5NWNrMqwwYevfazGmK/1kzGejpHxdmCxG5zm9ZRrzmzNLFcl0QByVSk
x4yfUXN7aus0ZbcdjDV+0rIh2Qz9yRWOrMft4dpnbQ79bAuUzI/3Kau2B9ju
kl3ubYXaxHuyNzEdwLLp2bOVszZorWSilnrZAbr319bsOmGPUguC+ze6OsI1
W8ZeQ6ojOpVkuWhW5gisCvYYeQcVtayKNU1QnWAu88RSXTgdNX8oNBcUO8Ho
oitFLXRGD69vnlA4UEHYS6ldF8xA91lJHfJSKlALzrusDXFDJ25GXB62poG9
Tiwr9GKiaPfnMXsUNLi15VZLE4OFjE0+rl5kOswf3OcnprDR9+bj3edy6bCw
4km51xIb0XH/WC4JA++vMu6GkDjoSJp4zvoYBm+e14/RZjnIzS9By8vGQJiq
VYbOcxBa+u5CUA4GecYXQtkiB1krFFuyjmNAZbgWzS3D0aW69rPjRRgERBbx
nzrhqGNsja+2AgP548QUpS+OqPF2mbI7GHh2MKesj+GoWbAq5HIrBhkR688G
5eAoZlbAbEEY3BmNCMg6jiP18eCOuXYMwswV8vEiHG2oyqEn9GCQCUZvbQWO
4gYnm4J0GLTf7j8ou4MjcvW70zF/Y0AJ0dNaEI7qU8n7skYwkAxQWufacTRv
dCc3vsJgyvAlJaEHR6cZuyPZ0x/9fW9cDdLhyGfkgNfGtxjEtPeKs0Zw1Fd7
2CTRY6AOn/zQ8ApHTP+SGqXhY1+Gu4g9jaN7pMrs+ncYFJoFGiR6HCVr60Va
IwaPlFGVSgOObJS3OaNLGDizDm/VGnHUEKk1mEwYxP3/Xzj6F+0FMDQ=
     "]]}},
  Axes->True,
  AxesOrigin->{0, 0},
  Epilog->{
    RGBColor[1, 0, 0], 
    PointBox[{{1, 1}, {5, 2}}], 
    RGBColor[0, 0, 1], 
    CircleBox[{1, 1}, 4], 
    CircleBox[{5, 2}, 4], 
    GrayLevel[0], 
    PointBox[
     NCache[{{
       Rational[1, 34] (102 - 799^Rational[1, 2]), 
        Rational[1, 34] (51 + 4 799^Rational[1, 2])}, {
       Rational[1, 34] (102 + 799^Rational[1, 2]), 
        Rational[1, 34] (51 - 4 799^Rational[1, 2])}}, {{2.1686297632292604`, 
      4.825480947082958}, {3.8313702367707396`, -1.8254809470829576`}}]]},
  PlotRange->{{0, 6}, {-10.49999951020408, 13.499999510204082`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 ImageSize->{79, 216},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellID->1737922620]
}, Open  ]],

Cell["\<\
This is a confirmation of the ruler and compass construction for a \
perpendicular bisector of a line segment.\
\>", "Text",
 CellID->1340391493]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Problems In Analytic Geometry - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "TutorialColor", 
  "LinkTrails" -> GridBox[{{
      RowBox[{
        ButtonBox[
        "Classroom Utilities", ButtonData -> 
         "paclet:ClassroomUtilities/guide/ClassroomUtilities", 
         BaseStyle -> {"Link", "DockedLinkTrail"}], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 32, 54.2800624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Tutorial", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "Some of the functions in the ClassroomUtilities` package are intended to \
facilitate the solutions of problems found in a typical high school algebra \
or geometry class. Analytic Geometry functions Create the expression for a \
line through two given points.", "synonyms" -> {}, "title" -> 
    "Problems In Analytic Geometry", "titlemodifier" -> "", "windowtitle" -> 
    "Problems In Analytic Geometry", "type" -> "Tutorial", "uri" -> 
    "ClassroomUtilities/tutorial/ProblemsInAnalyticGeometry"}, 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> Notebook], 
   Cell[
    StyleData["Output"], CellContext -> Notebook], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[622, 21, 3056, 67, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3703, 92, 66, 1, 51, "Title",
 CellID->509267359],
Cell[3772, 95, 216, 5, 50, "Text",
 CellID->1534169418],
Cell[3991, 102, 2842, 92, 163, "DefinitionBox",
 CellID->2096742444],
Cell[6836, 196, 67, 1, 31, "Caption",
 CellID->1891092685],
Cell[CellGroupData[{
Cell[6928, 201, 58, 1, 51, "Section",
 CellID->865725525],
Cell[6989, 204, 104, 2, 28, "Input",
 CellID->442055860],
Cell[7096, 208, 103, 2, 31, "MathCaption",
 CellID->836781195],
Cell[7202, 212, 104, 3, 28, "Input",
 CellID->2086258710],
Cell[CellGroupData[{
Cell[7331, 219, 299, 10, 28, "Input",
 CellID->2058623809],
Cell[7633, 231, 224, 8, 53, "Output",
 CellID->1526859018]
}, Open  ]],
Cell[7872, 242, 78, 1, 31, "MathCaption",
 CellID->446279958],
Cell[CellGroupData[{
Cell[7975, 247, 356, 12, 70, "Input",
 CellID->1614358782],
Cell[8334, 261, 381, 14, 53, "Output",
 CellID->1219652876]
}, Open  ]],
Cell[8730, 278, 104, 2, 70, "MathCaption",
 CellID->974476317],
Cell[CellGroupData[{
Cell[8859, 284, 426, 14, 70, "Input",
 CellID->1313167797],
Cell[9288, 300, 1026, 37, 84, "Output",
 CellID->1637417523]
}, Open  ]],
Cell[10329, 340, 67, 1, 70, "MathCaption",
 CellID->1026957464],
Cell[CellGroupData[{
Cell[10421, 345, 792, 25, 70, "Input",
 CellID->748026509],
Cell[11216, 372, 8205, 158, 70, "Output",
 CellID->73461614]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[19470, 536, 56, 1, 70, "Section",
 CellID->1047190250],
Cell[19529, 539, 105, 2, 70, "Input",
 CellID->1486904047],
Cell[CellGroupData[{
Cell[19659, 545, 52, 1, 70, "Subsection",
 CellID->1798068103],
Cell[19714, 548, 176, 4, 70, "Text",
 CellID->1498044585],
Cell[19893, 554, 69, 1, 70, "MathCaption",
 CellID->537062667],
Cell[19965, 557, 153, 5, 70, "Input",
 CellID->538475928],
Cell[20121, 564, 192, 6, 70, "Input",
 CellID->668833438],
Cell[CellGroupData[{
Cell[20338, 574, 182, 6, 70, "Input",
 CellID->1154184068],
Cell[20523, 582, 226, 8, 53, "Output",
 CellID->151847525]
}, Open  ]],
Cell[20764, 593, 74, 1, 70, "MathCaption",
 CellID->890780039],
Cell[CellGroupData[{
Cell[20863, 598, 502, 15, 70, "Input",
 CellID->1231612236],
Cell[21368, 615, 3351, 68, 210, "Output",
 CellID->111977019]
}, Open  ]],
Cell[24734, 686, 195, 4, 70, "Text",
 CellID->1891145988],
Cell[CellGroupData[{
Cell[24954, 694, 260, 8, 70, "Input",
 CellID->1732644716],
Cell[25217, 704, 192, 7, 56, "Output",
 CellID->1246795593]
}, Open  ]],
Cell[25424, 714, 114, 3, 70, "Text",
 CellID->1895587772],
Cell[25541, 719, 77, 1, 70, "MathCaption",
 CellID->1223780759],
Cell[CellGroupData[{
Cell[25643, 724, 333, 11, 70, "Input",
 CellID->1632329194],
Cell[25979, 737, 191, 7, 56, "Output",
 CellID->453173133]
}, Open  ]],
Cell[26185, 747, 524, 18, 70, "Text",
 CellID->1503834069],
Cell[26712, 767, 105, 2, 70, "MathCaption",
 CellID->1204156140],
Cell[CellGroupData[{
Cell[26842, 773, 653, 22, 70, "Input",
 CellID->366355652],
Cell[27498, 797, 1013, 37, 53, "Output",
 CellID->503827215]
}, Open  ]],
Cell[28526, 837, 164, 4, 70, "MathCaption",
 CellID->295982384],
Cell[CellGroupData[{
Cell[28715, 845, 1049, 32, 70, "Input",
 CellID->1003801436],
Cell[29767, 879, 5704, 115, 210, "Output",
 CellID->259204020]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[35520, 1000, 51, 1, 70, "Subsection",
 CellID->740726537],
Cell[35574, 1003, 313, 6, 70, "Text",
 CellID->428205700],
Cell[35890, 1011, 60, 1, 70, "MathCaption",
 CellID->635498803],
Cell[CellGroupData[{
Cell[35975, 1016, 270, 10, 70, "Input",
 CellID->1392205490],
Cell[36248, 1028, 280, 10, 36, "Output",
 CellID->1963336365]
}, Open  ]],
Cell[36543, 1041, 84, 1, 70, "MathCaption",
 CellID->1708050400],
Cell[CellGroupData[{
Cell[36652, 1046, 109, 3, 70, "Input",
 CellID->1789163351],
Cell[36764, 1051, 148, 5, 39, "Output",
 CellID->1402414963]
}, Open  ]],
Cell[36927, 1059, 99, 1, 70, "MathCaption",
 CellID->1755257316],
Cell[CellGroupData[{
Cell[37051, 1064, 280, 9, 70, "Input",
 CellID->626851173],
Cell[37334, 1075, 200, 7, 53, "Output",
 CellID->2080376076]
}, Open  ]],
Cell[37549, 1085, 126, 3, 70, "MathCaption",
 CellID->1699452525],
Cell[CellGroupData[{
Cell[37700, 1092, 279, 9, 70, "Input",
 CellID->1166082575],
Cell[37982, 1103, 202, 7, 53, "Output",
 CellID->414075503]
}, Open  ]],
Cell[CellGroupData[{
Cell[38221, 1115, 256, 7, 70, "Input",
 CellID->1519962641],
Cell[38480, 1124, 203, 7, 53, "Output",
 CellID->1694126222]
}, Open  ]],
Cell[38698, 1134, 66, 1, 70, "MathCaption",
 CellID->1841221228],
Cell[CellGroupData[{
Cell[38789, 1139, 540, 16, 70, "Input",
 CellID->165825726],
Cell[39332, 1157, 3896, 74, 237, "Output",
 CellID->1746686723]
}, Open  ]],
Cell[43243, 1234, 282, 9, 70, "Text",
 CellID->462298971],
Cell[43528, 1245, 59, 1, 70, "MathCaption",
 CellID->262970751],
Cell[CellGroupData[{
Cell[43612, 1250, 305, 10, 70, "Input",
 CellID->228066531],
Cell[43920, 1262, 334, 12, 36, "Output",
 CellID->2003360269]
}, Open  ]],
Cell[CellGroupData[{
Cell[44291, 1279, 304, 10, 70, "Input",
 CellID->79939770],
Cell[44598, 1291, 334, 12, 36, "Output",
 CellID->1370435015]
}, Open  ]],
Cell[44947, 1306, 203, 4, 70, "MathCaption",
 CellID->1139183030],
Cell[CellGroupData[{
Cell[45175, 1314, 376, 12, 70, "Input",
 CellID->1432472205],
Cell[45554, 1328, 1003, 36, 84, "Output",
 CellID->121182918]
}, Open  ]],
Cell[CellGroupData[{
Cell[46594, 1369, 375, 12, 70, "Input",
 CellID->906805470],
Cell[46972, 1383, 1004, 36, 84, "Output",
 CellID->2135575178]
}, Open  ]],
Cell[47991, 1422, 156, 4, 70, "MathCaption",
 CellID->1717824335],
Cell[CellGroupData[{
Cell[48172, 1430, 393, 13, 70, "Input",
 CellID->703448077],
Cell[48568, 1445, 1003, 36, 84, "Output",
 CellID->954353404]
}, Open  ]],
Cell[49586, 1484, 85, 1, 70, "MathCaption",
 CellID->262301585],
Cell[CellGroupData[{
Cell[49696, 1489, 287, 10, 70, "Input",
 CellID->1866295451],
Cell[49986, 1501, 138, 4, 36, "Output",
 CellID->681496875]
}, Open  ]],
Cell[50139, 1508, 75, 1, 70, "MathCaption",
 CellID->1387688674],
Cell[CellGroupData[{
Cell[50239, 1513, 980, 29, 70, "Input",
 CellID->1211137498],
Cell[51222, 1544, 4336, 85, 237, "Output",
 CellID->1737922620]
}, Open  ]],
Cell[55573, 1632, 155, 4, 70, "Text",
 CellID->1340391493]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[55767, 1641, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

