(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     22488,        744]
NotebookOptionsPosition[     17234,        582]
NotebookOutlinePosition[     19751,        644]
CellTagsIndexPosition[     19708,        641]
WindowTitle->Series Solutions of ODEs - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Classroom Utilities\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/guide/ClassroomUtilities"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"TutorialMoreAboutSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"ClassroomUtilities/tutorial/Series Solutions of ODEs\"\
\>":>None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["ClassroomUtilities/tutorial/Series Solutions of ODEs"]}, 
            Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/ClassroomUtilities/\
tutorial/Series Solutions of ODEs.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "ClassroomUtilities/tutorial/Series Solutions of ODEs", 
             ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell["Series Solutions of ODEs", "Title",
 CellID->509267359],

Cell["\<\
There are a number of functions for operating on symbolic series defined in \
the ClassroomUtilities` package. The allow the user to manipulate the index \
set, extract a finite number of terms from the beginning of a series and to \
do term by term differentiation of a series. \
\>", "Text",
 CellID->1534169418],

Cell["This loads the package.", "MathCaption",
 CellID->856451969],

Cell[BoxData[
 RowBox[{"<<", "ClassroomUtilities`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1952840789],

Cell[BoxData[GridBox[{
   {
    RowBox[{"MakeSeries", "[", 
     RowBox[{
      StyleBox["x", "TI"], ",", 
      StyleBox["a", "TI"], ",", 
      StyleBox["i", "TI"], ",", 
      SubscriptBox["\[DoubleStruckD]", "n"]}], "]"}], Cell[TextData[{
     "construct a series with general term ",
     Cell[BoxData[
      RowBox[{
       RowBox[{
        StyleBox["a", "TI"], "[", 
        StyleBox["i", "TI"], "]"}], 
       SuperscriptBox[
        StyleBox["x", "TI"], 
        StyleBox["i", "TI"]]}]], "InlineFormula"],
     " together with all its derivatives up to order ",
     Cell[BoxData[
      StyleBox["n", "TI"]], "InlineFormula"],
     "."
    }], "TableText"]},
   {
    RowBox[{"ChangeIndices", "[", 
     RowBox[{
      StyleBox["sum", "TI"], ",", 
      SubscriptBox[
       StyleBox["n", "TI"], 
       StyleBox["i", "TI"]], ",", 
      SubscriptBox[
       StyleBox["n", "TI"], 
       StyleBox[
        RowBox[{"i", "+", "k"}], "TI"]]}], "]"}], Cell[TextData[{
     "shifts the index set in a series by ",
     Cell[BoxData[
      StyleBox["k", "TI"]], "InlineFormula"],
     " units."
    }], "TableText"]},
   {
    RowBox[{"PartitionSum", "[", 
     RowBox[{
      StyleBox["sum", "TI"], ",", 
      StyleBox["n", "TI"]}], "]"}], Cell[
    "separates out the first n terms of a series.", "TableText"]},
   {
    RowBox[{"GetSummands", "[", 
     StyleBox["sum", "TI"], "]"}], Cell[
    "gets the general term out of a series.", "TableText"]}
  }]], "DefinitionBox",
 CellID->2096742444],

Cell["Series operations.", "Caption",
 CellID->1891092685],

Cell["\<\
To illustrate the use of these functions we will solve a simple differential \
equation assuming the solution can be given as a Taylor series.\
\>", "Text",
 CellID->1402563622],

Cell[CellGroupData[{

Cell["Solving the ODE y'[t]+ y[t]==0", "Subsection",
 CellID->1558445075],

Cell["We begin by creating a series for y and y'.", "MathCaption",
 CellID->836781195],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"y", "[", "t_", "]"}], ",", 
    RowBox[{"yprm", "[", "t_", "]"}]}], "}"}], "=", 
  RowBox[{"MakeSeries", "[", 
   RowBox[{"t", ",", "a", ",", "n", ",", "1"}], "]"}]}]], "Input",
 CellLabel->"In[9]:=",
 CellID->2058623809],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"n", "=", "0"}], "\[Infinity]"], 
    RowBox[{
     SuperscriptBox["t", "n"], " ", 
     RowBox[{"a", "[", "n", "]"}]}]}], ",", 
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"n", "=", "0"}], "\[Infinity]"], 
    RowBox[{"n", " ", 
     SuperscriptBox["t", 
      RowBox[{
       RowBox[{"-", "1"}], "+", "n"}]], " ", 
     RowBox[{"a", "[", "n", "]"}]}]}]}], "}"}]], "Output",
 ImageSize->{176, 38},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->2075816713]
}, Open  ]],

Cell["\<\
Next we change the indices on the second summation to be expressed in power \
of n.\
\>", "MathCaption",
 CellID->232166229],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"yprm2", "=", 
  RowBox[{"ChangeIndices", "[", 
   RowBox[{
    RowBox[{"yprm", "[", "t", "]"}], ",", "n", ",", 
    RowBox[{"n", "+", "1"}]}], "]"}]}]], "Input",
 CellLabel->"In[17]:=",
 CellID->40024204],

Cell[BoxData[
 RowBox[{
  UnderoverscriptBox["\[Sum]", 
   RowBox[{"n", "=", 
    RowBox[{"-", "1"}]}], "\[Infinity]"], 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "+", "n"}], ")"}], " ", 
   SuperscriptBox["t", "n"], " ", 
   RowBox[{"a", "[", 
    RowBox[{"1", "+", "n"}], "]"}]}]}]], "Output",
 ImageSize->{134, 38},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[17]=",
 CellID->310282303]
}, Open  ]],

Cell["\<\
Now we need to extract the first term from our summation for y and add the \
two expressions.\
\>", "MathCaption",
 CellID->571984131],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"serSum", "=", 
  RowBox[{
   RowBox[{"PartitionSum", "[", 
    RowBox[{
     RowBox[{"y", "[", "t", "]"}], ",", "1"}], "]"}], "+", 
   "yprm2"}]}]], "Input",
 CellLabel->"In[18]:=",
 CellID->1200485745],

Cell[BoxData[
 RowBox[{
  RowBox[{"a", "[", "0", "]"}], "+", 
  RowBox[{
   UnderoverscriptBox["\[Sum]", 
    RowBox[{"n", "=", "1"}], "\[Infinity]"], 
   RowBox[{
    SuperscriptBox["t", "n"], " ", 
    RowBox[{"a", "[", "n", "]"}]}]}], "+", 
  RowBox[{
   UnderoverscriptBox["\[Sum]", 
    RowBox[{"n", "=", 
     RowBox[{"-", "1"}]}], "\[Infinity]"], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "+", "n"}], ")"}], " ", 
    SuperscriptBox["t", "n"], " ", 
    RowBox[{"a", "[", 
     RowBox[{"1", "+", "n"}], "]"}]}]}]}]], "Output",
 ImageSize->{249, 38},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[18]=",
 CellID->824371605]
}, Open  ]],

Cell["\<\
Now we need to get the general term so we can find a closed form solution.\
\>", "MathCaption",
 CellID->679614795],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GetSummands", "[", 
  RowBox[{"serSum", "[", 
   RowBox[{"[", 
    RowBox[{"2", ";;"}], "]"}], "]"}], "]"}]], "Input",
 CellLabel->"In[19]:=",
 CellID->1967824819],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["t", "n"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"a", "[", "n", "]"}], "+", 
    RowBox[{"a", "[", 
     RowBox[{"1", "+", "n"}], "]"}], "+", 
    RowBox[{"n", " ", 
     RowBox[{"a", "[", 
      RowBox[{"1", "+", "n"}], "]"}]}]}], ")"}]}]], "Output",
 ImageSize->{194, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[19]=",
 CellID->1796558438]
}, Open  ]],

Cell["The largest index for a is n+1 so let's solve for that term.", \
"MathCaption",
 CellID->1137590646],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     SuperscriptBox["t", "n"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"a", "[", "n", "]"}], "+", 
       RowBox[{"a", "[", 
        RowBox[{"1", "+", "n"}], "]"}], "+", 
       RowBox[{"n", " ", 
        RowBox[{"a", "[", 
         RowBox[{"1", "+", "n"}], "]"}]}]}], ")"}]}], "\[Equal]", "0"}], ",", 
   RowBox[{"a", "[", 
    RowBox[{"n", "+", "1"}], "]"}]}], "]"}]], "Input",
 CellLabel->"In[20]:=",
 CellID->439078081],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", "[", 
     RowBox[{"1", "+", "n"}], "]"}], "\[Rule]", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{"a", "[", "n", "]"}], 
      RowBox[{"1", "+", "n"}]]}]}], "}"}], "}"}]], "Output",
 ImageSize->{138, 32},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[20]=",
 CellID->831496275]
}, Open  ]],

Cell[TextData[{
 "Now, using the power of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " we can find a closed form for the solution in two steps."
}], "Text",
 CellID->1281802896],

Cell["\<\
First we use RecurrenceTable to generate the coefficient of the first few \
terms of our solution.\
\>", "MathCaption",
 CellID->1852810015],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"coeffs", "=", 
  RowBox[{"RecurrenceTable", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"a", "[", 
        RowBox[{"1", "+", "n"}], "]"}], "==", 
       RowBox[{"-", 
        FractionBox[
         RowBox[{"a", "[", "n", "]"}], 
         RowBox[{"1", "+", "n"}]]}]}], ",", 
      RowBox[{
       RowBox[{"a", "[", "0", "]"}], "\[Equal]", "1"}]}], "}"}], ",", 
    RowBox[{"a", "[", "n", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"n", ",", "0", ",", "10"}], "}"}]}], "]"}]}]], "Input",
 CellLabel->"In[22]:=",
 CellID->1665040605],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", 
   RowBox[{"-", "1"}], ",", 
   FractionBox["1", "2"], ",", 
   RowBox[{"-", 
    FractionBox["1", "6"]}], ",", 
   FractionBox["1", "24"], ",", 
   RowBox[{"-", 
    FractionBox["1", "120"]}], ",", 
   FractionBox["1", "720"], ",", 
   RowBox[{"-", 
    FractionBox["1", "5040"]}], ",", 
   FractionBox["1", "40320"], ",", 
   RowBox[{"-", 
    FractionBox["1", "362880"]}], ",", 
   FractionBox["1", "3628800"]}], "}"}]], "Output",
 ImageSize->{452, 32},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[22]=",
 CellID->752176219]
}, Open  ]],

Cell["\<\
Now we use the FindGeneratingFunction to find a closed form for the solution.\
\>", "MathCaption",
 CellID->1736966491],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FindGeneratingFunction", "[", 
  RowBox[{"coeffs", ",", "x"}], "]"}]], "Input",
 CellLabel->"In[24]:=",
 CellID->303116919],

Cell[BoxData[
 SuperscriptBox["\[ExponentialE]", 
  RowBox[{"-", "x"}]]], "Output",
 ImageSize->{22, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[24]=",
 CellID->158980968]
}, Open  ]],

Cell["\<\
We can check our solution by substituting into our differential equation.\
\>", "Text",
 CellID->1906062444],

Cell["A check of the solution.", "MathCaption",
 CellID->1177934009],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"D", "[", 
    RowBox[{
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"-", "x"}]], ",", "x"}], "]"}], "+", " ", 
   SuperscriptBox["\[ExponentialE]", 
    RowBox[{"-", "x"}]]}], "\[Equal]", "0"}]], "Input",
 CellLabel->"In[25]:=",
 CellID->2032631217],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[25]=",
 CellID->263007643]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["Classroom Utilities",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/guide/ClassroomUtilities"]], "TutorialMoreAbout",
 CellID->763424279]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED WOLFRAM EDUCATION GROUP COURSES", "TutorialRelatedLinksSection",
 WholeCellGroupOpener->True,
 CellID->415694148],

Cell[TextData[{
 ButtonBox["M50",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[":",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["An",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["Introduction",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["to",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["Mathematica",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["in",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["the",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["Classroom",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"]
}], "TutorialRelatedLinks",
 CellID->790177859]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Series Solutions of ODEs - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "TutorialColor", 
  "LinkTrails" -> GridBox[{{
      RowBox[{
        StyleBox[
         ButtonBox[
         "Mathematica", ButtonData -> "paclet:guide/Mathematica", 
          BaseStyle -> {"Link", "DockedLinkTrail"}], FontSlant -> "Italic"], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 32, 58.1176624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Tutorial", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "There are a number of functions for operating on symbolic series defined \
in the ClassroomUtilities` package. The allow the user to manipulate the \
index set, extract a finite number of terms from the beginning of a series \
and to do term by term differentiation of a series. This loads the package. \
Series operations.", "synonyms" -> {}, "title" -> "Series Solutions of ODEs", 
    "titlemodifier" -> "", "windowtitle" -> "Series Solutions of ODEs", 
    "type" -> "Tutorial", "uri" -> 
    "ClassroomUtilities/tutorial/Series Solutions of ODEs"}, 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> Notebook], 
   Cell[
    StyleData["Output"], CellContext -> Notebook], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[617, 21, 2541, 58, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3183, 83, 61, 1, 51, "Title",
 CellID->509267359],
Cell[3247, 86, 324, 6, 90, "Text",
 CellID->1534169418],
Cell[3574, 94, 66, 1, 31, "MathCaption",
 CellID->856451969],
Cell[3643, 97, 107, 3, 28, "Input",
 CellID->1952840789],
Cell[3753, 102, 1501, 49, 132, "DefinitionBox",
 CellID->2096742444],
Cell[5257, 153, 58, 1, 31, "Caption",
 CellID->1891092685],
Cell[5318, 156, 187, 4, 50, "Text",
 CellID->1402563622],
Cell[CellGroupData[{
Cell[5530, 164, 73, 1, 51, "Subsection",
 CellID->1558445075],
Cell[5606, 167, 86, 1, 31, "MathCaption",
 CellID->836781195],
Cell[CellGroupData[{
Cell[5717, 172, 286, 9, 28, "Input",
 CellID->2058623809],
Cell[6006, 183, 622, 21, 59, "Output",
 CellID->2075816713]
}, Open  ]],
Cell[6643, 207, 134, 4, 70, "MathCaption",
 CellID->232166229],
Cell[CellGroupData[{
Cell[6802, 215, 228, 7, 70, "Input",
 CellID->40024204],
Cell[7033, 224, 430, 15, 59, "Output",
 CellID->310282303]
}, Open  ]],
Cell[7478, 242, 144, 4, 70, "MathCaption",
 CellID->571984131],
Cell[CellGroupData[{
Cell[7647, 250, 226, 8, 70, "Input",
 CellID->1200485745],
Cell[7876, 260, 672, 23, 59, "Output",
 CellID->824371605]
}, Open  ]],
Cell[8563, 286, 125, 3, 70, "MathCaption",
 CellID->679614795],
Cell[CellGroupData[{
Cell[8713, 293, 187, 6, 70, "Input",
 CellID->1967824819],
Cell[8903, 301, 433, 15, 36, "Output",
 CellID->1796558438]
}, Open  ]],
Cell[9351, 319, 106, 2, 70, "MathCaption",
 CellID->1137590646],
Cell[CellGroupData[{
Cell[9482, 325, 512, 17, 70, "Input",
 CellID->439078081],
Cell[9997, 344, 394, 14, 53, "Output",
 CellID->831496275]
}, Open  ]],
Cell[10406, 361, 187, 6, 70, "Text",
 CellID->1281802896],
Cell[10596, 369, 150, 4, 70, "MathCaption",
 CellID->1852810015],
Cell[CellGroupData[{
Cell[10771, 377, 590, 19, 70, "Input",
 CellID->1665040605],
Cell[11364, 398, 615, 21, 53, "Output",
 CellID->752176219]
}, Open  ]],
Cell[11994, 422, 129, 3, 70, "MathCaption",
 CellID->1736966491],
Cell[CellGroupData[{
Cell[12148, 429, 147, 4, 70, "Input",
 CellID->303116919],
Cell[12298, 435, 214, 7, 36, "Output",
 CellID->158980968]
}, Open  ]],
Cell[12527, 445, 118, 3, 70, "Text",
 CellID->1906062444],
Cell[12648, 450, 68, 1, 70, "MathCaption",
 CellID->1177934009],
Cell[CellGroupData[{
Cell[12741, 455, 305, 10, 70, "Input",
 CellID->2032631217],
Cell[13049, 467, 162, 5, 36, "Output",
 CellID->263007643]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[13260, 478, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[13328, 481, 179, 4, 70, "TutorialMoreAbout",
 CellID->763424279]
}, Open  ]],
Cell[CellGroupData[{
Cell[13544, 490, 127, 2, 70, "TutorialRelatedLinksSection",
 CellID->415694148],
Cell[13674, 494, 3506, 82, 70, "TutorialRelatedLinks",
 CellID->790177859]
}, Open  ]]
}, Open  ]],
Cell[17207, 580, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

