(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     32795,       1145]
NotebookOptionsPosition[     25675,        915]
NotebookOutlinePosition[     28191,        977]
CellTagsIndexPosition[     28148,        974]
WindowTitle->Using Integration Techniques Functions - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Classroom Utilities\"\>":>
       Documentation`HelpLookup[
       "paclet:ClassroomUtilities/guide/ClassroomUtilities"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"TutorialMoreAboutSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Functions \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PartsIntegrate\"\>":>
       Documentation`HelpLookup[
       "paclet:IntegrationTechniques/ref/PartsIntegrate"], \
"\<\"SubstituteIntegrate\"\>":>Documentation`HelpLookup[
       "paclet:IntegrationTechniques/ref/SIntegrate"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"TutorialFunction"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"ClassroomUtilities/tutorial/\
UsingIntegrationTechniquesFunctions\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
             "ClassroomUtilities/tutorial/\
UsingIntegrationTechniquesFunctions"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/ClassroomUtilities/\
tutorial/UsingIntegrationTechniquesFunctions.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "ClassroomUtilities/tutorial/\
UsingIntegrationTechniquesFunctions", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["Using Integration Techniques Functions", "Title",
 CellID->509267359],

Cell[TextData[{
 "Some of the functions defined in ",
 Cell[BoxData["ClassroomUtilities`"], "InlineFormula"],
 "  provide tools for having students become familiar with the concepts of \
modifying an integrand to produce an equivalent but recognizable integrand."
}], "Text",
 CellID->1534169418],

Cell[BoxData[GridBox[{
   {
    RowBox[{
     ButtonBox["PartsIntegrate",
      BaseStyle->"Link",
      ButtonData->"paclet:IntegrationTechniques/ref/PartsIntegrate"], "[", 
     RowBox[{
      StyleBox["intgrnd", "TI"], ",", 
      StyleBox["var", "TI"], ",", 
      RowBox[{"{", 
       RowBox[{
        StyleBox["u", "TI"], ",", 
        RowBox[{"\[DifferentialD]", 
         StyleBox["v", "TI"]}]}], "}"}]}], "]"}], Cell[
    "use the method of integration by parts", "TableText"]},
   {
    RowBox[{
     ButtonBox["SubstituteIntegrate",
      BaseStyle->"Link",
      ButtonData->"paclet:IntegrationTechniques/ref/SIntegrate"], "[", 
     RowBox[{
      StyleBox["intgrnd", "TI"], ",", 
      StyleBox["var", "TI"], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         StyleBox["expr", "TI"], "\[Rule]", 
         StyleBox["u", "TI"]}], ",", 
        RowBox[{
         RowBox[{"\[DifferentialD]", 
          StyleBox["var", "TI"]}], "\[Rule]", " ", 
         RowBox[{
          StyleBox["coef", "TI"], " ", 
          RowBox[{"\[DifferentialD]", 
           StyleBox["u", "TI"]}]}]}]}], "}"}]}], "]"}], Cell[
    "simplify an integral using substitution", "TableText"]}
  }]], "DefinitionBox",
 CellID->2096742444],

Cell["Functions for simplifying integrals.", "Caption",
 CellID->1891092685],

Cell["This loads the package:", "MathCaption",
 CellID->836781195],

Cell[BoxData[
 RowBox[{"<<", "ClassroomUtilities`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->2058623809],

Cell[CellGroupData[{

Cell["Integrating Using Substitution", "Section",
 CellID->501870413],

Cell[TextData[{
 "The ",
 Cell[BoxData[
  ButtonBox["SubstituteIntegrate",
   BaseStyle->"Link",
   ButtonData->"paclet:ClassroomUtilities/ref/SubstituteIntegrate"]], 
  "InlineFormula"],
 " function is intended for use where the method of substitution would be \
appropriate. The student is presented with the integral to evaluate and then \
asked to find appropriate substitutions to simplify the integral. The teacher \
would then enter the substitutions into the ",
 Cell[BoxData[
  ButtonBox["SubstituteIntegrate",
   BaseStyle->"Link",
   ButtonData->"paclet:ClassroomUtilities/ref/SubstituteIntegrate"]], 
  "InlineFormula"],
 " function and evaluate to obtain an unevaluated integral simpler in form \
than the original. The students could then be asked for the result of \
performing the simpler integration. "
}], "Text",
 CellID->1767749522],

Cell[TextData[{
 "Evaluate the integral ",
 Cell[BoxData[
  RowBox[{"\[Integral]", 
   RowBox[{
    FractionBox[
     RowBox[{"Cos", "[", 
      SqrtBox["x"], "]"}], 
     SqrtBox["x"]], 
    RowBox[{"\[DifferentialD]", "x"}]}]}]]],
 "."
}], "Text",
 CellID->129960009],

Cell["Putting this into the SubstituteIntegrate function.", "MathCaption",
 CellID->281161397],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SubstituteIntegrate", "[", 
  RowBox[{
   FractionBox[
    RowBox[{"Cos", "[", 
     SqrtBox["x"], "]"}], 
    SqrtBox["x"]], ",", "x", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SqrtBox["x"], "\[Rule]", " ", "u"}], ",", 
     RowBox[{
      RowBox[{"\[DifferentialD]", "x"}], "\[Rule]", " ", 
      RowBox[{"2", " ", 
       SqrtBox["x"], 
       RowBox[{"\[DifferentialD]", "u"}]}]}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[80]:=",
 CellID->1015575979],

Cell[BoxData[
 RowBox[{"\[Integral]", 
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{"Cos", "[", "u", "]"}]}], 
   RowBox[{"\[DifferentialD]", "u"}]}]}]], "Output",
 ImageSize->{86, 28},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[80]=",
 CellID->1740588641]
}, Open  ]],

Cell["\<\
The students should now recognize the integral as one they can solve.\
\>", "Text",
 CellID->2080891572],

Cell["Copy the result down and evaluate.", "MathCaption",
 CellID->1773459209],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Integral]", 
  RowBox[{"2", " ", 
   RowBox[{"Cos", "[", "u", "]"}], 
   RowBox[{"\[DifferentialD]", "u"}]}]}]], "Input",
 CellLabel->"In[81]:=",
 CellID->1603388950],

Cell[BoxData[
 RowBox[{"2", " ", 
  RowBox[{"Sin", "[", "u", "]"}]}]], "Output",
 ImageSize->{56, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[81]=",
 CellID->1051049853]
}, Open  ]],

Cell["\<\
Finally the original substitution is reversed using rules to get the final \
answer.\
\>", "MathCaption",
 CellID->34775858],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"2", " ", 
   RowBox[{"Sin", "[", "u", "]"}]}], "/.", 
  RowBox[{"{", 
   RowBox[{"u", "\[Rule]", " ", 
    SqrtBox["x"]}], "}"}]}]], "Input",
 CellLabel->"In[82]:=",
 CellID->1845657267],

Cell[BoxData[
 RowBox[{"2", " ", 
  RowBox[{"Sin", "[", 
   SqrtBox["x"], "]"}]}]], "Output",
 ImageSize->{73, 22},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[82]=",
 CellID->1941882796]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Integrating Using Method of Parts", "Section",
 CellID->1227375405],

Cell[TextData[{
 "Using ",
 Cell[BoxData[
  ButtonBox["PartsIntegrate",
   BaseStyle->"Link",
   ButtonData->"paclet:ClassroomUtilities/ref/PartsIntegrate"]], 
  "InlineFormula"],
 " is done in a manner similar to what was done in ",
 Cell[BoxData[
  ButtonBox["SubstituteIntegrate",
   BaseStyle->"Link",
   ButtonData->"paclet:ClassroomUtilities/ref/SubstituteIntegrate"]], 
  "InlineFormula"],
 ". The students are presented with an integral where integration by parts is \
appropriate and asked to identify the parts to use in carrying out the \
integration. As before the result is given as an expression with head Int."
}], "Text",
 CellID->1918002400],

Cell[TextData[{
 "Evaluate the integral ",
 Cell[BoxData[
  RowBox[{"\[Integral]", 
   RowBox[{"x", " ", 
    RowBox[{"Sin", "[", "x", "]"}], 
    RowBox[{"\[DifferentialD]", "x"}]}]}]]],
 "."
}], "Text",
 CellID->2136795330],

Cell[TextData[{
 "Using the ",
 Cell[BoxData[
  ButtonBox["PartsIntegrate",
   BaseStyle->"Link",
   ButtonData->"paclet:ClassroomUtilities/ref/PartsIntegrate"]], 
  "InlineFormula"],
 " function to simplify the integral."
}], "MathCaption",
 CellID->750082529],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PartsIntegrate", "[", 
  RowBox[{
   RowBox[{"x", " ", 
    RowBox[{"Sin", "[", "x", "]"}]}], ",", "x", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"Sin", "[", "x", "]"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[83]:=",
 CellID->908057013],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "x"}], " ", 
   RowBox[{"Cos", "[", "x", "]"}]}], "-", 
  RowBox[{"\[Integral]", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"Cos", "[", "x", "]"}]}], 
    RowBox[{"\[DifferentialD]", "x"}]}]}]}]], "Output",
 ImageSize->{157, 28},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[83]=",
 CellID->1915168913]
}, Open  ]],

Cell["Copy the result down and evaluate.", "MathCaption",
 CellID->2142996167],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "x"}], " ", 
   RowBox[{"Cos", "[", "x", "]"}]}], "-", 
  RowBox[{"\[Integral]", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"Cos", "[", "x", "]"}]}], 
    RowBox[{"\[DifferentialD]", "x"}]}]}]}]], "Input",
 CellLabel->"In[84]:=",
 CellID->1888334932],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "x"}], " ", 
   RowBox[{"Cos", "[", "x", "]"}]}], "+", 
  RowBox[{"Sin", "[", "x", "]"}]}]], "Output",
 ImageSize->{119, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[84]=",
 CellID->1759019555]
}, Open  ]],

Cell["\<\
In some instances integration by parts is used more than once. \
\>", "Text",
 CellID->491250268],

Cell[TextData[{
 "Here we take ",
 StyleBox["u = ",
  FontSlant->"Italic"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "2"], TraditionalForm]]],
 "and ",
 StyleBox["dv = sin(x)",
  FontSlant->"Italic"],
 " which returns an expression which requires a second use of integration by \
parts."
}], "MathCaption",
 CellID->1516115208],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PartsIntegrate", "[", 
  RowBox[{
   RowBox[{
    SuperscriptBox["x", "2"], " ", 
    RowBox[{"Sin", "[", "x", "]"}]}], ",", "x", ",", 
   RowBox[{"{", 
    RowBox[{
     SuperscriptBox["x", "2"], ",", 
     RowBox[{"Sin", "[", "x", "]"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[85]:=",
 CellID->1454829201],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox["x", "2"]}], " ", 
   RowBox[{"Cos", "[", "x", "]"}]}], "-", 
  RowBox[{"\[Integral]", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "2"}], " ", "x", " ", 
     RowBox[{"Cos", "[", "x", "]"}]}], 
    RowBox[{"\[DifferentialD]", "x"}]}]}]}]], "Output",
 ImageSize->{183, 28},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[85]=",
 CellID->1581776283]
}, Open  ]],

Cell[TextData[{
 "We can copy the above result down using \[CommandKey]\[KeyBar]\[ShiftKey]\
\[LeftModified]l\[RightModified] and apply ",
 Cell[BoxData[
  ButtonBox["PartsIntegrate",
   BaseStyle->"Link",
   ButtonData->"paclet:ClassroomUtilities/ref/PartsIntegrate"]], 
  "InlineFormula"],
 " to the integral."
}], "MathCaption",
 CellID->484536344],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox["x", "2"]}], " ", 
   RowBox[{"Cos", "[", "x", "]"}]}], "-", 
  RowBox[{"PartsIntegrate", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "2"}], " ", "x", " ", 
     RowBox[{"Cos", "[", "x", "]"}]}], ",", "x", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "2"}], " ", "x"}], ",", 
      RowBox[{"Cos", "[", "x", "]"}]}], "}"}]}], "]"}]}]], "Input",
 CellLabel->"In[86]:=",
 CellID->344344184],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox["x", "2"]}], " ", 
   RowBox[{"Cos", "[", "x", "]"}]}], "+", 
  RowBox[{"\[Integral]", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "2"}], " ", 
     RowBox[{"Sin", "[", "x", "]"}]}], 
    RowBox[{"\[DifferentialD]", "x"}]}]}], "+", 
  RowBox[{"2", " ", "x", " ", 
   RowBox[{"Sin", "[", "x", "]"}]}]}]], "Output",
 ImageSize->{248, 28},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[86]=",
 CellID->1213922168]
}, Open  ]],

Cell["Copy the results down and evaluate.", "MathCaption",
 CellID->694331832],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox["x", "2"]}], " ", 
   RowBox[{"Cos", "[", "x", "]"}]}], "+", 
  RowBox[{"\[Integral]", 
   RowBox[{
    RowBox[{"-", "2"}], " ", 
    RowBox[{"Sin", "[", "x", "]"}], 
    RowBox[{"\[DifferentialD]", "x"}]}]}], "+", 
  RowBox[{"2", " ", "x", " ", 
   RowBox[{"Sin", "[", "x", "]"}]}]}]], "Input",
 CellLabel->"In[87]:=",
 CellID->208962626],

Cell[BoxData[
 RowBox[{
  RowBox[{"2", " ", 
   RowBox[{"Cos", "[", "x", "]"}]}], "-", 
  RowBox[{
   SuperscriptBox["x", "2"], " ", 
   RowBox[{"Cos", "[", "x", "]"}]}], "+", 
  RowBox[{"2", " ", "x", " ", 
   RowBox[{"Sin", "[", "x", "]"}]}]}]], "Output",
 ImageSize->{202, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[87]=",
 CellID->1295231076]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Substitution Followed by Integration by Parts", "Section",
 CellID->1349967667],

Cell[TextData[{
 "In many instances several methods need to be used to simplify an integral \
to a recognizable form. ",
 Cell[BoxData[
  ButtonBox["SubstituteIntegrate",
   BaseStyle->"Link",
   ButtonData->"paclet:ClassroomUtilities/ref/SubstituteIntegrate"]], 
  "InlineFormula"],
 " can be combined with ",
 Cell[BoxData[
  ButtonBox["PartsIntegrate",
   BaseStyle->"Link",
   ButtonData->"paclet:ClassroomUtilities/ref/PartsIntegrate"]], 
  "InlineFormula"],
 " to solve such integrals."
}], "Text",
 CellID->741848714],

Cell[TextData[{
 "Evaluate the integral ",
 Cell[BoxData[
  RowBox[{"\[Integral]", 
   RowBox[{
    RowBox[{"Sin", "[", 
     SqrtBox["x"], "]"}], 
    RowBox[{"\[DifferentialD]", "x"}]}]}]]],
 ".",
 Cell[BoxData[
  ButtonBox["SubstituteIntegrate",
   BaseStyle->"Link",
   ButtonData->"paclet:ClassroomUtilities/ref/SIntegrate"]]]
}], "Text",
 CellID->2108765363],

Cell["We begin using substitution.", "MathCaption",
 CellID->2096829500],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SubstituteIntegrate", "[", 
  RowBox[{
   RowBox[{"Sin", "[", 
    SqrtBox["x"], "]"}], ",", "x", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"x", "\[Rule]", " ", 
      SuperscriptBox["r", "2"]}], ",", 
     RowBox[{
      RowBox[{"\[DifferentialD]", "x"}], "\[Rule]", " ", 
      RowBox[{"2", " ", "r", " ", 
       RowBox[{"\[DifferentialD]", "r"}]}]}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[88]:=",
 CellID->1206759230],

Cell[BoxData[
 RowBox[{"\[Integral]", 
  RowBox[{
   RowBox[{"2", " ", "r", " ", 
    RowBox[{"Sin", "[", 
     SqrtBox[
      SuperscriptBox["r", "2"]], "]"}]}], 
   RowBox[{"\[DifferentialD]", "r"}]}]}]], "Output",
 ImageSize->{120, 31},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[88]=",
 CellID->2054448340]
}, Open  ]],

Cell["Next we simplify the integrand assuming r>0.", "MathCaption",
 CellID->2009079237],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"2", " ", "r", " ", 
    RowBox[{"Sin", "[", 
     SqrtBox[
      SuperscriptBox["r", "2"]], "]"}]}], ",", 
   RowBox[{"r", ">", "0"}]}], "]"}]], "Input",
 CellLabel->"In[90]:=",
 CellID->1709354478],

Cell[BoxData[
 RowBox[{"2", " ", "r", " ", 
  RowBox[{"Sin", "[", "r", "]"}]}]], "Output",
 ImageSize->{66, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[90]=",
 CellID->1887103767]
}, Open  ]],

Cell[TextData[{
 "Now using ",
 Cell[BoxData[
  ButtonBox["PartsIntegrate",
   BaseStyle->"Link",
   ButtonData->"paclet:ClassroomUtilities/ref/PartsIntegrate"]], 
  "InlineFormula"],
 " gets us to an integral we recognize."
}], "MathCaption",
 CellID->405308593],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PartsIntegrate", "[", 
  RowBox[{
   RowBox[{"2", " ", "r", " ", 
    RowBox[{"Sin", "[", "r", "]"}]}], ",", "r", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"2", " ", "r"}], ",", 
     RowBox[{"Sin", "[", "r", "]"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[91]:=",
 CellID->203394267],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "2"}], " ", "r", " ", 
   RowBox[{"Cos", "[", "r", "]"}]}], "-", 
  RowBox[{"\[Integral]", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "2"}], " ", 
     RowBox[{"Cos", "[", "r", "]"}]}], 
    RowBox[{"\[DifferentialD]", "r"}]}]}]}]], "Output",
 ImageSize->{177, 28},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[91]=",
 CellID->367029426]
}, Open  ]],

Cell["Again, copy the results down and evaluate.", "MathCaption",
 CellID->4682779],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "2"}], " ", "r", " ", 
   RowBox[{"Cos", "[", "r", "]"}]}], "-", 
  RowBox[{"\[Integral]", 
   RowBox[{
    RowBox[{"-", "2"}], " ", 
    RowBox[{"Cos", "[", "r", "]"}], 
    RowBox[{"\[DifferentialD]", "r"}]}]}]}]], "Input",
 CellLabel->"In[93]:=",
 CellID->1394055361],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "2"}], " ", "r", " ", 
   RowBox[{"Cos", "[", "r", "]"}]}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"Sin", "[", "r", "]"}]}]}]], "Output",
 ImageSize->{139, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[93]=",
 CellID->1888703088]
}, Open  ]],

Cell["Now we need to reverse the substitution.", "MathCaption",
 CellID->1477013709],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"-", "2"}], " ", "r", " ", 
    RowBox[{"Cos", "[", "r", "]"}]}], "+", 
   RowBox[{"2", " ", 
    RowBox[{"Sin", "[", "r", "]"}]}]}], "/.", 
  RowBox[{"r", "\[Rule]", " ", 
   SqrtBox["x"]}]}]], "Input",
 CellLabel->"In[94]:=",
 CellID->1405931490],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "2"}], " ", 
   SqrtBox["x"], " ", 
   RowBox[{"Cos", "[", 
    SqrtBox["x"], "]"}]}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"Sin", "[", 
    SqrtBox["x"], "]"}]}]}]], "Output",
 ImageSize->{190, 22},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[94]=",
 CellID->727984489]
}, Open  ]],

Cell[TextData[{
 "Checking the result using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s ",
 Cell[BoxData[
  ButtonBox["Integrate",
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Integrate"]], "InlineFormula"],
 " function."
}], "MathCaption",
 CellID->1578536063],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Integral]", 
  RowBox[{
   RowBox[{"Sin", "[", 
    SqrtBox["x"], "]"}], 
   RowBox[{"\[DifferentialD]", "x"}]}]}]], "Input",
 CellLabel->"In[34]:=",
 CellID->442635803],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "2"}], " ", 
   SqrtBox["x"], " ", 
   RowBox[{"Cos", "[", 
    SqrtBox["x"], "]"}]}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"Sin", "[", 
    SqrtBox["x"], "]"}]}]}]], "Output",
 ImageSize->{190, 22},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[34]=",
 CellID->496587813]
}, Open  ]],

Cell[TextData[{
 "The purpose of the functions in this package is to encourage students to \
recognize the steps involved in obtaining a solution to an integral and not \
just rely on the magic box of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s Integrate function returning an answer with little or no understanding \
of how the solution was found."
}], "Text",
 CellID->2027282899]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["Classroom Utilities",
 BaseStyle->"Link",
 ButtonData->
  "paclet:ClassroomUtilities/guide/ClassroomUtilities"]], "TutorialMoreAbout",
 CellID->2046173039]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED WOLFRAM EDUCATION GROUP COURSES", "TutorialRelatedLinksSection",
 WholeCellGroupOpener->True,
 CellID->525705831],

Cell[TextData[{
 ButtonBox["M50",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[":",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["An",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["Introduction",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["to",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["Mathematica",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["in",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["the",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"],
 ButtonBox["Classroom",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/services/education/courses/m50.html"], None},
  ButtonNote->"http://www.wolfram.com/services/education/courses/m50.html"]
}], "TutorialRelatedLinks",
 CellID->790177859]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Using Integration Techniques Functions - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "TutorialColor", 
  "LinkTrails" -> GridBox[{{
      RowBox[{
        ButtonBox[
        "Classroom Utilities", ButtonData -> 
         "paclet:ClassroomUtilities/guide/ClassroomUtilities", 
         BaseStyle -> {"Link", "DockedLinkTrail"}], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 33, 1.7368624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Tutorial", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "Some of the functions defined in ClassroomUtilities` provide tools for \
having students become familiar with the concepts of modifying an integrand \
to produce an equivalent but recognizable integrand. Functions for \
simplifying integrals. This loads the package:", "synonyms" -> {}, "title" -> 
    "Using Integration Techniques Functions", "titlemodifier" -> "", 
    "windowtitle" -> "Using Integration Techniques Functions", "type" -> 
    "Tutorial", "uri" -> 
    "ClassroomUtilities/tutorial/UsingIntegrationTechniquesFunctions"}, 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> Notebook], 
   Cell[
    StyleData["Output"], CellContext -> Notebook], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[631, 21, 2999, 70, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3655, 95, 75, 1, 51, "Title",
 CellID->509267359],
Cell[3733, 98, 296, 6, 72, "Text",
 CellID->1534169418],
Cell[4032, 106, 1236, 37, 95, "DefinitionBox",
 CellID->2096742444],
Cell[5271, 145, 76, 1, 31, "Caption",
 CellID->1891092685],
Cell[5350, 148, 66, 1, 31, "MathCaption",
 CellID->836781195],
Cell[5419, 151, 107, 3, 28, "Input",
 CellID->2058623809],
Cell[CellGroupData[{
Cell[5551, 158, 69, 1, 51, "Section",
 CellID->501870413],
Cell[5623, 161, 852, 20, 134, "Text",
 CellID->1767749522],
Cell[6478, 183, 269, 12, 58, "Text",
 CellID->129960009],
Cell[6750, 197, 94, 1, 70, "MathCaption",
 CellID->281161397],
Cell[CellGroupData[{
Cell[6869, 202, 496, 17, 70, "Input",
 CellID->1015575979],
Cell[7368, 221, 295, 10, 49, "Output",
 CellID->1740588641]
}, Open  ]],
Cell[7678, 234, 114, 3, 70, "Text",
 CellID->2080891572],
Cell[7795, 239, 78, 1, 70, "MathCaption",
 CellID->1773459209],
Cell[CellGroupData[{
Cell[7898, 244, 192, 6, 70, "Input",
 CellID->1603388950],
Cell[8093, 252, 212, 7, 36, "Output",
 CellID->1051049853]
}, Open  ]],
Cell[8320, 262, 134, 4, 70, "MathCaption",
 CellID->34775858],
Cell[CellGroupData[{
Cell[8479, 270, 221, 8, 70, "Input",
 CellID->1845657267],
Cell[8703, 280, 225, 8, 43, "Output",
 CellID->1941882796]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[8977, 294, 73, 1, 70, "Section",
 CellID->1227375405],
Cell[9053, 297, 658, 17, 70, "Text",
 CellID->1918002400],
Cell[9714, 316, 225, 9, 70, "Text",
 CellID->2136795330],
Cell[9942, 327, 261, 9, 70, "MathCaption",
 CellID->750082529],
Cell[CellGroupData[{
Cell[10228, 340, 282, 9, 70, "Input",
 CellID->908057013],
Cell[10513, 351, 391, 14, 49, "Output",
 CellID->1915168913]
}, Open  ]],
Cell[10919, 368, 78, 1, 70, "MathCaption",
 CellID->2142996167],
Cell[CellGroupData[{
Cell[11022, 373, 302, 11, 70, "Input",
 CellID->1888334932],
Cell[11327, 386, 286, 10, 36, "Output",
 CellID->1759019555]
}, Open  ]],
Cell[11628, 399, 107, 3, 70, "Text",
 CellID->491250268],
Cell[11738, 404, 335, 13, 70, "MathCaption",
 CellID->1516115208],
Cell[CellGroupData[{
Cell[12098, 421, 336, 11, 70, "Input",
 CellID->1454829201],
Cell[12437, 434, 453, 16, 49, "Output",
 CellID->1581776283]
}, Open  ]],
Cell[12905, 453, 351, 10, 70, "MathCaption",
 CellID->484536344],
Cell[CellGroupData[{
Cell[13281, 467, 491, 17, 70, "Input",
 CellID->344344184],
Cell[13775, 486, 517, 18, 49, "Output",
 CellID->1213922168]
}, Open  ]],
Cell[14307, 507, 78, 1, 70, "MathCaption",
 CellID->694331832],
Cell[CellGroupData[{
Cell[14410, 512, 410, 14, 70, "Input",
 CellID->208962626],
Cell[14823, 528, 390, 13, 36, "Output",
 CellID->1295231076]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[15262, 547, 85, 1, 70, "Section",
 CellID->1349967667],
Cell[15350, 550, 524, 16, 70, "Text",
 CellID->741848714],
Cell[15877, 568, 364, 14, 70, "Text",
 CellID->2108765363],
Cell[16244, 584, 72, 1, 70, "MathCaption",
 CellID->2096829500],
Cell[CellGroupData[{
Cell[16341, 589, 458, 14, 70, "Input",
 CellID->1206759230],
Cell[16802, 605, 349, 12, 52, "Output",
 CellID->2054448340]
}, Open  ]],
Cell[17166, 620, 88, 1, 70, "MathCaption",
 CellID->2009079237],
Cell[CellGroupData[{
Cell[17279, 625, 262, 9, 70, "Input",
 CellID->1709354478],
Cell[17544, 636, 222, 7, 36, "Output",
 CellID->1887103767]
}, Open  ]],
Cell[17781, 646, 263, 9, 70, "MathCaption",
 CellID->405308593],
Cell[CellGroupData[{
Cell[18069, 659, 318, 10, 70, "Input",
 CellID->203394267],
Cell[18390, 671, 426, 15, 49, "Output",
 CellID->367029426]
}, Open  ]],
Cell[18831, 689, 83, 1, 70, "MathCaption",
 CellID->4682779],
Cell[CellGroupData[{
Cell[18939, 694, 321, 11, 70, "Input",
 CellID->1394055361],
Cell[19263, 707, 320, 11, 36, "Output",
 CellID->1888703088]
}, Open  ]],
Cell[19598, 721, 84, 1, 70, "MathCaption",
 CellID->1477013709],
Cell[CellGroupData[{
Cell[19707, 726, 307, 11, 70, "Input",
 CellID->1405931490],
Cell[20017, 739, 360, 14, 43, "Output",
 CellID->727984489]
}, Open  ]],
Cell[20392, 756, 278, 11, 70, "MathCaption",
 CellID->1578536063],
Cell[CellGroupData[{
Cell[20695, 771, 195, 7, 70, "Input",
 CellID->442635803],
Cell[20893, 780, 360, 14, 43, "Output",
 CellID->496587813]
}, Open  ]],
Cell[21268, 797, 395, 9, 70, "Text",
 CellID->2027282899]
}, Open  ]],
Cell[CellGroupData[{
Cell[21700, 811, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[21768, 814, 180, 4, 70, "TutorialMoreAbout",
 CellID->2046173039]
}, Open  ]],
Cell[CellGroupData[{
Cell[21985, 823, 127, 2, 70, "TutorialRelatedLinksSection",
 CellID->525705831],
Cell[22115, 827, 3506, 82, 70, "TutorialRelatedLinks",
 CellID->790177859]
}, Open  ]]
}, Open  ]],
Cell[25648, 913, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

