(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    287544,       5353]
NotebookOptionsPosition[    279713,       5105]
NotebookOutlinePosition[    282857,       5186]
CellTagsIndexPosition[    282776,       5181]
WindowTitle->Visualizing Statistical Concepts - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Functions \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"CreatePopulation\"\>":>
       Documentation`HelpLookup[
       "paclet:Classroom Utilities/ref/CreatePopulation"], \
"\<\"KSeparateSamples\"\>":>Documentation`HelpLookup[
       "paclet:Classroom Utilities/ref/KSeparateSamples"], "\<\"SRSChart\"\>":>
       Documentation`HelpLookup["paclet:Classroom Utilities/ref/SRSChart"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"TutorialFunction"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Classroom \
Utilities/tutorial/VisualizingStatisticalConcepts\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
             "Classroom Utilities/tutorial/VisualizingStatisticalConcepts"]}, 
            Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Classroom \
Utilities/tutorial/VisualizingStatisticalConcepts.html"], StandardForm]], 
              "Input", TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Classroom Utilities/tutorial/VisualizingStatisticalConcepts", 
             ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell["Visualizing Statistical Concepts", "Title",
 CellID->509267359],

Cell["\<\
The statistical functions defined in the ClassroomUtilities` context are \
intended for use in visualizing various concepts that students often have \
difficulty in grasping. In this tutorial we introduce the concepts of \
population, simple random sample and event.\
\>", "Text",
 CellID->1534169418],

Cell["This loads the package.", "MathCaption",
 CellID->1374342004],

Cell[BoxData[
 RowBox[{"<<", "ClassroomUtilities`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->231954096],

Cell[CellGroupData[{

Cell["Populations and Sampling", "Section",
 CellID->769672167],

Cell[BoxData[GridBox[{
   {
    RowBox[{
     ButtonBox["SRSChart",
      BaseStyle->"Link",
      ButtonData->"paclet:Classroom Utilities/ref/SRSChart"], "[", 
     RowBox[{
      StyleBox["pop", "TI"], ",", " ", 
      RowBox[{"{", 
       RowBox[{
        StyleBox["sz", "TI"], ",", 
        StyleBox["n", "TI"]}], "}"}]}], "]"}], Cell[TextData[{
     "visualize ",
     Cell[BoxData[
      StyleBox["n", "TI"]], "InlineFormula"],
     " samples of size ",
     Cell[BoxData[
      StyleBox["sz", "TI"]], "InlineFormula"]
    }], "TableText"]},
   {
    RowBox[{
     ButtonBox["CreatePopulation",
      BaseStyle->"Link",
      ButtonData->"paclet:Classroom Utilities/ref/CreatePopulation"], "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        SubscriptBox[
         StyleBox["gr", "TI"], 
         StyleBox["i", "TI"]], ",", 
        StyleBox["\[Ellipsis]", "TR"]}], "}"}], ",", 
      StyleBox["n", "TI"]}], "]"}], Cell["\<\
format a collection of graphics or images for use in SRSChart\
\>", "TableText"]},
   {
    RowBox[{
     ButtonBox["KSeparateSamples",
      BaseStyle->"Link",
      ButtonData->"paclet:Classroom Utilities/ref/KSeparateSamples"], "[", 
     RowBox[{
      StyleBox["data", "TI"], ",", 
      RowBox[{"{", 
       RowBox[{
        StyleBox["sz", "TI"], ",", 
        StyleBox["n", "TI"]}], "}"}]}], "]"}], Cell[TextData[{
     "get ",
     Cell[BoxData[
      StyleBox["n", "TI"]], "InlineFormula"],
     " simple random samples of size ",
     Cell[BoxData[
      StyleBox["sz", "TI"]], "InlineFormula"],
     " from ",
     Cell[BoxData[
      StyleBox["data", "TI"]], "InlineFormula"]
    }], "TableText"]}
  }]], "DefinitionBox",
 CellID->2096742444],

Cell["Functions for working with populations and samples.", "Caption",
 CellID->1891092685],

Cell["\<\
The SRSChart, simple random sample chart, can be used to illustrate the \
concept of a population and can include a random sample of a specified size.\
\>", "Text",
 CellID->598110709],

Cell["A population with 19 members and a sample of size 4.", "MathCaption",
 CellID->836781195],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GraphicsRow", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"SRSChart", "[", "19", "]"}], ",", 
     RowBox[{"SRSChart", "[", 
      RowBox[{"19", ",", 
       RowBox[{"{", "4", "}"}]}], "]"}]}], "}"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", " ", "Medium"}]}], "]"}]], "Input",
 CellLabel->"In[13]:=",
 CellID->2058623809],

Cell[BoxData[
 GraphicsBox[{{}, {InsetBox[
     GraphicsBox[{
       {GrayLevel[0.75], DiskBox[{0, 0}, 4.438380194395248]}, 
       {RGBColor[0.15, 0.15, 0.7], PolygonBox[CompressedData["
1:eJxdlT1oU1EYhi9NVGiGIP4NQoMQxD86CaKtnIODVXGpk2C7FAQnFZdKEXIH
C24WJ61OpkLSW6SmENM0cI6dnNxEER3qpAS1QyvYFDXkfM+FcwJ3ePm+vM85
373vOYcmbl25nomiaE/32dV9+rrP42O1ZufkTzN3+2z/h3JOz1+++m2q3jEX
1qb2vhzJ68xq8mfoaNZSz4oef7i6eWQsqxPpbwxMt0cbkX4mftekHvq3bv7d
t3Xqu0JH8oO37Opp/4poeK57sQWvIX7wQn9Zrw72p+A9detL+5+IhleVfngy
Dw0v9H8+ND26srkt8/unSqfPDX5+m7VOZ/Tg+zdLn/bn7EWpo5d7/usqlv5e
ee2rKotfXeqhf/zr3aOdG0upduttG3i18eOlM5NbBh4a3oL0w5sRP3ihv/xf
od37byt4D1y/gndXNLyK9MN7JX7wQn+Zj0736+aj4ck8Uh4ansxfw8MPXujf
Wfw4ce/GDtsrD3xR7nvJ2UJP/1AnNtr14b7dtih19EGnTdBvtsXvsNTRBfm/
fG/G53UMPPIHj/zBS/x+Q/7gzYqGJ3lS8CbvzPXfz88MwyN/Rb/fwKslv/Pn
D1Ra8MgfvKbogr9eHexPwSN/RX8eCl7V7zfkDx7+Bf99BLychifz1/Dkfevg
/dFv8IOHhkce3fmwbkp+fg35fi119Ij73kzsnw+WvF+SeugveUp11c+vId/w
0PAW/PPBknd4ceBPHtGJn1/6FTw0vIp/PljyDi/0J4/o2M8v80h5aHgl/3yw
+MEL/cnfmDvP7bx/n1nyR538ld15bxP/vrTk74XUZwN/8ofm/oMneUr75T4z
8Lj/4JE/eM3An3wE++M+Y71pv9zXCl7Vvy+Zh4YX+v8HQLtDdg==
         "]]}}], {192., -189.}, ImageScaled[{0.5, 0.5}], {360., 360.}], 
    InsetBox[
     GraphicsBox[{
       {GrayLevel[0.75], DiskBox[{0, 0}, 4.438380194395248]}, 
       {RGBColor[0.15, 0.15, 0.7], PolygonBox[CompressedData["
1:eJxdlT1oU1EYhkMTFZohCP4MQoMg4h+dBNFWzsHBqrjUSbBdCoKTikulCMlg
wc3i5N9kKqS9IjWFmLaBc+zk5CaK6FAnJfgztIJNsYac9yncE7jDy/fd9z3n
POfL3T9249LVbCaTKXSeHZ2np/O05z6O3bm2zfdNtoYbfV/M7MXL3ybqeV/s
6h/m2GqrPtiz0x9QHb0vaBf1uw35HVQdXdT7zwYmh5fWNlyjG/jblU6e6f/8
NudXJna9HFr56vrfv5n/tCfvX6uOHuo2bLqy+s91ddZX5HdB9dgfHfo3TSn9
vsX/vOrohfC+KafXZ8irqx77PzpSW2wf/+lG7i+vHRrJ+XA+ba0n47PLyd+B
wzlPPSdduXm690Ml7xP1B7+Cfyq/56o/jvzJS/NrO/iRBz/y4Jek+x158CMP
fg+lp8N67Gx6vZY86uSNhvXaJH0elrwrqsf+5V/vHmxfnd8635nu+y0Hv9ro
0dKp8XUHPzT8XqgfflPyg1/sv3T93+71E98N55vRD36NUN86/6Y0/EL3XBN+
6jfwW4z8yYPf+K3p3ruFqUH4Lcgffup38Kslfwpn91ab8CMPfuTBT+s1nC/7
gx951MmDH/uDH3nwi/3FwzCPgX/LMO86f8O8o5n3qvqZ91fyY95jfzQ8k/T7
9p78uS+3pbkv1fT6DHncl9hf99tG82fg9yTcZwM/zauB30y632l+LPzwL6bn
10bzZ+BHHnXy4Ece/MiDX+yv//tof3nL/vT/btmfvic2+j7Q7/Bjf2j29x/D
6Vn0
         "]], 
        {EdgeForm[{RGBColor[
         0.996078431372549, 0.3607843137254902, 0.027450980392156862`], 
         Thickness[
          NCache[
           Rational[1, 50], 0.02]]}], PolygonBox[CompressedData["
1:eJxTTMoPSmVmYGBgAWJ2IGYCsQ+t/mmtyeIQ03/oq0YMy4FVPhEvqrb9tt8h
1/o6cAfDgTlam3b9NnlnD5OfAeUvLrDlur6Y+8BKqHqPh1Ui69z5DzBDzVsK
lUc3X+/qwc23xLgdIOr/2zdYOundOcEC5TM7LLJuDdz99Y+9J1Qext8Jds8H
+3qoerD0w0f2MPO2QeXRzYfxIf75sL8BVf9+mPnbofIwvjtE//56VPfB3e8F
lUc3v+L9uUlsXzbvh/Eh4fN6P8y+TbHa9Vblv/bD7IPxYfatgaqH2TcBah7M
vgY08wE1W+sp
          "]]}}}], {576., -189.}, ImageScaled[{0.5, 0.5}], {360., 360.}]}, {}},
  ContentSelectable->True,
  ImageSize->Medium,
  PlotRangePadding->{6, 5}]], "Output",
 ImageSize->{364, 188},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[13]=",
 CellID->2109354559]
}, Open  ]],

Cell["\<\
You can add images to the members of the population to help illustrate what \
the population represents.\
\>", "Text",
 CellID->1262028437],

Cell["A population with happy faces.", "MathCaption",
 CellID->215959240],

Cell[BoxData[
 RowBox[{
  RowBox[{"pop", " ", "=", " ", 
   RowBox[{"CreatePopulation", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Happy", "[", "]"}], ",", 
       RowBox[{"Sleepy", "[", "]"}], ",", 
       RowBox[{"Grumpy", "[", "]"}], ",", 
       RowBox[{"Oh", "[", "]"}]}], "}"}], ",", "19"}], "]"}]}], 
  ";"}]], "Input",
 CellLabel->"In[14]:=",
 CellID->379583250],

Cell["Put the population into an SRSChart.", "MathCaption",
 CellID->1629251160],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GraphicsRow", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"SRSChart", "[", "pop", "]"}], ",", 
     RowBox[{"SRSChart", "[", 
      RowBox[{"pop", ",", 
       RowBox[{"{", "4", "}"}]}], "]"}]}], "}"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", " ", "Medium"}]}], "]"}]], "Input",
 CellLabel->"In[15]:=",
 CellID->260263223],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztXS2U8rwS5p5rVlbWVlZWYpHISuSHuecgrkBikUgkFonEXWwlEolFIpHc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    "], {{0, 188}, {360, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{360, 188},
  PlotRange->{{0, 360}, {0, 188}}]], "Output",
 ImageSize->{364, 188},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[15]=",
 CellID->87502714]
}, Open  ]],

Cell[TextData[{
 "For some situations it might be more appropriate to number the members of \
the population. The ",
 Cell[BoxData[
  ButtonBox["CreatePopulation",
   BaseStyle->"Link",
   ButtonData->"paclet:Classroom Utilities/ref/CreatePopulation"]], 
  "InlineFormula"],
 " function will work with any graphics primitive, so for this we use Text."
}], "Text",
 CellID->1478395671],

Cell["A numbered population.", "MathCaption",
 CellID->1010246707],

Cell[BoxData[
 RowBox[{
  RowBox[{"numPop", "=", " ", 
   RowBox[{"CreatePopulation", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Text", "[", 
        RowBox[{"Style", "[", 
         RowBox[{"#", ",", 
          RowBox[{"FontColor", "\[Rule]", "Yellow"}], ",", "24", ",", 
          "Bold"}], "]"}], "]"}], "&"}], "/@", 
      RowBox[{"Range", "[", "13", "]"}]}], ",", "7"}], "]"}]}], 
  ";"}]], "Input",
 CellLabel->"In[16]:=",
 CellID->1210691367],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SRSChart", "[", 
  RowBox[{"numPop", ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "2"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[17]:=",
 CellID->682249844],

Cell[BoxData[
 GraphicsBox[{
   {GrayLevel[0.75], DiskBox[{0, 0}, 2.6630281166371486`]}, 
   {RGBColor[0.15, 0.15, 0.7], 
    PolygonBox[{{{-0.9439676901250379, 
     0.45499999999999996`}, {-1.7320508075688772`, 0.91}, {-2.520133925012716,
      0.4550000000000003}, {-2.5201339250127166`, -0.4549999999999998}, \
{-1.7320508075688774`, -0.91}, {-0.9439676901250382, -0.4550000000000004}, \
{-0.943967690125038, 0.455}}}], {
     {EdgeForm[{RGBColor[
      0.996078431372549, 0.3607843137254902, 0.027450980392156862`], Thickness[
       NCache[
        Rational[1, 30], 0.03333333333333333]]}], PolygonBox[CompressedData["
1:eJxTTMoPSmVmYGAAYXYgZgLiTbHa9Vblv+w9HlaJrHP/b7/aJ+JF1bbXUD6z
Q8f7c5PYvmy294TKV0D5O+VaXwfu+GC/AqoeLP3wkf1GqHnboPLo5sP4OyDy
+1ej6t/fADV/O1QexneH6N+/AtV9B2D2eUHl0c2Hunc/jL8Son8/zD6o+v0w
+2B8mH1roOph9k2AmgezrwHNfADPXbso
       "]]}, 
     {EdgeForm[{RGBColor[
      0.996078431372549, 0.9882352941176471, 0.03529411764705882], Thickness[
       NCache[
        Rational[1, 30], 0.03333333333333333]]}], PolygonBox[CompressedData["
1:eJxTTMoPSmVmYGAAYXYgZgLihvfnJrF92bzf42GVyDr3//YrfSJeVG17DeUz
O2yK1a63Kv+13xMqD+PvlGt9Hbjjg/0aqHqw9MNH9hOg5m2DyqObz3Jo9U9r
TRYHsLTcXftVYP2/7eXB/Lf2s7U27fpt8s5eBSo/E8qXhvD3r0RVv58Zap4a
VB5mvjxU/+68f6K/zF/aw+wrL1rC1cI/wQZm306I/H4VVPX7YfZtWv2N3018
xR6YfTug5sHs2wXlw+wDACGWs0I=
       "]]}}, {InsetBox[
      GraphicsBox[InsetBox[
        StyleBox["1",
         StripOnInput->False,
         FontSize->24,
         FontWeight->Bold,
         FontColor->RGBColor[1, 1, 0]], {0, 0}]], 
      NCache[{Rational[-1, 2] 3^Rational[1, 2], Rational[
        3, 2]}, {-0.8660254037844386, 1.5}], Center, 1.2], InsetBox[
      GraphicsBox[InsetBox[
        StyleBox["2",
         StripOnInput->False,
         FontSize->24,
         FontWeight->Bold,
         FontColor->RGBColor[1, 1, 0]], {0, 0}]], 
      NCache[{Rational[1, 2] 3^Rational[1, 2], Rational[3, 2]}, {
       0.8660254037844386, 1.5}], Center, 1.2], InsetBox[
      GraphicsBox[InsetBox[
        StyleBox["3",
         StripOnInput->False,
         FontSize->24,
         FontWeight->Bold,
         FontColor->RGBColor[1, 1, 0]], {0, 0}]], 
      NCache[{-3^Rational[1, 2], 0}, {-1.7320508075688772`, 0}], Center, 1.2],
      InsetBox[
      GraphicsBox[InsetBox[
        StyleBox["4",
         StripOnInput->False,
         FontSize->24,
         FontWeight->Bold,
         FontColor->RGBColor[1, 1, 0]], {0, 0}]], {0, 0}, Center, 1.2], 
     InsetBox[
      GraphicsBox[InsetBox[
        StyleBox["5",
         StripOnInput->False,
         FontSize->24,
         FontWeight->Bold,
         FontColor->RGBColor[1, 1, 0]], {0, 0}]], 
      NCache[{3^Rational[1, 2], 0}, {1.7320508075688772`, 0}], Center, 1.2], 
     InsetBox[
      GraphicsBox[InsetBox[
        StyleBox["6",
         StripOnInput->False,
         FontSize->24,
         FontWeight->Bold,
         FontColor->RGBColor[1, 1, 0]], {0, 0}]], 
      NCache[{Rational[-1, 2] 3^Rational[1, 2], 
        Rational[-3, 2]}, {-0.8660254037844386, -1.5}], Center, 1.2], InsetBox[
      GraphicsBox[InsetBox[
        StyleBox["7",
         StripOnInput->False,
         FontSize->24,
         FontWeight->Bold,
         FontColor->RGBColor[1, 1, 0]], {0, 0}]], 
      NCache[{Rational[1, 2] 3^Rational[1, 2], Rational[-3, 2]}, {
       0.8660254037844386, -1.5}], Center, 1.2]}}}]], "Output",
 ImageSize->{184, 170},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[17]=",
 CellID->1157936775]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["A Classroom Example", "Section",
 CellID->946568311],

Cell["\<\
Sally has a pond in her garden that contains a number of fish of two \
different colors. She is not sure how many fish are in the pond or what \
fraction is yellow and what fraction is orange.\
\>", "Text",
 CellID->414746001],

Cell["\<\
For this example we will assume a population with 19 members approximately \
1/3 of which are orange and 2/3 are yellow. We begin by introducing graphics \
for the fish.\
\>", "Text",
 CellID->2051604292],

Cell["Define the fish graphics.", "MathCaption",
 CellID->157567382],

Cell[BoxData[
 RowBox[{
  RowBox[{"yellowFish", "=", " ", 
   GraphicsBox[{
     {RGBColor[1, 1, 0], PolygonBox[CompressedData["
1:eJxllX00FPgax0fs8RKuSttGV+hlk1hLRaiHtGu3yFzvIiYJeR11l4hdrVqJ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       "]]}, 
     {RGBColor[0.9, 0.9, 0.4], Opacity[0.9], PolygonBox[{CompressedData["
1:eJxlkmtME3YUxSvCBI2kcSCTThziDGPK1KEgQv8k1GlWrGQOcCABSlHrA4oG
CxQryxy64dawyESLvByDqQxaRAe2nlYGUkGRKY8C1UofygB59AFIWLd9qPuw
Dzcnv+Qk5+be481O+yzFgUKhBP0z/+pvDEuPIlqJav6kcFAoJwrRRv5671tw
1lWlpwmvEcq9gyujdvEI41yv7Gp9BcrCV1xl+zUSKX+kfLWpAUUF2bsNN1Sk
w2ahBX6ixLuNqZ9m1TwmCXRZnEdeK37ytArYl3uJYqKSGRvRAYqkcsee0H6y
sC1jhjrbCaUtWprsoyWkOt/t/t1uFJ95u4h1WEcM3Vvf4tQ+RXaVw6usv/Tk
4TH1y+LSITBYCZ2964wkJGVz/G6zEbzksa++CzESzwBv98GzL3DpQyn/Wwct
We3k42gL12OfK0VyflU3cZN1xbxv04Lzi/n4ioJ7ZE/YwSs7fh/AjGZb+tfW
RvIePSN6YFEfKGGbq2Z8dsLOisTlUSWlKtj9R3outQZTNAjkOgnn1z6HP4NF
o/+ghz3vxBEHYVLJS9j3ua6qGYgYHkPA4SXx1NOjkB+osYSPTSCAE6c5+WIC
uvW+nrlbp9BX2q+ePGdCO3uV+85ME0Ycw5qzvKzozJ5zkXuYYeNWXl9QN4Oq
azncLwRm+I2fT03mzkHwSMTaJjJDT9+gshrnEc9sWxQqNuPm6ZxH9er/eFYc
eTa32YpTriaD014z7hxKufuRZQoqrW9WxvcmBNdKmJ+nvQKtz085xJuC/b5n
juZvonEmMRcqYRWO65DXa1lyonkcvCRNJp+jxY9xsUu/EYyCfYi5Um9RI/XK
ApnTsmEs09T/2nXsCai1Ix6jeQbY/0/t6NJNb9HD3g/F/fLtN006uFS3y2n7
AWmuy2CTWA9tJn3hnw+K4DV4uWHA3whnH6tL2TsNxLX444obgUYETceI5qLr
yVqvyC2GxTokRrwWfBBShLCT7cX++c8QPL8941lPExy5rRXDo/0o3bV3neiU
ArwvHVtEG9VIKG9xvsBQgnOh+qln3x9vmGeMOZ6b+PCNvy724gRzvg1rZKZs
o/g2IvOCelqOykAtSHJP45T8jzfM7hfTHt9GndedCDfvQih+3vc83r8JZQFP
fNdcLCRlzPQoNV8OfY5keihfSv4GW2TtMw==
        "], CompressedData["
1:eJxTTMoPSmViYGCQBmIQfeiXffvPT8/3H3G4vq8h7ph9no9/xCPGl/u/OTHe
Pthx0V5kYqJofsrL/d1Pp4U5P7puv+tq8aXfx17u1z0n8Ywl9x5cPsHiuoFz
8iP7nncKZcsuvNjf8+nM6cTyp/ZzvAtDb5Y/3//nb9K2lKsv7PnmGC/aYv5s
v0WnY8LTC6/t3Tj6zfoinux3mdAslHbrnb3A+tfib9qe7n+WuPCayfu39hYL
BRjbA57t5ytc03074639LemaRKPQl/tlbmgdfFTw2v7MdPEE+blv9udJuzl+
qHhp33b9K3fZ4ff7E7YaPZz+65n9dffl62qmfdwvJZyktzLqif3sk+a370h8
3r9GgW3XN9YH9p6XTCfnm33Zf/+e6Qvv/Bv2s9cE8l/M+LL/sP7Xz8uULtlP
bm+Nurzny34VHa1HJZOOwvUvmy5x+ov3Kbj5cwqlFnHVn7PfJ7w5jufp+/1J
i9fc9/G5YM905cqmrfve7M/ZvFHo85oL9ksO3XCa0Pxq/zmj7oAc1vNw/zjc
zromef+0vUCpfMMVtRf7NzV7xRzfccI+78DzdsFDz/fvW+nRurH1iD0Ab/vw
nQ==
        "], CompressedData["
1:eJxTTMoPSmViYGCQBGIQ3dAzp9X27YT9n+Qv5cezv7BnmPXa5FLQzP0qad8q
ez69sG/gfhX9rKRzP1tNnYTinpf2DlHuocIuc+0X3Nn2PvD/S3sFoe8Rvy5s
tO/I7TaSTnll39BavrPj/RL7L09Zb7gvB/Lrgt5e/thhfyPwKkvg99f2CsGs
iydZ7dzvsjbzXmHXW/sLxwSzP2qe2L9B9UnzvLNv7X1u7990acml/VUv1Qw5
1ry1PxJVw9Xw8cb+DuP9s5hZ3tqvCNrMHtV4f7/OvRM2n9hf2x9gybt7Nvgh
3L0CL7evZ37+aL/Rq6jq+aue2r+4ueZXTO7j/VuKZ7HemvvEXs44QPK/3aP9
T3bu42fvfAo3L8bqmU1y/HP7hu0OTY+O39qv9tBJ7mDvC7j9BdMmPVUReWl/
ZIrHoumB1/YrvF55NTz8pb3KmS1/tbUv799zTC9wr+lL+4iZH7z/7jkJN++C
1apZCq8O7a9QiZt3buoz+4JCad4Hujv2C3B5XTxl+9yegbNLPvndpv3fZsmJ
TFn03B4A/kDX/g==
        "]}], {
       {RGBColor[1, 0.5, 0], DiskBox[{0.438701, 0.521234}, 0.25]}, 
       {GrayLevel[0], DiskBox[{0.438701, 0.521234}, 0.15]}}}},
    ImageSize->{126.17215140178803`, Automatic}]}], ";", " ", 
  RowBox[{"orangeFish", "=", " ", 
   GraphicsBox[{
     {RGBColor[1., 0.6, 0], PolygonBox[CompressedData["
1:eJxllX00FPgax0fs8RKuSttGV+hlk1hLRaiHtGu3yFzvIiYJeR11l4hdrVqJ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       "]]}, 
     {RGBColor[0.9, 0.9, 0.4], Opacity[0.9], PolygonBox[{CompressedData["
1:eJxlkmtME3YUxSvCBI2kcSCTThziDGPK1KEgQv8k1GlWrGQOcCABSlHrA4oG
CxQryxy64dawyESLvByDqQxaRAe2nlYGUkGRKY8C1UofygB59AFIWLd9qPuw
Dzcnv+Qk5+be481O+yzFgUKhBP0z/+pvDEuPIlqJav6kcFAoJwrRRv5671tw
1lWlpwmvEcq9gyujdvEI41yv7Gp9BcrCV1xl+zUSKX+kfLWpAUUF2bsNN1Sk
w2ahBX6ixLuNqZ9m1TwmCXRZnEdeK37ytArYl3uJYqKSGRvRAYqkcsee0H6y
sC1jhjrbCaUtWprsoyWkOt/t/t1uFJ95u4h1WEcM3Vvf4tQ+RXaVw6usv/Tk
4TH1y+LSITBYCZ2964wkJGVz/G6zEbzksa++CzESzwBv98GzL3DpQyn/Wwct
We3k42gL12OfK0VyflU3cZN1xbxv04Lzi/n4ioJ7ZE/YwSs7fh/AjGZb+tfW
RvIePSN6YFEfKGGbq2Z8dsLOisTlUSWlKtj9R3outQZTNAjkOgnn1z6HP4NF
o/+ghz3vxBEHYVLJS9j3ua6qGYgYHkPA4SXx1NOjkB+osYSPTSCAE6c5+WIC
uvW+nrlbp9BX2q+ePGdCO3uV+85ME0Ycw5qzvKzozJ5zkXuYYeNWXl9QN4Oq
azncLwRm+I2fT03mzkHwSMTaJjJDT9+gshrnEc9sWxQqNuPm6ZxH9er/eFYc
eTa32YpTriaD014z7hxKufuRZQoqrW9WxvcmBNdKmJ+nvQKtz085xJuC/b5n
juZvonEmMRcqYRWO65DXa1lyonkcvCRNJp+jxY9xsUu/EYyCfYi5Um9RI/XK
ApnTsmEs09T/2nXsCai1Ix6jeQbY/0/t6NJNb9HD3g/F/fLtN006uFS3y2n7
AWmuy2CTWA9tJn3hnw+K4DV4uWHA3whnH6tL2TsNxLX444obgUYETceI5qLr
yVqvyC2GxTokRrwWfBBShLCT7cX++c8QPL8941lPExy5rRXDo/0o3bV3neiU
ArwvHVtEG9VIKG9xvsBQgnOh+qln3x9vmGeMOZ6b+PCNvy724gRzvg1rZKZs
o/g2IvOCelqOykAtSHJP45T8jzfM7hfTHt9GndedCDfvQih+3vc83r8JZQFP
fNdcLCRlzPQoNV8OfY5keihfSv4GW2TtMw==
        "], CompressedData["
1:eJxTTMoPSmViYGCQBmIQfeiXffvPT8/3H3G4vq8h7ph9no9/xCPGl/u/OTHe
Pthx0V5kYqJofsrL/d1Pp4U5P7puv+tq8aXfx17u1z0n8Ywl9x5cPsHiuoFz
8iP7nncKZcsuvNjf8+nM6cTyp/ZzvAtDb5Y/3//nb9K2lKsv7PnmGC/aYv5s
v0WnY8LTC6/t3Tj6zfoinux3mdAslHbrnb3A+tfib9qe7n+WuPCayfu39hYL
BRjbA57t5ytc03074639LemaRKPQl/tlbmgdfFTw2v7MdPEE+blv9udJuzl+
qHhp33b9K3fZ4ff7E7YaPZz+65n9dffl62qmfdwvJZyktzLqif3sk+a370h8
3r9GgW3XN9YH9p6XTCfnm33Zf/+e6Qvv/Bv2s9cE8l/M+LL/sP7Xz8uULtlP
bm+Nurzny34VHa1HJZOOwvUvmy5x+ov3Kbj5cwqlFnHVn7PfJ7w5jufp+/1J
i9fc9/G5YM905cqmrfve7M/ZvFHo85oL9ksO3XCa0Pxq/zmj7oAc1vNw/zjc
zromef+0vUCpfMMVtRf7NzV7xRzfccI+78DzdsFDz/fvW+nRurH1iD0Ab/vw
nQ==
        "], CompressedData["
1:eJxTTMoPSmViYGCQBGIQ3dAzp9X27YT9n+Qv5cezv7BnmPXa5FLQzP0qad8q
ez69sG/gfhX9rKRzP1tNnYTinpf2DlHuocIuc+0X3Nn2PvD/S3sFoe8Rvy5s
tO/I7TaSTnll39BavrPj/RL7L09Zb7gvB/Lrgt5e/thhfyPwKkvg99f2CsGs
iydZ7dzvsjbzXmHXW/sLxwSzP2qe2L9B9UnzvLNv7X1u7990acml/VUv1Qw5
1ry1PxJVw9Xw8cb+DuP9s5hZ3tqvCNrMHtV4f7/OvRM2n9hf2x9gybt7Nvgh
3L0CL7evZ37+aL/Rq6jq+aue2r+4ueZXTO7j/VuKZ7HemvvEXs44QPK/3aP9
T3bu42fvfAo3L8bqmU1y/HP7hu0OTY+O39qv9tBJ7mDvC7j9BdMmPVUReWl/
ZIrHoumB1/YrvF55NTz8pb3KmS1/tbUv799zTC9wr+lL+4iZH7z/7jkJN++C
1apZCq8O7a9QiZt3buoz+4JCad4Hujv2C3B5XTxl+9yegbNLPvndpv3fZsmJ
TFn03B4A/kDX/g==
        "]}], {
       {RGBColor[1, 1, 0], DiskBox[{0.438701, 0.521234}, 0.25]}, 
       {GrayLevel[0], DiskBox[{0.438701, 0.521234}, 0.15]}}}},
    ImageSize->{120.17215140178803`, Automatic},
    PreserveImageOptions->False]}], ";"}]], "Input",
 CellLabel->"In[9]:=",
 CellID->1315899412],

Cell["Create a population using the fish.", "MathCaption",
 CellID->387652523],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SeedRandom", "[", "1234", "]"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"fishLis", "=", " ", 
   RowBox[{"RandomChoice", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"2", "/", "3"}], ",", 
        RowBox[{"1", "/", "3"}]}], "}"}], "\[Rule]", " ", 
      RowBox[{"{", 
       RowBox[{"yellowFish", ",", "orangeFish"}], "}"}]}], ",", "19"}], 
    "]"}]}], ";"}]}], "Input",
 CellLabel->"In[12]:=",
 CellID->1964272211],

Cell[BoxData[
 RowBox[{
  RowBox[{"fishPop", "=", " ", 
   RowBox[{"CreatePopulation", "[", "fishLis", "]"}]}], ";"}]], "Input",
 CellLabel->"In[14]:=",
 CellID->952490342],

Cell["Here is a depiction of the fish in the pond in Sally's yard.", \
"MathCaption",
 CellID->1748927774],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SRSChart", "[", 
  RowBox[{"fishPop", ",", 
   RowBox[{"ImageSize", "\[Rule]", " ", "Medium"}]}], "]"}]], "Input",
 CellLabel->"In[15]:=",
 CellID->1620421129],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztXS3UszyT7p41lZW1lUhkLbKyEvmxYs9BrEDWIpFILBJZWYtEIrFIJPLe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    "], {{0, 356}, {360, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{360, 356},
  PlotRange->{{0, 360}, {0, 356}}]], "Output",
 ImageSize->{364, 356},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[15]=",
 CellID->53428688]
}, Open  ]],

Cell["Visualize a random sample of size 6.", "MathCaption",
 CellID->1871328611],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SRSChart", "[", 
  RowBox[{"fishPop", ",", 
   RowBox[{"{", "6", "}"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", " ", "Medium"}]}], "]"}]], "Input",
 CellLabel->"In[16]:=",
 CellID->1521954762],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztfS3UszzQZvesqaysrUQisUgkEvmxYs9BrEBikUgkFolEYpFIJBaJRHYD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    "], {{0, 356}, {360, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{360, 356},
  PlotRange->{{0, 360}, {0, 356}}]], "Output",
 ImageSize->{364, 356},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[16]=",
 CellID->708131728]
}, Open  ]],

Cell[TextData[{
 "We can use the ",
 ButtonBox["Count",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Count"],
 " function to get information about our population."
}], "MathCaption",
 CellID->101874477],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Length", "[", "fishPop", "]"}], ",", 
   RowBox[{"Count", "[", 
    RowBox[{"fishLis", ",", "yellowFish"}], "]"}]}], "}"}]], "Input",
 CellLabel->"In[17]:=",
 CellID->659868280],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"19", ",", "12"}], "}"}]], "Output",
 ImageSize->{57, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[17]=",
 CellID->534721982]
}, Open  ]],

Cell["\<\
Take 10 samples of size 6 and count the number of yellow fish in each.\
\>", "MathCaption",
 CellID->677922792],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"samples", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"Count", "[", 
     RowBox[{
      RowBox[{"RandomSample", "[", 
       RowBox[{"fishLis", ",", "6"}], "]"}], ",", "yellowFish"}], "]"}], ",", 
    RowBox[{"{", "10", "}"}]}], "]"}]}]], "Input",
 CellLabel->"In[18]:=",
 CellID->1447617809],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "3", ",", "3", ",", "2", ",", "5", ",", "3", ",", "5", ",", "4", ",", "3", 
   ",", "3", ",", "4"}], "}"}]], "Output",
 ImageSize->{187, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[18]=",
 CellID->2000772426]
}, Open  ]],

Cell["\<\
Dividing by the sample size and taking the mean gives an estimate of the \
population fraction of yellow fish.\
\>", "MathCaption",
 CellID->2114655217],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Mean", "[", 
  RowBox[{"samples", "/", "6"}], "]"}]], "Input",
 CellLabel->"In[19]:=",
 CellID->155874269],

Cell[BoxData[
 FractionBox["7", "12"]], "Output",
 ImageSize->{23, 32},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[19]=",
 CellID->1683219056]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", 
  FractionBox["41", "60"], "]"}]], "Input",
 CellLabel->"In[20]:=",
 CellID->1229848649],

Cell[BoxData["0.6833333333333333`"], "Output",
 ImageSize->{60, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[20]=",
 CellID->1060592261]
}, Open  ]],

Cell["\<\
We can use our estimated fraction to see if this is close to the true \
proportion of the fish in the pond.\
\>", "MathCaption",
 CellID->86589164],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"N", "[", 
    RowBox[{"19", " ", 
     RowBox[{"41", "/", "60"}]}], "]"}], ",", 
   RowBox[{"Count", "[", 
    RowBox[{"fishLis", ",", "yellowFish"}], "]"}]}], "}"}]], "Input",
 CellLabel->"In[21]:=",
 CellID->1457650329],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"12.983333333333333`", ",", "12"}], "}"}]], "Output",
 ImageSize->{92, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[21]=",
 CellID->1259860527]
}, Open  ]],

Cell[TextData[{
 "We can increase the number of samples and compare our results with the pdf \
of the ",
 ButtonBox["HypergeometricDistribution",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HypergeometricDistribution"],
 ". We specify the bins explicitly to have the plot of the pdf fall in the \
middle of the bars."
}], "MathCaption",
 CellID->1490588277],

Cell[BoxData[
 RowBox[{
  RowBox[{"bigSamples", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Count", "[", 
      RowBox[{
       RowBox[{"RandomSample", "[", 
        RowBox[{"fishLis", ",", "6"}], "]"}], ",", "yellowFish"}], "]"}], ",", 
     RowBox[{"{", "1000", "}"}]}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[22]:=",
 CellID->1920429284],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Histogram", "[", 
     RowBox[{"bigSamples", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Range", "[", 
         RowBox[{"0", ",", "7"}], "]"}], "-", 
        RowBox[{"1", "/", "2"}]}], "}"}], ",", "\"\<PDF\>\""}], "]"}], ",", 
    RowBox[{"DiscretePlot", "[", 
     RowBox[{
      RowBox[{"PDF", "[", 
       RowBox[{
        RowBox[{"HypergeometricDistribution", "[", 
         RowBox[{"6", ",", "12", ",", "19"}], "]"}], ",", "n"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"n", ",", "0", ",", "6"}], "}"}], ",", 
      RowBox[{"PlotMarkers", "\[Rule]", " ", "Automatic"}]}], "]"}]}], "}"}], 
  "]"}]], "Input",
 CellLabel->"In[23]:=",
 CellID->2123245425],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0.798413061722744, 0.824719615472648, 0.968322270542458], 
    EdgeForm[Opacity[0.7]], {}, 
    {RGBColor[0.798413061722744, 0.824719615472648, 0.968322270542458], 
     EdgeForm[Opacity[0.7]], {}, 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{0.5, 0.}, {1.5, 0.009}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{38.296501457725945`, 62.09300291545189}, {
          37.154052915873336`, 40.27773676735785}}],
        StatusArea[#, 0.009]& ,
        TagBoxNote->"0.009"],
       StyleBox["0.009`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.009, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{1.5, 0.}, {2.5, 0.092}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{61.09300291545189, 84.88950437317784}, {
          17.568968507738397`, 40.27773676735785}}],
        StatusArea[#, 0.092]& ,
        TagBoxNote->"0.092"],
       StyleBox["0.092`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.092, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{2.5, 0.}, {3.5, 0.281}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{83.88950437317784, 
          107.6860058309038}, {-27.02839237343636, 40.27773676735785}}],
        StatusArea[#, 0.281]& ,
        TagBoxNote->"0.281"],
       StyleBox["0.281`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.281, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{3.5, 0.}, {4.5, 0.38}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{106.6860058309038, 
          130.48250728862973`}, {-50.38891473976598, 40.27773676735785}}],
        StatusArea[#, 0.38]& ,
        TagBoxNote->"0.38"],
       StyleBox["0.38`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.38, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{4.5, 0.}, {5.5, 0.214}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{129.48250728862973`, 
          153.2790087463557}, {-11.218745923496094`, 40.27773676735785}}],
        StatusArea[#, 0.214]& ,
        TagBoxNote->"0.214"],
       StyleBox["0.214`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.214, {
         GrayLevel[0]}], "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       TagBox[
        DynamicBox[{
          FEPrivate`If[
           CurrentValue["MouseOver"], 
           EdgeForm[{
             GrayLevel[0.5], 
             AbsoluteThickness[1.5], 
             Opacity[0.66]}], {}, {}], 
          RectangleBox[{5.5, 0.}, {6.5, 0.024}, "RoundingRadius" -> 0]},
         ImageSizeCache->{{152.2790087463557, 176.07551020408164`}, {
          33.61457983006582, 40.27773676735785}}],
        StatusArea[#, 0.024]& ,
        TagBoxNote->"0.024"],
       StyleBox["0.024`", {
         GrayLevel[0]}, StripOnInput -> False]],
      Annotation[#, 
       Style[0.024, {
         GrayLevel[0]}], "Tooltip"]& ]}, {}, {}}, {
    {RGBColor[0.24720000000000014`, 0.24, 0.6], 
     {Opacity[0.2], LineBox[{},
       VertexColors->None]}, 
     {Opacity[0.2], 
      LineBox[{{{0., 0.00025799793601651185`}, {0., 0}}, {{1., 
         0.009287925696594427}, {1., 0}}, {{2., 0.08513931888544891}, {
         2., 0}}, {{3., 0.2837977296181631}, {3., 0}}, {{4., 
         0.3831269349845201}, {4., 0}}, {{5., 0.2043343653250774}, {
         5., 0}}, {{6., 0.034055727554179564`}, {6., 0}}},
       VertexColors->None]}}, 
    {RGBColor[0.24720000000000014`, 0.24, 0.6], {}, 
     GeometricTransformationBox[InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->8.96], {0, 0}], {{{0., 0.00025799793601651185`}}, {{1., 
      0.009287925696594427}}, {{2., 0.08513931888544891}}, {{3., 
      0.2837977296181631}}, {{4., 0.3831269349845201}}, {{5., 
      0.2043343653250774}}, {{6., 0.034055727554179564`}}}], {}}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, True},
  AxesOrigin->{-0.5, 0},
  FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
  GridLines->{None, None},
  PlotRange->{{-0.5, 6.5}, {All, All}},
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.02], 
     Scaled[0.1]}},
  Ticks->{Automatic, Automatic}]], "Output",
 ImageSize->{184, 117},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[23]=",
 CellID->287117869]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Personalizing the Population", "Section",
 CellID->913275697],

Cell["\<\
Images can also be used in forming the population. In general for this to be \
successful you will need to mask the image to fit in an oval. This can be \
done using the image processing functions together with the graphics drawing \
tools. With a little work your population can include images of the members \
of any group.\
\>", "Text",
 CellID->1359014370],

Cell["\<\
Create a mask for the image and use it to set an alpha channel.\
\>", "MathCaption",
 CellID->1160940689],

Cell[BoxData[
 RowBox[{
  RowBox[{"img", "=", 
   GraphicsBox[
    TagBox[RasterBox[CompressedData["
1:eJw8vPVbXNfePvxc3/f94f0XntMQdNzd3d2NGcbQGdxJkBAIISQhIUKwQNxd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      "], {{0, 169}, {124, 0}}, {0, 255},
      ColorFunction->RGBColor],
     BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
     Selectable->False],
    BaseStyle->"ImageGraphics",
    ImageSizeRaw->{124, 169},
    PlotRange->{{0, 124}, {0, 169}}]}], ";", 
  RowBox[{"mask", "=", 
   GraphicsBox[{
     {EdgeForm[{GrayLevel[1.], Opacity[1.], AbsoluteThickness[1]}], EdgeForm[
      None], DiskBox[{60.5, 76.5}, {0., 2.}]}, 
     {EdgeForm[{GrayLevel[1.], Opacity[1.], AbsoluteThickness[1]}], EdgeForm[
      None], DiskBox[{72.5, 76.5}, {1., 7.}]}, 
     {EdgeForm[{GrayLevel[1.], Opacity[1.], AbsoluteThickness[1]}], EdgeForm[
      None], DiskBox[{60.5, 78.5}, {0., 4.}]}, 
     {EdgeForm[{GrayLevel[1.], Opacity[1.], AbsoluteThickness[1]}], EdgeForm[
      None], DiskBox[{61.5, 82.5}, {55., 66.}]}},
    Background->GrayLevel[0],
    BaseStyle->GrayLevel[1],
    ImagePadding->{{0., 0.}, {0., 0.}},
    ImageSize->{30., Automatic},
    ImageSizeRaw->{124, 169},
    PlotRange->{{0., 124.}, {0., 169.}},
    PlotRangePadding->Automatic]}], ";"}]], "Input",
 CellLabel->"In[73]:=",
 CellID->17283775],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"face", "=", 
  RowBox[{"SetAlphaChannel", "[", 
   RowBox[{"img", ",", "mask"}], "]"}]}]], "Input",
 CellLabel->"In[74]:=",
 CellID->578114080],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztfa+fpEj27X4+7x9Y2bZkypRYJBKJ/CIRT4SMJ5HIkFjEE0gkFolEYpEp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    "], {{0, 169}, {124, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{124, 169},
  PlotRange->{{0, 124}, {0, 169}}]], "Output",
 ImageSize->{128, 169},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[74]=",
 CellID->768679797]
}, Open  ]],

Cell[TextData[{
 "Use the resulting image to create a population for use in the ",
 Cell[BoxData[
  ButtonBox["SRSChart",
   BaseStyle->"Link",
   ButtonData->"paclet:Classroom Utilities/ref/SRSChart"]], "InlineFormula"],
 " function."
}], "MathCaption",
 CellID->2041227028],

Cell[BoxData[
 RowBox[{
  RowBox[{"imgPop", "=", 
   RowBox[{"CreatePopulation", "[", 
    RowBox[{
     RowBox[{"{", "face", "}"}], ",", "7", ",", "1.3"}], "]"}]}], 
  ";"}]], "Input",
 CellLabel->"In[75]:=",
 CellID->1468632315],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SRSChart", "[", 
  RowBox[{"imgPop", ",", 
   RowBox[{"{", "3", "}"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", " ", "Medium"}]}], "]"}]], "Input",
 CellLabel->"In[76]:=",
 CellID->667175473],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzsvSuYrMqzPXjnG3PlkW1LIpFY3CCRyH+a+T7EiJRpUyKRWCQSiUUikdiS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    "], {{0, 329}, {360, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{360, 329},
  PlotRange->{{0, 360}, {0, 329}}]], "Output",
 ImageSize->{364, 329},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[76]=",
 CellID->26324723]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Visualizing Statistical Concepts - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "TutorialColor", 
  "LinkTrails" -> GridBox[{{
      RowBox[{
        StyleBox[
         ButtonBox[
         "Mathematica", ButtonData -> "paclet:guide/Mathematica", 
          BaseStyle -> {"Link", "DockedLinkTrail"}], FontSlant -> "Italic"], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 33, 6.2608624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Tutorial", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "The statistical functions defined in the ClassroomUtilities` context are \
intended for use in visualizing various concepts that students often have \
difficulty in grasping. In this tutorial we introduce the concepts of \
population, simple random sample and event. This loads the package. Functions \
for working with populations and samples.", "synonyms" -> {}, "title" -> 
    "Visualizing Statistical Concepts", "titlemodifier" -> "", "windowtitle" -> 
    "Visualizing Statistical Concepts", "type" -> "Tutorial", "uri" -> 
    "Classroom Utilities/tutorial/VisualizingStatisticalConcepts"}, 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> Notebook], 
   Cell[
    StyleData["Output"], CellContext -> Notebook], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "RasterizedOutput"->{
  Cell[10651, 301, 14447, 245, 209, "Output",
   CellTags->"RasterizedOutput",
   CellID->87502714],
  Cell[41902, 933, 26777, 447, 377, "Output",
   CellTags->"RasterizedOutput",
   CellID->53428688],
  Cell[69019, 1396, 34013, 566, 377, "Output",
   CellTags->"RasterizedOutput",
   CellID->708131728],
  Cell[188649, 3570, 20849, 350, 190, "Output",
   CellTags->"RasterizedOutput",
   CellID->768679797],
  Cell[210264, 3953, 69383, 1145, 350, "Output",
   CellTags->"RasterizedOutput",
   CellID->26324723]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"RasterizedOutput", 282239, 5162}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[625, 21, 2788, 63, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3438, 88, 69, 1, 51, "Title",
 CellID->509267359],
Cell[3510, 91, 311, 6, 70, "Text",
 CellID->1534169418],
Cell[3824, 99, 67, 1, 31, "MathCaption",
 CellID->1374342004],
Cell[3894, 102, 106, 3, 28, "Input",
 CellID->231954096],
Cell[CellGroupData[{
Cell[4025, 109, 63, 1, 51, "Section",
 CellID->769672167],
Cell[4091, 112, 1697, 56, 100, "DefinitionBox",
 CellID->2096742444],
Cell[5791, 170, 91, 1, 31, "Caption",
 CellID->1891092685],
Cell[5885, 173, 194, 4, 50, "Text",
 CellID->598110709],
Cell[6082, 179, 95, 1, 31, "MathCaption",
 CellID->836781195],
Cell[CellGroupData[{
Cell[6202, 184, 362, 11, 28, "Input",
 CellID->2058623809],
Cell[6567, 197, 2971, 59, 209, "Output",
 CellID->2109354559]
}, Open  ]],
Cell[9553, 259, 149, 4, 70, "Text",
 CellID->1262028437],
Cell[9705, 265, 73, 1, 70, "MathCaption",
 CellID->215959240],
Cell[9781, 268, 396, 13, 70, "Input",
 CellID->379583250],
Cell[10180, 283, 80, 1, 70, "MathCaption",
 CellID->1629251160],
Cell[CellGroupData[{
Cell[10285, 288, 363, 11, 28, "Input",
 CellID->260263223],
Cell[10651, 301, 14447, 245, 209, "Output",
 CellTags->"RasterizedOutput",
 CellID->87502714]
}, Open  ]],
Cell[25113, 549, 384, 10, 70, "Text",
 CellID->1478395671],
Cell[25500, 561, 66, 1, 70, "MathCaption",
 CellID->1010246707],
Cell[25569, 564, 469, 15, 70, "Input",
 CellID->1210691367],
Cell[CellGroupData[{
Cell[26063, 583, 182, 6, 70, "Input",
 CellID->682249844],
Cell[26248, 591, 3490, 87, 191, "Output",
 CellID->1157936775]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[29787, 684, 58, 1, 70, "Section",
 CellID->946568311],
Cell[29848, 687, 236, 5, 70, "Text",
 CellID->414746001],
Cell[30087, 694, 214, 5, 70, "Text",
 CellID->2051604292],
Cell[30304, 701, 68, 1, 70, "MathCaption",
 CellID->157567382],
Cell[30375, 704, 10461, 186, 70, "Input",
 CellID->1315899412],
Cell[40839, 892, 78, 1, 70, "MathCaption",
 CellID->387652523],
Cell[40920, 895, 487, 16, 70, "Input",
 CellID->1964272211],
Cell[41410, 913, 172, 5, 70, "Input",
 CellID->952490342],
Cell[41585, 920, 106, 2, 70, "MathCaption",
 CellID->1748927774],
Cell[CellGroupData[{
Cell[41716, 926, 183, 5, 28, "Input",
 CellID->1620421129],
Cell[41902, 933, 26777, 447, 377, "Output",
 CellTags->"RasterizedOutput",
 CellID->53428688]
}, Open  ]],
Cell[68694, 1383, 80, 1, 70, "MathCaption",
 CellID->1871328611],
Cell[CellGroupData[{
Cell[68799, 1388, 217, 6, 28, "Input",
 CellID->1521954762],
Cell[69019, 1396, 34013, 566, 377, "Output",
 CellTags->"RasterizedOutput",
 CellID->708131728]
}, Open  ]],
Cell[103047, 1965, 204, 7, 70, "MathCaption",
 CellID->101874477],
Cell[CellGroupData[{
Cell[103276, 1976, 229, 7, 70, "Input",
 CellID->659868280],
Cell[103508, 1985, 206, 7, 36, "Output",
 CellID->534721982]
}, Open  ]],
Cell[103729, 1995, 121, 3, 70, "MathCaption",
 CellID->677922792],
Cell[CellGroupData[{
Cell[103875, 2002, 328, 10, 70, "Input",
 CellID->1447617809],
Cell[104206, 2014, 293, 9, 36, "Output",
 CellID->2000772426]
}, Open  ]],
Cell[104514, 2026, 162, 4, 70, "MathCaption",
 CellID->2114655217],
Cell[CellGroupData[{
Cell[104701, 2034, 130, 4, 70, "Input",
 CellID->155874269],
Cell[104834, 2040, 181, 6, 53, "Output",
 CellID->1683219056]
}, Open  ]],
Cell[CellGroupData[{
Cell[105052, 2051, 122, 4, 70, "Input",
 CellID->1229848649],
Cell[105177, 2057, 178, 5, 36, "Output",
 CellID->1060592261]
}, Open  ]],
Cell[105370, 2065, 157, 4, 70, "MathCaption",
 CellID->86589164],
Cell[CellGroupData[{
Cell[105552, 2073, 273, 9, 70, "Input",
 CellID->1457650329],
Cell[105828, 2084, 224, 7, 36, "Output",
 CellID->1259860527]
}, Open  ]],
Cell[106067, 2094, 360, 9, 70, "MathCaption",
 CellID->1490588277],
Cell[106430, 2105, 358, 11, 70, "Input",
 CellID->1920429284],
Cell[CellGroupData[{
Cell[106813, 2120, 758, 22, 70, "Input",
 CellID->2123245425],
Cell[107574, 2144, 5763, 160, 70, "Output",
 CellID->287117869]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[113386, 2310, 67, 1, 70, "Section",
 CellID->913275697],
Cell[113456, 2313, 370, 7, 70, "Text",
 CellID->1359014370],
Cell[113829, 2322, 115, 3, 70, "MathCaption",
 CellID->1160940689],
Cell[113947, 2327, 74507, 1232, 70, "Input",
 CellID->17283775],
Cell[CellGroupData[{
Cell[188479, 3563, 167, 5, 28, "Input",
 CellID->578114080],
Cell[188649, 3570, 20849, 350, 190, "Output",
 CellTags->"RasterizedOutput",
 CellID->768679797]
}, Open  ]],
Cell[209513, 3923, 275, 8, 70, "MathCaption",
 CellID->2041227028],
Cell[209791, 3933, 230, 8, 70, "Input",
 CellID->1468632315],
Cell[CellGroupData[{
Cell[210046, 3945, 215, 6, 28, "Input",
 CellID->667175473],
Cell[210264, 3953, 69383, 1145, 350, "Output",
 CellTags->"RasterizedOutput",
 CellID->26324723]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[279686, 5103, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

