(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    335713,       8328]
NotebookOptionsPosition[    323876,       7946]
NotebookOutlinePosition[    327971,       8052]
CellTagsIndexPosition[    327890,       8047]
WindowTitle->Working with Riemann Sums - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["CLASSROOM UTILITIES TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Using Integration Techniques Functions\"\>":>
       Documentation`HelpLookup[
       "paclet:Classroom \
Utilities/tutorial/UsingIntegrationTechniquesFunctions"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"RelatedTutorialsSection"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Classroom \
Utilities/tutorial/WorkingWithRiemannSums\"\>":>
       None, "\<\"Copy Mathematica URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Classroom Utilities/tutorial/WorkingWithRiemannSums"]}, 
            Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Classroom \
Utilities/tutorial/WorkingWithRiemannSums.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Classroom Utilities/tutorial/WorkingWithRiemannSums", ".html"]],
            None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell["Working with Riemann Sums", "Title",
 CellID->509267359],

Cell[TextData[{
 "The functions discussed in this tutorial are intended to help student \
visualize the approximation of volumes of revolution using disks, shells and \
washers. The three approximations, disks, shells and washers, each require \
slightly different forms of syntax explained in this tutorial. In each of the \
forms described in this tutorial the volumes are assumed to be generated by \
functions of the variable represented by the horizontal axis rotated around \
the vertical axis. That is, we assume the function ",
 StyleBox["y = f(x)",
  FontSlant->"Italic"],
 ", is rotated around the ",
 StyleBox["y-",
  FontSlant->"Italic"],
 "axis. "
}], "Text",
 CellID->1534169418],

Cell[TextData[StyleBox["This loads the package.", "MathCaption"]], \
"MathCaption",
 CellID->1463735601],

Cell[BoxData[
 RowBox[{"<<", "ClassroomUtilities`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1579069622],

Cell[BoxData[GridBox[{
   {
    RowBox[{"Shell", "[", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox[
        StyleBox["r", "TI"], 
        StyleBox["min", "TI"]], ",", 
       SubscriptBox[
        StyleBox["r", "TI"], 
        StyleBox["max", "TI"]], ",", 
       StyleBox["bot", "TI"], ",", 
       StyleBox["tp", "TI"]}], "}"}], "]"}], Cell[
    "create a shell or washer", "TableText"]},
   {
    RowBox[{"RiemannSumPlot3D", "[", 
     RowBox[{
      StyleBox["f", "TI"], ",", 
      RowBox[{"{", 
       RowBox[{
        StyleBox["x", "TI"], ",", 
        SubscriptBox[
         StyleBox["x", "TI"], 
         StyleBox["min", "TI"]], ",", 
        SubscriptBox[
         StyleBox["x", "TI"], 
         StyleBox["max", "TI"]]}], "}"}], ",", 
      StyleBox["n", "TI"], ",", "\"\<\!\(\*
StyleBox[\"type\", \"TI\"]\)\>\""}], "]"}], Cell[
    "visualize the approximation of the volume", "TableText"]},
   {
    RowBox[{"RiemannSum3D", "[", 
     RowBox[{
      StyleBox["f", "TI"], ",", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         StyleBox["x", "TI"], ",", 
         SubscriptBox[
          StyleBox["x", "TI"], 
          StyleBox["min", "TI"]], ",", 
         SubscriptBox[
          StyleBox["x", "TI"], 
          StyleBox["max", "TI"]]}], "}"}], ".", 
       StyleBox["n", "TI"], ".", "\"\<\!\(\*
StyleBox[\"type\", \"TI\"]\)\>\""}]}], "]"}], Cell[
    "get th numerical value of the approximation to the volume", "TableText"]}
  }]], "DefinitionBox",
 CellID->2096742444],

Cell["Functions for Riemann sums in three dimensions.", "Caption",
 CellID->1891092685],

Cell[TextData[{
 "For visualization disks are created using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s ",
 ButtonBox["Cylinder",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cylinder"],
 " function. "
}], "Text",
 CellID->820227328],

Cell[TextData[{
 "A stack of four disks for the function ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["y", "TI"], "=", 
    SuperscriptBox[
     StyleBox["x", "TI"], "2"]}], ",", " ", 
   RowBox[{"0", "\[LessEqual]", 
    StyleBox["x", "TI"], " ", "<", "1"}]}]], "InlineFormula"],
 ", rotated around the vertical axis."
}], "MathCaption",
 CellID->836781195],

Cell[BoxData[
 RowBox[{
  RowBox[{"hts", " ", "=", " ", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "0", ",", "h"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0", ",", 
         RowBox[{"h", "+", 
          RowBox[{"1", "/", "4"}]}]}], "}"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"h", ",", "0", ",", 
       RowBox[{"3", "/", "4"}], ",", 
       RowBox[{"1", "/", "4"}]}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->2058623809],

Cell[BoxData[
 RowBox[{
  RowBox[{"rds", "=", " ", 
   RowBox[{"Table", "[", 
    RowBox[{
     SqrtBox["h"], ",", 
     RowBox[{"{", 
      RowBox[{"h", ",", 
       RowBox[{"1", "/", "8"}], ",", 
       RowBox[{"3", "/", "4"}], ",", 
       RowBox[{"1", "/", "4"}]}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->863171603],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graphics3D", "[", 
  RowBox[{"{", 
   RowBox[{"Cylinder", "[", 
    RowBox[{"hts", ",", "rds"}], "]"}], "}"}], "]"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->1073096136],

Cell[BoxData[
 Graphics3DBox[
  CylinderBox[
   NCache[{{{0, 0, 0}, {0, 0, Rational[1, 4]}}, {{0, 0, Rational[1, 4]}, {
      0, 0, Rational[1, 2]}}, {{0, 0, Rational[1, 2]}, {
      0, 0, Rational[3, 4]}}, {{0, 0, Rational[3, 4]}, {0, 0, 1}}}, {{{0, 0, 
      0}, {0, 0, 0.25}}, {{0, 0, 0.25}, {0, 0, 0.5}}, {{0, 0, 0.5}, {
      0, 0, 0.75}}, {{0, 0, 0.75}, {0, 0, 1}}}], 
   NCache[{Rational[1, 2] 2^Rational[-1, 2], Rational[1, 2] 
     Rational[3, 2]^Rational[1, 2], Rational[1, 2] 
     Rational[5, 2]^Rational[1, 2]}, {0.35355339059327373`, 
    0.6123724356957945, 0.7905694150420949}]],
  ViewPoint->{1.5049660644324205`, -2.601200168495968, 1.5552603731604995`},
  ViewVertical->{0.053893137690073555`, -0.14718423096167468`, 
   1.97527955678799}]], "Output",
 ImageSize->{184, 140},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->1697323360,
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "From the reference point of geometry shells and washers are the same shape, \
hence the single function ",
 Cell[BoxData[
  ButtonBox["Shell",
   BaseStyle->"Link",
   ButtonData->"paclet:Classroom Utilities/ref/Shell"]], "InlineFormula"],
 " is used to generate both. The principle difference being that shells are \
nested one inside the other while washers are stacked one on top of the \
other. Both shells and washers can be used to visualize what happens when the \
region between two curves is rotated around the axis. In this example we use \
the two functions ",
 Cell[BoxData[
  RowBox[{
   StyleBox["y", "TI"], "=", 
   StyleBox["x", "TI"]}]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  RowBox[{
   StyleBox["y", "TI"], "=", 
   SuperscriptBox[
    StyleBox["x", "TI"], "4"]}]], "InlineFormula"],
 StyleBox[" ",
  FontSlant->"Italic"],
 " on the interval (0, 1). Four subdivisions are used in these examples and \
the dimensions of the resulting solids are determined by the function values \
at the midpoints of the sub-intervals."
}], "Text",
 CellID->1888428419],

Cell["Here we see the difference between washers and shells.", "MathCaption",
 CellID->538781999],

Cell[BoxData[
 RowBox[{
  RowBox[{"wshrPts", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         FractionBox["1", "8"], ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         FractionBox["1", 
          SuperscriptBox["2", 
           RowBox[{"3", "/", "4"}]]], ",", 
         FractionBox["1", "4"]}], "}"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         FractionBox["3", "8"], ",", 
         FractionBox["1", "4"]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         FractionBox[
          SuperscriptBox["3", 
           RowBox[{"1", "/", "4"}]], 
          SuperscriptBox["2", 
           RowBox[{"3", "/", "4"}]]], ",", 
         FractionBox["1", "2"]}], "}"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         FractionBox["5", "8"], ",", 
         FractionBox["1", "2"]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         FractionBox[
          SuperscriptBox["5", 
           RowBox[{"1", "/", "4"}]], 
          SuperscriptBox["2", 
           RowBox[{"3", "/", "4"}]]], ",", 
         FractionBox["3", "4"]}], "}"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         FractionBox["7", "8"], ",", 
         FractionBox["3", "4"]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         FractionBox[
          SuperscriptBox["7", 
           RowBox[{"1", "/", "4"}]], 
          SuperscriptBox["2", 
           RowBox[{"3", "/", "4"}]]], ",", "1"}], "}"}]}], "}"}]}], "}"}]}], 
  ";"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->1518001441],

Cell[BoxData[
 RowBox[{
  RowBox[{"plt1", "=", 
   RowBox[{"Plot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"x", ",", 
       SuperscriptBox["x", "4"]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"Prolog", "\[Rule]", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"GrayLevel", "[", ".95", "]"}], ",", 
        RowBox[{"EdgeForm", "[", 
         RowBox[{"GrayLevel", "[", ".4", "]"}], "]"}], ",", 
        RowBox[{"Rectangle", "@@@", "wshrPts"}]}], "}"}]}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", " ", "Automatic"}], ",", 
     RowBox[{"PlotLabel", "\[Rule]", " ", 
      RowBox[{"Style", "[", 
       RowBox[{"\"\<washers\>\"", ",", 
        RowBox[{"FontFamily", " ", "\[Rule]", " ", "\"\<Times\>\""}]}], 
       "]"}]}]}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[6]:=",
 CellID->944492527],

Cell[BoxData[
 RowBox[{
  RowBox[{"shlPts", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"r", ",", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"r", "+", 
            RowBox[{"1", "/", "8"}]}], ")"}], "4"]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"r", "+", 
          RowBox[{"1", "/", "4"}]}], ",", 
         RowBox[{"r", "+", 
          RowBox[{"1", "/", "8"}]}]}], "}"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"r", ",", "0", ",", 
       RowBox[{"3", "/", "4"}], ",", 
       RowBox[{"1", "/", "4"}]}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[7]:=",
 CellID->2034266312],

Cell[BoxData[
 RowBox[{
  RowBox[{"plt2", "=", 
   RowBox[{"Plot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"x", ",", 
       SuperscriptBox["x", "4"]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"Prolog", "\[Rule]", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"GrayLevel", "[", ".95", "]"}], ",", 
        RowBox[{"EdgeForm", "[", 
         RowBox[{"GrayLevel", "[", ".4", "]"}], "]"}], ",", 
        RowBox[{"Rectangle", "@@@", "shlPts"}]}], "}"}]}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", " ", "Automatic"}], ",", 
     RowBox[{"PlotLabel", "\[Rule]", " ", 
      RowBox[{"Style", "[", 
       RowBox[{"\"\<shells\>\"", ",", 
        RowBox[{"FontFamily", " ", "\[Rule]", " ", "\"\<Times\>\""}]}], 
       "]"}]}]}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[8]:=",
 CellID->1974245544],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GraphicsRow", "[", 
  RowBox[{"{", 
   RowBox[{"plt1", ",", "plt2"}], "}"}], "]"}]], "Input",
 CellLabel->"In[9]:=",
 CellID->375028211],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztXS20qzgQ7jlrVq58tnLllddWVlZWrFgkYkUkFomsrK3EbSQWWYmsRSKR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    "], {{0, 94}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 94},
  PlotRange->{{0, 180}, {0, 94}}]], "Output",
 ImageSize->{184, 94},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[9]=",
 CellID->182929267]
}, Open  ]],

Cell["Here is the same set rendered in three dimensions.", "MathCaption",
 CellID->966722352],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GraphicsRow", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"RiemannSumPlot3D", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"x", ",", 
        SuperscriptBox["x", "4"]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"z", ",", "0", ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"4", ",", 
        RowBox[{"3", " ", 
         RowBox[{"\[Pi]", "/", "2"}]}]}], "}"}], ",", "\"\<Washer\>\""}], 
     "]"}], ",", "\[IndentingNewLine]", 
    RowBox[{"RiemannSumPlot3D", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"x", ",", 
        SuperscriptBox["x", "4"]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"4", ",", 
        RowBox[{"3", " ", 
         RowBox[{"\[Pi]", "/", "2"}]}]}], "}"}], ",", "\"\<Shell\>\""}], 
     "]"}]}], "}"}], "]"}]], "Input",
 CellLabel->"In[10]:=",
 CellID->1988780509],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnL2W2zYQhX1OXiCvkNLllmn3EbZ0kcIqVaTYUu2WLl1uy9Jd8gop8whp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    "], {{0, 94}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 94},
  PlotRange->{{0, 180}, {0, 94}}]], "Output",
 ImageSize->{184, 94},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[10]=",
 CellID->237077931]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Types", "Section",
 CellID->185278677],

Cell[CellGroupData[{

Cell["Disks", "Subsection",
 CellID->819439256],

Cell["\<\
Disks are used when the region being rotated about the vertical axis falls \
between the axis and the curve.\
\>", "Text",
 CellID->502947381],

Cell[TextData[{
 "The region between ",
 Cell[BoxData[
  RowBox[{
   StyleBox["y", "TI"], "=", 
   SuperscriptBox[
    StyleBox["x", "TI"], "2"]}]], "InlineFormula"],
 " and the vertical axis for ",
 Cell[BoxData[
  RowBox[{"0", " ", "\[LessEqual]", " ", 
   StyleBox["x", "TI"], " ", "<", "2"}]], "InlineFormula"],
 "."
}], "MathCaption",
 CellID->1387711044],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     SuperscriptBox["x", "2"], ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "2"}], "}"}], ",", 
   RowBox[{"Filling", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"1", "\[Rule]", " ", 
      RowBox[{"{", "2", "}"}]}], "}"}]}], ",", 
   RowBox[{"AspectRatio", "\[Rule]", " ", "Automatic"}]}], "]"}]], "Input",
 CellLabel->"In[72]:=",
 CellID->540997875],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztXSuTrDgYnar9Jfszxq4ciUQucsTWFrItEtkSi0QisUgktQrbsmVLNtDd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    "], {{0, 216}, {113, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{113, 216},
  PlotRange->{{0, 113}, {0, 216}}]], "Output",
 ImageSize->{117, 216},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[72]=",
 CellID->5333532]
}, Open  ]],

Cell["Approximating the volume using 16 disks.", "MathCaption",
 CellID->1587110088],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSumPlot3D", "[", 
  RowBox[{
   SuperscriptBox["x", "2"], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"16", ",", 
     RowBox[{"3", " ", 
      RowBox[{"\[Pi]", "/", "2"}]}]}], "}"}], ",", "\"\<Disk\>\""}], 
  "]"}]], "Input",
 CellLabel->"In[74]:=",
 CellID->664869629],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnTuSJCkShttsL7BXWDHFvsIcocQRp8QWVhix1LmKi6PtFeZMI7LZxSRF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    "], {{0, 197}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 197},
  PlotRange->{{0, 180}, {0, 197}}]], "Output",
 ImageSize->{184, 197},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[74]=",
 CellID->195577273]
}, Open  ]],

Cell["The numerical estimate for the volume.", "MathCaption",
 CellID->2046796015],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSum3D", "[", 
  RowBox[{
   SuperscriptBox["x", "2"], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "2"}], "}"}], ",", "16", ",", 
   "\"\<Disk\>\""}], "]"}]], "Input",
 CellLabel->"In[79]:=",
 CellID->2107566459],

Cell[BoxData["25.13274137210232`"], "Output",
 ImageSize->{53, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[79]=",
 CellID->819447681]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Shells", "Subsection",
 CellID->511610507],

Cell[TextData[{
 "Shells are good for approximating the volume generated by rotating the \
region between the curve ",
 Cell[BoxData[
  RowBox[{
   StyleBox["y", "TI"], "=", 
   RowBox[{
    StyleBox["f", "TI"], 
    RowBox[{"(", 
     StyleBox["x", "TI"], ")"}]}]}]], "InlineFormula"],
 " and the ",
 StyleBox["x",
  FontSlant->"Italic"],
 "-axis about the vertical axis."
}], "Text",
 CellID->113268561],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    SuperscriptBox["x", "2"], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "2"}], "}"}], ",", 
   RowBox[{"Filling", "\[Rule]", " ", "Axis"}], ",", 
   RowBox[{"AspectRatio", "\[Rule]", " ", "Automatic"}]}], "]"}]], "Input",
 CellLabel->"In[75]:=",
 CellID->1034193485],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztXS2T4zgUnKr7Jfczhh4MDAxcw4Crq8ChgYaBoYaGgqaBhqpDpoaGgdkn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    "], {{0, 216}, {113, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{113, 216},
  PlotRange->{{0, 113}, {0, 216}}]], "Output",
 ImageSize->{117, 216},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[75]=",
 CellID->184261237]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSumPlot3D", "[", 
  RowBox[{
   SuperscriptBox["x", "2"], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"8", ",", 
     RowBox[{"3", " ", 
      RowBox[{"\[Pi]", "/", "2"}]}]}], "}"}], ",", "\"\<Shell\>\""}], 
  "]"}]], "Input",
 CellLabel->"In[77]:=",
 CellID->1135960384],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3a2W5LoRAOA9Jy+QVwi8cGDoPELDhXfggICBTQcuHDjUcFlC8gCBeYTQ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    "], {{0, 186}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 186},
  PlotRange->{{0, 180}, {0, 186}}]], "Output",
 ImageSize->{184, 186},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[77]=",
 CellID->85626100]
}, Open  ]],

Cell["A numerical estimate for the generated volume.", "MathCaption",
 CellID->644546356],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"RiemannSum3D", "[", 
   RowBox[{
    SuperscriptBox["x", "2"], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "2"}], "}"}], ",", "8", ",", 
    "\"\<Shell\>\""}], "]"}], 
  RowBox[{"{", 
   RowBox[{"1", ",", "1.0"}], "}"}]}]], "Input",
 CellLabel->"In[81]:=",
 CellID->983089824],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"127", " ", "\[Pi]"}], "16"], ",", "24.936391687868984`"}], 
  "}"}]], "Output",
 ImageSize->{114, 32},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[81]=",
 CellID->2136777556]
}, Open  ]],

Cell["\<\
Combining the volumes in the above examples should give us the volume of a \
cylinder of radius two and height four. We can check that the estimates are \
close.\
\>", "Text",
 CellID->451354911],

Cell["\<\
The volume of the cylinder compared with the sum of the estimates.\
\>", "MathCaption",
 CellID->2041109219],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SuperscriptBox["2", "2"], " ", "\[Pi]", " ", "4."}], ",", 
   RowBox[{
    RowBox[{"RiemannSum3D", "[", 
     RowBox[{
      SuperscriptBox["x", "2"], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "0", ",", "2"}], "}"}], ",", "16", ",", 
      "\"\<Disk\>\""}], "]"}], "+", " ", 
    RowBox[{"RiemannSum3D", "[", 
     RowBox[{
      SuperscriptBox["x", "2"], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "0.", ",", "2"}], "}"}], ",", "8", ",", 
      "\"\<Shell\>\""}], "]"}]}]}], " ", "}"}]], "Input",
 CellLabel->"In[82]:=",
 CellID->1009747973],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"50.26548245743669`", ",", "50.06913305997131`"}], "}"}]], "Output",
 ImageSize->{127, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[82]=",
 CellID->1418403617]
}, Open  ]],

Cell["\<\
Shells can also be used in examining the case of the region being bounded by \
two curves. In this case the inner radius is taken to be \[Pi]/4.\
\>", "Text",
 CellID->1018866555],

Cell["\<\
Note the vertical orientation of the slices forming the walls of the shells.\
\>", "MathCaption",
 CellID->1212294338],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSumPlot3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Sin", "[", 
      RowBox[{"x", "-", 
       RowBox[{"\[Pi]", "/", "4"}]}], "]"}], ",", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"2", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"x", "-", 
           RowBox[{"\[Pi]", "/", "4"}]}], ")"}], "/", 
         RowBox[{"(", "\[Pi]", ")"}]}]}], ")"}], "2"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"\[Pi]", "/", "4"}], ",", 
     RowBox[{"3", 
      RowBox[{"\[Pi]", "/", "4"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"8", ",", 
     RowBox[{"3", " ", 
      RowBox[{"\[Pi]", "/", "2"}]}]}], "}"}], ",", "\"\<Shell\>\""}], 
  "]"}]], "Input",
 CellLabel->"In[94]:=",
 CellID->55588707],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnb1y3DYQxz2TF8grpHSpMq0e4UqXVqkihcurMnOlSpdqWbpLXiFlHiFt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    "], {{0, 129}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 129},
  PlotRange->{{0, 180}, {0, 129}}]], "Output",
 ImageSize->{184, 129},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[94]=",
 CellID->15002190]
}, Open  ]],

Cell[TextData[{
 "In some instances volumes estimates given by ",
 Cell[BoxData[
  ButtonBox["RiemannSum3D",
   BaseStyle->"Link",
   ButtonData->"paclet:Classroom Utilities/ref/RiemannSum3D"]], 
  "InlineFormula"],
 " may be negative if the given curves intersect in the specified interval. \
The value returned is equivalent to that obtained by ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["f", "TI"], 
    StyleBox["1", "TR"]], " ", "-", 
   SubscriptBox[
    StyleBox["f", "TI"], 
    StyleBox["2", "TR"]]}]], "InlineFormula"],
 " and will be negative when ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["f", "TI"], 
    StyleBox["2", "TR"]], ">", 
   SubscriptBox[
    StyleBox["f", "TI"], 
    StyleBox["1", "TR"]]}]], "InlineFormula"],
 "."
}], "Text",
 CellID->1814949465],

Cell["A volume estimate where the bounding curves intersect.", "MathCaption",
 CellID->997936267],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSumPlot3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Sin", "[", "x", "]"}], ",", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"2", 
        RowBox[{"x", "/", 
         RowBox[{"(", "\[Pi]", ")"}]}]}], ")"}], "2"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", 
     RowBox[{"3", " ", 
      RowBox[{"\[Pi]", "/", "4"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"24", ",", 
     RowBox[{"3", " ", 
      RowBox[{"\[Pi]", "/", "2"}]}]}], "}"}], ",", "\"\<Shell\>\""}], 
  "]"}]], "Input",
 CellLabel->"In[102]:=",
 CellID->356978448],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztna2WI7sRgPecvEBeIfDCgaH7CAMX3oEDAgaaDhy454QMLXhZ8gqBeYTQ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    "], {{0, 151}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 151},
  PlotRange->{{0, 180}, {0, 151}}]], "Output",
 ImageSize->{184, 151},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[102]=",
 CellID->303393418]
}, Open  ]],

Cell["A negative estimate for a portion of the volume.", "MathCaption",
 CellID->467556399],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSum3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Sin", "[", "x", "]"}], ",", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"2", 
        RowBox[{"x", "/", 
         RowBox[{"(", "\[Pi]", ")"}]}]}], ")"}], "2"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"\[Pi]", "/", "2."}], ",", 
     RowBox[{"3", " ", 
      RowBox[{"\[Pi]", "/", "4"}]}]}], "}"}], ",", "8", ",", 
   "\"\<Shell\>\""}], "]"}]], "Input",
 CellLabel->"In[101]:=",
 CellID->577733620],

Cell[BoxData[
 RowBox[{"-", "7.102973701373259`"}]], "Output",
 ImageSize->{61, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[101]=",
 CellID->1096815486]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Washers", "Subsection",
 CellID->1853579677],

Cell[TextData[{
 "In the case of shells the sub-intervals are generated by partitioning an \
interval on the horizontal axis. With washers the partitioned interval lies \
on the vertical axis and thus the ",
 Cell[BoxData[
  ButtonBox["RiemannSumPlot3D",
   BaseStyle->"Link",
   ButtonData->"paclet:Classroom Utilities/ref/RiemannSumPlot3D"]], 
  "InlineFormula"],
 " function requires the interval be specified explicitly. This is equivalent \
to slicing the solid with horizontal planes. The inner and outer radii are \
give by functions of the variable on the vertical axis. These functions are \
calculated by the program from lists of functions with the independent \
variable on the horizontal axis. "
}], "Text",
 CellID->1715847957],

Cell["Note the additional argument specifying the interval for z.", \
"MathCaption",
 CellID->830377205],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSumPlot3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Sin", "[", 
      RowBox[{"x", "-", 
       RowBox[{"\[Pi]", "/", "4"}]}], "]"}], ",", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"2", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"x", "-", 
           RowBox[{"\[Pi]", "/", "4"}]}], ")"}], "/", 
         RowBox[{"(", "\[Pi]", ")"}]}]}], ")"}], "2"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"\[Pi]", "/", "4"}], ",", 
     RowBox[{"3", 
      RowBox[{"\[Pi]", "/", "4"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"z", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"8", ",", 
     RowBox[{"3", " ", 
      RowBox[{"\[Pi]", "/", "2"}]}]}], "}"}], ",", "\"\<Washer\>\""}], 
  "]"}]], "Input",
 CellLabel->"In[95]:=",
 CellID->1846201383],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztna125TYQx/ecvkBfoXDhwtJ9hMDADQwoCLysJ3DhwlDBsvYVCvsIpQsL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    "], {{0, 130}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 130},
  PlotRange->{{0, 180}, {0, 130}}]], "Output",
 ImageSize->{184, 130},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[95]=",
 CellID->239815465]
}, Open  ]],

Cell["\<\
In some cases the inner or outer radius of the washer may need to given as \
several functions of x on an interval. These functions should pass the \
horizontal line test, that is they should not take the same y value on the \
interval.\
\>", "Text",
 CellID->1800946229],

Cell["\<\
Acceptable and unacceptable lists of functions for defining the radii of a \
washer.\
\>", "MathCaption",
 CellID->1868749985],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GraphicsRow", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Plot", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"x", ",", 
        RowBox[{
         RowBox[{"3", "/", "2"}], "-", "x"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
      RowBox[{"AspectRatio", "\[Rule]", " ", "Automatic"}], ",", 
      RowBox[{"PlotLabel", "\[Rule]", " ", "\"\<unacceptable\>\""}]}], "]"}], 
    ",", 
    RowBox[{"Plot", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"x", ",", 
        RowBox[{
         RowBox[{"3", "/", "2"}], "-", "x"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "0", ",", 
        RowBox[{"3", "/", "4"}]}], "}"}], ",", 
      RowBox[{"AspectRatio", "\[Rule]", " ", "Automatic"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"PlotLabel", "\[Rule]", " ", "\"\<acceptable\>\""}]}], "]"}]}], 
   "}"}], "]"}]], "Input",
 CellLabel->"In[99]:=",
 CellID->1132660520],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnS135DYUhnNOSeHC0oELB4YGBuYnrGFAwcChhQMDy3oKl9Vw/8JCw9CF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    "], {{0, 163}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 163},
  PlotRange->{{0, 180}, {0, 163}}]], "Output",
 ImageSize->{184, 163},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[99]=",
 CellID->288383423]
}, Open  ]],

Cell["\<\
Complicated forms of revolution can be generated using washers. Two examples \
using lists of functions are given.\
\>", "Text",
 CellID->63591113],

Cell["Some function definitions to use.", "MathCaption",
 CellID->1476463632],

Cell[BoxData[
 RowBox[{
  RowBox[{"f1", "[", "x_", "]"}], " ", ":=", " ", "x"}]], "Input",
 CellLabel->"In[22]:=",
 CellID->56398676],

Cell[BoxData[
 RowBox[{
  RowBox[{"f2", "[", "x_", "]"}], ":=", " ", 
  RowBox[{"2", "-", "x"}]}]], "Input",
 CellLabel->"In[16]:=",
 CellID->750721939],

Cell[BoxData[
 RowBox[{
  RowBox[{"f3", "[", "x_", "]"}], ":=", " ", 
  RowBox[{"4", "-", " ", 
   SqrtBox["x"]}]}]], "Input",
 CellLabel->"In[17]:=",
 CellID->917202648],

Cell[BoxData[
 RowBox[{
  RowBox[{"f4", "[", "x_", "]"}], ":=", " ", 
  RowBox[{"3", "+", 
   SqrtBox["x"]}]}]], "Input",
 CellLabel->"In[18]:=",
 CellID->787286770],

Cell[BoxData[
 RowBox[{
  RowBox[{"f5", "[", "x_", "]"}], ":=", " ", 
  RowBox[{"2", "+", 
   SqrtBox["x"]}]}]], "Input",
 CellLabel->"In[19]:=",
 CellID->1281994223],

Cell[BoxData[
 RowBox[{
  RowBox[{"vrtLn", "=", 
   RowBox[{"MakeVerticalSegment", "[", 
    RowBox[{".5", ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "2", ",", "3"}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[20]:=",
 CellID->773475610],

Cell["\<\
A volume with varying inner radius and constant outer radius.\
\>", "MathCaption",
 CellID->1086061979],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSumPlot3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"f1", "[", "x", "]"}], ",", 
       RowBox[{"f2", "[", "x", "]"}], ",", 
       RowBox[{"f3", "[", "x", "]"}], ",", "vrtLn"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"MakeVerticalSegment", "[", 
       RowBox[{"1", ",", 
        RowBox[{"{", 
         RowBox[{"z", ",", "0", ",", "4"}], "}"}]}], "]"}], "}"}]}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"z", ",", "0", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"48", ",", 
     RowBox[{"3", 
      RowBox[{"\[Pi]", "/", "2"}]}]}], "}"}], ",", "\"\<Washer\>\""}], 
  "]"}]], "Input",
 CellLabel->"In[23]:=",
 CellID->869539919],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztna123EoSx3POvsC+wsLAwEvzCIaGd6DBgkBTLzMMDC0YeF9h4cWLLl24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    "], {{0, 216}, {133, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{133, 216},
  PlotRange->{{0, 133}, {0, 216}}]], "Output",
 ImageSize->{137, 216},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[23]=",
 CellID->641133084]
}, Open  ]],

Cell["A volume with variation in both inner and outer radii.", "MathCaption",
 CellID->1215626950],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RiemannSumPlot3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"f1", "[", "x", "]"}], ",", 
       RowBox[{"f2", "[", "x", "]"}], ",", 
       RowBox[{"f3", "[", "x", "]"}], ",", "vrtLn"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"f1", "[", 
        RowBox[{"x", "-", "3"}], "]"}], ",", 
       RowBox[{"f2", "[", 
        RowBox[{"x", "-", "3"}], "]"}], ",", 
       RowBox[{"f3", "[", 
        RowBox[{"x", "-", "3"}], "]"}], ",", 
       RowBox[{"MakeVerticalSegment", "[", 
        RowBox[{"3.0", ",", 
         RowBox[{"{", 
          RowBox[{"z", ",", "2", ",", "3"}], "}"}]}], "]"}]}], "}"}]}], "}"}],
    ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1", ",", "3", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", "0", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"24", ",", 
     RowBox[{"3", " ", 
      RowBox[{"\[Pi]", "/", "2"}]}]}], "}"}], ",", "\"\<Washer\>\"", ",", 
   RowBox[{"PlotStyle", "\[Rule]", " ", "Red"}], ",", 
   RowBox[{"CapStyle", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"Red", ",", 
      RowBox[{"EdgeForm", "[", "Yellow", "]"}]}], "}"}]}]}], "]"}]], "Input",
 CellLabel->"In[24]:=",
 CellID->2030214739],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnDty5DgShhkxJ9krySxzZLaxhkxdYY/Qbt1g9wprzhHWHXPMMblqUUIl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    "], {{0, 156}, {180, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{180, 156},
  PlotRange->{{0, 180}, {0, 156}}]], "Output",
 ImageSize->{184, 156},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"RasterizedOutput",
 CellLabel->"Out[24]=",
 CellID->75032485]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Dynamic Examples", "Section",
 CellID->1177568547],

Cell["\<\
The rendering functions are computationally demanding and so may not update \
as quickly as you might expect. You can however use simpler examples to \
illustrate the ideas.\
\>", "Text",
 CellID->127436189],

Cell[CellGroupData[{

Cell["Disks", "Subsection",
 CellID->778832464],

Cell["A manipulate using 8 disks, varying the angle.", "MathCaption",
 CellID->924393983],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Manipulate", "[", 
  RowBox[{
   RowBox[{"RiemannSumPlot3D", "[", 
    RowBox[{
     SuperscriptBox["x", "2"], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"8", ",", "ang"}], "}"}], ",", "\"\<Disk\>\"", ",", 
     RowBox[{"PlotRange", "\[Rule]", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "2"}], ",", "2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "2"}], ",", "2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "4"}], "}"}]}], "}"}]}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"ang", ",", "0", ",", 
     RowBox[{"2", "\[Pi]"}]}], "}"}], ",", 
   RowBox[{"SaveDefinitions", "\[Rule]", " ", "True"}]}], "]"}]], "Input",
 CellLabel->"In[11]:=",
 CellID->1392969883],

Cell[BoxData[
 TagBox[
  StyleBox[
   DynamicModuleBox[{$CellContext`ang$$ = 4.0212385965949355`, 
    Typeset`show$$ = True, Typeset`bookmarkList$$ = {}, 
    Typeset`bookmarkMode$$ = "Menu", Typeset`animator$$, Typeset`animvar$$ = 
    1, Typeset`name$$ = "\"untitled\"", Typeset`specs$$ = {{
      Hold[$CellContext`ang$$], 0, 2 Pi}}, Typeset`size$$ = {
    180., {95., 100.}}, Typeset`update$$ = 0, Typeset`initDone$$, 
    Typeset`skipInitDone$$ = False, $CellContext`ang$1930$$ = 0}, 
    DynamicBox[Manipulate`ManipulateBoxes[
     1, StandardForm, "Variables" :> {$CellContext`ang$$ = 0}, 
      "ControllerVariables" :> {
        Hold[$CellContext`ang$$, $CellContext`ang$1930$$, 0]}, 
      "OtherVariables" :> {
       Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
        Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
        Typeset`specs$$, Typeset`size$$, Typeset`update$$, Typeset`initDone$$,
         Typeset`skipInitDone$$}, "Body" :> 
      ClassroomUtilities`RiemannSumPlot3D[$CellContext`x^2, {$CellContext`x, 
         0, 2}, {8, $CellContext`ang$$}, "Disk", 
        PlotRange -> {{-2, 2}, {-2, 2}, {0, 4}}], 
      "Specifications" :> {{$CellContext`ang$$, 0, 2 Pi}}, "Options" :> {}, 
      "DefaultOptions" :> {}],
     ImageSizeCache->{472., {126., 131.}},
     SingleEvaluation->True],
    Deinitialization:>None,
    DynamicModuleValues:>{},
    Initialization:>({ClassroomUtilities`RiemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, 
         Pattern[Private`n, 
          Blank[Integer]], "Shell", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{
         Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           ClassroomUtilities`Shell[
            Table[{Private`r, Private`r + Private`stp, 
              ReplaceAll[
              Private`f1, Private`var :> Private`r + Private`stp/2], 
              ReplaceAll[
              Private`f2, Private`var :> Private`r + Private`stp/2]}, {
             Private`r, Private`varmin, Private`varmax - Private`stp, 
              Private`stp}], ClassroomUtilities`CapStyle -> Private`cpstyl]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`f, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, 
         Pattern[Private`n, 
          Blank[Integer]], "Shell", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           ClassroomUtilities`Shell[
            Table[{Private`r, Private`r + Private`stp, 0, 
              ReplaceAll[
              Private`f, Private`var :> Private`r + Private`stp/2]}, {
             Private`r, Private`varmin, Private`varmax - Private`stp, 
              Private`stp}], ClassroomUtilities`CapStyle -> Private`cpstyl]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, {
          Pattern[Private`n, 
           Blank[Integer]], 
          PatternTest[
           Pattern[Private`ang, 
            Blank[]], NumericQ]}, "Shell", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           ClassroomUtilities`Shell[
            Table[{Private`r, Private`r + Private`stp, 
              ReplaceAll[
              Private`f1, Private`var :> Private`r + Private`stp/2], 
              ReplaceAll[
              Private`f2, Private`var :> Private`r + Private`stp/2]}, {
             Private`r, Private`varmin, Private`varmax - Private`stp, 
              Private`stp}], Private`ang, ClassroomUtilities`CapStyle -> 
            Private`cpstyl]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`f, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, {
          Pattern[Private`n, 
           Blank[Integer]], 
          PatternTest[
           Pattern[Private`ang, 
            Blank[]], NumericQ]}, "Shell", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           ClassroomUtilities`Shell[
            Table[{Private`r, Private`r + Private`stp, 0, 
              ReplaceAll[
              Private`f, Private`var :> Private`r + Private`stp/2]}, {
             Private`r, Private`varmin, Private`varmax - Private`stp, 
              Private`stp}], Private`ang, ClassroomUtilities`CapStyle -> 
            Private`cpstyl]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`f, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, 
         Pattern[Private`n, 
          Blank[Integer]], "Disk", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           Private`makeDisks[
           Private`f, {Private`var, Private`varmin, Private`varmax}, 
            Private`n]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`f, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, {
          Pattern[Private`n, 
           Blank[Integer]], 
          PatternTest[
           Pattern[Private`ang, 
            Blank[]], NumericQ]}, "Disk", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           Private`makeDisks[
           Private`f, {Private`var, Private`varmin, Private`varmax}, 
            Private`n, Private`ang, ClassroomUtilities`CapStyle -> 
            Private`cpstyl]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`fnLis, 
          Blank[List]], 
         Pattern[Private`varlis, 
          Blank[List]], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`res22 = 
          Private`riemannSumPlot3D[
           Private`fnLis, Private`varlis, Private`rnglis, Private`n, "Washer",
             Private`opts]}, If[
           MatchQ[Private`res22, {$Failed, 
             Alternatives[1, 2]}], 
           Message[
            MessageName[ClassroomUtilities`MakeRadiusFunction, "ovrlp"], 
            Part[{"inner radius", "outer radius"}, 
             Part[Private`res22, 2]]]]; Condition[Private`res22, 
           FreeQ[Private`res22, $Failed]]], 
       ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`fnLis, 
          Blank[List]], 
         Pattern[Private`varlis, 
          Blank[List]], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`res22 = 
          Private`riemannSumPlot3D[
           Private`fnLis, Private`varlis, Private`rnglis, {
            Private`n, Private`ang}, "Washer", Private`opts]}, If[
           MatchQ[Private`res22, {$Failed, 
             Alternatives[1, 2]}], 
           Message[
            MessageName[ClassroomUtilities`MakeRadiusFunction, "ovrlp"], 
            Part[{"inner radius", "outer radius"}, 
             Part[Private`res22, 2]]]]; Condition[Private`res22, 
           FreeQ[Private`res22, $Failed]]], 
       Options[ClassroomUtilities`RiemannSumPlot3D] = {
        AlignmentPoint -> Center, AspectRatio -> Automatic, 
         AutomaticImageSize -> False, Axes -> False, AxesEdge -> Automatic, 
         AxesLabel -> None, AxesOrigin -> Automatic, AxesStyle -> {}, 
         Background -> None, BaselinePosition -> Automatic, BaseStyle -> {}, 
         Boxed -> True, BoxRatios -> Automatic, BoxStyle -> {}, 
         ClassroomUtilities`CapStyle -> {
           EdgeForm[
            GrayLevel[0.65]], 
           GrayLevel[0.4]}, ColorOutput -> Automatic, ContentSelectable -> 
         Automatic, ControllerLinking -> Automatic, ControllerMethod -> 
         Automatic, ControllerPath -> Automatic, CoordinatesToolOptions -> 
         Automatic, DisplayFunction :> $DisplayFunction, Epilog -> {}, 
         FaceGrids -> None, FaceGridsStyle -> {}, FormatType :> 
         TraditionalForm, ImageMargins -> 0., ImagePadding -> All, ImageSize -> 
         Automatic, LabelStyle -> {}, Lighting -> Automatic, Method -> 
         Automatic, PlotLabel -> None, PlotRange -> All, PlotRangePadding -> 
         Automatic, PlotRegion -> Automatic, PlotStyle -> Automatic, 
         PreserveImageOptions -> Automatic, Prolog -> {}, RotationAction -> 
         "Fit", SphericalRegion -> False, Ticks -> Automatic, 
         TicksStyle -> {}, ViewAngle -> Automatic, ViewCenter -> Automatic, 
         ViewMatrix -> Automatic, ViewPoint -> {1.3, -2.4, 2.}, ViewRange -> 
         All, ViewVector -> Automatic, ViewVertical -> {0, 0, 1}}, 
       TagSet[ClassroomUtilities`RiemannSumPlot3D, 
        MessageName[ClassroomUtilities`RiemannSumPlot3D, "usage"], 
        "RiemannSumPlot3D[f,{var,varmin,varmax},n,\"type\"] creates n \
partitions of \"type\" to approximate the volume obtained by revolving the \
function f about the vertical axis."], 
       TagSet[ClassroomUtilities`CapStyle, 
        MessageName[ClassroomUtilities`CapStyle, "usage"], 
        "CapStyle is an option for RiemannSumPlot that controls the \
appearance of the flat surfaces of shells and washers."], 
       ClassroomUtilities`Shell[{
          Pattern[Private`r, 
           Blank[]], 
          Pattern[Private`r, 
           Blank[]], 
          BlankSequence[]}, 
         BlankNullSequence[]] := {}, Condition[
         ClassroomUtilities`Shell[{
           PatternTest[
            Pattern[Private`r1, 
             Blank[]], NumericQ], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       With[{Private`lis1 = Range[0., 2 Pi - Pi/36, Pi/36], Private`lis2 = 
          ConstantArray[1., 72], Private`rimPos = Join[
            Flatten[
             
             Table[{{73, 1, 2}, {2, 73, 74}, {217, 145, 146}, {146, 217, 
               218}} + Private`i, {Private`i, 0, 70}], 1], {{144, 72, 1}, {1, 
            144, 73}, {288, 216, 145}, {145, 288, 217}}], Private`flatPts1 = 
          Join[
            Range[72], {1, 145}, 
            Range[216, 145, -1]], Private`cpstyl = ReplaceAll[
            OptionValue[ClassroomUtilities`CapStyle], List -> Directive]}, 
         GraphicsComplex[
          Join[
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k2], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k2]], {
           EdgeForm[], 
           Polygon[Private`rimPos], Private`cpstyl, 
           Polygon[{Private`flatPts1, Private`flatPts1 + 72}]}]], 
       ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], 
         Pattern[Private`opts, 
          OptionsPattern[]]] := With[{Private`cpstyl = ReplaceAll[
            OptionValue[ClassroomUtilities`CapStyle], List -> Directive]}, 
         Map[
         ClassroomUtilities`Shell[#, ClassroomUtilities`CapStyle -> 
           Private`cpstyl]& , Private`shls]], Condition[
         ClassroomUtilities`Shell[{
           PatternTest[
            Pattern[Private`r1, 
             Blank[]], NumericQ], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, {
           Condition[
            PatternTest[
             Pattern[Private`stp, 
              Blank[Integer]], Positive], 
            Inequality[1, LessEqual, Private`stp, Less, 72]]}, 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       With[{Private`lis1 = Range[0., 2 Pi - Pi/36, Pi/36], Private`lis2 = 
          ConstantArray[1., 72], Private`rimPos = Flatten[
            
            Table[{{73, 1, 2}, {2, 73, 74}, {217, 145, 146}, {146, 217, 218}} + 
             Private`i, {Private`i, 0, Private`stp - 1}], 1], 
          Private`flatPts1 = Join[
            Take[
             Range[72], Private`stp + 1], 
            Take[
             Range[216, 145, -1], -(Private`stp + 1)]], Private`cpstyl = 
          ReplaceAll[
            OptionValue[ClassroomUtilities`CapStyle], List -> Directive]}, 
         GraphicsComplex[
          Join[
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k2], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k2]], {
           EdgeForm[], 
           Polygon[Private`rimPos], Private`cpstyl, 
           Polygon[{Private`flatPts1, Private`flatPts1 + 72}], 
           
           Polygon[{{1, 73, 217, 145}, {
             Private`stp + 1, Private`stp + 73, Private`stp + 217, 
              Private`stp + 145}}]}]], ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], {
          Condition[
           PatternTest[
            Pattern[Private`stp, 
             Blank[Integer]], Positive], 
           Inequality[1, LessEqual, Private`stp, Less, 72]]}, 
         Pattern[Private`opts, 
          OptionsPattern[]]] := With[{Private`cpstyl = ReplaceAll[
            OptionValue[ClassroomUtilities`CapStyle], List -> Directive]}, 
         Map[
         ClassroomUtilities`Shell[#, {Private`stp}, 
           ClassroomUtilities`CapStyle -> Private`cpstyl]& , Private`shls]], 
       Condition[
         ClassroomUtilities`Shell[{
           PatternTest[
            Pattern[Private`r1, 
             Blank[]], NumericQ], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, {0}, 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       With[{Private`lis1 = Range[0., 2 Pi - Pi/36, Pi/36], Private`lis2 = 
          ConstantArray[1., 72], Private`cpstyl = ReplaceAll[
            OptionValue[ClassroomUtilities`CapStyle], List -> Directive]}, 
         GraphicsComplex[
          Join[
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k2], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k2]], {
           EdgeForm[], Private`cpstyl, 
           Polygon[{{1, 73, 217, 145}}]}]], ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], {0}, 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Map[ClassroomUtilities`Shell[#, {0}, Private`opts]& , Private`shls], 
       Condition[
         ClassroomUtilities`Shell[{
           Pattern[Private`r1, 
            Blank[]], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, {72}, 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       ClassroomUtilities`Shell[{
         Private`r1, Private`r2, Private`k1, Private`k2}, Private`opts], 
       ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], {72}, 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       ClassroomUtilities`Shell[Private`shls, Private`opts], Condition[
         ClassroomUtilities`Shell[{
           Pattern[Private`r1, 
            Blank[]], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, 
          PatternTest[
           Pattern[Private`nn, 
            Blank[]], Positive], 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       Block[{Private`rr = ReplaceAll[
            Part[
             Private`closest[Private`nn], 1], {1 -> 72}]}, 
         ClassroomUtilities`Shell[{
          Private`r1, Private`r2, Private`k1, Private`k2}, Private`rr, 
          Private`opts]], ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], 
         PatternTest[
          Pattern[Private`nn, 
           Blank[]], Positive], 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`rr = ReplaceAll[
            Part[
             Private`closest[Private`nn], 1], {1 -> 72}]}, 
         ClassroomUtilities`Shell[Private`shls, Private`rr, Private`opts]], 
       Condition[
         ClassroomUtilities`Shell[{
           Pattern[Private`r1, 
            Blank[]], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, 0, 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       ClassroomUtilities`Shell[{
         Private`r1, Private`r2, Private`k1, Private`k2}, {0}, Private`opts], 
       ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], 0, 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       ClassroomUtilities`Shell[Private`shls, {0}, Private`opts], Condition[
         ClassroomUtilities`Shell[{
           Pattern[Private`r1, 
            Blank[]], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, 0., 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       ClassroomUtilities`Shell[{
         Private`r1, Private`r2, Private`k1, Private`k2}, {0}, Private`opts], 
       ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], 0., 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       ClassroomUtilities`Shell[Private`shls, {0}, Private`opts], 
       Options[ClassroomUtilities`Shell] = {
        ClassroomUtilities`CapStyle -> GrayLevel[0.75]}, 
       TagSet[ClassroomUtilities`Shell, 
        MessageName[ClassroomUtilities`Shell, "usage"], 
        "Shell[{r1,r2,btm,tp}] construct a washer with inner radius r1, outer \
radius r2 and thickness tp-btm."], Private`makePts[
         Pattern[Private`lis1, 
          Blank[List]], 
         Pattern[Private`lis2, 
          Blank[List]], 
         PatternTest[
          Pattern[Private`r, 
           Blank[]], NumericQ], 
         PatternTest[
          Pattern[Private`k, 
           Blank[]], NumericQ]] := 
       Transpose[{
         Private`r Cos[Private`lis1], Private`r Sin[Private`lis1], Private`k 
          Private`lis2}], Private`closest[
         PatternTest[
          Pattern[Private`val, 
           Blank[]], NumericQ]] := 
       With[{Private`vals = Range[0, 2 Pi - Pi/36, Pi/36], Private`val2 = Mod[
            N[Private`val], 2 Pi]}, 
         Position[
          Abs[Private`vals - Private`val2], 
          Min[
           Abs[Private`vals - Private`val2]]]], Private`makeDisks[
         Pattern[Private`fun, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NumericQ], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], NumericQ]}, 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/100, 
          Private`stpV, Private`funPrsV, Private`newFn}, 
         Private`newFn = 
          Private`invFun[
           Private`fun, {Private`var, Private`varmin, Private`varmax}]; {
           Private`stpV, Private`funPrsV} = 
          Private`vertFunPrs[Private`newFn, Private`n]; Cylinder[
           Map[{{0, 0, # - Private`stpV/2}, {0, 0, # + Private`stpV/2}}& , 
            Part[Private`funPrsV, All, 1]], 
           Part[Private`funPrsV, All, 2]]], Private`makeDisks[
         Pattern[Private`fun, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NumericQ], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], NumericQ]}, 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], 
         Pattern[Private`ang, 
          Blank[]], 
         Pattern[Private`opts, 
          OptionsPattern[ClassroomUtilities`CapStyle -> Automatic]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/100, 
          Private`stpV, Private`funPrsV, Private`newFn, Private`cpStyl = 
          OptionValue[ClassroomUtilities`CapStyle]}, 
         Private`newFn = 
          Private`invFun[
           Private`fun, {Private`var, Private`varmin, Private`varmax}]; {
           Private`stpV, Private`funPrsV} = 
          Private`vertFunPrs[Private`newFn, Private`n]; 
         ClassroomUtilities`Shell[
           Map[Private`shllFun[#, Private`stpV/2]& , Private`funPrsV], 
           Private`ang, ClassroomUtilities`CapStyle -> Private`cpStyl]], 
       Private`invFun[
         Pattern[Private`fun, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]]}] := 
       Block[{Private`funPrs, 
          Private`stp = (Private`varmax - Private`varmin)/100}, 
         Private`funPrs = Table[{
             Function[Private`var, Private`fun][Private`var2], 
             Private`var2}, {Private`var2, 
             Range[Private`varmin, Private`varmax, Private`stp]}]; 
         Interpolation[Private`funPrs, Method -> "Spline"]], 
       Private`vertFunPrs[
         Pattern[Private`fun, 
          Blank[InterpolatingFunction]], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive]] := Block[{Private`stpV = Part[
             Differences[
              Part[Private`fun, 1, 1]], 1]/Private`n}, {Private`stpV, 
          Map[{#, 
            Private`fun[#]}& , 
           Range[Part[Private`fun, 1, 1, 1] + Private`stpV/2, 
            Part[Private`fun, 1, 1, 2], Private`stpV]]}], Private`vertFunPrs[{
          Pattern[Private`minval, 
           Blank[]], {
           Pattern[Private`fun, 
            Blank[]], 
           Pattern[Private`fun2, 
            Blank[]]}}, {
          Pattern[Private`var, 
           Blank[]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]], 
          Pattern[Private`stp, 
           Blank[]]}] := Table[{Private`minval, 
          Min[{Private`fun, Private`fun2}], Private`var - Private`stp/2, 
          Private`var + Private`stp/2}, {
         Private`var, Private`varmin + Private`stp/2, Private`varmax, 
          Private`stp}], Private`vertFunPrs[{{
           Pattern[Private`fun, 
            Blank[]], 
           Pattern[Private`fun2, 
            Blank[]]}, 
          Pattern[Private`maxval, 
           Blank[]]}, {
          Pattern[Private`var, 
           Blank[]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]], 
          Pattern[Private`stp, 
           Blank[]]}] := Table[{Private`maxval, 
          Max[{Private`fun, Private`fun2}], Private`var - Private`stp/2, 
          Private`var + Private`stp/2}, {
         Private`var, Private`varmin + Private`stp/2, Private`varmax, 
          Private`stp}], Private`vertFunPrs[{
          Pattern[Private`fun, 
           Blank[]], 
          Pattern[Private`fun2, 
           Blank[]]}, {
          Pattern[Private`var, 
           Blank[]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]], 
          Pattern[Private`stp, 
           Blank[]]}] := 
       Table[{Private`fun, Private`fun2, Private`var - Private`stp/2, 
          Private`var + Private`stp/2}, {
         Private`var, Private`varmin + Private`stp/2, Private`varmax, 
          Private`stp}], Private`shllFun[{
          Pattern[Private`k, 
           Blank[]], 
          Pattern[Private`r, 
           Blank[]]}, 
         PatternTest[
          Pattern[Private`stp, 
           Blank[]], Positive]] := {
        0, Private`r, Private`k - Private`stp, Private`k + Private`stp}, 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, Private`varlis]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, Private`varlis]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, Private`n, "Washer", Private`opts], 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, Private`varlis]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, Private`varlis]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, Private`ang, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, {Private`n, Private`ang}, "Washer", Private`opts], 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, {
          Pattern[Private`x, 
           Blank[Symbol]], 
          Pattern[Private`x0, 
           Blank[]], 
          Pattern[Private`xm, 
           Blank[]], 
          Pattern[Private`x1, 
           Blank[]]}, 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, {
                Private`x, Private`x0, Private`xm}]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, {
                Private`x, Private`xm, Private`x1}]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`xm, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, Private`n, "Washer", Private`opts], 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, {
          Pattern[Private`x, 
           Blank[Symbol]], 
          Pattern[Private`x0, 
           Blank[]], 
          Pattern[Private`xm, 
           Blank[]], 
          Pattern[Private`x1, 
           Blank[]]}, 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, {
                Private`x, Private`x0, Private`xm}]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, {
                Private`x, Private`xm, Private`x1}]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, Private`ang, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`xm, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, {Private`n, Private`ang}, "Washer", Private`opts], 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, {
          Pattern[Private`x, 
           Blank[Symbol]], 
          Pattern[Private`x0, 
           Blank[]], 
          Pattern[Private`xm1, 
           Blank[]], 
          Pattern[Private`xm2, 
           Blank[]], 
          Pattern[Private`x1, 
           Blank[]]}, 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, {
                Private`x, Private`x0, Private`xm1}]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, {
                Private`x, Private`xm2, Private`x1}]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`xm1, 
            Blank[]], 
           Pattern[Private`xm2, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, Private`n, "Washer", Private`opts], 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, {
          Pattern[Private`x, 
           Blank[Symbol]], 
          Pattern[Private`x0, 
           Blank[]], 
          Pattern[Private`xm1, 
           Blank[]], 
          Pattern[Private`xm2, 
           Blank[]], 
          Pattern[Private`x1, 
           Blank[]]}, 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, {
                Private`x, Private`x0, Private`xm1}]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, {
                Private`x, Private`xm2, Private`x1}]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, Private`ang, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`xm1, 
            Blank[]], 
           Pattern[Private`xm2, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, {Private`n, Private`ang}, "Washer", Private`opts], 
       Options[Private`riemannSumPlot3D] = {
        AlignmentPoint -> Center, AspectRatio -> Automatic, 
         AutomaticImageSize -> False, Axes -> False, AxesEdge -> Automatic, 
         AxesLabel -> None, AxesOrigin -> Automatic, AxesStyle -> {}, 
         Background -> None, BaselinePosition -> Automatic, BaseStyle -> {}, 
         Boxed -> True, BoxRatios -> Automatic, BoxStyle -> {}, 
         ClassroomUtilities`CapStyle -> {
           EdgeForm[
            GrayLevel[0.65]], 
           GrayLevel[0.4]}, ColorOutput -> Automatic, ContentSelectable -> 
         Automatic, ControllerLinking -> Automatic, ControllerMethod -> 
         Automatic, ControllerPath -> Automatic, CoordinatesToolOptions -> 
         Automatic, DisplayFunction :> $DisplayFunction, Epilog -> {}, 
         FaceGrids -> None, FaceGridsStyle -> {}, FormatType :> 
         TraditionalForm, ImageMargins -> 0., ImagePadding -> All, ImageSize -> 
         Automatic, LabelStyle -> {}, Lighting -> Automatic, Method -> 
         Automatic, PlotLabel -> None, PlotRange -> All, PlotRangePadding -> 
         Automatic, PlotRegion -> Automatic, PlotStyle -> Automatic, 
         PreserveImageOptions -> Automatic, Prolog -> {}, RotationAction -> 
         "Fit", SphericalRegion -> False, Ticks -> Automatic, 
         TicksStyle -> {}, ViewAngle -> Automatic, ViewCenter -> Automatic, 
         ViewMatrix -> Automatic, ViewPoint -> {1.3, -2.4, 2.}, ViewRange -> 
         All, ViewVector -> Automatic, ViewVertical -> {0, 0, 1}}, 
       Private`makeRadiusFunction[
         Pattern[Private`lis, 
          Blank[List]], 
         Pattern[Private`varlis, {
           Pattern[Private`var, 
            Blank[Symbol]], 
           PatternTest[
            Pattern[Private`varmin, 
             Blank[]], NumericQ], 
           PatternTest[
            Pattern[Private`varmax, 
             Blank[]], NumericQ]}]] := 
       Block[{Private`invrs = 
          Map[Private`interpInv[#, Private`varlis]& , Private`lis], 
          Private`res}, 
         If[
          Private`validTst[
           Part[Private`invrs, All, 1, 1]], 
          Apply[Function, 
           ReplaceAll[
            FullForm[
             Private`makePcws[Private`invrs, Private`var]], {
            Private`var :> #}]], $Failed]], Private`interpInv[
         Pattern[Private`f1, 
          Except[
           Blank[ClassroomUtilities`VerticalSegment]]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]]}, 
         Optional[
          Pattern[Private`n, 
           Blank[Integer]], 50]] := 
       Block[{Private`valTbl = 
          Table[{Private`f1, Private`var}, {
            Private`var, Private`varmin, 
             Private`varmax, (Private`varmax - Private`varmin)/Private`n}]}, 
         Interpolation[Private`valTbl, Method -> "Spline"]], Private`interpInv[
         Pattern[Private`f1, 
          Blank[ClassroomUtilities`VerticalSegment]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]]}] := 
       ReplaceAll[Private`f1, Part[Private`f1, 2, 2, 3] :> Private`var], 
       TagSet[ClassroomUtilities`VerticalSegment, 
        MessageName[ClassroomUtilities`VerticalSegment, "usage"], 
        "VerticalSegment[int,{val,ineq}] creates a vertical line segment on \
the interval int for use in MakeRadiusFunction."], Private`validTst[
         Pattern[Private`lis, 
          Blank[List]]] := OrderedQ[
         Flatten[
          SortBy[Private`lis, First]]], Private`makePcws[
         Pattern[Private`lis, {
           Repeated[
            Alternatives[
             Blank[InterpolatingFunction], 
             Blank[ClassroomUtilities`VerticalSegment]]]}], 
         Pattern[Private`var, 
          Blank[Symbol]]] := Piecewise[
         Map[Private`pcwsComponent[#, Private`var]& , Private`lis]], 
       Private`pcwsComponent[
         Pattern[Private`f1, 
          Blank[InterpolatingFunction]], 
         Pattern[Private`var, 
          Blank[Symbol]]] := {
         Private`f1[Private`var], 
         Private`makePcwsInt[
          Part[Private`f1, 1, 1], Private`var]}, Private`pcwsComponent[
         Pattern[Private`f1, 
          Blank[ClassroomUtilities`VerticalSegment]], 
         Pattern[Private`var, 
          Blank[Symbol]]] := 
       Block[{Private`fun = Private`GetFunctionDefinition[Private`f1]}, 
         ReplaceAll[Private`fun, Part[Private`fun, 2, 3] :> Private`var]], 
       Private`makePcwsInt[{
          Pattern[Private`a0, 
           Blank[]], 
          Pattern[Private`a1, 
           Blank[]]}, 
         Pattern[Private`var, 
          Blank[Symbol]]] := 
       Inequality[Private`a0, LessEqual, Private`var, Less, Private`a1], 
       Private`GetFunctionDefinition[
         Pattern[Private`vrt, 
          Blank[ClassroomUtilities`VerticalSegment]]] := 
       Part[Private`vrt, 2], $DisplayFunction = Identity, 
       ClassroomUtilities`MakeRadiusFunction[
         Pattern[Private`lis, 
          Blank[List]], 
         Pattern[Private`varlis, {
           Pattern[Private`var, 
            Blank[Symbol]], 
           PatternTest[
            Pattern[Private`varmin, 
             Blank[]], NumericQ], 
           PatternTest[
            Pattern[Private`varmax, 
             Blank[]], NumericQ]}]] := 
       Block[{Private`res = 
          Private`makeRadiusFunction[Private`lis, Private`varlis]}, 
         If[Private`res === $Failed, 
           Message[
            MessageName[ClassroomUtilities`MakeRadiusFunction, "ovrlp"], 
            "radii"]]; Condition[Private`res, Private`res =!= $Failed]], 
       TagSet[ClassroomUtilities`MakeRadiusFunction, 
        MessageName[ClassroomUtilities`MakeRadiusFunction, "ovrlp"], 
        "Two or more of the inverse function support intervals for the `1` \
are not disjoint."], 
       TagSet[ClassroomUtilities`MakeRadiusFunction, 
        MessageName[ClassroomUtilities`MakeRadiusFunction, "usage"], 
        "MakeRadiusFunction[fnlis,{var,varmin,varmax}] creates an \
interpolating function for the inverses of the functions in fnlis."]}; 
     Typeset`initDone$$ = True),
    SynchronousInitialization->True,
    UnsavedVariables:>{Typeset`initDone$$},
    UntrackedVariables:>{Typeset`size$$}], "Manipulate",
   Deployed->True,
   StripOnInput->False],
  Manipulate`InterpretManipulate[1]]], "Output",
 ImageSize->{476, 259},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->409097559]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Shells", "Subsection",
 CellID->772804348],

Cell["A manipulate using 8 shells.", "MathCaption",
 CellID->541579780],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Manipulate", "[", 
  RowBox[{
   RowBox[{"RiemannSumPlot3D", "[", 
    RowBox[{
     SuperscriptBox["x", "2"], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"8", ",", "ang"}], "}"}], ",", "\"\<Shell\>\"", ",", 
     RowBox[{"PlotRange", "\[Rule]", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "2"}], ",", "2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "2"}], ",", "2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "4"}], "}"}]}], "}"}]}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"ang", ",", "0", ",", 
     RowBox[{"2", "\[Pi]"}]}], "}"}], ",", 
   RowBox[{"SaveDefinitions", " ", "\[Rule]", " ", "True"}]}], "]"}]], "Input",
 CellLabel->"In[12]:=",
 CellID->1295666474],

Cell[BoxData[
 TagBox[
  StyleBox[
   DynamicModuleBox[{$CellContext`ang$$ = 4.09663682028109, Typeset`show$$ = 
    True, Typeset`bookmarkList$$ = {}, Typeset`bookmarkMode$$ = "Menu", 
    Typeset`animator$$, Typeset`animvar$$ = 1, Typeset`name$$ = 
    "\"untitled\"", Typeset`specs$$ = {{
      Hold[$CellContext`ang$$], 0, 2 Pi}}, Typeset`size$$ = {
    180., {95., 100.}}, Typeset`update$$ = 0, Typeset`initDone$$, 
    Typeset`skipInitDone$$ = False, $CellContext`ang$2028$$ = 0}, 
    DynamicBox[Manipulate`ManipulateBoxes[
     1, StandardForm, "Variables" :> {$CellContext`ang$$ = 0}, 
      "ControllerVariables" :> {
        Hold[$CellContext`ang$$, $CellContext`ang$2028$$, 0]}, 
      "OtherVariables" :> {
       Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
        Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
        Typeset`specs$$, Typeset`size$$, Typeset`update$$, Typeset`initDone$$,
         Typeset`skipInitDone$$}, "Body" :> 
      ClassroomUtilities`RiemannSumPlot3D[$CellContext`x^2, {$CellContext`x, 
         0, 2}, {8, $CellContext`ang$$}, "Shell", 
        PlotRange -> {{-2, 2}, {-2, 2}, {0, 4}}], 
      "Specifications" :> {{$CellContext`ang$$, 0, 2 Pi}}, "Options" :> {}, 
      "DefaultOptions" :> {}],
     ImageSizeCache->{472., {126., 131.}},
     SingleEvaluation->True],
    Deinitialization:>None,
    DynamicModuleValues:>{},
    Initialization:>({ClassroomUtilities`RiemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, 
         Pattern[Private`n, 
          Blank[Integer]], "Shell", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           ClassroomUtilities`Shell[
            Table[{Private`r, Private`r + Private`stp, 
              ReplaceAll[
              Private`f1, Private`var :> Private`r + Private`stp/2], 
              ReplaceAll[
              Private`f2, Private`var :> Private`r + Private`stp/2]}, {
             Private`r, Private`varmin, Private`varmax - Private`stp, 
              Private`stp}], ClassroomUtilities`CapStyle -> Private`cpstyl]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`f, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, 
         Pattern[Private`n, 
          Blank[Integer]], "Shell", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           ClassroomUtilities`Shell[
            Table[{Private`r, Private`r + Private`stp, 0, 
              ReplaceAll[
              Private`f, Private`var :> Private`r + Private`stp/2]}, {
             Private`r, Private`varmin, Private`varmax - Private`stp, 
              Private`stp}], ClassroomUtilities`CapStyle -> Private`cpstyl]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, {
          Pattern[Private`n, 
           Blank[Integer]], 
          PatternTest[
           Pattern[Private`ang, 
            Blank[]], NumericQ]}, "Shell", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           ClassroomUtilities`Shell[
            Table[{Private`r, Private`r + Private`stp, 
              ReplaceAll[
              Private`f1, Private`var :> Private`r + Private`stp/2], 
              ReplaceAll[
              Private`f2, Private`var :> Private`r + Private`stp/2]}, {
             Private`r, Private`varmin, Private`varmax - Private`stp, 
              Private`stp}], Private`ang, ClassroomUtilities`CapStyle -> 
            Private`cpstyl]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`f, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, {
          Pattern[Private`n, 
           Blank[Integer]], 
          PatternTest[
           Pattern[Private`ang, 
            Blank[]], NumericQ]}, "Shell", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           ClassroomUtilities`Shell[
            Table[{Private`r, Private`r + Private`stp, 0, 
              ReplaceAll[
              Private`f, Private`var :> Private`r + Private`stp/2]}, {
             Private`r, Private`varmin, Private`varmax - Private`stp, 
              Private`stp}], Private`ang, ClassroomUtilities`CapStyle -> 
            Private`cpstyl]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`f, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, 
         Pattern[Private`n, 
          Blank[Integer]], "Disk", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           Private`makeDisks[
           Private`f, {Private`var, Private`varmin, Private`varmax}, 
            Private`n]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`f, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, {
          Pattern[Private`n, 
           Blank[Integer]], 
          PatternTest[
           Pattern[Private`ang, 
            Blank[]], NumericQ]}, "Disk", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           Private`makeDisks[
           Private`f, {Private`var, Private`varmin, Private`varmax}, 
            Private`n, Private`ang, ClassroomUtilities`CapStyle -> 
            Private`cpstyl]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`fnLis, 
          Blank[List]], 
         Pattern[Private`varlis, 
          Blank[List]], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`res22 = 
          Private`riemannSumPlot3D[
           Private`fnLis, Private`varlis, Private`rnglis, Private`n, "Washer",
             Private`opts]}, If[
           MatchQ[Private`res22, {$Failed, 
             Alternatives[1, 2]}], 
           Message[
            MessageName[ClassroomUtilities`MakeRadiusFunction, "ovrlp"], 
            Part[{"inner radius", "outer radius"}, 
             Part[Private`res22, 2]]]]; Condition[Private`res22, 
           FreeQ[Private`res22, $Failed]]], 
       ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`fnLis, 
          Blank[List]], 
         Pattern[Private`varlis, 
          Blank[List]], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`res22 = 
          Private`riemannSumPlot3D[
           Private`fnLis, Private`varlis, Private`rnglis, {
            Private`n, Private`ang}, "Washer", Private`opts]}, If[
           MatchQ[Private`res22, {$Failed, 
             Alternatives[1, 2]}], 
           Message[
            MessageName[ClassroomUtilities`MakeRadiusFunction, "ovrlp"], 
            Part[{"inner radius", "outer radius"}, 
             Part[Private`res22, 2]]]]; Condition[Private`res22, 
           FreeQ[Private`res22, $Failed]]], 
       Options[ClassroomUtilities`RiemannSumPlot3D] = {
        AlignmentPoint -> Center, AspectRatio -> Automatic, 
         AutomaticImageSize -> False, Axes -> False, AxesEdge -> Automatic, 
         AxesLabel -> None, AxesOrigin -> Automatic, AxesStyle -> {}, 
         Background -> None, BaselinePosition -> Automatic, BaseStyle -> {}, 
         Boxed -> True, BoxRatios -> Automatic, BoxStyle -> {}, 
         ClassroomUtilities`CapStyle -> {
           EdgeForm[
            GrayLevel[0.65]], 
           GrayLevel[0.4]}, ColorOutput -> Automatic, ContentSelectable -> 
         Automatic, ControllerLinking -> Automatic, ControllerMethod -> 
         Automatic, ControllerPath -> Automatic, CoordinatesToolOptions -> 
         Automatic, DisplayFunction :> $DisplayFunction, Epilog -> {}, 
         FaceGrids -> None, FaceGridsStyle -> {}, FormatType :> 
         TraditionalForm, ImageMargins -> 0., ImagePadding -> All, ImageSize -> 
         Automatic, LabelStyle -> {}, Lighting -> Automatic, Method -> 
         Automatic, PlotLabel -> None, PlotRange -> All, PlotRangePadding -> 
         Automatic, PlotRegion -> Automatic, PlotStyle -> Automatic, 
         PreserveImageOptions -> Automatic, Prolog -> {}, RotationAction -> 
         "Fit", SphericalRegion -> False, Ticks -> Automatic, 
         TicksStyle -> {}, ViewAngle -> Automatic, ViewCenter -> Automatic, 
         ViewMatrix -> Automatic, ViewPoint -> {1.3, -2.4, 2.}, ViewRange -> 
         All, ViewVector -> Automatic, ViewVertical -> {0, 0, 1}}, 
       TagSet[ClassroomUtilities`RiemannSumPlot3D, 
        MessageName[ClassroomUtilities`RiemannSumPlot3D, "usage"], 
        "RiemannSumPlot3D[f,{var,varmin,varmax},n,\"type\"] creates n \
partitions of \"type\" to approximate the volume obtained by revolving the \
function f about the vertical axis."], 
       TagSet[ClassroomUtilities`CapStyle, 
        MessageName[ClassroomUtilities`CapStyle, "usage"], 
        "CapStyle is an option for RiemannSumPlot that controls the \
appearance of the flat surfaces of shells and washers."], 
       ClassroomUtilities`Shell[{
          Pattern[Private`r, 
           Blank[]], 
          Pattern[Private`r, 
           Blank[]], 
          BlankSequence[]}, 
         BlankNullSequence[]] := {}, Condition[
         ClassroomUtilities`Shell[{
           PatternTest[
            Pattern[Private`r1, 
             Blank[]], NumericQ], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       With[{Private`lis1 = Range[0., 2 Pi - Pi/36, Pi/36], Private`lis2 = 
          ConstantArray[1., 72], Private`rimPos = Join[
            Flatten[
             
             Table[{{73, 1, 2}, {2, 73, 74}, {217, 145, 146}, {146, 217, 
               218}} + Private`i, {Private`i, 0, 70}], 1], {{144, 72, 1}, {1, 
            144, 73}, {288, 216, 145}, {145, 288, 217}}], Private`flatPts1 = 
          Join[
            Range[72], {1, 145}, 
            Range[216, 145, -1]], Private`cpstyl = ReplaceAll[
            OptionValue[ClassroomUtilities`CapStyle], List -> Directive]}, 
         GraphicsComplex[
          Join[
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k2], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k2]], {
           EdgeForm[], 
           Polygon[Private`rimPos], Private`cpstyl, 
           Polygon[{Private`flatPts1, Private`flatPts1 + 72}]}]], 
       ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], 
         Pattern[Private`opts, 
          OptionsPattern[]]] := With[{Private`cpstyl = ReplaceAll[
            OptionValue[ClassroomUtilities`CapStyle], List -> Directive]}, 
         Map[
         ClassroomUtilities`Shell[#, ClassroomUtilities`CapStyle -> 
           Private`cpstyl]& , Private`shls]], Condition[
         ClassroomUtilities`Shell[{
           PatternTest[
            Pattern[Private`r1, 
             Blank[]], NumericQ], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, {
           Condition[
            PatternTest[
             Pattern[Private`stp, 
              Blank[Integer]], Positive], 
            Inequality[1, LessEqual, Private`stp, Less, 72]]}, 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       With[{Private`lis1 = Range[0., 2 Pi - Pi/36, Pi/36], Private`lis2 = 
          ConstantArray[1., 72], Private`rimPos = Flatten[
            
            Table[{{73, 1, 2}, {2, 73, 74}, {217, 145, 146}, {146, 217, 218}} + 
             Private`i, {Private`i, 0, Private`stp - 1}], 1], 
          Private`flatPts1 = Join[
            Take[
             Range[72], Private`stp + 1], 
            Take[
             Range[216, 145, -1], -(Private`stp + 1)]], Private`cpstyl = 
          ReplaceAll[
            OptionValue[ClassroomUtilities`CapStyle], List -> Directive]}, 
         GraphicsComplex[
          Join[
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k2], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k2]], {
           EdgeForm[], 
           Polygon[Private`rimPos], Private`cpstyl, 
           Polygon[{Private`flatPts1, Private`flatPts1 + 72}], 
           
           Polygon[{{1, 73, 217, 145}, {
             Private`stp + 1, Private`stp + 73, Private`stp + 217, 
              Private`stp + 145}}]}]], ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], {
          Condition[
           PatternTest[
            Pattern[Private`stp, 
             Blank[Integer]], Positive], 
           Inequality[1, LessEqual, Private`stp, Less, 72]]}, 
         Pattern[Private`opts, 
          OptionsPattern[]]] := With[{Private`cpstyl = ReplaceAll[
            OptionValue[ClassroomUtilities`CapStyle], List -> Directive]}, 
         Map[
         ClassroomUtilities`Shell[#, {Private`stp}, 
           ClassroomUtilities`CapStyle -> Private`cpstyl]& , Private`shls]], 
       Condition[
         ClassroomUtilities`Shell[{
           PatternTest[
            Pattern[Private`r1, 
             Blank[]], NumericQ], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, {0}, 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       With[{Private`lis1 = Range[0., 2 Pi - Pi/36, Pi/36], Private`lis2 = 
          ConstantArray[1., 72], Private`cpstyl = ReplaceAll[
            OptionValue[ClassroomUtilities`CapStyle], List -> Directive]}, 
         GraphicsComplex[
          Join[
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k2], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k2]], {
           EdgeForm[], Private`cpstyl, 
           Polygon[{{1, 73, 217, 145}}]}]], ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], {0}, 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Map[ClassroomUtilities`Shell[#, {0}, Private`opts]& , Private`shls], 
       Condition[
         ClassroomUtilities`Shell[{
           Pattern[Private`r1, 
            Blank[]], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, {72}, 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       ClassroomUtilities`Shell[{
         Private`r1, Private`r2, Private`k1, Private`k2}, Private`opts], 
       ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], {72}, 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       ClassroomUtilities`Shell[Private`shls, Private`opts], Condition[
         ClassroomUtilities`Shell[{
           Pattern[Private`r1, 
            Blank[]], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, 
          PatternTest[
           Pattern[Private`nn, 
            Blank[]], Positive], 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       Block[{Private`rr = ReplaceAll[
            Part[
             Private`closest[Private`nn], 1], {1 -> 72}]}, 
         ClassroomUtilities`Shell[{
          Private`r1, Private`r2, Private`k1, Private`k2}, Private`rr, 
          Private`opts]], ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], 
         PatternTest[
          Pattern[Private`nn, 
           Blank[]], Positive], 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`rr = ReplaceAll[
            Part[
             Private`closest[Private`nn], 1], {1 -> 72}]}, 
         ClassroomUtilities`Shell[Private`shls, Private`rr, Private`opts]], 
       Condition[
         ClassroomUtilities`Shell[{
           Pattern[Private`r1, 
            Blank[]], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, 0, 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       ClassroomUtilities`Shell[{
         Private`r1, Private`r2, Private`k1, Private`k2}, {0}, Private`opts], 
       ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], 0, 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       ClassroomUtilities`Shell[Private`shls, {0}, Private`opts], Condition[
         ClassroomUtilities`Shell[{
           Pattern[Private`r1, 
            Blank[]], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, 0., 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       ClassroomUtilities`Shell[{
         Private`r1, Private`r2, Private`k1, Private`k2}, {0}, Private`opts], 
       ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], 0., 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       ClassroomUtilities`Shell[Private`shls, {0}, Private`opts], 
       Options[ClassroomUtilities`Shell] = {
        ClassroomUtilities`CapStyle -> GrayLevel[0.75]}, 
       TagSet[ClassroomUtilities`Shell, 
        MessageName[ClassroomUtilities`Shell, "usage"], 
        "Shell[{r1,r2,btm,tp}] construct a washer with inner radius r1, outer \
radius r2 and thickness tp-btm."], Private`makePts[
         Pattern[Private`lis1, 
          Blank[List]], 
         Pattern[Private`lis2, 
          Blank[List]], 
         PatternTest[
          Pattern[Private`r, 
           Blank[]], NumericQ], 
         PatternTest[
          Pattern[Private`k, 
           Blank[]], NumericQ]] := 
       Transpose[{
         Private`r Cos[Private`lis1], Private`r Sin[Private`lis1], Private`k 
          Private`lis2}], Private`closest[
         PatternTest[
          Pattern[Private`val, 
           Blank[]], NumericQ]] := 
       With[{Private`vals = Range[0, 2 Pi - Pi/36, Pi/36], Private`val2 = Mod[
            N[Private`val], 2 Pi]}, 
         Position[
          Abs[Private`vals - Private`val2], 
          Min[
           Abs[Private`vals - Private`val2]]]], Private`makeDisks[
         Pattern[Private`fun, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NumericQ], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], NumericQ]}, 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/100, 
          Private`stpV, Private`funPrsV, Private`newFn}, 
         Private`newFn = 
          Private`invFun[
           Private`fun, {Private`var, Private`varmin, Private`varmax}]; {
           Private`stpV, Private`funPrsV} = 
          Private`vertFunPrs[Private`newFn, Private`n]; Cylinder[
           Map[{{0, 0, # - Private`stpV/2}, {0, 0, # + Private`stpV/2}}& , 
            Part[Private`funPrsV, All, 1]], 
           Part[Private`funPrsV, All, 2]]], Private`makeDisks[
         Pattern[Private`fun, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NumericQ], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], NumericQ]}, 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], 
         Pattern[Private`ang, 
          Blank[]], 
         Pattern[Private`opts, 
          OptionsPattern[ClassroomUtilities`CapStyle -> Automatic]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/100, 
          Private`stpV, Private`funPrsV, Private`newFn, Private`cpStyl = 
          OptionValue[ClassroomUtilities`CapStyle]}, 
         Private`newFn = 
          Private`invFun[
           Private`fun, {Private`var, Private`varmin, Private`varmax}]; {
           Private`stpV, Private`funPrsV} = 
          Private`vertFunPrs[Private`newFn, Private`n]; 
         ClassroomUtilities`Shell[
           Map[Private`shllFun[#, Private`stpV/2]& , Private`funPrsV], 
           Private`ang, ClassroomUtilities`CapStyle -> Private`cpStyl]], 
       Private`invFun[
         Pattern[Private`fun, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]]}] := 
       Block[{Private`funPrs, 
          Private`stp = (Private`varmax - Private`varmin)/100}, 
         Private`funPrs = Table[{
             Function[Private`var, Private`fun][Private`var2], 
             Private`var2}, {Private`var2, 
             Range[Private`varmin, Private`varmax, Private`stp]}]; 
         Interpolation[Private`funPrs, Method -> "Spline"]], 
       Private`vertFunPrs[
         Pattern[Private`fun, 
          Blank[InterpolatingFunction]], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive]] := Block[{Private`stpV = Part[
             Differences[
              Part[Private`fun, 1, 1]], 1]/Private`n}, {Private`stpV, 
          Map[{#, 
            Private`fun[#]}& , 
           Range[Part[Private`fun, 1, 1, 1] + Private`stpV/2, 
            Part[Private`fun, 1, 1, 2], Private`stpV]]}], Private`vertFunPrs[{
          Pattern[Private`minval, 
           Blank[]], {
           Pattern[Private`fun, 
            Blank[]], 
           Pattern[Private`fun2, 
            Blank[]]}}, {
          Pattern[Private`var, 
           Blank[]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]], 
          Pattern[Private`stp, 
           Blank[]]}] := Table[{Private`minval, 
          Min[{Private`fun, Private`fun2}], Private`var - Private`stp/2, 
          Private`var + Private`stp/2}, {
         Private`var, Private`varmin + Private`stp/2, Private`varmax, 
          Private`stp}], Private`vertFunPrs[{{
           Pattern[Private`fun, 
            Blank[]], 
           Pattern[Private`fun2, 
            Blank[]]}, 
          Pattern[Private`maxval, 
           Blank[]]}, {
          Pattern[Private`var, 
           Blank[]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]], 
          Pattern[Private`stp, 
           Blank[]]}] := Table[{Private`maxval, 
          Max[{Private`fun, Private`fun2}], Private`var - Private`stp/2, 
          Private`var + Private`stp/2}, {
         Private`var, Private`varmin + Private`stp/2, Private`varmax, 
          Private`stp}], Private`vertFunPrs[{
          Pattern[Private`fun, 
           Blank[]], 
          Pattern[Private`fun2, 
           Blank[]]}, {
          Pattern[Private`var, 
           Blank[]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]], 
          Pattern[Private`stp, 
           Blank[]]}] := 
       Table[{Private`fun, Private`fun2, Private`var - Private`stp/2, 
          Private`var + Private`stp/2}, {
         Private`var, Private`varmin + Private`stp/2, Private`varmax, 
          Private`stp}], Private`shllFun[{
          Pattern[Private`k, 
           Blank[]], 
          Pattern[Private`r, 
           Blank[]]}, 
         PatternTest[
          Pattern[Private`stp, 
           Blank[]], Positive]] := {
        0, Private`r, Private`k - Private`stp, Private`k + Private`stp}, 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, Private`varlis]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, Private`varlis]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, Private`n, "Washer", Private`opts], 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, Private`varlis]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, Private`varlis]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, Private`ang, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, {Private`n, Private`ang}, "Washer", Private`opts], 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, {
          Pattern[Private`x, 
           Blank[Symbol]], 
          Pattern[Private`x0, 
           Blank[]], 
          Pattern[Private`xm, 
           Blank[]], 
          Pattern[Private`x1, 
           Blank[]]}, 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, {
                Private`x, Private`x0, Private`xm}]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, {
                Private`x, Private`xm, Private`x1}]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`xm, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, Private`n, "Washer", Private`opts], 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, {
          Pattern[Private`x, 
           Blank[Symbol]], 
          Pattern[Private`x0, 
           Blank[]], 
          Pattern[Private`xm, 
           Blank[]], 
          Pattern[Private`x1, 
           Blank[]]}, 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, {
                Private`x, Private`x0, Private`xm}]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, {
                Private`x, Private`xm, Private`x1}]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, Private`ang, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`xm, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, {Private`n, Private`ang}, "Washer", Private`opts], 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, {
          Pattern[Private`x, 
           Blank[Symbol]], 
          Pattern[Private`x0, 
           Blank[]], 
          Pattern[Private`xm1, 
           Blank[]], 
          Pattern[Private`xm2, 
           Blank[]], 
          Pattern[Private`x1, 
           Blank[]]}, 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, {
                Private`x, Private`x0, Private`xm1}]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, {
                Private`x, Private`xm2, Private`x1}]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`xm1, 
            Blank[]], 
           Pattern[Private`xm2, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, Private`n, "Washer", Private`opts], 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, {
          Pattern[Private`x, 
           Blank[Symbol]], 
          Pattern[Private`x0, 
           Blank[]], 
          Pattern[Private`xm1, 
           Blank[]], 
          Pattern[Private`xm2, 
           Blank[]], 
          Pattern[Private`x1, 
           Blank[]]}, 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, {
                Private`x, Private`x0, Private`xm1}]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, {
                Private`x, Private`xm2, Private`x1}]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, Private`ang, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`xm1, 
            Blank[]], 
           Pattern[Private`xm2, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, {Private`n, Private`ang}, "Washer", Private`opts], 
       Options[Private`riemannSumPlot3D] = {
        AlignmentPoint -> Center, AspectRatio -> Automatic, 
         AutomaticImageSize -> False, Axes -> False, AxesEdge -> Automatic, 
         AxesLabel -> None, AxesOrigin -> Automatic, AxesStyle -> {}, 
         Background -> None, BaselinePosition -> Automatic, BaseStyle -> {}, 
         Boxed -> True, BoxRatios -> Automatic, BoxStyle -> {}, 
         ClassroomUtilities`CapStyle -> {
           EdgeForm[
            GrayLevel[0.65]], 
           GrayLevel[0.4]}, ColorOutput -> Automatic, ContentSelectable -> 
         Automatic, ControllerLinking -> Automatic, ControllerMethod -> 
         Automatic, ControllerPath -> Automatic, CoordinatesToolOptions -> 
         Automatic, DisplayFunction :> $DisplayFunction, Epilog -> {}, 
         FaceGrids -> None, FaceGridsStyle -> {}, FormatType :> 
         TraditionalForm, ImageMargins -> 0., ImagePadding -> All, ImageSize -> 
         Automatic, LabelStyle -> {}, Lighting -> Automatic, Method -> 
         Automatic, PlotLabel -> None, PlotRange -> All, PlotRangePadding -> 
         Automatic, PlotRegion -> Automatic, PlotStyle -> Automatic, 
         PreserveImageOptions -> Automatic, Prolog -> {}, RotationAction -> 
         "Fit", SphericalRegion -> False, Ticks -> Automatic, 
         TicksStyle -> {}, ViewAngle -> Automatic, ViewCenter -> Automatic, 
         ViewMatrix -> Automatic, ViewPoint -> {1.3, -2.4, 2.}, ViewRange -> 
         All, ViewVector -> Automatic, ViewVertical -> {0, 0, 1}}, 
       Private`makeRadiusFunction[
         Pattern[Private`lis, 
          Blank[List]], 
         Pattern[Private`varlis, {
           Pattern[Private`var, 
            Blank[Symbol]], 
           PatternTest[
            Pattern[Private`varmin, 
             Blank[]], NumericQ], 
           PatternTest[
            Pattern[Private`varmax, 
             Blank[]], NumericQ]}]] := 
       Block[{Private`invrs = 
          Map[Private`interpInv[#, Private`varlis]& , Private`lis], 
          Private`res}, 
         If[
          Private`validTst[
           Part[Private`invrs, All, 1, 1]], 
          Apply[Function, 
           ReplaceAll[
            FullForm[
             Private`makePcws[Private`invrs, Private`var]], {
            Private`var :> #}]], $Failed]], Private`interpInv[
         Pattern[Private`f1, 
          Except[
           Blank[ClassroomUtilities`VerticalSegment]]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]]}, 
         Optional[
          Pattern[Private`n, 
           Blank[Integer]], 50]] := 
       Block[{Private`valTbl = 
          Table[{Private`f1, Private`var}, {
            Private`var, Private`varmin, 
             Private`varmax, (Private`varmax - Private`varmin)/Private`n}]}, 
         Interpolation[Private`valTbl, Method -> "Spline"]], Private`interpInv[
         Pattern[Private`f1, 
          Blank[ClassroomUtilities`VerticalSegment]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]]}] := 
       ReplaceAll[Private`f1, Part[Private`f1, 2, 2, 3] :> Private`var], 
       TagSet[ClassroomUtilities`VerticalSegment, 
        MessageName[ClassroomUtilities`VerticalSegment, "usage"], 
        "VerticalSegment[int,{val,ineq}] creates a vertical line segment on \
the interval int for use in MakeRadiusFunction."], Private`validTst[
         Pattern[Private`lis, 
          Blank[List]]] := OrderedQ[
         Flatten[
          SortBy[Private`lis, First]]], Private`makePcws[
         Pattern[Private`lis, {
           Repeated[
            Alternatives[
             Blank[InterpolatingFunction], 
             Blank[ClassroomUtilities`VerticalSegment]]]}], 
         Pattern[Private`var, 
          Blank[Symbol]]] := Piecewise[
         Map[Private`pcwsComponent[#, Private`var]& , Private`lis]], 
       Private`pcwsComponent[
         Pattern[Private`f1, 
          Blank[InterpolatingFunction]], 
         Pattern[Private`var, 
          Blank[Symbol]]] := {
         Private`f1[Private`var], 
         Private`makePcwsInt[
          Part[Private`f1, 1, 1], Private`var]}, Private`pcwsComponent[
         Pattern[Private`f1, 
          Blank[ClassroomUtilities`VerticalSegment]], 
         Pattern[Private`var, 
          Blank[Symbol]]] := 
       Block[{Private`fun = Private`GetFunctionDefinition[Private`f1]}, 
         ReplaceAll[Private`fun, Part[Private`fun, 2, 3] :> Private`var]], 
       Private`makePcwsInt[{
          Pattern[Private`a0, 
           Blank[]], 
          Pattern[Private`a1, 
           Blank[]]}, 
         Pattern[Private`var, 
          Blank[Symbol]]] := 
       Inequality[Private`a0, LessEqual, Private`var, Less, Private`a1], 
       Private`GetFunctionDefinition[
         Pattern[Private`vrt, 
          Blank[ClassroomUtilities`VerticalSegment]]] := 
       Part[Private`vrt, 2], $DisplayFunction = Identity, 
       ClassroomUtilities`MakeRadiusFunction[
         Pattern[Private`lis, 
          Blank[List]], 
         Pattern[Private`varlis, {
           Pattern[Private`var, 
            Blank[Symbol]], 
           PatternTest[
            Pattern[Private`varmin, 
             Blank[]], NumericQ], 
           PatternTest[
            Pattern[Private`varmax, 
             Blank[]], NumericQ]}]] := 
       Block[{Private`res = 
          Private`makeRadiusFunction[Private`lis, Private`varlis]}, 
         If[Private`res === $Failed, 
           Message[
            MessageName[ClassroomUtilities`MakeRadiusFunction, "ovrlp"], 
            "radii"]]; Condition[Private`res, Private`res =!= $Failed]], 
       TagSet[ClassroomUtilities`MakeRadiusFunction, 
        MessageName[ClassroomUtilities`MakeRadiusFunction, "ovrlp"], 
        "Two or more of the inverse function support intervals for the `1` \
are not disjoint."], 
       TagSet[ClassroomUtilities`MakeRadiusFunction, 
        MessageName[ClassroomUtilities`MakeRadiusFunction, "usage"], 
        "MakeRadiusFunction[fnlis,{var,varmin,varmax}] creates an \
interpolating function for the inverses of the functions in fnlis."]}; 
     Typeset`initDone$$ = True),
    SynchronousInitialization->True,
    UnsavedVariables:>{Typeset`initDone$$},
    UntrackedVariables:>{Typeset`size$$}], "Manipulate",
   Deployed->True,
   StripOnInput->False],
  Manipulate`InterpretManipulate[1]]], "Output",
 ImageSize->{476, 259},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->1607520066]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Washers", "Subsection",
 CellID->346207032],

Cell["\<\
Visualizing washers with a cross section enclosed by two curves.\
\>", "MathCaption",
 CellID->1168387101],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Manipulate", "[", 
  RowBox[{
   RowBox[{"RiemannSumPlot3D", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Sin", "[", 
        RowBox[{"x", "-", 
         RowBox[{"\[Pi]", "/", "4"}]}], "]"}], ",", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"2", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"x", "-", 
             RowBox[{"\[Pi]", "/", "4"}]}], ")"}], "/", 
           RowBox[{"(", "\[Pi]", ")"}]}]}], ")"}], "2"]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"\[Pi]", "/", "4"}], ",", 
       RowBox[{"3", 
        RowBox[{"\[Pi]", "/", "4"}]}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"8", ",", "ang"}], "}"}], ",", "\"\<Washer\>\"", ",", 
     RowBox[{"PlotRange", "\[Rule]", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "2.4"}], ",", "2.4"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "2.4"}], ",", "2.4"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"ang", ",", "0", ",", " ", 
     RowBox[{"2", " ", "\[Pi]"}]}], "}"}], ",", 
   RowBox[{"SaveDefinitions", "\[Rule]", " ", "True"}]}], "]"}]], "Input",
 CellLabel->"In[13]:=",
 CellID->491538339],

Cell[BoxData[
 TagBox[
  StyleBox[
   DynamicModuleBox[{$CellContext`ang$$ = 4.09663682028109, Typeset`show$$ = 
    True, Typeset`bookmarkList$$ = {}, Typeset`bookmarkMode$$ = "Menu", 
    Typeset`animator$$, Typeset`animvar$$ = 1, Typeset`name$$ = 
    "\"untitled\"", Typeset`specs$$ = {{
      Hold[$CellContext`ang$$], 0, 2 Pi}}, Typeset`size$$ = {
    180., {62., 67.}}, Typeset`update$$ = 0, Typeset`initDone$$, 
    Typeset`skipInitDone$$ = False, $CellContext`ang$2091$$ = 0}, 
    DynamicBox[Manipulate`ManipulateBoxes[
     1, StandardForm, "Variables" :> {$CellContext`ang$$ = 0}, 
      "ControllerVariables" :> {
        Hold[$CellContext`ang$$, $CellContext`ang$2091$$, 0]}, 
      "OtherVariables" :> {
       Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
        Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
        Typeset`specs$$, Typeset`size$$, Typeset`update$$, Typeset`initDone$$,
         Typeset`skipInitDone$$}, "Body" :> 
      ClassroomUtilities`RiemannSumPlot3D[{
         Sin[$CellContext`x - Pi/4], (
          2 (($CellContext`x - Pi/4)/Pi))^2}, {$CellContext`x, Pi/4, 
         3 (Pi/4)}, {$CellContext`z, 0, 1}, {8, $CellContext`ang$$}, "Washer",
         PlotRange -> {{-2.4, 2.4}, {-2.4, 2.4}, {0, 1}}], 
      "Specifications" :> {{$CellContext`ang$$, 0, 2 Pi}}, "Options" :> {}, 
      "DefaultOptions" :> {}],
     ImageSizeCache->{472., {93., 98.}},
     SingleEvaluation->True],
    Deinitialization:>None,
    DynamicModuleValues:>{},
    Initialization:>({ClassroomUtilities`RiemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, 
         Pattern[Private`n, 
          Blank[Integer]], "Shell", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           ClassroomUtilities`Shell[
            Table[{Private`r, Private`r + Private`stp, 
              ReplaceAll[
              Private`f1, Private`var :> Private`r + Private`stp/2], 
              ReplaceAll[
              Private`f2, Private`var :> Private`r + Private`stp/2]}, {
             Private`r, Private`varmin, Private`varmax - Private`stp, 
              Private`stp}], ClassroomUtilities`CapStyle -> Private`cpstyl]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`f, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, 
         Pattern[Private`n, 
          Blank[Integer]], "Shell", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           ClassroomUtilities`Shell[
            Table[{Private`r, Private`r + Private`stp, 0, 
              ReplaceAll[
              Private`f, Private`var :> Private`r + Private`stp/2]}, {
             Private`r, Private`varmin, Private`varmax - Private`stp, 
              Private`stp}], ClassroomUtilities`CapStyle -> Private`cpstyl]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, {
          Pattern[Private`n, 
           Blank[Integer]], 
          PatternTest[
           Pattern[Private`ang, 
            Blank[]], NumericQ]}, "Shell", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           ClassroomUtilities`Shell[
            Table[{Private`r, Private`r + Private`stp, 
              ReplaceAll[
              Private`f1, Private`var :> Private`r + Private`stp/2], 
              ReplaceAll[
              Private`f2, Private`var :> Private`r + Private`stp/2]}, {
             Private`r, Private`varmin, Private`varmax - Private`stp, 
              Private`stp}], Private`ang, ClassroomUtilities`CapStyle -> 
            Private`cpstyl]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`f, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, {
          Pattern[Private`n, 
           Blank[Integer]], 
          PatternTest[
           Pattern[Private`ang, 
            Blank[]], NumericQ]}, "Shell", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           ClassroomUtilities`Shell[
            Table[{Private`r, Private`r + Private`stp, 0, 
              ReplaceAll[
              Private`f, Private`var :> Private`r + Private`stp/2]}, {
             Private`r, Private`varmin, Private`varmax - Private`stp, 
              Private`stp}], Private`ang, ClassroomUtilities`CapStyle -> 
            Private`cpstyl]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`f, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, 
         Pattern[Private`n, 
          Blank[Integer]], "Disk", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           Private`makeDisks[
           Private`f, {Private`var, Private`varmin, Private`varmax}, 
            Private`n]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`f, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, {
          Pattern[Private`n, 
           Blank[Integer]], 
          PatternTest[
           Pattern[Private`ang, 
            Blank[]], NumericQ]}, "Disk", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           Private`makeDisks[
           Private`f, {Private`var, Private`varmin, Private`varmax}, 
            Private`n, Private`ang, ClassroomUtilities`CapStyle -> 
            Private`cpstyl]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`fnLis, 
          Blank[List]], 
         Pattern[Private`varlis, 
          Blank[List]], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`res22 = 
          Private`riemannSumPlot3D[
           Private`fnLis, Private`varlis, Private`rnglis, Private`n, "Washer",
             Private`opts]}, If[
           MatchQ[Private`res22, {$Failed, 
             Alternatives[1, 2]}], 
           Message[
            MessageName[ClassroomUtilities`MakeRadiusFunction, "ovrlp"], 
            Part[{"inner radius", "outer radius"}, 
             Part[Private`res22, 2]]]]; Condition[Private`res22, 
           FreeQ[Private`res22, $Failed]]], 
       ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`fnLis, 
          Blank[List]], 
         Pattern[Private`varlis, 
          Blank[List]], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`res22 = 
          Private`riemannSumPlot3D[
           Private`fnLis, Private`varlis, Private`rnglis, {
            Private`n, Private`ang}, "Washer", Private`opts]}, If[
           MatchQ[Private`res22, {$Failed, 
             Alternatives[1, 2]}], 
           Message[
            MessageName[ClassroomUtilities`MakeRadiusFunction, "ovrlp"], 
            Part[{"inner radius", "outer radius"}, 
             Part[Private`res22, 2]]]]; Condition[Private`res22, 
           FreeQ[Private`res22, $Failed]]], 
       Options[ClassroomUtilities`RiemannSumPlot3D] = {
        AlignmentPoint -> Center, AspectRatio -> Automatic, 
         AutomaticImageSize -> False, Axes -> False, AxesEdge -> Automatic, 
         AxesLabel -> None, AxesOrigin -> Automatic, AxesStyle -> {}, 
         Background -> None, BaselinePosition -> Automatic, BaseStyle -> {}, 
         Boxed -> True, BoxRatios -> Automatic, BoxStyle -> {}, 
         ClassroomUtilities`CapStyle -> {
           EdgeForm[
            GrayLevel[0.65]], 
           GrayLevel[0.4]}, ColorOutput -> Automatic, ContentSelectable -> 
         Automatic, ControllerLinking -> Automatic, ControllerMethod -> 
         Automatic, ControllerPath -> Automatic, CoordinatesToolOptions -> 
         Automatic, DisplayFunction :> $DisplayFunction, Epilog -> {}, 
         FaceGrids -> None, FaceGridsStyle -> {}, FormatType :> 
         TraditionalForm, ImageMargins -> 0., ImagePadding -> All, ImageSize -> 
         Automatic, LabelStyle -> {}, Lighting -> Automatic, Method -> 
         Automatic, PlotLabel -> None, PlotRange -> All, PlotRangePadding -> 
         Automatic, PlotRegion -> Automatic, PlotStyle -> Automatic, 
         PreserveImageOptions -> Automatic, Prolog -> {}, RotationAction -> 
         "Fit", SphericalRegion -> False, Ticks -> Automatic, 
         TicksStyle -> {}, ViewAngle -> Automatic, ViewCenter -> Automatic, 
         ViewMatrix -> Automatic, ViewPoint -> {1.3, -2.4, 2.}, ViewRange -> 
         All, ViewVector -> Automatic, ViewVertical -> {0, 0, 1}}, 
       TagSet[ClassroomUtilities`RiemannSumPlot3D, 
        MessageName[ClassroomUtilities`RiemannSumPlot3D, "usage"], 
        "RiemannSumPlot3D[f,{var,varmin,varmax},n,\"type\"] creates n \
partitions of \"type\" to approximate the volume obtained by revolving the \
function f about the vertical axis."], 
       TagSet[ClassroomUtilities`CapStyle, 
        MessageName[ClassroomUtilities`CapStyle, "usage"], 
        "CapStyle is an option for RiemannSumPlot that controls the \
appearance of the flat surfaces of shells and washers."], 
       ClassroomUtilities`Shell[{
          Pattern[Private`r, 
           Blank[]], 
          Pattern[Private`r, 
           Blank[]], 
          BlankSequence[]}, 
         BlankNullSequence[]] := {}, Condition[
         ClassroomUtilities`Shell[{
           PatternTest[
            Pattern[Private`r1, 
             Blank[]], NumericQ], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       With[{
         Private`lis1 = Range[0., 2 Pi - Pi/36, Pi/36], Private`lis2 = 
          ConstantArray[1., 72], Private`rimPos = Join[
            Flatten[
             
             Table[{{73, 1, 2}, {2, 73, 74}, {217, 145, 146}, {146, 217, 
               218}} + Private`i, {Private`i, 0, 70}], 1], {{144, 72, 1}, {1, 
            144, 73}, {288, 216, 145}, {145, 288, 217}}], Private`flatPts1 = 
          Join[
            Range[72], {1, 145}, 
            Range[216, 145, -1]], Private`cpstyl = ReplaceAll[
            OptionValue[ClassroomUtilities`CapStyle], List -> Directive]}, 
         GraphicsComplex[
          Join[
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k2], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k2]], {
           EdgeForm[], 
           Polygon[Private`rimPos], Private`cpstyl, 
           Polygon[{Private`flatPts1, Private`flatPts1 + 72}]}]], 
       ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], 
         Pattern[Private`opts, 
          OptionsPattern[]]] := With[{Private`cpstyl = ReplaceAll[
            OptionValue[ClassroomUtilities`CapStyle], List -> Directive]}, 
         Map[
         ClassroomUtilities`Shell[#, ClassroomUtilities`CapStyle -> 
           Private`cpstyl]& , Private`shls]], Condition[
         ClassroomUtilities`Shell[{
           PatternTest[
            Pattern[Private`r1, 
             Blank[]], NumericQ], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, {
           Condition[
            PatternTest[
             Pattern[Private`stp, 
              Blank[Integer]], Positive], 
            Inequality[1, LessEqual, Private`stp, Less, 72]]}, 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       With[{Private`lis1 = Range[0., 2 Pi - Pi/36, Pi/36], Private`lis2 = 
          ConstantArray[1., 72], Private`rimPos = Flatten[
            
            Table[{{73, 1, 2}, {2, 73, 74}, {217, 145, 146}, {146, 217, 218}} + 
             Private`i, {Private`i, 0, Private`stp - 1}], 1], 
          Private`flatPts1 = Join[
            Take[
             Range[72], Private`stp + 1], 
            Take[
             Range[216, 145, -1], -(Private`stp + 1)]], Private`cpstyl = 
          ReplaceAll[
            OptionValue[ClassroomUtilities`CapStyle], List -> Directive]}, 
         GraphicsComplex[
          Join[
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k2], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k2]], {
           EdgeForm[], 
           Polygon[Private`rimPos], Private`cpstyl, 
           Polygon[{Private`flatPts1, Private`flatPts1 + 72}], 
           
           Polygon[{{1, 73, 217, 145}, {
             Private`stp + 1, Private`stp + 73, Private`stp + 217, 
              Private`stp + 145}}]}]], ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], {
          Condition[
           PatternTest[
            Pattern[Private`stp, 
             Blank[Integer]], Positive], 
           Inequality[1, LessEqual, Private`stp, Less, 72]]}, 
         Pattern[Private`opts, 
          OptionsPattern[]]] := With[{Private`cpstyl = ReplaceAll[
            OptionValue[ClassroomUtilities`CapStyle], List -> Directive]}, 
         Map[
         ClassroomUtilities`Shell[#, {Private`stp}, 
           ClassroomUtilities`CapStyle -> Private`cpstyl]& , Private`shls]], 
       Condition[
         ClassroomUtilities`Shell[{
           PatternTest[
            Pattern[Private`r1, 
             Blank[]], NumericQ], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, {0}, 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       With[{Private`lis1 = Range[0., 2 Pi - Pi/36, Pi/36], Private`lis2 = 
          ConstantArray[1., 72], Private`cpstyl = ReplaceAll[
            OptionValue[ClassroomUtilities`CapStyle], List -> Directive]}, 
         GraphicsComplex[
          Join[
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k2], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k2]], {
           EdgeForm[], Private`cpstyl, 
           Polygon[{{1, 73, 217, 145}}]}]], ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], {0}, 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Map[ClassroomUtilities`Shell[#, {0}, Private`opts]& , Private`shls], 
       Condition[
         ClassroomUtilities`Shell[{
           Pattern[Private`r1, 
            Blank[]], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, {72}, 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       ClassroomUtilities`Shell[{
         Private`r1, Private`r2, Private`k1, Private`k2}, Private`opts], 
       ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], {72}, 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       ClassroomUtilities`Shell[Private`shls, Private`opts], Condition[
         ClassroomUtilities`Shell[{
           Pattern[Private`r1, 
            Blank[]], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, 
          PatternTest[
           Pattern[Private`nn, 
            Blank[]], Positive], 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       Block[{Private`rr = ReplaceAll[
            Part[
             Private`closest[Private`nn], 1], {1 -> 72}]}, 
         ClassroomUtilities`Shell[{
          Private`r1, Private`r2, Private`k1, Private`k2}, Private`rr, 
          Private`opts]], ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], 
         PatternTest[
          Pattern[Private`nn, 
           Blank[]], Positive], 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`rr = ReplaceAll[
            Part[
             Private`closest[Private`nn], 1], {1 -> 72}]}, 
         ClassroomUtilities`Shell[Private`shls, Private`rr, Private`opts]], 
       Condition[
         ClassroomUtilities`Shell[{
           Pattern[Private`r1, 
            Blank[]], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, 0, 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       ClassroomUtilities`Shell[{
         Private`r1, Private`r2, Private`k1, Private`k2}, {0}, Private`opts], 
       ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], 0, 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       ClassroomUtilities`Shell[Private`shls, {0}, Private`opts], Condition[
         ClassroomUtilities`Shell[{
           Pattern[Private`r1, 
            Blank[]], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, 0., 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       ClassroomUtilities`Shell[{
         Private`r1, Private`r2, Private`k1, Private`k2}, {0}, Private`opts], 
       ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], 0., 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       ClassroomUtilities`Shell[Private`shls, {0}, Private`opts], 
       Options[ClassroomUtilities`Shell] = {
        ClassroomUtilities`CapStyle -> GrayLevel[0.75]}, 
       TagSet[ClassroomUtilities`Shell, 
        MessageName[ClassroomUtilities`Shell, "usage"], 
        "Shell[{r1,r2,btm,tp}] construct a washer with inner radius r1, outer \
radius r2 and thickness tp-btm."], Private`makePts[
         Pattern[Private`lis1, 
          Blank[List]], 
         Pattern[Private`lis2, 
          Blank[List]], 
         PatternTest[
          Pattern[Private`r, 
           Blank[]], NumericQ], 
         PatternTest[
          Pattern[Private`k, 
           Blank[]], NumericQ]] := 
       Transpose[{
         Private`r Cos[Private`lis1], Private`r Sin[Private`lis1], Private`k 
          Private`lis2}], Private`closest[
         PatternTest[
          Pattern[Private`val, 
           Blank[]], NumericQ]] := 
       With[{Private`vals = Range[0, 2 Pi - Pi/36, Pi/36], Private`val2 = Mod[
            N[Private`val], 2 Pi]}, 
         Position[
          Abs[Private`vals - Private`val2], 
          Min[
           Abs[Private`vals - Private`val2]]]], Private`makeDisks[
         Pattern[Private`fun, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NumericQ], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], NumericQ]}, 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/100, 
          Private`stpV, Private`funPrsV, Private`newFn}, 
         Private`newFn = 
          Private`invFun[
           Private`fun, {Private`var, Private`varmin, Private`varmax}]; {
           Private`stpV, Private`funPrsV} = 
          Private`vertFunPrs[Private`newFn, Private`n]; Cylinder[
           Map[{{0, 0, # - Private`stpV/2}, {0, 0, # + Private`stpV/2}}& , 
            Part[Private`funPrsV, All, 1]], 
           Part[Private`funPrsV, All, 2]]], Private`makeDisks[
         Pattern[Private`fun, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NumericQ], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], NumericQ]}, 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], 
         Pattern[Private`ang, 
          Blank[]], 
         Pattern[Private`opts, 
          OptionsPattern[ClassroomUtilities`CapStyle -> Automatic]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/100, 
          Private`stpV, Private`funPrsV, Private`newFn, Private`cpStyl = 
          OptionValue[ClassroomUtilities`CapStyle]}, 
         Private`newFn = 
          Private`invFun[
           Private`fun, {Private`var, Private`varmin, Private`varmax}]; {
           Private`stpV, Private`funPrsV} = 
          Private`vertFunPrs[Private`newFn, Private`n]; 
         ClassroomUtilities`Shell[
           Map[Private`shllFun[#, Private`stpV/2]& , Private`funPrsV], 
           Private`ang, ClassroomUtilities`CapStyle -> Private`cpStyl]], 
       Private`invFun[
         Pattern[Private`fun, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]]}] := 
       Block[{Private`funPrs, 
          Private`stp = (Private`varmax - Private`varmin)/100}, 
         Private`funPrs = Table[{
             Function[Private`var, Private`fun][Private`var2], 
             Private`var2}, {Private`var2, 
             Range[Private`varmin, Private`varmax, Private`stp]}]; 
         Interpolation[Private`funPrs, Method -> "Spline"]], 
       Private`vertFunPrs[
         Pattern[Private`fun, 
          Blank[InterpolatingFunction]], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive]] := Block[{Private`stpV = Part[
             Differences[
              Part[Private`fun, 1, 1]], 1]/Private`n}, {Private`stpV, 
          Map[{#, 
            Private`fun[#]}& , 
           Range[Part[Private`fun, 1, 1, 1] + Private`stpV/2, 
            Part[Private`fun, 1, 1, 2], Private`stpV]]}], Private`vertFunPrs[{
          Pattern[Private`minval, 
           Blank[]], {
           Pattern[Private`fun, 
            Blank[]], 
           Pattern[Private`fun2, 
            Blank[]]}}, {
          Pattern[Private`var, 
           Blank[]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]], 
          Pattern[Private`stp, 
           Blank[]]}] := Table[{Private`minval, 
          Min[{Private`fun, Private`fun2}], Private`var - Private`stp/2, 
          Private`var + Private`stp/2}, {
         Private`var, Private`varmin + Private`stp/2, Private`varmax, 
          Private`stp}], Private`vertFunPrs[{{
           Pattern[Private`fun, 
            Blank[]], 
           Pattern[Private`fun2, 
            Blank[]]}, 
          Pattern[Private`maxval, 
           Blank[]]}, {
          Pattern[Private`var, 
           Blank[]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]], 
          Pattern[Private`stp, 
           Blank[]]}] := Table[{Private`maxval, 
          Max[{Private`fun, Private`fun2}], Private`var - Private`stp/2, 
          Private`var + Private`stp/2}, {
         Private`var, Private`varmin + Private`stp/2, Private`varmax, 
          Private`stp}], Private`vertFunPrs[{
          Pattern[Private`fun, 
           Blank[]], 
          Pattern[Private`fun2, 
           Blank[]]}, {
          Pattern[Private`var, 
           Blank[]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]], 
          Pattern[Private`stp, 
           Blank[]]}] := 
       Table[{Private`fun, Private`fun2, Private`var - Private`stp/2, 
          Private`var + Private`stp/2}, {
         Private`var, Private`varmin + Private`stp/2, Private`varmax, 
          Private`stp}], Private`shllFun[{
          Pattern[Private`k, 
           Blank[]], 
          Pattern[Private`r, 
           Blank[]]}, 
         PatternTest[
          Pattern[Private`stp, 
           Blank[]], Positive]] := {
        0, Private`r, Private`k - Private`stp, Private`k + Private`stp}, 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, Private`varlis]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, Private`varlis]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, Private`n, "Washer", Private`opts], 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, Private`varlis]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, Private`varlis]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, Private`ang, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, {Private`n, Private`ang}, "Washer", Private`opts], 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, {
          Pattern[Private`x, 
           Blank[Symbol]], 
          Pattern[Private`x0, 
           Blank[]], 
          Pattern[Private`xm, 
           Blank[]], 
          Pattern[Private`x1, 
           Blank[]]}, 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, {
                Private`x, Private`x0, Private`xm}]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, {
                Private`x, Private`xm, Private`x1}]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`xm, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, Private`n, "Washer", Private`opts], 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, {
          Pattern[Private`x, 
           Blank[Symbol]], 
          Pattern[Private`x0, 
           Blank[]], 
          Pattern[Private`xm, 
           Blank[]], 
          Pattern[Private`x1, 
           Blank[]]}, 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, {
                Private`x, Private`x0, Private`xm}]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, {
                Private`x, Private`xm, Private`x1}]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, Private`ang, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`xm, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, {Private`n, Private`ang}, "Washer", Private`opts], 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, {
          Pattern[Private`x, 
           Blank[Symbol]], 
          Pattern[Private`x0, 
           Blank[]], 
          Pattern[Private`xm1, 
           Blank[]], 
          Pattern[Private`xm2, 
           Blank[]], 
          Pattern[Private`x1, 
           Blank[]]}, 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, {
                Private`x, Private`x0, Private`xm1}]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, {
                Private`x, Private`xm2, Private`x1}]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`xm1, 
            Blank[]], 
           Pattern[Private`xm2, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, Private`n, "Washer", Private`opts], 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, {
          Pattern[Private`x, 
           Blank[Symbol]], 
          Pattern[Private`x0, 
           Blank[]], 
          Pattern[Private`xm1, 
           Blank[]], 
          Pattern[Private`xm2, 
           Blank[]], 
          Pattern[Private`x1, 
           Blank[]]}, 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, {
                Private`x, Private`x0, Private`xm1}]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, {
                Private`x, Private`xm2, Private`x1}]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, Private`ang, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`xm1, 
            Blank[]], 
           Pattern[Private`xm2, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, {Private`n, Private`ang}, "Washer", Private`opts], 
       Options[Private`riemannSumPlot3D] = {
        AlignmentPoint -> Center, AspectRatio -> Automatic, 
         AutomaticImageSize -> False, Axes -> False, AxesEdge -> Automatic, 
         AxesLabel -> None, AxesOrigin -> Automatic, AxesStyle -> {}, 
         Background -> None, BaselinePosition -> Automatic, BaseStyle -> {}, 
         Boxed -> True, BoxRatios -> Automatic, BoxStyle -> {}, 
         ClassroomUtilities`CapStyle -> {
           EdgeForm[
            GrayLevel[0.65]], 
           GrayLevel[0.4]}, ColorOutput -> Automatic, ContentSelectable -> 
         Automatic, ControllerLinking -> Automatic, ControllerMethod -> 
         Automatic, ControllerPath -> Automatic, CoordinatesToolOptions -> 
         Automatic, DisplayFunction :> $DisplayFunction, Epilog -> {}, 
         FaceGrids -> None, FaceGridsStyle -> {}, FormatType :> 
         TraditionalForm, ImageMargins -> 0., ImagePadding -> All, ImageSize -> 
         Automatic, LabelStyle -> {}, Lighting -> Automatic, Method -> 
         Automatic, PlotLabel -> None, PlotRange -> All, PlotRangePadding -> 
         Automatic, PlotRegion -> Automatic, PlotStyle -> Automatic, 
         PreserveImageOptions -> Automatic, Prolog -> {}, RotationAction -> 
         "Fit", SphericalRegion -> False, Ticks -> Automatic, 
         TicksStyle -> {}, ViewAngle -> Automatic, ViewCenter -> Automatic, 
         ViewMatrix -> Automatic, ViewPoint -> {1.3, -2.4, 2.}, ViewRange -> 
         All, ViewVector -> Automatic, ViewVertical -> {0, 0, 1}}, 
       Private`makeRadiusFunction[
         Pattern[Private`lis, 
          Blank[List]], 
         Pattern[Private`varlis, {
           Pattern[Private`var, 
            Blank[Symbol]], 
           PatternTest[
            Pattern[Private`varmin, 
             Blank[]], NumericQ], 
           PatternTest[
            Pattern[Private`varmax, 
             Blank[]], NumericQ]}]] := 
       Block[{Private`invrs = 
          Map[Private`interpInv[#, Private`varlis]& , Private`lis], 
          Private`res}, 
         If[
          Private`validTst[
           Part[Private`invrs, All, 1, 1]], 
          Apply[Function, 
           ReplaceAll[
            FullForm[
             Private`makePcws[Private`invrs, Private`var]], {
            Private`var :> #}]], $Failed]], Private`interpInv[
         Pattern[Private`f1, 
          Except[
           Blank[ClassroomUtilities`VerticalSegment]]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]]}, 
         Optional[
          Pattern[Private`n, 
           Blank[Integer]], 50]] := 
       Block[{Private`valTbl = 
          Table[{Private`f1, Private`var}, {
            Private`var, Private`varmin, 
             Private`varmax, (Private`varmax - Private`varmin)/Private`n}]}, 
         Interpolation[Private`valTbl, Method -> "Spline"]], Private`interpInv[
         Pattern[Private`f1, 
          Blank[ClassroomUtilities`VerticalSegment]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]]}] := 
       ReplaceAll[Private`f1, Part[Private`f1, 2, 2, 3] :> Private`var], 
       TagSet[ClassroomUtilities`VerticalSegment, 
        MessageName[ClassroomUtilities`VerticalSegment, "usage"], 
        "VerticalSegment[int,{val,ineq}] creates a vertical line segment on \
the interval int for use in MakeRadiusFunction."], Private`validTst[
         Pattern[Private`lis, 
          Blank[List]]] := OrderedQ[
         Flatten[
          SortBy[Private`lis, First]]], Private`makePcws[
         Pattern[Private`lis, {
           Repeated[
            Alternatives[
             Blank[InterpolatingFunction], 
             Blank[ClassroomUtilities`VerticalSegment]]]}], 
         Pattern[Private`var, 
          Blank[Symbol]]] := Piecewise[
         Map[Private`pcwsComponent[#, Private`var]& , Private`lis]], 
       Private`pcwsComponent[
         Pattern[Private`f1, 
          Blank[InterpolatingFunction]], 
         Pattern[Private`var, 
          Blank[Symbol]]] := {
         Private`f1[Private`var], 
         Private`makePcwsInt[
          Part[Private`f1, 1, 1], Private`var]}, Private`pcwsComponent[
         Pattern[Private`f1, 
          Blank[ClassroomUtilities`VerticalSegment]], 
         Pattern[Private`var, 
          Blank[Symbol]]] := 
       Block[{Private`fun = Private`GetFunctionDefinition[Private`f1]}, 
         ReplaceAll[Private`fun, Part[Private`fun, 2, 3] :> Private`var]], 
       Private`makePcwsInt[{
          Pattern[Private`a0, 
           Blank[]], 
          Pattern[Private`a1, 
           Blank[]]}, 
         Pattern[Private`var, 
          Blank[Symbol]]] := 
       Inequality[Private`a0, LessEqual, Private`var, Less, Private`a1], 
       Private`GetFunctionDefinition[
         Pattern[Private`vrt, 
          Blank[ClassroomUtilities`VerticalSegment]]] := 
       Part[Private`vrt, 2], $DisplayFunction = Identity, 
       ClassroomUtilities`MakeRadiusFunction[
         Pattern[Private`lis, 
          Blank[List]], 
         Pattern[Private`varlis, {
           Pattern[Private`var, 
            Blank[Symbol]], 
           PatternTest[
            Pattern[Private`varmin, 
             Blank[]], NumericQ], 
           PatternTest[
            Pattern[Private`varmax, 
             Blank[]], NumericQ]}]] := 
       Block[{Private`res = 
          Private`makeRadiusFunction[Private`lis, Private`varlis]}, 
         If[Private`res === $Failed, 
           Message[
            MessageName[ClassroomUtilities`MakeRadiusFunction, "ovrlp"], 
            "radii"]]; Condition[Private`res, Private`res =!= $Failed]], 
       TagSet[ClassroomUtilities`MakeRadiusFunction, 
        MessageName[ClassroomUtilities`MakeRadiusFunction, "ovrlp"], 
        "Two or more of the inverse function support intervals for the `1` \
are not disjoint."], 
       TagSet[ClassroomUtilities`MakeRadiusFunction, 
        MessageName[ClassroomUtilities`MakeRadiusFunction, "usage"], 
        "MakeRadiusFunction[fnlis,{var,varmin,varmax}] creates an \
interpolating function for the inverses of the functions in fnlis."]}; 
     Typeset`initDone$$ = True),
    SynchronousInitialization->True,
    UnsavedVariables:>{Typeset`initDone$$},
    UntrackedVariables:>{Typeset`size$$}], "Manipulate",
   Deployed->True,
   StripOnInput->False],
  Manipulate`InterpretManipulate[1]]], "Output",
 ImageSize->{476, 193},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[13]=",
 CellID->97986355]
}, Open  ]],

Cell["Varying the number of washers.", "MathCaption",
 CellID->59663834],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Manipulate", "[", 
  RowBox[{
   RowBox[{"RiemannSumPlot3D", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Sin", "[", 
        RowBox[{"x", "-", 
         RowBox[{"\[Pi]", "/", "4"}]}], "]"}], ",", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"2", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"x", "-", 
             RowBox[{"\[Pi]", "/", "4"}]}], ")"}], "/", 
           RowBox[{"(", "\[Pi]", ")"}]}]}], ")"}], "2"]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"\[Pi]", "/", "4"}], ",", 
       RowBox[{"3", 
        RowBox[{"\[Pi]", "/", "4"}]}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"n", ",", 
       RowBox[{"5", " ", 
        RowBox[{"\[Pi]", "/", "4"}]}]}], "}"}], ",", "\"\<Washer\>\"", ",", 
     RowBox[{"PlotRange", "\[Rule]", "  ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "2.4"}], ",", "2.4"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "2.4"}], ",", "2.4"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"n", ",", "4", ",", "18", ",", "1"}], "}"}], ",", 
   RowBox[{"SaveDefinitions", " ", "\[Rule]", " ", "True"}]}], "]"}]], "Input",
 CellLabel->"In[14]:=",
 CellID->2042678536],

Cell[BoxData[
 TagBox[
  StyleBox[
   DynamicModuleBox[{$CellContext`n$$ = 4, Typeset`show$$ = True, 
    Typeset`bookmarkList$$ = {}, Typeset`bookmarkMode$$ = "Menu", 
    Typeset`animator$$, Typeset`animvar$$ = 1, Typeset`name$$ = 
    "\"untitled\"", Typeset`specs$$ = {{
      Hold[$CellContext`n$$], 4, 18, 1}}, Typeset`size$$ = {180., {62., 67.}},
     Typeset`update$$ = 0, Typeset`initDone$$, Typeset`skipInitDone$$ = 
    False, $CellContext`n$2155$$ = 0}, 
    DynamicBox[Manipulate`ManipulateBoxes[
     1, StandardForm, "Variables" :> {$CellContext`n$$ = 4}, 
      "ControllerVariables" :> {
        Hold[$CellContext`n$$, $CellContext`n$2155$$, 0]}, 
      "OtherVariables" :> {
       Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
        Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
        Typeset`specs$$, Typeset`size$$, Typeset`update$$, Typeset`initDone$$,
         Typeset`skipInitDone$$}, "Body" :> 
      ClassroomUtilities`RiemannSumPlot3D[{
         Sin[$CellContext`x - Pi/4], (
          2 (($CellContext`x - Pi/4)/Pi))^2}, {$CellContext`x, Pi/4, 
         3 (Pi/4)}, {$CellContext`z, 0, 1}, {$CellContext`n$$, 5 (Pi/4)}, 
        "Washer", PlotRange -> {{-2.4, 2.4}, {-2.4, 2.4}, {0, 1}}], 
      "Specifications" :> {{$CellContext`n$$, 4, 18, 1}}, "Options" :> {}, 
      "DefaultOptions" :> {}],
     ImageSizeCache->{459., {93., 98.}},
     SingleEvaluation->True],
    Deinitialization:>None,
    DynamicModuleValues:>{},
    Initialization:>({ClassroomUtilities`RiemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, 
         Pattern[Private`n, 
          Blank[Integer]], "Shell", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           ClassroomUtilities`Shell[
            Table[{Private`r, Private`r + Private`stp, 
              ReplaceAll[
              Private`f1, Private`var :> Private`r + Private`stp/2], 
              ReplaceAll[
              Private`f2, Private`var :> Private`r + Private`stp/2]}, {
             Private`r, Private`varmin, Private`varmax - Private`stp, 
              Private`stp}], ClassroomUtilities`CapStyle -> Private`cpstyl]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`f, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, 
         Pattern[Private`n, 
          Blank[Integer]], "Shell", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           ClassroomUtilities`Shell[
            Table[{Private`r, Private`r + Private`stp, 0, 
              ReplaceAll[
              Private`f, Private`var :> Private`r + Private`stp/2]}, {
             Private`r, Private`varmin, Private`varmax - Private`stp, 
              Private`stp}], ClassroomUtilities`CapStyle -> Private`cpstyl]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, {
          Pattern[Private`n, 
           Blank[Integer]], 
          PatternTest[
           Pattern[Private`ang, 
            Blank[]], NumericQ]}, "Shell", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           ClassroomUtilities`Shell[
            Table[{Private`r, Private`r + Private`stp, 
              ReplaceAll[
              Private`f1, Private`var :> Private`r + Private`stp/2], 
              ReplaceAll[
              Private`f2, Private`var :> Private`r + Private`stp/2]}, {
             Private`r, Private`varmin, Private`varmax - Private`stp, 
              Private`stp}], Private`ang, ClassroomUtilities`CapStyle -> 
            Private`cpstyl]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`f, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, {
          Pattern[Private`n, 
           Blank[Integer]], 
          PatternTest[
           Pattern[Private`ang, 
            Blank[]], NumericQ]}, "Shell", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           ClassroomUtilities`Shell[
            Table[{Private`r, Private`r + Private`stp, 0, 
              ReplaceAll[
              Private`f, Private`var :> Private`r + Private`stp/2]}, {
             Private`r, Private`varmin, Private`varmax - Private`stp, 
              Private`stp}], Private`ang, ClassroomUtilities`CapStyle -> 
            Private`cpstyl]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`f, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, 
         Pattern[Private`n, 
          Blank[Integer]], "Disk", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           Private`makeDisks[
           Private`f, {Private`var, Private`varmin, Private`varmax}, 
            Private`n]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`f, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NonNegative], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], Positive]}, {
          Pattern[Private`n, 
           Blank[Integer]], 
          PatternTest[
           Pattern[Private`ang, 
            Blank[]], NumericQ]}, "Disk", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/Private`n, 
          Private`pltStyl = ReplaceAll[
            OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}], 
          Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle]}, 
         Graphics3D[{Private`pltStyl, 
           Private`makeDisks[
           Private`f, {Private`var, Private`varmin, Private`varmax}, 
            Private`n, Private`ang, ClassroomUtilities`CapStyle -> 
            Private`cpstyl]}, 
          FilterRules[{Private`opts}, 
           Options[Graphics3D]]]], ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`fnLis, 
          Blank[List]], 
         Pattern[Private`varlis, 
          Blank[List]], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`res22 = 
          Private`riemannSumPlot3D[
           Private`fnLis, Private`varlis, Private`rnglis, Private`n, "Washer",
             Private`opts]}, If[
           MatchQ[Private`res22, {$Failed, 
             Alternatives[1, 2]}], 
           Message[
            MessageName[ClassroomUtilities`MakeRadiusFunction, "ovrlp"], 
            Part[{"inner radius", "outer radius"}, 
             Part[Private`res22, 2]]]]; Condition[Private`res22, 
           FreeQ[Private`res22, $Failed]]], 
       ClassroomUtilities`RiemannSumPlot3D[
         Pattern[Private`fnLis, 
          Blank[List]], 
         Pattern[Private`varlis, 
          Blank[List]], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`res22 = 
          Private`riemannSumPlot3D[
           Private`fnLis, Private`varlis, Private`rnglis, {
            Private`n, Private`ang}, "Washer", Private`opts]}, If[
           MatchQ[Private`res22, {$Failed, 
             Alternatives[1, 2]}], 
           Message[
            MessageName[ClassroomUtilities`MakeRadiusFunction, "ovrlp"], 
            Part[{"inner radius", "outer radius"}, 
             Part[Private`res22, 2]]]]; Condition[Private`res22, 
           FreeQ[Private`res22, $Failed]]], 
       Options[ClassroomUtilities`RiemannSumPlot3D] = {
        AlignmentPoint -> Center, AspectRatio -> Automatic, 
         AutomaticImageSize -> False, Axes -> False, AxesEdge -> Automatic, 
         AxesLabel -> None, AxesOrigin -> Automatic, AxesStyle -> {}, 
         Background -> None, BaselinePosition -> Automatic, BaseStyle -> {}, 
         Boxed -> True, BoxRatios -> Automatic, BoxStyle -> {}, 
         ClassroomUtilities`CapStyle -> {
           EdgeForm[
            GrayLevel[0.65]], 
           GrayLevel[0.4]}, ColorOutput -> Automatic, ContentSelectable -> 
         Automatic, ControllerLinking -> Automatic, ControllerMethod -> 
         Automatic, ControllerPath -> Automatic, CoordinatesToolOptions -> 
         Automatic, DisplayFunction :> $DisplayFunction, Epilog -> {}, 
         FaceGrids -> None, FaceGridsStyle -> {}, FormatType :> 
         TraditionalForm, ImageMargins -> 0., ImagePadding -> All, ImageSize -> 
         Automatic, LabelStyle -> {}, Lighting -> Automatic, Method -> 
         Automatic, PlotLabel -> None, PlotRange -> All, PlotRangePadding -> 
         Automatic, PlotRegion -> Automatic, PlotStyle -> Automatic, 
         PreserveImageOptions -> Automatic, Prolog -> {}, RotationAction -> 
         "Fit", SphericalRegion -> False, Ticks -> Automatic, 
         TicksStyle -> {}, ViewAngle -> Automatic, ViewCenter -> Automatic, 
         ViewMatrix -> Automatic, ViewPoint -> {1.3, -2.4, 2.}, ViewRange -> 
         All, ViewVector -> Automatic, ViewVertical -> {0, 0, 1}}, 
       TagSet[ClassroomUtilities`RiemannSumPlot3D, 
        MessageName[ClassroomUtilities`RiemannSumPlot3D, "usage"], 
        "RiemannSumPlot3D[f,{var,varmin,varmax},n,\"type\"] creates n \
partitions of \"type\" to approximate the volume obtained by revolving the \
function f about the vertical axis."], 
       TagSet[ClassroomUtilities`CapStyle, 
        MessageName[ClassroomUtilities`CapStyle, "usage"], 
        "CapStyle is an option for RiemannSumPlot that controls the \
appearance of the flat surfaces of shells and washers."], 
       ClassroomUtilities`Shell[{
          Pattern[Private`r, 
           Blank[]], 
          Pattern[Private`r, 
           Blank[]], 
          BlankSequence[]}, 
         BlankNullSequence[]] := {}, Condition[
         ClassroomUtilities`Shell[{
           PatternTest[
            Pattern[Private`r1, 
             Blank[]], NumericQ], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       With[{
         Private`lis1 = Range[0., 2 Pi - Pi/36, Pi/36], Private`lis2 = 
          ConstantArray[1., 72], Private`rimPos = Join[
            Flatten[
             
             Table[{{73, 1, 2}, {2, 73, 74}, {217, 145, 146}, {146, 217, 
               218}} + Private`i, {Private`i, 0, 70}], 1], {{144, 72, 1}, {1, 
            144, 73}, {288, 216, 145}, {145, 288, 217}}], Private`flatPts1 = 
          Join[
            Range[72], {1, 145}, 
            Range[216, 145, -1]], Private`cpstyl = ReplaceAll[
            OptionValue[ClassroomUtilities`CapStyle], List -> Directive]}, 
         GraphicsComplex[
          Join[
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k2], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k2]], {
           EdgeForm[], 
           Polygon[Private`rimPos], Private`cpstyl, 
           Polygon[{Private`flatPts1, Private`flatPts1 + 72}]}]], 
       ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], 
         Pattern[Private`opts, 
          OptionsPattern[]]] := With[{Private`cpstyl = ReplaceAll[
            OptionValue[ClassroomUtilities`CapStyle], List -> Directive]}, 
         Map[
         ClassroomUtilities`Shell[#, ClassroomUtilities`CapStyle -> 
           Private`cpstyl]& , Private`shls]], Condition[
         ClassroomUtilities`Shell[{
           PatternTest[
            Pattern[Private`r1, 
             Blank[]], NumericQ], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, {
           Condition[
            PatternTest[
             Pattern[Private`stp, 
              Blank[Integer]], Positive], 
            Inequality[1, LessEqual, Private`stp, Less, 72]]}, 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       With[{Private`lis1 = Range[0., 2 Pi - Pi/36, Pi/36], Private`lis2 = 
          ConstantArray[1., 72], Private`rimPos = Flatten[
            
            Table[{{73, 1, 2}, {2, 73, 74}, {217, 145, 146}, {146, 217, 218}} + 
             Private`i, {Private`i, 0, Private`stp - 1}], 1], 
          Private`flatPts1 = Join[
            Take[
             Range[72], Private`stp + 1], 
            Take[
             Range[216, 145, -1], -(Private`stp + 1)]], Private`cpstyl = 
          ReplaceAll[
            OptionValue[ClassroomUtilities`CapStyle], List -> Directive]}, 
         GraphicsComplex[
          Join[
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k2], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k2]], {
           EdgeForm[], 
           Polygon[Private`rimPos], Private`cpstyl, 
           Polygon[{Private`flatPts1, Private`flatPts1 + 72}], 
           
           Polygon[{{1, 73, 217, 145}, {
             Private`stp + 1, Private`stp + 73, Private`stp + 217, 
              Private`stp + 145}}]}]], ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], {
          Condition[
           PatternTest[
            Pattern[Private`stp, 
             Blank[Integer]], Positive], 
           Inequality[1, LessEqual, Private`stp, Less, 72]]}, 
         Pattern[Private`opts, 
          OptionsPattern[]]] := With[{Private`cpstyl = ReplaceAll[
            OptionValue[ClassroomUtilities`CapStyle], List -> Directive]}, 
         Map[
         ClassroomUtilities`Shell[#, {Private`stp}, 
           ClassroomUtilities`CapStyle -> Private`cpstyl]& , Private`shls]], 
       Condition[
         ClassroomUtilities`Shell[{
           PatternTest[
            Pattern[Private`r1, 
             Blank[]], NumericQ], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, {0}, 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       With[{Private`lis1 = Range[0., 2 Pi - Pi/36, Pi/36], Private`lis2 = 
          ConstantArray[1., 72], Private`cpstyl = ReplaceAll[
            OptionValue[ClassroomUtilities`CapStyle], List -> Directive]}, 
         GraphicsComplex[
          Join[
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r1, Private`k2], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k1], 
           Private`makePts[
           Private`lis1, Private`lis2, Private`r2, Private`k2]], {
           EdgeForm[], Private`cpstyl, 
           Polygon[{{1, 73, 217, 145}}]}]], ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], {0}, 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Map[ClassroomUtilities`Shell[#, {0}, Private`opts]& , Private`shls], 
       Condition[
         ClassroomUtilities`Shell[{
           Pattern[Private`r1, 
            Blank[]], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, {72}, 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       ClassroomUtilities`Shell[{
         Private`r1, Private`r2, Private`k1, Private`k2}, Private`opts], 
       ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], {72}, 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       ClassroomUtilities`Shell[Private`shls, Private`opts], Condition[
         ClassroomUtilities`Shell[{
           Pattern[Private`r1, 
            Blank[]], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, 
          PatternTest[
           Pattern[Private`nn, 
            Blank[]], Positive], 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       Block[{Private`rr = ReplaceAll[
            Part[
             Private`closest[Private`nn], 1], {1 -> 72}]}, 
         ClassroomUtilities`Shell[{
          Private`r1, Private`r2, Private`k1, Private`k2}, Private`rr, 
          Private`opts]], ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], 
         PatternTest[
          Pattern[Private`nn, 
           Blank[]], Positive], 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`rr = ReplaceAll[
            Part[
             Private`closest[Private`nn], 1], {1 -> 72}]}, 
         ClassroomUtilities`Shell[Private`shls, Private`rr, Private`opts]], 
       Condition[
         ClassroomUtilities`Shell[{
           Pattern[Private`r1, 
            Blank[]], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, 0, 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       ClassroomUtilities`Shell[{
         Private`r1, Private`r2, Private`k1, Private`k2}, {0}, Private`opts], 
       ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], 0, 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       ClassroomUtilities`Shell[Private`shls, {0}, Private`opts], Condition[
         ClassroomUtilities`Shell[{
           Pattern[Private`r1, 
            Blank[]], 
           Pattern[Private`r2, 
            Blank[]], 
           Pattern[Private`k1, 
            Blank[]], 
           Pattern[Private`k2, 
            Blank[]]}, 0., 
          Pattern[Private`opts, 
           OptionsPattern[]]], Private`r1 != Private`r2] := 
       ClassroomUtilities`Shell[{
         Private`r1, Private`r2, Private`k1, Private`k2}, {0}, Private`opts], 
       ClassroomUtilities`Shell[
         Pattern[Private`shls, {
           Repeated[{
             PatternTest[
              Blank[], NumericQ], 
             Blank[], 
             Blank[], 
             Blank[]}]}], 0., 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       ClassroomUtilities`Shell[Private`shls, {0}, Private`opts], 
       Options[ClassroomUtilities`Shell] = {
        ClassroomUtilities`CapStyle -> GrayLevel[0.75]}, 
       TagSet[ClassroomUtilities`Shell, 
        MessageName[ClassroomUtilities`Shell, "usage"], 
        "Shell[{r1,r2,btm,tp}] construct a washer with inner radius r1, outer \
radius r2 and thickness tp-btm."], Private`makePts[
         Pattern[Private`lis1, 
          Blank[List]], 
         Pattern[Private`lis2, 
          Blank[List]], 
         PatternTest[
          Pattern[Private`r, 
           Blank[]], NumericQ], 
         PatternTest[
          Pattern[Private`k, 
           Blank[]], NumericQ]] := 
       Transpose[{
         Private`r Cos[Private`lis1], Private`r Sin[Private`lis1], Private`k 
          Private`lis2}], Private`closest[
         PatternTest[
          Pattern[Private`val, 
           Blank[]], NumericQ]] := 
       With[{Private`vals = Range[0, 2 Pi - Pi/36, Pi/36], Private`val2 = Mod[
            N[Private`val], 2 Pi]}, 
         Position[
          Abs[Private`vals - Private`val2], 
          Min[
           Abs[Private`vals - Private`val2]]]], Private`makeDisks[
         Pattern[Private`fun, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NumericQ], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], NumericQ]}, 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/100, 
          Private`stpV, Private`funPrsV, Private`newFn}, 
         Private`newFn = 
          Private`invFun[
           Private`fun, {Private`var, Private`varmin, Private`varmax}]; {
           Private`stpV, Private`funPrsV} = 
          Private`vertFunPrs[Private`newFn, Private`n]; Cylinder[
           Map[{{0, 0, # - Private`stpV/2}, {0, 0, # + Private`stpV/2}}& , 
            Part[Private`funPrsV, All, 1]], 
           Part[Private`funPrsV, All, 2]]], Private`makeDisks[
         Pattern[Private`fun, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          PatternTest[
           Pattern[Private`varmin, 
            Blank[]], NumericQ], 
          PatternTest[
           Pattern[Private`varmax, 
            Blank[]], NumericQ]}, 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], 
         Pattern[Private`ang, 
          Blank[]], 
         Pattern[Private`opts, 
          OptionsPattern[ClassroomUtilities`CapStyle -> Automatic]]] := 
       Block[{Private`stp = (Private`varmax - Private`varmin)/100, 
          Private`stpV, Private`funPrsV, Private`newFn, Private`cpStyl = 
          OptionValue[ClassroomUtilities`CapStyle]}, 
         Private`newFn = 
          Private`invFun[
           Private`fun, {Private`var, Private`varmin, Private`varmax}]; {
           Private`stpV, Private`funPrsV} = 
          Private`vertFunPrs[Private`newFn, Private`n]; 
         ClassroomUtilities`Shell[
           Map[Private`shllFun[#, Private`stpV/2]& , Private`funPrsV], 
           Private`ang, ClassroomUtilities`CapStyle -> Private`cpStyl]], 
       Private`invFun[
         Pattern[Private`fun, 
          Blank[]], {
          Pattern[Private`var, 
           Blank[]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]]}] := 
       Block[{Private`funPrs, 
          Private`stp = (Private`varmax - Private`varmin)/100}, 
         Private`funPrs = Table[{
             Function[Private`var, Private`fun][Private`var2], 
             Private`var2}, {Private`var2, 
             Range[Private`varmin, Private`varmax, Private`stp]}]; 
         Interpolation[Private`funPrs, Method -> "Spline"]], 
       Private`vertFunPrs[
         Pattern[Private`fun, 
          Blank[InterpolatingFunction]], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive]] := Block[{Private`stpV = Part[
             Differences[
              Part[Private`fun, 1, 1]], 1]/Private`n}, {Private`stpV, 
          Map[{#, 
            Private`fun[#]}& , 
           Range[Part[Private`fun, 1, 1, 1] + Private`stpV/2, 
            Part[Private`fun, 1, 1, 2], Private`stpV]]}], Private`vertFunPrs[{
          Pattern[Private`minval, 
           Blank[]], {
           Pattern[Private`fun, 
            Blank[]], 
           Pattern[Private`fun2, 
            Blank[]]}}, {
          Pattern[Private`var, 
           Blank[]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]], 
          Pattern[Private`stp, 
           Blank[]]}] := Table[{Private`minval, 
          Min[{Private`fun, Private`fun2}], Private`var - Private`stp/2, 
          Private`var + Private`stp/2}, {
         Private`var, Private`varmin + Private`stp/2, Private`varmax, 
          Private`stp}], Private`vertFunPrs[{{
           Pattern[Private`fun, 
            Blank[]], 
           Pattern[Private`fun2, 
            Blank[]]}, 
          Pattern[Private`maxval, 
           Blank[]]}, {
          Pattern[Private`var, 
           Blank[]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]], 
          Pattern[Private`stp, 
           Blank[]]}] := Table[{Private`maxval, 
          Max[{Private`fun, Private`fun2}], Private`var - Private`stp/2, 
          Private`var + Private`stp/2}, {
         Private`var, Private`varmin + Private`stp/2, Private`varmax, 
          Private`stp}], Private`vertFunPrs[{
          Pattern[Private`fun, 
           Blank[]], 
          Pattern[Private`fun2, 
           Blank[]]}, {
          Pattern[Private`var, 
           Blank[]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]], 
          Pattern[Private`stp, 
           Blank[]]}] := 
       Table[{Private`fun, Private`fun2, Private`var - Private`stp/2, 
          Private`var + Private`stp/2}, {
         Private`var, Private`varmin + Private`stp/2, Private`varmax, 
          Private`stp}], Private`shllFun[{
          Pattern[Private`k, 
           Blank[]], 
          Pattern[Private`r, 
           Blank[]]}, 
         PatternTest[
          Pattern[Private`stp, 
           Blank[]], Positive]] := {
        0, Private`r, Private`k - Private`stp, Private`k + Private`stp}, 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, Private`varlis]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, Private`varlis]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, Private`n, "Washer", Private`opts], 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, Private`varlis]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, Private`varlis]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, Private`ang, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, {Private`n, Private`ang}, "Washer", Private`opts], 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, {
          Pattern[Private`x, 
           Blank[Symbol]], 
          Pattern[Private`x0, 
           Blank[]], 
          Pattern[Private`xm, 
           Blank[]], 
          Pattern[Private`x1, 
           Blank[]]}, 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, {
                Private`x, Private`x0, Private`xm}]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, {
                Private`x, Private`xm, Private`x1}]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`xm, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, Private`n, "Washer", Private`opts], 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, {
          Pattern[Private`x, 
           Blank[Symbol]], 
          Pattern[Private`x0, 
           Blank[]], 
          Pattern[Private`xm, 
           Blank[]], 
          Pattern[Private`x1, 
           Blank[]]}, 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, {
                Private`x, Private`x0, Private`xm}]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, {
                Private`x, Private`xm, Private`x1}]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, Private`ang, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`xm, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, {Private`n, Private`ang}, "Washer", Private`opts], 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, {
          Pattern[Private`x, 
           Blank[Symbol]], 
          Pattern[Private`x0, 
           Blank[]], 
          Pattern[Private`xm1, 
           Blank[]], 
          Pattern[Private`xm2, 
           Blank[]], 
          Pattern[Private`x1, 
           Blank[]]}, 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, {
                Private`x, Private`x0, Private`xm1}]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, {
                Private`x, Private`xm2, Private`x1}]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`xm1, 
            Blank[]], 
           Pattern[Private`xm2, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], 
         PatternTest[
          Pattern[Private`n, 
           Blank[Integer]], Positive], "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, Private`n, "Washer", Private`opts], 
       Private`riemannSumPlot3D[{{
           Pattern[Private`f1, 
            BlankSequence[]]}, {
           Pattern[Private`f2, 
            BlankSequence[]]}}, {
          Pattern[Private`x, 
           Blank[Symbol]], 
          Pattern[Private`x0, 
           Blank[]], 
          Pattern[Private`xm1, 
           Blank[]], 
          Pattern[Private`xm2, 
           Blank[]], 
          Pattern[Private`x1, 
           Blank[]]}, 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := Block[{Private`res}, Private`res = Catch[
            
            Block[{Private`stp = (Private`z1 - Private`z0)/Private`n, 
              Private`shlpts, Private`pltStyl = ReplaceAll[
                OptionValue[PlotStyle], {Automatic -> {}, List -> Directive}],
               Private`cpstyl = OptionValue[ClassroomUtilities`CapStyle], 
              Private`fnInv1, Private`fnInv2, Private`shlls}, 
             Private`fnInv1 = 
              Private`makeRadiusFunction[{Private`f1}, {
                Private`x, Private`x0, Private`xm1}]; 
             Private`fnInv2 = 
              Private`makeRadiusFunction[{Private`f2}, {
                Private`x, Private`xm2, Private`x1}]; 
             Which[Private`fnInv1 === $Failed, 
               (Throw[#]& )[{$Failed, 1}], Private`fnInv2 === $Failed, 
               Throw[{$Failed, 2}], True, Private`shlpts = Table[{
                   Private`fnInv1[Private`t + Private`stp/2], 
                   Private`fnInv2[Private`t + Private`stp/2], Private`t, 
                   Private`t + Private`stp}, {
                  Private`t, Private`z0, Private`z1 - Private`stp, 
                   Private`stp}]; Graphics3D[{Private`pltStyl, 
                  ClassroomUtilities`Shell[
                  Private`shlpts, Private`ang, ClassroomUtilities`CapStyle -> 
                   Private`cpstyl]}, 
                 FilterRules[{Private`opts}, 
                  Options[Graphics3D]]]]]]; Private`res], 
       Private`riemannSumPlot3D[{
          Pattern[Private`f1, 
           Blank[]], 
          Pattern[Private`f2, 
           Blank[]]}, 
         Pattern[Private`varlis, {
           Pattern[Private`x, 
            Blank[Symbol]], 
           Pattern[Private`x0, 
            Blank[]], 
           Pattern[Private`xm1, 
            Blank[]], 
           Pattern[Private`xm2, 
            Blank[]], 
           Pattern[Private`x1, 
            Blank[]]}], 
         Pattern[Private`rnglis, {
           Pattern[Private`z, 
            Blank[Symbol]], 
           Pattern[Private`z0, 
            Blank[]], 
           Pattern[Private`z1, 
            Blank[]]}], {
          PatternTest[
           Pattern[Private`n, 
            Blank[Integer]], Positive], 
          Condition[
           Pattern[Private`ang, 
            Blank[]], 0 <= Private`ang <= 2 Pi]}, "Washer", 
         Pattern[Private`opts, 
          OptionsPattern[]]] := 
       Private`riemannSumPlot3D[{{Private`f1}, {Private`f2}}, Private`varlis, 
         Private`rnglis, {Private`n, Private`ang}, "Washer", Private`opts], 
       Options[Private`riemannSumPlot3D] = {
        AlignmentPoint -> Center, AspectRatio -> Automatic, 
         AutomaticImageSize -> False, Axes -> False, AxesEdge -> Automatic, 
         AxesLabel -> None, AxesOrigin -> Automatic, AxesStyle -> {}, 
         Background -> None, BaselinePosition -> Automatic, BaseStyle -> {}, 
         Boxed -> True, BoxRatios -> Automatic, BoxStyle -> {}, 
         ClassroomUtilities`CapStyle -> {
           EdgeForm[
            GrayLevel[0.65]], 
           GrayLevel[0.4]}, ColorOutput -> Automatic, ContentSelectable -> 
         Automatic, ControllerLinking -> Automatic, ControllerMethod -> 
         Automatic, ControllerPath -> Automatic, CoordinatesToolOptions -> 
         Automatic, DisplayFunction :> $DisplayFunction, Epilog -> {}, 
         FaceGrids -> None, FaceGridsStyle -> {}, FormatType :> 
         TraditionalForm, ImageMargins -> 0., ImagePadding -> All, ImageSize -> 
         Automatic, LabelStyle -> {}, Lighting -> Automatic, Method -> 
         Automatic, PlotLabel -> None, PlotRange -> All, PlotRangePadding -> 
         Automatic, PlotRegion -> Automatic, PlotStyle -> Automatic, 
         PreserveImageOptions -> Automatic, Prolog -> {}, RotationAction -> 
         "Fit", SphericalRegion -> False, Ticks -> Automatic, 
         TicksStyle -> {}, ViewAngle -> Automatic, ViewCenter -> Automatic, 
         ViewMatrix -> Automatic, ViewPoint -> {1.3, -2.4, 2.}, ViewRange -> 
         All, ViewVector -> Automatic, ViewVertical -> {0, 0, 1}}, 
       Private`makeRadiusFunction[
         Pattern[Private`lis, 
          Blank[List]], 
         Pattern[Private`varlis, {
           Pattern[Private`var, 
            Blank[Symbol]], 
           PatternTest[
            Pattern[Private`varmin, 
             Blank[]], NumericQ], 
           PatternTest[
            Pattern[Private`varmax, 
             Blank[]], NumericQ]}]] := 
       Block[{Private`invrs = 
          Map[Private`interpInv[#, Private`varlis]& , Private`lis], 
          Private`res}, 
         If[
          Private`validTst[
           Part[Private`invrs, All, 1, 1]], 
          Apply[Function, 
           ReplaceAll[
            FullForm[
             Private`makePcws[Private`invrs, Private`var]], {
            Private`var :> #}]], $Failed]], Private`interpInv[
         Pattern[Private`f1, 
          Except[
           Blank[ClassroomUtilities`VerticalSegment]]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]]}, 
         Optional[
          Pattern[Private`n, 
           Blank[Integer]], 50]] := 
       Block[{Private`valTbl = 
          Table[{Private`f1, Private`var}, {
            Private`var, Private`varmin, 
             Private`varmax, (Private`varmax - Private`varmin)/Private`n}]}, 
         Interpolation[Private`valTbl, Method -> "Spline"]], Private`interpInv[
         Pattern[Private`f1, 
          Blank[ClassroomUtilities`VerticalSegment]], {
          Pattern[Private`var, 
           Blank[Symbol]], 
          Pattern[Private`varmin, 
           Blank[]], 
          Pattern[Private`varmax, 
           Blank[]]}] := 
       ReplaceAll[Private`f1, Part[Private`f1, 2, 2, 3] :> Private`var], 
       TagSet[ClassroomUtilities`VerticalSegment, 
        MessageName[ClassroomUtilities`VerticalSegment, "usage"], 
        "VerticalSegment[int,{val,ineq}] creates a vertical line segment on \
the interval int for use in MakeRadiusFunction."], Private`validTst[
         Pattern[Private`lis, 
          Blank[List]]] := OrderedQ[
         Flatten[
          SortBy[Private`lis, First]]], Private`makePcws[
         Pattern[Private`lis, {
           Repeated[
            Alternatives[
             Blank[InterpolatingFunction], 
             Blank[ClassroomUtilities`VerticalSegment]]]}], 
         Pattern[Private`var, 
          Blank[Symbol]]] := Piecewise[
         Map[Private`pcwsComponent[#, Private`var]& , Private`lis]], 
       Private`pcwsComponent[
         Pattern[Private`f1, 
          Blank[InterpolatingFunction]], 
         Pattern[Private`var, 
          Blank[Symbol]]] := {
         Private`f1[Private`var], 
         Private`makePcwsInt[
          Part[Private`f1, 1, 1], Private`var]}, Private`pcwsComponent[
         Pattern[Private`f1, 
          Blank[ClassroomUtilities`VerticalSegment]], 
         Pattern[Private`var, 
          Blank[Symbol]]] := 
       Block[{Private`fun = Private`GetFunctionDefinition[Private`f1]}, 
         ReplaceAll[Private`fun, Part[Private`fun, 2, 3] :> Private`var]], 
       Private`makePcwsInt[{
          Pattern[Private`a0, 
           Blank[]], 
          Pattern[Private`a1, 
           Blank[]]}, 
         Pattern[Private`var, 
          Blank[Symbol]]] := 
       Inequality[Private`a0, LessEqual, Private`var, Less, Private`a1], 
       Private`GetFunctionDefinition[
         Pattern[Private`vrt, 
          Blank[ClassroomUtilities`VerticalSegment]]] := 
       Part[Private`vrt, 2], $DisplayFunction = Identity, 
       ClassroomUtilities`MakeRadiusFunction[
         Pattern[Private`lis, 
          Blank[List]], 
         Pattern[Private`varlis, {
           Pattern[Private`var, 
            Blank[Symbol]], 
           PatternTest[
            Pattern[Private`varmin, 
             Blank[]], NumericQ], 
           PatternTest[
            Pattern[Private`varmax, 
             Blank[]], NumericQ]}]] := 
       Block[{Private`res = 
          Private`makeRadiusFunction[Private`lis, Private`varlis]}, 
         If[Private`res === $Failed, 
           Message[
            MessageName[ClassroomUtilities`MakeRadiusFunction, "ovrlp"], 
            "radii"]]; Condition[Private`res, Private`res =!= $Failed]], 
       TagSet[ClassroomUtilities`MakeRadiusFunction, 
        MessageName[ClassroomUtilities`MakeRadiusFunction, "ovrlp"], 
        "Two or more of the inverse function support intervals for the `1` \
are not disjoint."], 
       TagSet[ClassroomUtilities`MakeRadiusFunction, 
        MessageName[ClassroomUtilities`MakeRadiusFunction, "usage"], 
        "MakeRadiusFunction[fnlis,{var,varmin,varmax}] creates an \
interpolating function for the inverses of the functions in fnlis."]}; 
     Typeset`initDone$$ = True),
    SynchronousInitialization->True,
    UnsavedVariables:>{Typeset`initDone$$},
    UntrackedVariables:>{Typeset`size$$}], "Manipulate",
   Deployed->True,
   StripOnInput->False],
  Manipulate`InterpretManipulate[1]]], "Output",
 ImageSize->{463, 193},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[14]=",
 CellID->1884194883]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 WholeCellGroupOpener->True,
 CellID->415694126],

Cell[TextData[ButtonBox["Using Integration Techniques Functions",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Classroom \
Utilities/tutorial/UsingIntegrationTechniquesFunctions"]], "RelatedTutorials",
 CellID->806871991]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 602},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Working with Riemann Sums - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "TutorialColor", 
  "LinkTrails" -> GridBox[{{
      RowBox[{
        StyleBox[
         ButtonBox[
         "Mathematica", ButtonData -> "paclet:guide/Mathematica", 
          BaseStyle -> {"Link", "DockedLinkTrail"}], FontSlant -> "Italic"], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2012, 1, 4, 18, 33, 13.3432624}", 
    "history" -> {"0.0", "", "", ""}, "context" -> "ClassroomUtilities`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Classroom Utilities Tutorial", "language" -> "en", "paclet" -> 
    "ClassroomUtilities", "status" -> "None", "summary" -> 
    "The functions discussed in this tutorial are intended to help student \
visualize the approximation of volumes of revolution using disks, shells and \
washers. The three approximations, disks, shells and washers, each require \
slightly different forms of syntax explained in this tutorial. In each of the \
forms described in this tutorial the volumes are assumed to be generated by \
functions of the variable represented by the horizontal axis rotated around \
the vertical axis. That is, we assume the function y = f(x), is rotated \
around the y-axis. This loads the package. Functions for Riemann sums in \
three dimensions.", "synonyms" -> {}, "title" -> "Working with Riemann Sums", 
    "titlemodifier" -> "", "windowtitle" -> "Working with Riemann Sums", 
    "type" -> "Tutorial", "uri" -> 
    "Classroom Utilities/tutorial/WorkingWithRiemannSums"}, 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (April 26, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> Notebook], 
   Cell[
    StyleData["Output"], CellContext -> Notebook], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (April 26, 2011)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "RasterizedOutput"->{
  Cell[18209, 511, 4423, 81, 115, "Output",
   CellTags->"RasterizedOutput",
   CellID->182929267],
  Cell[23777, 633, 3964, 73, 115, "Output",
   CellTags->"RasterizedOutput",
   CellID->237077931],
  Cell[28906, 758, 3734, 69, 237, "Output",
   CellTags->"RasterizedOutput",
   CellID->5333532],
  Cell[33128, 849, 5204, 93, 218, "Output",
   CellTags->"RasterizedOutput",
   CellID->195577273],
  Cell[39770, 1005, 3708, 69, 237, "Output",
   CellTags->"RasterizedOutput",
   CellID->184261237],
  Cell[43880, 1093, 5691, 101, 207, "Output",
   CellTags->"RasterizedOutput",
   CellID->85626100],
  Cell[52690, 1315, 4504, 82, 150, "Output",
   CellTags->"RasterizedOutput",
   CellID->15002190],
  Cell[58760, 1458, 6063, 107, 172, "Output",
   CellTags->"RasterizedOutput",
   CellID->303393418],
  Cell[67535, 1662, 4594, 83, 151, "Output",
   CellTags->"RasterizedOutput",
   CellID->239815465],
  Cell[73572, 1795, 4383, 80, 184, "Output",
   CellTags->"RasterizedOutput",
   CellID->288383423],
  Cell[80213, 1969, 7372, 129, 237, "Output",
   CellTags->"RasterizedOutput",
   CellID->641133084],
  Cell[89002, 2145, 5146, 92, 177, "Output",
   CellTags->"RasterizedOutput",
   CellID->75032485]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"RasterizedOutput", 326660, 8007}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[618, 21, 2601, 61, 46, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3244, 86, 62, 1, 51, "Title",
 CellID->509267359],
Cell[3309, 89, 693, 15, 130, "Text",
 CellID->1534169418],
Cell[4005, 106, 104, 2, 31, "MathCaption",
 CellID->1463735601],
Cell[4112, 110, 107, 3, 28, "Input",
 CellID->1579069622],
Cell[4222, 115, 1507, 48, 131, "DefinitionBox",
 CellID->2096742444],
Cell[5732, 165, 87, 1, 31, "Caption",
 CellID->1891092685],
Cell[5822, 168, 248, 10, 30, "Text",
 CellID->820227328],
Cell[6073, 180, 367, 12, 32, "MathCaption",
 CellID->836781195],
Cell[6443, 194, 558, 18, 28, "Input",
 CellID->2058623809],
Cell[7004, 214, 347, 12, 37, "Input",
 CellID->863171603],
Cell[CellGroupData[{
Cell[7376, 230, 195, 6, 70, "Input",
 CellID->1073096136],
Cell[7574, 238, 5053, 89, 161, 770, 14, "CachedBoxData", "BoxData", "Output",
 CellID->1697323360]
}, Open  ]],
Cell[12642, 330, 1096, 28, 70, "Text",
 CellID->1888428419],
Cell[13741, 360, 97, 1, 70, "MathCaption",
 CellID->538781999],
Cell[13841, 363, 1697, 59, 70, "Input",
 CellID->1518001441],
Cell[15541, 424, 878, 24, 70, "Input",
 CellID->944492527],
Cell[16422, 450, 719, 24, 70, "Input",
 CellID->2034266312],
Cell[17144, 476, 877, 24, 70, "Input",
 CellID->1974245544],
Cell[CellGroupData[{
Cell[18046, 504, 160, 5, 28, "Input",
 CellID->375028211],
Cell[18209, 511, 4423, 81, 115, "Output",
 CellTags->"RasterizedOutput",
 CellID->182929267]
}, Open  ]],
Cell[22647, 595, 93, 1, 70, "MathCaption",
 CellID->966722352],
Cell[CellGroupData[{
Cell[22765, 600, 1009, 31, 49, "Input",
 CellID->1988780509],
Cell[23777, 633, 3964, 73, 115, "Output",
 CellTags->"RasterizedOutput",
 CellID->237077931]
}, Open  ]],
Cell[CellGroupData[{
Cell[27778, 711, 48, 1, 70, "Section",
 CellID->185278677],
Cell[CellGroupData[{
Cell[27851, 716, 47, 1, 70, "Subsection",
 CellID->819439256],
Cell[27901, 719, 152, 4, 70, "Text",
 CellID->502947381],
Cell[28056, 725, 360, 13, 70, "MathCaption",
 CellID->1387711044],
Cell[CellGroupData[{
Cell[28441, 742, 462, 14, 30, "Input",
 CellID->540997875],
Cell[28906, 758, 3734, 69, 237, "Output",
 CellTags->"RasterizedOutput",
 CellID->5333532]
}, Open  ]],
Cell[32655, 830, 84, 1, 70, "MathCaption",
 CellID->1587110088],
Cell[CellGroupData[{
Cell[32764, 835, 361, 12, 30, "Input",
 CellID->664869629],
Cell[33128, 849, 5204, 93, 218, "Output",
 CellTags->"RasterizedOutput",
 CellID->195577273]
}, Open  ]],
Cell[38347, 945, 82, 1, 70, "MathCaption",
 CellID->2046796015],
Cell[CellGroupData[{
Cell[38454, 950, 253, 8, 70, "Input",
 CellID->2107566459],
Cell[38710, 960, 176, 5, 36, "Output",
 CellID->819447681]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[38935, 971, 48, 1, 70, "Subsection",
 CellID->511610507],
Cell[38986, 974, 405, 15, 70, "Text",
 CellID->113268561],
Cell[CellGroupData[{
Cell[39416, 993, 351, 10, 30, "Input",
 CellID->1034193485],
Cell[39770, 1005, 3708, 69, 237, "Output",
 CellTags->"RasterizedOutput",
 CellID->184261237]
}, Open  ]],
Cell[CellGroupData[{
Cell[43515, 1079, 362, 12, 30, "Input",
 CellID->1135960384],
Cell[43880, 1093, 5691, 101, 207, "Output",
 CellTags->"RasterizedOutput",
 CellID->85626100]
}, Open  ]],
Cell[49586, 1197, 89, 1, 70, "MathCaption",
 CellID->644546356],
Cell[CellGroupData[{
Cell[49700, 1202, 324, 11, 70, "Input",
 CellID->983089824],
Cell[50027, 1215, 281, 10, 53, "Output",
 CellID->2136777556]
}, Open  ]],
Cell[50323, 1228, 205, 5, 70, "Text",
 CellID->451354911],
Cell[50531, 1235, 118, 3, 70, "MathCaption",
 CellID->2041109219],
Cell[CellGroupData[{
Cell[50674, 1242, 615, 19, 70, "Input",
 CellID->1009747973],
Cell[51292, 1263, 240, 7, 36, "Output",
 CellID->1418403617]
}, Open  ]],
Cell[51547, 1273, 189, 4, 70, "Text",
 CellID->1018866555],
Cell[51739, 1279, 128, 3, 70, "MathCaption",
 CellID->1212294338],
Cell[CellGroupData[{
Cell[51892, 1286, 795, 27, 49, "Input",
 CellID->55588707],
Cell[52690, 1315, 4504, 82, 150, "Output",
 CellTags->"RasterizedOutput",
 CellID->15002190]
}, Open  ]],
Cell[57209, 1400, 806, 28, 70, "Text",
 CellID->1814949465],
Cell[58018, 1430, 97, 1, 70, "MathCaption",
 CellID->997936267],
Cell[CellGroupData[{
Cell[58140, 1435, 617, 21, 30, "Input",
 CellID->356978448],
Cell[58760, 1458, 6063, 107, 172, "Output",
 CellTags->"RasterizedOutput",
 CellID->303393418]
}, Open  ]],
Cell[64838, 1568, 91, 1, 70, "MathCaption",
 CellID->467556399],
Cell[CellGroupData[{
Cell[64954, 1573, 538, 18, 70, "Input",
 CellID->577733620],
Cell[65495, 1593, 195, 6, 36, "Output",
 CellID->1096815486]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[65739, 1605, 50, 1, 70, "Subsection",
 CellID->1853579677],
Cell[65792, 1608, 741, 15, 70, "Text",
 CellID->1715847957],
Cell[66536, 1625, 104, 2, 70, "MathCaption",
 CellID->830377205],
Cell[CellGroupData[{
Cell[66665, 1631, 867, 29, 49, "Input",
 CellID->1846201383],
Cell[67535, 1662, 4594, 83, 151, "Output",
 CellTags->"RasterizedOutput",
 CellID->239815465]
}, Open  ]],
Cell[72144, 1748, 281, 6, 70, "Text",
 CellID->1800946229],
Cell[72428, 1756, 136, 4, 70, "MathCaption",
 CellID->1868749985],
Cell[CellGroupData[{
Cell[72589, 1764, 980, 29, 83, "Input",
 CellID->1132660520],
Cell[73572, 1795, 4383, 80, 184, "Output",
 CellTags->"RasterizedOutput",
 CellID->288383423]
}, Open  ]],
Cell[77970, 1878, 157, 4, 70, "Text",
 CellID->63591113],
Cell[78130, 1884, 77, 1, 70, "MathCaption",
 CellID->1476463632],
Cell[78210, 1887, 133, 4, 70, "Input",
 CellID->56398676],
Cell[78346, 1893, 152, 5, 70, "Input",
 CellID->750721939],
Cell[78501, 1900, 170, 6, 70, "Input",
 CellID->917202648],
Cell[78674, 1908, 165, 6, 70, "Input",
 CellID->787286770],
Cell[78842, 1916, 166, 6, 70, "Input",
 CellID->1281994223],
Cell[79011, 1924, 251, 8, 70, "Input",
 CellID->773475610],
Cell[79265, 1934, 113, 3, 70, "MathCaption",
 CellID->1086061979],
Cell[CellGroupData[{
Cell[79403, 1941, 807, 26, 65, "Input",
 CellID->869539919],
Cell[80213, 1969, 7372, 129, 237, "Output",
 CellTags->"RasterizedOutput",
 CellID->641133084]
}, Open  ]],
Cell[87600, 2101, 98, 1, 70, "MathCaption",
 CellID->1215626950],
Cell[CellGroupData[{
Cell[87723, 2106, 1276, 37, 83, "Input",
 CellID->2030214739],
Cell[89002, 2145, 5146, 92, 177, "Output",
 CellTags->"RasterizedOutput",
 CellID->75032485]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[94209, 2244, 56, 1, 70, "Section",
 CellID->1177568547],
Cell[94268, 2247, 217, 5, 70, "Text",
 CellID->127436189],
Cell[CellGroupData[{
Cell[94510, 2256, 47, 1, 70, "Subsection",
 CellID->778832464],
Cell[94560, 2259, 89, 1, 70, "MathCaption",
 CellID->924393983],
Cell[CellGroupData[{
Cell[94674, 2264, 873, 26, 70, "Input",
 CellID->1392969883],
Cell[95550, 2292, 55842, 1368, 70, "Output",
 CellID->409097559]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[151441, 3666, 48, 1, 70, "Subsection",
 CellID->772804348],
Cell[151492, 3669, 71, 1, 70, "MathCaption",
 CellID->541579780],
Cell[CellGroupData[{
Cell[151588, 3674, 879, 26, 70, "Input",
 CellID->1295666474],
Cell[152470, 3702, 55831, 1367, 70, "Output",
 CellID->1607520066]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[208350, 5075, 49, 1, 70, "Subsection",
 CellID->346207032],
Cell[208402, 5078, 116, 3, 70, "MathCaption",
 CellID->1168387101],
Cell[CellGroupData[{
Cell[208543, 5085, 1430, 43, 70, "Input",
 CellID->491538339],
Cell[209976, 5130, 55948, 1370, 70, "Output",
 CellID->97986355]
}, Open  ]],
Cell[265939, 6503, 72, 1, 70, "MathCaption",
 CellID->59663834],
Cell[CellGroupData[{
Cell[266036, 6508, 1470, 44, 70, "Input",
 CellID->2042678536],
Cell[267509, 6554, 55928, 1370, 70, "Output",
 CellID->1884194883]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[323498, 7931, 101, 2, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[323602, 7935, 220, 5, 70, "RelatedTutorials",
 CellID->806871991]
}, Open  ]]
}, Open  ]],
Cell[323849, 7944, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

